/* Generated by Cython 0.29.24 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [],
        "name": "glycopeptidepy._c.structure.fragmentation_strategy.glycan",
        "sources": [
            "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx"
        ]
    },
    "module_name": "glycopeptidepy._c.structure.fragmentation_strategy.glycan"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_24"
#define CYTHON_HEX_VERSION 0x001D18F0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
#if PY_VERSION_HEX >= 0x030800A4 && PY_VERSION_HEX < 0x030800B2
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, 0, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__glycopeptidepy___c__structure__fragmentation_strategy__glycan
#define __PYX_HAVE_API__glycopeptidepy___c__structure__fragmentation_strategy__glycan
/* Early includes */
#include <string.h>
#include <stdio.h>
#include "pythread.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx",
  "stringsource",
  "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pxd",
  "ccomposition.pxd",
  "type.pxd",
  "bool.pxd",
  "complex.pxd",
  "glycopeptidepy/_c/count_table.pxd",
  "glycopeptidepy/_c/structure/fragment.pxd",
  "glycopeptidepy/_c/structure/base.pxd",
  "glycopeptidepy/_c/structure/sequence_methods.pxd",
  "glycopeptidepy/_c/structure/glycan.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition;
struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTableIterator;
struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_SimpleFragment;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_StubFragment;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment__NameTree;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycanCompositionWithOffsetProxyBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr;
struct __pyx_opt_args_5glypy_11composition_12ccomposition_12CComposition_calc_mass;
struct __pyx_opt_args_5glypy_11composition_12ccomposition_calculate_mass;

/* "glypy/composition/ccomposition.pxd":6
 *     cdef tuple _mass_args
 *     cpdef CComposition clone(self)
 *     cpdef double calc_mass(self, int average=?, charge=?, dict mass_data=?) except -1             # <<<<<<<<<<<<<<
 *     cpdef _from_formula(self, str formula, dict mass_data)
 *     cpdef _from_dict(self, comp)
 */
struct __pyx_opt_args_5glypy_11composition_12ccomposition_12CComposition_calc_mass {
  int __pyx_n;
  int average;
  PyObject *charge;
  PyObject *mass_data;
};

/* "glypy/composition/ccomposition.pxd":31
 * 
 * 
 * cpdef double calculate_mass(CComposition composition=?, str formula=?, int average=?, charge=?, mass_data=?) except -1             # <<<<<<<<<<<<<<
 */
struct __pyx_opt_args_5glypy_11composition_12ccomposition_calculate_mass {
  int __pyx_n;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition;
  PyObject *formula;
  int average;
  PyObject *charge;
  PyObject *mass_data;
};
struct __pyx_t_14glycopeptidepy_2_c_11count_table_count_table_bin_cell;
struct __pyx_t_14glycopeptidepy_2_c_11count_table_count_table_bin;
struct __pyx_t_14glycopeptidepy_2_c_11count_table_count_table;
struct __pyx_opt_args_14glycopeptidepy_2_c_11count_table_10CountTable_pop;
struct __pyx_opt_args_14glycopeptidepy_2_c_11count_table_10CountTable_get;

/* "glycopeptidepy/_c/count_table.pxd":3
 * from cpython cimport PyObject
 * 
 * cdef struct count_table_bin_cell:             # <<<<<<<<<<<<<<
 *     long value
 *     PyObject* key
 */
struct __pyx_t_14glycopeptidepy_2_c_11count_table_count_table_bin_cell {
  long value;
  PyObject *key;
};

/* "glycopeptidepy/_c/count_table.pxd":8
 * 
 * 
 * cdef struct count_table_bin:             # <<<<<<<<<<<<<<
 *     count_table_bin_cell* cells
 *     size_t size
 */
struct __pyx_t_14glycopeptidepy_2_c_11count_table_count_table_bin {
  struct __pyx_t_14glycopeptidepy_2_c_11count_table_count_table_bin_cell *cells;
  size_t size;
  size_t used;
};

/* "glycopeptidepy/_c/count_table.pxd":14
 * 
 * 
 * cdef struct count_table:             # <<<<<<<<<<<<<<
 *     count_table_bin* bins
 *     size_t size
 */
struct __pyx_t_14glycopeptidepy_2_c_11count_table_count_table {
  struct __pyx_t_14glycopeptidepy_2_c_11count_table_count_table_bin *bins;
  size_t size;
  size_t count;
};

/* "glycopeptidepy/_c/count_table.pxd":99
 *     cpdef clear(self)
 *     cpdef setdefault(self, key, value)
 *     cpdef long pop(self, object key, object default=?) except *             # <<<<<<<<<<<<<<
 *     cpdef get(self, object key, object default=?)
 * 
 */
struct __pyx_opt_args_14glycopeptidepy_2_c_11count_table_10CountTable_pop {
  int __pyx_n;
  PyObject *__pyx_default;
};

/* "glycopeptidepy/_c/count_table.pxd":100
 *     cpdef setdefault(self, key, value)
 *     cpdef long pop(self, object key, object default=?) except *
 *     cpdef get(self, object key, object default=?)             # <<<<<<<<<<<<<<
 * 
 *     cdef void increment(self, object key, long value)
 */
struct __pyx_opt_args_14glycopeptidepy_2_c_11count_table_10CountTable_get {
  int __pyx_n;
  PyObject *__pyx_default;
};
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_8fragment_15PeptideFragment__create;

/* "glycopeptidepy/_c/structure/fragment.pxd":96
 * 
 *     @staticmethod
 *     cdef PeptideFragment _create(IonSeriesBase kind, int position, CountTable modification_dict, double mass,             # <<<<<<<<<<<<<<
 *                                  list flanking_amino_acids=*, dict glycosylation=*,
 *                                  ChemicalShiftBase chemical_shift=*, CComposition composition=?, double* delta_mass=?)
 */
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_8fragment_15PeptideFragment__create {
  int __pyx_n;
  PyObject *flanking_amino_acids;
  PyObject *glycosylation;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase *chemical_shift;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition;
  double *delta_mass;
};
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods__make_terminal_group;
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_parsed_string;
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_components;
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_sequence;

/* "glycopeptidepy/_c/structure/sequence_methods.pxd":12
 * cdef AminoAcidResidueBase _parse_residue(basestring residue_string)
 * 
 * cdef TerminalGroup _make_terminal_group(basestring base_composition_formula, ModificationBase modification=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods__make_terminal_group {
  int __pyx_n;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *modification;
};

/* "glycopeptidepy/_c/structure/sequence_methods.pxd":17
 * cdef class _PeptideSequenceCore(PeptideSequenceBase):
 * 
 *     cpdef _init_from_parsed_string(self, list seq_list, glycan=*, n_term=*, c_term=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef _init_from_string(self, sequence, parser_function)
 */
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_parsed_string {
  int __pyx_n;
  PyObject *glycan;
  PyObject *n_term;
  PyObject *c_term;
};

/* "glycopeptidepy/_c/structure/sequence_methods.pxd":21
 *     cpdef _init_from_string(self, sequence, parser_function)
 * 
 *     cpdef _init_from_components(self, list seq_list, glycan_composition=*, ModificationBase n_term=*, ModificationBase c_term=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef _init_termini(self, TerminalGroup n_term, TerminalGroup c_term)
 */
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_components {
  int __pyx_n;
  PyObject *glycan_composition;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *n_term;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *c_term;
};

/* "glycopeptidepy/_c/structure/sequence_methods.pxd":33
 *     cdef void set_c_term(self, _value)
 * 
 *     cpdef basestring get_sequence(self, bint include_glycan=*, bint include_termini=*, str implicit_n_term=*, str implicit_c_term=*)             # <<<<<<<<<<<<<<
 *     cpdef clone(self)
 * 
 */
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_sequence {
  int __pyx_n;
  int include_glycan;
  int include_termini;
  PyObject *implicit_n_term;
  PyObject *implicit_c_term;
};
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_6glycan_20GlycosylationManager_pop;

/* "glycopeptidepy/_c/structure/glycan.pxd":25
 *     cpdef values(self)
 *     cpdef items(self)
 *     cpdef pop(self, key, default=*)             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t get_size(self)
 */
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_6glycan_20GlycosylationManager_pop {
  int __pyx_n;
  PyObject *__pyx_default;
};
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_n_glycan_composition_fragments;
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_o_glycan_composition_fragments;
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_gag_linker_composition_fragments;

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pxd":45
 *     cpdef list _combinate_sites(self, list per_site_shifts, object glycan)
 * 
 *     cpdef list n_glycan_composition_fragments(self, object glycan, int core_count=*, int iteration_count=*)             # <<<<<<<<<<<<<<
 *     cpdef list o_glycan_composition_fragments(self, glycan, long core_count=*, long iteration_count=*)
 *     cpdef list gag_linker_composition_fragments(self, glycan, long core_count=*, long iteration_count=*)
 */
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_n_glycan_composition_fragments {
  int __pyx_n;
  int core_count;
  int iteration_count;
};

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pxd":46
 * 
 *     cpdef list n_glycan_composition_fragments(self, object glycan, int core_count=*, int iteration_count=*)
 *     cpdef list o_glycan_composition_fragments(self, glycan, long core_count=*, long iteration_count=*)             # <<<<<<<<<<<<<<
 *     cpdef list gag_linker_composition_fragments(self, glycan, long core_count=*, long iteration_count=*)
 */
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_o_glycan_composition_fragments {
  int __pyx_n;
  long core_count;
  long iteration_count;
};

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pxd":47
 *     cpdef list n_glycan_composition_fragments(self, object glycan, int core_count=*, int iteration_count=*)
 *     cpdef list o_glycan_composition_fragments(self, glycan, long core_count=*, long iteration_count=*)
 *     cpdef list gag_linker_composition_fragments(self, glycan, long core_count=*, long iteration_count=*)             # <<<<<<<<<<<<<<
 */
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_gag_linker_composition_fragments {
  int __pyx_n;
  long core_count;
  long iteration_count;
};

/* "glypy/composition/ccomposition.pxd":2
 * 
 * cdef class CComposition(dict):             # <<<<<<<<<<<<<<
 *     cdef object _mass
 *     cdef tuple _mass_args
 */
struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition {
  PyDictObject __pyx_base;
  struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *__pyx_vtab;
  PyObject *_mass;
  PyObject *_mass_args;
};


/* "glycopeptidepy/_c/count_table.pxd":51
 * 
 * 
 * cdef class CountTableIterator(object):             # <<<<<<<<<<<<<<
 *     cdef :
 *         count_table* table
 */
struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTableIterator {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTableIterator *__pyx_vtab;
  struct __pyx_t_14glycopeptidepy_2_c_11count_table_count_table *table;
  size_t bin_index;
  size_t cell_index;
  PyObject *next_key;
  long next_value;
};


/* "glycopeptidepy/_c/count_table.pxd":68
 * 
 * 
 * cdef class CountTable(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         count_table* table
 */
struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *__pyx_vtab;
  struct __pyx_t_14glycopeptidepy_2_c_11count_table_count_table *table;
};


/* "glycopeptidepy/_c/structure/fragment.pxd":6
 * 
 * 
 * cdef class ChemicalShiftBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public str name
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase *__pyx_vtab;
  PyObject *name;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition;
  double mass;
};


/* "glycopeptidepy/_c/structure/fragment.pxd":15
 * 
 * 
 * cdef class IonSeriesBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public str name
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase {
  PyObject_HEAD
  PyObject *name;
  int direction;
  int includes_peptide;
  double mass_shift;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition_shift;
  Py_hash_t _hash;
};


/* "glycopeptidepy/_c/structure/fragment.pxd":25
 * 
 * 
 * cdef class FragmentBase(object):             # <<<<<<<<<<<<<<
 *     """Base class for all Fragment types. Defines basic
 *     name generation and neutral loss handling functions.
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase *__pyx_vtab;
  PyObject *_name;
  Py_hash_t _hash;
  double mass;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase *_chemical_shift;
};


/* "glycopeptidepy/_c/structure/fragment.pxd":60
 * 
 * 
 * cdef class SimpleFragment(FragmentBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public IonSeriesBase kind
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_SimpleFragment {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase __pyx_base;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase *kind;
  int is_glycosylated;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition;
};


/* "glycopeptidepy/_c/structure/fragment.pxd":69
 * 
 * 
 * cdef class StubFragment(SimpleFragment):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public object glycosylation
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_StubFragment {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_SimpleFragment __pyx_base;
  PyObject *glycosylation;
  int is_extended;
};


/* "glycopeptidepy/_c/structure/fragment.pxd":85
 * 
 * 
 * cdef class PeptideFragment(FragmentBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public IonSeriesBase kind
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase __pyx_base;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase *kind;
  int position;
  struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *modification_dict;
  double bare_mass;
  PyObject *flanking_amino_acids;
  PyObject *glycosylation;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition;
};


/* "glycopeptidepy/_c/structure/fragment.pxd":113
 * 
 * @cython.final
 * cdef class _NameTree(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public object name
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment__NameTree {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment__NameTree *__pyx_vtab;
  PyObject *name;
  PyObject *children;
};


/* "glycopeptidepy/_c/structure/base.pxd":5
 * 
 * 
 * cdef class PeptideSequenceBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public list sequence
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_vtab;
  PyObject *sequence;
  double _mass;
  PyObject *_glycosylation_manager;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *_n_term;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *_c_term;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *_total_composition;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *_peptide_composition;
};


/* "glycopeptidepy/_c/structure/base.pxd":21
 * 
 * 
 * cdef class TerminalGroup(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public CComposition base_composition
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_vtab;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *base_composition;
  double mass;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *_modification;
};


/* "glycopeptidepy/_c/structure/base.pxd":35
 * 
 * 
 * cdef class AminoAcidResidueBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public basestring name
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_vtab;
  PyObject *name;
  PyObject *symbol;
  double mass;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition;
  PyObject *neutral_loss;
  Py_hash_t _hash;
};


/* "glycopeptidepy/_c/structure/base.pxd":50
 * 
 * 
 * cdef class ModificationBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public basestring name
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_vtab;
  PyObject *name;
  double mass;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition;
};


/* "glycopeptidepy/_c/structure/base.pxd":69
 * 
 * 
 * cdef class SequencePosition(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public AminoAcidResidueBase amino_acid
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_vtab;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *amino_acid;
  PyObject *modifications;
};


/* "glycopeptidepy/_c/structure/sequence_methods.pxd":15
 * 
 * 
 * cdef class _PeptideSequenceCore(PeptideSequenceBase):             # <<<<<<<<<<<<<<
 * 
 *     cpdef _init_from_parsed_string(self, list seq_list, glycan=*, n_term=*, c_term=*)
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase __pyx_base;
};


/* "glycopeptidepy/_c/structure/fragmentation_strategy/base.pxd":28
 * 
 * 
 * cdef class FragmentationStrategyBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public _PeptideSequenceCore peptide
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase *__pyx_vtab;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *peptide;
  int compute_compositions;
};


/* "glycopeptidepy/_c/structure/glycan.pxd":5
 * from glycopeptidepy._c.structure.base cimport PeptideSequenceBase
 * 
 * cdef class GlycanCompositionWithOffsetProxyBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public object obj
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycanCompositionWithOffsetProxyBase {
  PyObject_HEAD
  PyObject *obj;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition_offset;
};


/* "glycopeptidepy/_c/structure/glycan.pxd":11
 * 
 * 
 * cdef class GlycosylationManager(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public dict mapping
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *__pyx_vtab;
  PyObject *mapping;
  PyObject *parent;
  PyObject *_aggregate;
  PyObject *_proxy;
  PyObject *_type_track;
  int _total_glycosylation_size;
};


/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pxd":8
 * 
 * 
 * cdef class GlycanCompositionFragment(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public double mass
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_vtab;
  double mass;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition;
  struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *key;
  int is_extended;
  Py_hash_t _hash_key;
};


/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pxd":23
 * 
 * 
 * cdef class GlycanCompositionFragmentStrategyBase(FragmentationStrategyBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public bint _use_query
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase __pyx_base;
  int _use_query;
  PyObject *_generator;
};


/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pxd":33
 * 
 * 
 * cdef class StubGlycopeptideStrategy(GlycanCompositionFragmentStrategyBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public bint extended
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase __pyx_base;
  int extended;
  int extended_fucosylation;
};


/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":46
 * 
 * @cython.final
 * cdef class _CompositionTree(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public _NameTree root
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_vtab;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment__NameTree *root;
};


/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":593
 *         return result
 * 
 *     def n_glycan_stub_fragments(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int core_count
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments {
  PyObject_HEAD
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_base_composition;
  double __pyx_v_base_mass;
  PyObject *__pyx_v_combos;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_composition;
  int __pyx_v_core_count;
  PyObject *__pyx_v_core_shifts;
  PyObject *__pyx_v_glycan;
  PyObject *__pyx_v_glycosylation;
  size_t __pyx_v_i;
  int __pyx_v_is_extended;
  int __pyx_v_is_glycosylated;
  size_t __pyx_v_n_combos;
  PyObject *__pyx_v_name;
  PyObject *__pyx_v_name_key;
  PyObject *__pyx_v_per_site_shifts;
  PyObject *__pyx_v_result;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_site;
  size_t __pyx_t_0;
  size_t __pyx_t_1;
  size_t __pyx_t_2;
};


/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":640
 *                 is_extended=site.is_extended)
 * 
 *     def o_glycan_stub_fragments(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             list per_site_shifts
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments {
  PyObject_HEAD
  struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *__pyx_v_aggregate_glycosylation;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_base_composition;
  double __pyx_v_base_mass;
  PyObject *__pyx_v_combos;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_composition;
  size_t __pyx_v_core_count;
  PyObject *__pyx_v_core_shifts;
  PyObject *__pyx_v_glycan;
  PyObject *__pyx_v_glycosylation;
  size_t __pyx_v_i;
  int __pyx_v_is_extended;
  int __pyx_v_is_glycosylated;
  double __pyx_v_mass;
  Py_ssize_t __pyx_v_n_combos;
  size_t __pyx_v_n_positions;
  PyObject *__pyx_v_name;
  PyObject *__pyx_v_per_site_shifts;
  PyObject *__pyx_v_positions;
  PyObject *__pyx_v_result;
  PyObject *__pyx_v_seen;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_site;
  size_t __pyx_t_0;
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
};


/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":792
 *         return core_shifts
 * 
 *     def gag_linker_stub_fragments(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             list per_site_shifts
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments {
  PyObject_HEAD
  struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *__pyx_v_aggregate_glycosylation;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_base_composition;
  double __pyx_v_base_mass;
  PyObject *__pyx_v_combos;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_composition;
  size_t __pyx_v_core_count;
  PyObject *__pyx_v_core_shifts;
  PyObject *__pyx_v_glycan;
  PyObject *__pyx_v_glycosylation;
  size_t __pyx_v_i;
  int __pyx_v_is_extended;
  int __pyx_v_is_glycosylated;
  double __pyx_v_mass;
  Py_ssize_t __pyx_v_n_combos;
  size_t __pyx_v_n_positions;
  PyObject *__pyx_v_name;
  PyObject *__pyx_v_per_site_shifts;
  PyObject *__pyx_v_positions;
  PyObject *__pyx_v_result;
  PyObject *__pyx_v_seen;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_site;
  size_t __pyx_t_0;
  Py_ssize_t __pyx_t_1;
  Py_ssize_t __pyx_t_2;
};


/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":914
 *             if (<GlycosylationManager>self.peptide.glycosylation_manager).get_size() > 0:
 *                 raise ValueError("Unknown Glycan Class Detected")
 *         return (a for a in [])             # <<<<<<<<<<<<<<
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr {
  PyObject_HEAD
};



/* "glypy/composition/ccomposition.pxd":2
 * 
 * cdef class CComposition(dict):             # <<<<<<<<<<<<<<
 *     cdef object _mass
 *     cdef tuple _mass_args
 */

struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition {
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*clone)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, int __pyx_skip_dispatch);
  double (*calc_mass)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, int __pyx_skip_dispatch, struct __pyx_opt_args_5glypy_11composition_12ccomposition_12CComposition_calc_mass *__pyx_optional_args);
  PyObject *(*_from_formula)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*_from_dict)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *, int __pyx_skip_dispatch);
  long (*getitem)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *);
  void (*setitem)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *, long);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*add_from)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*subtract_from)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*_create)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *);
};
static struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *__pyx_vtabptr_5glypy_11composition_12ccomposition_CComposition;


/* "glycopeptidepy/_c/count_table.pxd":51
 * 
 * 
 * cdef class CountTableIterator(object):             # <<<<<<<<<<<<<<
 *     cdef :
 *         count_table* table
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTableIterator {
  struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTableIterator *(*_create)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *);
  void (*advance)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTableIterator *);
  int (*has_more)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTableIterator *);
  int (*_locate_next_value)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTableIterator *, PyObject **, long *);
  int (*get_next_value)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTableIterator *, PyObject **, long *);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTableIterator *__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTableIterator;


/* "glycopeptidepy/_c/count_table.pxd":68
 * 
 * 
 * cdef class CountTable(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         count_table* table
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable {
  struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *(*_create)(void);
  struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *(*_create_from)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *);
  int (*_initialize_table)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *);
  void (*_update_from_dict)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, PyObject *);
  void (*_update_from_count_table)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *);
  void (*_add_from)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *);
  void (*_subtract_from)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *);
  int (*_add_from_dict)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, PyObject *);
  int (*_subtract_from_dict)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, PyObject *);
  void (*_scale_by)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, long);
  int (*equal_to)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *);
  PyObject *(*_to_dict)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, int __pyx_skip_dispatch);
  PyObject *(*update)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *(*copy)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, int __pyx_skip_dispatch);
  PyObject *(*items)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, int __pyx_skip_dispatch);
  PyObject *(*values)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, int __pyx_skip_dispatch);
  PyObject *(*keys)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, int __pyx_skip_dispatch);
  PyObject *(*clear)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, int __pyx_skip_dispatch);
  PyObject *(*setdefault)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  long (*pop)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_11count_table_10CountTable_pop *__pyx_optional_args);
  PyObject *(*get)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_11count_table_10CountTable_get *__pyx_optional_args);
  void (*increment)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, PyObject *, long);
  void (*decrement)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, PyObject *, long);
  long (*getitem)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, PyObject *);
  int (*setitem)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, PyObject *, long);
  long (*delitem)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, PyObject *);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable;


/* "glycopeptidepy/_c/structure/fragment.pxd":6
 * 
 * 
 * cdef class ChemicalShiftBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public str name
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase *(*clone)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase;


/* "glycopeptidepy/_c/structure/fragment.pxd":25
 * 
 * 
 * cdef class FragmentBase(object):             # <<<<<<<<<<<<<<
 *     """Base class for all Fragment types. Defines basic
 *     name generation and neutral loss handling functions.
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase *(*get_series)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase *, int __pyx_skip_dispatch);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase *(*get_chemical_shift)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase *, int __pyx_skip_dispatch);
  PyObject *(*set_chemical_shift)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase *, int __pyx_skip_dispatch);
  PyObject *(*base_name)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase *, int __pyx_skip_dispatch);
  PyObject *(*get_fragment_name)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase *, int __pyx_skip_dispatch);
  void (*_update_hash_name)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase *);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase;


/* "glycopeptidepy/_c/structure/fragment.pxd":60
 * 
 * 
 * cdef class SimpleFragment(FragmentBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public IonSeriesBase kind
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_SimpleFragment {
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase __pyx_base;
  PyObject *(*clone)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_SimpleFragment *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_SimpleFragment *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_SimpleFragment;


/* "glycopeptidepy/_c/structure/fragment.pxd":69
 * 
 * 
 * cdef class StubFragment(SimpleFragment):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public object glycosylation
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_StubFragment {
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_SimpleFragment __pyx_base;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_StubFragment *(*_create)(PyObject *, double, struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase *, struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase *, int, PyObject *, int);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_StubFragment *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_StubFragment;


/* "glycopeptidepy/_c/structure/fragment.pxd":85
 * 
 * 
 * cdef class PeptideFragment(FragmentBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public IonSeriesBase kind
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment {
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase __pyx_base;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment *(*_create)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase *, int, struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, double, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_8fragment_15PeptideFragment__create *__pyx_optional_args);
  PyObject *(*clone)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment *, int __pyx_skip_dispatch);
  void (*_update_mass_with_modifications)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment *);
  int (*_is_glycosylated)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment *);
  long (*get_glycosylation_size)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment *);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment;


/* "glycopeptidepy/_c/structure/fragment.pxd":113
 * 
 * @cython.final
 * cdef class _NameTree(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public object name
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment__NameTree {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment__NameTree *(*_create)(void);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment__NameTree *(*get)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment__NameTree *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*__pyx_fuse_0traverse)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment__NameTree *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*__pyx_fuse_1traverse)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment__NameTree *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment__NameTree *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment__NameTree;


/* "glycopeptidepy/_c/structure/base.pxd":5
 * 
 * 
 * cdef class PeptideSequenceBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public list sequence
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *(*get)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *, Py_ssize_t);
  PyObject *(*_invalidate)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase;


/* "glycopeptidepy/_c/structure/base.pxd":21
 * 
 * 
 * cdef class TerminalGroup(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public CComposition base_composition
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *(*get_modification)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*get_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *(*modify)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *, int __pyx_skip_dispatch);
  void (*set_modification)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *);
  int (*equal_to)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, PyObject *);
  PyObject *(*serialize)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_TerminalGroup;


/* "glycopeptidepy/_c/structure/base.pxd":35
 * 
 * 
 * cdef class AminoAcidResidueBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public basestring name
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *(*_create)(PyObject *, PyObject *, double, struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *);
  int (*equal_to)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase;


/* "glycopeptidepy/_c/structure/base.pxd":50
 * 
 * 
 * cdef class ModificationBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public basestring name
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase {
  int (*is_a)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*serialize)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *, int __pyx_skip_dispatch);
  int (*is_tracked_for)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_ModificationBase;


/* "glycopeptidepy/_c/structure/base.pxd":69
 * 
 * 
 * cdef class SequencePosition(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public AminoAcidResidueBase amino_acid
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_SequencePosition {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *(*_create)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *, PyObject *);
  double (*get_mass)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *);
  int (*has_modification)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *, PyObject *, int __pyx_skip_dispatch);
  int (*is_modified)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *, int __pyx_skip_dispatch);
  PyObject *(*drop_modification)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*add_modification)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_SequencePosition;


/* "glycopeptidepy/_c/structure/sequence_methods.pxd":15
 * 
 * 
 * cdef class _PeptideSequenceCore(PeptideSequenceBase):             # <<<<<<<<<<<<<<
 * 
 *     cpdef _init_from_parsed_string(self, list seq_list, glycan=*, n_term=*, c_term=*)
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore {
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase __pyx_base;
  PyObject *(*_init_from_parsed_string)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_parsed_string *__pyx_optional_args);
  PyObject *(*_init_from_string)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*_init_from_components)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_components *__pyx_optional_args);
  PyObject *(*_init_termini)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, int __pyx_skip_dispatch);
  PyObject *(*_initialize_fields)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch);
  size_t (*get_size)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *(*get_n_term)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *);
  void (*set_n_term)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *(*get_c_term)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *);
  void (*set_c_term)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *);
  PyObject *(*get_sequence)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_sequence *__pyx_optional_args);
  PyObject *(*clone)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch);
  int (*base_sequence_equality)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch);
  int (*modified_sequence_equality)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch);
  int (*full_structure_equality)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*peptide_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*total_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore;


/* "glycopeptidepy/_c/structure/fragmentation_strategy/base.pxd":28
 * 
 * 
 * cdef class FragmentationStrategyBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public _PeptideSequenceCore peptide
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase {
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*peptide_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase *, int __pyx_skip_dispatch);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*total_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase;


/* "glycopeptidepy/_c/structure/glycan.pxd":11
 * 
 * 
 * cdef class GlycosylationManager(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public dict mapping
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager {
  PyObject *(*update)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*clear)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  PyObject *(*keys)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  PyObject *(*values)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  PyObject *(*items)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  PyObject *(*pop)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_6glycan_20GlycosylationManager_pop *__pyx_optional_args);
  size_t (*get_size)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *(*copy)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *(*clone)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  PyObject *(*invalidate)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  PyObject *(*get_aggregate)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *);
  int (*set_aggregate)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, PyObject *);
  PyObject *(*_patch_aggregate)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  PyObject *(*get_glycosylation_types)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *);
  PyObject *(*_update_track)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*total_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  double (*mass)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  int (*is_fully_specified_topologies)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  PyObject *(*get_glycan_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *);
  PyObject *(*_make_glycan_composition_proxy)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *);
  int (*total_glycosylation_size)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *);
  int (*count_glycosylation_type)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, PyObject *, int __pyx_skip_dispatch);
  void (*_init)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *(*_create)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *, PyObject *);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager;


/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":135
 * 
 * @cython.freelist(10000)
 * cdef class GlycanCompositionFragment(object):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, mass, composition, key, is_extended=False):
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *(*copy)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *, int __pyx_skip_dispatch);
  PyObject *(*_get_glycan_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *, int __pyx_skip_dispatch);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *(*_create)(double, struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, int);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment;


/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":109
 * 
 * 
 * cdef class GlycanCompositionFragmentStrategyBase(FragmentationStrategyBase):             # <<<<<<<<<<<<<<
 *     def __init__(self, peptide, use_query=False, *args, **kwargs):
 *         super(GlycanCompositionFragmentStrategyBase, self).__init__(peptide, *args, **kwargs)
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase {
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase __pyx_base;
  PyObject *(*glycan_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *, int __pyx_skip_dispatch);
  int (*_guess_query_mode)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *, PyObject *, int __pyx_skip_dispatch);
  long (*count_glycosylation_type)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase;


/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":227
 * 
 * 
 * cdef class StubGlycopeptideStrategy(GlycanCompositionFragmentStrategyBase):             # <<<<<<<<<<<<<<
 * 
 *     """A fragmentation strategy that generates intact peptide + glycan Y fragments from
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy {
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase __pyx_base;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *(*fucosylate_increment)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *, int __pyx_skip_dispatch);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *(*xylosylate_increment)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *, int __pyx_skip_dispatch);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *(*modified_increment)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*fucosylate_extended)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *, long, int __pyx_skip_dispatch);
  int (*_validate_glycan_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*_combinate_sites)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*n_glycan_composition_fragments)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_n_glycan_composition_fragments *__pyx_optional_args);
  PyObject *(*o_glycan_composition_fragments)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_o_glycan_composition_fragments *__pyx_optional_args);
  PyObject *(*gag_linker_composition_fragments)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_gag_linker_composition_fragments *__pyx_optional_args);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy;


/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":46
 * 
 * @cython.final
 * cdef class _CompositionTree(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public _NameTree root
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree {
  PyObject *(*build)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree;
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_build(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *, PyObject *, int __pyx_skip_dispatch);

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* IterFinish.proto */
static CYTHON_INLINE int __Pyx_IterFinish(void);

/* UnpackItemEndCheck.proto */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* StrEquals.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* IterNext.proto */
#define __Pyx_PyIter_Next(obj) __Pyx_PyIter_Next2(obj, NULL)
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next2(PyObject *, PyObject *);

/* ListCompAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_ListComp_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len)) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_ListComp_Append(L,x) PyList_Append(L,x)
#endif

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* ListExtend.proto */
static CYTHON_INLINE int __Pyx_PyList_Extend(PyObject* L, PyObject* v) {
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject* none = _PyList_Extend((PyListObject*)L, v);
    if (unlikely(!none))
        return -1;
    Py_DECREF(none);
    return 0;
#else
    return PyList_SetSlice(L, PY_SSIZE_T_MAX, PY_SSIZE_T_MAX, v);
#endif
}

/* None.proto */
static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname);

/* pyfrozenset_new.proto */
static CYTHON_INLINE PyObject* __Pyx_PyFrozenSet_New(PyObject* it);

/* PySetContains.proto */
static CYTHON_INLINE int __Pyx_PySet_ContainsTF(PyObject* key, PyObject* set, int eq);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* CallNextTpDealloc.proto */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc);

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* CoroutineBase.proto */
typedef PyObject *(*__pyx_coroutine_body_t)(PyObject *, PyThreadState *, PyObject *);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_ExcInfoStruct  _PyErr_StackItem
#else
typedef struct {
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
} __Pyx_ExcInfoStruct;
#endif
typedef struct {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    __Pyx_ExcInfoStruct gi_exc_state;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    PyObject *gi_frame;
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static CYTHON_INLINE void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *self);
static int __Pyx_Coroutine_clear(PyObject *self);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static PyObject *__Pyx_Coroutine_Close(PyObject *self);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_Coroutine_SwapException(self)
#define __Pyx_Coroutine_ResetAndClearException(self)  __Pyx_Coroutine_ExceptionClear(&(self)->gi_exc_state)
#else
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->gi_exc_state.exc_type, &(self)->gi_exc_state.exc_value, &(self)->gi_exc_state.exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(&(self)->gi_exc_state);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->gi_exc_state.exc_type, (self)->gi_exc_state.exc_value, (self)->gi_exc_state.exc_traceback);\
    (self)->gi_exc_state.exc_type = (self)->gi_exc_state.exc_value = (self)->gi_exc_state.exc_traceback = NULL;\
    }
#endif
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state);

/* PatchModuleWithCoroutine.proto */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

/* PatchGeneratorABC.proto */
static int __Pyx_patch_abc(void);

/* Generator.proto */
#define __Pyx_Generator_USED
static PyTypeObject *__pyx_GeneratorType = 0;
#define __Pyx_Generator_CheckExact(obj) (Py_TYPE(obj) == __pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig);

/* FunctionImport.proto */
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_build(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self, PyObject *__pyx_v_pair_sequence, CYTHON_UNUSED int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_glycan_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase__guess_query_mode(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, PyObject *__pyx_v_glycan_composition, int __pyx_skip_dispatch); /* proto*/
static long __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_count_glycosylation_type(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, PyObject *__pyx_v_glycotype, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__get_glycan_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create(double __pyx_v_mass, struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_composition, struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *__pyx_v_key, int __pyx_v_is_extended); /* proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_copy(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy__validate_glycan_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *__pyx_v_aggregate_glycosylation, PyObject *__pyx_v_glycan, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_fucosylate_increment(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_shift, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_xylosylate_increment(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_shift, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_fucosylate_extended(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_shift, long __pyx_v_fucose_count, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_modified_increment(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_base, PyObject *__pyx_v_shift, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_n_glycan_composition_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_glycan, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_n_glycan_composition_fragments *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy__combinate_sites(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_per_site_shifts, PyObject *__pyx_v_glycan, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_o_glycan_composition_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_glycan, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_o_glycan_composition_fragments *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_gag_linker_composition_fragments(CYTHON_UNUSED struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_glycan, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_gag_linker_composition_fragments *__pyx_optional_args); /* proto*/

/* Module declarations from 'glypy.composition.ccomposition' */
static PyTypeObject *__pyx_ptype_5glypy_11composition_12ccomposition_CComposition = 0;
static PyObject **__pyx_vp_5glypy_11composition_12ccomposition__atom = 0;
#define __pyx_v_5glypy_11composition_12ccomposition__atom (*__pyx_vp_5glypy_11composition_12ccomposition__atom)
static PyObject **__pyx_vp_5glypy_11composition_12ccomposition__formula = 0;
#define __pyx_v_5glypy_11composition_12ccomposition__formula (*__pyx_vp_5glypy_11composition_12ccomposition__formula)
static PyObject **__pyx_vp_5glypy_11composition_12ccomposition__isotope_string = 0;
#define __pyx_v_5glypy_11composition_12ccomposition__isotope_string (*__pyx_vp_5glypy_11composition_12ccomposition__isotope_string)
static PyObject **__pyx_vp_5glypy_11composition_12ccomposition_isotope_pattern = 0;
#define __pyx_v_5glypy_11composition_12ccomposition_isotope_pattern (*__pyx_vp_5glypy_11composition_12ccomposition_isotope_pattern)
static PyObject **__pyx_vp_5glypy_11composition_12ccomposition_formula_pattern = 0;
#define __pyx_v_5glypy_11composition_12ccomposition_formula_pattern (*__pyx_vp_5glypy_11composition_12ccomposition_formula_pattern)

/* Module declarations from 'cpython.version' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'cpython.exc' */

/* Module declarations from 'cpython.module' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'cpython.tuple' */

/* Module declarations from 'cpython.list' */

/* Module declarations from 'cpython.sequence' */

/* Module declarations from 'cpython.mapping' */

/* Module declarations from 'cpython.iterator' */

/* Module declarations from 'cpython.number' */

/* Module declarations from 'cpython.int' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.bool' */
static PyTypeObject *__pyx_ptype_7cpython_4bool_bool = 0;

/* Module declarations from 'cpython.long' */

/* Module declarations from 'cpython.float' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.complex' */
static PyTypeObject *__pyx_ptype_7cpython_7complex_complex = 0;

/* Module declarations from 'cpython.string' */

/* Module declarations from 'cpython.unicode' */

/* Module declarations from 'cpython.dict' */

/* Module declarations from 'cpython.instance' */

/* Module declarations from 'cpython.function' */

/* Module declarations from 'cpython.method' */

/* Module declarations from 'cpython.weakref' */

/* Module declarations from 'cpython.getargs' */

/* Module declarations from 'cpython.pythread' */

/* Module declarations from 'cpython.pystate' */

/* Module declarations from 'cpython.cobject' */

/* Module declarations from 'cpython.oldbuffer' */

/* Module declarations from 'cpython.set' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'cpython.bytes' */

/* Module declarations from 'cpython.pycapsule' */

/* Module declarations from 'cpython' */

/* Module declarations from 'glycopeptidepy._c.count_table' */
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_11count_table_CountTableIterator = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_11count_table_CountTable = 0;

/* Module declarations from 'cython' */

/* Module declarations from 'glycopeptidepy._c.structure.fragment' */
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_SimpleFragment = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_StubFragment = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment__NameTree = 0;
static PyObject *(*__pyx_fuse_0__pyx_f_14glycopeptidepy_2_c_9structure_8fragment_build_name_from_composition)(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *); /*proto*/
static PyObject *(*__pyx_fuse_1__pyx_f_14glycopeptidepy_2_c_9structure_8fragment_build_name_from_composition)(PyObject *); /*proto*/
static PyObject *(*__pyx_fuse_2__pyx_f_14glycopeptidepy_2_c_9structure_8fragment_build_name_from_composition)(PyObject *); /*proto*/

/* Module declarations from 'glycopeptidepy._c.structure.base' */
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_SequencePosition = 0;

/* Module declarations from 'glycopeptidepy._c.structure.sequence_methods' */
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore = 0;

/* Module declarations from 'glycopeptidepy._c.structure.fragmentation_strategy.base' */
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase = 0;
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase **__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__n_glycosylation = 0;
#define __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__n_glycosylation (*__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__n_glycosylation)
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase **__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__o_glycosylation = 0;
#define __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__o_glycosylation (*__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__o_glycosylation)
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase **__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__gag_linker_glycosylation = 0;
#define __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__gag_linker_glycosylation (*__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__gag_linker_glycosylation)
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase **__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__modification_hexnac = 0;
#define __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__modification_hexnac (*__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__modification_hexnac)
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase **__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__modification_xylose = 0;
#define __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__modification_xylose (*__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__modification_xylose)
static PyObject **__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_glycosylation_type_to_core = 0;
#define __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_glycosylation_type_to_core (*__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_glycosylation_type_to_core)

/* Module declarations from 'glycopeptidepy._c.structure.glycan' */
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_6glycan_GlycanCompositionWithOffsetProxyBase = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager = 0;

/* Module declarations from 'glycopeptidepy._c.structure.fragmentation_strategy.glycan' */
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_product = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_n_linked = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_o_linked = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_glycosaminoglycan = 0;
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_IonSeries_stub_glycopeptide = 0;
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__composition_tree_root = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__composition_name_cache = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX = 0;
static struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX_composition = 0;
static double __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX_mass;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC = 0;
static struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC_composition = 0;
static double __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC_mass;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC = 0;
static struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC_composition = 0;
static double __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC_mass;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__DHEX = 0;
static struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__DHEX_composition = 0;
static double __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__DHEX_mass;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL = 0;
static struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL_composition = 0;
static double __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL_mass;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__AHEX = 0;
static struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__AHEX_composition = 0;
static double __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__AHEX_mass;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Hex2NAc = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Glc2NAc = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Gal2NAc = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_hexnac_hex_composition_cache = 0;
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__prepare_glycan_composition_from_mapping(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *); /*proto*/
static struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_hexnac_hex_composition(long, long); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle__CompositionTree__set_state(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *, PyObject *); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *, PyObject *); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle_GlycanCompositionFragment__set_state(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *, PyObject *); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle_StubGlycopeptideStrategy__set_state(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "glycopeptidepy._c.structure.fragmentation_strategy.glycan"
extern int __pyx_module_is_main_glycopeptidepy___c__structure__fragmentation_strategy__glycan;
int __pyx_module_is_main_glycopeptidepy___c__structure__fragmentation_strategy__glycan = 0;

/* Implementation of 'glycopeptidepy._c.structure.fragmentation_strategy.glycan' */
static PyObject *__pyx_builtin_super;
static PyObject *__pyx_builtin_KeyError;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_ValueError;
static const char __pyx_k_Fuc[] = "Fuc";
static const char __pyx_k_Hex[] = "Hex";
static const char __pyx_k_Pen[] = "Pen";
static const char __pyx_k_Xyl[] = "Xyl";
static const char __pyx_k_key[] = "key";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_obj[] = "obj";
static const char __pyx_k_aHex[] = "aHex";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_base[] = "base";
static const char __pyx_k_copy[] = "copy";
static const char __pyx_k_dHex[] = "dHex";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_init[] = "__init__";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_mass[] = "mass";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_root[] = "root";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_send[] = "send";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_exact[] = "exact";
static const char __pyx_k_query[] = "query";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_shift[] = "shift";
static const char __pyx_k_super[] = "super";
static const char __pyx_k_throw[] = "throw";
static const char __pyx_k_HexNAc[] = "HexNAc";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_glycan[] = "glycan";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_name_2[] = "__name__";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_Gal2NAc[] = "Gal2NAc";
static const char __pyx_k_Glc2NAc[] = "Glc2NAc";
static const char __pyx_k_Hex2NAc[] = "Hex2NAc";
static const char __pyx_k_genexpr[] = "genexpr";
static const char __pyx_k_peptide[] = "peptide";
static const char __pyx_k_product[] = "product";
static const char __pyx_k_KeyError[] = "KeyError";
static const char __pyx_k_extended[] = "extended";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_n_linked[] = "n_linked";
static const char __pyx_k_o_linked[] = "o_linked";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_IonSeries[] = "IonSeries";
static const char __pyx_k_itertools[] = "itertools";
static const char __pyx_k_product_2[] = "_product";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_use_query[] = "use_query";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_core_count[] = "core_count";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_composition[] = "composition";
static const char __pyx_k_is_extended[] = "is_extended";
static const char __pyx_k_fucose_count[] = "fucose_count";
static const char __pyx_k_getitem_fast[] = "_getitem_fast";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_setitem_fast[] = "_setitem_fast";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_ChemicalShift[] = "ChemicalShift";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_stub_fragments[] = "stub_fragments";
static const char __pyx_k_CompositionTree[] = "_CompositionTree";
static const char __pyx_k_combinate_sites[] = "_combinate_sites";
static const char __pyx_k_from_iupac_lite[] = "from_iupac_lite";
static const char __pyx_k_iteration_count[] = "iteration_count";
static const char __pyx_k_per_site_shifts[] = "per_site_shifts";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_composition_tree[] = "_composition_tree";
static const char __pyx_k_guess_query_mode[] = "_guess_query_mode";
static const char __pyx_k_GlycosylationType[] = "GlycosylationType";
static const char __pyx_k_glycosaminoglycan[] = "glycosaminoglycan";
static const char __pyx_k_stub_glycopeptide[] = "stub_glycopeptide";
static const char __pyx_k_total_composition[] = "total_composition";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_glycan_composition[] = "glycan_composition";
static const char __pyx_k_modified_increment[] = "modified_increment";
static const char __pyx_k_fucosylate_extended[] = "fucosylate_extended";
static const char __pyx_k_fucosylate_increment[] = "fucosylate_increment";
static const char __pyx_k_xylosylate_increment[] = "xylosylate_increment";
static const char __pyx_k_extended_fucosylation[] = "extended_fucosylation";
static const char __pyx_k_glycosylation_manager[] = "glycosylation_manager";
static const char __pyx_k_get_glycan_composition[] = "_get_glycan_composition";
static const char __pyx_k_aggregate_glycosylation[] = "aggregate_glycosylation";
static const char __pyx_k_n_glycan_stub_fragments[] = "n_glycan_stub_fragments";
static const char __pyx_k_o_glycan_stub_fragments[] = "o_glycan_stub_fragments";
static const char __pyx_k_StubGlycopeptideStrategy[] = "StubGlycopeptideStrategy";
static const char __pyx_k_count_glycosylation_type[] = "count_glycosylation_type";
static const char __pyx_k_GlycanCompositionFragment[] = "GlycanCompositionFragment";
static const char __pyx_k_HashableGlycanComposition[] = "HashableGlycanComposition";
static const char __pyx_k_gag_linker_stub_fragments[] = "gag_linker_stub_fragments";
static const char __pyx_k_FrozenMonosaccharideResidue[] = "FrozenMonosaccharideResidue";
static const char __pyx_k_format_negative_composition[] = "format_negative_composition";
static const char __pyx_k_validate_glycan_composition[] = "_validate_glycan_composition";
static const char __pyx_k_Unknown_Glycan_Class_Detected[] = "Unknown Glycan Class Detected";
static const char __pyx_k_pyx_unpickle__CompositionTree[] = "__pyx_unpickle__CompositionTree";
static const char __pyx_k_stub_fragments_locals_genexpr[] = "stub_fragments.<locals>.genexpr";
static const char __pyx_k_n_glycan_composition_fragments[] = "n_glycan_composition_fragments";
static const char __pyx_k_o_glycan_composition_fragments[] = "o_glycan_composition_fragments";
static const char __pyx_k_pyx_unpickle_GlycanComposition[] = "__pyx_unpickle_GlycanCompositionFragmentStrategyBase";
static const char __pyx_k_pyx_unpickle_StubGlycopeptideS[] = "__pyx_unpickle_StubGlycopeptideStrategy";
static const char __pyx_k_glycopeptidepy_structure_glycan[] = "glycopeptidepy.structure.glycan";
static const char __pyx_k_self___class_____name___self_ma[] = "{self.__class__.__name__}({self.mass}, {self.composition}, {self.key}, {self.is_extended})";
static const char __pyx_k_Does_not_support_mixed_type_glyc[] = "Does not support mixed-type glycan fragmentation (yet)";
static const char __pyx_k_GlycanCompositionFragmentStrateg[] = "GlycanCompositionFragmentStrategyBase";
static const char __pyx_k_Incompatible_checksums_s_vs_0x18[] = "Incompatible checksums (%s vs 0x18c151e = (_hash_key, composition, is_extended, key, mass))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x58[] = "Incompatible checksums (%s vs 0x58be74f = (_generator, _use_query, compute_compositions, peptide))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x63[] = "Incompatible checksums (%s vs 0x63a9f0e = (root))";
static const char __pyx_k_Incompatible_checksums_s_vs_0xf5[] = "Incompatible checksums (%s vs 0xf5d0479 = (_generator, _use_query, compute_compositions, extended, extended_fucosylation, peptide))";
static const char __pyx_k_StubGlycopeptideStrategy_gag_lin[] = "StubGlycopeptideStrategy.gag_linker_stub_fragments";
static const char __pyx_k_StubGlycopeptideStrategy_n_glyca[] = "StubGlycopeptideStrategy.n_glycan_stub_fragments";
static const char __pyx_k_StubGlycopeptideStrategy_o_glyca[] = "StubGlycopeptideStrategy.o_glycan_stub_fragments";
static const char __pyx_k_gag_linker_composition_fragments[] = "gag_linker_composition_fragments";
static const char __pyx_k_glycopeptidepy__c_structure_frag[] = "glycopeptidepy._c.structure.fragmentation_strategy.glycan";
static const char __pyx_k_glycopeptidepy_structure_fragmen[] = "glycopeptidepy.structure.fragment";
static const char __pyx_k_glypy_structure_glycan_compositi[] = "glypy.structure.glycan_composition";
static const char __pyx_k_pyx_unpickle_GlycanComposition_2[] = "__pyx_unpickle_GlycanCompositionFragment";
static PyObject *__pyx_n_s_ChemicalShift;
static PyObject *__pyx_n_s_CompositionTree;
static PyObject *__pyx_kp_s_Does_not_support_mixed_type_glyc;
static PyObject *__pyx_n_s_FrozenMonosaccharideResidue;
static PyObject *__pyx_n_s_Fuc;
static PyObject *__pyx_n_s_Gal2NAc;
static PyObject *__pyx_n_s_Glc2NAc;
static PyObject *__pyx_n_s_GlycanCompositionFragment;
static PyObject *__pyx_n_s_GlycanCompositionFragmentStrateg;
static PyObject *__pyx_n_s_GlycosylationType;
static PyObject *__pyx_n_s_HashableGlycanComposition;
static PyObject *__pyx_n_s_Hex;
static PyObject *__pyx_n_s_Hex2NAc;
static PyObject *__pyx_n_s_HexNAc;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x18;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x58;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x63;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0xf5;
static PyObject *__pyx_n_s_IonSeries;
static PyObject *__pyx_n_s_KeyError;
static PyObject *__pyx_n_s_Pen;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_StubGlycopeptideStrategy;
static PyObject *__pyx_n_s_StubGlycopeptideStrategy_gag_lin;
static PyObject *__pyx_n_s_StubGlycopeptideStrategy_n_glyca;
static PyObject *__pyx_n_s_StubGlycopeptideStrategy_o_glyca;
static PyObject *__pyx_kp_s_Unknown_Glycan_Class_Detected;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_Xyl;
static PyObject *__pyx_n_s_aHex;
static PyObject *__pyx_n_s_aggregate_glycosylation;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_base;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_n_s_combinate_sites;
static PyObject *__pyx_n_s_composition;
static PyObject *__pyx_n_s_composition_tree;
static PyObject *__pyx_n_s_copy;
static PyObject *__pyx_n_s_core_count;
static PyObject *__pyx_n_s_count_glycosylation_type;
static PyObject *__pyx_n_s_dHex;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_exact;
static PyObject *__pyx_n_s_extended;
static PyObject *__pyx_n_s_extended_fucosylation;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_format_negative_composition;
static PyObject *__pyx_n_s_from_iupac_lite;
static PyObject *__pyx_n_s_fucose_count;
static PyObject *__pyx_n_s_fucosylate_extended;
static PyObject *__pyx_n_s_fucosylate_increment;
static PyObject *__pyx_n_s_gag_linker_composition_fragments;
static PyObject *__pyx_n_s_gag_linker_stub_fragments;
static PyObject *__pyx_n_s_genexpr;
static PyObject *__pyx_n_s_get_glycan_composition;
static PyObject *__pyx_n_s_getitem_fast;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_glycan;
static PyObject *__pyx_n_s_glycan_composition;
static PyObject *__pyx_n_s_glycopeptidepy__c_structure_frag;
static PyObject *__pyx_n_s_glycopeptidepy_structure_fragmen;
static PyObject *__pyx_n_s_glycopeptidepy_structure_glycan;
static PyObject *__pyx_n_s_glycosaminoglycan;
static PyObject *__pyx_n_s_glycosylation_manager;
static PyObject *__pyx_n_s_glypy_structure_glycan_compositi;
static PyObject *__pyx_n_s_guess_query_mode;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_init;
static PyObject *__pyx_n_s_is_extended;
static PyObject *__pyx_n_s_iteration_count;
static PyObject *__pyx_n_s_itertools;
static PyObject *__pyx_n_s_key;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_mass;
static PyObject *__pyx_n_s_modified_increment;
static PyObject *__pyx_n_s_n_glycan_composition_fragments;
static PyObject *__pyx_n_s_n_glycan_stub_fragments;
static PyObject *__pyx_n_s_n_linked;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_o_glycan_composition_fragments;
static PyObject *__pyx_n_s_o_glycan_stub_fragments;
static PyObject *__pyx_n_s_o_linked;
static PyObject *__pyx_n_s_obj;
static PyObject *__pyx_n_s_peptide;
static PyObject *__pyx_n_s_per_site_shifts;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_product;
static PyObject *__pyx_n_s_product_2;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_GlycanComposition;
static PyObject *__pyx_n_s_pyx_unpickle_GlycanComposition_2;
static PyObject *__pyx_n_s_pyx_unpickle_StubGlycopeptideS;
static PyObject *__pyx_n_s_pyx_unpickle__CompositionTree;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_query;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_root;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_kp_s_self___class_____name___self_ma;
static PyObject *__pyx_n_s_send;
static PyObject *__pyx_n_s_setitem_fast;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_n_s_shift;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_stub_fragments;
static PyObject *__pyx_n_s_stub_fragments_locals_genexpr;
static PyObject *__pyx_n_s_stub_glycopeptide;
static PyObject *__pyx_n_s_super;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_throw;
static PyObject *__pyx_n_s_total_composition;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_n_s_use_query;
static PyObject *__pyx_n_s_validate_glycan_composition;
static PyObject *__pyx_n_s_xylosylate_increment;
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree___init__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self, PyObject *__pyx_v_root); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_2__getitem__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4build(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self, PyObject *__pyx_v_pair_sequence); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_6__reduce_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_8__setstate_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase___init__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, PyObject *__pyx_v_peptide, PyObject *__pyx_v_use_query, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_2glycan_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_4_guess_query_mode(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, PyObject *__pyx_v_glycan_composition); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_6count_glycosylation_type(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, PyObject *__pyx_v_glycotype); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_use_query___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_use_query_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_8__reduce_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10__setstate_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment___init__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_mass, PyObject *__pyx_v_composition, PyObject *__pyx_v_key, PyObject *__pyx_v_is_extended); /* proto */
static Py_hash_t __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_2__hash__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_4_get_glycan_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_6__eq__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_8__getitem__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_key); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_10__setitem__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_12__repr__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_14copy(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_4mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_4mass_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11is_extended___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11is_extended_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_9_hash_key___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_9_hash_key_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_16__reduce_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_18__setstate_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy___init__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_peptide, PyObject *__pyx_v_extended, PyObject *__pyx_v_use_query, PyObject *__pyx_v_extended_fucosylation, CYTHON_UNUSED PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_2__next__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_4_validate_glycan_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *__pyx_v_aggregate_glycosylation, PyObject *__pyx_v_glycan); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_6fucosylate_increment(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_shift); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_8xylosylate_increment(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_shift); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_10fucosylate_extended(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_shift, long __pyx_v_fucose_count); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_12modified_increment(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_base, PyObject *__pyx_v_shift); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_14n_glycan_composition_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_glycan, int __pyx_v_core_count, int __pyx_v_iteration_count); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_16_combinate_sites(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_per_site_shifts, PyObject *__pyx_v_glycan); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_18n_glycan_stub_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_21o_glycan_stub_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_24o_glycan_composition_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_glycan, long __pyx_v_core_count, long __pyx_v_iteration_count); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_26gag_linker_stub_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_29gag_linker_composition_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_glycan, long __pyx_v_core_count, long __pyx_v_iteration_count); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_14stub_fragments_genexpr(CYTHON_UNUSED PyObject *__pyx_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_31stub_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_8extended___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_8extended_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_21extended_fucosylation___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_21extended_fucosylation_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_33__reduce_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_35__setstate_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle__CompositionTree(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_2__pyx_unpickle_GlycanCompositionFragmentStrategyBase(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_4__pyx_unpickle_GlycanCompositionFragment(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_6__pyx_unpickle_StubGlycopeptideStrategy(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_2;
static PyObject *__pyx_int_25957662;
static PyObject *__pyx_int_93054799;
static PyObject *__pyx_int_104505102;
static PyObject *__pyx_int_257754233;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__2;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_tuple__7;
static PyObject *__pyx_tuple__8;
static PyObject *__pyx_tuple__9;
static PyObject *__pyx_tuple__10;
static PyObject *__pyx_tuple__11;
static PyObject *__pyx_tuple__12;
static PyObject *__pyx_tuple__13;
static PyObject *__pyx_tuple__15;
static PyObject *__pyx_tuple__17;
static PyObject *__pyx_tuple__19;
static PyObject *__pyx_codeobj__14;
static PyObject *__pyx_codeobj__16;
static PyObject *__pyx_codeobj__18;
static PyObject *__pyx_codeobj__20;
/* Late includes */

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":50
 *         public _NameTree root
 * 
 *     def __init__(self, root=None):             # <<<<<<<<<<<<<<
 *         if root is None:
 *             root = _NameTree()
 */

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_root = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_root,0};
    PyObject* values[1] = {0};
    values[0] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_root);
          if (value) { values[0] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 50, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_root = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 50, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan._CompositionTree.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree___init__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *)__pyx_v_self), __pyx_v_root);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree___init__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self, PyObject *__pyx_v_root) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);
  __Pyx_INCREF(__pyx_v_root);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":51
 * 
 *     def __init__(self, root=None):
 *         if root is None:             # <<<<<<<<<<<<<<
 *             root = _NameTree()
 *         self.root = root
 */
  __pyx_t_1 = (__pyx_v_root == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":52
 *     def __init__(self, root=None):
 *         if root is None:
 *             root = _NameTree()             # <<<<<<<<<<<<<<
 *         self.root = root
 * 
 */
    __pyx_t_3 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment__NameTree)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 52, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_root, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":51
 * 
 *     def __init__(self, root=None):
 *         if root is None:             # <<<<<<<<<<<<<<
 *             root = _NameTree()
 *         self.root = root
 */
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":53
 *         if root is None:
 *             root = _NameTree()
 *         self.root = root             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, key):
 */
  if (!(likely(((__pyx_v_root) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_root, __pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment__NameTree))))) __PYX_ERR(0, 53, __pyx_L1_error)
  __pyx_t_3 = __pyx_v_root;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->root);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->root));
  __pyx_v_self->root = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment__NameTree *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":50
 *         public _NameTree root
 * 
 *     def __init__(self, root=None):             # <<<<<<<<<<<<<<
 *         if root is None:
 *             root = _NameTree()
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan._CompositionTree.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_root);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":55
 *         self.root = root
 * 
 *     def __getitem__(self, key):             # <<<<<<<<<<<<<<
 *         return self.root[key]
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_3__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_2__getitem__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *)__pyx_v_self), ((PyObject *)__pyx_v_key));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_2__getitem__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":56
 * 
 *     def __getitem__(self, key):
 *         return self.root[key]             # <<<<<<<<<<<<<<
 * 
 *     cpdef object build(self, list pair_sequence):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetItem(((PyObject *)__pyx_v_self->root), __pyx_v_key); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 56, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":55
 *         self.root = root
 * 
 *     def __getitem__(self, key):             # <<<<<<<<<<<<<<
 *         return self.root[key]
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan._CompositionTree.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":58
 *         return self.root[key]
 * 
 *     cpdef object build(self, list pair_sequence):             # <<<<<<<<<<<<<<
 *         cdef:
 *             tuple item
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_5build(PyObject *__pyx_v_self, PyObject *__pyx_v_pair_sequence); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_build(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self, PyObject *__pyx_v_pair_sequence, CYTHON_UNUSED int __pyx_skip_dispatch) {
  PyObject *__pyx_v_node = NULL;
  PyObject *__pyx_v_k = NULL;
  PyObject *__pyx_v_v = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  Py_ssize_t __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *(*__pyx_t_9)(PyObject *);
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("build", 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":63
 *             int i, n
 *         # n = PyList_Size(pair_sequence)
 *         node = self.root.traverse(pair_sequence)             # <<<<<<<<<<<<<<
 *         if node.name is None:
 *             # modify the hashable glycan composition before calculating
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment__NameTree *)__pyx_v_self->root->__pyx_vtab)->__pyx_fuse_0traverse(__pyx_v_self->root, __pyx_v_pair_sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 63, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_node = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":64
 *         # n = PyList_Size(pair_sequence)
 *         node = self.root.traverse(pair_sequence)
 *         if node.name is None:             # <<<<<<<<<<<<<<
 *             # modify the hashable glycan composition before calculating
 *             # its hash value
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_node, __pyx_n_s_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 64, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_t_1 == Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":67
 *             # modify the hashable glycan composition before calculating
 *             # its hash value
 *             node.name = HashableGlycanComposition()             # <<<<<<<<<<<<<<
 *             for k, v in pair_sequence:
 *                 node.name._setitem_fast(k, v)
 */
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_HashableGlycanComposition); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 67, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_node, __pyx_n_s_name, __pyx_t_1) < 0) __PYX_ERR(0, 67, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":68
 *             # its hash value
 *             node.name = HashableGlycanComposition()
 *             for k, v in pair_sequence:             # <<<<<<<<<<<<<<
 *                 node.name._setitem_fast(k, v)
 *             # Force the population of the _mass and _str caches
 */
    if (unlikely(__pyx_v_pair_sequence == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 68, __pyx_L1_error)
    }
    __pyx_t_1 = __pyx_v_pair_sequence; __Pyx_INCREF(__pyx_t_1); __pyx_t_6 = 0;
    for (;;) {
      if (__pyx_t_6 >= PyList_GET_SIZE(__pyx_t_1)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_4 = PyList_GET_ITEM(__pyx_t_1, __pyx_t_6); __Pyx_INCREF(__pyx_t_4); __pyx_t_6++; if (unlikely(0 < 0)) __PYX_ERR(0, 68, __pyx_L1_error)
      #else
      __pyx_t_4 = PySequence_ITEM(__pyx_t_1, __pyx_t_6); __pyx_t_6++; if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 68, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      #endif
      if ((likely(PyTuple_CheckExact(__pyx_t_4))) || (PyList_CheckExact(__pyx_t_4))) {
        PyObject* sequence = __pyx_t_4;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 68, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_5 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_7 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_5 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_7 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_7);
        #else
        __pyx_t_5 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 68, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_7 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 68, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_7);
        #endif
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_8 = PyObject_GetIter(__pyx_t_4); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 68, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_9 = Py_TYPE(__pyx_t_8)->tp_iternext;
        index = 0; __pyx_t_5 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_5)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_5);
        index = 1; __pyx_t_7 = __pyx_t_9(__pyx_t_8); if (unlikely(!__pyx_t_7)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_7);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_8), 2) < 0) __PYX_ERR(0, 68, __pyx_L1_error)
        __pyx_t_9 = NULL;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L7_unpacking_done;
        __pyx_L6_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_9 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 68, __pyx_L1_error)
        __pyx_L7_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_k, __pyx_t_5);
      __pyx_t_5 = 0;
      __Pyx_XDECREF_SET(__pyx_v_v, __pyx_t_7);
      __pyx_t_7 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":69
 *             node.name = HashableGlycanComposition()
 *             for k, v in pair_sequence:
 *                 node.name._setitem_fast(k, v)             # <<<<<<<<<<<<<<
 *             # Force the population of the _mass and _str caches
 *             node.name.mass()
 */
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_node, __pyx_n_s_name); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 69, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_setitem_fast); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 69, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_7 = NULL;
      __pyx_t_10 = 0;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
        __pyx_t_7 = PyMethod_GET_SELF(__pyx_t_5);
        if (likely(__pyx_t_7)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
          __Pyx_INCREF(__pyx_t_7);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_5, function);
          __pyx_t_10 = 1;
        }
      }
      #if CYTHON_FAST_PYCALL
      if (PyFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_k, __pyx_v_v};
        __pyx_t_4 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 69, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_4);
      } else
      #endif
      #if CYTHON_FAST_PYCCALL
      if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
        PyObject *__pyx_temp[3] = {__pyx_t_7, __pyx_v_k, __pyx_v_v};
        __pyx_t_4 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_10, 2+__pyx_t_10); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 69, __pyx_L1_error)
        __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_GOTREF(__pyx_t_4);
      } else
      #endif
      {
        __pyx_t_8 = PyTuple_New(2+__pyx_t_10); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 69, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_8);
        if (__pyx_t_7) {
          __Pyx_GIVEREF(__pyx_t_7); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_7); __pyx_t_7 = NULL;
        }
        __Pyx_INCREF(__pyx_v_k);
        __Pyx_GIVEREF(__pyx_v_k);
        PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_10, __pyx_v_k);
        __Pyx_INCREF(__pyx_v_v);
        __Pyx_GIVEREF(__pyx_v_v);
        PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_10, __pyx_v_v);
        __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 69, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      }
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":68
 *             # its hash value
 *             node.name = HashableGlycanComposition()
 *             for k, v in pair_sequence:             # <<<<<<<<<<<<<<
 *                 node.name._setitem_fast(k, v)
 *             # Force the population of the _mass and _str caches
 */
    }
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":71
 *                 node.name._setitem_fast(k, v)
 *             # Force the population of the _mass and _str caches
 *             node.name.mass()             # <<<<<<<<<<<<<<
 *             str(node.name)
 *         return node.name
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_node, __pyx_n_s_name); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 71, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_mass); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 71, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_5);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 71, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":72
 *             # Force the population of the _mass and _str caches
 *             node.name.mass()
 *             str(node.name)             # <<<<<<<<<<<<<<
 *         return node.name
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_node, __pyx_n_s_name); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 72, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 72, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":64
 *         # n = PyList_Size(pair_sequence)
 *         node = self.root.traverse(pair_sequence)
 *         if node.name is None:             # <<<<<<<<<<<<<<
 *             # modify the hashable glycan composition before calculating
 *             # its hash value
 */
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":73
 *             node.name.mass()
 *             str(node.name)
 *         return node.name             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_node, __pyx_n_s_name); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 73, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_r = __pyx_t_5;
  __pyx_t_5 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":58
 *         return self.root[key]
 * 
 *     cpdef object build(self, list pair_sequence):             # <<<<<<<<<<<<<<
 *         cdef:
 *             tuple item
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan._CompositionTree.build", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_node);
  __Pyx_XDECREF(__pyx_v_k);
  __Pyx_XDECREF(__pyx_v_v);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_5build(PyObject *__pyx_v_self, PyObject *__pyx_v_pair_sequence); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4build[] = "_CompositionTree.build(self, list pair_sequence)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_5build(PyObject *__pyx_v_self, PyObject *__pyx_v_pair_sequence) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("build (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_pair_sequence), (&PyList_Type), 1, "pair_sequence", 1))) __PYX_ERR(0, 58, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4build(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *)__pyx_v_self), ((PyObject*)__pyx_v_pair_sequence));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4build(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self, PyObject *__pyx_v_pair_sequence) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("build", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_build(__pyx_v_self, __pyx_v_pair_sequence, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 58, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan._CompositionTree.build", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":48
 * cdef class _CompositionTree(object):
 *     cdef:
 *         public _NameTree root             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, root=None):
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->root));
  __pyx_r = ((PyObject *)__pyx_v_self->root);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment__NameTree))))) __PYX_ERR(0, 48, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->root);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->root));
  __pyx_v_self->root = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment__NameTree *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan._CompositionTree.root.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->root);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->root));
  __pyx_v_self->root = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment__NameTree *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_6__reduce_cython__[] = "_CompositionTree.__reduce_cython__(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_7__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_6__reduce_cython__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_6__reduce_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.root,)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->root));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->root));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self->root));
  __pyx_v_state = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.root,)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_1 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v__dict = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "(tree fragment)":7
 *     state = (self.root,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_2 = (__pyx_v__dict != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_1 = PyTuple_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.root is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.root,)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.root is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle__CompositionTree, (type(self), 0x63a9f0e, None), state
 */
  /*else*/ {
    __pyx_t_3 = (((PyObject *)__pyx_v_self->root) != Py_None);
    __pyx_v_use_setstate = __pyx_t_3;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.root is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle__CompositionTree, (type(self), 0x63a9f0e, None), state
 *     else:
 */
  __pyx_t_3 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_3) {

    /* "(tree fragment)":13
 *         use_setstate = self.root is not None
 *     if use_setstate:
 *         return __pyx_unpickle__CompositionTree, (type(self), 0x63a9f0e, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle__CompositionTree, (type(self), 0x63a9f0e, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle__CompositionTree); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_104505102);
    __Pyx_GIVEREF(__pyx_int_104505102);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_104505102);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_1, 2, Py_None);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.root is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle__CompositionTree, (type(self), 0x63a9f0e, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle__CompositionTree, (type(self), 0x63a9f0e, None), state
 *     else:
 *         return __pyx_unpickle__CompositionTree, (type(self), 0x63a9f0e, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle__CompositionTree__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_5, __pyx_n_s_pyx_unpickle__CompositionTree); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_104505102);
    __Pyx_GIVEREF(__pyx_int_104505102);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_int_104505102);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_5);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_1);
    __pyx_t_5 = 0;
    __pyx_t_1 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan._CompositionTree.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle__CompositionTree, (type(self), 0x63a9f0e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle__CompositionTree__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_8__setstate_cython__[] = "_CompositionTree.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_9__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_8__setstate_cython__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_8__setstate_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle__CompositionTree, (type(self), 0x63a9f0e, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle__CompositionTree__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle__CompositionTree__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle__CompositionTree, (type(self), 0x63a9f0e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle__CompositionTree__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan._CompositionTree.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":80
 * 
 * 
 * cdef _prepare_glycan_composition_from_mapping(CountTable mapping):             # <<<<<<<<<<<<<<
 *     pair_sequence = mapping.items()
 *     return _composition_tree_root.build(pair_sequence)
 */

static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__prepare_glycan_composition_from_mapping(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *__pyx_v_mapping) {
  PyObject *__pyx_v_pair_sequence = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_prepare_glycan_composition_from_mapping", 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":81
 * 
 * cdef _prepare_glycan_composition_from_mapping(CountTable mapping):
 *     pair_sequence = mapping.items()             # <<<<<<<<<<<<<<
 *     return _composition_tree_root.build(pair_sequence)
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_mapping->__pyx_vtab)->items(__pyx_v_mapping, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 81, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_pair_sequence = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":82
 * cdef _prepare_glycan_composition_from_mapping(CountTable mapping):
 *     pair_sequence = mapping.items()
 *     return _composition_tree_root.build(pair_sequence)             # <<<<<<<<<<<<<<
 * 
 * cdef:
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_build(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__composition_tree_root, __pyx_v_pair_sequence, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 82, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":80
 * 
 * 
 * cdef _prepare_glycan_composition_from_mapping(CountTable mapping):             # <<<<<<<<<<<<<<
 *     pair_sequence = mapping.items()
 *     return _composition_tree_root.build(pair_sequence)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan._prepare_glycan_composition_from_mapping", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_pair_sequence);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":110
 * 
 * cdef class GlycanCompositionFragmentStrategyBase(FragmentationStrategyBase):
 *     def __init__(self, peptide, use_query=False, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         super(GlycanCompositionFragmentStrategyBase, self).__init__(peptide, *args, **kwargs)
 *         # to be initialized closer to when the glycan composition will
 */

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_peptide = 0;
  PyObject *__pyx_v_use_query = 0;
  PyObject *__pyx_v_args = 0;
  PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  if (PyTuple_GET_SIZE(__pyx_args) > 2) {
    __pyx_v_args = PyTuple_GetSlice(__pyx_args, 2, PyTuple_GET_SIZE(__pyx_args));
    if (unlikely(!__pyx_v_args)) {
      __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
      __Pyx_RefNannyFinishContext();
      return -1;
    }
    __Pyx_GOTREF(__pyx_v_args);
  } else {
    __pyx_v_args = __pyx_empty_tuple; __Pyx_INCREF(__pyx_empty_tuple);
  }
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_peptide,&__pyx_n_s_use_query,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        default:
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_peptide)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_use_query);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        const Py_ssize_t used_pos_args = (pos_args < 2) ? pos_args : 2;
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, used_pos_args, "__init__") < 0)) __PYX_ERR(0, 110, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        default:
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        case  0:
        goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_peptide = values[0];
    __pyx_v_use_query = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 110, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_args); __pyx_v_args = 0;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragmentStrategyBase.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase___init__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_v_self), __pyx_v_peptide, __pyx_v_use_query, __pyx_v_args, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_args);
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase___init__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, PyObject *__pyx_v_peptide, PyObject *__pyx_v_use_query, PyObject *__pyx_v_args, PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":111
 * cdef class GlycanCompositionFragmentStrategyBase(FragmentationStrategyBase):
 *     def __init__(self, peptide, use_query=False, *args, **kwargs):
 *         super(GlycanCompositionFragmentStrategyBase, self).__init__(peptide, *args, **kwargs)             # <<<<<<<<<<<<<<
 *         # to be initialized closer to when the glycan composition will
 *         # be used. Determine whether to use the fast-path __getitem__ or
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_init); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_peptide);
  __Pyx_GIVEREF(__pyx_v_peptide);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_peptide);
  __pyx_t_3 = PyNumber_Add(__pyx_t_2, __pyx_v_args); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyDict_Copy(__pyx_v_kwargs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 111, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":116
 *         # go through the slower but more general GlycanComposition.query
 *         # method to look up monosaccharides.
 *         self._use_query = use_query             # <<<<<<<<<<<<<<
 *         self._generator = None
 * 
 */
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_v_use_query); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 116, __pyx_L1_error)
  __pyx_v_self->_use_query = __pyx_t_5;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":117
 *         # method to look up monosaccharides.
 *         self._use_query = use_query
 *         self._generator = None             # <<<<<<<<<<<<<<
 * 
 *     cpdef glycan_composition(self):
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_generator);
  __Pyx_DECREF(__pyx_v_self->_generator);
  __pyx_v_self->_generator = Py_None;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":110
 * 
 * cdef class GlycanCompositionFragmentStrategyBase(FragmentationStrategyBase):
 *     def __init__(self, peptide, use_query=False, *args, **kwargs):             # <<<<<<<<<<<<<<
 *         super(GlycanCompositionFragmentStrategyBase, self).__init__(peptide, *args, **kwargs)
 *         # to be initialized closer to when the glycan composition will
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragmentStrategyBase.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":119
 *         self._generator = None
 * 
 *     cpdef glycan_composition(self):             # <<<<<<<<<<<<<<
 *         return self.peptide.glycan_composition.obj
 * 
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_3glycan_composition(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_glycan_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("glycan_composition", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_glycan_composition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 119, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_3glycan_composition)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 119, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":120
 * 
 *     cpdef glycan_composition(self):
 *         return self.peptide.glycan_composition.obj             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint _guess_query_mode(self, glycan_composition):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->__pyx_base.peptide), __pyx_n_s_glycan_composition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_obj); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 120, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":119
 *         self._generator = None
 * 
 *     cpdef glycan_composition(self):             # <<<<<<<<<<<<<<
 *         return self.peptide.glycan_composition.obj
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragmentStrategyBase.glycan_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_3glycan_composition(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_2glycan_composition[] = "GlycanCompositionFragmentStrategyBase.glycan_composition(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_3glycan_composition(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("glycan_composition (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_2glycan_composition(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_2glycan_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("glycan_composition", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_glycan_composition(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 119, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragmentStrategyBase.glycan_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":122
 *         return self.peptide.glycan_composition.obj
 * 
 *     cpdef bint _guess_query_mode(self, glycan_composition):             # <<<<<<<<<<<<<<
 *         # these guesses will work for N-glycans and common types of mucin-type O-glycans
 *         # and GAG linkers
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_5_guess_query_mode(PyObject *__pyx_v_self, PyObject *__pyx_v_glycan_composition); /*proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase__guess_query_mode(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, PyObject *__pyx_v_glycan_composition, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_flag = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_guess_query_mode", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_guess_query_mode); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 122, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_5_guess_query_mode)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_glycan_composition) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_glycan_composition);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 122, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 122, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":125
 *         # these guesses will work for N-glycans and common types of mucin-type O-glycans
 *         # and GAG linkers
 *         flag = glycan_composition._getitem_fast(_Hex2NAc) +\             # <<<<<<<<<<<<<<
 *             glycan_composition._getitem_fast(_Glc2NAc) +\
 *             glycan_composition._getitem_fast(_Gal2NAc)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_glycan_composition, __pyx_n_s_getitem_fast); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Hex2NAc) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Hex2NAc);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":126
 *         # and GAG linkers
 *         flag = glycan_composition._getitem_fast(_Hex2NAc) +\
 *             glycan_composition._getitem_fast(_Glc2NAc) +\             # <<<<<<<<<<<<<<
 *             glycan_composition._getitem_fast(_Gal2NAc)
 *         return flag or self._use_query
 */
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_glycan_composition, __pyx_n_s_getitem_fast); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Glc2NAc) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Glc2NAc);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":125
 *         # these guesses will work for N-glycans and common types of mucin-type O-glycans
 *         # and GAG linkers
 *         flag = glycan_composition._getitem_fast(_Hex2NAc) +\             # <<<<<<<<<<<<<<
 *             glycan_composition._getitem_fast(_Glc2NAc) +\
 *             glycan_composition._getitem_fast(_Gal2NAc)
 */
  __pyx_t_3 = PyNumber_Add(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 125, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":127
 *         flag = glycan_composition._getitem_fast(_Hex2NAc) +\
 *             glycan_composition._getitem_fast(_Glc2NAc) +\
 *             glycan_composition._getitem_fast(_Gal2NAc)             # <<<<<<<<<<<<<<
 *         return flag or self._use_query
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_glycan_composition, __pyx_n_s_getitem_fast); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_4, __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Gal2NAc) : __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Gal2NAc);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 127, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":126
 *         # and GAG linkers
 *         flag = glycan_composition._getitem_fast(_Hex2NAc) +\
 *             glycan_composition._getitem_fast(_Glc2NAc) +\             # <<<<<<<<<<<<<<
 *             glycan_composition._getitem_fast(_Gal2NAc)
 *         return flag or self._use_query
 */
  __pyx_t_1 = PyNumber_Add(__pyx_t_3, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 126, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_flag = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":128
 *             glycan_composition._getitem_fast(_Glc2NAc) +\
 *             glycan_composition._getitem_fast(_Gal2NAc)
 *         return flag or self._use_query             # <<<<<<<<<<<<<<
 * 
 *     cpdef long count_glycosylation_type(self, glycotype):
 */
  __pyx_t_6 = __Pyx_PyObject_IsTrue(__pyx_v_flag); if (unlikely(__pyx_t_6 < 0)) __PYX_ERR(0, 128, __pyx_L1_error)
  if (!__pyx_t_6) {
  } else {
    __pyx_t_5 = __pyx_t_6;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_6 = (__pyx_v_self->_use_query != 0);
  __pyx_t_5 = __pyx_t_6;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":122
 *         return self.peptide.glycan_composition.obj
 * 
 *     cpdef bint _guess_query_mode(self, glycan_composition):             # <<<<<<<<<<<<<<
 *         # these guesses will work for N-glycans and common types of mucin-type O-glycans
 *         # and GAG linkers
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragmentStrategyBase._guess_query_mode", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_flag);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_5_guess_query_mode(PyObject *__pyx_v_self, PyObject *__pyx_v_glycan_composition); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_4_guess_query_mode[] = "GlycanCompositionFragmentStrategyBase._guess_query_mode(self, glycan_composition) -> bool";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_5_guess_query_mode(PyObject *__pyx_v_self, PyObject *__pyx_v_glycan_composition) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_guess_query_mode (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_4_guess_query_mode(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_v_self), ((PyObject *)__pyx_v_glycan_composition));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_4_guess_query_mode(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, PyObject *__pyx_v_glycan_composition) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_guess_query_mode", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase__guess_query_mode(__pyx_v_self, __pyx_v_glycan_composition, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 122, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragmentStrategyBase._guess_query_mode", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":130
 *         return flag or self._use_query
 * 
 *     cpdef long count_glycosylation_type(self, glycotype):             # <<<<<<<<<<<<<<
 *         return (<GlycosylationManager>self.peptide.glycosylation_manager).count_glycosylation_type(glycotype)
 * 
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_7count_glycosylation_type(PyObject *__pyx_v_self, PyObject *__pyx_v_glycotype); /*proto*/
static long __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_count_glycosylation_type(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, PyObject *__pyx_v_glycotype, int __pyx_skip_dispatch) {
  long __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  long __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("count_glycosylation_type", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_count_glycosylation_type); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 130, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_7count_glycosylation_type)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_glycotype) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_glycotype);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 130, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyInt_As_long(__pyx_t_2); if (unlikely((__pyx_t_5 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 130, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":131
 * 
 *     cpdef long count_glycosylation_type(self, glycotype):
 *         return (<GlycosylationManager>self.peptide.glycosylation_manager).count_glycosylation_type(glycotype)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->__pyx_base.peptide), __pyx_n_s_glycosylation_manager); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 131, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *)((struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *)__pyx_t_1)->__pyx_vtab)->count_glycosylation_type(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *)__pyx_t_1), __pyx_v_glycotype, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":130
 *         return flag or self._use_query
 * 
 *     cpdef long count_glycosylation_type(self, glycotype):             # <<<<<<<<<<<<<<
 *         return (<GlycosylationManager>self.peptide.glycosylation_manager).count_glycosylation_type(glycotype)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragmentStrategyBase.count_glycosylation_type", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_7count_glycosylation_type(PyObject *__pyx_v_self, PyObject *__pyx_v_glycotype); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_6count_glycosylation_type[] = "GlycanCompositionFragmentStrategyBase.count_glycosylation_type(self, glycotype) -> long";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_7count_glycosylation_type(PyObject *__pyx_v_self, PyObject *__pyx_v_glycotype) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("count_glycosylation_type (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_6count_glycosylation_type(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_v_self), ((PyObject *)__pyx_v_glycotype));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_6count_glycosylation_type(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, PyObject *__pyx_v_glycotype) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("count_glycosylation_type", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_count_glycosylation_type(__pyx_v_self, __pyx_v_glycotype, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 130, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragmentStrategyBase.count_glycosylation_type", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pxd":25
 * cdef class GlycanCompositionFragmentStrategyBase(FragmentationStrategyBase):
 *     cdef:
 *         public bint _use_query             # <<<<<<<<<<<<<<
 *         public object _generator
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_use_query_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_use_query_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_use_query___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_use_query___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->_use_query); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragmentStrategyBase._use_query.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_use_query_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_use_query_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_use_query_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_use_query_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 25, __pyx_L1_error)
  __pyx_v_self->_use_query = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragmentStrategyBase._use_query.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pxd":26
 *     cdef:
 *         public bint _use_query
 *         public object _generator             # <<<<<<<<<<<<<<
 * 
 *     cpdef glycan_composition(self)
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_generator);
  __pyx_r = __pyx_v_self->_generator;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->_generator);
  __Pyx_DECREF(__pyx_v_self->_generator);
  __pyx_v_self->_generator = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_generator);
  __Pyx_DECREF(__pyx_v_self->_generator);
  __pyx_v_self->_generator = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_8__reduce_cython__[] = "GlycanCompositionFragmentStrategyBase.__reduce_cython__(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_8__reduce_cython__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_8__reduce_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._generator, self._use_query, self.compute_compositions, self.peptide)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->_use_query); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.compute_compositions); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(__pyx_v_self->_generator);
  __Pyx_GIVEREF(__pyx_v_self->_generator);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v_self->_generator);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.peptide));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.peptide));
  PyTuple_SET_ITEM(__pyx_t_3, 3, ((PyObject *)__pyx_v_self->__pyx_base.peptide));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._generator, self._use_query, self.compute_compositions, self.peptide)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._generator, self._use_query, self.compute_compositions, self.peptide)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._generator is not None or self.peptide is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._generator, self._use_query, self.compute_compositions, self.peptide)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._generator is not None or self.peptide is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_GlycanCompositionFragmentStrategyBase, (type(self), 0x58be74f, None), state
 */
  /*else*/ {
    __pyx_t_4 = (__pyx_v_self->_generator != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (((PyObject *)__pyx_v_self->__pyx_base.peptide) != Py_None);
    __pyx_t_4 = (__pyx_t_6 != 0);
    __pyx_t_5 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._generator is not None or self.peptide is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_GlycanCompositionFragmentStrategyBase, (type(self), 0x58be74f, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self._generator is not None or self.peptide is not None
 *     if use_setstate:
 *         return __pyx_unpickle_GlycanCompositionFragmentStrategyBase, (type(self), 0x58be74f, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_GlycanCompositionFragmentStrategyBase, (type(self), 0x58be74f, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_GlycanComposition); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_93054799);
    __Pyx_GIVEREF(__pyx_int_93054799);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_93054799);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._generator is not None or self.peptide is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_GlycanCompositionFragmentStrategyBase, (type(self), 0x58be74f, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_GlycanCompositionFragmentStrategyBase, (type(self), 0x58be74f, None), state
 *     else:
 *         return __pyx_unpickle_GlycanCompositionFragmentStrategyBase, (type(self), 0x58be74f, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_GlycanComposition); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_93054799);
    __Pyx_GIVEREF(__pyx_int_93054799);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_93054799);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragmentStrategyBase.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_GlycanCompositionFragmentStrategyBase, (type(self), 0x58be74f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10__setstate_cython__[] = "GlycanCompositionFragmentStrategyBase.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10__setstate_cython__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10__setstate_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_GlycanCompositionFragmentStrategyBase, (type(self), 0x58be74f, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_GlycanCompositionFragmentStrategyBase, (type(self), 0x58be74f, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragmentStrategyBase.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":137
 * cdef class GlycanCompositionFragment(object):
 * 
 *     def __init__(self, mass, composition, key, is_extended=False):             # <<<<<<<<<<<<<<
 *         self.mass = mass
 *         self.composition = composition
 */

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_mass = 0;
  PyObject *__pyx_v_composition = 0;
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_is_extended = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_mass,&__pyx_n_s_composition,&__pyx_n_s_key,&__pyx_n_s_is_extended,0};
    PyObject* values[4] = {0,0,0,0};
    values[3] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_mass)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_composition)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 3, 4, 1); __PYX_ERR(0, 137, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_key)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__init__", 0, 3, 4, 2); __PYX_ERR(0, 137, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_is_extended);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 137, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_mass = values[0];
    __pyx_v_composition = values[1];
    __pyx_v_key = values[2];
    __pyx_v_is_extended = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 3, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 137, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment___init__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self), __pyx_v_mass, __pyx_v_composition, __pyx_v_key, __pyx_v_is_extended);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment___init__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_mass, PyObject *__pyx_v_composition, PyObject *__pyx_v_key, PyObject *__pyx_v_is_extended) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":138
 * 
 *     def __init__(self, mass, composition, key, is_extended=False):
 *         self.mass = mass             # <<<<<<<<<<<<<<
 *         self.composition = composition
 *         self.key = key
 */
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_mass); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 138, __pyx_L1_error)
  __pyx_v_self->mass = __pyx_t_1;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":139
 *     def __init__(self, mass, composition, key, is_extended=False):
 *         self.mass = mass
 *         self.composition = composition             # <<<<<<<<<<<<<<
 *         self.key = key
 *         self.is_extended = is_extended
 */
  if (!(likely(((__pyx_v_composition) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_composition, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 139, __pyx_L1_error)
  __pyx_t_2 = __pyx_v_composition;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->composition));
  __pyx_v_self->composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":140
 *         self.mass = mass
 *         self.composition = composition
 *         self.key = key             # <<<<<<<<<<<<<<
 *         self.is_extended = is_extended
 *         self._hash_key = -1
 */
  if (!(likely(((__pyx_v_key) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_key, __pyx_ptype_14glycopeptidepy_2_c_11count_table_CountTable))))) __PYX_ERR(0, 140, __pyx_L1_error)
  __pyx_t_2 = __pyx_v_key;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __Pyx_GOTREF(__pyx_v_self->key);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->key));
  __pyx_v_self->key = ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":141
 *         self.composition = composition
 *         self.key = key
 *         self.is_extended = is_extended             # <<<<<<<<<<<<<<
 *         self._hash_key = -1
 * 
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_is_extended); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 141, __pyx_L1_error)
  __pyx_v_self->is_extended = __pyx_t_3;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":142
 *         self.key = key
 *         self.is_extended = is_extended
 *         self._hash_key = -1             # <<<<<<<<<<<<<<
 * 
 *     def __hash__(self):
 */
  __pyx_v_self->_hash_key = -1L;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":137
 * cdef class GlycanCompositionFragment(object):
 * 
 *     def __init__(self, mass, composition, key, is_extended=False):             # <<<<<<<<<<<<<<
 *         self.mass = mass
 *         self.composition = composition
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":144
 *         self._hash_key = -1
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         if self._hash_key == -1:
 *             self._hash_key = hash(self._get_glycan_composition())
 */

/* Python wrapper */
static Py_hash_t __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3__hash__(PyObject *__pyx_v_self); /*proto*/
static Py_hash_t __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3__hash__(PyObject *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__hash__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_2__hash__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_hash_t __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_2__hash__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  Py_hash_t __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__hash__", 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":145
 * 
 *     def __hash__(self):
 *         if self._hash_key == -1:             # <<<<<<<<<<<<<<
 *             self._hash_key = hash(self._get_glycan_composition())
 *         return self._hash_key
 */
  __pyx_t_1 = ((__pyx_v_self->_hash_key == -1L) != 0);
  if (__pyx_t_1) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":146
 *     def __hash__(self):
 *         if self._hash_key == -1:
 *             self._hash_key = hash(self._get_glycan_composition())             # <<<<<<<<<<<<<<
 *         return self._hash_key
 * 
 */
    __pyx_t_2 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self->__pyx_vtab)->_get_glycan_composition(__pyx_v_self, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyObject_Hash(__pyx_t_2); if (unlikely(__pyx_t_3 == ((Py_hash_t)-1))) __PYX_ERR(0, 146, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_v_self->_hash_key = __pyx_t_3;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":145
 * 
 *     def __hash__(self):
 *         if self._hash_key == -1:             # <<<<<<<<<<<<<<
 *             self._hash_key = hash(self._get_glycan_composition())
 *         return self._hash_key
 */
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":147
 *         if self._hash_key == -1:
 *             self._hash_key = hash(self._get_glycan_composition())
 *         return self._hash_key             # <<<<<<<<<<<<<<
 * 
 *     cpdef _get_glycan_composition(self):
 */
  __pyx_r = __pyx_v_self->_hash_key;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":144
 *         self._hash_key = -1
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         if self._hash_key == -1:
 *             self._hash_key = hash(self._get_glycan_composition())
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment.__hash__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  if (unlikely(__pyx_r == -1) && !PyErr_Occurred()) __pyx_r = -2;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":149
 *         return self._hash_key
 * 
 *     cpdef _get_glycan_composition(self):             # <<<<<<<<<<<<<<
 *         return _prepare_glycan_composition_from_mapping(self.key)
 * 
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_5_get_glycan_composition(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__get_glycan_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_glycan_composition", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_glycan_composition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 149, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_5_get_glycan_composition)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 149, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":150
 * 
 *     cpdef _get_glycan_composition(self):
 *         return _prepare_glycan_composition_from_mapping(self.key)             # <<<<<<<<<<<<<<
 * 
 *     def __eq__(self, other):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_v_self->key);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__prepare_glycan_composition_from_mapping(((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 150, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":149
 *         return self._hash_key
 * 
 *     cpdef _get_glycan_composition(self):             # <<<<<<<<<<<<<<
 *         return _prepare_glycan_composition_from_mapping(self.key)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment._get_glycan_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_5_get_glycan_composition(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_4_get_glycan_composition[] = "GlycanCompositionFragment._get_glycan_composition(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_5_get_glycan_composition(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_get_glycan_composition (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_4_get_glycan_composition(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_4_get_glycan_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_glycan_composition", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__get_glycan_composition(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 149, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment._get_glycan_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":152
 *         return _prepare_glycan_composition_from_mapping(self.key)
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         return self.key == other.key and self.is_extended == other.is_extended
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_7__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_7__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__eq__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_6__eq__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_6__eq__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__eq__", 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":153
 * 
 *     def __eq__(self, other):
 *         return self.key == other.key and self.is_extended == other.is_extended             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_n_s_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(((PyObject *)__pyx_v_self->key), __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 153, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_4 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_4 < 0)) __PYX_ERR(0, 153, __pyx_L1_error)
  if (__pyx_t_4) {
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else {
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_1 = __pyx_t_3;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->is_extended); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_n_s_is_extended); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 153, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_5 = PyObject_RichCompare(__pyx_t_3, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 153, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_INCREF(__pyx_t_5);
  __pyx_t_1 = __pyx_t_5;
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":152
 *         return _prepare_glycan_composition_from_mapping(self.key)
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         return self.key == other.key and self.is_extended == other.is_extended
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":156
 * 
 *     @staticmethod
 *     cdef GlycanCompositionFragment _create(double mass, CComposition composition, CountTable key, bint is_extended):             # <<<<<<<<<<<<<<
 *         cdef:
 *             GlycanCompositionFragment self
 */

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create(double __pyx_v_mass, struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_composition, struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *__pyx_v_key, int __pyx_v_is_extended) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_create", 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":159
 *         cdef:
 *             GlycanCompositionFragment self
 *         self = GlycanCompositionFragment.__new__(GlycanCompositionFragment)             # <<<<<<<<<<<<<<
 *         self.mass = mass
 *         self.composition = composition
 */
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment(((PyTypeObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment), __pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 159, __pyx_L1_error)
  __Pyx_GOTREF(((PyObject *)__pyx_t_1));
  __pyx_v_self = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":160
 *             GlycanCompositionFragment self
 *         self = GlycanCompositionFragment.__new__(GlycanCompositionFragment)
 *         self.mass = mass             # <<<<<<<<<<<<<<
 *         self.composition = composition
 *         self.key = key
 */
  __pyx_v_self->mass = __pyx_v_mass;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":161
 *         self = GlycanCompositionFragment.__new__(GlycanCompositionFragment)
 *         self.mass = mass
 *         self.composition = composition             # <<<<<<<<<<<<<<
 *         self.key = key
 *         self.is_extended = is_extended
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_composition));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_composition));
  __Pyx_GOTREF(__pyx_v_self->composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->composition));
  __pyx_v_self->composition = __pyx_v_composition;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":162
 *         self.mass = mass
 *         self.composition = composition
 *         self.key = key             # <<<<<<<<<<<<<<
 *         self.is_extended = is_extended
 *         self._hash_key = -1
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_key));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_key));
  __Pyx_GOTREF(__pyx_v_self->key);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->key));
  __pyx_v_self->key = __pyx_v_key;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":163
 *         self.composition = composition
 *         self.key = key
 *         self.is_extended = is_extended             # <<<<<<<<<<<<<<
 *         self._hash_key = -1
 *         return self
 */
  __pyx_v_self->is_extended = __pyx_v_is_extended;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":164
 *         self.key = key
 *         self.is_extended = is_extended
 *         self._hash_key = -1             # <<<<<<<<<<<<<<
 *         return self
 * 
 */
  __pyx_v_self->_hash_key = -1L;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":165
 *         self.is_extended = is_extended
 *         self._hash_key = -1
 *         return self             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, key):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __pyx_r = __pyx_v_self;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":156
 * 
 *     @staticmethod
 *     cdef GlycanCompositionFragment _create(double mass, CComposition composition, CountTable key, bint is_extended):             # <<<<<<<<<<<<<<
 *         cdef:
 *             GlycanCompositionFragment self
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment._create", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_self);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":167
 *         return self
 * 
 *     def __getitem__(self, key):             # <<<<<<<<<<<<<<
 *         if key == "mass":
 *             return self.mass
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_9__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_9__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_8__getitem__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self), ((PyObject *)__pyx_v_key));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_8__getitem__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_key) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":168
 * 
 *     def __getitem__(self, key):
 *         if key == "mass":             # <<<<<<<<<<<<<<
 *             return self.mass
 *         elif key == "composition":
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_key, __pyx_n_s_mass, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 168, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":169
 *     def __getitem__(self, key):
 *         if key == "mass":
 *             return self.mass             # <<<<<<<<<<<<<<
 *         elif key == "composition":
 *             return self.composition
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->mass); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 169, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":168
 * 
 *     def __getitem__(self, key):
 *         if key == "mass":             # <<<<<<<<<<<<<<
 *             return self.mass
 *         elif key == "composition":
 */
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":170
 *         if key == "mass":
 *             return self.mass
 *         elif key == "composition":             # <<<<<<<<<<<<<<
 *             return self.composition
 *         elif key == "key":
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_key, __pyx_n_s_composition, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 170, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":171
 *             return self.mass
 *         elif key == "composition":
 *             return self.composition             # <<<<<<<<<<<<<<
 *         elif key == "key":
 *             return self.key
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->composition));
    __pyx_r = ((PyObject *)__pyx_v_self->composition);
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":170
 *         if key == "mass":
 *             return self.mass
 *         elif key == "composition":             # <<<<<<<<<<<<<<
 *             return self.composition
 *         elif key == "key":
 */
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":172
 *         elif key == "composition":
 *             return self.composition
 *         elif key == "key":             # <<<<<<<<<<<<<<
 *             return self.key
 *         elif key == 'is_extended':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_key, __pyx_n_s_key, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 172, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":173
 *             return self.composition
 *         elif key == "key":
 *             return self.key             # <<<<<<<<<<<<<<
 *         elif key == 'is_extended':
 *             return self.is_extended
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->key));
    __pyx_r = ((PyObject *)__pyx_v_self->key);
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":172
 *         elif key == "composition":
 *             return self.composition
 *         elif key == "key":             # <<<<<<<<<<<<<<
 *             return self.key
 *         elif key == 'is_extended':
 */
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":174
 *         elif key == "key":
 *             return self.key
 *         elif key == 'is_extended':             # <<<<<<<<<<<<<<
 *             return self.is_extended
 *         else:
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_key, __pyx_n_s_is_extended, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 174, __pyx_L1_error)
  if (likely(__pyx_t_1)) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":175
 *             return self.key
 *         elif key == 'is_extended':
 *             return self.is_extended             # <<<<<<<<<<<<<<
 *         else:
 *             raise KeyError(key)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->is_extended); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 175, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":174
 *         elif key == "key":
 *             return self.key
 *         elif key == 'is_extended':             # <<<<<<<<<<<<<<
 *             return self.is_extended
 *         else:
 */
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":177
 *             return self.is_extended
 *         else:
 *             raise KeyError(key)             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, key, value):
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_KeyError, __pyx_v_key); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 177, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 177, __pyx_L1_error)
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":167
 *         return self
 * 
 *     def __getitem__(self, key):             # <<<<<<<<<<<<<<
 *         if key == "mass":
 *             return self.mass
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":179
 *             raise KeyError(key)
 * 
 *     def __setitem__(self, key, value):             # <<<<<<<<<<<<<<
 *         if key == "mass":
 *             self.mass = value
 */

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_10__setitem__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self), ((PyObject *)__pyx_v_key), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_10__setitem__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_key, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  double __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":180
 * 
 *     def __setitem__(self, key, value):
 *         if key == "mass":             # <<<<<<<<<<<<<<
 *             self.mass = value
 *         elif key == "composition":
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_key, __pyx_n_s_mass, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 180, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":181
 *     def __setitem__(self, key, value):
 *         if key == "mass":
 *             self.mass = value             # <<<<<<<<<<<<<<
 *         elif key == "composition":
 *             self.composition = value
 */
    __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 181, __pyx_L1_error)
    __pyx_v_self->mass = __pyx_t_2;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":180
 * 
 *     def __setitem__(self, key, value):
 *         if key == "mass":             # <<<<<<<<<<<<<<
 *             self.mass = value
 *         elif key == "composition":
 */
    goto __pyx_L3;
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":182
 *         if key == "mass":
 *             self.mass = value
 *         elif key == "composition":             # <<<<<<<<<<<<<<
 *             self.composition = value
 *         elif key == "key":
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_key, __pyx_n_s_composition, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 182, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":183
 *             self.mass = value
 *         elif key == "composition":
 *             self.composition = value             # <<<<<<<<<<<<<<
 *         elif key == "key":
 *             self.key = value
 */
    if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 183, __pyx_L1_error)
    __pyx_t_3 = __pyx_v_value;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->composition);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->composition));
    __pyx_v_self->composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":182
 *         if key == "mass":
 *             self.mass = value
 *         elif key == "composition":             # <<<<<<<<<<<<<<
 *             self.composition = value
 *         elif key == "key":
 */
    goto __pyx_L3;
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":184
 *         elif key == "composition":
 *             self.composition = value
 *         elif key == "key":             # <<<<<<<<<<<<<<
 *             self.key = value
 *         elif key == 'is_extended':
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_key, __pyx_n_s_key, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 184, __pyx_L1_error)
  if (__pyx_t_1) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":185
 *             self.composition = value
 *         elif key == "key":
 *             self.key = value             # <<<<<<<<<<<<<<
 *         elif key == 'is_extended':
 *             self.is_extended = value
 */
    if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_14glycopeptidepy_2_c_11count_table_CountTable))))) __PYX_ERR(0, 185, __pyx_L1_error)
    __pyx_t_3 = __pyx_v_value;
    __Pyx_INCREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->key);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->key));
    __pyx_v_self->key = ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":184
 *         elif key == "composition":
 *             self.composition = value
 *         elif key == "key":             # <<<<<<<<<<<<<<
 *             self.key = value
 *         elif key == 'is_extended':
 */
    goto __pyx_L3;
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":186
 *         elif key == "key":
 *             self.key = value
 *         elif key == 'is_extended':             # <<<<<<<<<<<<<<
 *             self.is_extended = value
 *         else:
 */
  __pyx_t_1 = (__Pyx_PyString_Equals(__pyx_v_key, __pyx_n_s_is_extended, Py_EQ)); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 186, __pyx_L1_error)
  if (likely(__pyx_t_1)) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":187
 *             self.key = value
 *         elif key == 'is_extended':
 *             self.is_extended = value             # <<<<<<<<<<<<<<
 *         else:
 *             raise KeyError(key)
 */
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 187, __pyx_L1_error)
    __pyx_v_self->is_extended = __pyx_t_1;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":186
 *         elif key == "key":
 *             self.key = value
 *         elif key == 'is_extended':             # <<<<<<<<<<<<<<
 *             self.is_extended = value
 *         else:
 */
    goto __pyx_L3;
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":189
 *             self.is_extended = value
 *         else:
 *             raise KeyError(key)             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  /*else*/ {
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(__pyx_builtin_KeyError, __pyx_v_key); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 189, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(0, 189, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":179
 *             raise KeyError(key)
 * 
 *     def __setitem__(self, key, value):             # <<<<<<<<<<<<<<
 *         if key == "mass":
 *             self.mass = value
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":191
 *             raise KeyError(key)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         template = "{self.__class__.__name__}({self.mass}, {self.composition}, {self.key}, {self.is_extended})"
 *         return template.format(self=self)
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_13__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_13__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_12__repr__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_12__repr__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self) {
  PyObject *__pyx_v_template = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":192
 * 
 *     def __repr__(self):
 *         template = "{self.__class__.__name__}({self.mass}, {self.composition}, {self.key}, {self.is_extended})"             # <<<<<<<<<<<<<<
 *         return template.format(self=self)
 * 
 */
  __Pyx_INCREF(__pyx_kp_s_self___class_____name___self_ma);
  __pyx_v_template = __pyx_kp_s_self___class_____name___self_ma;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":193
 *     def __repr__(self):
 *         template = "{self.__class__.__name__}({self.mass}, {self.composition}, {self.key}, {self.is_extended})"
 *         return template.format(self=self)             # <<<<<<<<<<<<<<
 * 
 *     cpdef GlycanCompositionFragment copy(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_template, __pyx_n_s_format); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_self, ((PyObject *)__pyx_v_self)) < 0) __PYX_ERR(0, 193, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 193, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":191
 *             raise KeyError(key)
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         template = "{self.__class__.__name__}({self.mass}, {self.composition}, {self.key}, {self.is_extended})"
 *         return template.format(self=self)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_template);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":195
 *         return template.format(self=self)
 * 
 *     cpdef GlycanCompositionFragment copy(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             GlycanCompositionFragment inst
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_15copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_copy(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_inst = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_hash_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_copy); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 195, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_15copy)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 195, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment))))) __PYX_ERR(0, 195, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":200
 *         inst = GlycanCompositionFragment._create(
 *             self.mass,
 *             self.composition.clone() if self.composition is not None else None, self.key.copy(),             # <<<<<<<<<<<<<<
 *             self.is_extended)
 *         inst._hash_key = self._hash_key
 */
  __pyx_t_5 = (((PyObject *)__pyx_v_self->composition) != Py_None);
  if ((__pyx_t_5 != 0)) {
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_v_self->composition->__pyx_vtab)->clone(__pyx_v_self->composition, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 200, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = __pyx_t_2;
    __pyx_t_2 = 0;
  } else {
    __Pyx_INCREF(Py_None);
    __pyx_t_1 = Py_None;
  }
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_self->key->__pyx_vtab)->copy(__pyx_v_self->key, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 200, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":198
 *         cdef:
 *             GlycanCompositionFragment inst
 *         inst = GlycanCompositionFragment._create(             # <<<<<<<<<<<<<<
 *             self.mass,
 *             self.composition.clone() if self.composition is not None else None, self.key.copy(),
 */
  __pyx_t_3 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create(__pyx_v_self->mass, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1), ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_2), __pyx_v_self->is_extended)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 198, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_inst = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":202
 *             self.composition.clone() if self.composition is not None else None, self.key.copy(),
 *             self.is_extended)
 *         inst._hash_key = self._hash_key             # <<<<<<<<<<<<<<
 *         return inst
 * 
 */
  __pyx_t_6 = __pyx_v_self->_hash_key;
  __pyx_v_inst->_hash_key = __pyx_t_6;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":203
 *             self.is_extended)
 *         inst._hash_key = self._hash_key
 *         return inst             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_inst));
  __pyx_r = __pyx_v_inst;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":195
 *         return template.format(self=self)
 * 
 *     cpdef GlycanCompositionFragment copy(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             GlycanCompositionFragment inst
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_15copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_14copy[] = "GlycanCompositionFragment.copy(self) -> GlycanCompositionFragment";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_15copy(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("copy (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_14copy(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_14copy(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("copy", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_copy(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 195, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment.copy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pxd":10
 * cdef class GlycanCompositionFragment(object):
 *     cdef:
 *         public double mass             # <<<<<<<<<<<<<<
 *         public CComposition composition
 *         public CountTable key
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_4mass_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_4mass_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_4mass___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_4mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 10, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment.mass.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_4mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_4mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_4mass_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_4mass_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 10, __pyx_L1_error)
  __pyx_v_self->mass = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment.mass.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pxd":11
 *     cdef:
 *         public double mass
 *         public CComposition composition             # <<<<<<<<<<<<<<
 *         public CountTable key
 *         public bint is_extended
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->composition));
  __pyx_r = ((PyObject *)__pyx_v_self->composition);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(2, 11, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->composition));
  __pyx_v_self->composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment.composition.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->composition));
  __pyx_v_self->composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pxd":12
 *         public double mass
 *         public CComposition composition
 *         public CountTable key             # <<<<<<<<<<<<<<
 *         public bint is_extended
 *         public Py_hash_t _hash_key
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->key));
  __pyx_r = ((PyObject *)__pyx_v_self->key);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_14glycopeptidepy_2_c_11count_table_CountTable))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->key);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->key));
  __pyx_v_self->key = ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment.key.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->key);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->key));
  __pyx_v_self->key = ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pxd":13
 *         public CComposition composition
 *         public CountTable key
 *         public bint is_extended             # <<<<<<<<<<<<<<
 *         public Py_hash_t _hash_key
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11is_extended_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11is_extended_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11is_extended___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11is_extended___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->is_extended); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment.is_extended.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11is_extended_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11is_extended_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11is_extended_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11is_extended_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_v_self->is_extended = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment.is_extended.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pxd":14
 *         public CountTable key
 *         public bint is_extended
 *         public Py_hash_t _hash_key             # <<<<<<<<<<<<<<
 * 
 *     cpdef GlycanCompositionFragment copy(self)
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_9_hash_key_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_9_hash_key_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_9_hash_key___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_9_hash_key___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromHash_t(__pyx_v_self->_hash_key); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment._hash_key.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_9_hash_key_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_9_hash_key_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_9_hash_key_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_9_hash_key_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_hash_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_AsHash_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (Py_hash_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 14, __pyx_L1_error)
  __pyx_v_self->_hash_key = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment._hash_key.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_17__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_16__reduce_cython__[] = "GlycanCompositionFragment.__reduce_cython__(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_17__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_16__reduce_cython__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_16__reduce_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._hash_key, self.composition, self.is_extended, self.key, self.mass)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_FromHash_t(__pyx_v_self->_hash_key); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->is_extended); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyFloat_FromDouble(__pyx_v_self->mass); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = PyTuple_New(5); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->composition));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->composition));
  PyTuple_SET_ITEM(__pyx_t_4, 1, ((PyObject *)__pyx_v_self->composition));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->key));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->key));
  PyTuple_SET_ITEM(__pyx_t_4, 3, ((PyObject *)__pyx_v_self->key));
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_4, 4, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_4);
  __pyx_t_4 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._hash_key, self.composition, self.is_extended, self.key, self.mass)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_4 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v__dict = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "(tree fragment)":7
 *     state = (self._hash_key, self.composition, self.is_extended, self.key, self.mass)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_5 = (__pyx_v__dict != Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_4 = PyTuple_New(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v__dict);
    __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_3));
    __pyx_t_3 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.composition is not None or self.key is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._hash_key, self.composition, self.is_extended, self.key, self.mass)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.composition is not None or self.key is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_GlycanCompositionFragment, (type(self), 0x18c151e, None), state
 */
  /*else*/ {
    __pyx_t_5 = (((PyObject *)__pyx_v_self->composition) != Py_None);
    __pyx_t_7 = (__pyx_t_5 != 0);
    if (!__pyx_t_7) {
    } else {
      __pyx_t_6 = __pyx_t_7;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_7 = (((PyObject *)__pyx_v_self->key) != Py_None);
    __pyx_t_5 = (__pyx_t_7 != 0);
    __pyx_t_6 = __pyx_t_5;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_6;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.composition is not None or self.key is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_GlycanCompositionFragment, (type(self), 0x18c151e, None), state
 *     else:
 */
  __pyx_t_6 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":13
 *         use_setstate = self.composition is not None or self.key is not None
 *     if use_setstate:
 *         return __pyx_unpickle_GlycanCompositionFragment, (type(self), 0x18c151e, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_GlycanCompositionFragment, (type(self), 0x18c151e, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_GlycanComposition_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_25957662);
    __Pyx_GIVEREF(__pyx_int_25957662);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_25957662);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_4, 2, Py_None);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_4);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_3 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.composition is not None or self.key is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_GlycanCompositionFragment, (type(self), 0x18c151e, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_GlycanCompositionFragment, (type(self), 0x18c151e, None), state
 *     else:
 *         return __pyx_unpickle_GlycanCompositionFragment, (type(self), 0x18c151e, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_GlycanCompositionFragment__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_GlycanComposition_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_4, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_25957662);
    __Pyx_GIVEREF(__pyx_int_25957662);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_int_25957662);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_state);
    __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_4);
    __pyx_t_2 = 0;
    __pyx_t_4 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_GlycanCompositionFragment, (type(self), 0x18c151e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_GlycanCompositionFragment__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_19__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_18__setstate_cython__[] = "GlycanCompositionFragment.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_19__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_18__setstate_cython__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_18__setstate_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_GlycanCompositionFragment, (type(self), 0x18c151e, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_GlycanCompositionFragment__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle_GlycanCompositionFragment__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_GlycanCompositionFragment, (type(self), 0x18c151e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_GlycanCompositionFragment__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":208
 * cdef dict hexnac_hex_composition_cache = dict()
 * 
 * cdef CComposition hexnac_hex_composition(long hexnac, long hexose):             # <<<<<<<<<<<<<<
 *     cdef:
 *         tuple key
 */

static struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_hexnac_hex_composition(long __pyx_v_hexnac, long __pyx_v_hexose) {
  PyObject *__pyx_v_key = 0;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_result = 0;
  PyObject *__pyx_v_ptemp;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("hexnac_hex_composition", 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":214
 *         PyObject* ptemp
 * 
 *     key = (hexnac, hexose)             # <<<<<<<<<<<<<<
 *     ptemp = PyDict_GetItem(hexnac_hex_composition_cache, key)
 *     if ptemp == NULL:
 */
  __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v_hexnac); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v_hexose); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 214, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_key = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":215
 * 
 *     key = (hexnac, hexose)
 *     ptemp = PyDict_GetItem(hexnac_hex_composition_cache, key)             # <<<<<<<<<<<<<<
 *     if ptemp == NULL:
 *         result = (_HEXNAC_composition * hexnac)
 */
  __pyx_t_3 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_hexnac_hex_composition_cache;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_v_ptemp = PyDict_GetItem(__pyx_t_3, __pyx_v_key);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":216
 *     key = (hexnac, hexose)
 *     ptemp = PyDict_GetItem(hexnac_hex_composition_cache, key)
 *     if ptemp == NULL:             # <<<<<<<<<<<<<<
 *         result = (_HEXNAC_composition * hexnac)
 *         if hexose > 0:
 */
  __pyx_t_4 = ((__pyx_v_ptemp == NULL) != 0);
  if (__pyx_t_4) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":217
 *     ptemp = PyDict_GetItem(hexnac_hex_composition_cache, key)
 *     if ptemp == NULL:
 *         result = (_HEXNAC_composition * hexnac)             # <<<<<<<<<<<<<<
 *         if hexose > 0:
 *             result.add_from(<CComposition>(_HEX_composition * hexose))
 */
    __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_v_hexnac); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_2 = PyNumber_Multiply(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC_composition), __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 217, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 217, __pyx_L1_error)
    __pyx_v_result = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":218
 *     if ptemp == NULL:
 *         result = (_HEXNAC_composition * hexnac)
 *         if hexose > 0:             # <<<<<<<<<<<<<<
 *             result.add_from(<CComposition>(_HEX_composition * hexose))
 *         PyDict_SetItem(hexnac_hex_composition_cache, key, result.clone())
 */
    __pyx_t_4 = ((__pyx_v_hexose > 0) != 0);
    if (__pyx_t_4) {

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":219
 *         result = (_HEXNAC_composition * hexnac)
 *         if hexose > 0:
 *             result.add_from(<CComposition>(_HEX_composition * hexose))             # <<<<<<<<<<<<<<
 *         PyDict_SetItem(hexnac_hex_composition_cache, key, result.clone())
 *         return result
 */
      __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v_hexose); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 219, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = PyNumber_Multiply(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX_composition), __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 219, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_v_result->__pyx_vtab)->add_from(__pyx_v_result, __pyx_t_3)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 219, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":218
 *     if ptemp == NULL:
 *         result = (_HEXNAC_composition * hexnac)
 *         if hexose > 0:             # <<<<<<<<<<<<<<
 *             result.add_from(<CComposition>(_HEX_composition * hexose))
 *         PyDict_SetItem(hexnac_hex_composition_cache, key, result.clone())
 */
    }

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":220
 *         if hexose > 0:
 *             result.add_from(<CComposition>(_HEX_composition * hexose))
 *         PyDict_SetItem(hexnac_hex_composition_cache, key, result.clone())             # <<<<<<<<<<<<<<
 *         return result
 *     else:
 */
    __pyx_t_2 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_hexnac_hex_composition_cache;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_v_result->__pyx_vtab)->clone(__pyx_v_result, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 220, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyDict_SetItem(__pyx_t_2, __pyx_v_key, __pyx_t_3); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 220, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":221
 *             result.add_from(<CComposition>(_HEX_composition * hexose))
 *         PyDict_SetItem(hexnac_hex_composition_cache, key, result.clone())
 *         return result             # <<<<<<<<<<<<<<
 *     else:
 *         result = <CComposition>ptemp
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __Pyx_INCREF(((PyObject *)__pyx_v_result));
    __pyx_r = __pyx_v_result;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":216
 *     key = (hexnac, hexose)
 *     ptemp = PyDict_GetItem(hexnac_hex_composition_cache, key)
 *     if ptemp == NULL:             # <<<<<<<<<<<<<<
 *         result = (_HEXNAC_composition * hexnac)
 *         if hexose > 0:
 */
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":223
 *         return result
 *     else:
 *         result = <CComposition>ptemp             # <<<<<<<<<<<<<<
 *         return result.clone()
 * 
 */
  /*else*/ {
    __pyx_t_3 = ((PyObject *)__pyx_v_ptemp);
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_result = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":224
 *     else:
 *         result = <CComposition>ptemp
 *         return result.clone()             # <<<<<<<<<<<<<<
 * 
 * 
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_v_result->__pyx_vtab)->clone(__pyx_v_result, 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 224, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_3);
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":208
 * cdef dict hexnac_hex_composition_cache = dict()
 * 
 * cdef CComposition hexnac_hex_composition(long hexnac, long hexose):             # <<<<<<<<<<<<<<
 *     cdef:
 *         tuple key
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.hexnac_hex_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF((PyObject *)__pyx_v_result);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":243
 *     _composition_tree = _composition_tree_root
 * 
 *     def __init__(self, peptide, extended=True, use_query=False, extended_fucosylation=False, **kwargs):             # <<<<<<<<<<<<<<
 *         self.extended = extended
 *         self.extended_fucosylation = extended_fucosylation
 */

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_peptide = 0;
  PyObject *__pyx_v_extended = 0;
  PyObject *__pyx_v_use_query = 0;
  PyObject *__pyx_v_extended_fucosylation = 0;
  CYTHON_UNUSED PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_peptide,&__pyx_n_s_extended,&__pyx_n_s_use_query,&__pyx_n_s_extended_fucosylation,0};
    PyObject* values[4] = {0,0,0,0};
    values[1] = ((PyObject *)Py_True);
    values[2] = ((PyObject *)Py_False);
    values[3] = ((PyObject *)Py_False);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_peptide)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_extended);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_use_query);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_extended_fucosylation);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 243, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_peptide = values[0];
    __pyx_v_extended = values[1];
    __pyx_v_use_query = values[2];
    __pyx_v_extended_fucosylation = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 243, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy___init__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self), __pyx_v_peptide, __pyx_v_extended, __pyx_v_use_query, __pyx_v_extended_fucosylation, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy___init__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_peptide, PyObject *__pyx_v_extended, PyObject *__pyx_v_use_query, PyObject *__pyx_v_extended_fucosylation, CYTHON_UNUSED PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":244
 * 
 *     def __init__(self, peptide, extended=True, use_query=False, extended_fucosylation=False, **kwargs):
 *         self.extended = extended             # <<<<<<<<<<<<<<
 *         self.extended_fucosylation = extended_fucosylation
 *         super(StubGlycopeptideStrategy, self).__init__(peptide, use_query)
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_extended); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 244, __pyx_L1_error)
  __pyx_v_self->extended = __pyx_t_1;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":245
 *     def __init__(self, peptide, extended=True, use_query=False, extended_fucosylation=False, **kwargs):
 *         self.extended = extended
 *         self.extended_fucosylation = extended_fucosylation             # <<<<<<<<<<<<<<
 *         super(StubGlycopeptideStrategy, self).__init__(peptide, use_query)
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_extended_fucosylation); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 245, __pyx_L1_error)
  __pyx_v_self->extended_fucosylation = __pyx_t_1;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":246
 *         self.extended = extended
 *         self.extended_fucosylation = extended_fucosylation
 *         super(StubGlycopeptideStrategy, self).__init__(peptide, use_query)             # <<<<<<<<<<<<<<
 * 
 *     def __next__(self):
 */
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy));
  __Pyx_GIVEREF(((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy));
  __Pyx_INCREF(((PyObject *)__pyx_v_self));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self));
  __pyx_t_4 = __Pyx_PyObject_Call(__pyx_builtin_super, __pyx_t_3, NULL); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_init); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_peptide, __pyx_v_use_query};
    __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 246, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_peptide, __pyx_v_use_query};
    __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 246, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_2);
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 246, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(__pyx_v_peptide);
    __Pyx_GIVEREF(__pyx_v_peptide);
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_peptide);
    __Pyx_INCREF(__pyx_v_use_query);
    __Pyx_GIVEREF(__pyx_v_use_query);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_use_query);
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 246, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":243
 *     _composition_tree = _composition_tree_root
 * 
 *     def __init__(self, peptide, extended=True, use_query=False, extended_fucosylation=False, **kwargs):             # <<<<<<<<<<<<<<
 *         self.extended = extended
 *         self.extended_fucosylation = extended_fucosylation
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":248
 *         super(StubGlycopeptideStrategy, self).__init__(peptide, use_query)
 * 
 *     def __next__(self):             # <<<<<<<<<<<<<<
 *         if self._generator is None:
 *             self._generator = self.stub_fragments()
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_3__next__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_3__next__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__next__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_2__next__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_2__next__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__next__", 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":249
 * 
 *     def __next__(self):
 *         if self._generator is None:             # <<<<<<<<<<<<<<
 *             self._generator = self.stub_fragments()
 *         return next(self._generator)
 */
  __pyx_t_1 = (__pyx_v_self->__pyx_base._generator == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":250
 *     def __next__(self):
 *         if self._generator is None:
 *             self._generator = self.stub_fragments()             # <<<<<<<<<<<<<<
 *         return next(self._generator)
 * 
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_stub_fragments); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 250, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 250, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GIVEREF(__pyx_t_3);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base._generator);
    __Pyx_DECREF(__pyx_v_self->__pyx_base._generator);
    __pyx_v_self->__pyx_base._generator = __pyx_t_3;
    __pyx_t_3 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":249
 * 
 *     def __next__(self):
 *         if self._generator is None:             # <<<<<<<<<<<<<<
 *             self._generator = self.stub_fragments()
 *         return next(self._generator)
 */
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":251
 *         if self._generator is None:
 *             self._generator = self.stub_fragments()
 *         return next(self._generator)             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint _validate_glycan_composition(self, CountTable aggregate_glycosylation, glycan):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_3 = __pyx_v_self->__pyx_base._generator;
  __Pyx_INCREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyIter_Next(__pyx_t_3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 251, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":248
 *         super(StubGlycopeptideStrategy, self).__init__(peptide, use_query)
 * 
 *     def __next__(self):             # <<<<<<<<<<<<<<
 *         if self._generator is None:
 *             self._generator = self.stub_fragments()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.__next__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":253
 *         return next(self._generator)
 * 
 *     cpdef bint _validate_glycan_composition(self, CountTable aggregate_glycosylation, glycan):             # <<<<<<<<<<<<<<
 *         cdef:
 *             bint invalid = False
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_5_validate_glycan_composition(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy__validate_glycan_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *__pyx_v_aggregate_glycosylation, PyObject *__pyx_v_glycan, int __pyx_skip_dispatch) {
  int __pyx_v_invalid;
  long __pyx_v_v;
  long __pyx_v_ov;
  PyObject *__pyx_v_key = 0;
  PyObject *__pyx_v_pkey;
  struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTableIterator *__pyx_v_iterator = 0;
  PyObject *__pyx_v_value = NULL;
  PyObject *__pyx_v_getter = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  PyObject *(*__pyx_t_9)(PyObject *);
  long __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_validate_glycan_composition", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_validate_glycan_composition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 253, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_5_validate_glycan_composition)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_aggregate_glycosylation), __pyx_v_glycan};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 253, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_aggregate_glycosylation), __pyx_v_glycan};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 253, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 253, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(((PyObject *)__pyx_v_aggregate_glycosylation));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_aggregate_glycosylation));
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, ((PyObject *)__pyx_v_aggregate_glycosylation));
          __Pyx_INCREF(__pyx_v_glycan);
          __Pyx_GIVEREF(__pyx_v_glycan);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_glycan);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 253, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_7 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 253, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_7;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":255
 *     cpdef bint _validate_glycan_composition(self, CountTable aggregate_glycosylation, glycan):
 *         cdef:
 *             bint invalid = False             # <<<<<<<<<<<<<<
 *             int i, n
 *             long v, ov
 */
  __pyx_v_invalid = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":261
 *             PyObject* pkey
 *             CountTableIterator iterator
 *         if self._use_query:             # <<<<<<<<<<<<<<
 *             for key, value in aggregate_glycosylation.items():
 *                 if glycan.query(key) < value:
 */
  __pyx_t_7 = (__pyx_v_self->__pyx_base._use_query != 0);
  if (__pyx_t_7) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":262
 *             CountTableIterator iterator
 *         if self._use_query:
 *             for key, value in aggregate_glycosylation.items():             # <<<<<<<<<<<<<<
 *                 if glycan.query(key) < value:
 *                     invalid = True
 */
    __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_aggregate_glycosylation->__pyx_vtab)->items(__pyx_v_aggregate_glycosylation, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 262, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (unlikely(__pyx_t_1 == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
      __PYX_ERR(0, 262, __pyx_L1_error)
    }
    __pyx_t_2 = __pyx_t_1; __Pyx_INCREF(__pyx_t_2); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    for (;;) {
      if (__pyx_t_8 >= PyList_GET_SIZE(__pyx_t_2)) break;
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_1 = PyList_GET_ITEM(__pyx_t_2, __pyx_t_8); __Pyx_INCREF(__pyx_t_1); __pyx_t_8++; if (unlikely(0 < 0)) __PYX_ERR(0, 262, __pyx_L1_error)
      #else
      __pyx_t_1 = PySequence_ITEM(__pyx_t_2, __pyx_t_8); __pyx_t_8++; if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 262, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      #endif
      if ((likely(PyTuple_CheckExact(__pyx_t_1))) || (PyList_CheckExact(__pyx_t_1))) {
        PyObject* sequence = __pyx_t_1;
        Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
        if (unlikely(size != 2)) {
          if (size > 2) __Pyx_RaiseTooManyValuesError(2);
          else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
          __PYX_ERR(0, 262, __pyx_L1_error)
        }
        #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
        if (likely(PyTuple_CheckExact(sequence))) {
          __pyx_t_3 = PyTuple_GET_ITEM(sequence, 0); 
          __pyx_t_6 = PyTuple_GET_ITEM(sequence, 1); 
        } else {
          __pyx_t_3 = PyList_GET_ITEM(sequence, 0); 
          __pyx_t_6 = PyList_GET_ITEM(sequence, 1); 
        }
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_6);
        #else
        __pyx_t_3 = PySequence_ITEM(sequence, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 262, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_6 = PySequence_ITEM(sequence, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 262, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_6);
        #endif
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      } else {
        Py_ssize_t index = -1;
        __pyx_t_4 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 262, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_9 = Py_TYPE(__pyx_t_4)->tp_iternext;
        index = 0; __pyx_t_3 = __pyx_t_9(__pyx_t_4); if (unlikely(!__pyx_t_3)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_3);
        index = 1; __pyx_t_6 = __pyx_t_9(__pyx_t_4); if (unlikely(!__pyx_t_6)) goto __pyx_L6_unpacking_failed;
        __Pyx_GOTREF(__pyx_t_6);
        if (__Pyx_IternextUnpackEndCheck(__pyx_t_9(__pyx_t_4), 2) < 0) __PYX_ERR(0, 262, __pyx_L1_error)
        __pyx_t_9 = NULL;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        goto __pyx_L7_unpacking_done;
        __pyx_L6_unpacking_failed:;
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        __pyx_t_9 = NULL;
        if (__Pyx_IterFinish() == 0) __Pyx_RaiseNeedMoreValuesError(index);
        __PYX_ERR(0, 262, __pyx_L1_error)
        __pyx_L7_unpacking_done:;
      }
      __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_3);
      __pyx_t_3 = 0;
      __Pyx_XDECREF_SET(__pyx_v_value, __pyx_t_6);
      __pyx_t_6 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":263
 *         if self._use_query:
 *             for key, value in aggregate_glycosylation.items():
 *                 if glycan.query(key) < value:             # <<<<<<<<<<<<<<
 *                     invalid = True
 *                     break
 */
      __pyx_t_6 = __Pyx_PyObject_GetAttrStr(__pyx_v_glycan, __pyx_n_s_query); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 263, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_6);
      __pyx_t_3 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_3)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_3);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_3, __pyx_v_key) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_v_key);
      __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 263, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_6 = PyObject_RichCompare(__pyx_t_1, __pyx_v_value, Py_LT); __Pyx_XGOTREF(__pyx_t_6); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 263, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_7 = __Pyx_PyObject_IsTrue(__pyx_t_6); if (unlikely(__pyx_t_7 < 0)) __PYX_ERR(0, 263, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (__pyx_t_7) {

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":264
 *             for key, value in aggregate_glycosylation.items():
 *                 if glycan.query(key) < value:
 *                     invalid = True             # <<<<<<<<<<<<<<
 *                     break
 *         else:
 */
        __pyx_v_invalid = 1;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":265
 *                 if glycan.query(key) < value:
 *                     invalid = True
 *                     break             # <<<<<<<<<<<<<<
 *         else:
 *             iterator = CountTableIterator._create(aggregate_glycosylation)
 */
        goto __pyx_L5_break;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":263
 *         if self._use_query:
 *             for key, value in aggregate_glycosylation.items():
 *                 if glycan.query(key) < value:             # <<<<<<<<<<<<<<
 *                     invalid = True
 *                     break
 */
      }

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":262
 *             CountTableIterator iterator
 *         if self._use_query:
 *             for key, value in aggregate_glycosylation.items():             # <<<<<<<<<<<<<<
 *                 if glycan.query(key) < value:
 *                     invalid = True
 */
    }
    __pyx_L5_break:;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":261
 *             PyObject* pkey
 *             CountTableIterator iterator
 *         if self._use_query:             # <<<<<<<<<<<<<<
 *             for key, value in aggregate_glycosylation.items():
 *                 if glycan.query(key) < value:
 */
    goto __pyx_L3;
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":267
 *                     break
 *         else:
 *             iterator = CountTableIterator._create(aggregate_glycosylation)             # <<<<<<<<<<<<<<
 *             pkey = NULL
 *             getter = glycan._getitem_fast
 */
  /*else*/ {
    __pyx_t_2 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTableIterator->_create(__pyx_v_aggregate_glycosylation)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 267, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_iterator = ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTableIterator *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":268
 *         else:
 *             iterator = CountTableIterator._create(aggregate_glycosylation)
 *             pkey = NULL             # <<<<<<<<<<<<<<
 *             getter = glycan._getitem_fast
 *             while iterator.has_more():
 */
    __pyx_v_pkey = NULL;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":269
 *             iterator = CountTableIterator._create(aggregate_glycosylation)
 *             pkey = NULL
 *             getter = glycan._getitem_fast             # <<<<<<<<<<<<<<
 *             while iterator.has_more():
 *                 iterator.get_next_value(&pkey, &v)
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_glycan, __pyx_n_s_getitem_fast); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 269, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_getter = __pyx_t_2;
    __pyx_t_2 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":270
 *             pkey = NULL
 *             getter = glycan._getitem_fast
 *             while iterator.has_more():             # <<<<<<<<<<<<<<
 *                 iterator.get_next_value(&pkey, &v)
 *                 key = <object>pkey
 */
    while (1) {
      __pyx_t_7 = (((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTableIterator *)__pyx_v_iterator->__pyx_vtab)->has_more(__pyx_v_iterator) != 0);
      if (!__pyx_t_7) break;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":271
 *             getter = glycan._getitem_fast
 *             while iterator.has_more():
 *                 iterator.get_next_value(&pkey, &v)             # <<<<<<<<<<<<<<
 *                 key = <object>pkey
 *                 ov = PyInt_AsLong(getter(key))
 */
      (void)(((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTableIterator *)__pyx_v_iterator->__pyx_vtab)->get_next_value(__pyx_v_iterator, (&__pyx_v_pkey), (&__pyx_v_v)));

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":272
 *             while iterator.has_more():
 *                 iterator.get_next_value(&pkey, &v)
 *                 key = <object>pkey             # <<<<<<<<<<<<<<
 *                 ov = PyInt_AsLong(getter(key))
 *                 if ov < v:
 */
      __pyx_t_2 = ((PyObject *)__pyx_v_pkey);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_XDECREF_SET(__pyx_v_key, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":273
 *                 iterator.get_next_value(&pkey, &v)
 *                 key = <object>pkey
 *                 ov = PyInt_AsLong(getter(key))             # <<<<<<<<<<<<<<
 *                 if ov < v:
 *                     invalid = True
 */
      __Pyx_INCREF(__pyx_v_getter);
      __pyx_t_6 = __pyx_v_getter; __pyx_t_1 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_6))) {
        __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_6);
        if (likely(__pyx_t_1)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_6);
          __Pyx_INCREF(__pyx_t_1);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_6, function);
        }
      }
      __pyx_t_2 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_6, __pyx_t_1, __pyx_v_key) : __Pyx_PyObject_CallOneArg(__pyx_t_6, __pyx_v_key);
      __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 273, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __pyx_t_10 = PyInt_AsLong(__pyx_t_2); if (unlikely(__pyx_t_10 == ((long)-1L) && PyErr_Occurred())) __PYX_ERR(0, 273, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_v_ov = __pyx_t_10;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":274
 *                 key = <object>pkey
 *                 ov = PyInt_AsLong(getter(key))
 *                 if ov < v:             # <<<<<<<<<<<<<<
 *                     invalid = True
 *                     break
 */
      __pyx_t_7 = ((__pyx_v_ov < __pyx_v_v) != 0);
      if (__pyx_t_7) {

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":275
 *                 ov = PyInt_AsLong(getter(key))
 *                 if ov < v:
 *                     invalid = True             # <<<<<<<<<<<<<<
 *                     break
 *         return invalid
 */
        __pyx_v_invalid = 1;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":276
 *                 if ov < v:
 *                     invalid = True
 *                     break             # <<<<<<<<<<<<<<
 *         return invalid
 * 
 */
        goto __pyx_L10_break;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":274
 *                 key = <object>pkey
 *                 ov = PyInt_AsLong(getter(key))
 *                 if ov < v:             # <<<<<<<<<<<<<<
 *                     invalid = True
 *                     break
 */
      }
    }
    __pyx_L10_break:;
  }
  __pyx_L3:;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":277
 *                     invalid = True
 *                     break
 *         return invalid             # <<<<<<<<<<<<<<
 * 
 *     cpdef GlycanCompositionFragment fucosylate_increment(self, GlycanCompositionFragment shift):
 */
  __pyx_r = __pyx_v_invalid;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":253
 *         return next(self._generator)
 * 
 *     cpdef bint _validate_glycan_composition(self, CountTable aggregate_glycosylation, glycan):             # <<<<<<<<<<<<<<
 *         cdef:
 *             bint invalid = False
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy._validate_glycan_composition", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF((PyObject *)__pyx_v_iterator);
  __Pyx_XDECREF(__pyx_v_value);
  __Pyx_XDECREF(__pyx_v_getter);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_5_validate_glycan_composition(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_4_validate_glycan_composition[] = "StubGlycopeptideStrategy._validate_glycan_composition(self, CountTable aggregate_glycosylation, glycan) -> bool";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_5_validate_glycan_composition(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *__pyx_v_aggregate_glycosylation = 0;
  PyObject *__pyx_v_glycan = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_validate_glycan_composition (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_aggregate_glycosylation,&__pyx_n_s_glycan,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_aggregate_glycosylation)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_glycan)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_validate_glycan_composition", 1, 2, 2, 1); __PYX_ERR(0, 253, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_validate_glycan_composition") < 0)) __PYX_ERR(0, 253, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_aggregate_glycosylation = ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)values[0]);
    __pyx_v_glycan = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_validate_glycan_composition", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 253, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy._validate_glycan_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_aggregate_glycosylation), __pyx_ptype_14glycopeptidepy_2_c_11count_table_CountTable, 1, "aggregate_glycosylation", 0))) __PYX_ERR(0, 253, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_4_validate_glycan_composition(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self), __pyx_v_aggregate_glycosylation, __pyx_v_glycan);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_4_validate_glycan_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *__pyx_v_aggregate_glycosylation, PyObject *__pyx_v_glycan) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_validate_glycan_composition", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy__validate_glycan_composition(__pyx_v_self, __pyx_v_aggregate_glycosylation, __pyx_v_glycan, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 253, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy._validate_glycan_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":279
 *         return invalid
 * 
 *     cpdef GlycanCompositionFragment fucosylate_increment(self, GlycanCompositionFragment shift):             # <<<<<<<<<<<<<<
 *         cdef:
 *             GlycanCompositionFragment fucosylated
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_7fucosylate_increment(PyObject *__pyx_v_self, PyObject *__pyx_v_shift); /*proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_fucosylate_increment(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_shift, int __pyx_skip_dispatch) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_fucosylated = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("fucosylate_increment", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_fucosylate_increment); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 279, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_7fucosylate_increment)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_shift)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_shift));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 279, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment))))) __PYX_ERR(0, 279, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":282
 *         cdef:
 *             GlycanCompositionFragment fucosylated
 *         fucosylated = shift.copy()             # <<<<<<<<<<<<<<
 *         fucosylated.mass += _FUC_mass
 *         if self.compute_compositions:
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_shift->__pyx_vtab)->copy(__pyx_v_shift, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 282, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_fucosylated = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":283
 *             GlycanCompositionFragment fucosylated
 *         fucosylated = shift.copy()
 *         fucosylated.mass += _FUC_mass             # <<<<<<<<<<<<<<
 *         if self.compute_compositions:
 *             fucosylated.composition.add_from(_FUC_composition)
 */
  __pyx_v_fucosylated->mass = (__pyx_v_fucosylated->mass + __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC_mass);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":284
 *         fucosylated = shift.copy()
 *         fucosylated.mass += _FUC_mass
 *         if self.compute_compositions:             # <<<<<<<<<<<<<<
 *             fucosylated.composition.add_from(_FUC_composition)
 *         fucosylated.key.increment(_FUC, 1)
 */
  __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.compute_compositions != 0);
  if (__pyx_t_5) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":285
 *         fucosylated.mass += _FUC_mass
 *         if self.compute_compositions:
 *             fucosylated.composition.add_from(_FUC_composition)             # <<<<<<<<<<<<<<
 *         fucosylated.key.increment(_FUC, 1)
 *         return fucosylated
 */
    __pyx_t_1 = ((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC_composition);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_v_fucosylated->composition->__pyx_vtab)->add_from(__pyx_v_fucosylated->composition, __pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 285, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":284
 *         fucosylated = shift.copy()
 *         fucosylated.mass += _FUC_mass
 *         if self.compute_compositions:             # <<<<<<<<<<<<<<
 *             fucosylated.composition.add_from(_FUC_composition)
 *         fucosylated.key.increment(_FUC, 1)
 */
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":286
 *         if self.compute_compositions:
 *             fucosylated.composition.add_from(_FUC_composition)
 *         fucosylated.key.increment(_FUC, 1)             # <<<<<<<<<<<<<<
 *         return fucosylated
 * 
 */
  __pyx_t_2 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC;
  __Pyx_INCREF(__pyx_t_2);
  ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_fucosylated->key->__pyx_vtab)->increment(__pyx_v_fucosylated->key, __pyx_t_2, 1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":287
 *             fucosylated.composition.add_from(_FUC_composition)
 *         fucosylated.key.increment(_FUC, 1)
 *         return fucosylated             # <<<<<<<<<<<<<<
 * 
 *     cpdef GlycanCompositionFragment xylosylate_increment(self, GlycanCompositionFragment shift):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_fucosylated));
  __pyx_r = __pyx_v_fucosylated;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":279
 *         return invalid
 * 
 *     cpdef GlycanCompositionFragment fucosylate_increment(self, GlycanCompositionFragment shift):             # <<<<<<<<<<<<<<
 *         cdef:
 *             GlycanCompositionFragment fucosylated
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.fucosylate_increment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_fucosylated);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_7fucosylate_increment(PyObject *__pyx_v_self, PyObject *__pyx_v_shift); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_6fucosylate_increment[] = "StubGlycopeptideStrategy.fucosylate_increment(self, GlycanCompositionFragment shift) -> GlycanCompositionFragment";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_7fucosylate_increment(PyObject *__pyx_v_self, PyObject *__pyx_v_shift) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fucosylate_increment (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_shift), __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment, 1, "shift", 0))) __PYX_ERR(0, 279, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_6fucosylate_increment(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self), ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_shift));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_6fucosylate_increment(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_shift) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("fucosylate_increment", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_fucosylate_increment(__pyx_v_self, __pyx_v_shift, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 279, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.fucosylate_increment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":289
 *         return fucosylated
 * 
 *     cpdef GlycanCompositionFragment xylosylate_increment(self, GlycanCompositionFragment shift):             # <<<<<<<<<<<<<<
 *         cdef:
 *             GlycanCompositionFragment xylosylated
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_9xylosylate_increment(PyObject *__pyx_v_self, PyObject *__pyx_v_shift); /*proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_xylosylate_increment(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_shift, int __pyx_skip_dispatch) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_xylosylated = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("xylosylate_increment", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_xylosylate_increment); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 289, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_9xylosylate_increment)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_shift)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_shift));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 289, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment))))) __PYX_ERR(0, 289, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":292
 *         cdef:
 *             GlycanCompositionFragment xylosylated
 *         xylosylated = shift.copy()             # <<<<<<<<<<<<<<
 *         xylosylated.mass += _XYL_mass
 *         if self.compute_compositions:
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_shift->__pyx_vtab)->copy(__pyx_v_shift, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 292, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_xylosylated = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":293
 *             GlycanCompositionFragment xylosylated
 *         xylosylated = shift.copy()
 *         xylosylated.mass += _XYL_mass             # <<<<<<<<<<<<<<
 *         if self.compute_compositions:
 *             xylosylated.composition.add_from(_XYL_composition)
 */
  __pyx_v_xylosylated->mass = (__pyx_v_xylosylated->mass + __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL_mass);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":294
 *         xylosylated = shift.copy()
 *         xylosylated.mass += _XYL_mass
 *         if self.compute_compositions:             # <<<<<<<<<<<<<<
 *             xylosylated.composition.add_from(_XYL_composition)
 *         xylosylated.key.increment(_XYL, 1)
 */
  __pyx_t_5 = (__pyx_v_self->__pyx_base.__pyx_base.compute_compositions != 0);
  if (__pyx_t_5) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":295
 *         xylosylated.mass += _XYL_mass
 *         if self.compute_compositions:
 *             xylosylated.composition.add_from(_XYL_composition)             # <<<<<<<<<<<<<<
 *         xylosylated.key.increment(_XYL, 1)
 *         return xylosylated
 */
    __pyx_t_1 = ((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL_composition);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_v_xylosylated->composition->__pyx_vtab)->add_from(__pyx_v_xylosylated->composition, __pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 295, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":294
 *         xylosylated = shift.copy()
 *         xylosylated.mass += _XYL_mass
 *         if self.compute_compositions:             # <<<<<<<<<<<<<<
 *             xylosylated.composition.add_from(_XYL_composition)
 *         xylosylated.key.increment(_XYL, 1)
 */
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":296
 *         if self.compute_compositions:
 *             xylosylated.composition.add_from(_XYL_composition)
 *         xylosylated.key.increment(_XYL, 1)             # <<<<<<<<<<<<<<
 *         return xylosylated
 * 
 */
  __pyx_t_2 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL;
  __Pyx_INCREF(__pyx_t_2);
  ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_xylosylated->key->__pyx_vtab)->increment(__pyx_v_xylosylated->key, __pyx_t_2, 1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":297
 *             xylosylated.composition.add_from(_XYL_composition)
 *         xylosylated.key.increment(_XYL, 1)
 *         return xylosylated             # <<<<<<<<<<<<<<
 * 
 *     cpdef list fucosylate_extended(self, GlycanCompositionFragment shift, long fucose_count):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_xylosylated));
  __pyx_r = __pyx_v_xylosylated;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":289
 *         return fucosylated
 * 
 *     cpdef GlycanCompositionFragment xylosylate_increment(self, GlycanCompositionFragment shift):             # <<<<<<<<<<<<<<
 *         cdef:
 *             GlycanCompositionFragment xylosylated
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.xylosylate_increment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_xylosylated);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_9xylosylate_increment(PyObject *__pyx_v_self, PyObject *__pyx_v_shift); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_8xylosylate_increment[] = "StubGlycopeptideStrategy.xylosylate_increment(self, GlycanCompositionFragment shift) -> GlycanCompositionFragment";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_9xylosylate_increment(PyObject *__pyx_v_self, PyObject *__pyx_v_shift) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("xylosylate_increment (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_shift), __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment, 1, "shift", 0))) __PYX_ERR(0, 289, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_8xylosylate_increment(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self), ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_shift));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_8xylosylate_increment(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_shift) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("xylosylate_increment", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_xylosylate_increment(__pyx_v_self, __pyx_v_shift, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 289, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.xylosylate_increment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":299
 *         return xylosylated
 * 
 *     cpdef list fucosylate_extended(self, GlycanCompositionFragment shift, long fucose_count):             # <<<<<<<<<<<<<<
 *         cdef:
 *             GlycanCompositionFragment fucosylated
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_11fucosylate_extended(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_fucosylate_extended(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_shift, long __pyx_v_fucose_count, int __pyx_skip_dispatch) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_fucosylated = 0;
  size_t __pyx_v_i;
  PyObject *__pyx_v_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  long __pyx_t_8;
  long __pyx_t_9;
  size_t __pyx_t_10;
  int __pyx_t_11;
  size_t __pyx_t_12;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("fucosylate_extended", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_fucosylate_extended); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 299, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_11fucosylate_extended)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_v_fucose_count); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 299, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_4 = __pyx_t_1; __pyx_t_5 = NULL;
        __pyx_t_6 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_4))) {
          __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
          if (likely(__pyx_t_5)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
            __Pyx_INCREF(__pyx_t_5);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_4, function);
            __pyx_t_6 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, ((PyObject *)__pyx_v_shift), __pyx_t_3};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 299, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_4)) {
          PyObject *__pyx_temp[3] = {__pyx_t_5, ((PyObject *)__pyx_v_shift), __pyx_t_3};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_4, __pyx_temp+1-__pyx_t_6, 2+__pyx_t_6); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 299, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        } else
        #endif
        {
          __pyx_t_7 = PyTuple_New(2+__pyx_t_6); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 299, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_7);
          if (__pyx_t_5) {
            __Pyx_GIVEREF(__pyx_t_5); PyTuple_SET_ITEM(__pyx_t_7, 0, __pyx_t_5); __pyx_t_5 = NULL;
          }
          __Pyx_INCREF(((PyObject *)__pyx_v_shift));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_shift));
          PyTuple_SET_ITEM(__pyx_t_7, 0+__pyx_t_6, ((PyObject *)__pyx_v_shift));
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_7, 1+__pyx_t_6, __pyx_t_3);
          __pyx_t_3 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_4, __pyx_t_7, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 299, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        }
        __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 299, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":304
 *             size_t i
 *             list result
 *         result = [None for i in range(fucose_count)]             # <<<<<<<<<<<<<<
 *         for i in range(1, fucose_count + 1):
 *             fucosylated = shift.copy()
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 304, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_8 = __pyx_v_fucose_count;
  __pyx_t_9 = __pyx_t_8;
  for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
    __pyx_v_i = __pyx_t_10;
    if (unlikely(__Pyx_ListComp_Append(__pyx_t_1, (PyObject*)Py_None))) __PYX_ERR(0, 304, __pyx_L1_error)
  }
  __pyx_v_result = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":305
 *             list result
 *         result = [None for i in range(fucose_count)]
 *         for i in range(1, fucose_count + 1):             # <<<<<<<<<<<<<<
 *             fucosylated = shift.copy()
 *             fucosylated.mass += _FUC_mass * i
 */
  __pyx_t_8 = (__pyx_v_fucose_count + 1);
  __pyx_t_9 = __pyx_t_8;
  for (__pyx_t_10 = 1; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
    __pyx_v_i = __pyx_t_10;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":306
 *         result = [None for i in range(fucose_count)]
 *         for i in range(1, fucose_count + 1):
 *             fucosylated = shift.copy()             # <<<<<<<<<<<<<<
 *             fucosylated.mass += _FUC_mass * i
 *             if self.compute_compositions:
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_shift->__pyx_vtab)->copy(__pyx_v_shift, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 306, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_fucosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":307
 *         for i in range(1, fucose_count + 1):
 *             fucosylated = shift.copy()
 *             fucosylated.mass += _FUC_mass * i             # <<<<<<<<<<<<<<
 *             if self.compute_compositions:
 *                 fucosylated.composition.add_from(<CComposition>(i * _FUC_composition))
 */
    __pyx_v_fucosylated->mass = (__pyx_v_fucosylated->mass + (__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC_mass * __pyx_v_i));

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":308
 *             fucosylated = shift.copy()
 *             fucosylated.mass += _FUC_mass * i
 *             if self.compute_compositions:             # <<<<<<<<<<<<<<
 *                 fucosylated.composition.add_from(<CComposition>(i * _FUC_composition))
 *             fucosylated.key.increment(_FUC, i)
 */
    __pyx_t_11 = (__pyx_v_self->__pyx_base.__pyx_base.compute_compositions != 0);
    if (__pyx_t_11) {

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":309
 *             fucosylated.mass += _FUC_mass * i
 *             if self.compute_compositions:
 *                 fucosylated.composition.add_from(<CComposition>(i * _FUC_composition))             # <<<<<<<<<<<<<<
 *             fucosylated.key.increment(_FUC, i)
 *             result[i - 1] = fucosylated
 */
      __pyx_t_1 = __Pyx_PyInt_FromSize_t(__pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = PyNumber_Multiply(__pyx_t_1, ((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC_composition)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_v_fucosylated->composition->__pyx_vtab)->add_from(__pyx_v_fucosylated->composition, __pyx_t_2)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 309, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":308
 *             fucosylated = shift.copy()
 *             fucosylated.mass += _FUC_mass * i
 *             if self.compute_compositions:             # <<<<<<<<<<<<<<
 *                 fucosylated.composition.add_from(<CComposition>(i * _FUC_composition))
 *             fucosylated.key.increment(_FUC, i)
 */
    }

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":310
 *             if self.compute_compositions:
 *                 fucosylated.composition.add_from(<CComposition>(i * _FUC_composition))
 *             fucosylated.key.increment(_FUC, i)             # <<<<<<<<<<<<<<
 *             result[i - 1] = fucosylated
 *         return result
 */
    __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC;
    __Pyx_INCREF(__pyx_t_1);
    ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_fucosylated->key->__pyx_vtab)->increment(__pyx_v_fucosylated->key, __pyx_t_1, __pyx_v_i);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":311
 *                 fucosylated.composition.add_from(<CComposition>(i * _FUC_composition))
 *             fucosylated.key.increment(_FUC, i)
 *             result[i - 1] = fucosylated             # <<<<<<<<<<<<<<
 *         return result
 * 
 */
    __pyx_t_12 = (__pyx_v_i - 1);
    if (unlikely(__Pyx_SetItemInt(__pyx_v_result, __pyx_t_12, ((PyObject *)__pyx_v_fucosylated), size_t, 0, __Pyx_PyInt_FromSize_t, 1, 0, 1) < 0)) __PYX_ERR(0, 311, __pyx_L1_error)
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":312
 *             fucosylated.key.increment(_FUC, i)
 *             result[i - 1] = fucosylated
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     cpdef GlycanCompositionFragment modified_increment(self, GlycanCompositionFragment base, object shift):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":299
 *         return xylosylated
 * 
 *     cpdef list fucosylate_extended(self, GlycanCompositionFragment shift, long fucose_count):             # <<<<<<<<<<<<<<
 *         cdef:
 *             GlycanCompositionFragment fucosylated
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.fucosylate_extended", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_fucosylated);
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_11fucosylate_extended(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_10fucosylate_extended[] = "StubGlycopeptideStrategy.fucosylate_extended(self, GlycanCompositionFragment shift, long fucose_count) -> list";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_11fucosylate_extended(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_shift = 0;
  long __pyx_v_fucose_count;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("fucosylate_extended (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_shift,&__pyx_n_s_fucose_count,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_shift)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_fucose_count)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("fucosylate_extended", 1, 2, 2, 1); __PYX_ERR(0, 299, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "fucosylate_extended") < 0)) __PYX_ERR(0, 299, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_shift = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)values[0]);
    __pyx_v_fucose_count = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v_fucose_count == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 299, __pyx_L3_error)
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("fucosylate_extended", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 299, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.fucosylate_extended", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_shift), __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment, 1, "shift", 0))) __PYX_ERR(0, 299, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_10fucosylate_extended(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self), __pyx_v_shift, __pyx_v_fucose_count);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_10fucosylate_extended(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_shift, long __pyx_v_fucose_count) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("fucosylate_extended", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_fucosylate_extended(__pyx_v_self, __pyx_v_shift, __pyx_v_fucose_count, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 299, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.fucosylate_extended", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":314
 *         return result
 * 
 *     cpdef GlycanCompositionFragment modified_increment(self, GlycanCompositionFragment base, object shift):             # <<<<<<<<<<<<<<
 *         cdef:
 *             GlycanCompositionFragment modified
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_13modified_increment(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_modified_increment(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_base, PyObject *__pyx_v_shift, int __pyx_skip_dispatch) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_modified = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  double __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("modified_increment", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_modified_increment); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 314, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_13modified_increment)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_base), __pyx_v_shift};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 314, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_base), __pyx_v_shift};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 314, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 314, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(((PyObject *)__pyx_v_base));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_base));
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, ((PyObject *)__pyx_v_base));
          __Pyx_INCREF(__pyx_v_shift);
          __Pyx_GIVEREF(__pyx_v_shift);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_shift);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 314, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment))))) __PYX_ERR(0, 314, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":317
 *         cdef:
 *             GlycanCompositionFragment modified
 *         modified = base.copy()             # <<<<<<<<<<<<<<
 * 
 *         modified.mass += shift.mass()
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v_base->__pyx_vtab)->copy(__pyx_v_base, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_modified = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":319
 *         modified = base.copy()
 * 
 *         modified.mass += shift.mass()             # <<<<<<<<<<<<<<
 *         if self.compute_compositions:
 *             modified.composition.add_from(shift.total_composition())
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_modified->mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_v_shift, __pyx_n_s_mass); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_6 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_6)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_2 = (__pyx_t_6) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_6) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_3 = PyNumber_InPlaceAdd(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_7 = __pyx_PyFloat_AsDouble(__pyx_t_3); if (unlikely((__pyx_t_7 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 319, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_modified->mass = __pyx_t_7;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":320
 * 
 *         modified.mass += shift.mass()
 *         if self.compute_compositions:             # <<<<<<<<<<<<<<
 *             modified.composition.add_from(shift.total_composition())
 *         modified.key.increment(shift, 1)
 */
  __pyx_t_8 = (__pyx_v_self->__pyx_base.__pyx_base.compute_compositions != 0);
  if (__pyx_t_8) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":321
 *         modified.mass += shift.mass()
 *         if self.compute_compositions:
 *             modified.composition.add_from(shift.total_composition())             # <<<<<<<<<<<<<<
 *         modified.key.increment(shift, 1)
 *         return modified
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_shift, __pyx_n_s_total_composition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 321, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_1) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_1) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 321, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_v_modified->composition->__pyx_vtab)->add_from(__pyx_v_modified->composition, __pyx_t_3)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 321, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":320
 * 
 *         modified.mass += shift.mass()
 *         if self.compute_compositions:             # <<<<<<<<<<<<<<
 *             modified.composition.add_from(shift.total_composition())
 *         modified.key.increment(shift, 1)
 */
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":322
 *         if self.compute_compositions:
 *             modified.composition.add_from(shift.total_composition())
 *         modified.key.increment(shift, 1)             # <<<<<<<<<<<<<<
 *         return modified
 * 
 */
  ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_modified->key->__pyx_vtab)->increment(__pyx_v_modified->key, __pyx_v_shift, 1);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":323
 *             modified.composition.add_from(shift.total_composition())
 *         modified.key.increment(shift, 1)
 *         return modified             # <<<<<<<<<<<<<<
 * 
 *     cpdef list n_glycan_composition_fragments(self, object glycan, int core_count=1, int  iteration_count=0):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_modified));
  __pyx_r = __pyx_v_modified;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":314
 *         return result
 * 
 *     cpdef GlycanCompositionFragment modified_increment(self, GlycanCompositionFragment base, object shift):             # <<<<<<<<<<<<<<
 *         cdef:
 *             GlycanCompositionFragment modified
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.modified_increment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_modified);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_13modified_increment(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_12modified_increment[] = "StubGlycopeptideStrategy.modified_increment(self, GlycanCompositionFragment base, shift) -> GlycanCompositionFragment";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_13modified_increment(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_base = 0;
  PyObject *__pyx_v_shift = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("modified_increment (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_base,&__pyx_n_s_shift,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_base)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_shift)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("modified_increment", 1, 2, 2, 1); __PYX_ERR(0, 314, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "modified_increment") < 0)) __PYX_ERR(0, 314, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_base = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)values[0]);
    __pyx_v_shift = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("modified_increment", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 314, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.modified_increment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_base), __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment, 1, "base", 0))) __PYX_ERR(0, 314, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_12modified_increment(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self), __pyx_v_base, __pyx_v_shift);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_12modified_increment(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_base, PyObject *__pyx_v_shift) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("modified_increment", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_modified_increment(__pyx_v_self, __pyx_v_base, __pyx_v_shift, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 314, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.modified_increment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":325
 *         return modified
 * 
 *     cpdef list n_glycan_composition_fragments(self, object glycan, int core_count=1, int  iteration_count=0):             # <<<<<<<<<<<<<<
 *         """Generate theoretical N-glycan Y fragment compositions containing the core motif
 *         plus a portion of the extended branches if :attr:`extended` is used. Unless :attr:`extend
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_15n_glycan_composition_fragments(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_n_glycan_composition_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_glycan, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_n_glycan_composition_fragments *__pyx_optional_args) {
  int __pyx_v_core_count = ((int)1);
  int __pyx_v_iteration_count = ((int)0);
  long __pyx_v_fucose_count;
  long __pyx_v_xylose_count;
  long __pyx_v_hexnac_count;
  long __pyx_v_hexose_count;
  long __pyx_v_hexnac_in_aggregate;
  long __pyx_v_hexose_in_aggregate;
  long __pyx_v_base_hexnac;
  long __pyx_v_extra_hexnac_count;
  long __pyx_v_extra_hexose_count;
  struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *__pyx_v_key_ct = 0;
  PyObject *__pyx_v_core_shifts = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_shift = NULL;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_fucosylated = NULL;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_xylosylated = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  long __pyx_t_10;
  long __pyx_t_11;
  long __pyx_t_12;
  int __pyx_t_13;
  long __pyx_t_14;
  long __pyx_t_15;
  long __pyx_t_16;
  int __pyx_t_17;
  long __pyx_t_18;
  long __pyx_t_19;
  long __pyx_t_20;
  long __pyx_t_21;
  long __pyx_t_22;
  long __pyx_t_23;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("n_glycan_composition_fragments", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_core_count = __pyx_optional_args->core_count;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_iteration_count = __pyx_optional_args->iteration_count;
      }
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_n_glycan_composition_fragments); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 325, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_15n_glycan_composition_fragments)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = __Pyx_PyInt_From_int(__pyx_v_core_count); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 325, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = __Pyx_PyInt_From_int(__pyx_v_iteration_count); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 325, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_5 = __pyx_t_1; __pyx_t_6 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_v_glycan, __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 325, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_v_glycan, __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 325, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 325, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          if (__pyx_t_6) {
            __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
          }
          __Pyx_INCREF(__pyx_v_glycan);
          __Pyx_GIVEREF(__pyx_v_glycan);
          PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_v_glycan);
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_t_4);
          __pyx_t_3 = 0;
          __pyx_t_4 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 325, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 325, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":354
 *             list core_shifts
 * 
 *         if self._use_query:             # <<<<<<<<<<<<<<
 *             fucose_count = glycan.query('Fuc') + glycan.query('dHex')
 *             xylose_count = glycan.query('Xyl')
 */
  __pyx_t_9 = (__pyx_v_self->__pyx_base._use_query != 0);
  if (__pyx_t_9) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":355
 * 
 *         if self._use_query:
 *             fucose_count = glycan.query('Fuc') + glycan.query('dHex')             # <<<<<<<<<<<<<<
 *             xylose_count = glycan.query('Xyl')
 *             hexnac_in_aggregate = glycan.query('HexNAc')
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_glycan, __pyx_n_s_query); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 355, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_n_s_Fuc) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_n_s_Fuc);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 355, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_glycan, __pyx_n_s_query); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 355, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_2 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_8, __pyx_n_s_dHex) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_n_s_dHex);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 355, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyNumber_Add(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 355, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_long(__pyx_t_5); if (unlikely((__pyx_t_10 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 355, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_fucose_count = __pyx_t_10;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":356
 *         if self._use_query:
 *             fucose_count = glycan.query('Fuc') + glycan.query('dHex')
 *             xylose_count = glycan.query('Xyl')             # <<<<<<<<<<<<<<
 *             hexnac_in_aggregate = glycan.query('HexNAc')
 *             hexose_in_aggregate = glycan.query('Hex')
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_glycan, __pyx_n_s_query); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 356, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_n_s_Xyl) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_n_s_Xyl);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 356, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_long(__pyx_t_5); if (unlikely((__pyx_t_10 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 356, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_xylose_count = __pyx_t_10;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":357
 *             fucose_count = glycan.query('Fuc') + glycan.query('dHex')
 *             xylose_count = glycan.query('Xyl')
 *             hexnac_in_aggregate = glycan.query('HexNAc')             # <<<<<<<<<<<<<<
 *             hexose_in_aggregate = glycan.query('Hex')
 *         else:
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_glycan, __pyx_n_s_query); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 357, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_n_s_HexNAc) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_n_s_HexNAc);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 357, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_long(__pyx_t_5); if (unlikely((__pyx_t_10 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 357, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_hexnac_in_aggregate = __pyx_t_10;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":358
 *             xylose_count = glycan.query('Xyl')
 *             hexnac_in_aggregate = glycan.query('HexNAc')
 *             hexose_in_aggregate = glycan.query('Hex')             # <<<<<<<<<<<<<<
 *         else:
 *             fucose_count = glycan[_FUC] + glycan[_DHEX]
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_glycan, __pyx_n_s_query); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 358, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_n_s_Hex) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_n_s_Hex);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 358, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_long(__pyx_t_5); if (unlikely((__pyx_t_10 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 358, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_hexose_in_aggregate = __pyx_t_10;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":354
 *             list core_shifts
 * 
 *         if self._use_query:             # <<<<<<<<<<<<<<
 *             fucose_count = glycan.query('Fuc') + glycan.query('dHex')
 *             xylose_count = glycan.query('Xyl')
 */
    goto __pyx_L3;
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":360
 *             hexose_in_aggregate = glycan.query('Hex')
 *         else:
 *             fucose_count = glycan[_FUC] + glycan[_DHEX]             # <<<<<<<<<<<<<<
 *             xylose_count = glycan[_XYL]
 *             hexnac_in_aggregate = glycan[_HEXNAC]
 */
  /*else*/ {
    __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_v_glycan, __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_glycan, __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__DHEX); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = PyNumber_Add(__pyx_t_5, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_long(__pyx_t_1); if (unlikely((__pyx_t_10 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 360, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_fucose_count = __pyx_t_10;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":361
 *         else:
 *             fucose_count = glycan[_FUC] + glycan[_DHEX]
 *             xylose_count = glycan[_XYL]             # <<<<<<<<<<<<<<
 *             hexnac_in_aggregate = glycan[_HEXNAC]
 *             hexose_in_aggregate = glycan[_HEX]
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_glycan, __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 361, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = __Pyx_PyInt_As_long(__pyx_t_1); if (unlikely((__pyx_t_10 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 361, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_xylose_count = __pyx_t_10;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":362
 *             fucose_count = glycan[_FUC] + glycan[_DHEX]
 *             xylose_count = glycan[_XYL]
 *             hexnac_in_aggregate = glycan[_HEXNAC]             # <<<<<<<<<<<<<<
 *             hexose_in_aggregate = glycan[_HEX]
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_glycan, __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 362, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = __Pyx_PyInt_As_long(__pyx_t_1); if (unlikely((__pyx_t_10 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 362, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_hexnac_in_aggregate = __pyx_t_10;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":363
 *             xylose_count = glycan[_XYL]
 *             hexnac_in_aggregate = glycan[_HEXNAC]
 *             hexose_in_aggregate = glycan[_HEX]             # <<<<<<<<<<<<<<
 * 
 *         core_shifts = []
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_glycan, __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = __Pyx_PyInt_As_long(__pyx_t_1); if (unlikely((__pyx_t_10 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 363, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_hexose_in_aggregate = __pyx_t_10;
  }
  __pyx_L3:;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":365
 *             hexose_in_aggregate = glycan[_HEX]
 * 
 *         core_shifts = []             # <<<<<<<<<<<<<<
 *         base_hexnac = min(hexnac_in_aggregate + 1, 3)
 *         for hexnac_count in range(base_hexnac):
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 365, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_core_shifts = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":366
 * 
 *         core_shifts = []
 *         base_hexnac = min(hexnac_in_aggregate + 1, 3)             # <<<<<<<<<<<<<<
 *         for hexnac_count in range(base_hexnac):
 *             if hexnac_count == 0:
 */
  __pyx_t_10 = 3;
  __pyx_t_11 = (__pyx_v_hexnac_in_aggregate + 1);
  if (((__pyx_t_10 < __pyx_t_11) != 0)) {
    __pyx_t_12 = __pyx_t_10;
  } else {
    __pyx_t_12 = __pyx_t_11;
  }
  __pyx_v_base_hexnac = __pyx_t_12;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":367
 *         core_shifts = []
 *         base_hexnac = min(hexnac_in_aggregate + 1, 3)
 *         for hexnac_count in range(base_hexnac):             # <<<<<<<<<<<<<<
 *             if hexnac_count == 0:
 *                 shift = GlycanCompositionFragment._create(0,
 */
  __pyx_t_12 = __pyx_v_base_hexnac;
  __pyx_t_10 = __pyx_t_12;
  for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_10; __pyx_t_11+=1) {
    __pyx_v_hexnac_count = __pyx_t_11;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":368
 *         base_hexnac = min(hexnac_in_aggregate + 1, 3)
 *         for hexnac_count in range(base_hexnac):
 *             if hexnac_count == 0:             # <<<<<<<<<<<<<<
 *                 shift = GlycanCompositionFragment._create(0,
 *                     CComposition(),
 */
    switch (__pyx_v_hexnac_count) {
      case 0:

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":370
 *             if hexnac_count == 0:
 *                 shift = GlycanCompositionFragment._create(0,
 *                     CComposition(),             # <<<<<<<<<<<<<<
 *                     CountTable._create(),
 *                     False)
 */
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_5glypy_11composition_12ccomposition_CComposition)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 370, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":371
 *                 shift = GlycanCompositionFragment._create(0,
 *                     CComposition(),
 *                     CountTable._create(),             # <<<<<<<<<<<<<<
 *                     False)
 *                 core_shifts.append(shift)
 */
      __pyx_t_2 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable->_create()); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 371, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":369
 *         for hexnac_count in range(base_hexnac):
 *             if hexnac_count == 0:
 *                 shift = GlycanCompositionFragment._create(0,             # <<<<<<<<<<<<<<
 *                     CComposition(),
 *                     CountTable._create(),
 */
      __pyx_t_5 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create(0.0, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1), ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_2), 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 369, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF_SET(__pyx_v_shift, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_5));
      __pyx_t_5 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":373
 *                     CountTable._create(),
 *                     False)
 *                 core_shifts.append(shift)             # <<<<<<<<<<<<<<
 *             elif hexnac_count == 1:
 *                 key_ct = CountTable._create()
 */
      __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_shift)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 373, __pyx_L1_error)

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":368
 *         base_hexnac = min(hexnac_in_aggregate + 1, 3)
 *         for hexnac_count in range(base_hexnac):
 *             if hexnac_count == 0:             # <<<<<<<<<<<<<<
 *                 shift = GlycanCompositionFragment._create(0,
 *                     CComposition(),
 */
      break;
      case 1:

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":375
 *                 core_shifts.append(shift)
 *             elif hexnac_count == 1:
 *                 key_ct = CountTable._create()             # <<<<<<<<<<<<<<
 *                 key_ct.setitem(_HEXNAC, hexnac_count)
 *                 shift = GlycanCompositionFragment._create(
 */
      __pyx_t_5 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable->_create()); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 375, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_XDECREF_SET(__pyx_v_key_ct, ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_5));
      __pyx_t_5 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":376
 *             elif hexnac_count == 1:
 *                 key_ct = CountTable._create()
 *                 key_ct.setitem(_HEXNAC, hexnac_count)             # <<<<<<<<<<<<<<
 *                 shift = GlycanCompositionFragment._create(
 *                     (hexnac_count * _HEXNAC_mass),
 */
      __pyx_t_5 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC;
      __Pyx_INCREF(__pyx_t_5);
      __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_5, __pyx_v_hexnac_count); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 376, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":379
 *                 shift = GlycanCompositionFragment._create(
 *                     (hexnac_count * _HEXNAC_mass),
 *                     hexnac_hex_composition(hexnac_count, 0),             # <<<<<<<<<<<<<<
 *                     key_ct,
 *                     False,
 */
      __pyx_t_5 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_hexnac_hex_composition(__pyx_v_hexnac_count, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 379, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":377
 *                 key_ct = CountTable._create()
 *                 key_ct.setitem(_HEXNAC, hexnac_count)
 *                 shift = GlycanCompositionFragment._create(             # <<<<<<<<<<<<<<
 *                     (hexnac_count * _HEXNAC_mass),
 *                     hexnac_hex_composition(hexnac_count, 0),
 */
      __pyx_t_2 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create((__pyx_v_hexnac_count * __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC_mass), ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_5), __pyx_v_key_ct, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 377, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF_SET(__pyx_v_shift, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":383
 *                     False,
 *                 )
 *                 core_shifts.append(shift)             # <<<<<<<<<<<<<<
 *                 if iteration_count < fucose_count:
 *                     fucosylated = self.fucosylate_increment(shift)
 */
      __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_shift)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 383, __pyx_L1_error)

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":384
 *                 )
 *                 core_shifts.append(shift)
 *                 if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                     fucosylated = self.fucosylate_increment(shift)
 *                     core_shifts.append(fucosylated)
 */
      __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_fucose_count) != 0);
      if (__pyx_t_9) {

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":385
 *                 core_shifts.append(shift)
 *                 if iteration_count < fucose_count:
 *                     fucosylated = self.fucosylate_increment(shift)             # <<<<<<<<<<<<<<
 *                     core_shifts.append(fucosylated)
 *             elif hexnac_count == 2:
 */
        __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->fucosylate_increment(__pyx_v_self, __pyx_v_shift, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 385, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_fucosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":386
 *                 if iteration_count < fucose_count:
 *                     fucosylated = self.fucosylate_increment(shift)
 *                     core_shifts.append(fucosylated)             # <<<<<<<<<<<<<<
 *             elif hexnac_count == 2:
 *                 key_ct = CountTable._create()
 */
        __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_fucosylated)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 386, __pyx_L1_error)

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":384
 *                 )
 *                 core_shifts.append(shift)
 *                 if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                     fucosylated = self.fucosylate_increment(shift)
 *                     core_shifts.append(fucosylated)
 */
      }

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":374
 *                     False)
 *                 core_shifts.append(shift)
 *             elif hexnac_count == 1:             # <<<<<<<<<<<<<<
 *                 key_ct = CountTable._create()
 *                 key_ct.setitem(_HEXNAC, hexnac_count)
 */
      break;
      case 2:

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":388
 *                     core_shifts.append(fucosylated)
 *             elif hexnac_count == 2:
 *                 key_ct = CountTable._create()             # <<<<<<<<<<<<<<
 *                 key_ct.setitem(_HEXNAC, hexnac_count)
 *                 shift = GlycanCompositionFragment._create(
 */
      __pyx_t_2 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable->_create()); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 388, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_XDECREF_SET(__pyx_v_key_ct, ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":389
 *             elif hexnac_count == 2:
 *                 key_ct = CountTable._create()
 *                 key_ct.setitem(_HEXNAC, hexnac_count)             # <<<<<<<<<<<<<<
 *                 shift = GlycanCompositionFragment._create(
 *                     (hexnac_count * _HEXNAC_mass),
 */
      __pyx_t_2 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC;
      __Pyx_INCREF(__pyx_t_2);
      __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_2, __pyx_v_hexnac_count); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 389, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":392
 *                 shift = GlycanCompositionFragment._create(
 *                     (hexnac_count * _HEXNAC_mass),
 *                     hexnac_hex_composition(hexnac_count, 0),             # <<<<<<<<<<<<<<
 *                     key_ct,
 *                     False,
 */
      __pyx_t_2 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_hexnac_hex_composition(__pyx_v_hexnac_count, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 392, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":390
 *                 key_ct = CountTable._create()
 *                 key_ct.setitem(_HEXNAC, hexnac_count)
 *                 shift = GlycanCompositionFragment._create(             # <<<<<<<<<<<<<<
 *                     (hexnac_count * _HEXNAC_mass),
 *                     hexnac_hex_composition(hexnac_count, 0),
 */
      __pyx_t_5 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create((__pyx_v_hexnac_count * __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC_mass), ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_2), __pyx_v_key_ct, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 390, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF_SET(__pyx_v_shift, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_5));
      __pyx_t_5 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":396
 *                     False,
 *                 )
 *                 core_shifts.append(shift)             # <<<<<<<<<<<<<<
 * 
 *                 if not self.extended_fucosylation:
 */
      __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_shift)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 396, __pyx_L1_error)

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":398
 *                 core_shifts.append(shift)
 * 
 *                 if not self.extended_fucosylation:             # <<<<<<<<<<<<<<
 *                     if iteration_count < fucose_count:
 *                         fucosylated = self.fucosylate_increment(shift)
 */
      __pyx_t_9 = ((!(__pyx_v_self->extended_fucosylation != 0)) != 0);
      if (__pyx_t_9) {

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":399
 * 
 *                 if not self.extended_fucosylation:
 *                     if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                         fucosylated = self.fucosylate_increment(shift)
 *                         core_shifts.append(fucosylated)
 */
        __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_fucose_count) != 0);
        if (__pyx_t_9) {

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":400
 *                 if not self.extended_fucosylation:
 *                     if iteration_count < fucose_count:
 *                         fucosylated = self.fucosylate_increment(shift)             # <<<<<<<<<<<<<<
 *                         core_shifts.append(fucosylated)
 *                         if iteration_count < xylose_count:
 */
          __pyx_t_5 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->fucosylate_increment(__pyx_v_self, __pyx_v_shift, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 400, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_XDECREF_SET(__pyx_v_fucosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_5));
          __pyx_t_5 = 0;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":401
 *                     if iteration_count < fucose_count:
 *                         fucosylated = self.fucosylate_increment(shift)
 *                         core_shifts.append(fucosylated)             # <<<<<<<<<<<<<<
 *                         if iteration_count < xylose_count:
 *                             xylosylated = self.xylosylate_increment(fucosylated)
 */
          __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_fucosylated)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 401, __pyx_L1_error)

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":402
 *                         fucosylated = self.fucosylate_increment(shift)
 *                         core_shifts.append(fucosylated)
 *                         if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                             xylosylated = self.xylosylate_increment(fucosylated)
 *                             core_shifts.append(xylosylated)
 */
          __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_xylose_count) != 0);
          if (__pyx_t_9) {

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":403
 *                         core_shifts.append(fucosylated)
 *                         if iteration_count < xylose_count:
 *                             xylosylated = self.xylosylate_increment(fucosylated)             # <<<<<<<<<<<<<<
 *                             core_shifts.append(xylosylated)
 *                 elif fucose_count > 0:
 */
            __pyx_t_5 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->xylosylate_increment(__pyx_v_self, __pyx_v_fucosylated, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 403, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_XDECREF_SET(__pyx_v_xylosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_5));
            __pyx_t_5 = 0;

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":404
 *                         if iteration_count < xylose_count:
 *                             xylosylated = self.xylosylate_increment(fucosylated)
 *                             core_shifts.append(xylosylated)             # <<<<<<<<<<<<<<
 *                 elif fucose_count > 0:
 *                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 */
            __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_xylosylated)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 404, __pyx_L1_error)

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":402
 *                         fucosylated = self.fucosylate_increment(shift)
 *                         core_shifts.append(fucosylated)
 *                         if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                             xylosylated = self.xylosylate_increment(fucosylated)
 *                             core_shifts.append(xylosylated)
 */
          }

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":399
 * 
 *                 if not self.extended_fucosylation:
 *                     if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                         fucosylated = self.fucosylate_increment(shift)
 *                         core_shifts.append(fucosylated)
 */
        }

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":398
 *                 core_shifts.append(shift)
 * 
 *                 if not self.extended_fucosylation:             # <<<<<<<<<<<<<<
 *                     if iteration_count < fucose_count:
 *                         fucosylated = self.fucosylate_increment(shift)
 */
        goto __pyx_L7;
      }

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":405
 *                             xylosylated = self.xylosylate_increment(fucosylated)
 *                             core_shifts.append(xylosylated)
 *                 elif fucose_count > 0:             # <<<<<<<<<<<<<<
 *                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 *                     if iteration_count < xylose_count:
 */
      __pyx_t_9 = ((__pyx_v_fucose_count > 0) != 0);
      if (__pyx_t_9) {

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":406
 *                             core_shifts.append(xylosylated)
 *                 elif fucose_count > 0:
 *                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))             # <<<<<<<<<<<<<<
 *                     if iteration_count < xylose_count:
 *                         xylosylated = self.xylosylate_increment(fucosylated)
 */
        __pyx_t_5 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->fucosylate_extended(__pyx_v_self, __pyx_v_shift, __pyx_v_fucose_count, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 406, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __pyx_t_13 = __Pyx_PyList_Extend(__pyx_v_core_shifts, __pyx_t_5); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 406, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":407
 *                 elif fucose_count > 0:
 *                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 *                     if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                         xylosylated = self.xylosylate_increment(fucosylated)
 *                         core_shifts.append(xylosylated)
 */
        __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_xylose_count) != 0);
        if (__pyx_t_9) {

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":408
 *                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 *                     if iteration_count < xylose_count:
 *                         xylosylated = self.xylosylate_increment(fucosylated)             # <<<<<<<<<<<<<<
 *                         core_shifts.append(xylosylated)
 * 
 */
          if (unlikely(!__pyx_v_fucosylated)) { __Pyx_RaiseUnboundLocalError("fucosylated"); __PYX_ERR(0, 408, __pyx_L1_error) }
          __pyx_t_5 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->xylosylate_increment(__pyx_v_self, __pyx_v_fucosylated, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 408, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_XDECREF_SET(__pyx_v_xylosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_5));
          __pyx_t_5 = 0;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":409
 *                     if iteration_count < xylose_count:
 *                         xylosylated = self.xylosylate_increment(fucosylated)
 *                         core_shifts.append(xylosylated)             # <<<<<<<<<<<<<<
 * 
 *                 if iteration_count < xylose_count:
 */
          __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_xylosylated)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 409, __pyx_L1_error)

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":407
 *                 elif fucose_count > 0:
 *                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 *                     if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                         xylosylated = self.xylosylate_increment(fucosylated)
 *                         core_shifts.append(xylosylated)
 */
        }

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":405
 *                             xylosylated = self.xylosylate_increment(fucosylated)
 *                             core_shifts.append(xylosylated)
 *                 elif fucose_count > 0:             # <<<<<<<<<<<<<<
 *                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 *                     if iteration_count < xylose_count:
 */
      }
      __pyx_L7:;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":411
 *                         core_shifts.append(xylosylated)
 * 
 *                 if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                     xylosylated = self.xylosylate_increment(shift)
 *                     core_shifts.append(xylosylated)
 */
      __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_xylose_count) != 0);
      if (__pyx_t_9) {

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":412
 * 
 *                 if iteration_count < xylose_count:
 *                     xylosylated = self.xylosylate_increment(shift)             # <<<<<<<<<<<<<<
 *                     core_shifts.append(xylosylated)
 * 
 */
        __pyx_t_5 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->xylosylate_increment(__pyx_v_self, __pyx_v_shift, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 412, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_XDECREF_SET(__pyx_v_xylosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_5));
        __pyx_t_5 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":413
 *                 if iteration_count < xylose_count:
 *                     xylosylated = self.xylosylate_increment(shift)
 *                     core_shifts.append(xylosylated)             # <<<<<<<<<<<<<<
 * 
 *                 for hexose_count in range(1, min(hexose_in_aggregate + 1, 4)):
 */
        __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_xylosylated)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 413, __pyx_L1_error)

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":411
 *                         core_shifts.append(xylosylated)
 * 
 *                 if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                     xylosylated = self.xylosylate_increment(shift)
 *                     core_shifts.append(xylosylated)
 */
      }

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":415
 *                     core_shifts.append(xylosylated)
 * 
 *                 for hexose_count in range(1, min(hexose_in_aggregate + 1, 4)):             # <<<<<<<<<<<<<<
 *                     key_ct = CountTable._create()
 *                     key_ct.setitem(_HEXNAC, hexnac_count)
 */
      __pyx_t_14 = 4;
      __pyx_t_15 = (__pyx_v_hexose_in_aggregate + 1);
      if (((__pyx_t_14 < __pyx_t_15) != 0)) {
        __pyx_t_16 = __pyx_t_14;
      } else {
        __pyx_t_16 = __pyx_t_15;
      }
      __pyx_t_14 = __pyx_t_16;
      __pyx_t_16 = __pyx_t_14;
      for (__pyx_t_15 = 1; __pyx_t_15 < __pyx_t_16; __pyx_t_15+=1) {
        __pyx_v_hexose_count = __pyx_t_15;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":416
 * 
 *                 for hexose_count in range(1, min(hexose_in_aggregate + 1, 4)):
 *                     key_ct = CountTable._create()             # <<<<<<<<<<<<<<
 *                     key_ct.setitem(_HEXNAC, hexnac_count)
 *                     key_ct.setitem(_HEX, hexose_count)
 */
        __pyx_t_5 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable->_create()); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 416, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF_SET(__pyx_v_key_ct, ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_5));
        __pyx_t_5 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":417
 *                 for hexose_count in range(1, min(hexose_in_aggregate + 1, 4)):
 *                     key_ct = CountTable._create()
 *                     key_ct.setitem(_HEXNAC, hexnac_count)             # <<<<<<<<<<<<<<
 *                     key_ct.setitem(_HEX, hexose_count)
 *                     shift = GlycanCompositionFragment._create(
 */
        __pyx_t_5 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC;
        __Pyx_INCREF(__pyx_t_5);
        __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_5, __pyx_v_hexnac_count); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 417, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":418
 *                     key_ct = CountTable._create()
 *                     key_ct.setitem(_HEXNAC, hexnac_count)
 *                     key_ct.setitem(_HEX, hexose_count)             # <<<<<<<<<<<<<<
 *                     shift = GlycanCompositionFragment._create(
 *                         (hexnac_count * _HEXNAC_mass) + (hexose_count * _HEX_mass),
 */
        __pyx_t_5 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX;
        __Pyx_INCREF(__pyx_t_5);
        __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_5, __pyx_v_hexose_count); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 418, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":421
 *                     shift = GlycanCompositionFragment._create(
 *                         (hexnac_count * _HEXNAC_mass) + (hexose_count * _HEX_mass),
 *                         hexnac_hex_composition(hexnac_count, hexose_count),             # <<<<<<<<<<<<<<
 *                         key_ct,
 *                         False,
 */
        __pyx_t_5 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_hexnac_hex_composition(__pyx_v_hexnac_count, __pyx_v_hexose_count)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 421, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":419
 *                     key_ct.setitem(_HEXNAC, hexnac_count)
 *                     key_ct.setitem(_HEX, hexose_count)
 *                     shift = GlycanCompositionFragment._create(             # <<<<<<<<<<<<<<
 *                         (hexnac_count * _HEXNAC_mass) + (hexose_count * _HEX_mass),
 *                         hexnac_hex_composition(hexnac_count, hexose_count),
 */
        __pyx_t_2 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create(((__pyx_v_hexnac_count * __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC_mass) + (__pyx_v_hexose_count * __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX_mass)), ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_5), __pyx_v_key_ct, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 419, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_DECREF_SET(__pyx_v_shift, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":425
 *                         False,
 *                     )
 *                     core_shifts.append(shift)             # <<<<<<<<<<<<<<
 * 
 *                     if not self.extended_fucosylation:
 */
        __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_shift)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 425, __pyx_L1_error)

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":427
 *                     core_shifts.append(shift)
 * 
 *                     if not self.extended_fucosylation:             # <<<<<<<<<<<<<<
 *                         if iteration_count < fucose_count:
 *                             fucosylated = self.fucosylate_increment(shift)
 */
        __pyx_t_9 = ((!(__pyx_v_self->extended_fucosylation != 0)) != 0);
        if (__pyx_t_9) {

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":428
 * 
 *                     if not self.extended_fucosylation:
 *                         if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                             fucosylated = self.fucosylate_increment(shift)
 *                             core_shifts.append(fucosylated)
 */
          __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_fucose_count) != 0);
          if (__pyx_t_9) {

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":429
 *                     if not self.extended_fucosylation:
 *                         if iteration_count < fucose_count:
 *                             fucosylated = self.fucosylate_increment(shift)             # <<<<<<<<<<<<<<
 *                             core_shifts.append(fucosylated)
 *                             if iteration_count < xylose_count:
 */
            __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->fucosylate_increment(__pyx_v_self, __pyx_v_shift, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 429, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_XDECREF_SET(__pyx_v_fucosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2));
            __pyx_t_2 = 0;

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":430
 *                         if iteration_count < fucose_count:
 *                             fucosylated = self.fucosylate_increment(shift)
 *                             core_shifts.append(fucosylated)             # <<<<<<<<<<<<<<
 *                             if iteration_count < xylose_count:
 *                                 xylosylated = self.xylosylate_increment(fucosylated)
 */
            __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_fucosylated)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 430, __pyx_L1_error)

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":431
 *                             fucosylated = self.fucosylate_increment(shift)
 *                             core_shifts.append(fucosylated)
 *                             if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                                 xylosylated = self.xylosylate_increment(fucosylated)
 *                                 core_shifts.append(xylosylated)
 */
            __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_xylose_count) != 0);
            if (__pyx_t_9) {

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":432
 *                             core_shifts.append(fucosylated)
 *                             if iteration_count < xylose_count:
 *                                 xylosylated = self.xylosylate_increment(fucosylated)             # <<<<<<<<<<<<<<
 *                                 core_shifts.append(xylosylated)
 *                     elif fucose_count > 0:
 */
              __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->xylosylate_increment(__pyx_v_self, __pyx_v_fucosylated, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 432, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_XDECREF_SET(__pyx_v_xylosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2));
              __pyx_t_2 = 0;

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":433
 *                             if iteration_count < xylose_count:
 *                                 xylosylated = self.xylosylate_increment(fucosylated)
 *                                 core_shifts.append(xylosylated)             # <<<<<<<<<<<<<<
 *                     elif fucose_count > 0:
 *                         core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 */
              __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_xylosylated)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 433, __pyx_L1_error)

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":431
 *                             fucosylated = self.fucosylate_increment(shift)
 *                             core_shifts.append(fucosylated)
 *                             if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                                 xylosylated = self.xylosylate_increment(fucosylated)
 *                                 core_shifts.append(xylosylated)
 */
            }

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":428
 * 
 *                     if not self.extended_fucosylation:
 *                         if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                             fucosylated = self.fucosylate_increment(shift)
 *                             core_shifts.append(fucosylated)
 */
          }

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":427
 *                     core_shifts.append(shift)
 * 
 *                     if not self.extended_fucosylation:             # <<<<<<<<<<<<<<
 *                         if iteration_count < fucose_count:
 *                             fucosylated = self.fucosylate_increment(shift)
 */
          goto __pyx_L14;
        }

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":434
 *                                 xylosylated = self.xylosylate_increment(fucosylated)
 *                                 core_shifts.append(xylosylated)
 *                     elif fucose_count > 0:             # <<<<<<<<<<<<<<
 *                         core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 *                         if iteration_count < xylose_count:
 */
        __pyx_t_9 = ((__pyx_v_fucose_count > 0) != 0);
        if (__pyx_t_9) {

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":435
 *                                 core_shifts.append(xylosylated)
 *                     elif fucose_count > 0:
 *                         core_shifts.extend(self.fucosylate_extended(shift, fucose_count))             # <<<<<<<<<<<<<<
 *                         if iteration_count < xylose_count:
 *                             xylosylated = self.xylosylate_increment(fucosylated)
 */
          __pyx_t_2 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->fucosylate_extended(__pyx_v_self, __pyx_v_shift, __pyx_v_fucose_count, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 435, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_13 = __Pyx_PyList_Extend(__pyx_v_core_shifts, __pyx_t_2); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 435, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":436
 *                     elif fucose_count > 0:
 *                         core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 *                         if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                             xylosylated = self.xylosylate_increment(fucosylated)
 *                             core_shifts.append(xylosylated)
 */
          __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_xylose_count) != 0);
          if (__pyx_t_9) {

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":437
 *                         core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 *                         if iteration_count < xylose_count:
 *                             xylosylated = self.xylosylate_increment(fucosylated)             # <<<<<<<<<<<<<<
 *                             core_shifts.append(xylosylated)
 * 
 */
            if (unlikely(!__pyx_v_fucosylated)) { __Pyx_RaiseUnboundLocalError("fucosylated"); __PYX_ERR(0, 437, __pyx_L1_error) }
            __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->xylosylate_increment(__pyx_v_self, __pyx_v_fucosylated, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 437, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_2);
            __Pyx_XDECREF_SET(__pyx_v_xylosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2));
            __pyx_t_2 = 0;

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":438
 *                         if iteration_count < xylose_count:
 *                             xylosylated = self.xylosylate_increment(fucosylated)
 *                             core_shifts.append(xylosylated)             # <<<<<<<<<<<<<<
 * 
 *                     if iteration_count < xylose_count:
 */
            __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_xylosylated)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 438, __pyx_L1_error)

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":436
 *                     elif fucose_count > 0:
 *                         core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 *                         if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                             xylosylated = self.xylosylate_increment(fucosylated)
 *                             core_shifts.append(xylosylated)
 */
          }

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":434
 *                                 xylosylated = self.xylosylate_increment(fucosylated)
 *                                 core_shifts.append(xylosylated)
 *                     elif fucose_count > 0:             # <<<<<<<<<<<<<<
 *                         core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 *                         if iteration_count < xylose_count:
 */
        }
        __pyx_L14:;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":440
 *                             core_shifts.append(xylosylated)
 * 
 *                     if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                         xylosylated = self.xylosylate_increment(shift)
 *                         core_shifts.append(xylosylated)
 */
        __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_xylose_count) != 0);
        if (__pyx_t_9) {

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":441
 * 
 *                     if iteration_count < xylose_count:
 *                         xylosylated = self.xylosylate_increment(shift)             # <<<<<<<<<<<<<<
 *                         core_shifts.append(xylosylated)
 * 
 */
          __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->xylosylate_increment(__pyx_v_self, __pyx_v_shift, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 441, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_XDECREF_SET(__pyx_v_xylosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2));
          __pyx_t_2 = 0;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":442
 *                     if iteration_count < xylose_count:
 *                         xylosylated = self.xylosylate_increment(shift)
 *                         core_shifts.append(xylosylated)             # <<<<<<<<<<<<<<
 * 
 *                     # After the core motif has been exhausted, speculatively add
 */
          __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_xylosylated)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 442, __pyx_L1_error)

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":440
 *                             core_shifts.append(xylosylated)
 * 
 *                     if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                         xylosylated = self.xylosylate_increment(shift)
 *                         core_shifts.append(xylosylated)
 */
        }

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":448
 *                     # exhausted. This doesn't allow for one missing alpha-mannose while still
 *                     # showing an extension on the other.
 *                     if hexose_count == 3 and hexnac_in_aggregate >= 2 * core_count and self.extended:             # <<<<<<<<<<<<<<
 *                         for extra_hexnac_count in range(0, hexnac_in_aggregate - hexnac_count + 1):
 *                             if extra_hexnac_count + hexnac_count > hexnac_in_aggregate:
 */
        __pyx_t_17 = ((__pyx_v_hexose_count == 3) != 0);
        if (__pyx_t_17) {
        } else {
          __pyx_t_9 = __pyx_t_17;
          goto __pyx_L20_bool_binop_done;
        }
        __pyx_t_17 = ((__pyx_v_hexnac_in_aggregate >= (2 * __pyx_v_core_count)) != 0);
        if (__pyx_t_17) {
        } else {
          __pyx_t_9 = __pyx_t_17;
          goto __pyx_L20_bool_binop_done;
        }
        __pyx_t_17 = (__pyx_v_self->extended != 0);
        __pyx_t_9 = __pyx_t_17;
        __pyx_L20_bool_binop_done:;
        if (__pyx_t_9) {

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":449
 *                     # showing an extension on the other.
 *                     if hexose_count == 3 and hexnac_in_aggregate >= 2 * core_count and self.extended:
 *                         for extra_hexnac_count in range(0, hexnac_in_aggregate - hexnac_count + 1):             # <<<<<<<<<<<<<<
 *                             if extra_hexnac_count + hexnac_count > hexnac_in_aggregate:
 *                                 continue
 */
          __pyx_t_18 = ((__pyx_v_hexnac_in_aggregate - __pyx_v_hexnac_count) + 1);
          __pyx_t_19 = __pyx_t_18;
          for (__pyx_t_20 = 0; __pyx_t_20 < __pyx_t_19; __pyx_t_20+=1) {
            __pyx_v_extra_hexnac_count = __pyx_t_20;

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":450
 *                     if hexose_count == 3 and hexnac_in_aggregate >= 2 * core_count and self.extended:
 *                         for extra_hexnac_count in range(0, hexnac_in_aggregate - hexnac_count + 1):
 *                             if extra_hexnac_count + hexnac_count > hexnac_in_aggregate:             # <<<<<<<<<<<<<<
 *                                 continue
 *                             if extra_hexnac_count > 0:
 */
            __pyx_t_9 = (((__pyx_v_extra_hexnac_count + __pyx_v_hexnac_count) > __pyx_v_hexnac_in_aggregate) != 0);
            if (__pyx_t_9) {

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":451
 *                         for extra_hexnac_count in range(0, hexnac_in_aggregate - hexnac_count + 1):
 *                             if extra_hexnac_count + hexnac_count > hexnac_in_aggregate:
 *                                 continue             # <<<<<<<<<<<<<<
 *                             if extra_hexnac_count > 0:
 *                                 key_ct = CountTable._create()
 */
              goto __pyx_L23_continue;

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":450
 *                     if hexose_count == 3 and hexnac_in_aggregate >= 2 * core_count and self.extended:
 *                         for extra_hexnac_count in range(0, hexnac_in_aggregate - hexnac_count + 1):
 *                             if extra_hexnac_count + hexnac_count > hexnac_in_aggregate:             # <<<<<<<<<<<<<<
 *                                 continue
 *                             if extra_hexnac_count > 0:
 */
            }

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":452
 *                             if extra_hexnac_count + hexnac_count > hexnac_in_aggregate:
 *                                 continue
 *                             if extra_hexnac_count > 0:             # <<<<<<<<<<<<<<
 *                                 key_ct = CountTable._create()
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 */
            __pyx_t_9 = ((__pyx_v_extra_hexnac_count > 0) != 0);
            if (__pyx_t_9) {

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":453
 *                                 continue
 *                             if extra_hexnac_count > 0:
 *                                 key_ct = CountTable._create()             # <<<<<<<<<<<<<<
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 *                                 key_ct.setitem(_HEX, hexose_count)
 */
              __pyx_t_2 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable->_create()); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 453, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_DECREF_SET(__pyx_v_key_ct, ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_2));
              __pyx_t_2 = 0;

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":454
 *                             if extra_hexnac_count > 0:
 *                                 key_ct = CountTable._create()
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)             # <<<<<<<<<<<<<<
 *                                 key_ct.setitem(_HEX, hexose_count)
 *                                 shift = GlycanCompositionFragment._create(
 */
              __pyx_t_2 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC;
              __Pyx_INCREF(__pyx_t_2);
              __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_2, (__pyx_v_hexnac_count + __pyx_v_extra_hexnac_count)); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 454, __pyx_L1_error)
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":455
 *                                 key_ct = CountTable._create()
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 *                                 key_ct.setitem(_HEX, hexose_count)             # <<<<<<<<<<<<<<
 *                                 shift = GlycanCompositionFragment._create(
 *                                     ((hexnac_count + extra_hexnac_count) * _HEXNAC_mass) + (
 */
              __pyx_t_2 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX;
              __Pyx_INCREF(__pyx_t_2);
              __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_2, __pyx_v_hexose_count); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 455, __pyx_L1_error)
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":459
 *                                     ((hexnac_count + extra_hexnac_count) * _HEXNAC_mass) + (
 *                                         hexose_count * _HEX_mass),
 *                                     hexnac_hex_composition(hexnac_count + extra_hexnac_count, hexose_count),             # <<<<<<<<<<<<<<
 *                                     key_ct,
 *                                     True
 */
              __pyx_t_2 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_hexnac_hex_composition((__pyx_v_hexnac_count + __pyx_v_extra_hexnac_count), __pyx_v_hexose_count)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 459, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":456
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 *                                 key_ct.setitem(_HEX, hexose_count)
 *                                 shift = GlycanCompositionFragment._create(             # <<<<<<<<<<<<<<
 *                                     ((hexnac_count + extra_hexnac_count) * _HEXNAC_mass) + (
 *                                         hexose_count * _HEX_mass),
 */
              __pyx_t_5 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create((((__pyx_v_hexnac_count + __pyx_v_extra_hexnac_count) * __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC_mass) + (__pyx_v_hexose_count * __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX_mass)), ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_2), __pyx_v_key_ct, 1)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 456, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_5);
              __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
              __Pyx_DECREF_SET(__pyx_v_shift, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_5));
              __pyx_t_5 = 0;

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":463
 *                                     True
 *                                 )
 *                                 core_shifts.append(shift)             # <<<<<<<<<<<<<<
 * 
 *                                 if not self.extended_fucosylation:
 */
              __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_shift)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 463, __pyx_L1_error)

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":465
 *                                 core_shifts.append(shift)
 * 
 *                                 if not self.extended_fucosylation:             # <<<<<<<<<<<<<<
 *                                     if iteration_count < fucose_count:
 *                                         fucosylated = self.fucosylate_increment(shift)
 */
              __pyx_t_9 = ((!(__pyx_v_self->extended_fucosylation != 0)) != 0);
              if (__pyx_t_9) {

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":466
 * 
 *                                 if not self.extended_fucosylation:
 *                                     if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                                         fucosylated = self.fucosylate_increment(shift)
 *                                         core_shifts.append(fucosylated)
 */
                __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_fucose_count) != 0);
                if (__pyx_t_9) {

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":467
 *                                 if not self.extended_fucosylation:
 *                                     if iteration_count < fucose_count:
 *                                         fucosylated = self.fucosylate_increment(shift)             # <<<<<<<<<<<<<<
 *                                         core_shifts.append(fucosylated)
 *                                         if iteration_count < xylose_count:
 */
                  __pyx_t_5 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->fucosylate_increment(__pyx_v_self, __pyx_v_shift, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 467, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_5);
                  __Pyx_XDECREF_SET(__pyx_v_fucosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_5));
                  __pyx_t_5 = 0;

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":468
 *                                     if iteration_count < fucose_count:
 *                                         fucosylated = self.fucosylate_increment(shift)
 *                                         core_shifts.append(fucosylated)             # <<<<<<<<<<<<<<
 *                                         if iteration_count < xylose_count:
 *                                             xylosylated = self.xylosylate_increment(fucosylated)
 */
                  __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_fucosylated)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 468, __pyx_L1_error)

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":469
 *                                         fucosylated = self.fucosylate_increment(shift)
 *                                         core_shifts.append(fucosylated)
 *                                         if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                                             xylosylated = self.xylosylate_increment(fucosylated)
 *                                             core_shifts.append(xylosylated)
 */
                  __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_xylose_count) != 0);
                  if (__pyx_t_9) {

                    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":470
 *                                         core_shifts.append(fucosylated)
 *                                         if iteration_count < xylose_count:
 *                                             xylosylated = self.xylosylate_increment(fucosylated)             # <<<<<<<<<<<<<<
 *                                             core_shifts.append(xylosylated)
 *                                 elif fucose_count > 0:
 */
                    __pyx_t_5 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->xylosylate_increment(__pyx_v_self, __pyx_v_fucosylated, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 470, __pyx_L1_error)
                    __Pyx_GOTREF(__pyx_t_5);
                    __Pyx_XDECREF_SET(__pyx_v_xylosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_5));
                    __pyx_t_5 = 0;

                    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":471
 *                                         if iteration_count < xylose_count:
 *                                             xylosylated = self.xylosylate_increment(fucosylated)
 *                                             core_shifts.append(xylosylated)             # <<<<<<<<<<<<<<
 *                                 elif fucose_count > 0:
 *                                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 */
                    __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_xylosylated)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 471, __pyx_L1_error)

                    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":469
 *                                         fucosylated = self.fucosylate_increment(shift)
 *                                         core_shifts.append(fucosylated)
 *                                         if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                                             xylosylated = self.xylosylate_increment(fucosylated)
 *                                             core_shifts.append(xylosylated)
 */
                  }

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":466
 * 
 *                                 if not self.extended_fucosylation:
 *                                     if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                                         fucosylated = self.fucosylate_increment(shift)
 *                                         core_shifts.append(fucosylated)
 */
                }

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":465
 *                                 core_shifts.append(shift)
 * 
 *                                 if not self.extended_fucosylation:             # <<<<<<<<<<<<<<
 *                                     if iteration_count < fucose_count:
 *                                         fucosylated = self.fucosylate_increment(shift)
 */
                goto __pyx_L27;
              }

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":472
 *                                             xylosylated = self.xylosylate_increment(fucosylated)
 *                                             core_shifts.append(xylosylated)
 *                                 elif fucose_count > 0:             # <<<<<<<<<<<<<<
 *                                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 * 
 */
              __pyx_t_9 = ((__pyx_v_fucose_count > 0) != 0);
              if (__pyx_t_9) {

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":473
 *                                             core_shifts.append(xylosylated)
 *                                 elif fucose_count > 0:
 *                                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))             # <<<<<<<<<<<<<<
 * 
 *                                 if iteration_count < xylose_count:
 */
                __pyx_t_5 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->fucosylate_extended(__pyx_v_self, __pyx_v_shift, __pyx_v_fucose_count, 0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 473, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_5);
                __pyx_t_13 = __Pyx_PyList_Extend(__pyx_v_core_shifts, __pyx_t_5); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 473, __pyx_L1_error)
                __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":472
 *                                             xylosylated = self.xylosylate_increment(fucosylated)
 *                                             core_shifts.append(xylosylated)
 *                                 elif fucose_count > 0:             # <<<<<<<<<<<<<<
 *                                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 * 
 */
              }
              __pyx_L27:;

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":475
 *                                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 * 
 *                                 if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                                     xylosylated = self.xylosylate_increment(shift)
 *                                     core_shifts.append(xylosylated)
 */
              __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_xylose_count) != 0);
              if (__pyx_t_9) {

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":476
 * 
 *                                 if iteration_count < xylose_count:
 *                                     xylosylated = self.xylosylate_increment(shift)             # <<<<<<<<<<<<<<
 *                                     core_shifts.append(xylosylated)
 * 
 */
                __pyx_t_5 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->xylosylate_increment(__pyx_v_self, __pyx_v_shift, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 476, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_5);
                __Pyx_XDECREF_SET(__pyx_v_xylosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_5));
                __pyx_t_5 = 0;

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":477
 *                                 if iteration_count < xylose_count:
 *                                     xylosylated = self.xylosylate_increment(shift)
 *                                     core_shifts.append(xylosylated)             # <<<<<<<<<<<<<<
 * 
 *                             for extra_hexose_count in range(1, hexose_in_aggregate - hexose_count + 1):
 */
                __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_xylosylated)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 477, __pyx_L1_error)

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":475
 *                                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 * 
 *                                 if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                                     xylosylated = self.xylosylate_increment(shift)
 *                                     core_shifts.append(xylosylated)
 */
              }

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":452
 *                             if extra_hexnac_count + hexnac_count > hexnac_in_aggregate:
 *                                 continue
 *                             if extra_hexnac_count > 0:             # <<<<<<<<<<<<<<
 *                                 key_ct = CountTable._create()
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 */
            }

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":479
 *                                     core_shifts.append(xylosylated)
 * 
 *                             for extra_hexose_count in range(1, hexose_in_aggregate - hexose_count + 1):             # <<<<<<<<<<<<<<
 *                                 if extra_hexose_count + hexose_count > hexose_in_aggregate:
 *                                     continue
 */
            __pyx_t_21 = ((__pyx_v_hexose_in_aggregate - __pyx_v_hexose_count) + 1);
            __pyx_t_22 = __pyx_t_21;
            for (__pyx_t_23 = 1; __pyx_t_23 < __pyx_t_22; __pyx_t_23+=1) {
              __pyx_v_extra_hexose_count = __pyx_t_23;

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":480
 * 
 *                             for extra_hexose_count in range(1, hexose_in_aggregate - hexose_count + 1):
 *                                 if extra_hexose_count + hexose_count > hexose_in_aggregate:             # <<<<<<<<<<<<<<
 *                                     continue
 *                                 key_ct = CountTable._create()
 */
              __pyx_t_9 = (((__pyx_v_extra_hexose_count + __pyx_v_hexose_count) > __pyx_v_hexose_in_aggregate) != 0);
              if (__pyx_t_9) {

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":481
 *                             for extra_hexose_count in range(1, hexose_in_aggregate - hexose_count + 1):
 *                                 if extra_hexose_count + hexose_count > hexose_in_aggregate:
 *                                     continue             # <<<<<<<<<<<<<<
 *                                 key_ct = CountTable._create()
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 */
                goto __pyx_L31_continue;

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":480
 * 
 *                             for extra_hexose_count in range(1, hexose_in_aggregate - hexose_count + 1):
 *                                 if extra_hexose_count + hexose_count > hexose_in_aggregate:             # <<<<<<<<<<<<<<
 *                                     continue
 *                                 key_ct = CountTable._create()
 */
              }

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":482
 *                                 if extra_hexose_count + hexose_count > hexose_in_aggregate:
 *                                     continue
 *                                 key_ct = CountTable._create()             # <<<<<<<<<<<<<<
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 *                                 key_ct.setitem(_HEX, hexose_count + extra_hexose_count)
 */
              __pyx_t_5 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable->_create()); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 482, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_5);
              __Pyx_DECREF_SET(__pyx_v_key_ct, ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_5));
              __pyx_t_5 = 0;

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":483
 *                                     continue
 *                                 key_ct = CountTable._create()
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)             # <<<<<<<<<<<<<<
 *                                 key_ct.setitem(_HEX, hexose_count + extra_hexose_count)
 *                                 shift = GlycanCompositionFragment._create(
 */
              __pyx_t_5 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC;
              __Pyx_INCREF(__pyx_t_5);
              __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_5, (__pyx_v_hexnac_count + __pyx_v_extra_hexnac_count)); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 483, __pyx_L1_error)
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":484
 *                                 key_ct = CountTable._create()
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 *                                 key_ct.setitem(_HEX, hexose_count + extra_hexose_count)             # <<<<<<<<<<<<<<
 *                                 shift = GlycanCompositionFragment._create(
 *                                     ((hexnac_count + extra_hexnac_count) * _HEXNAC_mass) + (
 */
              __pyx_t_5 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX;
              __Pyx_INCREF(__pyx_t_5);
              __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_5, (__pyx_v_hexose_count + __pyx_v_extra_hexose_count)); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 484, __pyx_L1_error)
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":488
 *                                     ((hexnac_count + extra_hexnac_count) * _HEXNAC_mass) + (
 *                                         (hexose_count + extra_hexose_count) * _HEX_mass),
 *                                     hexnac_hex_composition(             # <<<<<<<<<<<<<<
 *                                         hexnac_count + extra_hexnac_count, hexose_count + extra_hexose_count),
 * 
 */
              __pyx_t_5 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_hexnac_hex_composition((__pyx_v_hexnac_count + __pyx_v_extra_hexnac_count), (__pyx_v_hexose_count + __pyx_v_extra_hexose_count))); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 488, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_5);

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":485
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 *                                 key_ct.setitem(_HEX, hexose_count + extra_hexose_count)
 *                                 shift = GlycanCompositionFragment._create(             # <<<<<<<<<<<<<<
 *                                     ((hexnac_count + extra_hexnac_count) * _HEXNAC_mass) + (
 *                                         (hexose_count + extra_hexose_count) * _HEX_mass),
 */
              __pyx_t_2 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create((((__pyx_v_hexnac_count + __pyx_v_extra_hexnac_count) * __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC_mass) + ((__pyx_v_hexose_count + __pyx_v_extra_hexose_count) * __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX_mass)), ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_5), __pyx_v_key_ct, 1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 485, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
              __Pyx_DECREF_SET(__pyx_v_shift, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2));
              __pyx_t_2 = 0;

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":494
 *                                     True
 *                                 )
 *                                 core_shifts.append(shift)             # <<<<<<<<<<<<<<
 * 
 *                                 if not self.extended_fucosylation:
 */
              __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_shift)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 494, __pyx_L1_error)

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":496
 *                                 core_shifts.append(shift)
 * 
 *                                 if not self.extended_fucosylation:             # <<<<<<<<<<<<<<
 *                                     if iteration_count < fucose_count:
 *                                         fucosylated = self.fucosylate_increment(shift)
 */
              __pyx_t_9 = ((!(__pyx_v_self->extended_fucosylation != 0)) != 0);
              if (__pyx_t_9) {

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":497
 * 
 *                                 if not self.extended_fucosylation:
 *                                     if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                                         fucosylated = self.fucosylate_increment(shift)
 *                                         core_shifts.append(fucosylated)
 */
                __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_fucose_count) != 0);
                if (__pyx_t_9) {

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":498
 *                                 if not self.extended_fucosylation:
 *                                     if iteration_count < fucose_count:
 *                                         fucosylated = self.fucosylate_increment(shift)             # <<<<<<<<<<<<<<
 *                                         core_shifts.append(fucosylated)
 *                                         if iteration_count < xylose_count:
 */
                  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->fucosylate_increment(__pyx_v_self, __pyx_v_shift, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 498, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_2);
                  __Pyx_XDECREF_SET(__pyx_v_fucosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2));
                  __pyx_t_2 = 0;

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":499
 *                                     if iteration_count < fucose_count:
 *                                         fucosylated = self.fucosylate_increment(shift)
 *                                         core_shifts.append(fucosylated)             # <<<<<<<<<<<<<<
 *                                         if iteration_count < xylose_count:
 *                                             xylosylated = self.xylosylate_increment(fucosylated)
 */
                  __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_fucosylated)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 499, __pyx_L1_error)

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":500
 *                                         fucosylated = self.fucosylate_increment(shift)
 *                                         core_shifts.append(fucosylated)
 *                                         if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                                             xylosylated = self.xylosylate_increment(fucosylated)
 *                                             core_shifts.append(xylosylated)
 */
                  __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_xylose_count) != 0);
                  if (__pyx_t_9) {

                    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":501
 *                                         core_shifts.append(fucosylated)
 *                                         if iteration_count < xylose_count:
 *                                             xylosylated = self.xylosylate_increment(fucosylated)             # <<<<<<<<<<<<<<
 *                                             core_shifts.append(xylosylated)
 *                                 elif fucose_count > 0:
 */
                    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->xylosylate_increment(__pyx_v_self, __pyx_v_fucosylated, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 501, __pyx_L1_error)
                    __Pyx_GOTREF(__pyx_t_2);
                    __Pyx_XDECREF_SET(__pyx_v_xylosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2));
                    __pyx_t_2 = 0;

                    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":502
 *                                         if iteration_count < xylose_count:
 *                                             xylosylated = self.xylosylate_increment(fucosylated)
 *                                             core_shifts.append(xylosylated)             # <<<<<<<<<<<<<<
 *                                 elif fucose_count > 0:
 *                                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 */
                    __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_xylosylated)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 502, __pyx_L1_error)

                    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":500
 *                                         fucosylated = self.fucosylate_increment(shift)
 *                                         core_shifts.append(fucosylated)
 *                                         if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                                             xylosylated = self.xylosylate_increment(fucosylated)
 *                                             core_shifts.append(xylosylated)
 */
                  }

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":497
 * 
 *                                 if not self.extended_fucosylation:
 *                                     if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                                         fucosylated = self.fucosylate_increment(shift)
 *                                         core_shifts.append(fucosylated)
 */
                }

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":496
 *                                 core_shifts.append(shift)
 * 
 *                                 if not self.extended_fucosylation:             # <<<<<<<<<<<<<<
 *                                     if iteration_count < fucose_count:
 *                                         fucosylated = self.fucosylate_increment(shift)
 */
                goto __pyx_L34;
              }

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":503
 *                                             xylosylated = self.xylosylate_increment(fucosylated)
 *                                             core_shifts.append(xylosylated)
 *                                 elif fucose_count > 0:             # <<<<<<<<<<<<<<
 *                                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 *                                     if iteration_count < xylose_count:
 */
              __pyx_t_9 = ((__pyx_v_fucose_count > 0) != 0);
              if (__pyx_t_9) {

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":504
 *                                             core_shifts.append(xylosylated)
 *                                 elif fucose_count > 0:
 *                                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))             # <<<<<<<<<<<<<<
 *                                     if iteration_count < xylose_count:
 *                                         xylosylated = self.xylosylate_increment(fucosylated)
 */
                __pyx_t_2 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->fucosylate_extended(__pyx_v_self, __pyx_v_shift, __pyx_v_fucose_count, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 504, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_2);
                __pyx_t_13 = __Pyx_PyList_Extend(__pyx_v_core_shifts, __pyx_t_2); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 504, __pyx_L1_error)
                __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":505
 *                                 elif fucose_count > 0:
 *                                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 *                                     if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                                         xylosylated = self.xylosylate_increment(fucosylated)
 *                                         core_shifts.append(xylosylated)
 */
                __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_xylose_count) != 0);
                if (__pyx_t_9) {

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":506
 *                                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 *                                     if iteration_count < xylose_count:
 *                                         xylosylated = self.xylosylate_increment(fucosylated)             # <<<<<<<<<<<<<<
 *                                         core_shifts.append(xylosylated)
 * 
 */
                  if (unlikely(!__pyx_v_fucosylated)) { __Pyx_RaiseUnboundLocalError("fucosylated"); __PYX_ERR(0, 506, __pyx_L1_error) }
                  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->xylosylate_increment(__pyx_v_self, __pyx_v_fucosylated, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 506, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_2);
                  __Pyx_XDECREF_SET(__pyx_v_xylosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2));
                  __pyx_t_2 = 0;

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":507
 *                                     if iteration_count < xylose_count:
 *                                         xylosylated = self.xylosylate_increment(fucosylated)
 *                                         core_shifts.append(xylosylated)             # <<<<<<<<<<<<<<
 * 
 *                                 if iteration_count < xylose_count:
 */
                  __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_xylosylated)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 507, __pyx_L1_error)

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":505
 *                                 elif fucose_count > 0:
 *                                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 *                                     if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                                         xylosylated = self.xylosylate_increment(fucosylated)
 *                                         core_shifts.append(xylosylated)
 */
                }

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":503
 *                                             xylosylated = self.xylosylate_increment(fucosylated)
 *                                             core_shifts.append(xylosylated)
 *                                 elif fucose_count > 0:             # <<<<<<<<<<<<<<
 *                                     core_shifts.extend(self.fucosylate_extended(shift, fucose_count))
 *                                     if iteration_count < xylose_count:
 */
              }
              __pyx_L34:;

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":509
 *                                         core_shifts.append(xylosylated)
 * 
 *                                 if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                                     xylosylated = self.xylosylate_increment(shift)
 *                                     core_shifts.append(xylosylated)
 */
              __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_xylose_count) != 0);
              if (__pyx_t_9) {

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":510
 * 
 *                                 if iteration_count < xylose_count:
 *                                     xylosylated = self.xylosylate_increment(shift)             # <<<<<<<<<<<<<<
 *                                     core_shifts.append(xylosylated)
 *         return core_shifts
 */
                __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->xylosylate_increment(__pyx_v_self, __pyx_v_shift, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 510, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_2);
                __Pyx_XDECREF_SET(__pyx_v_xylosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2));
                __pyx_t_2 = 0;

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":511
 *                                 if iteration_count < xylose_count:
 *                                     xylosylated = self.xylosylate_increment(shift)
 *                                     core_shifts.append(xylosylated)             # <<<<<<<<<<<<<<
 *         return core_shifts
 * 
 */
                __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_xylosylated)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 511, __pyx_L1_error)

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":509
 *                                         core_shifts.append(xylosylated)
 * 
 *                                 if iteration_count < xylose_count:             # <<<<<<<<<<<<<<
 *                                     xylosylated = self.xylosylate_increment(shift)
 *                                     core_shifts.append(xylosylated)
 */
              }
              __pyx_L31_continue:;
            }
            __pyx_L23_continue:;
          }

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":448
 *                     # exhausted. This doesn't allow for one missing alpha-mannose while still
 *                     # showing an extension on the other.
 *                     if hexose_count == 3 and hexnac_in_aggregate >= 2 * core_count and self.extended:             # <<<<<<<<<<<<<<
 *                         for extra_hexnac_count in range(0, hexnac_in_aggregate - hexnac_count + 1):
 *                             if extra_hexnac_count + hexnac_count > hexnac_in_aggregate:
 */
        }
      }

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":387
 *                     fucosylated = self.fucosylate_increment(shift)
 *                     core_shifts.append(fucosylated)
 *             elif hexnac_count == 2:             # <<<<<<<<<<<<<<
 *                 key_ct = CountTable._create()
 *                 key_ct.setitem(_HEXNAC, hexnac_count)
 */
      break;
      default: break;
    }
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":512
 *                                     xylosylated = self.xylosylate_increment(shift)
 *                                     core_shifts.append(xylosylated)
 *         return core_shifts             # <<<<<<<<<<<<<<
 * 
 *     cpdef list _combinate_sites(self, list per_site_shifts, object glycan):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_core_shifts);
  __pyx_r = __pyx_v_core_shifts;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":325
 *         return modified
 * 
 *     cpdef list n_glycan_composition_fragments(self, object glycan, int core_count=1, int  iteration_count=0):             # <<<<<<<<<<<<<<
 *         """Generate theoretical N-glycan Y fragment compositions containing the core motif
 *         plus a portion of the extended branches if :attr:`extended` is used. Unless :attr:`extend
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.n_glycan_composition_fragments", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_key_ct);
  __Pyx_XDECREF(__pyx_v_core_shifts);
  __Pyx_XDECREF((PyObject *)__pyx_v_shift);
  __Pyx_XDECREF((PyObject *)__pyx_v_fucosylated);
  __Pyx_XDECREF((PyObject *)__pyx_v_xylosylated);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_15n_glycan_composition_fragments(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_14n_glycan_composition_fragments[] = "StubGlycopeptideStrategy.n_glycan_composition_fragments(self, glycan, int core_count=1, int iteration_count=0) -> list\nGenerate theoretical N-glycan Y fragment compositions containing the core motif\n        plus a portion of the extended branches if :attr:`extended` is used. Unless :attr:`extend\n\n        Parameters\n        ----------\n        glycan : :class:`~.GlycanComposition`\n            Description\n        core_count : int, optional\n            The total number of glycans attached to the peptide\n        iteration_count : int, optional\n            The core index. Unless :attr:`extended_fucosylation` is true, this will\n            limit the number of Fucose residues to one per core at most, and if\n            the iteration count exceeds the number of cores no Fucose will be added\n\n        Returns\n        -------\n        list of dict\n            The compositions corresponding to the theoretical fragments\n        ";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_15n_glycan_composition_fragments(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_glycan = 0;
  int __pyx_v_core_count;
  int __pyx_v_iteration_count;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("n_glycan_composition_fragments (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_glycan,&__pyx_n_s_core_count,&__pyx_n_s_iteration_count,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_glycan)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_core_count);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_iteration_count);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "n_glycan_composition_fragments") < 0)) __PYX_ERR(0, 325, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_glycan = values[0];
    if (values[1]) {
      __pyx_v_core_count = __Pyx_PyInt_As_int(values[1]); if (unlikely((__pyx_v_core_count == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 325, __pyx_L3_error)
    } else {
      __pyx_v_core_count = ((int)1);
    }
    if (values[2]) {
      __pyx_v_iteration_count = __Pyx_PyInt_As_int(values[2]); if (unlikely((__pyx_v_iteration_count == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 325, __pyx_L3_error)
    } else {
      __pyx_v_iteration_count = ((int)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("n_glycan_composition_fragments", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 325, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.n_glycan_composition_fragments", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_14n_glycan_composition_fragments(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self), __pyx_v_glycan, __pyx_v_core_count, __pyx_v_iteration_count);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_14n_glycan_composition_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_glycan, int __pyx_v_core_count, int __pyx_v_iteration_count) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_n_glycan_composition_fragments __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("n_glycan_composition_fragments", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 2;
  __pyx_t_2.core_count = __pyx_v_core_count;
  __pyx_t_2.iteration_count = __pyx_v_iteration_count;
  __pyx_t_1 = __pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy->n_glycan_composition_fragments(__pyx_v_self, __pyx_v_glycan, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 325, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.n_glycan_composition_fragments", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":514
 *         return core_shifts
 * 
 *     cpdef list _combinate_sites(self, list per_site_shifts, object glycan):             # <<<<<<<<<<<<<<
 *         cdef:
 *             list result
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_17_combinate_sites(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy__combinate_sites(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_per_site_shifts, PyObject *__pyx_v_glycan, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_result = 0;
  PyObject *__pyx_v_combos = 0;
  Py_ssize_t __pyx_v_i;
  Py_ssize_t __pyx_v_n;
  Py_ssize_t __pyx_v_j;
  Py_ssize_t __pyx_v_m;
  PyObject *__pyx_v_positions = 0;
  int __pyx_v_is_extended;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_composition = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_intermediate = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_site = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *__pyx_v_aggregate_glycosylation = 0;
  PyObject *__pyx_v_glycosylation = 0;
  PyObject *__pyx_v_seen = 0;
  int __pyx_v_core_count;
  double __pyx_v_total_mass;
  double __pyx_v_acc_mass;
  PyObject *__pyx_v_invalid = NULL;
  PyObject *__pyx_v_name_key = NULL;
  PyObject *__pyx_v_ptemp;
  PyObject *__pyx_v_name = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  Py_ssize_t __pyx_t_7;
  double __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  Py_ssize_t __pyx_t_10;
  PyObject *__pyx_t_11;
  int __pyx_t_12;
  Py_ssize_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  Py_ssize_t __pyx_t_15;
  int __pyx_t_16;
  int __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_combinate_sites", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_combinate_sites); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 514, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_17_combinate_sites)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_per_site_shifts, __pyx_v_glycan};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 514, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_per_site_shifts, __pyx_v_glycan};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 514, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 514, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(__pyx_v_per_site_shifts);
          __Pyx_GIVEREF(__pyx_v_per_site_shifts);
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_per_site_shifts);
          __Pyx_INCREF(__pyx_v_glycan);
          __Pyx_GIVEREF(__pyx_v_glycan);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_glycan);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 514, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 514, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":530
 *             double acc_mass
 * 
 *         result = []             # <<<<<<<<<<<<<<
 *         seen = set()
 *         core_count = PyList_Size(per_site_shifts)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 530, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_result = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":531
 * 
 *         result = []
 *         seen = set()             # <<<<<<<<<<<<<<
 *         core_count = PyList_Size(per_site_shifts)
 *         combos = list(product(*per_site_shifts))
 */
  __pyx_t_1 = PySet_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 531, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_seen = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":532
 *         result = []
 *         seen = set()
 *         core_count = PyList_Size(per_site_shifts)             # <<<<<<<<<<<<<<
 *         combos = list(product(*per_site_shifts))
 *         n = PyList_Size(combos)
 */
  __pyx_t_7 = PyList_Size(__pyx_v_per_site_shifts); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1L))) __PYX_ERR(0, 532, __pyx_L1_error)
  __pyx_v_core_count = __pyx_t_7;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":533
 *         seen = set()
 *         core_count = PyList_Size(per_site_shifts)
 *         combos = list(product(*per_site_shifts))             # <<<<<<<<<<<<<<
 *         n = PyList_Size(combos)
 *         total_mass = PyFloat_AsDouble(glycan.mass())
 */
  __pyx_t_1 = PySequence_Tuple(__pyx_v_per_site_shifts); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_Call(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_product, __pyx_t_1, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PySequence_List(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 533, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_combos = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":534
 *         core_count = PyList_Size(per_site_shifts)
 *         combos = list(product(*per_site_shifts))
 *         n = PyList_Size(combos)             # <<<<<<<<<<<<<<
 *         total_mass = PyFloat_AsDouble(glycan.mass())
 *         for i in range(n):
 */
  __pyx_t_7 = PyList_Size(__pyx_v_combos); if (unlikely(__pyx_t_7 == ((Py_ssize_t)-1L))) __PYX_ERR(0, 534, __pyx_L1_error)
  __pyx_v_n = __pyx_t_7;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":535
 *         combos = list(product(*per_site_shifts))
 *         n = PyList_Size(combos)
 *         total_mass = PyFloat_AsDouble(glycan.mass())             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             positions = <tuple>PyList_GET_ITEM(combos, i)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_glycan, __pyx_n_s_mass); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_8 = PyFloat_AsDouble(__pyx_t_1); if (unlikely(__pyx_t_8 == ((double)-1.0) && PyErr_Occurred())) __PYX_ERR(0, 535, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_total_mass = __pyx_t_8;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":536
 *         n = PyList_Size(combos)
 *         total_mass = PyFloat_AsDouble(glycan.mass())
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             positions = <tuple>PyList_GET_ITEM(combos, i)
 *             aggregate_glycosylation = None
 */
  __pyx_t_7 = __pyx_v_n;
  __pyx_t_9 = __pyx_t_7;
  for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
    __pyx_v_i = __pyx_t_10;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":537
 *         total_mass = PyFloat_AsDouble(glycan.mass())
 *         for i in range(n):
 *             positions = <tuple>PyList_GET_ITEM(combos, i)             # <<<<<<<<<<<<<<
 *             aggregate_glycosylation = None
 *             composition = None
 */
    __pyx_t_11 = PyList_GET_ITEM(__pyx_v_combos, __pyx_v_i);
    __pyx_t_1 = ((PyObject *)__pyx_t_11);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_positions, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":538
 *         for i in range(n):
 *             positions = <tuple>PyList_GET_ITEM(combos, i)
 *             aggregate_glycosylation = None             # <<<<<<<<<<<<<<
 *             composition = None
 *             if core_count == 1:
 */
    __Pyx_INCREF(Py_None);
    __Pyx_XDECREF_SET(__pyx_v_aggregate_glycosylation, ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)Py_None));

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":539
 *             positions = <tuple>PyList_GET_ITEM(combos, i)
 *             aggregate_glycosylation = None
 *             composition = None             # <<<<<<<<<<<<<<
 *             if core_count == 1:
 *                 intermediate = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, 0)
 */
    __Pyx_INCREF(Py_None);
    __Pyx_XDECREF_SET(__pyx_v_composition, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None));

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":540
 *             aggregate_glycosylation = None
 *             composition = None
 *             if core_count == 1:             # <<<<<<<<<<<<<<
 *                 intermediate = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, 0)
 *                 aggregate_glycosylation = intermediate.key
 */
    __pyx_t_12 = ((__pyx_v_core_count == 1) != 0);
    if (__pyx_t_12) {

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":541
 *             composition = None
 *             if core_count == 1:
 *                 intermediate = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, 0)             # <<<<<<<<<<<<<<
 *                 aggregate_glycosylation = intermediate.key
 *             else:
 */
      __pyx_t_11 = PyTuple_GET_ITEM(__pyx_v_positions, 0);
      __pyx_t_1 = ((PyObject *)__pyx_t_11);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_intermediate, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":542
 *             if core_count == 1:
 *                 intermediate = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, 0)
 *                 aggregate_glycosylation = intermediate.key             # <<<<<<<<<<<<<<
 *             else:
 *                 intermediate = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, 0)
 */
      __pyx_t_1 = ((PyObject *)__pyx_v_intermediate->key);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_DECREF_SET(__pyx_v_aggregate_glycosylation, ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":540
 *             aggregate_glycosylation = None
 *             composition = None
 *             if core_count == 1:             # <<<<<<<<<<<<<<
 *                 intermediate = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, 0)
 *                 aggregate_glycosylation = intermediate.key
 */
      goto __pyx_L5;
    }

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":544
 *                 aggregate_glycosylation = intermediate.key
 *             else:
 *                 intermediate = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, 0)             # <<<<<<<<<<<<<<
 *                 acc_mass = intermediate.mass
 *                 m = core_count
 */
    /*else*/ {
      __pyx_t_11 = PyTuple_GET_ITEM(__pyx_v_positions, 0);
      __pyx_t_1 = ((PyObject *)__pyx_t_11);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_intermediate, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":545
 *             else:
 *                 intermediate = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, 0)
 *                 acc_mass = intermediate.mass             # <<<<<<<<<<<<<<
 *                 m = core_count
 * 
 */
      __pyx_t_8 = __pyx_v_intermediate->mass;
      __pyx_v_acc_mass = __pyx_t_8;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":546
 *                 intermediate = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, 0)
 *                 acc_mass = intermediate.mass
 *                 m = core_count             # <<<<<<<<<<<<<<
 * 
 *                 # A quick pre-scan to skip all combinations that exceed the mass of the total
 */
      __pyx_v_m = __pyx_v_core_count;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":549
 * 
 *                 # A quick pre-scan to skip all combinations that exceed the mass of the total
 *                 for j in range(1, m):             # <<<<<<<<<<<<<<
 *                     site = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, j)
 *                     acc_mass += site.mass
 */
      __pyx_t_13 = __pyx_v_m;
      __pyx_t_14 = __pyx_t_13;
      for (__pyx_t_15 = 1; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
        __pyx_v_j = __pyx_t_15;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":550
 *                 # A quick pre-scan to skip all combinations that exceed the mass of the total
 *                 for j in range(1, m):
 *                     site = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, j)             # <<<<<<<<<<<<<<
 *                     acc_mass += site.mass
 *                 invalid = total_mass < acc_mass
 */
        __pyx_t_11 = PyTuple_GET_ITEM(__pyx_v_positions, __pyx_v_j);
        __pyx_t_1 = ((PyObject *)__pyx_t_11);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_XDECREF_SET(__pyx_v_site, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":551
 *                 for j in range(1, m):
 *                     site = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, j)
 *                     acc_mass += site.mass             # <<<<<<<<<<<<<<
 *                 invalid = total_mass < acc_mass
 *                 if invalid:
 */
        __pyx_v_acc_mass = (__pyx_v_acc_mass + __pyx_v_site->mass);
      }

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":552
 *                     site = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, j)
 *                     acc_mass += site.mass
 *                 invalid = total_mass < acc_mass             # <<<<<<<<<<<<<<
 *                 if invalid:
 *                     continue
 */
      __pyx_t_1 = __Pyx_PyBool_FromLong((__pyx_v_total_mass < __pyx_v_acc_mass)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 552, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_invalid, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":553
 *                     acc_mass += site.mass
 *                 invalid = total_mass < acc_mass
 *                 if invalid:             # <<<<<<<<<<<<<<
 *                     continue
 * 
 */
      __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_v_invalid); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 553, __pyx_L1_error)
      if (__pyx_t_12) {

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":554
 *                 invalid = total_mass < acc_mass
 *                 if invalid:
 *                     continue             # <<<<<<<<<<<<<<
 * 
 *                 # Now do the expensive accumulation
 */
        goto __pyx_L3_continue;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":553
 *                     acc_mass += site.mass
 *                 invalid = total_mass < acc_mass
 *                 if invalid:             # <<<<<<<<<<<<<<
 *                     continue
 * 
 */
      }

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":557
 * 
 *                 # Now do the expensive accumulation
 *                 aggregate_glycosylation = intermediate.key.copy()             # <<<<<<<<<<<<<<
 *                 if self.compute_compositions:
 *                     composition = intermediate.composition.copy()
 */
      __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_intermediate->key->__pyx_vtab)->copy(__pyx_v_intermediate->key, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 557, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF_SET(__pyx_v_aggregate_glycosylation, ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":558
 *                 # Now do the expensive accumulation
 *                 aggregate_glycosylation = intermediate.key.copy()
 *                 if self.compute_compositions:             # <<<<<<<<<<<<<<
 *                     composition = intermediate.composition.copy()
 *                 else:
 */
      __pyx_t_12 = (__pyx_v_self->__pyx_base.__pyx_base.compute_compositions != 0);
      if (__pyx_t_12) {

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":559
 *                 aggregate_glycosylation = intermediate.key.copy()
 *                 if self.compute_compositions:
 *                     composition = intermediate.composition.copy()             # <<<<<<<<<<<<<<
 *                 else:
 *                     composition = None
 */
        __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_intermediate->composition), __pyx_n_s_copy); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 559, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __pyx_t_3 = NULL;
        if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
          __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
          if (likely(__pyx_t_3)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
            __Pyx_INCREF(__pyx_t_3);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_2, function);
          }
        }
        __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 559, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 559, __pyx_L1_error)
        __Pyx_DECREF_SET(__pyx_v_composition, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":558
 *                 # Now do the expensive accumulation
 *                 aggregate_glycosylation = intermediate.key.copy()
 *                 if self.compute_compositions:             # <<<<<<<<<<<<<<
 *                     composition = intermediate.composition.copy()
 *                 else:
 */
        goto __pyx_L9;
      }

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":561
 *                     composition = intermediate.composition.copy()
 *                 else:
 *                     composition = None             # <<<<<<<<<<<<<<
 *                 is_extended = intermediate.is_extended
 *                 for j in range(1, m):
 */
      /*else*/ {
        __Pyx_INCREF(Py_None);
        __Pyx_DECREF_SET(__pyx_v_composition, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None));
      }
      __pyx_L9:;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":562
 *                 else:
 *                     composition = None
 *                 is_extended = intermediate.is_extended             # <<<<<<<<<<<<<<
 *                 for j in range(1, m):
 *                     site = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, j)
 */
      __pyx_t_12 = __pyx_v_intermediate->is_extended;
      __pyx_v_is_extended = __pyx_t_12;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":563
 *                     composition = None
 *                 is_extended = intermediate.is_extended
 *                 for j in range(1, m):             # <<<<<<<<<<<<<<
 *                     site = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, j)
 *                     aggregate_glycosylation._add_from(site.key)
 */
      __pyx_t_13 = __pyx_v_m;
      __pyx_t_14 = __pyx_t_13;
      for (__pyx_t_15 = 1; __pyx_t_15 < __pyx_t_14; __pyx_t_15+=1) {
        __pyx_v_j = __pyx_t_15;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":564
 *                 is_extended = intermediate.is_extended
 *                 for j in range(1, m):
 *                     site = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, j)             # <<<<<<<<<<<<<<
 *                     aggregate_glycosylation._add_from(site.key)
 *                     if self.compute_compositions:
 */
        __pyx_t_11 = PyTuple_GET_ITEM(__pyx_v_positions, __pyx_v_j);
        __pyx_t_1 = ((PyObject *)__pyx_t_11);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_XDECREF_SET(__pyx_v_site, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":565
 *                 for j in range(1, m):
 *                     site = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, j)
 *                     aggregate_glycosylation._add_from(site.key)             # <<<<<<<<<<<<<<
 *                     if self.compute_compositions:
 *                         composition.add_from(site.composition)
 */
        __pyx_t_1 = ((PyObject *)__pyx_v_site->key);
        __Pyx_INCREF(__pyx_t_1);
        ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_aggregate_glycosylation->__pyx_vtab)->_add_from(__pyx_v_aggregate_glycosylation, ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_1));
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":566
 *                     site = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, j)
 *                     aggregate_glycosylation._add_from(site.key)
 *                     if self.compute_compositions:             # <<<<<<<<<<<<<<
 *                         composition.add_from(site.composition)
 *                     is_extended |= site.is_extended
 */
        __pyx_t_12 = (__pyx_v_self->__pyx_base.__pyx_base.compute_compositions != 0);
        if (__pyx_t_12) {

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":567
 *                     aggregate_glycosylation._add_from(site.key)
 *                     if self.compute_compositions:
 *                         composition.add_from(site.composition)             # <<<<<<<<<<<<<<
 *                     is_extended |= site.is_extended
 *                 invalid = self._validate_glycan_composition(
 */
          __pyx_t_1 = ((PyObject *)__pyx_v_site->composition);
          __Pyx_INCREF(__pyx_t_1);
          __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_v_composition->__pyx_vtab)->add_from(__pyx_v_composition, __pyx_t_1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 567, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":566
 *                     site = <GlycanCompositionFragment>PyTuple_GET_ITEM(positions, j)
 *                     aggregate_glycosylation._add_from(site.key)
 *                     if self.compute_compositions:             # <<<<<<<<<<<<<<
 *                         composition.add_from(site.composition)
 *                     is_extended |= site.is_extended
 */
        }

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":568
 *                     if self.compute_compositions:
 *                         composition.add_from(site.composition)
 *                     is_extended |= site.is_extended             # <<<<<<<<<<<<<<
 *                 invalid = self._validate_glycan_composition(
 *                     aggregate_glycosylation, glycan)
 */
        __pyx_v_is_extended = (__pyx_v_is_extended | __pyx_v_site->is_extended);
      }

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":569
 *                         composition.add_from(site.composition)
 *                     is_extended |= site.is_extended
 *                 invalid = self._validate_glycan_composition(             # <<<<<<<<<<<<<<
 *                     aggregate_glycosylation, glycan)
 *                 if invalid:
 */
      __pyx_t_2 = __Pyx_PyBool_FromLong(((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_validate_glycan_composition(__pyx_v_self, __pyx_v_aggregate_glycosylation, __pyx_v_glycan, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 569, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF_SET(__pyx_v_invalid, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":571
 *                 invalid = self._validate_glycan_composition(
 *                     aggregate_glycosylation, glycan)
 *                 if invalid:             # <<<<<<<<<<<<<<
 *                     continue
 * 
 */
      __pyx_t_12 = __Pyx_PyObject_IsTrue(__pyx_v_invalid); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 571, __pyx_L1_error)
      if (__pyx_t_12) {

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":572
 *                     aggregate_glycosylation, glycan)
 *                 if invalid:
 *                     continue             # <<<<<<<<<<<<<<
 * 
 *                 intermediate = GlycanCompositionFragment._create(
 */
        goto __pyx_L3_continue;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":571
 *                 invalid = self._validate_glycan_composition(
 *                     aggregate_glycosylation, glycan)
 *                 if invalid:             # <<<<<<<<<<<<<<
 *                     continue
 * 
 */
      }

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":574
 *                     continue
 * 
 *                 intermediate = GlycanCompositionFragment._create(             # <<<<<<<<<<<<<<
 *                     acc_mass, composition, aggregate_glycosylation, is_extended)
 * 
 */
      __pyx_t_2 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create(__pyx_v_acc_mass, __pyx_v_composition, __pyx_v_aggregate_glycosylation, __pyx_v_is_extended)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 574, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF_SET(__pyx_v_intermediate, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2));
      __pyx_t_2 = 0;
    }
    __pyx_L5:;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":577
 *                     acc_mass, composition, aggregate_glycosylation, is_extended)
 * 
 *             glycosylation = _prepare_glycan_composition_from_mapping(             # <<<<<<<<<<<<<<
 *                 aggregate_glycosylation)
 *             name_key = str(glycosylation)
 */
    __pyx_t_2 = __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__prepare_glycan_composition_from_mapping(__pyx_v_aggregate_glycosylation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 577, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_glycosylation, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":579
 *             glycosylation = _prepare_glycan_composition_from_mapping(
 *                 aggregate_glycosylation)
 *             name_key = str(glycosylation)             # <<<<<<<<<<<<<<
 *             # May not need to do this second round of building.
 *             ptemp = PyDict_GetItem(_composition_name_cache, name_key)
 */
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_glycosylation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 579, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_XDECREF_SET(__pyx_v_name_key, __pyx_t_2);
    __pyx_t_2 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":581
 *             name_key = str(glycosylation)
 *             # May not need to do this second round of building.
 *             ptemp = PyDict_GetItem(_composition_name_cache, name_key)             # <<<<<<<<<<<<<<
 *             if ptemp == NULL:
 *                 name = build_name_from_composition(aggregate_glycosylation)
 */
    __pyx_t_2 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__composition_name_cache;
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v_ptemp = PyDict_GetItem(__pyx_t_2, __pyx_v_name_key);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":582
 *             # May not need to do this second round of building.
 *             ptemp = PyDict_GetItem(_composition_name_cache, name_key)
 *             if ptemp == NULL:             # <<<<<<<<<<<<<<
 *                 name = build_name_from_composition(aggregate_glycosylation)
 *                 PyDict_SetItem(_composition_name_cache, name_key, name)
 */
    __pyx_t_12 = ((__pyx_v_ptemp == NULL) != 0);
    if (__pyx_t_12) {

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":583
 *             ptemp = PyDict_GetItem(_composition_name_cache, name_key)
 *             if ptemp == NULL:
 *                 name = build_name_from_composition(aggregate_glycosylation)             # <<<<<<<<<<<<<<
 *                 PyDict_SetItem(_composition_name_cache, name_key, name)
 *             else:
 */
      __pyx_t_2 = __pyx_fuse_0__pyx_f_14glycopeptidepy_2_c_9structure_8fragment_build_name_from_composition(__pyx_v_aggregate_glycosylation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 583, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_XDECREF_SET(__pyx_v_name, __pyx_t_2);
      __pyx_t_2 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":584
 *             if ptemp == NULL:
 *                 name = build_name_from_composition(aggregate_glycosylation)
 *                 PyDict_SetItem(_composition_name_cache, name_key, name)             # <<<<<<<<<<<<<<
 *             else:
 *                 name = <str>ptemp
 */
      __pyx_t_2 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__composition_name_cache;
      __Pyx_INCREF(__pyx_t_2);
      __pyx_t_5 = PyDict_SetItem(__pyx_t_2, __pyx_v_name_key, __pyx_v_name); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 584, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":582
 *             # May not need to do this second round of building.
 *             ptemp = PyDict_GetItem(_composition_name_cache, name_key)
 *             if ptemp == NULL:             # <<<<<<<<<<<<<<
 *                 name = build_name_from_composition(aggregate_glycosylation)
 *                 PyDict_SetItem(_composition_name_cache, name_key, name)
 */
      goto __pyx_L14;
    }

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":586
 *                 PyDict_SetItem(_composition_name_cache, name_key, name)
 *             else:
 *                 name = <str>ptemp             # <<<<<<<<<<<<<<
 *             if name in seen:
 *                 continue
 */
    /*else*/ {
      __pyx_t_2 = ((PyObject *)__pyx_v_ptemp);
      __Pyx_INCREF(__pyx_t_2);
      __Pyx_XDECREF_SET(__pyx_v_name, __pyx_t_2);
      __pyx_t_2 = 0;
    }
    __pyx_L14:;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":587
 *             else:
 *                 name = <str>ptemp
 *             if name in seen:             # <<<<<<<<<<<<<<
 *                 continue
 *             seen.add(name)
 */
    __pyx_t_12 = (__Pyx_PySet_ContainsTF(__pyx_v_name, __pyx_v_seen, Py_EQ)); if (unlikely(__pyx_t_12 < 0)) __PYX_ERR(0, 587, __pyx_L1_error)
    __pyx_t_16 = (__pyx_t_12 != 0);
    if (__pyx_t_16) {

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":588
 *                 name = <str>ptemp
 *             if name in seen:
 *                 continue             # <<<<<<<<<<<<<<
 *             seen.add(name)
 *             result.append((intermediate, name, glycosylation))
 */
      goto __pyx_L3_continue;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":587
 *             else:
 *                 name = <str>ptemp
 *             if name in seen:             # <<<<<<<<<<<<<<
 *                 continue
 *             seen.add(name)
 */
    }

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":589
 *             if name in seen:
 *                 continue
 *             seen.add(name)             # <<<<<<<<<<<<<<
 *             result.append((intermediate, name, glycosylation))
 *         return result
 */
    __pyx_t_17 = PySet_Add(__pyx_v_seen, __pyx_v_name); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(0, 589, __pyx_L1_error)

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":590
 *                 continue
 *             seen.add(name)
 *             result.append((intermediate, name, glycosylation))             # <<<<<<<<<<<<<<
 *         return result
 * 
 */
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 590, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)__pyx_v_intermediate));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_intermediate));
    PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_intermediate));
    __Pyx_INCREF(__pyx_v_name);
    __Pyx_GIVEREF(__pyx_v_name);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_name);
    __Pyx_INCREF(__pyx_v_glycosylation);
    __Pyx_GIVEREF(__pyx_v_glycosylation);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_glycosylation);
    __pyx_t_17 = __Pyx_PyList_Append(__pyx_v_result, __pyx_t_2); if (unlikely(__pyx_t_17 == ((int)-1))) __PYX_ERR(0, 590, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_L3_continue:;
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":591
 *             seen.add(name)
 *             result.append((intermediate, name, glycosylation))
 *         return result             # <<<<<<<<<<<<<<
 * 
 *     def n_glycan_stub_fragments(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_result);
  __pyx_r = __pyx_v_result;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":514
 *         return core_shifts
 * 
 *     cpdef list _combinate_sites(self, list per_site_shifts, object glycan):             # <<<<<<<<<<<<<<
 *         cdef:
 *             list result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy._combinate_sites", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_result);
  __Pyx_XDECREF(__pyx_v_combos);
  __Pyx_XDECREF(__pyx_v_positions);
  __Pyx_XDECREF((PyObject *)__pyx_v_composition);
  __Pyx_XDECREF((PyObject *)__pyx_v_intermediate);
  __Pyx_XDECREF((PyObject *)__pyx_v_site);
  __Pyx_XDECREF((PyObject *)__pyx_v_aggregate_glycosylation);
  __Pyx_XDECREF(__pyx_v_glycosylation);
  __Pyx_XDECREF(__pyx_v_seen);
  __Pyx_XDECREF(__pyx_v_invalid);
  __Pyx_XDECREF(__pyx_v_name_key);
  __Pyx_XDECREF(__pyx_v_name);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_17_combinate_sites(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_16_combinate_sites[] = "StubGlycopeptideStrategy._combinate_sites(self, list per_site_shifts, glycan) -> list";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_17_combinate_sites(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_per_site_shifts = 0;
  PyObject *__pyx_v_glycan = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_combinate_sites (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_per_site_shifts,&__pyx_n_s_glycan,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_per_site_shifts)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_glycan)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_combinate_sites", 1, 2, 2, 1); __PYX_ERR(0, 514, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_combinate_sites") < 0)) __PYX_ERR(0, 514, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_per_site_shifts = ((PyObject*)values[0]);
    __pyx_v_glycan = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_combinate_sites", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 514, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy._combinate_sites", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_per_site_shifts), (&PyList_Type), 1, "per_site_shifts", 1))) __PYX_ERR(0, 514, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_16_combinate_sites(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self), __pyx_v_per_site_shifts, __pyx_v_glycan);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_16_combinate_sites(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_per_site_shifts, PyObject *__pyx_v_glycan) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_combinate_sites", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy__combinate_sites(__pyx_v_self, __pyx_v_per_site_shifts, __pyx_v_glycan, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 514, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy._combinate_sites", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_20generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":593
 *         return result
 * 
 *     def n_glycan_stub_fragments(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int core_count
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_19n_glycan_stub_fragments(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_18n_glycan_stub_fragments[] = "StubGlycopeptideStrategy.n_glycan_stub_fragments(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_19n_glycan_stub_fragments(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("n_glycan_stub_fragments (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_18n_glycan_stub_fragments(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_18n_glycan_stub_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("n_glycan_stub_fragments", 0);
  __pyx_cur_scope = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments *)__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments(__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 593, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_20generator, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_n_glycan_stub_fragments, __pyx_n_s_StubGlycopeptideStrategy_n_glyca, __pyx_n_s_glycopeptidepy__c_structure_frag); if (unlikely(!gen)) __PYX_ERR(0, 593, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.n_glycan_stub_fragments", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_20generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments *__pyx_cur_scope = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  int __pyx_t_4;
  size_t __pyx_t_5;
  struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_n_glycan_composition_fragments __pyx_t_6;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  size_t __pyx_t_9;
  size_t __pyx_t_10;
  PyObject *__pyx_t_11;
  int __pyx_t_12;
  PyObject *__pyx_t_13 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("n_glycan_stub_fragments", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L9_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 593, __pyx_L1_error)

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":609
 *             tuple result
 * 
 *         glycan = self.glycan_composition()             # <<<<<<<<<<<<<<
 *         self._use_query = self._guess_query_mode(glycan)
 *         core_count = self.count_glycosylation_type(GlycosylationType_n_linked)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.glycan_composition(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_cur_scope->__pyx_v_self), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 609, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_glycan = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":610
 * 
 *         glycan = self.glycan_composition()
 *         self._use_query = self._guess_query_mode(glycan)             # <<<<<<<<<<<<<<
 *         core_count = self.count_glycosylation_type(GlycosylationType_n_linked)
 *         per_site_shifts = []
 */
  __pyx_cur_scope->__pyx_v_self->__pyx_base._use_query = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base._guess_query_mode(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_cur_scope->__pyx_v_self), __pyx_cur_scope->__pyx_v_glycan, 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":611
 *         glycan = self.glycan_composition()
 *         self._use_query = self._guess_query_mode(glycan)
 *         core_count = self.count_glycosylation_type(GlycosylationType_n_linked)             # <<<<<<<<<<<<<<
 *         per_site_shifts = []
 *         base_composition = self.peptide_composition()
 */
  __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_n_linked;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_core_count = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.count_glycosylation_type(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_cur_scope->__pyx_v_self), __pyx_t_1, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":612
 *         self._use_query = self._guess_query_mode(glycan)
 *         core_count = self.count_glycosylation_type(GlycosylationType_n_linked)
 *         per_site_shifts = []             # <<<<<<<<<<<<<<
 *         base_composition = self.peptide_composition()
 *         base_mass = base_composition.calc_mass()
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 612, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_per_site_shifts = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":613
 *         core_count = self.count_glycosylation_type(GlycosylationType_n_linked)
 *         per_site_shifts = []
 *         base_composition = self.peptide_composition()             # <<<<<<<<<<<<<<
 *         base_mass = base_composition.calc_mass()
 *         for i in range(core_count):
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.peptide_composition(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase *)__pyx_cur_scope->__pyx_v_self), 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 613, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_base_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":614
 *         per_site_shifts = []
 *         base_composition = self.peptide_composition()
 *         base_mass = base_composition.calc_mass()             # <<<<<<<<<<<<<<
 *         for i in range(core_count):
 *             core_shifts = self.n_glycan_composition_fragments(glycan, core_count, i)
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_cur_scope->__pyx_v_base_composition->__pyx_vtab)->calc_mass(__pyx_cur_scope->__pyx_v_base_composition, 0, NULL); if (unlikely(__pyx_t_2 == ((double)-1.0))) __PYX_ERR(0, 614, __pyx_L1_error)
  __pyx_cur_scope->__pyx_v_base_mass = __pyx_t_2;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":615
 *         base_composition = self.peptide_composition()
 *         base_mass = base_composition.calc_mass()
 *         for i in range(core_count):             # <<<<<<<<<<<<<<
 *             core_shifts = self.n_glycan_composition_fragments(glycan, core_count, i)
 *             per_site_shifts.append(core_shifts)
 */
  __pyx_t_3 = __pyx_cur_scope->__pyx_v_core_count;
  __pyx_t_4 = __pyx_t_3;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_cur_scope->__pyx_v_i = __pyx_t_5;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":616
 *         base_mass = base_composition.calc_mass()
 *         for i in range(core_count):
 *             core_shifts = self.n_glycan_composition_fragments(glycan, core_count, i)             # <<<<<<<<<<<<<<
 *             per_site_shifts.append(core_shifts)
 *         combos = self._combinate_sites(per_site_shifts, glycan)
 */
    __pyx_t_6.__pyx_n = 2;
    __pyx_t_6.core_count = __pyx_cur_scope->__pyx_v_core_count;
    __pyx_t_6.iteration_count = __pyx_cur_scope->__pyx_v_i;
    __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->n_glycan_composition_fragments(__pyx_cur_scope->__pyx_v_self, __pyx_cur_scope->__pyx_v_glycan, 0, &__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 616, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_core_shifts);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_core_shifts, ((PyObject*)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":617
 *         for i in range(core_count):
 *             core_shifts = self.n_glycan_composition_fragments(glycan, core_count, i)
 *             per_site_shifts.append(core_shifts)             # <<<<<<<<<<<<<<
 *         combos = self._combinate_sites(per_site_shifts, glycan)
 *         n_combos = PyList_Size(combos)
 */
    __pyx_t_7 = __Pyx_PyList_Append(__pyx_cur_scope->__pyx_v_per_site_shifts, __pyx_cur_scope->__pyx_v_core_shifts); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 617, __pyx_L1_error)
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":618
 *             core_shifts = self.n_glycan_composition_fragments(glycan, core_count, i)
 *             per_site_shifts.append(core_shifts)
 *         combos = self._combinate_sites(per_site_shifts, glycan)             # <<<<<<<<<<<<<<
 *         n_combos = PyList_Size(combos)
 *         for i in range(n_combos):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_combinate_sites(__pyx_cur_scope->__pyx_v_self, __pyx_cur_scope->__pyx_v_per_site_shifts, __pyx_cur_scope->__pyx_v_glycan, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_combos = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":619
 *             per_site_shifts.append(core_shifts)
 *         combos = self._combinate_sites(per_site_shifts, glycan)
 *         n_combos = PyList_Size(combos)             # <<<<<<<<<<<<<<
 *         for i in range(n_combos):
 *             result = <tuple>PyList_GetItem(combos, i)
 */
  __pyx_t_8 = PyList_Size(__pyx_cur_scope->__pyx_v_combos); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1L))) __PYX_ERR(0, 619, __pyx_L1_error)
  __pyx_cur_scope->__pyx_v_n_combos = __pyx_t_8;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":620
 *         combos = self._combinate_sites(per_site_shifts, glycan)
 *         n_combos = PyList_Size(combos)
 *         for i in range(n_combos):             # <<<<<<<<<<<<<<
 *             result = <tuple>PyList_GetItem(combos, i)
 *             site = <GlycanCompositionFragment>PyTuple_GetItem(result, 0)
 */
  __pyx_t_5 = __pyx_cur_scope->__pyx_v_n_combos;
  __pyx_t_9 = __pyx_t_5;
  for (__pyx_t_10 = 0; __pyx_t_10 < __pyx_t_9; __pyx_t_10+=1) {
    __pyx_cur_scope->__pyx_v_i = __pyx_t_10;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":621
 *         n_combos = PyList_Size(combos)
 *         for i in range(n_combos):
 *             result = <tuple>PyList_GetItem(combos, i)             # <<<<<<<<<<<<<<
 *             site = <GlycanCompositionFragment>PyTuple_GetItem(result, 0)
 *             name = <str>PyTuple_GetItem(result, 1)
 */
    __pyx_t_11 = PyList_GetItem(__pyx_cur_scope->__pyx_v_combos, __pyx_cur_scope->__pyx_v_i); if (unlikely(__pyx_t_11 == ((PyObject *)NULL))) __PYX_ERR(0, 621, __pyx_L1_error)
    __pyx_t_1 = ((PyObject *)__pyx_t_11);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_result);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_result, ((PyObject*)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":622
 *         for i in range(n_combos):
 *             result = <tuple>PyList_GetItem(combos, i)
 *             site = <GlycanCompositionFragment>PyTuple_GetItem(result, 0)             # <<<<<<<<<<<<<<
 *             name = <str>PyTuple_GetItem(result, 1)
 *             glycosylation = <object>PyTuple_GetItem(result, 2)
 */
    __pyx_t_11 = PyTuple_GetItem(__pyx_cur_scope->__pyx_v_result, 0); if (unlikely(__pyx_t_11 == ((PyObject *)NULL))) __PYX_ERR(0, 622, __pyx_L1_error)
    __pyx_t_1 = ((PyObject *)__pyx_t_11);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XGOTREF(((PyObject *)__pyx_cur_scope->__pyx_v_site));
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_site, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":623
 *             result = <tuple>PyList_GetItem(combos, i)
 *             site = <GlycanCompositionFragment>PyTuple_GetItem(result, 0)
 *             name = <str>PyTuple_GetItem(result, 1)             # <<<<<<<<<<<<<<
 *             glycosylation = <object>PyTuple_GetItem(result, 2)
 *             is_glycosylated = (site.mass != 0)
 */
    __pyx_t_11 = PyTuple_GetItem(__pyx_cur_scope->__pyx_v_result, 1); if (unlikely(__pyx_t_11 == ((PyObject *)NULL))) __PYX_ERR(0, 623, __pyx_L1_error)
    __pyx_t_1 = ((PyObject *)__pyx_t_11);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_name);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_name, ((PyObject*)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":624
 *             site = <GlycanCompositionFragment>PyTuple_GetItem(result, 0)
 *             name = <str>PyTuple_GetItem(result, 1)
 *             glycosylation = <object>PyTuple_GetItem(result, 2)             # <<<<<<<<<<<<<<
 *             is_glycosylated = (site.mass != 0)
 *             composition = None
 */
    __pyx_t_11 = PyTuple_GetItem(__pyx_cur_scope->__pyx_v_result, 2); if (unlikely(__pyx_t_11 == ((PyObject *)NULL))) __PYX_ERR(0, 624, __pyx_L1_error)
    __pyx_t_1 = ((PyObject *)__pyx_t_11);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_glycosylation);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_glycosylation, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":625
 *             name = <str>PyTuple_GetItem(result, 1)
 *             glycosylation = <object>PyTuple_GetItem(result, 2)
 *             is_glycosylated = (site.mass != 0)             # <<<<<<<<<<<<<<
 *             composition = None
 *             if self.compute_compositions:
 */
    __pyx_cur_scope->__pyx_v_is_glycosylated = (__pyx_cur_scope->__pyx_v_site->mass != 0.0);

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":626
 *             glycosylation = <object>PyTuple_GetItem(result, 2)
 *             is_glycosylated = (site.mass != 0)
 *             composition = None             # <<<<<<<<<<<<<<
 *             if self.compute_compositions:
 *                 composition = base_composition.clone()
 */
    __Pyx_INCREF(Py_None);
    __Pyx_XGOTREF(((PyObject *)__pyx_cur_scope->__pyx_v_composition));
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_composition, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None));
    __Pyx_GIVEREF(Py_None);

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":627
 *             is_glycosylated = (site.mass != 0)
 *             composition = None
 *             if self.compute_compositions:             # <<<<<<<<<<<<<<
 *                 composition = base_composition.clone()
 *                 composition.add_from(site.composition)
 */
    __pyx_t_12 = (__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.compute_compositions != 0);
    if (__pyx_t_12) {

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":628
 *             composition = None
 *             if self.compute_compositions:
 *                 composition = base_composition.clone()             # <<<<<<<<<<<<<<
 *                 composition.add_from(site.composition)
 *             yield StubFragment._create(
 */
      __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_cur_scope->__pyx_v_base_composition->__pyx_vtab)->clone(__pyx_cur_scope->__pyx_v_base_composition, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 628, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(((PyObject *)__pyx_cur_scope->__pyx_v_composition));
      __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_composition, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1));
      __Pyx_GIVEREF(__pyx_t_1);
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":629
 *             if self.compute_compositions:
 *                 composition = base_composition.clone()
 *                 composition.add_from(site.composition)             # <<<<<<<<<<<<<<
 *             yield StubFragment._create(
 *                 name=name,
 */
      __pyx_t_1 = ((PyObject *)__pyx_cur_scope->__pyx_v_site->composition);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_13 = ((PyObject *)((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_cur_scope->__pyx_v_composition->__pyx_vtab)->add_from(__pyx_cur_scope->__pyx_v_composition, __pyx_t_1)); if (unlikely(!__pyx_t_13)) __PYX_ERR(0, 629, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_13);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":627
 *             is_glycosylated = (site.mass != 0)
 *             composition = None
 *             if self.compute_compositions:             # <<<<<<<<<<<<<<
 *                 composition = base_composition.clone()
 *                 composition.add_from(site.composition)
 */
    }

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":635
 *                 composition=composition,
 *                 is_glycosylated=is_glycosylated,
 *                 kind=IonSeries_stub_glycopeptide,             # <<<<<<<<<<<<<<
 *                 chemical_shift=None,
 *                 glycosylation=glycosylation,
 */
    __pyx_t_13 = ((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_IonSeries_stub_glycopeptide);
    __Pyx_INCREF(__pyx_t_13);

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":630
 *                 composition = base_composition.clone()
 *                 composition.add_from(site.composition)
 *             yield StubFragment._create(             # <<<<<<<<<<<<<<
 *                 name=name,
 *                 mass=base_mass + site.mass,
 */
    __pyx_t_1 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_StubFragment->_create(__pyx_cur_scope->__pyx_v_name, (__pyx_cur_scope->__pyx_v_base_mass + __pyx_cur_scope->__pyx_v_site->mass), ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase *)__pyx_t_13), __pyx_cur_scope->__pyx_v_composition, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase *)Py_None), __pyx_cur_scope->__pyx_v_is_glycosylated, __pyx_cur_scope->__pyx_v_glycosylation, __pyx_cur_scope->__pyx_v_site->is_extended)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 630, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_13); __pyx_t_13 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_5;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_9;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_10;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L9_resume_from_yield:;
    __pyx_t_5 = __pyx_cur_scope->__pyx_t_0;
    __pyx_t_9 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_10 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 630, __pyx_L1_error)
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":593
 *         return result
 * 
 *     def n_glycan_stub_fragments(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             int core_count
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("n_glycan_stub_fragments", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_23generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":640
 *                 is_extended=site.is_extended)
 * 
 *     def o_glycan_stub_fragments(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             list per_site_shifts
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_22o_glycan_stub_fragments(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_21o_glycan_stub_fragments[] = "StubGlycopeptideStrategy.o_glycan_stub_fragments(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_22o_glycan_stub_fragments(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("o_glycan_stub_fragments (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_21o_glycan_stub_fragments(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_21o_glycan_stub_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("o_glycan_stub_fragments", 0);
  __pyx_cur_scope = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments *)__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments(__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 640, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_23generator1, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_o_glycan_stub_fragments, __pyx_n_s_StubGlycopeptideStrategy_o_glyca, __pyx_n_s_glycopeptidepy__c_structure_frag); if (unlikely(!gen)) __PYX_ERR(0, 640, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.o_glycan_stub_fragments", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_23generator1(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments *__pyx_cur_scope = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  size_t __pyx_t_3;
  size_t __pyx_t_4;
  size_t __pyx_t_5;
  struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_o_glycan_composition_fragments __pyx_t_6;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  PyObject *__pyx_t_10;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("o_glycan_stub_fragments", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L9_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 640, __pyx_L1_error)

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":653
 *             set seen
 * 
 *         glycan = self.glycan_composition()             # <<<<<<<<<<<<<<
 *         self._use_query = self._guess_query_mode(glycan)
 *         core_count = self.count_glycosylation_type(GlycosylationType_o_linked)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.glycan_composition(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_cur_scope->__pyx_v_self), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 653, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_glycan = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":654
 * 
 *         glycan = self.glycan_composition()
 *         self._use_query = self._guess_query_mode(glycan)             # <<<<<<<<<<<<<<
 *         core_count = self.count_glycosylation_type(GlycosylationType_o_linked)
 *         per_site_shifts = []
 */
  __pyx_cur_scope->__pyx_v_self->__pyx_base._use_query = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base._guess_query_mode(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_cur_scope->__pyx_v_self), __pyx_cur_scope->__pyx_v_glycan, 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":655
 *         glycan = self.glycan_composition()
 *         self._use_query = self._guess_query_mode(glycan)
 *         core_count = self.count_glycosylation_type(GlycosylationType_o_linked)             # <<<<<<<<<<<<<<
 *         per_site_shifts = []
 * 
 */
  __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_o_linked;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_core_count = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.count_glycosylation_type(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_cur_scope->__pyx_v_self), __pyx_t_1, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":656
 *         self._use_query = self._guess_query_mode(glycan)
 *         core_count = self.count_glycosylation_type(GlycosylationType_o_linked)
 *         per_site_shifts = []             # <<<<<<<<<<<<<<
 * 
 *         base_composition = self.peptide_composition()
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 656, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_per_site_shifts = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":658
 *         per_site_shifts = []
 * 
 *         base_composition = self.peptide_composition()             # <<<<<<<<<<<<<<
 *         base_mass = base_composition.calc_mass()
 * 
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.peptide_composition(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase *)__pyx_cur_scope->__pyx_v_self), 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 658, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_base_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":659
 * 
 *         base_composition = self.peptide_composition()
 *         base_mass = base_composition.calc_mass()             # <<<<<<<<<<<<<<
 * 
 *         for i in range(core_count):
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_cur_scope->__pyx_v_base_composition->__pyx_vtab)->calc_mass(__pyx_cur_scope->__pyx_v_base_composition, 0, NULL); if (unlikely(__pyx_t_2 == ((double)-1.0))) __PYX_ERR(0, 659, __pyx_L1_error)
  __pyx_cur_scope->__pyx_v_base_mass = __pyx_t_2;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":661
 *         base_mass = base_composition.calc_mass()
 * 
 *         for i in range(core_count):             # <<<<<<<<<<<<<<
 *             core_shifts = self.o_glycan_composition_fragments(glycan, core_count, i)
 *             per_site_shifts.append(core_shifts)
 */
  __pyx_t_3 = __pyx_cur_scope->__pyx_v_core_count;
  __pyx_t_4 = __pyx_t_3;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_cur_scope->__pyx_v_i = __pyx_t_5;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":662
 * 
 *         for i in range(core_count):
 *             core_shifts = self.o_glycan_composition_fragments(glycan, core_count, i)             # <<<<<<<<<<<<<<
 *             per_site_shifts.append(core_shifts)
 *         combos = self._combinate_sites(per_site_shifts, glycan)
 */
    __pyx_t_6.__pyx_n = 2;
    __pyx_t_6.core_count = __pyx_cur_scope->__pyx_v_core_count;
    __pyx_t_6.iteration_count = __pyx_cur_scope->__pyx_v_i;
    __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->o_glycan_composition_fragments(__pyx_cur_scope->__pyx_v_self, __pyx_cur_scope->__pyx_v_glycan, 0, &__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 662, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_core_shifts);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_core_shifts, ((PyObject*)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":663
 *         for i in range(core_count):
 *             core_shifts = self.o_glycan_composition_fragments(glycan, core_count, i)
 *             per_site_shifts.append(core_shifts)             # <<<<<<<<<<<<<<
 *         combos = self._combinate_sites(per_site_shifts, glycan)
 *         n_combos = PyList_Size(combos)
 */
    __pyx_t_7 = __Pyx_PyList_Append(__pyx_cur_scope->__pyx_v_per_site_shifts, __pyx_cur_scope->__pyx_v_core_shifts); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 663, __pyx_L1_error)
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":664
 *             core_shifts = self.o_glycan_composition_fragments(glycan, core_count, i)
 *             per_site_shifts.append(core_shifts)
 *         combos = self._combinate_sites(per_site_shifts, glycan)             # <<<<<<<<<<<<<<
 *         n_combos = PyList_Size(combos)
 *         for i in range(n_combos):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_combinate_sites(__pyx_cur_scope->__pyx_v_self, __pyx_cur_scope->__pyx_v_per_site_shifts, __pyx_cur_scope->__pyx_v_glycan, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 664, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_combos = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":665
 *             per_site_shifts.append(core_shifts)
 *         combos = self._combinate_sites(per_site_shifts, glycan)
 *         n_combos = PyList_Size(combos)             # <<<<<<<<<<<<<<
 *         for i in range(n_combos):
 *             result = <tuple>PyList_GetItem(combos, i)
 */
  __pyx_t_8 = PyList_Size(__pyx_cur_scope->__pyx_v_combos); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1L))) __PYX_ERR(0, 665, __pyx_L1_error)
  __pyx_cur_scope->__pyx_v_n_combos = __pyx_t_8;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":666
 *         combos = self._combinate_sites(per_site_shifts, glycan)
 *         n_combos = PyList_Size(combos)
 *         for i in range(n_combos):             # <<<<<<<<<<<<<<
 *             result = <tuple>PyList_GetItem(combos, i)
 *             site = <GlycanCompositionFragment>PyTuple_GetItem(result, 0)
 */
  __pyx_t_8 = __pyx_cur_scope->__pyx_v_n_combos;
  __pyx_t_9 = __pyx_t_8;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_9; __pyx_t_3+=1) {
    __pyx_cur_scope->__pyx_v_i = __pyx_t_3;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":667
 *         n_combos = PyList_Size(combos)
 *         for i in range(n_combos):
 *             result = <tuple>PyList_GetItem(combos, i)             # <<<<<<<<<<<<<<
 *             site = <GlycanCompositionFragment>PyTuple_GetItem(result, 0)
 *             name = <str>PyTuple_GetItem(result, 1)
 */
    __pyx_t_10 = PyList_GetItem(__pyx_cur_scope->__pyx_v_combos, __pyx_cur_scope->__pyx_v_i); if (unlikely(__pyx_t_10 == ((PyObject *)NULL))) __PYX_ERR(0, 667, __pyx_L1_error)
    __pyx_t_1 = ((PyObject *)__pyx_t_10);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_result);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_result, ((PyObject*)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":668
 *         for i in range(n_combos):
 *             result = <tuple>PyList_GetItem(combos, i)
 *             site = <GlycanCompositionFragment>PyTuple_GetItem(result, 0)             # <<<<<<<<<<<<<<
 *             name = <str>PyTuple_GetItem(result, 1)
 *             glycosylation = <object>PyTuple_GetItem(result, 2)
 */
    __pyx_t_10 = PyTuple_GetItem(__pyx_cur_scope->__pyx_v_result, 0); if (unlikely(__pyx_t_10 == ((PyObject *)NULL))) __PYX_ERR(0, 668, __pyx_L1_error)
    __pyx_t_1 = ((PyObject *)__pyx_t_10);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XGOTREF(((PyObject *)__pyx_cur_scope->__pyx_v_site));
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_site, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":669
 *             result = <tuple>PyList_GetItem(combos, i)
 *             site = <GlycanCompositionFragment>PyTuple_GetItem(result, 0)
 *             name = <str>PyTuple_GetItem(result, 1)             # <<<<<<<<<<<<<<
 *             glycosylation = <object>PyTuple_GetItem(result, 2)
 *             is_glycosylated = (site.mass != 0)
 */
    __pyx_t_10 = PyTuple_GetItem(__pyx_cur_scope->__pyx_v_result, 1); if (unlikely(__pyx_t_10 == ((PyObject *)NULL))) __PYX_ERR(0, 669, __pyx_L1_error)
    __pyx_t_1 = ((PyObject *)__pyx_t_10);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_name);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_name, ((PyObject*)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":670
 *             site = <GlycanCompositionFragment>PyTuple_GetItem(result, 0)
 *             name = <str>PyTuple_GetItem(result, 1)
 *             glycosylation = <object>PyTuple_GetItem(result, 2)             # <<<<<<<<<<<<<<
 *             is_glycosylated = (site.mass != 0)
 *             composition = None
 */
    __pyx_t_10 = PyTuple_GetItem(__pyx_cur_scope->__pyx_v_result, 2); if (unlikely(__pyx_t_10 == ((PyObject *)NULL))) __PYX_ERR(0, 670, __pyx_L1_error)
    __pyx_t_1 = ((PyObject *)__pyx_t_10);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_glycosylation);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_glycosylation, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":671
 *             name = <str>PyTuple_GetItem(result, 1)
 *             glycosylation = <object>PyTuple_GetItem(result, 2)
 *             is_glycosylated = (site.mass != 0)             # <<<<<<<<<<<<<<
 *             composition = None
 *             if self.compute_compositions:
 */
    __pyx_cur_scope->__pyx_v_is_glycosylated = (__pyx_cur_scope->__pyx_v_site->mass != 0.0);

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":672
 *             glycosylation = <object>PyTuple_GetItem(result, 2)
 *             is_glycosylated = (site.mass != 0)
 *             composition = None             # <<<<<<<<<<<<<<
 *             if self.compute_compositions:
 *                 composition = base_composition.clone()
 */
    __Pyx_INCREF(Py_None);
    __Pyx_XGOTREF(((PyObject *)__pyx_cur_scope->__pyx_v_composition));
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_composition, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None));
    __Pyx_GIVEREF(Py_None);

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":673
 *             is_glycosylated = (site.mass != 0)
 *             composition = None
 *             if self.compute_compositions:             # <<<<<<<<<<<<<<
 *                 composition = base_composition.clone()
 *                 composition.add_from(site.composition)
 */
    __pyx_t_11 = (__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.compute_compositions != 0);
    if (__pyx_t_11) {

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":674
 *             composition = None
 *             if self.compute_compositions:
 *                 composition = base_composition.clone()             # <<<<<<<<<<<<<<
 *                 composition.add_from(site.composition)
 *             yield StubFragment._create(
 */
      __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_cur_scope->__pyx_v_base_composition->__pyx_vtab)->clone(__pyx_cur_scope->__pyx_v_base_composition, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 674, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(((PyObject *)__pyx_cur_scope->__pyx_v_composition));
      __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_composition, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1));
      __Pyx_GIVEREF(__pyx_t_1);
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":675
 *             if self.compute_compositions:
 *                 composition = base_composition.clone()
 *                 composition.add_from(site.composition)             # <<<<<<<<<<<<<<
 *             yield StubFragment._create(
 *                 name=name,
 */
      __pyx_t_1 = ((PyObject *)__pyx_cur_scope->__pyx_v_site->composition);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_12 = ((PyObject *)((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_cur_scope->__pyx_v_composition->__pyx_vtab)->add_from(__pyx_cur_scope->__pyx_v_composition, __pyx_t_1)); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 675, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":673
 *             is_glycosylated = (site.mass != 0)
 *             composition = None
 *             if self.compute_compositions:             # <<<<<<<<<<<<<<
 *                 composition = base_composition.clone()
 *                 composition.add_from(site.composition)
 */
    }

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":681
 *                 composition=composition,
 *                 is_glycosylated=is_glycosylated,
 *                 kind=IonSeries_stub_glycopeptide,             # <<<<<<<<<<<<<<
 *                 chemical_shift=None,
 *                 glycosylation=glycosylation,
 */
    __pyx_t_12 = ((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_IonSeries_stub_glycopeptide);
    __Pyx_INCREF(__pyx_t_12);

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":676
 *                 composition = base_composition.clone()
 *                 composition.add_from(site.composition)
 *             yield StubFragment._create(             # <<<<<<<<<<<<<<
 *                 name=name,
 *                 mass=base_mass + site.mass,
 */
    __pyx_t_1 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_StubFragment->_create(__pyx_cur_scope->__pyx_v_name, (__pyx_cur_scope->__pyx_v_base_mass + __pyx_cur_scope->__pyx_v_site->mass), ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase *)__pyx_t_12), __pyx_cur_scope->__pyx_v_composition, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase *)Py_None), __pyx_cur_scope->__pyx_v_is_glycosylated, __pyx_cur_scope->__pyx_v_glycosylation, __pyx_cur_scope->__pyx_v_site->is_extended)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 676, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_3;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_8;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_9;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L9_resume_from_yield:;
    __pyx_t_3 = __pyx_cur_scope->__pyx_t_0;
    __pyx_t_8 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_9 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 676, __pyx_L1_error)
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":640
 *                 is_extended=site.is_extended)
 * 
 *     def o_glycan_stub_fragments(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             list per_site_shifts
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("o_glycan_stub_fragments", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":686
 *                 is_extended=site.is_extended)
 * 
 *     cpdef list o_glycan_composition_fragments(self, glycan, long core_count=1, long iteration_count=0):             # <<<<<<<<<<<<<<
 *         cdef:
 *             list core_shifts
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_25o_glycan_composition_fragments(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_o_glycan_composition_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_glycan, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_o_glycan_composition_fragments *__pyx_optional_args) {
  long __pyx_v_core_count = ((long)1);
  long __pyx_v_iteration_count = ((long)0);
  PyObject *__pyx_v_core_shifts = 0;
  long __pyx_v_fucose_count;
  long __pyx_v_hexnac_in_aggregate;
  long __pyx_v_hexose_in_aggregate;
  long __pyx_v_hexnac_count;
  long __pyx_v_hexose_count;
  long __pyx_v_extra_hexnac_count;
  long __pyx_v_extra_hexose_count;
  struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *__pyx_v_key_ct = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_shift = NULL;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_fucosylated = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  long __pyx_t_10;
  int __pyx_t_11;
  long __pyx_t_12;
  int __pyx_t_13;
  long __pyx_t_14;
  long __pyx_t_15;
  long __pyx_t_16;
  long __pyx_t_17;
  long __pyx_t_18;
  long __pyx_t_19;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("o_glycan_composition_fragments", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_core_count = __pyx_optional_args->core_count;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_iteration_count = __pyx_optional_args->iteration_count;
      }
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_o_glycan_composition_fragments); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 686, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_25o_glycan_composition_fragments)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_v_core_count); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 686, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = __Pyx_PyInt_From_long(__pyx_v_iteration_count); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 686, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_5 = __pyx_t_1; __pyx_t_6 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_v_glycan, __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 686, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_v_glycan, __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 686, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 686, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          if (__pyx_t_6) {
            __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
          }
          __Pyx_INCREF(__pyx_v_glycan);
          __Pyx_GIVEREF(__pyx_v_glycan);
          PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_v_glycan);
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_t_4);
          __pyx_t_3 = 0;
          __pyx_t_4 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 686, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 686, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":694
 *             CountTable key_ct
 * 
 *         if self._use_query:             # <<<<<<<<<<<<<<
 *             fucose_count = glycan.query('Fuc') + glycan.query('dHex')
 *             hexnac_in_aggregate = glycan.query('HexNAc')
 */
  __pyx_t_9 = (__pyx_v_self->__pyx_base._use_query != 0);
  if (__pyx_t_9) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":695
 * 
 *         if self._use_query:
 *             fucose_count = glycan.query('Fuc') + glycan.query('dHex')             # <<<<<<<<<<<<<<
 *             hexnac_in_aggregate = glycan.query('HexNAc')
 *             hexose_in_aggregate = glycan.query('Hex')
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_glycan, __pyx_n_s_query); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 695, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_n_s_Fuc) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_n_s_Fuc);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 695, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_glycan, __pyx_n_s_query); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 695, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_5))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_5);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_5, function);
      }
    }
    __pyx_t_2 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_5, __pyx_t_8, __pyx_n_s_dHex) : __Pyx_PyObject_CallOneArg(__pyx_t_5, __pyx_n_s_dHex);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 695, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_5 = PyNumber_Add(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 695, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_long(__pyx_t_5); if (unlikely((__pyx_t_10 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 695, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_fucose_count = __pyx_t_10;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":696
 *         if self._use_query:
 *             fucose_count = glycan.query('Fuc') + glycan.query('dHex')
 *             hexnac_in_aggregate = glycan.query('HexNAc')             # <<<<<<<<<<<<<<
 *             hexose_in_aggregate = glycan.query('Hex')
 *         else:
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_glycan, __pyx_n_s_query); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 696, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_n_s_HexNAc) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_n_s_HexNAc);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 696, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_long(__pyx_t_5); if (unlikely((__pyx_t_10 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 696, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_hexnac_in_aggregate = __pyx_t_10;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":697
 *             fucose_count = glycan.query('Fuc') + glycan.query('dHex')
 *             hexnac_in_aggregate = glycan.query('HexNAc')
 *             hexose_in_aggregate = glycan.query('Hex')             # <<<<<<<<<<<<<<
 *         else:
 *             fucose_count = glycan[_FUC] + glycan[_DHEX]
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_glycan, __pyx_n_s_query); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 697, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_1, __pyx_n_s_Hex) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_n_s_Hex);
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 697, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_long(__pyx_t_5); if (unlikely((__pyx_t_10 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 697, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_hexose_in_aggregate = __pyx_t_10;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":694
 *             CountTable key_ct
 * 
 *         if self._use_query:             # <<<<<<<<<<<<<<
 *             fucose_count = glycan.query('Fuc') + glycan.query('dHex')
 *             hexnac_in_aggregate = glycan.query('HexNAc')
 */
    goto __pyx_L3;
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":699
 *             hexose_in_aggregate = glycan.query('Hex')
 *         else:
 *             fucose_count = glycan[_FUC] + glycan[_DHEX]             # <<<<<<<<<<<<<<
 *             hexnac_in_aggregate = glycan[_HEXNAC]
 *             hexose_in_aggregate = glycan[_HEX]
 */
  /*else*/ {
    __pyx_t_5 = __Pyx_PyObject_GetItem(__pyx_v_glycan, __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 699, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_2 = __Pyx_PyObject_GetItem(__pyx_v_glycan, __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__DHEX); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 699, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_1 = PyNumber_Add(__pyx_t_5, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 699, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_10 = __Pyx_PyInt_As_long(__pyx_t_1); if (unlikely((__pyx_t_10 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 699, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_fucose_count = __pyx_t_10;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":700
 *         else:
 *             fucose_count = glycan[_FUC] + glycan[_DHEX]
 *             hexnac_in_aggregate = glycan[_HEXNAC]             # <<<<<<<<<<<<<<
 *             hexose_in_aggregate = glycan[_HEX]
 *         core_shifts = []
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_glycan, __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 700, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = __Pyx_PyInt_As_long(__pyx_t_1); if (unlikely((__pyx_t_10 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 700, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_hexnac_in_aggregate = __pyx_t_10;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":701
 *             fucose_count = glycan[_FUC] + glycan[_DHEX]
 *             hexnac_in_aggregate = glycan[_HEXNAC]
 *             hexose_in_aggregate = glycan[_HEX]             # <<<<<<<<<<<<<<
 *         core_shifts = []
 *         for hexnac_count in range(3):
 */
    __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_glycan, __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 701, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_10 = __Pyx_PyInt_As_long(__pyx_t_1); if (unlikely((__pyx_t_10 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 701, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_hexose_in_aggregate = __pyx_t_10;
  }
  __pyx_L3:;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":702
 *             hexnac_in_aggregate = glycan[_HEXNAC]
 *             hexose_in_aggregate = glycan[_HEX]
 *         core_shifts = []             # <<<<<<<<<<<<<<
 *         for hexnac_count in range(3):
 *             if hexnac_in_aggregate < hexnac_count:
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 702, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_core_shifts = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":703
 *             hexose_in_aggregate = glycan[_HEX]
 *         core_shifts = []
 *         for hexnac_count in range(3):             # <<<<<<<<<<<<<<
 *             if hexnac_in_aggregate < hexnac_count:
 *                 continue
 */
  for (__pyx_t_10 = 0; __pyx_t_10 < 3; __pyx_t_10+=1) {
    __pyx_v_hexnac_count = __pyx_t_10;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":704
 *         core_shifts = []
 *         for hexnac_count in range(3):
 *             if hexnac_in_aggregate < hexnac_count:             # <<<<<<<<<<<<<<
 *                 continue
 *             if hexnac_count == 0:
 */
    __pyx_t_9 = ((__pyx_v_hexnac_in_aggregate < __pyx_v_hexnac_count) != 0);
    if (__pyx_t_9) {

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":705
 *         for hexnac_count in range(3):
 *             if hexnac_in_aggregate < hexnac_count:
 *                 continue             # <<<<<<<<<<<<<<
 *             if hexnac_count == 0:
 *                 shift = GlycanCompositionFragment._create(
 */
      goto __pyx_L4_continue;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":704
 *         core_shifts = []
 *         for hexnac_count in range(3):
 *             if hexnac_in_aggregate < hexnac_count:             # <<<<<<<<<<<<<<
 *                 continue
 *             if hexnac_count == 0:
 */
    }

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":706
 *             if hexnac_in_aggregate < hexnac_count:
 *                 continue
 *             if hexnac_count == 0:             # <<<<<<<<<<<<<<
 *                 shift = GlycanCompositionFragment._create(
 *                     0,
 */
    __pyx_t_9 = ((__pyx_v_hexnac_count == 0) != 0);
    if (__pyx_t_9) {

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":709
 *                 shift = GlycanCompositionFragment._create(
 *                     0,
 *                     CComposition(),             # <<<<<<<<<<<<<<
 *                     CountTable._create(),
 *                     False
 */
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_5glypy_11composition_12ccomposition_CComposition)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 709, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":710
 *                     0,
 *                     CComposition(),
 *                     CountTable._create(),             # <<<<<<<<<<<<<<
 *                     False
 *                 )
 */
      __pyx_t_2 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable->_create()); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 710, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":707
 *                 continue
 *             if hexnac_count == 0:
 *                 shift = GlycanCompositionFragment._create(             # <<<<<<<<<<<<<<
 *                     0,
 *                     CComposition(),
 */
      __pyx_t_5 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create(0.0, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1), ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_2), 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 707, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF_SET(__pyx_v_shift, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_5));
      __pyx_t_5 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":713
 *                     False
 *                 )
 *                 core_shifts.append(shift)             # <<<<<<<<<<<<<<
 *             elif hexnac_count >= 1:
 *                 key_ct = CountTable._create()
 */
      __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_shift)); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 713, __pyx_L1_error)

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":706
 *             if hexnac_in_aggregate < hexnac_count:
 *                 continue
 *             if hexnac_count == 0:             # <<<<<<<<<<<<<<
 *                 shift = GlycanCompositionFragment._create(
 *                     0,
 */
      goto __pyx_L7;
    }

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":714
 *                 )
 *                 core_shifts.append(shift)
 *             elif hexnac_count >= 1:             # <<<<<<<<<<<<<<
 *                 key_ct = CountTable._create()
 *                 key_ct.setitem(_HEXNAC, hexnac_count)
 */
    __pyx_t_9 = ((__pyx_v_hexnac_count >= 1) != 0);
    if (__pyx_t_9) {

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":715
 *                 core_shifts.append(shift)
 *             elif hexnac_count >= 1:
 *                 key_ct = CountTable._create()             # <<<<<<<<<<<<<<
 *                 key_ct.setitem(_HEXNAC, hexnac_count)
 *                 shift = GlycanCompositionFragment._create(
 */
      __pyx_t_5 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable->_create()); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 715, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_XDECREF_SET(__pyx_v_key_ct, ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_5));
      __pyx_t_5 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":716
 *             elif hexnac_count >= 1:
 *                 key_ct = CountTable._create()
 *                 key_ct.setitem(_HEXNAC, hexnac_count)             # <<<<<<<<<<<<<<
 *                 shift = GlycanCompositionFragment._create(
 *                     (hexnac_count * _HEXNAC_mass),
 */
      __pyx_t_5 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC;
      __Pyx_INCREF(__pyx_t_5);
      __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_5, __pyx_v_hexnac_count); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 716, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":719
 *                 shift = GlycanCompositionFragment._create(
 *                     (hexnac_count * _HEXNAC_mass),
 *                     hexnac_hex_composition(hexnac_count, 0),             # <<<<<<<<<<<<<<
 *                     key_ct,
 *                     False
 */
      __pyx_t_5 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_hexnac_hex_composition(__pyx_v_hexnac_count, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 719, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":717
 *                 key_ct = CountTable._create()
 *                 key_ct.setitem(_HEXNAC, hexnac_count)
 *                 shift = GlycanCompositionFragment._create(             # <<<<<<<<<<<<<<
 *                     (hexnac_count * _HEXNAC_mass),
 *                     hexnac_hex_composition(hexnac_count, 0),
 */
      __pyx_t_2 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create((__pyx_v_hexnac_count * __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC_mass), ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_5), __pyx_v_key_ct, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 717, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_XDECREF_SET(__pyx_v_shift, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":723
 *                     False
 *                 )
 *                 core_shifts.append(shift)             # <<<<<<<<<<<<<<
 *                 if iteration_count < fucose_count:
 *                     fucosylated = self.fucosylate_increment(shift)
 */
      __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_shift)); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 723, __pyx_L1_error)

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":724
 *                 )
 *                 core_shifts.append(shift)
 *                 if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                     fucosylated = self.fucosylate_increment(shift)
 *                     core_shifts.append(fucosylated)
 */
      __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_fucose_count) != 0);
      if (__pyx_t_9) {

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":725
 *                 core_shifts.append(shift)
 *                 if iteration_count < fucose_count:
 *                     fucosylated = self.fucosylate_increment(shift)             # <<<<<<<<<<<<<<
 *                     core_shifts.append(fucosylated)
 *                 for hexose_count in range(0, 2):
 */
        __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->fucosylate_increment(__pyx_v_self, __pyx_v_shift, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 725, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_fucosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":726
 *                 if iteration_count < fucose_count:
 *                     fucosylated = self.fucosylate_increment(shift)
 *                     core_shifts.append(fucosylated)             # <<<<<<<<<<<<<<
 *                 for hexose_count in range(0, 2):
 *                     if hexose_in_aggregate < hexose_count:
 */
        __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_fucosylated)); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 726, __pyx_L1_error)

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":724
 *                 )
 *                 core_shifts.append(shift)
 *                 if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                     fucosylated = self.fucosylate_increment(shift)
 *                     core_shifts.append(fucosylated)
 */
      }

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":727
 *                     fucosylated = self.fucosylate_increment(shift)
 *                     core_shifts.append(fucosylated)
 *                 for hexose_count in range(0, 2):             # <<<<<<<<<<<<<<
 *                     if hexose_in_aggregate < hexose_count:
 *                         continue
 */
      for (__pyx_t_12 = 0; __pyx_t_12 < 2; __pyx_t_12+=1) {
        __pyx_v_hexose_count = __pyx_t_12;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":728
 *                     core_shifts.append(fucosylated)
 *                 for hexose_count in range(0, 2):
 *                     if hexose_in_aggregate < hexose_count:             # <<<<<<<<<<<<<<
 *                         continue
 *                     if hexose_count > 0:
 */
        __pyx_t_9 = ((__pyx_v_hexose_in_aggregate < __pyx_v_hexose_count) != 0);
        if (__pyx_t_9) {

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":729
 *                 for hexose_count in range(0, 2):
 *                     if hexose_in_aggregate < hexose_count:
 *                         continue             # <<<<<<<<<<<<<<
 *                     if hexose_count > 0:
 *                         key_ct = CountTable._create()
 */
          goto __pyx_L9_continue;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":728
 *                     core_shifts.append(fucosylated)
 *                 for hexose_count in range(0, 2):
 *                     if hexose_in_aggregate < hexose_count:             # <<<<<<<<<<<<<<
 *                         continue
 *                     if hexose_count > 0:
 */
        }

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":730
 *                     if hexose_in_aggregate < hexose_count:
 *                         continue
 *                     if hexose_count > 0:             # <<<<<<<<<<<<<<
 *                         key_ct = CountTable._create()
 *                         key_ct.setitem(_HEXNAC, hexnac_count)
 */
        __pyx_t_9 = ((__pyx_v_hexose_count > 0) != 0);
        if (__pyx_t_9) {

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":731
 *                         continue
 *                     if hexose_count > 0:
 *                         key_ct = CountTable._create()             # <<<<<<<<<<<<<<
 *                         key_ct.setitem(_HEXNAC, hexnac_count)
 *                         key_ct.setitem(_HEX, hexose_count)
 */
          __pyx_t_2 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable->_create()); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 731, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF_SET(__pyx_v_key_ct, ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_2));
          __pyx_t_2 = 0;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":732
 *                     if hexose_count > 0:
 *                         key_ct = CountTable._create()
 *                         key_ct.setitem(_HEXNAC, hexnac_count)             # <<<<<<<<<<<<<<
 *                         key_ct.setitem(_HEX, hexose_count)
 *                         shift = GlycanCompositionFragment._create(
 */
          __pyx_t_2 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC;
          __Pyx_INCREF(__pyx_t_2);
          __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_2, __pyx_v_hexnac_count); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 732, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":733
 *                         key_ct = CountTable._create()
 *                         key_ct.setitem(_HEXNAC, hexnac_count)
 *                         key_ct.setitem(_HEX, hexose_count)             # <<<<<<<<<<<<<<
 *                         shift = GlycanCompositionFragment._create(
 *                             (
 */
          __pyx_t_2 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX;
          __Pyx_INCREF(__pyx_t_2);
          __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_2, __pyx_v_hexose_count); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 733, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":738
 *                                 (hexnac_count) * _HEXNAC_mass) + (
 *                                 (hexose_count) * _HEX_mass),
 *                             hexnac_hex_composition(hexnac_count, hexose_count),             # <<<<<<<<<<<<<<
 *                             key_ct,
 *                             False
 */
          __pyx_t_2 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_hexnac_hex_composition(__pyx_v_hexnac_count, __pyx_v_hexose_count)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 738, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":734
 *                         key_ct.setitem(_HEXNAC, hexnac_count)
 *                         key_ct.setitem(_HEX, hexose_count)
 *                         shift = GlycanCompositionFragment._create(             # <<<<<<<<<<<<<<
 *                             (
 *                                 (hexnac_count) * _HEXNAC_mass) + (
 */
          __pyx_t_5 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create(((__pyx_v_hexnac_count * __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC_mass) + (__pyx_v_hexose_count * __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX_mass)), ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_2), __pyx_v_key_ct, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 734, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF_SET(__pyx_v_shift, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_5));
          __pyx_t_5 = 0;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":742
 *                             False
 *                         )
 *                         core_shifts.append(shift)             # <<<<<<<<<<<<<<
 *                         if iteration_count < fucose_count:
 *                             fucosylated = self.fucosylate_increment(shift)
 */
          __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_shift)); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 742, __pyx_L1_error)

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":743
 *                         )
 *                         core_shifts.append(shift)
 *                         if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                             fucosylated = self.fucosylate_increment(shift)
 *                             core_shifts.append(fucosylated)
 */
          __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_fucose_count) != 0);
          if (__pyx_t_9) {

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":744
 *                         core_shifts.append(shift)
 *                         if iteration_count < fucose_count:
 *                             fucosylated = self.fucosylate_increment(shift)             # <<<<<<<<<<<<<<
 *                             core_shifts.append(fucosylated)
 *                     # After the core motif has been exhausted, speculatively add
 */
            __pyx_t_5 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->fucosylate_increment(__pyx_v_self, __pyx_v_shift, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 744, __pyx_L1_error)
            __Pyx_GOTREF(__pyx_t_5);
            __Pyx_XDECREF_SET(__pyx_v_fucosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_5));
            __pyx_t_5 = 0;

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":745
 *                         if iteration_count < fucose_count:
 *                             fucosylated = self.fucosylate_increment(shift)
 *                             core_shifts.append(fucosylated)             # <<<<<<<<<<<<<<
 *                     # After the core motif has been exhausted, speculatively add
 *                     # on the remaining core monosaccharides sequentially until
 */
            __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_fucosylated)); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 745, __pyx_L1_error)

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":743
 *                         )
 *                         core_shifts.append(shift)
 *                         if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                             fucosylated = self.fucosylate_increment(shift)
 *                             core_shifts.append(fucosylated)
 */
          }

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":730
 *                     if hexose_in_aggregate < hexose_count:
 *                         continue
 *                     if hexose_count > 0:             # <<<<<<<<<<<<<<
 *                         key_ct = CountTable._create()
 *                         key_ct.setitem(_HEXNAC, hexnac_count)
 */
        }

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":749
 *                     # on the remaining core monosaccharides sequentially until
 *                     # exhausted.
 *                     if self.extended and hexnac_in_aggregate - hexnac_count >= 0:             # <<<<<<<<<<<<<<
 *                         for extra_hexnac_count in range(hexnac_in_aggregate - hexnac_count + 1):
 *                             if extra_hexnac_count:
 */
        __pyx_t_13 = (__pyx_v_self->extended != 0);
        if (__pyx_t_13) {
        } else {
          __pyx_t_9 = __pyx_t_13;
          goto __pyx_L15_bool_binop_done;
        }
        __pyx_t_13 = (((__pyx_v_hexnac_in_aggregate - __pyx_v_hexnac_count) >= 0) != 0);
        __pyx_t_9 = __pyx_t_13;
        __pyx_L15_bool_binop_done:;
        if (__pyx_t_9) {

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":750
 *                     # exhausted.
 *                     if self.extended and hexnac_in_aggregate - hexnac_count >= 0:
 *                         for extra_hexnac_count in range(hexnac_in_aggregate - hexnac_count + 1):             # <<<<<<<<<<<<<<
 *                             if extra_hexnac_count:
 *                                 key_ct = CountTable._create()
 */
          __pyx_t_14 = ((__pyx_v_hexnac_in_aggregate - __pyx_v_hexnac_count) + 1);
          __pyx_t_15 = __pyx_t_14;
          for (__pyx_t_16 = 0; __pyx_t_16 < __pyx_t_15; __pyx_t_16+=1) {
            __pyx_v_extra_hexnac_count = __pyx_t_16;

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":751
 *                     if self.extended and hexnac_in_aggregate - hexnac_count >= 0:
 *                         for extra_hexnac_count in range(hexnac_in_aggregate - hexnac_count + 1):
 *                             if extra_hexnac_count:             # <<<<<<<<<<<<<<
 *                                 key_ct = CountTable._create()
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 */
            __pyx_t_9 = (__pyx_v_extra_hexnac_count != 0);
            if (__pyx_t_9) {

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":752
 *                         for extra_hexnac_count in range(hexnac_in_aggregate - hexnac_count + 1):
 *                             if extra_hexnac_count:
 *                                 key_ct = CountTable._create()             # <<<<<<<<<<<<<<
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 *                                 if hexose_count > 0:
 */
              __pyx_t_5 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable->_create()); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 752, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_5);
              __Pyx_DECREF_SET(__pyx_v_key_ct, ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_5));
              __pyx_t_5 = 0;

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":753
 *                             if extra_hexnac_count:
 *                                 key_ct = CountTable._create()
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)             # <<<<<<<<<<<<<<
 *                                 if hexose_count > 0:
 *                                     key_ct.setitem(_HEX, hexose_count)
 */
              __pyx_t_5 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC;
              __Pyx_INCREF(__pyx_t_5);
              __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_5, (__pyx_v_hexnac_count + __pyx_v_extra_hexnac_count)); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 753, __pyx_L1_error)
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":754
 *                                 key_ct = CountTable._create()
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 *                                 if hexose_count > 0:             # <<<<<<<<<<<<<<
 *                                     key_ct.setitem(_HEX, hexose_count)
 *                                 shift = GlycanCompositionFragment._create(
 */
              __pyx_t_9 = ((__pyx_v_hexose_count > 0) != 0);
              if (__pyx_t_9) {

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":755
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 *                                 if hexose_count > 0:
 *                                     key_ct.setitem(_HEX, hexose_count)             # <<<<<<<<<<<<<<
 *                                 shift = GlycanCompositionFragment._create(
 *                                     (
 */
                __pyx_t_5 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX;
                __Pyx_INCREF(__pyx_t_5);
                __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_5, __pyx_v_hexose_count); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 755, __pyx_L1_error)
                __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":754
 *                                 key_ct = CountTable._create()
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 *                                 if hexose_count > 0:             # <<<<<<<<<<<<<<
 *                                     key_ct.setitem(_HEX, hexose_count)
 *                                 shift = GlycanCompositionFragment._create(
 */
              }

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":760
 *                                         (hexnac_count + extra_hexnac_count) * _HEXNAC_mass) + (
 *                                         (hexose_count) * _HEX_mass),
 *                                     hexnac_hex_composition(hexnac_count + extra_hexnac_count, hexose_count),             # <<<<<<<<<<<<<<
 *                                     key_ct,
 *                                     True,
 */
              __pyx_t_5 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_hexnac_hex_composition((__pyx_v_hexnac_count + __pyx_v_extra_hexnac_count), __pyx_v_hexose_count)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 760, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_5);

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":756
 *                                 if hexose_count > 0:
 *                                     key_ct.setitem(_HEX, hexose_count)
 *                                 shift = GlycanCompositionFragment._create(             # <<<<<<<<<<<<<<
 *                                     (
 *                                         (hexnac_count + extra_hexnac_count) * _HEXNAC_mass) + (
 */
              __pyx_t_2 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create((((__pyx_v_hexnac_count + __pyx_v_extra_hexnac_count) * __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC_mass) + (__pyx_v_hexose_count * __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX_mass)), ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_5), __pyx_v_key_ct, 1)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 756, __pyx_L1_error)
              __Pyx_GOTREF(__pyx_t_2);
              __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
              __Pyx_DECREF_SET(__pyx_v_shift, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2));
              __pyx_t_2 = 0;

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":764
 *                                     True,
 *                                 )
 *                                 core_shifts.append(shift)             # <<<<<<<<<<<<<<
 *                                 if iteration_count < fucose_count:
 *                                     fucosylated = self.fucosylate_increment(shift)
 */
              __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_shift)); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 764, __pyx_L1_error)

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":765
 *                                 )
 *                                 core_shifts.append(shift)
 *                                 if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                                     fucosylated = self.fucosylate_increment(shift)
 *                                     core_shifts.append(fucosylated)
 */
              __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_fucose_count) != 0);
              if (__pyx_t_9) {

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":766
 *                                 core_shifts.append(shift)
 *                                 if iteration_count < fucose_count:
 *                                     fucosylated = self.fucosylate_increment(shift)             # <<<<<<<<<<<<<<
 *                                     core_shifts.append(fucosylated)
 *                             if hexose_in_aggregate > hexose_count and hexose_count > 0:
 */
                __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->fucosylate_increment(__pyx_v_self, __pyx_v_shift, 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 766, __pyx_L1_error)
                __Pyx_GOTREF(__pyx_t_2);
                __Pyx_XDECREF_SET(__pyx_v_fucosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_2));
                __pyx_t_2 = 0;

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":767
 *                                 if iteration_count < fucose_count:
 *                                     fucosylated = self.fucosylate_increment(shift)
 *                                     core_shifts.append(fucosylated)             # <<<<<<<<<<<<<<
 *                             if hexose_in_aggregate > hexose_count and hexose_count > 0:
 *                                 for extra_hexose_count in range(hexose_in_aggregate - hexose_count):
 */
                __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_fucosylated)); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 767, __pyx_L1_error)

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":765
 *                                 )
 *                                 core_shifts.append(shift)
 *                                 if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                                     fucosylated = self.fucosylate_increment(shift)
 *                                     core_shifts.append(fucosylated)
 */
              }

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":751
 *                     if self.extended and hexnac_in_aggregate - hexnac_count >= 0:
 *                         for extra_hexnac_count in range(hexnac_in_aggregate - hexnac_count + 1):
 *                             if extra_hexnac_count:             # <<<<<<<<<<<<<<
 *                                 key_ct = CountTable._create()
 *                                 key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 */
            }

            /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":768
 *                                     fucosylated = self.fucosylate_increment(shift)
 *                                     core_shifts.append(fucosylated)
 *                             if hexose_in_aggregate > hexose_count and hexose_count > 0:             # <<<<<<<<<<<<<<
 *                                 for extra_hexose_count in range(hexose_in_aggregate - hexose_count):
 *                                     if extra_hexose_count > 0 and extra_hexose_count + hexose_count > 0:
 */
            __pyx_t_13 = ((__pyx_v_hexose_in_aggregate > __pyx_v_hexose_count) != 0);
            if (__pyx_t_13) {
            } else {
              __pyx_t_9 = __pyx_t_13;
              goto __pyx_L23_bool_binop_done;
            }
            __pyx_t_13 = ((__pyx_v_hexose_count > 0) != 0);
            __pyx_t_9 = __pyx_t_13;
            __pyx_L23_bool_binop_done:;
            if (__pyx_t_9) {

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":769
 *                                     core_shifts.append(fucosylated)
 *                             if hexose_in_aggregate > hexose_count and hexose_count > 0:
 *                                 for extra_hexose_count in range(hexose_in_aggregate - hexose_count):             # <<<<<<<<<<<<<<
 *                                     if extra_hexose_count > 0 and extra_hexose_count + hexose_count > 0:
 *                                         key_ct = CountTable._create()
 */
              __pyx_t_17 = (__pyx_v_hexose_in_aggregate - __pyx_v_hexose_count);
              __pyx_t_18 = __pyx_t_17;
              for (__pyx_t_19 = 0; __pyx_t_19 < __pyx_t_18; __pyx_t_19+=1) {
                __pyx_v_extra_hexose_count = __pyx_t_19;

                /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":770
 *                             if hexose_in_aggregate > hexose_count and hexose_count > 0:
 *                                 for extra_hexose_count in range(hexose_in_aggregate - hexose_count):
 *                                     if extra_hexose_count > 0 and extra_hexose_count + hexose_count > 0:             # <<<<<<<<<<<<<<
 *                                         key_ct = CountTable._create()
 *                                         if hexnac_count > 0:
 */
                __pyx_t_13 = ((__pyx_v_extra_hexose_count > 0) != 0);
                if (__pyx_t_13) {
                } else {
                  __pyx_t_9 = __pyx_t_13;
                  goto __pyx_L28_bool_binop_done;
                }
                __pyx_t_13 = (((__pyx_v_extra_hexose_count + __pyx_v_hexose_count) > 0) != 0);
                __pyx_t_9 = __pyx_t_13;
                __pyx_L28_bool_binop_done:;
                if (__pyx_t_9) {

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":771
 *                                 for extra_hexose_count in range(hexose_in_aggregate - hexose_count):
 *                                     if extra_hexose_count > 0 and extra_hexose_count + hexose_count > 0:
 *                                         key_ct = CountTable._create()             # <<<<<<<<<<<<<<
 *                                         if hexnac_count > 0:
 *                                             key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 */
                  __pyx_t_2 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable->_create()); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 771, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_2);
                  __Pyx_DECREF_SET(__pyx_v_key_ct, ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_2));
                  __pyx_t_2 = 0;

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":772
 *                                     if extra_hexose_count > 0 and extra_hexose_count + hexose_count > 0:
 *                                         key_ct = CountTable._create()
 *                                         if hexnac_count > 0:             # <<<<<<<<<<<<<<
 *                                             key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 *                                         key_ct.setitem(_HEX, hexose_count + extra_hexose_count)
 */
                  __pyx_t_9 = ((__pyx_v_hexnac_count > 0) != 0);
                  if (__pyx_t_9) {

                    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":773
 *                                         key_ct = CountTable._create()
 *                                         if hexnac_count > 0:
 *                                             key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)             # <<<<<<<<<<<<<<
 *                                         key_ct.setitem(_HEX, hexose_count + extra_hexose_count)
 *                                         shift = GlycanCompositionFragment._create(
 */
                    __pyx_t_2 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC;
                    __Pyx_INCREF(__pyx_t_2);
                    __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_2, (__pyx_v_hexnac_count + __pyx_v_extra_hexnac_count)); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 773, __pyx_L1_error)
                    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

                    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":772
 *                                     if extra_hexose_count > 0 and extra_hexose_count + hexose_count > 0:
 *                                         key_ct = CountTable._create()
 *                                         if hexnac_count > 0:             # <<<<<<<<<<<<<<
 *                                             key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 *                                         key_ct.setitem(_HEX, hexose_count + extra_hexose_count)
 */
                  }

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":774
 *                                         if hexnac_count > 0:
 *                                             key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 *                                         key_ct.setitem(_HEX, hexose_count + extra_hexose_count)             # <<<<<<<<<<<<<<
 *                                         shift = GlycanCompositionFragment._create(
 *                                             (
 */
                  __pyx_t_2 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX;
                  __Pyx_INCREF(__pyx_t_2);
                  __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_2, (__pyx_v_hexose_count + __pyx_v_extra_hexose_count)); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 774, __pyx_L1_error)
                  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":779
 *                                                 (hexnac_count + extra_hexnac_count) * _HEXNAC_mass) + (
 *                                                 (hexose_count + extra_hexose_count) * _HEX_mass),
 *                                             hexnac_hex_composition(             # <<<<<<<<<<<<<<
 *                                                 hexnac_count + extra_hexnac_count, hexose_count + extra_hexose_count),
 *                                             key_ct,
 */
                  __pyx_t_2 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_hexnac_hex_composition((__pyx_v_hexnac_count + __pyx_v_extra_hexnac_count), (__pyx_v_hexose_count + __pyx_v_extra_hexose_count))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 779, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_2);

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":775
 *                                             key_ct.setitem(_HEXNAC, hexnac_count + extra_hexnac_count)
 *                                         key_ct.setitem(_HEX, hexose_count + extra_hexose_count)
 *                                         shift = GlycanCompositionFragment._create(             # <<<<<<<<<<<<<<
 *                                             (
 *                                                 (hexnac_count + extra_hexnac_count) * _HEXNAC_mass) + (
 */
                  __pyx_t_5 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create((((__pyx_v_hexnac_count + __pyx_v_extra_hexnac_count) * __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC_mass) + ((__pyx_v_hexose_count + __pyx_v_extra_hexose_count) * __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX_mass)), ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_2), __pyx_v_key_ct, 1)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 775, __pyx_L1_error)
                  __Pyx_GOTREF(__pyx_t_5);
                  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
                  __Pyx_DECREF_SET(__pyx_v_shift, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_5));
                  __pyx_t_5 = 0;

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":784
 *                                             True
 *                                         )
 *                                         core_shifts.append(shift)             # <<<<<<<<<<<<<<
 *                                         if iteration_count < fucose_count:
 *                                             fucosylated = self.fucosylate_increment(
 */
                  __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_shift)); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 784, __pyx_L1_error)

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":785
 *                                         )
 *                                         core_shifts.append(shift)
 *                                         if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                                             fucosylated = self.fucosylate_increment(
 *                                                 shift)
 */
                  __pyx_t_9 = ((__pyx_v_iteration_count < __pyx_v_fucose_count) != 0);
                  if (__pyx_t_9) {

                    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":786
 *                                         core_shifts.append(shift)
 *                                         if iteration_count < fucose_count:
 *                                             fucosylated = self.fucosylate_increment(             # <<<<<<<<<<<<<<
 *                                                 shift)
 *                                             core_shifts.append(fucosylated)
 */
                    __pyx_t_5 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->fucosylate_increment(__pyx_v_self, __pyx_v_shift, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 786, __pyx_L1_error)
                    __Pyx_GOTREF(__pyx_t_5);
                    __Pyx_XDECREF_SET(__pyx_v_fucosylated, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_5));
                    __pyx_t_5 = 0;

                    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":788
 *                                             fucosylated = self.fucosylate_increment(
 *                                                 shift)
 *                                             core_shifts.append(fucosylated)             # <<<<<<<<<<<<<<
 * 
 *         return core_shifts
 */
                    __pyx_t_11 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_fucosylated)); if (unlikely(__pyx_t_11 == ((int)-1))) __PYX_ERR(0, 788, __pyx_L1_error)

                    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":785
 *                                         )
 *                                         core_shifts.append(shift)
 *                                         if iteration_count < fucose_count:             # <<<<<<<<<<<<<<
 *                                             fucosylated = self.fucosylate_increment(
 *                                                 shift)
 */
                  }

                  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":770
 *                             if hexose_in_aggregate > hexose_count and hexose_count > 0:
 *                                 for extra_hexose_count in range(hexose_in_aggregate - hexose_count):
 *                                     if extra_hexose_count > 0 and extra_hexose_count + hexose_count > 0:             # <<<<<<<<<<<<<<
 *                                         key_ct = CountTable._create()
 *                                         if hexnac_count > 0:
 */
                }
              }

              /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":768
 *                                     fucosylated = self.fucosylate_increment(shift)
 *                                     core_shifts.append(fucosylated)
 *                             if hexose_in_aggregate > hexose_count and hexose_count > 0:             # <<<<<<<<<<<<<<
 *                                 for extra_hexose_count in range(hexose_in_aggregate - hexose_count):
 *                                     if extra_hexose_count > 0 and extra_hexose_count + hexose_count > 0:
 */
            }
          }

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":749
 *                     # on the remaining core monosaccharides sequentially until
 *                     # exhausted.
 *                     if self.extended and hexnac_in_aggregate - hexnac_count >= 0:             # <<<<<<<<<<<<<<
 *                         for extra_hexnac_count in range(hexnac_in_aggregate - hexnac_count + 1):
 *                             if extra_hexnac_count:
 */
        }
        __pyx_L9_continue:;
      }

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":714
 *                 )
 *                 core_shifts.append(shift)
 *             elif hexnac_count >= 1:             # <<<<<<<<<<<<<<
 *                 key_ct = CountTable._create()
 *                 key_ct.setitem(_HEXNAC, hexnac_count)
 */
    }
    __pyx_L7:;
    __pyx_L4_continue:;
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":790
 *                                             core_shifts.append(fucosylated)
 * 
 *         return core_shifts             # <<<<<<<<<<<<<<
 * 
 *     def gag_linker_stub_fragments(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_core_shifts);
  __pyx_r = __pyx_v_core_shifts;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":686
 *                 is_extended=site.is_extended)
 * 
 *     cpdef list o_glycan_composition_fragments(self, glycan, long core_count=1, long iteration_count=0):             # <<<<<<<<<<<<<<
 *         cdef:
 *             list core_shifts
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.o_glycan_composition_fragments", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_core_shifts);
  __Pyx_XDECREF((PyObject *)__pyx_v_key_ct);
  __Pyx_XDECREF((PyObject *)__pyx_v_shift);
  __Pyx_XDECREF((PyObject *)__pyx_v_fucosylated);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_25o_glycan_composition_fragments(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_24o_glycan_composition_fragments[] = "StubGlycopeptideStrategy.o_glycan_composition_fragments(self, glycan, long core_count=1, long iteration_count=0) -> list";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_25o_glycan_composition_fragments(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_glycan = 0;
  long __pyx_v_core_count;
  long __pyx_v_iteration_count;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("o_glycan_composition_fragments (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_glycan,&__pyx_n_s_core_count,&__pyx_n_s_iteration_count,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_glycan)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_core_count);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_iteration_count);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "o_glycan_composition_fragments") < 0)) __PYX_ERR(0, 686, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_glycan = values[0];
    if (values[1]) {
      __pyx_v_core_count = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v_core_count == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 686, __pyx_L3_error)
    } else {
      __pyx_v_core_count = ((long)1);
    }
    if (values[2]) {
      __pyx_v_iteration_count = __Pyx_PyInt_As_long(values[2]); if (unlikely((__pyx_v_iteration_count == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 686, __pyx_L3_error)
    } else {
      __pyx_v_iteration_count = ((long)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("o_glycan_composition_fragments", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 686, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.o_glycan_composition_fragments", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_24o_glycan_composition_fragments(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self), __pyx_v_glycan, __pyx_v_core_count, __pyx_v_iteration_count);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_24o_glycan_composition_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_glycan, long __pyx_v_core_count, long __pyx_v_iteration_count) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_o_glycan_composition_fragments __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("o_glycan_composition_fragments", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 2;
  __pyx_t_2.core_count = __pyx_v_core_count;
  __pyx_t_2.iteration_count = __pyx_v_iteration_count;
  __pyx_t_1 = __pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy->o_glycan_composition_fragments(__pyx_v_self, __pyx_v_glycan, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 686, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.o_glycan_composition_fragments", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_28generator2(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":792
 *         return core_shifts
 * 
 *     def gag_linker_stub_fragments(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             list per_site_shifts
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_27gag_linker_stub_fragments(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_26gag_linker_stub_fragments[] = "StubGlycopeptideStrategy.gag_linker_stub_fragments(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_27gag_linker_stub_fragments(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gag_linker_stub_fragments (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_26gag_linker_stub_fragments(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_26gag_linker_stub_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gag_linker_stub_fragments", 0);
  __pyx_cur_scope = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments *)__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments(__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 792, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_28generator2, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_gag_linker_stub_fragments, __pyx_n_s_StubGlycopeptideStrategy_gag_lin, __pyx_n_s_glycopeptidepy__c_structure_frag); if (unlikely(!gen)) __PYX_ERR(0, 792, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.gag_linker_stub_fragments", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_28generator2(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments *__pyx_cur_scope = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  size_t __pyx_t_3;
  size_t __pyx_t_4;
  size_t __pyx_t_5;
  struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_gag_linker_composition_fragments __pyx_t_6;
  int __pyx_t_7;
  Py_ssize_t __pyx_t_8;
  Py_ssize_t __pyx_t_9;
  PyObject *__pyx_t_10;
  int __pyx_t_11;
  PyObject *__pyx_t_12 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gag_linker_stub_fragments", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L9_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 792, __pyx_L1_error)

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":804
 *             str name
 *             set seen
 *         glycan = self.glycan_composition()             # <<<<<<<<<<<<<<
 *         self._use_query = self._guess_query_mode(glycan)
 *         core_count = self.count_glycosylation_type(GlycosylationType_glycosaminoglycan)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.glycan_composition(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_cur_scope->__pyx_v_self), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 804, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_glycan = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":805
 *             set seen
 *         glycan = self.glycan_composition()
 *         self._use_query = self._guess_query_mode(glycan)             # <<<<<<<<<<<<<<
 *         core_count = self.count_glycosylation_type(GlycosylationType_glycosaminoglycan)
 *         per_site_shifts = []
 */
  __pyx_cur_scope->__pyx_v_self->__pyx_base._use_query = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base._guess_query_mode(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_cur_scope->__pyx_v_self), __pyx_cur_scope->__pyx_v_glycan, 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":806
 *         glycan = self.glycan_composition()
 *         self._use_query = self._guess_query_mode(glycan)
 *         core_count = self.count_glycosylation_type(GlycosylationType_glycosaminoglycan)             # <<<<<<<<<<<<<<
 *         per_site_shifts = []
 * 
 */
  __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_glycosaminoglycan;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_core_count = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.count_glycosylation_type(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_cur_scope->__pyx_v_self), __pyx_t_1, 0);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":807
 *         self._use_query = self._guess_query_mode(glycan)
 *         core_count = self.count_glycosylation_type(GlycosylationType_glycosaminoglycan)
 *         per_site_shifts = []             # <<<<<<<<<<<<<<
 * 
 *         base_composition = self.peptide_composition()
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 807, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_per_site_shifts = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":809
 *         per_site_shifts = []
 * 
 *         base_composition = self.peptide_composition()             # <<<<<<<<<<<<<<
 *         base_mass = base_composition.calc_mass()
 *         for i in range(core_count):
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.__pyx_base.peptide_composition(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase *)__pyx_cur_scope->__pyx_v_self), 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 809, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_base_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":810
 * 
 *         base_composition = self.peptide_composition()
 *         base_mass = base_composition.calc_mass()             # <<<<<<<<<<<<<<
 *         for i in range(core_count):
 *             core_shifts = self.gag_linker_composition_fragments(glycan, core_count, i)
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_cur_scope->__pyx_v_base_composition->__pyx_vtab)->calc_mass(__pyx_cur_scope->__pyx_v_base_composition, 0, NULL); if (unlikely(__pyx_t_2 == ((double)-1.0))) __PYX_ERR(0, 810, __pyx_L1_error)
  __pyx_cur_scope->__pyx_v_base_mass = __pyx_t_2;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":811
 *         base_composition = self.peptide_composition()
 *         base_mass = base_composition.calc_mass()
 *         for i in range(core_count):             # <<<<<<<<<<<<<<
 *             core_shifts = self.gag_linker_composition_fragments(glycan, core_count, i)
 *             per_site_shifts.append(core_shifts)
 */
  __pyx_t_3 = __pyx_cur_scope->__pyx_v_core_count;
  __pyx_t_4 = __pyx_t_3;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_cur_scope->__pyx_v_i = __pyx_t_5;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":812
 *         base_mass = base_composition.calc_mass()
 *         for i in range(core_count):
 *             core_shifts = self.gag_linker_composition_fragments(glycan, core_count, i)             # <<<<<<<<<<<<<<
 *             per_site_shifts.append(core_shifts)
 *         combos = self._combinate_sites(per_site_shifts, glycan)
 */
    __pyx_t_6.__pyx_n = 2;
    __pyx_t_6.core_count = __pyx_cur_scope->__pyx_v_core_count;
    __pyx_t_6.iteration_count = __pyx_cur_scope->__pyx_v_i;
    __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->gag_linker_composition_fragments(__pyx_cur_scope->__pyx_v_self, __pyx_cur_scope->__pyx_v_glycan, 0, &__pyx_t_6); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 812, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_core_shifts);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_core_shifts, ((PyObject*)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":813
 *         for i in range(core_count):
 *             core_shifts = self.gag_linker_composition_fragments(glycan, core_count, i)
 *             per_site_shifts.append(core_shifts)             # <<<<<<<<<<<<<<
 *         combos = self._combinate_sites(per_site_shifts, glycan)
 *         n_combos = PyList_Size(combos)
 */
    __pyx_t_7 = __Pyx_PyList_Append(__pyx_cur_scope->__pyx_v_per_site_shifts, __pyx_cur_scope->__pyx_v_core_shifts); if (unlikely(__pyx_t_7 == ((int)-1))) __PYX_ERR(0, 813, __pyx_L1_error)
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":814
 *             core_shifts = self.gag_linker_composition_fragments(glycan, core_count, i)
 *             per_site_shifts.append(core_shifts)
 *         combos = self._combinate_sites(per_site_shifts, glycan)             # <<<<<<<<<<<<<<
 *         n_combos = PyList_Size(combos)
 *         for i in range(n_combos):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->_combinate_sites(__pyx_cur_scope->__pyx_v_self, __pyx_cur_scope->__pyx_v_per_site_shifts, __pyx_cur_scope->__pyx_v_glycan, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 814, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_cur_scope->__pyx_v_combos = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":815
 *             per_site_shifts.append(core_shifts)
 *         combos = self._combinate_sites(per_site_shifts, glycan)
 *         n_combos = PyList_Size(combos)             # <<<<<<<<<<<<<<
 *         for i in range(n_combos):
 *             result = <tuple>PyList_GetItem(combos, i)
 */
  __pyx_t_8 = PyList_Size(__pyx_cur_scope->__pyx_v_combos); if (unlikely(__pyx_t_8 == ((Py_ssize_t)-1L))) __PYX_ERR(0, 815, __pyx_L1_error)
  __pyx_cur_scope->__pyx_v_n_combos = __pyx_t_8;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":816
 *         combos = self._combinate_sites(per_site_shifts, glycan)
 *         n_combos = PyList_Size(combos)
 *         for i in range(n_combos):             # <<<<<<<<<<<<<<
 *             result = <tuple>PyList_GetItem(combos, i)
 *             site = <GlycanCompositionFragment>PyTuple_GetItem(result, 0)
 */
  __pyx_t_8 = __pyx_cur_scope->__pyx_v_n_combos;
  __pyx_t_9 = __pyx_t_8;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_9; __pyx_t_3+=1) {
    __pyx_cur_scope->__pyx_v_i = __pyx_t_3;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":817
 *         n_combos = PyList_Size(combos)
 *         for i in range(n_combos):
 *             result = <tuple>PyList_GetItem(combos, i)             # <<<<<<<<<<<<<<
 *             site = <GlycanCompositionFragment>PyTuple_GetItem(result, 0)
 *             name = <str>PyTuple_GetItem(result, 1)
 */
    __pyx_t_10 = PyList_GetItem(__pyx_cur_scope->__pyx_v_combos, __pyx_cur_scope->__pyx_v_i); if (unlikely(__pyx_t_10 == ((PyObject *)NULL))) __PYX_ERR(0, 817, __pyx_L1_error)
    __pyx_t_1 = ((PyObject *)__pyx_t_10);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_result);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_result, ((PyObject*)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":818
 *         for i in range(n_combos):
 *             result = <tuple>PyList_GetItem(combos, i)
 *             site = <GlycanCompositionFragment>PyTuple_GetItem(result, 0)             # <<<<<<<<<<<<<<
 *             name = <str>PyTuple_GetItem(result, 1)
 *             glycosylation = <object>PyTuple_GetItem(result, 2)
 */
    __pyx_t_10 = PyTuple_GetItem(__pyx_cur_scope->__pyx_v_result, 0); if (unlikely(__pyx_t_10 == ((PyObject *)NULL))) __PYX_ERR(0, 818, __pyx_L1_error)
    __pyx_t_1 = ((PyObject *)__pyx_t_10);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XGOTREF(((PyObject *)__pyx_cur_scope->__pyx_v_site));
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_site, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":819
 *             result = <tuple>PyList_GetItem(combos, i)
 *             site = <GlycanCompositionFragment>PyTuple_GetItem(result, 0)
 *             name = <str>PyTuple_GetItem(result, 1)             # <<<<<<<<<<<<<<
 *             glycosylation = <object>PyTuple_GetItem(result, 2)
 *             is_glycosylated = (site.mass != 0)
 */
    __pyx_t_10 = PyTuple_GetItem(__pyx_cur_scope->__pyx_v_result, 1); if (unlikely(__pyx_t_10 == ((PyObject *)NULL))) __PYX_ERR(0, 819, __pyx_L1_error)
    __pyx_t_1 = ((PyObject *)__pyx_t_10);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_name);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_name, ((PyObject*)__pyx_t_1));
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":820
 *             site = <GlycanCompositionFragment>PyTuple_GetItem(result, 0)
 *             name = <str>PyTuple_GetItem(result, 1)
 *             glycosylation = <object>PyTuple_GetItem(result, 2)             # <<<<<<<<<<<<<<
 *             is_glycosylated = (site.mass != 0)
 *             composition = None
 */
    __pyx_t_10 = PyTuple_GetItem(__pyx_cur_scope->__pyx_v_result, 2); if (unlikely(__pyx_t_10 == ((PyObject *)NULL))) __PYX_ERR(0, 820, __pyx_L1_error)
    __pyx_t_1 = ((PyObject *)__pyx_t_10);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XGOTREF(__pyx_cur_scope->__pyx_v_glycosylation);
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_glycosylation, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":821
 *             name = <str>PyTuple_GetItem(result, 1)
 *             glycosylation = <object>PyTuple_GetItem(result, 2)
 *             is_glycosylated = (site.mass != 0)             # <<<<<<<<<<<<<<
 *             composition = None
 *             if self.compute_compositions:
 */
    __pyx_cur_scope->__pyx_v_is_glycosylated = (__pyx_cur_scope->__pyx_v_site->mass != 0.0);

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":822
 *             glycosylation = <object>PyTuple_GetItem(result, 2)
 *             is_glycosylated = (site.mass != 0)
 *             composition = None             # <<<<<<<<<<<<<<
 *             if self.compute_compositions:
 *                 composition = base_composition.clone()
 */
    __Pyx_INCREF(Py_None);
    __Pyx_XGOTREF(((PyObject *)__pyx_cur_scope->__pyx_v_composition));
    __Pyx_XDECREF_SET(__pyx_cur_scope->__pyx_v_composition, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None));
    __Pyx_GIVEREF(Py_None);

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":823
 *             is_glycosylated = (site.mass != 0)
 *             composition = None
 *             if self.compute_compositions:             # <<<<<<<<<<<<<<
 *                 composition = base_composition.clone()
 *                 composition.add_from(site.composition)
 */
    __pyx_t_11 = (__pyx_cur_scope->__pyx_v_self->__pyx_base.__pyx_base.compute_compositions != 0);
    if (__pyx_t_11) {

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":824
 *             composition = None
 *             if self.compute_compositions:
 *                 composition = base_composition.clone()             # <<<<<<<<<<<<<<
 *                 composition.add_from(site.composition)
 *             yield StubFragment._create(
 */
      __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_cur_scope->__pyx_v_base_composition->__pyx_vtab)->clone(__pyx_cur_scope->__pyx_v_base_composition, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 824, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_GOTREF(((PyObject *)__pyx_cur_scope->__pyx_v_composition));
      __Pyx_DECREF_SET(__pyx_cur_scope->__pyx_v_composition, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1));
      __Pyx_GIVEREF(__pyx_t_1);
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":825
 *             if self.compute_compositions:
 *                 composition = base_composition.clone()
 *                 composition.add_from(site.composition)             # <<<<<<<<<<<<<<
 *             yield StubFragment._create(
 *                 name=name,
 */
      __pyx_t_1 = ((PyObject *)__pyx_cur_scope->__pyx_v_site->composition);
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_12 = ((PyObject *)((struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *)__pyx_cur_scope->__pyx_v_composition->__pyx_vtab)->add_from(__pyx_cur_scope->__pyx_v_composition, __pyx_t_1)); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 825, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_12);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":823
 *             is_glycosylated = (site.mass != 0)
 *             composition = None
 *             if self.compute_compositions:             # <<<<<<<<<<<<<<
 *                 composition = base_composition.clone()
 *                 composition.add_from(site.composition)
 */
    }

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":831
 *                 composition=composition,
 *                 is_glycosylated=is_glycosylated,
 *                 kind=IonSeries_stub_glycopeptide,             # <<<<<<<<<<<<<<
 *                 chemical_shift=None,
 *                 glycosylation=glycosylation,
 */
    __pyx_t_12 = ((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_IonSeries_stub_glycopeptide);
    __Pyx_INCREF(__pyx_t_12);

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":826
 *                 composition = base_composition.clone()
 *                 composition.add_from(site.composition)
 *             yield StubFragment._create(             # <<<<<<<<<<<<<<
 *                 name=name,
 *                 mass=base_mass + site.mass,
 */
    __pyx_t_1 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_StubFragment->_create(__pyx_cur_scope->__pyx_v_name, (__pyx_cur_scope->__pyx_v_base_mass + __pyx_cur_scope->__pyx_v_site->mass), ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase *)__pyx_t_12), __pyx_cur_scope->__pyx_v_composition, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase *)Py_None), __pyx_cur_scope->__pyx_v_is_glycosylated, __pyx_cur_scope->__pyx_v_glycosylation, __pyx_cur_scope->__pyx_v_site->is_extended)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 826, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    __pyx_cur_scope->__pyx_t_0 = __pyx_t_3;
    __pyx_cur_scope->__pyx_t_1 = __pyx_t_8;
    __pyx_cur_scope->__pyx_t_2 = __pyx_t_9;
    __Pyx_XGIVEREF(__pyx_r);
    __Pyx_RefNannyFinishContext();
    __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
    /* return from generator, yielding value */
    __pyx_generator->resume_label = 1;
    return __pyx_r;
    __pyx_L9_resume_from_yield:;
    __pyx_t_3 = __pyx_cur_scope->__pyx_t_0;
    __pyx_t_8 = __pyx_cur_scope->__pyx_t_1;
    __pyx_t_9 = __pyx_cur_scope->__pyx_t_2;
    if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 826, __pyx_L1_error)
  }
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":792
 *         return core_shifts
 * 
 *     def gag_linker_stub_fragments(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             list per_site_shifts
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("gag_linker_stub_fragments", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":836
 *                 is_extended=site.is_extended)
 * 
 *     cpdef list gag_linker_composition_fragments(self, glycan, long core_count=1, long iteration_count=0):             # <<<<<<<<<<<<<<
 *         cdef:
 *             long xyl_in_aggregate
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_30gag_linker_composition_fragments(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_gag_linker_composition_fragments(CYTHON_UNUSED struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_glycan, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_gag_linker_composition_fragments *__pyx_optional_args) {
  long __pyx_v_core_count = ((long)1);
  long __pyx_v_iteration_count = ((long)0);
  long __pyx_v_xyl_in_aggregate;
  long __pyx_v_xyl_count;
  long __pyx_v_hexose_count;
  PyObject *__pyx_v_core_shifts = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *__pyx_v_key_ct = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v_shift = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  long __pyx_t_9;
  int __pyx_t_10;
  long __pyx_t_11;
  long __pyx_t_12;
  int __pyx_t_13;
  long __pyx_t_14;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gag_linker_composition_fragments", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_core_count = __pyx_optional_args->core_count;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_iteration_count = __pyx_optional_args->iteration_count;
      }
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_gag_linker_composition_fragments); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 836, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_30gag_linker_composition_fragments)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_v_core_count); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 836, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = __Pyx_PyInt_From_long(__pyx_v_iteration_count); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 836, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_5 = __pyx_t_1; __pyx_t_6 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_v_glycan, __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 836, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[4] = {__pyx_t_6, __pyx_v_glycan, __pyx_t_3, __pyx_t_4};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 3+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 836, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(3+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 836, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          if (__pyx_t_6) {
            __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
          }
          __Pyx_INCREF(__pyx_v_glycan);
          __Pyx_GIVEREF(__pyx_v_glycan);
          PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_v_glycan);
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_t_4);
          __pyx_t_3 = 0;
          __pyx_t_4 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 836, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (!(likely(PyList_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 836, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":842
 *             list core_shifts
 *             CountTable key_ct
 *         xyl_in_aggregate = glycan[_XYL]             # <<<<<<<<<<<<<<
 *         if xyl_in_aggregate == 0:
 *             xyl_in_aggregate = glycan.query("Pen", exact=False)
 */
  __pyx_t_1 = __Pyx_PyObject_GetItem(__pyx_v_glycan, __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 842, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_9 = __Pyx_PyInt_As_long(__pyx_t_1); if (unlikely((__pyx_t_9 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 842, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_xyl_in_aggregate = __pyx_t_9;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":843
 *             CountTable key_ct
 *         xyl_in_aggregate = glycan[_XYL]
 *         if xyl_in_aggregate == 0:             # <<<<<<<<<<<<<<
 *             xyl_in_aggregate = glycan.query("Pen", exact=False)
 *         core_shifts = []
 */
  __pyx_t_10 = ((__pyx_v_xyl_in_aggregate == 0) != 0);
  if (__pyx_t_10) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":844
 *         xyl_in_aggregate = glycan[_XYL]
 *         if xyl_in_aggregate == 0:
 *             xyl_in_aggregate = glycan.query("Pen", exact=False)             # <<<<<<<<<<<<<<
 *         core_shifts = []
 *         for xyl_count in range(max(0, xyl_in_aggregate) + 1):
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_glycan, __pyx_n_s_query); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 844, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 844, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_exact, Py_False) < 0) __PYX_ERR(0, 844, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_tuple_, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 844, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_9 = __Pyx_PyInt_As_long(__pyx_t_5); if (unlikely((__pyx_t_9 == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 844, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_v_xyl_in_aggregate = __pyx_t_9;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":843
 *             CountTable key_ct
 *         xyl_in_aggregate = glycan[_XYL]
 *         if xyl_in_aggregate == 0:             # <<<<<<<<<<<<<<
 *             xyl_in_aggregate = glycan.query("Pen", exact=False)
 *         core_shifts = []
 */
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":845
 *         if xyl_in_aggregate == 0:
 *             xyl_in_aggregate = glycan.query("Pen", exact=False)
 *         core_shifts = []             # <<<<<<<<<<<<<<
 *         for xyl_count in range(max(0, xyl_in_aggregate) + 1):
 *             if xyl_count == 0:
 */
  __pyx_t_5 = PyList_New(0); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 845, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_v_core_shifts = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":846
 *             xyl_in_aggregate = glycan.query("Pen", exact=False)
 *         core_shifts = []
 *         for xyl_count in range(max(0, xyl_in_aggregate) + 1):             # <<<<<<<<<<<<<<
 *             if xyl_count == 0:
 *                 shift = GlycanCompositionFragment._create(
 */
  __pyx_t_9 = __pyx_v_xyl_in_aggregate;
  __pyx_t_11 = 0;
  if (((__pyx_t_9 > __pyx_t_11) != 0)) {
    __pyx_t_12 = __pyx_t_9;
  } else {
    __pyx_t_12 = __pyx_t_11;
  }
  __pyx_t_9 = (__pyx_t_12 + 1);
  __pyx_t_12 = __pyx_t_9;
  for (__pyx_t_11 = 0; __pyx_t_11 < __pyx_t_12; __pyx_t_11+=1) {
    __pyx_v_xyl_count = __pyx_t_11;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":847
 *         core_shifts = []
 *         for xyl_count in range(max(0, xyl_in_aggregate) + 1):
 *             if xyl_count == 0:             # <<<<<<<<<<<<<<
 *                 shift = GlycanCompositionFragment._create(
 *                     0,
 */
    __pyx_t_10 = ((__pyx_v_xyl_count == 0) != 0);
    if (__pyx_t_10) {

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":850
 *                 shift = GlycanCompositionFragment._create(
 *                     0,
 *                     CComposition(),             # <<<<<<<<<<<<<<
 *                     CountTable._create(),
 *                     False
 */
      __pyx_t_5 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_5glypy_11composition_12ccomposition_CComposition)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 850, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":851
 *                     0,
 *                     CComposition(),
 *                     CountTable._create(),             # <<<<<<<<<<<<<<
 *                     False
 *                 )
 */
      __pyx_t_2 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable->_create()); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 851, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":848
 *         for xyl_count in range(max(0, xyl_in_aggregate) + 1):
 *             if xyl_count == 0:
 *                 shift = GlycanCompositionFragment._create(             # <<<<<<<<<<<<<<
 *                     0,
 *                     CComposition(),
 */
      __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create(0.0, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_5), ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_2), 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 848, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF_SET(__pyx_v_shift, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":854
 *                     False
 *                 )
 *                 core_shifts.append(shift)             # <<<<<<<<<<<<<<
 *             else:
 *                 key_ct = CountTable._create()
 */
      __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_shift)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 854, __pyx_L1_error)

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":847
 *         core_shifts = []
 *         for xyl_count in range(max(0, xyl_in_aggregate) + 1):
 *             if xyl_count == 0:             # <<<<<<<<<<<<<<
 *                 shift = GlycanCompositionFragment._create(
 *                     0,
 */
      goto __pyx_L6;
    }

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":856
 *                 core_shifts.append(shift)
 *             else:
 *                 key_ct = CountTable._create()             # <<<<<<<<<<<<<<
 *                 key_ct.setitem(_XYL, xyl_count)
 *                 shift = GlycanCompositionFragment._create(
 */
    /*else*/ {
      __pyx_t_1 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable->_create()); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 856, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_key_ct, ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":857
 *             else:
 *                 key_ct = CountTable._create()
 *                 key_ct.setitem(_XYL, xyl_count)             # <<<<<<<<<<<<<<
 *                 shift = GlycanCompositionFragment._create(
 *                     _XYL_mass * xyl_count,
 */
      __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL;
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_1, __pyx_v_xyl_count); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 857, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":860
 *                 shift = GlycanCompositionFragment._create(
 *                     _XYL_mass * xyl_count,
 *                     _XYL_composition * xyl_count,             # <<<<<<<<<<<<<<
 *                     key_ct,
 *                     False
 */
      __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v_xyl_count); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 860, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = PyNumber_Multiply(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL_composition), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 860, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 860, __pyx_L1_error)

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":858
 *                 key_ct = CountTable._create()
 *                 key_ct.setitem(_XYL, xyl_count)
 *                 shift = GlycanCompositionFragment._create(             # <<<<<<<<<<<<<<
 *                     _XYL_mass * xyl_count,
 *                     _XYL_composition * xyl_count,
 */
      __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create((__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL_mass * __pyx_v_xyl_count), ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_2), __pyx_v_key_ct, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 858, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_XDECREF_SET(__pyx_v_shift, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":864
 *                     False
 *                 )
 *                 core_shifts.append(shift)             # <<<<<<<<<<<<<<
 *             if xyl_count > 0:
 *                 # TODO: Handle modified Hexose residues here too.
 */
      __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_shift)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 864, __pyx_L1_error)
    }
    __pyx_L6:;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":865
 *                 )
 *                 core_shifts.append(shift)
 *             if xyl_count > 0:             # <<<<<<<<<<<<<<
 *                 # TODO: Handle modified Hexose residues here too.
 *                 for hexose_count in range(1, 3):
 */
    __pyx_t_10 = ((__pyx_v_xyl_count > 0) != 0);
    if (__pyx_t_10) {

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":867
 *             if xyl_count > 0:
 *                 # TODO: Handle modified Hexose residues here too.
 *                 for hexose_count in range(1, 3):             # <<<<<<<<<<<<<<
 *                     key_ct = CountTable._create()
 *                     key_ct.setitem(_XYL, xyl_count)
 */
      for (__pyx_t_14 = 1; __pyx_t_14 < 3; __pyx_t_14+=1) {
        __pyx_v_hexose_count = __pyx_t_14;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":868
 *                 # TODO: Handle modified Hexose residues here too.
 *                 for hexose_count in range(1, 3):
 *                     key_ct = CountTable._create()             # <<<<<<<<<<<<<<
 *                     key_ct.setitem(_XYL, xyl_count)
 *                     key_ct.setitem(_HEX, hexose_count)
 */
        __pyx_t_1 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable->_create()); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 868, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_XDECREF_SET(__pyx_v_key_ct, ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":869
 *                 for hexose_count in range(1, 3):
 *                     key_ct = CountTable._create()
 *                     key_ct.setitem(_XYL, xyl_count)             # <<<<<<<<<<<<<<
 *                     key_ct.setitem(_HEX, hexose_count)
 *                     shift = GlycanCompositionFragment._create(
 */
        __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_1, __pyx_v_xyl_count); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 869, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":870
 *                     key_ct = CountTable._create()
 *                     key_ct.setitem(_XYL, xyl_count)
 *                     key_ct.setitem(_HEX, hexose_count)             # <<<<<<<<<<<<<<
 *                     shift = GlycanCompositionFragment._create(
 *                         ((_XYL_mass * xyl_count) + (
 */
        __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_1, __pyx_v_hexose_count); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 870, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":875
 *                             _HEX_mass * hexose_count)),
 *                         (
 *                             (_XYL_composition * xyl_count) + (             # <<<<<<<<<<<<<<
 *                                 _HEX_composition * hexose_count)),
 *                         key_ct,
 */
        __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v_xyl_count); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 875, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_2 = PyNumber_Multiply(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL_composition), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 875, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":876
 *                         (
 *                             (_XYL_composition * xyl_count) + (
 *                                 _HEX_composition * hexose_count)),             # <<<<<<<<<<<<<<
 *                         key_ct,
 *                         False
 */
        __pyx_t_1 = __Pyx_PyInt_From_long(__pyx_v_hexose_count); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 876, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_5 = PyNumber_Multiply(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX_composition), __pyx_t_1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 876, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":875
 *                             _HEX_mass * hexose_count)),
 *                         (
 *                             (_XYL_composition * xyl_count) + (             # <<<<<<<<<<<<<<
 *                                 _HEX_composition * hexose_count)),
 *                         key_ct,
 */
        __pyx_t_1 = PyNumber_Add(__pyx_t_2, __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 875, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 875, __pyx_L1_error)

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":871
 *                     key_ct.setitem(_XYL, xyl_count)
 *                     key_ct.setitem(_HEX, hexose_count)
 *                     shift = GlycanCompositionFragment._create(             # <<<<<<<<<<<<<<
 *                         ((_XYL_mass * xyl_count) + (
 *                             _HEX_mass * hexose_count)),
 */
        __pyx_t_5 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create(((__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL_mass * __pyx_v_xyl_count) + (__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX_mass * __pyx_v_hexose_count)), ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1), __pyx_v_key_ct, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 871, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_5);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_DECREF_SET(__pyx_v_shift, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_5));
        __pyx_t_5 = 0;

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":880
 *                         False
 *                     )
 *                     core_shifts.append(shift)             # <<<<<<<<<<<<<<
 *                     if hexose_count == 2:
 *                         key_ct = CountTable._create()
 */
        __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_shift)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 880, __pyx_L1_error)

        /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":881
 *                     )
 *                     core_shifts.append(shift)
 *                     if hexose_count == 2:             # <<<<<<<<<<<<<<
 *                         key_ct = CountTable._create()
 *                         key_ct.setitem(_XYL, xyl_count)
 */
        __pyx_t_10 = ((__pyx_v_hexose_count == 2) != 0);
        if (__pyx_t_10) {

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":882
 *                     core_shifts.append(shift)
 *                     if hexose_count == 2:
 *                         key_ct = CountTable._create()             # <<<<<<<<<<<<<<
 *                         key_ct.setitem(_XYL, xyl_count)
 *                         key_ct.setitem(_HEX, hexose_count)
 */
          __pyx_t_5 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable->_create()); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 882, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF_SET(__pyx_v_key_ct, ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_5));
          __pyx_t_5 = 0;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":883
 *                     if hexose_count == 2:
 *                         key_ct = CountTable._create()
 *                         key_ct.setitem(_XYL, xyl_count)             # <<<<<<<<<<<<<<
 *                         key_ct.setitem(_HEX, hexose_count)
 *                         key_ct.setitem(_AHEX, 1)
 */
          __pyx_t_5 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL;
          __Pyx_INCREF(__pyx_t_5);
          __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_5, __pyx_v_xyl_count); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 883, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":884
 *                         key_ct = CountTable._create()
 *                         key_ct.setitem(_XYL, xyl_count)
 *                         key_ct.setitem(_HEX, hexose_count)             # <<<<<<<<<<<<<<
 *                         key_ct.setitem(_AHEX, 1)
 *                         shift = GlycanCompositionFragment._create(
 */
          __pyx_t_5 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX;
          __Pyx_INCREF(__pyx_t_5);
          __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_5, __pyx_v_hexose_count); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 884, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":885
 *                         key_ct.setitem(_XYL, xyl_count)
 *                         key_ct.setitem(_HEX, hexose_count)
 *                         key_ct.setitem(_AHEX, 1)             # <<<<<<<<<<<<<<
 *                         shift = GlycanCompositionFragment._create(
 *                             ((_XYL_mass * xyl_count) + (
 */
          __pyx_t_5 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__AHEX;
          __Pyx_INCREF(__pyx_t_5);
          __pyx_t_7 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_v_key_ct->__pyx_vtab)->setitem(__pyx_v_key_ct, __pyx_t_5, 1); if (unlikely(__pyx_t_7 == ((int)1))) __PYX_ERR(0, 885, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":890
 *                                 _HEX_mass * hexose_count) + _AHEX_mass),
 *                             (
 *                                 (_XYL_composition * xyl_count) + (             # <<<<<<<<<<<<<<
 *                                     _HEX_composition * hexose_count) + _AHEX_composition),
 *                             key_ct,
 */
          __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_v_xyl_count); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 890, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_1 = PyNumber_Multiply(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL_composition), __pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 890, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_1);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":891
 *                             (
 *                                 (_XYL_composition * xyl_count) + (
 *                                     _HEX_composition * hexose_count) + _AHEX_composition),             # <<<<<<<<<<<<<<
 *                             key_ct,
 *                             False
 */
          __pyx_t_5 = __Pyx_PyInt_From_long(__pyx_v_hexose_count); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 891, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_2 = PyNumber_Multiply(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX_composition), __pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 891, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":890
 *                                 _HEX_mass * hexose_count) + _AHEX_mass),
 *                             (
 *                                 (_XYL_composition * xyl_count) + (             # <<<<<<<<<<<<<<
 *                                     _HEX_composition * hexose_count) + _AHEX_composition),
 *                             key_ct,
 */
          __pyx_t_5 = PyNumber_Add(__pyx_t_1, __pyx_t_2); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 890, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":891
 *                             (
 *                                 (_XYL_composition * xyl_count) + (
 *                                     _HEX_composition * hexose_count) + _AHEX_composition),             # <<<<<<<<<<<<<<
 *                             key_ct,
 *                             False
 */
          __pyx_t_2 = PyNumber_Add(__pyx_t_5, ((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__AHEX_composition)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 891, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 891, __pyx_L1_error)

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":886
 *                         key_ct.setitem(_HEX, hexose_count)
 *                         key_ct.setitem(_AHEX, 1)
 *                         shift = GlycanCompositionFragment._create(             # <<<<<<<<<<<<<<
 *                             ((_XYL_mass * xyl_count) + (
 *                                 _HEX_mass * hexose_count) + _AHEX_mass),
 */
          __pyx_t_5 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create((((__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL_mass * __pyx_v_xyl_count) + (__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX_mass * __pyx_v_hexose_count)) + __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__AHEX_mass), ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_2), __pyx_v_key_ct, 0)); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 886, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __Pyx_DECREF_SET(__pyx_v_shift, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_t_5));
          __pyx_t_5 = 0;

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":895
 *                             False
 *                         )
 *                         core_shifts.append(shift)             # <<<<<<<<<<<<<<
 *         return core_shifts
 * 
 */
          __pyx_t_13 = __Pyx_PyList_Append(__pyx_v_core_shifts, ((PyObject *)__pyx_v_shift)); if (unlikely(__pyx_t_13 == ((int)-1))) __PYX_ERR(0, 895, __pyx_L1_error)

          /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":881
 *                     )
 *                     core_shifts.append(shift)
 *                     if hexose_count == 2:             # <<<<<<<<<<<<<<
 *                         key_ct = CountTable._create()
 *                         key_ct.setitem(_XYL, xyl_count)
 */
        }
      }

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":865
 *                 )
 *                 core_shifts.append(shift)
 *             if xyl_count > 0:             # <<<<<<<<<<<<<<
 *                 # TODO: Handle modified Hexose residues here too.
 *                 for hexose_count in range(1, 3):
 */
    }
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":896
 *                         )
 *                         core_shifts.append(shift)
 *         return core_shifts             # <<<<<<<<<<<<<<
 * 
 *     def stub_fragments(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_core_shifts);
  __pyx_r = __pyx_v_core_shifts;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":836
 *                 is_extended=site.is_extended)
 * 
 *     cpdef list gag_linker_composition_fragments(self, glycan, long core_count=1, long iteration_count=0):             # <<<<<<<<<<<<<<
 *         cdef:
 *             long xyl_in_aggregate
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.gag_linker_composition_fragments", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_core_shifts);
  __Pyx_XDECREF((PyObject *)__pyx_v_key_ct);
  __Pyx_XDECREF((PyObject *)__pyx_v_shift);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_30gag_linker_composition_fragments(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_29gag_linker_composition_fragments[] = "StubGlycopeptideStrategy.gag_linker_composition_fragments(self, glycan, long core_count=1, long iteration_count=0) -> list";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_30gag_linker_composition_fragments(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_glycan = 0;
  long __pyx_v_core_count;
  long __pyx_v_iteration_count;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("gag_linker_composition_fragments (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_glycan,&__pyx_n_s_core_count,&__pyx_n_s_iteration_count,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_glycan)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_core_count);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_iteration_count);
          if (value) { values[2] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "gag_linker_composition_fragments") < 0)) __PYX_ERR(0, 836, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_glycan = values[0];
    if (values[1]) {
      __pyx_v_core_count = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v_core_count == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 836, __pyx_L3_error)
    } else {
      __pyx_v_core_count = ((long)1);
    }
    if (values[2]) {
      __pyx_v_iteration_count = __Pyx_PyInt_As_long(values[2]); if (unlikely((__pyx_v_iteration_count == (long)-1) && PyErr_Occurred())) __PYX_ERR(0, 836, __pyx_L3_error)
    } else {
      __pyx_v_iteration_count = ((long)0);
    }
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("gag_linker_composition_fragments", 0, 1, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 836, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.gag_linker_composition_fragments", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_29gag_linker_composition_fragments(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self), __pyx_v_glycan, __pyx_v_core_count, __pyx_v_iteration_count);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_29gag_linker_composition_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_glycan, long __pyx_v_core_count, long __pyx_v_iteration_count) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_gag_linker_composition_fragments __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("gag_linker_composition_fragments", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 2;
  __pyx_t_2.core_count = __pyx_v_core_count;
  __pyx_t_2.iteration_count = __pyx_v_iteration_count;
  __pyx_t_1 = __pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy->gag_linker_composition_fragments(__pyx_v_self, __pyx_v_glycan, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 836, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.gag_linker_composition_fragments", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":898
 *         return core_shifts
 * 
 *     def stub_fragments(self):             # <<<<<<<<<<<<<<
 *         n_glycan = self.count_glycosylation_type(GlycosylationType_n_linked) > 0
 *         o_glycan = self.count_glycosylation_type(GlycosylationType_o_linked) > 0
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_32stub_fragments(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_31stub_fragments[] = "StubGlycopeptideStrategy.stub_fragments(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_32stub_fragments(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("stub_fragments (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_31stub_fragments(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_14stub_fragments_2generator3(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":914
 *             if (<GlycosylationManager>self.peptide.glycosylation_manager).get_size() > 0:
 *                 raise ValueError("Unknown Glycan Class Detected")
 *         return (a for a in [])             # <<<<<<<<<<<<<<
 */

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_14stub_fragments_genexpr(CYTHON_UNUSED PyObject *__pyx_self) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("genexpr", 0);
  __pyx_cur_scope = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr *)__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr(__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 914, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_14stub_fragments_2generator3, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_genexpr, __pyx_n_s_stub_fragments_locals_genexpr, __pyx_n_s_glycopeptidepy__c_structure_frag); if (unlikely(!gen)) __PYX_ERR(0, 914, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.stub_fragments.genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_14stub_fragments_2generator3(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  PyObject *__pyx_r = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("genexpr", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 914, __pyx_L1_error)

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("genexpr", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":898
 *         return core_shifts
 * 
 *     def stub_fragments(self):             # <<<<<<<<<<<<<<
 *         n_glycan = self.count_glycosylation_type(GlycosylationType_n_linked) > 0
 *         o_glycan = self.count_glycosylation_type(GlycosylationType_o_linked) > 0
 */

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_31stub_fragments(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self) {
  PyObject *__pyx_v_n_glycan = NULL;
  PyObject *__pyx_v_o_glycan = NULL;
  PyObject *__pyx_v_gag_linker = NULL;
  PyObject *__pyx_gb_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_14stub_fragments_2generator3 = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("stub_fragments", 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":899
 * 
 *     def stub_fragments(self):
 *         n_glycan = self.count_glycosylation_type(GlycosylationType_n_linked) > 0             # <<<<<<<<<<<<<<
 *         o_glycan = self.count_glycosylation_type(GlycosylationType_o_linked) > 0
 *         gag_linker = self.count_glycosylation_type(GlycosylationType_glycosaminoglycan) > 0
 */
  __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_n_linked;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong((((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.count_glycosylation_type(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_v_self), __pyx_t_1, 0) > 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 899, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_n_glycan = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":900
 *     def stub_fragments(self):
 *         n_glycan = self.count_glycosylation_type(GlycosylationType_n_linked) > 0
 *         o_glycan = self.count_glycosylation_type(GlycosylationType_o_linked) > 0             # <<<<<<<<<<<<<<
 *         gag_linker = self.count_glycosylation_type(GlycosylationType_glycosaminoglycan) > 0
 *         if (n_glycan + o_glycan + gag_linker) > 2:
 */
  __pyx_t_2 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_o_linked;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyBool_FromLong((((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.count_glycosylation_type(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_v_self), __pyx_t_2, 0) > 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 900, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_o_glycan = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":901
 *         n_glycan = self.count_glycosylation_type(GlycosylationType_n_linked) > 0
 *         o_glycan = self.count_glycosylation_type(GlycosylationType_o_linked) > 0
 *         gag_linker = self.count_glycosylation_type(GlycosylationType_glycosaminoglycan) > 0             # <<<<<<<<<<<<<<
 *         if (n_glycan + o_glycan + gag_linker) > 2:
 *             raise ValueError(
 */
  __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_glycosaminoglycan;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong((((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self->__pyx_base.__pyx_base.__pyx_vtab)->__pyx_base.count_glycosylation_type(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_v_self), __pyx_t_1, 0) > 0)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 901, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_gag_linker = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":902
 *         o_glycan = self.count_glycosylation_type(GlycosylationType_o_linked) > 0
 *         gag_linker = self.count_glycosylation_type(GlycosylationType_glycosaminoglycan) > 0
 *         if (n_glycan + o_glycan + gag_linker) > 2:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "Does not support mixed-type glycan fragmentation (yet)")
 */
  __pyx_t_2 = PyNumber_Add(__pyx_v_n_glycan, __pyx_v_o_glycan); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 902, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyNumber_Add(__pyx_t_2, __pyx_v_gag_linker); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 902, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = PyObject_RichCompare(__pyx_t_1, __pyx_int_2, Py_GT); __Pyx_XGOTREF(__pyx_t_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 902, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 902, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(__pyx_t_3)) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":903
 *         gag_linker = self.count_glycosylation_type(GlycosylationType_glycosaminoglycan) > 0
 *         if (n_glycan + o_glycan + gag_linker) > 2:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "Does not support mixed-type glycan fragmentation (yet)")
 *         if n_glycan:
 */
    __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__2, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 903, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 903, __pyx_L1_error)

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":902
 *         o_glycan = self.count_glycosylation_type(GlycosylationType_o_linked) > 0
 *         gag_linker = self.count_glycosylation_type(GlycosylationType_glycosaminoglycan) > 0
 *         if (n_glycan + o_glycan + gag_linker) > 2:             # <<<<<<<<<<<<<<
 *             raise ValueError(
 *                 "Does not support mixed-type glycan fragmentation (yet)")
 */
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":905
 *             raise ValueError(
 *                 "Does not support mixed-type glycan fragmentation (yet)")
 *         if n_glycan:             # <<<<<<<<<<<<<<
 *             return self.n_glycan_stub_fragments()
 *         elif o_glycan:
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_n_glycan); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 905, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":906
 *                 "Does not support mixed-type glycan fragmentation (yet)")
 *         if n_glycan:
 *             return self.n_glycan_stub_fragments()             # <<<<<<<<<<<<<<
 *         elif o_glycan:
 *             return self.o_glycan_stub_fragments()
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_n_glycan_stub_fragments); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 906, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 906, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":905
 *             raise ValueError(
 *                 "Does not support mixed-type glycan fragmentation (yet)")
 *         if n_glycan:             # <<<<<<<<<<<<<<
 *             return self.n_glycan_stub_fragments()
 *         elif o_glycan:
 */
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":907
 *         if n_glycan:
 *             return self.n_glycan_stub_fragments()
 *         elif o_glycan:             # <<<<<<<<<<<<<<
 *             return self.o_glycan_stub_fragments()
 *         elif gag_linker:
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_o_glycan); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 907, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":908
 *             return self.n_glycan_stub_fragments()
 *         elif o_glycan:
 *             return self.o_glycan_stub_fragments()             # <<<<<<<<<<<<<<
 *         elif gag_linker:
 *             return self.gag_linker_stub_fragments()
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_o_glycan_stub_fragments); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 908, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 908, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":907
 *         if n_glycan:
 *             return self.n_glycan_stub_fragments()
 *         elif o_glycan:             # <<<<<<<<<<<<<<
 *             return self.o_glycan_stub_fragments()
 *         elif gag_linker:
 */
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":909
 *         elif o_glycan:
 *             return self.o_glycan_stub_fragments()
 *         elif gag_linker:             # <<<<<<<<<<<<<<
 *             return self.gag_linker_stub_fragments()
 *         else:
 */
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_v_gag_linker); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 909, __pyx_L1_error)
  if (__pyx_t_3) {

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":910
 *             return self.o_glycan_stub_fragments()
 *         elif gag_linker:
 *             return self.gag_linker_stub_fragments()             # <<<<<<<<<<<<<<
 *         else:
 *             if (<GlycosylationManager>self.peptide.glycosylation_manager).get_size() > 0:
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_gag_linker_stub_fragments); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 910, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 910, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":909
 *         elif o_glycan:
 *             return self.o_glycan_stub_fragments()
 *         elif gag_linker:             # <<<<<<<<<<<<<<
 *             return self.gag_linker_stub_fragments()
 *         else:
 */
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":912
 *             return self.gag_linker_stub_fragments()
 *         else:
 *             if (<GlycosylationManager>self.peptide.glycosylation_manager).get_size() > 0:             # <<<<<<<<<<<<<<
 *                 raise ValueError("Unknown Glycan Class Detected")
 *         return (a for a in [])
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.peptide), __pyx_n_s_glycosylation_manager); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 912, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = ((((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *)((struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *)__pyx_t_2)->__pyx_vtab)->get_size(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *)__pyx_t_2)) > 0) != 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(__pyx_t_3)) {

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":913
 *         else:
 *             if (<GlycosylationManager>self.peptide.glycosylation_manager).get_size() > 0:
 *                 raise ValueError("Unknown Glycan Class Detected")             # <<<<<<<<<<<<<<
 *         return (a for a in [])
 */
      __pyx_t_2 = __Pyx_PyObject_Call(__pyx_builtin_ValueError, __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 913, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_Raise(__pyx_t_2, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __PYX_ERR(0, 913, __pyx_L1_error)

      /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":912
 *             return self.gag_linker_stub_fragments()
 *         else:
 *             if (<GlycosylationManager>self.peptide.glycosylation_manager).get_size() > 0:             # <<<<<<<<<<<<<<
 *                 raise ValueError("Unknown Glycan Class Detected")
 *         return (a for a in [])
 */
    }
  }

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":914
 *             if (<GlycosylationManager>self.peptide.glycosylation_manager).get_size() > 0:
 *                 raise ValueError("Unknown Glycan Class Detected")
 *         return (a for a in [])             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_14stub_fragments_genexpr(NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 914, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":898
 *         return core_shifts
 * 
 *     def stub_fragments(self):             # <<<<<<<<<<<<<<
 *         n_glycan = self.count_glycosylation_type(GlycosylationType_n_linked) > 0
 *         o_glycan = self.count_glycosylation_type(GlycosylationType_o_linked) > 0
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.stub_fragments", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_n_glycan);
  __Pyx_XDECREF(__pyx_v_o_glycan);
  __Pyx_XDECREF(__pyx_v_gag_linker);
  __Pyx_XDECREF(__pyx_gb_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_14stub_fragments_2generator3);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pxd":35
 * cdef class StubGlycopeptideStrategy(GlycanCompositionFragmentStrategyBase):
 *     cdef:
 *         public bint extended             # <<<<<<<<<<<<<<
 *         public bint extended_fucosylation
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_8extended_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_8extended_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_8extended___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_8extended___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->extended); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.extended.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_8extended_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_8extended_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_8extended_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_8extended_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 35, __pyx_L1_error)
  __pyx_v_self->extended = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.extended.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pxd":36
 *     cdef:
 *         public bint extended
 *         public bint extended_fucosylation             # <<<<<<<<<<<<<<
 * 
 *     cpdef GlycanCompositionFragment fucosylate_increment(self, GlycanCompositionFragment shift)
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_21extended_fucosylation_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_21extended_fucosylation_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_21extended_fucosylation___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_21extended_fucosylation___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->extended_fucosylation); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.extended_fucosylation.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_21extended_fucosylation_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_21extended_fucosylation_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_21extended_fucosylation_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_21extended_fucosylation_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_v_value); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(2, 36, __pyx_L1_error)
  __pyx_v_self->extended_fucosylation = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.extended_fucosylation.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_34__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_33__reduce_cython__[] = "StubGlycopeptideStrategy.__reduce_cython__(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_34__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_33__reduce_cython__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_33__reduce_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._generator, self._use_query, self.compute_compositions, self.extended, self.extended_fucosylation, self.peptide)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base._use_query); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyBool_FromLong(__pyx_v_self->__pyx_base.__pyx_base.compute_compositions); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_self->extended); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_self->extended_fucosylation); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = PyTuple_New(6); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_INCREF(__pyx_v_self->__pyx_base._generator);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._generator);
  PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v_self->__pyx_base._generator);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_5, 3, __pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_4);
  PyTuple_SET_ITEM(__pyx_t_5, 4, __pyx_t_4);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.peptide));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.peptide));
  PyTuple_SET_ITEM(__pyx_t_5, 5, ((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.peptide));
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_t_3 = 0;
  __pyx_t_4 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_5);
  __pyx_t_5 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._generator, self._use_query, self.compute_compositions, self.extended, self.extended_fucosylation, self.peptide)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_5 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __pyx_v__dict = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "(tree fragment)":7
 *     state = (self._generator, self._use_query, self.compute_compositions, self.extended, self.extended_fucosylation, self.peptide)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_6 = (__pyx_v__dict != Py_None);
  __pyx_t_7 = (__pyx_t_6 != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_5 = PyTuple_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_5, 0, __pyx_v__dict);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._generator is not None or self.peptide is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._generator, self._use_query, self.compute_compositions, self.extended, self.extended_fucosylation, self.peptide)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._generator is not None or self.peptide is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_StubGlycopeptideStrategy, (type(self), 0xf5d0479, None), state
 */
  /*else*/ {
    __pyx_t_6 = (__pyx_v_self->__pyx_base._generator != Py_None);
    __pyx_t_8 = (__pyx_t_6 != 0);
    if (!__pyx_t_8) {
    } else {
      __pyx_t_7 = __pyx_t_8;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_8 = (((PyObject *)__pyx_v_self->__pyx_base.__pyx_base.peptide) != Py_None);
    __pyx_t_6 = (__pyx_t_8 != 0);
    __pyx_t_7 = __pyx_t_6;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_7;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._generator is not None or self.peptide is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_StubGlycopeptideStrategy, (type(self), 0xf5d0479, None), state
 *     else:
 */
  __pyx_t_7 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_7) {

    /* "(tree fragment)":13
 *         use_setstate = self._generator is not None or self.peptide is not None
 *     if use_setstate:
 *         return __pyx_unpickle_StubGlycopeptideStrategy, (type(self), 0xf5d0479, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_StubGlycopeptideStrategy, (type(self), 0xf5d0479, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_4, __pyx_n_s_pyx_unpickle_StubGlycopeptideS); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_257754233);
    __Pyx_GIVEREF(__pyx_int_257754233);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_257754233);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_5, 2, Py_None);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_4);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_5);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_4 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._generator is not None or self.peptide is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_StubGlycopeptideStrategy, (type(self), 0xf5d0479, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_StubGlycopeptideStrategy, (type(self), 0xf5d0479, None), state
 *     else:
 *         return __pyx_unpickle_StubGlycopeptideStrategy, (type(self), 0xf5d0479, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_StubGlycopeptideStrategy__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_pyx_unpickle_StubGlycopeptideS); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_5 = PyTuple_New(3); if (unlikely(!__pyx_t_5)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_5, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_257754233);
    __Pyx_GIVEREF(__pyx_int_257754233);
    PyTuple_SET_ITEM(__pyx_t_5, 1, __pyx_int_257754233);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_5, 2, __pyx_v_state);
    __pyx_t_4 = PyTuple_New(2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3);
    __Pyx_GIVEREF(__pyx_t_5);
    PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
    __pyx_t_3 = 0;
    __pyx_t_5 = 0;
    __pyx_r = __pyx_t_4;
    __pyx_t_4 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_StubGlycopeptideStrategy, (type(self), 0xf5d0479, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_StubGlycopeptideStrategy__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_36__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_35__setstate_cython__[] = "StubGlycopeptideStrategy.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_36__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_35__setstate_cython__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_35__setstate_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_StubGlycopeptideStrategy, (type(self), 0xf5d0479, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_StubGlycopeptideStrategy__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle_StubGlycopeptideStrategy__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_StubGlycopeptideStrategy, (type(self), 0xf5d0479, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_StubGlycopeptideStrategy__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle__CompositionTree(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_1__pyx_unpickle__CompositionTree(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle__CompositionTree[] = "__pyx_unpickle__CompositionTree(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_1__pyx_unpickle__CompositionTree = {"__pyx_unpickle__CompositionTree", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_1__pyx_unpickle__CompositionTree, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle__CompositionTree};
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_1__pyx_unpickle__CompositionTree(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle__CompositionTree (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__CompositionTree", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__CompositionTree", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle__CompositionTree") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__CompositionTree", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.__pyx_unpickle__CompositionTree", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle__CompositionTree(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle__CompositionTree(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle__CompositionTree", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x63a9f0e:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x63a9f0e = (root))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x63a9f0e) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x63a9f0e:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x63a9f0e = (root))" % __pyx_checksum)
 *     __pyx_result = _CompositionTree.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x63a9f0e:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x63a9f0e = (root))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = _CompositionTree.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x63, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x63a9f0e:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x63a9f0e = (root))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x63a9f0e = (root))" % __pyx_checksum)
 *     __pyx_result = _CompositionTree.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle__CompositionTree__set_state(<_CompositionTree> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x63a9f0e = (root))" % __pyx_checksum)
 *     __pyx_result = _CompositionTree.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle__CompositionTree__set_state(<_CompositionTree> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = _CompositionTree.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle__CompositionTree__set_state(<_CompositionTree> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle__CompositionTree__set_state(_CompositionTree __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle__CompositionTree__set_state(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x63a9f0e = (root))" % __pyx_checksum)
 *     __pyx_result = _CompositionTree.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle__CompositionTree__set_state(<_CompositionTree> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle__CompositionTree__set_state(<_CompositionTree> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle__CompositionTree__set_state(_CompositionTree __pyx_result, tuple __pyx_state):
 *     __pyx_result.root = __pyx_state[0]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle__CompositionTree(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.__pyx_unpickle__CompositionTree", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle__CompositionTree__set_state(<_CompositionTree> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle__CompositionTree__set_state(_CompositionTree __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.root = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle__CompositionTree__set_state(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle__CompositionTree__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle__CompositionTree__set_state(_CompositionTree __pyx_result, tuple __pyx_state):
 *     __pyx_result.root = __pyx_state[0]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment__NameTree))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->root);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->root));
  __pyx_v___pyx_result->root = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment__NameTree *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle__CompositionTree__set_state(_CompositionTree __pyx_result, tuple __pyx_state):
 *     __pyx_result.root = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 1) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result.root = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[1])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle__CompositionTree__set_state(_CompositionTree __pyx_result, tuple __pyx_state):
 *     __pyx_result.root = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[1])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle__CompositionTree__set_state(<_CompositionTree> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle__CompositionTree__set_state(_CompositionTree __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.root = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.__pyx_unpickle__CompositionTree__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_GlycanCompositionFragmentStrategyBase(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_3__pyx_unpickle_GlycanCompositionFragmentStrategyBase(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_2__pyx_unpickle_GlycanCompositionFragmentStrategyBase[] = "__pyx_unpickle_GlycanCompositionFragmentStrategyBase(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_3__pyx_unpickle_GlycanCompositionFragmentStrategyBase = {"__pyx_unpickle_GlycanCompositionFragmentStrategyBase", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_3__pyx_unpickle_GlycanCompositionFragmentStrategyBase, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_2__pyx_unpickle_GlycanCompositionFragmentStrategyBase};
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_3__pyx_unpickle_GlycanCompositionFragmentStrategyBase(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_GlycanCompositionFragmentStrategyBase (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_GlycanCompositionFragmentStrategyBase", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_GlycanCompositionFragmentStrategyBase", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_GlycanCompositionFragmentStrategyBase") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_GlycanCompositionFragmentStrategyBase", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.__pyx_unpickle_GlycanCompositionFragmentStrategyBase", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_2__pyx_unpickle_GlycanCompositionFragmentStrategyBase(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_2__pyx_unpickle_GlycanCompositionFragmentStrategyBase(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_GlycanCompositionFragmentStrategyBase", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x58be74f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x58be74f = (_generator, _use_query, compute_compositions, peptide))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x58be74f) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x58be74f:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x58be74f = (_generator, _use_query, compute_compositions, peptide))" % __pyx_checksum)
 *     __pyx_result = GlycanCompositionFragmentStrategyBase.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x58be74f:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x58be74f = (_generator, _use_query, compute_compositions, peptide))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = GlycanCompositionFragmentStrategyBase.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x58, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x58be74f:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x58be74f = (_generator, _use_query, compute_compositions, peptide))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x58be74f = (_generator, _use_query, compute_compositions, peptide))" % __pyx_checksum)
 *     __pyx_result = GlycanCompositionFragmentStrategyBase.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(<GlycanCompositionFragmentStrategyBase> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x58be74f = (_generator, _use_query, compute_compositions, peptide))" % __pyx_checksum)
 *     __pyx_result = GlycanCompositionFragmentStrategyBase.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(<GlycanCompositionFragmentStrategyBase> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = GlycanCompositionFragmentStrategyBase.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(<GlycanCompositionFragmentStrategyBase> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(GlycanCompositionFragmentStrategyBase __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x58be74f = (_generator, _use_query, compute_compositions, peptide))" % __pyx_checksum)
 *     __pyx_result = GlycanCompositionFragmentStrategyBase.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(<GlycanCompositionFragmentStrategyBase> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(<GlycanCompositionFragmentStrategyBase> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(GlycanCompositionFragmentStrategyBase __pyx_result, tuple __pyx_state):
 *     __pyx_result._generator = __pyx_state[0]; __pyx_result._use_query = __pyx_state[1]; __pyx_result.compute_compositions = __pyx_state[2]; __pyx_result.peptide = __pyx_state[3]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_GlycanCompositionFragmentStrategyBase(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.__pyx_unpickle_GlycanCompositionFragmentStrategyBase", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(<GlycanCompositionFragmentStrategyBase> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(GlycanCompositionFragmentStrategyBase __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._generator = __pyx_state[0]; __pyx_result._use_query = __pyx_state[1]; __pyx_result.compute_compositions = __pyx_state[2]; __pyx_result.peptide = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(GlycanCompositionFragmentStrategyBase __pyx_result, tuple __pyx_state):
 *     __pyx_result._generator = __pyx_state[0]; __pyx_result._use_query = __pyx_state[1]; __pyx_result.compute_compositions = __pyx_state[2]; __pyx_result.peptide = __pyx_state[3]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[4])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_generator);
  __Pyx_DECREF(__pyx_v___pyx_result->_generator);
  __pyx_v___pyx_result->_generator = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_use_query = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.compute_compositions = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.peptide);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.peptide));
  __pyx_v___pyx_result->__pyx_base.peptide = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(GlycanCompositionFragmentStrategyBase __pyx_result, tuple __pyx_state):
 *     __pyx_result._generator = __pyx_state[0]; __pyx_result._use_query = __pyx_state[1]; __pyx_result.compute_compositions = __pyx_state[2]; __pyx_result.peptide = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[4])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 4) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._generator = __pyx_state[0]; __pyx_result._use_query = __pyx_state[1]; __pyx_result.compute_compositions = __pyx_state[2]; __pyx_result.peptide = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[4])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(GlycanCompositionFragmentStrategyBase __pyx_result, tuple __pyx_state):
 *     __pyx_result._generator = __pyx_state[0]; __pyx_result._use_query = __pyx_state[1]; __pyx_result.compute_compositions = __pyx_state[2]; __pyx_result.peptide = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[4])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(<GlycanCompositionFragmentStrategyBase> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state(GlycanCompositionFragmentStrategyBase __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._generator = __pyx_state[0]; __pyx_result._use_query = __pyx_state[1]; __pyx_result.compute_compositions = __pyx_state[2]; __pyx_result.peptide = __pyx_state[3]
 *     if len(__pyx_state) > 4 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.__pyx_unpickle_GlycanCompositionFragmentStrategyBase__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_GlycanCompositionFragment(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_5__pyx_unpickle_GlycanCompositionFragment(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_4__pyx_unpickle_GlycanCompositionFragment[] = "__pyx_unpickle_GlycanCompositionFragment(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_5__pyx_unpickle_GlycanCompositionFragment = {"__pyx_unpickle_GlycanCompositionFragment", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_5__pyx_unpickle_GlycanCompositionFragment, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_4__pyx_unpickle_GlycanCompositionFragment};
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_5__pyx_unpickle_GlycanCompositionFragment(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_GlycanCompositionFragment (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_GlycanCompositionFragment", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_GlycanCompositionFragment", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_GlycanCompositionFragment") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_GlycanCompositionFragment", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.__pyx_unpickle_GlycanCompositionFragment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_4__pyx_unpickle_GlycanCompositionFragment(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_4__pyx_unpickle_GlycanCompositionFragment(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_GlycanCompositionFragment", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x18c151e:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x18c151e = (_hash_key, composition, is_extended, key, mass))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x18c151e) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x18c151e:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x18c151e = (_hash_key, composition, is_extended, key, mass))" % __pyx_checksum)
 *     __pyx_result = GlycanCompositionFragment.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x18c151e:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x18c151e = (_hash_key, composition, is_extended, key, mass))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = GlycanCompositionFragment.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x18, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x18c151e:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x18c151e = (_hash_key, composition, is_extended, key, mass))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x18c151e = (_hash_key, composition, is_extended, key, mass))" % __pyx_checksum)
 *     __pyx_result = GlycanCompositionFragment.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_GlycanCompositionFragment__set_state(<GlycanCompositionFragment> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x18c151e = (_hash_key, composition, is_extended, key, mass))" % __pyx_checksum)
 *     __pyx_result = GlycanCompositionFragment.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_GlycanCompositionFragment__set_state(<GlycanCompositionFragment> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = GlycanCompositionFragment.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_GlycanCompositionFragment__set_state(<GlycanCompositionFragment> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_GlycanCompositionFragment__set_state(GlycanCompositionFragment __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle_GlycanCompositionFragment__set_state(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x18c151e = (_hash_key, composition, is_extended, key, mass))" % __pyx_checksum)
 *     __pyx_result = GlycanCompositionFragment.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_GlycanCompositionFragment__set_state(<GlycanCompositionFragment> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_GlycanCompositionFragment__set_state(<GlycanCompositionFragment> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_GlycanCompositionFragment__set_state(GlycanCompositionFragment __pyx_result, tuple __pyx_state):
 *     __pyx_result._hash_key = __pyx_state[0]; __pyx_result.composition = __pyx_state[1]; __pyx_result.is_extended = __pyx_state[2]; __pyx_result.key = __pyx_state[3]; __pyx_result.mass = __pyx_state[4]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_GlycanCompositionFragment(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.__pyx_unpickle_GlycanCompositionFragment", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_GlycanCompositionFragment__set_state(<GlycanCompositionFragment> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_GlycanCompositionFragment__set_state(GlycanCompositionFragment __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._hash_key = __pyx_state[0]; __pyx_result.composition = __pyx_state[1]; __pyx_result.is_extended = __pyx_state[2]; __pyx_result.key = __pyx_state[3]; __pyx_result.mass = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle_GlycanCompositionFragment__set_state(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_hash_t __pyx_t_2;
  int __pyx_t_3;
  double __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_GlycanCompositionFragment__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_GlycanCompositionFragment__set_state(GlycanCompositionFragment __pyx_result, tuple __pyx_state):
 *     __pyx_result._hash_key = __pyx_state[0]; __pyx_result.composition = __pyx_state[1]; __pyx_result.is_extended = __pyx_state[2]; __pyx_result.key = __pyx_state[3]; __pyx_result.mass = __pyx_state[4]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[5])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_AsHash_t(__pyx_t_1); if (unlikely((__pyx_t_2 == (Py_hash_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_hash_key = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->composition);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->composition));
  __pyx_v___pyx_result->composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_3 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->is_extended = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_14glycopeptidepy_2_c_11count_table_CountTable))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->key);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->key));
  __pyx_v___pyx_result->key = ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_4 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->mass = __pyx_t_4;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_GlycanCompositionFragment__set_state(GlycanCompositionFragment __pyx_result, tuple __pyx_state):
 *     __pyx_result._hash_key = __pyx_state[0]; __pyx_result.composition = __pyx_state[1]; __pyx_result.is_extended = __pyx_state[2]; __pyx_result.key = __pyx_state[3]; __pyx_result.mass = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[5])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 5) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_3 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_3 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._hash_key = __pyx_state[0]; __pyx_result.composition = __pyx_state[1]; __pyx_result.is_extended = __pyx_state[2]; __pyx_result.key = __pyx_state[3]; __pyx_result.mass = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[5])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_GlycanCompositionFragment__set_state(GlycanCompositionFragment __pyx_result, tuple __pyx_state):
 *     __pyx_result._hash_key = __pyx_state[0]; __pyx_result.composition = __pyx_state[1]; __pyx_result.is_extended = __pyx_state[2]; __pyx_result.key = __pyx_state[3]; __pyx_result.mass = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[5])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_GlycanCompositionFragment__set_state(<GlycanCompositionFragment> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_GlycanCompositionFragment__set_state(GlycanCompositionFragment __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._hash_key = __pyx_state[0]; __pyx_result.composition = __pyx_state[1]; __pyx_result.is_extended = __pyx_state[2]; __pyx_result.key = __pyx_state[3]; __pyx_result.mass = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.__pyx_unpickle_GlycanCompositionFragment__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_StubGlycopeptideStrategy(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_7__pyx_unpickle_StubGlycopeptideStrategy(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_6__pyx_unpickle_StubGlycopeptideStrategy[] = "__pyx_unpickle_StubGlycopeptideStrategy(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_7__pyx_unpickle_StubGlycopeptideStrategy = {"__pyx_unpickle_StubGlycopeptideStrategy", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_7__pyx_unpickle_StubGlycopeptideStrategy, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_6__pyx_unpickle_StubGlycopeptideStrategy};
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_7__pyx_unpickle_StubGlycopeptideStrategy(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_StubGlycopeptideStrategy (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_StubGlycopeptideStrategy", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_StubGlycopeptideStrategy", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_StubGlycopeptideStrategy") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_StubGlycopeptideStrategy", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.__pyx_unpickle_StubGlycopeptideStrategy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_6__pyx_unpickle_StubGlycopeptideStrategy(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_6__pyx_unpickle_StubGlycopeptideStrategy(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_StubGlycopeptideStrategy", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf5d0479:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf5d0479 = (_generator, _use_query, compute_compositions, extended, extended_fucosylation, peptide))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0xf5d0479) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf5d0479:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf5d0479 = (_generator, _use_query, compute_compositions, extended, extended_fucosylation, peptide))" % __pyx_checksum)
 *     __pyx_result = StubGlycopeptideStrategy.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0xf5d0479:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf5d0479 = (_generator, _use_query, compute_compositions, extended, extended_fucosylation, peptide))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = StubGlycopeptideStrategy.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0xf5, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0xf5d0479:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf5d0479 = (_generator, _use_query, compute_compositions, extended, extended_fucosylation, peptide))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf5d0479 = (_generator, _use_query, compute_compositions, extended, extended_fucosylation, peptide))" % __pyx_checksum)
 *     __pyx_result = StubGlycopeptideStrategy.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_StubGlycopeptideStrategy__set_state(<StubGlycopeptideStrategy> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf5d0479 = (_generator, _use_query, compute_compositions, extended, extended_fucosylation, peptide))" % __pyx_checksum)
 *     __pyx_result = StubGlycopeptideStrategy.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_StubGlycopeptideStrategy__set_state(<StubGlycopeptideStrategy> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = StubGlycopeptideStrategy.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_StubGlycopeptideStrategy__set_state(<StubGlycopeptideStrategy> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_StubGlycopeptideStrategy__set_state(StubGlycopeptideStrategy __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle_StubGlycopeptideStrategy__set_state(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0xf5d0479 = (_generator, _use_query, compute_compositions, extended, extended_fucosylation, peptide))" % __pyx_checksum)
 *     __pyx_result = StubGlycopeptideStrategy.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_StubGlycopeptideStrategy__set_state(<StubGlycopeptideStrategy> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_StubGlycopeptideStrategy__set_state(<StubGlycopeptideStrategy> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_StubGlycopeptideStrategy__set_state(StubGlycopeptideStrategy __pyx_result, tuple __pyx_state):
 *     __pyx_result._generator = __pyx_state[0]; __pyx_result._use_query = __pyx_state[1]; __pyx_result.compute_compositions = __pyx_state[2]; __pyx_result.extended = __pyx_state[3]; __pyx_result.extended_fucosylation = __pyx_state[4]; __pyx_result.peptide = __pyx_state[5]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_StubGlycopeptideStrategy(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.__pyx_unpickle_StubGlycopeptideStrategy", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_StubGlycopeptideStrategy__set_state(<StubGlycopeptideStrategy> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_StubGlycopeptideStrategy__set_state(StubGlycopeptideStrategy __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._generator = __pyx_state[0]; __pyx_result._use_query = __pyx_state[1]; __pyx_result.compute_compositions = __pyx_state[2]; __pyx_result.extended = __pyx_state[3]; __pyx_result.extended_fucosylation = __pyx_state[4]; __pyx_result.peptide = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_unpickle_StubGlycopeptideStrategy__set_state(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  Py_ssize_t __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_StubGlycopeptideStrategy__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_StubGlycopeptideStrategy__set_state(StubGlycopeptideStrategy __pyx_result, tuple __pyx_state):
 *     __pyx_result._generator = __pyx_state[0]; __pyx_result._use_query = __pyx_state[1]; __pyx_result.compute_compositions = __pyx_state[2]; __pyx_result.extended = __pyx_state[3]; __pyx_result.extended_fucosylation = __pyx_state[4]; __pyx_result.peptide = __pyx_state[5]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._generator);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._generator);
  __pyx_v___pyx_result->__pyx_base._generator = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base._use_query = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base.__pyx_base.compute_compositions = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->extended = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->extended_fucosylation = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.__pyx_base.peptide);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base.__pyx_base.peptide));
  __pyx_v___pyx_result->__pyx_base.__pyx_base.peptide = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_StubGlycopeptideStrategy__set_state(StubGlycopeptideStrategy __pyx_result, tuple __pyx_state):
 *     __pyx_result._generator = __pyx_state[0]; __pyx_result._use_query = __pyx_state[1]; __pyx_result.compute_compositions = __pyx_state[2]; __pyx_result.extended = __pyx_state[3]; __pyx_result.extended_fucosylation = __pyx_state[4]; __pyx_result.peptide = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_3 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_3 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_4 = ((__pyx_t_3 > 6) != 0);
  if (__pyx_t_4) {
  } else {
    __pyx_t_2 = __pyx_t_4;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_4 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = (__pyx_t_4 != 0);
  __pyx_t_2 = __pyx_t_5;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_2) {

    /* "(tree fragment)":14
 *     __pyx_result._generator = __pyx_state[0]; __pyx_result._use_query = __pyx_state[1]; __pyx_result.compute_compositions = __pyx_state[2]; __pyx_result.extended = __pyx_state[3]; __pyx_result.extended_fucosylation = __pyx_state[4]; __pyx_result.peptide = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])             # <<<<<<<<<<<<<<
 */
    __pyx_t_6 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_t_6, __pyx_n_s_update); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_6 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_8 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_7))) {
      __pyx_t_8 = PyMethod_GET_SELF(__pyx_t_7);
      if (likely(__pyx_t_8)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_7);
        __Pyx_INCREF(__pyx_t_8);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_7, function);
      }
    }
    __pyx_t_1 = (__pyx_t_8) ? __Pyx_PyObject_Call2Args(__pyx_t_7, __pyx_t_8, __pyx_t_6) : __Pyx_PyObject_CallOneArg(__pyx_t_7, __pyx_t_6);
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_StubGlycopeptideStrategy__set_state(StubGlycopeptideStrategy __pyx_result, tuple __pyx_state):
 *     __pyx_result._generator = __pyx_state[0]; __pyx_result._use_query = __pyx_state[1]; __pyx_result.compute_compositions = __pyx_state[2]; __pyx_result.extended = __pyx_state[3]; __pyx_result.extended_fucosylation = __pyx_state[4]; __pyx_result.peptide = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_StubGlycopeptideStrategy__set_state(<StubGlycopeptideStrategy> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_StubGlycopeptideStrategy__set_state(StubGlycopeptideStrategy __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._generator = __pyx_state[0]; __pyx_result._use_query = __pyx_state[1]; __pyx_result.compute_compositions = __pyx_state[2]; __pyx_result.extended = __pyx_state[3]; __pyx_result.extended_fucosylation = __pyx_state[4]; __pyx_result.peptide = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.fragmentation_strategy.glycan.__pyx_unpickle_StubGlycopeptideStrategy__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment;

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *__pyx_freelist_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment[10000];
static int __pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment = 0;

static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *p;
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment)) & ((t->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    o = (PyObject*)__pyx_freelist_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment[--__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment];
    memset(o, 0, sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
      o = (*t->tp_alloc)(t, 0);
    } else {
      o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
    }
    if (unlikely(!o)) return 0;
  }
  p = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)o);
  p->__pyx_vtab = __pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment;
  p->composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  p->key = ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment(PyObject *o) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->composition);
  Py_CLEAR(p->key);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment < 10000) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment)) & ((Py_TYPE(o)->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    __pyx_freelist_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment[__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment++] = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)o;
  if (p->composition) {
    e = (*v)(((PyObject *)p->composition), a); if (e) return e;
  }
  if (p->key) {
    e = (*v)(((PyObject *)p->key), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *)o;
  tmp = ((PyObject*)p->composition);
  p->composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->key);
  p->key = ((struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
static PyObject *__pyx_sq_item_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11__setitem__(o, i, v);
  }
  else {
    PyErr_Format(PyExc_NotImplementedError,
      "Subscript deletion not supported by %.200s", Py_TYPE(o)->tp_name);
    return -1;
  }
}

static PyObject *__pyx_tp_richcompare_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment(PyObject *o1, PyObject *o2, int op) {
  switch (op) {
    case Py_EQ: {
      return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_7__eq__(o1, o2);
    }
    case Py_NE: {
      PyObject *ret;
      ret = __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_7__eq__(o1, o2);
      if (likely(ret && ret != Py_NotImplemented)) {
        int b = __Pyx_PyObject_IsTrue(ret); Py_DECREF(ret);
        if (unlikely(b < 0)) return NULL;
        ret = (b) ? Py_False : Py_True;
        Py_INCREF(ret);
      }
      return ret;
    }
    default: {
      return __Pyx_NewRef(Py_NotImplemented);
    }
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_mass(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_4mass_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_mass(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_4mass_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_composition(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_composition(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11composition_5__del__(o);
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_key(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_key(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3key_5__del__(o);
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_is_extended(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11is_extended_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_is_extended(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_11is_extended_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__hash_key(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_9_hash_key_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__hash_key(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_9_hash_key_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment[] = {
  {"_get_glycan_composition", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_5_get_glycan_composition, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_4_get_glycan_composition},
  {"copy", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_15copy, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_14copy},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_17__reduce_cython__, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_16__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_19__setstate_cython__, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_18__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment[] = {
  {(char *)"mass", __pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_mass, __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_mass, (char *)"mass: 'double'", 0},
  {(char *)"composition", __pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_composition, __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_composition, (char *)"composition: glypy.composition.ccomposition.CComposition", 0},
  {(char *)"key", __pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_key, __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_key, (char *)"key: glycopeptidepy._c.count_table.CountTable", 0},
  {(char *)"is_extended", __pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_is_extended, __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_is_extended, (char *)"is_extended: 'bool'", 0},
  {(char *)"_hash_key", __pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__hash_key, __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__hash_key, (char *)"_hash_key: 'Py_hash_t'", 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_GlycanCompositionFragment = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_GlycanCompositionFragment = {
  0, /*mp_length*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_9__getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment = {
  PyVarObject_HEAD_INIT(0, 0)
  "glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragment", /*tp_name*/
  sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_13__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_GlycanCompositionFragment, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_GlycanCompositionFragment, /*tp_as_mapping*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_3__hash__, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "GlycanCompositionFragment(mass, composition, key, is_extended=False)", /*tp_doc*/
  __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment, /*tp_traverse*/
  __pyx_tp_clear_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment, /*tp_clear*/
  __pyx_tp_richcompare_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase;

static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *p;
  PyObject *o = __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase*)__pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase;
  p->_generator = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase(PyObject *o) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->_generator);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase)) __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase);
}

static int __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)o;
  e = ((likely(__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase)) ? ((__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase->tp_traverse) ? __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase)); if (e) return e;
  if (p->_generator) {
    e = (*v)(p->_generator, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *)o;
  if (likely(__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase)) { if (__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase->tp_clear) __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase);
  tmp = ((PyObject*)p->_generator);
  p->_generator = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase__use_query(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_use_query_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase__use_query(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_use_query_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase__generator(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase__generator(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10_generator_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase[] = {
  {"glycan_composition", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_3glycan_composition, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_2glycan_composition},
  {"_guess_query_mode", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_5_guess_query_mode, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_4_guess_query_mode},
  {"count_glycosylation_type", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_7count_glycosylation_type, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_6count_glycosylation_type},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_9__reduce_cython__, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_11__setstate_cython__, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase[] = {
  {(char *)"_use_query", __pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase__use_query, __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase__use_query, (char *)"_use_query: 'bool'", 0},
  {(char *)"_generator", __pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase__generator, __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase__generator, (char *)"_generator: object", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase = {
  PyVarObject_HEAD_INIT(0, 0)
  "glycopeptidepy._c.structure.fragmentation_strategy.glycan.GlycanCompositionFragmentStrategyBase", /*tp_name*/
  sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "GlycanCompositionFragmentStrategyBase(peptide, use_query=False, *args, **kwargs)", /*tp_doc*/
  __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase, /*tp_traverse*/
  __pyx_tp_clear_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy;

static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *p;
  PyObject *o = __pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *)o);
  p->__pyx_base.__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase*)__pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy;
  return o;
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_extended(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_8extended_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_extended(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_8extended_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_extended_fucosylation(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_21extended_fucosylation_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_extended_fucosylation(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_21extended_fucosylation_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_specialmethod___pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_3__next__(PyObject *self, CYTHON_UNUSED PyObject *arg) {return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_3__next__(self);}

static PyMethodDef __pyx_methods_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy[] = {
  {"__next__", (PyCFunction)__pyx_specialmethod___pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_3__next__, METH_NOARGS|METH_COEXIST, 0},
  {"_validate_glycan_composition", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_5_validate_glycan_composition, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_4_validate_glycan_composition},
  {"fucosylate_increment", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_7fucosylate_increment, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_6fucosylate_increment},
  {"xylosylate_increment", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_9xylosylate_increment, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_8xylosylate_increment},
  {"fucosylate_extended", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_11fucosylate_extended, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_10fucosylate_extended},
  {"modified_increment", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_13modified_increment, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_12modified_increment},
  {"n_glycan_composition_fragments", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_15n_glycan_composition_fragments, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_14n_glycan_composition_fragments},
  {"_combinate_sites", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_17_combinate_sites, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_16_combinate_sites},
  {"n_glycan_stub_fragments", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_19n_glycan_stub_fragments, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_18n_glycan_stub_fragments},
  {"o_glycan_stub_fragments", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_22o_glycan_stub_fragments, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_21o_glycan_stub_fragments},
  {"o_glycan_composition_fragments", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_25o_glycan_composition_fragments, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_24o_glycan_composition_fragments},
  {"gag_linker_stub_fragments", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_27gag_linker_stub_fragments, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_26gag_linker_stub_fragments},
  {"gag_linker_composition_fragments", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_30gag_linker_composition_fragments, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_29gag_linker_composition_fragments},
  {"stub_fragments", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_32stub_fragments, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_31stub_fragments},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_34__reduce_cython__, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_33__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_36__setstate_cython__, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_35__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy[] = {
  {(char *)"extended", __pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_extended, __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_extended, (char *)"extended: 'bool'", 0},
  {(char *)"extended_fucosylation", __pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_extended_fucosylation, __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_extended_fucosylation, (char *)"extended_fucosylation: 'bool'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy = {
  PyVarObject_HEAD_INIT(0, 0)
  "glycopeptidepy._c.structure.fragmentation_strategy.glycan.StubGlycopeptideStrategy", /*tp_name*/
  sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "StubGlycopeptideStrategy(peptide, extended=True, use_query=False, extended_fucosylation=False, **kwargs)\nA fragmentation strategy that generates intact peptide + glycan Y fragments from\n    glycopeptides where the glycan structure is not fully known. When the structure is\n    known, :class:`CADFragmentationStrategy` should be used instead\n\n    Attributes\n    ----------\n    extended : bool\n        Whether to fragment beyond the conserved core\n    extended_fucosylation : bool\n        Whether to consider multiple ``Fuc`` residues per fragment rather than one per core\n    ", /*tp_doc*/
  __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase, /*tp_traverse*/
  __pyx_tp_clear_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_3__next__, /*tp_iternext*/
  __pyx_methods_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree;

static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *p;
  PyObject *o;
  o = (*t->tp_alloc)(t, 0);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *)o);
  p->__pyx_vtab = __pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree;
  p->root = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment__NameTree *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree(PyObject *o) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->root);
  (*Py_TYPE(o)->tp_free)(o);
}

static int __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *)o;
  if (p->root) {
    e = (*v)(((PyObject *)p->root), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *)o;
  tmp = ((PyObject*)p->root);
  p->root = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment__NameTree *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
static PyObject *__pyx_sq_item_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_root(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_root(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4root_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree[] = {
  {"build", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_5build, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_4build},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_7__reduce_cython__, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_6__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_9__setstate_cython__, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_8__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree[] = {
  {(char *)"root", __pyx_getprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_root, __pyx_setprop_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_root, (char *)"root: glycopeptidepy._c.structure.fragment._NameTree", 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence__CompositionTree = {
  0, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping__CompositionTree = {
  0, /*mp_length*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_3__getitem__, /*mp_subscript*/
  0, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree = {
  PyVarObject_HEAD_INIT(0, 0)
  "glycopeptidepy._c.structure.fragmentation_strategy.glycan._CompositionTree", /*tp_name*/
  sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence__CompositionTree, /*tp_as_sequence*/
  &__pyx_tp_as_mapping__CompositionTree, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "_CompositionTree(root=None)", /*tp_doc*/
  __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree, /*tp_traverse*/
  __pyx_tp_clear_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments *__pyx_freelist_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments[8];
static int __pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments = 0;

static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments)))) {
    o = (PyObject*)__pyx_freelist_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments[--__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments];
    memset(o, 0, sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments(PyObject *o) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_base_composition);
  Py_CLEAR(p->__pyx_v_combos);
  Py_CLEAR(p->__pyx_v_composition);
  Py_CLEAR(p->__pyx_v_core_shifts);
  Py_CLEAR(p->__pyx_v_glycan);
  Py_CLEAR(p->__pyx_v_glycosylation);
  Py_CLEAR(p->__pyx_v_name);
  Py_CLEAR(p->__pyx_v_name_key);
  Py_CLEAR(p->__pyx_v_per_site_shifts);
  Py_CLEAR(p->__pyx_v_result);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_site);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments)))) {
    __pyx_freelist_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments[__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments++] = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments *)o;
  if (p->__pyx_v_base_composition) {
    e = (*v)(((PyObject *)p->__pyx_v_base_composition), a); if (e) return e;
  }
  if (p->__pyx_v_combos) {
    e = (*v)(p->__pyx_v_combos, a); if (e) return e;
  }
  if (p->__pyx_v_composition) {
    e = (*v)(((PyObject *)p->__pyx_v_composition), a); if (e) return e;
  }
  if (p->__pyx_v_core_shifts) {
    e = (*v)(p->__pyx_v_core_shifts, a); if (e) return e;
  }
  if (p->__pyx_v_glycan) {
    e = (*v)(p->__pyx_v_glycan, a); if (e) return e;
  }
  if (p->__pyx_v_glycosylation) {
    e = (*v)(p->__pyx_v_glycosylation, a); if (e) return e;
  }
  if (p->__pyx_v_per_site_shifts) {
    e = (*v)(p->__pyx_v_per_site_shifts, a); if (e) return e;
  }
  if (p->__pyx_v_result) {
    e = (*v)(p->__pyx_v_result, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_v_site) {
    e = (*v)(((PyObject *)p->__pyx_v_site), a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments = {
  PyVarObject_HEAD_INIT(0, 0)
  "glycopeptidepy._c.structure.fragmentation_strategy.glycan.__pyx_scope_struct__n_glycan_stub_fragments", /*tp_name*/
  sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments *__pyx_freelist_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments[8];
static int __pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments = 0;

static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments)))) {
    o = (PyObject*)__pyx_freelist_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments[--__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments];
    memset(o, 0, sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments(PyObject *o) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_aggregate_glycosylation);
  Py_CLEAR(p->__pyx_v_base_composition);
  Py_CLEAR(p->__pyx_v_combos);
  Py_CLEAR(p->__pyx_v_composition);
  Py_CLEAR(p->__pyx_v_core_shifts);
  Py_CLEAR(p->__pyx_v_glycan);
  Py_CLEAR(p->__pyx_v_glycosylation);
  Py_CLEAR(p->__pyx_v_name);
  Py_CLEAR(p->__pyx_v_per_site_shifts);
  Py_CLEAR(p->__pyx_v_positions);
  Py_CLEAR(p->__pyx_v_result);
  Py_CLEAR(p->__pyx_v_seen);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_site);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments)))) {
    __pyx_freelist_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments[__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments++] = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments *)o;
  if (p->__pyx_v_aggregate_glycosylation) {
    e = (*v)(((PyObject *)p->__pyx_v_aggregate_glycosylation), a); if (e) return e;
  }
  if (p->__pyx_v_base_composition) {
    e = (*v)(((PyObject *)p->__pyx_v_base_composition), a); if (e) return e;
  }
  if (p->__pyx_v_combos) {
    e = (*v)(p->__pyx_v_combos, a); if (e) return e;
  }
  if (p->__pyx_v_composition) {
    e = (*v)(((PyObject *)p->__pyx_v_composition), a); if (e) return e;
  }
  if (p->__pyx_v_core_shifts) {
    e = (*v)(p->__pyx_v_core_shifts, a); if (e) return e;
  }
  if (p->__pyx_v_glycan) {
    e = (*v)(p->__pyx_v_glycan, a); if (e) return e;
  }
  if (p->__pyx_v_glycosylation) {
    e = (*v)(p->__pyx_v_glycosylation, a); if (e) return e;
  }
  if (p->__pyx_v_per_site_shifts) {
    e = (*v)(p->__pyx_v_per_site_shifts, a); if (e) return e;
  }
  if (p->__pyx_v_positions) {
    e = (*v)(p->__pyx_v_positions, a); if (e) return e;
  }
  if (p->__pyx_v_result) {
    e = (*v)(p->__pyx_v_result, a); if (e) return e;
  }
  if (p->__pyx_v_seen) {
    e = (*v)(p->__pyx_v_seen, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_v_site) {
    e = (*v)(((PyObject *)p->__pyx_v_site), a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments = {
  PyVarObject_HEAD_INIT(0, 0)
  "glycopeptidepy._c.structure.fragmentation_strategy.glycan.__pyx_scope_struct_1_o_glycan_stub_fragments", /*tp_name*/
  sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments *__pyx_freelist_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments[8];
static int __pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments = 0;

static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments)))) {
    o = (PyObject*)__pyx_freelist_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments[--__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments];
    memset(o, 0, sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments(PyObject *o) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_aggregate_glycosylation);
  Py_CLEAR(p->__pyx_v_base_composition);
  Py_CLEAR(p->__pyx_v_combos);
  Py_CLEAR(p->__pyx_v_composition);
  Py_CLEAR(p->__pyx_v_core_shifts);
  Py_CLEAR(p->__pyx_v_glycan);
  Py_CLEAR(p->__pyx_v_glycosylation);
  Py_CLEAR(p->__pyx_v_name);
  Py_CLEAR(p->__pyx_v_per_site_shifts);
  Py_CLEAR(p->__pyx_v_positions);
  Py_CLEAR(p->__pyx_v_result);
  Py_CLEAR(p->__pyx_v_seen);
  Py_CLEAR(p->__pyx_v_self);
  Py_CLEAR(p->__pyx_v_site);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments)))) {
    __pyx_freelist_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments[__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments++] = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments *)o;
  if (p->__pyx_v_aggregate_glycosylation) {
    e = (*v)(((PyObject *)p->__pyx_v_aggregate_glycosylation), a); if (e) return e;
  }
  if (p->__pyx_v_base_composition) {
    e = (*v)(((PyObject *)p->__pyx_v_base_composition), a); if (e) return e;
  }
  if (p->__pyx_v_combos) {
    e = (*v)(p->__pyx_v_combos, a); if (e) return e;
  }
  if (p->__pyx_v_composition) {
    e = (*v)(((PyObject *)p->__pyx_v_composition), a); if (e) return e;
  }
  if (p->__pyx_v_core_shifts) {
    e = (*v)(p->__pyx_v_core_shifts, a); if (e) return e;
  }
  if (p->__pyx_v_glycan) {
    e = (*v)(p->__pyx_v_glycan, a); if (e) return e;
  }
  if (p->__pyx_v_glycosylation) {
    e = (*v)(p->__pyx_v_glycosylation, a); if (e) return e;
  }
  if (p->__pyx_v_per_site_shifts) {
    e = (*v)(p->__pyx_v_per_site_shifts, a); if (e) return e;
  }
  if (p->__pyx_v_positions) {
    e = (*v)(p->__pyx_v_positions, a); if (e) return e;
  }
  if (p->__pyx_v_result) {
    e = (*v)(p->__pyx_v_result, a); if (e) return e;
  }
  if (p->__pyx_v_seen) {
    e = (*v)(p->__pyx_v_seen, a); if (e) return e;
  }
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  if (p->__pyx_v_site) {
    e = (*v)(((PyObject *)p->__pyx_v_site), a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments = {
  PyVarObject_HEAD_INIT(0, 0)
  "glycopeptidepy._c.structure.fragmentation_strategy.glycan.__pyx_scope_struct_2_gag_linker_stub_fragments", /*tp_name*/
  sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr *__pyx_freelist_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr[8];
static int __pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr = 0;

static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr)))) {
    o = (PyObject*)__pyx_freelist_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr[--__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr];
    memset(o, 0, sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr));
    (void) PyObject_INIT(o, t);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr(PyObject *o) {
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr)))) {
    __pyx_freelist_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr[__pyx_freecount_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr++] = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static PyTypeObject __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr = {
  PyVarObject_HEAD_INIT(0, 0)
  "glycopeptidepy._c.structure.fragmentation_strategy.glycan.__pyx_scope_struct_3_genexpr", /*tp_name*/
  sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER, /*tp_flags*/
  0, /*tp_doc*/
  0, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_glycan(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_glycan},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "glycan",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_ChemicalShift, __pyx_k_ChemicalShift, sizeof(__pyx_k_ChemicalShift), 0, 0, 1, 1},
  {&__pyx_n_s_CompositionTree, __pyx_k_CompositionTree, sizeof(__pyx_k_CompositionTree), 0, 0, 1, 1},
  {&__pyx_kp_s_Does_not_support_mixed_type_glyc, __pyx_k_Does_not_support_mixed_type_glyc, sizeof(__pyx_k_Does_not_support_mixed_type_glyc), 0, 0, 1, 0},
  {&__pyx_n_s_FrozenMonosaccharideResidue, __pyx_k_FrozenMonosaccharideResidue, sizeof(__pyx_k_FrozenMonosaccharideResidue), 0, 0, 1, 1},
  {&__pyx_n_s_Fuc, __pyx_k_Fuc, sizeof(__pyx_k_Fuc), 0, 0, 1, 1},
  {&__pyx_n_s_Gal2NAc, __pyx_k_Gal2NAc, sizeof(__pyx_k_Gal2NAc), 0, 0, 1, 1},
  {&__pyx_n_s_Glc2NAc, __pyx_k_Glc2NAc, sizeof(__pyx_k_Glc2NAc), 0, 0, 1, 1},
  {&__pyx_n_s_GlycanCompositionFragment, __pyx_k_GlycanCompositionFragment, sizeof(__pyx_k_GlycanCompositionFragment), 0, 0, 1, 1},
  {&__pyx_n_s_GlycanCompositionFragmentStrateg, __pyx_k_GlycanCompositionFragmentStrateg, sizeof(__pyx_k_GlycanCompositionFragmentStrateg), 0, 0, 1, 1},
  {&__pyx_n_s_GlycosylationType, __pyx_k_GlycosylationType, sizeof(__pyx_k_GlycosylationType), 0, 0, 1, 1},
  {&__pyx_n_s_HashableGlycanComposition, __pyx_k_HashableGlycanComposition, sizeof(__pyx_k_HashableGlycanComposition), 0, 0, 1, 1},
  {&__pyx_n_s_Hex, __pyx_k_Hex, sizeof(__pyx_k_Hex), 0, 0, 1, 1},
  {&__pyx_n_s_Hex2NAc, __pyx_k_Hex2NAc, sizeof(__pyx_k_Hex2NAc), 0, 0, 1, 1},
  {&__pyx_n_s_HexNAc, __pyx_k_HexNAc, sizeof(__pyx_k_HexNAc), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x18, __pyx_k_Incompatible_checksums_s_vs_0x18, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x18), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x58, __pyx_k_Incompatible_checksums_s_vs_0x58, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x58), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x63, __pyx_k_Incompatible_checksums_s_vs_0x63, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x63), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0xf5, __pyx_k_Incompatible_checksums_s_vs_0xf5, sizeof(__pyx_k_Incompatible_checksums_s_vs_0xf5), 0, 0, 1, 0},
  {&__pyx_n_s_IonSeries, __pyx_k_IonSeries, sizeof(__pyx_k_IonSeries), 0, 0, 1, 1},
  {&__pyx_n_s_KeyError, __pyx_k_KeyError, sizeof(__pyx_k_KeyError), 0, 0, 1, 1},
  {&__pyx_n_s_Pen, __pyx_k_Pen, sizeof(__pyx_k_Pen), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_StubGlycopeptideStrategy, __pyx_k_StubGlycopeptideStrategy, sizeof(__pyx_k_StubGlycopeptideStrategy), 0, 0, 1, 1},
  {&__pyx_n_s_StubGlycopeptideStrategy_gag_lin, __pyx_k_StubGlycopeptideStrategy_gag_lin, sizeof(__pyx_k_StubGlycopeptideStrategy_gag_lin), 0, 0, 1, 1},
  {&__pyx_n_s_StubGlycopeptideStrategy_n_glyca, __pyx_k_StubGlycopeptideStrategy_n_glyca, sizeof(__pyx_k_StubGlycopeptideStrategy_n_glyca), 0, 0, 1, 1},
  {&__pyx_n_s_StubGlycopeptideStrategy_o_glyca, __pyx_k_StubGlycopeptideStrategy_o_glyca, sizeof(__pyx_k_StubGlycopeptideStrategy_o_glyca), 0, 0, 1, 1},
  {&__pyx_kp_s_Unknown_Glycan_Class_Detected, __pyx_k_Unknown_Glycan_Class_Detected, sizeof(__pyx_k_Unknown_Glycan_Class_Detected), 0, 0, 1, 0},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_Xyl, __pyx_k_Xyl, sizeof(__pyx_k_Xyl), 0, 0, 1, 1},
  {&__pyx_n_s_aHex, __pyx_k_aHex, sizeof(__pyx_k_aHex), 0, 0, 1, 1},
  {&__pyx_n_s_aggregate_glycosylation, __pyx_k_aggregate_glycosylation, sizeof(__pyx_k_aggregate_glycosylation), 0, 0, 1, 1},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_base, __pyx_k_base, sizeof(__pyx_k_base), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_n_s_combinate_sites, __pyx_k_combinate_sites, sizeof(__pyx_k_combinate_sites), 0, 0, 1, 1},
  {&__pyx_n_s_composition, __pyx_k_composition, sizeof(__pyx_k_composition), 0, 0, 1, 1},
  {&__pyx_n_s_composition_tree, __pyx_k_composition_tree, sizeof(__pyx_k_composition_tree), 0, 0, 1, 1},
  {&__pyx_n_s_copy, __pyx_k_copy, sizeof(__pyx_k_copy), 0, 0, 1, 1},
  {&__pyx_n_s_core_count, __pyx_k_core_count, sizeof(__pyx_k_core_count), 0, 0, 1, 1},
  {&__pyx_n_s_count_glycosylation_type, __pyx_k_count_glycosylation_type, sizeof(__pyx_k_count_glycosylation_type), 0, 0, 1, 1},
  {&__pyx_n_s_dHex, __pyx_k_dHex, sizeof(__pyx_k_dHex), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_exact, __pyx_k_exact, sizeof(__pyx_k_exact), 0, 0, 1, 1},
  {&__pyx_n_s_extended, __pyx_k_extended, sizeof(__pyx_k_extended), 0, 0, 1, 1},
  {&__pyx_n_s_extended_fucosylation, __pyx_k_extended_fucosylation, sizeof(__pyx_k_extended_fucosylation), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_format_negative_composition, __pyx_k_format_negative_composition, sizeof(__pyx_k_format_negative_composition), 0, 0, 1, 1},
  {&__pyx_n_s_from_iupac_lite, __pyx_k_from_iupac_lite, sizeof(__pyx_k_from_iupac_lite), 0, 0, 1, 1},
  {&__pyx_n_s_fucose_count, __pyx_k_fucose_count, sizeof(__pyx_k_fucose_count), 0, 0, 1, 1},
  {&__pyx_n_s_fucosylate_extended, __pyx_k_fucosylate_extended, sizeof(__pyx_k_fucosylate_extended), 0, 0, 1, 1},
  {&__pyx_n_s_fucosylate_increment, __pyx_k_fucosylate_increment, sizeof(__pyx_k_fucosylate_increment), 0, 0, 1, 1},
  {&__pyx_n_s_gag_linker_composition_fragments, __pyx_k_gag_linker_composition_fragments, sizeof(__pyx_k_gag_linker_composition_fragments), 0, 0, 1, 1},
  {&__pyx_n_s_gag_linker_stub_fragments, __pyx_k_gag_linker_stub_fragments, sizeof(__pyx_k_gag_linker_stub_fragments), 0, 0, 1, 1},
  {&__pyx_n_s_genexpr, __pyx_k_genexpr, sizeof(__pyx_k_genexpr), 0, 0, 1, 1},
  {&__pyx_n_s_get_glycan_composition, __pyx_k_get_glycan_composition, sizeof(__pyx_k_get_glycan_composition), 0, 0, 1, 1},
  {&__pyx_n_s_getitem_fast, __pyx_k_getitem_fast, sizeof(__pyx_k_getitem_fast), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_glycan, __pyx_k_glycan, sizeof(__pyx_k_glycan), 0, 0, 1, 1},
  {&__pyx_n_s_glycan_composition, __pyx_k_glycan_composition, sizeof(__pyx_k_glycan_composition), 0, 0, 1, 1},
  {&__pyx_n_s_glycopeptidepy__c_structure_frag, __pyx_k_glycopeptidepy__c_structure_frag, sizeof(__pyx_k_glycopeptidepy__c_structure_frag), 0, 0, 1, 1},
  {&__pyx_n_s_glycopeptidepy_structure_fragmen, __pyx_k_glycopeptidepy_structure_fragmen, sizeof(__pyx_k_glycopeptidepy_structure_fragmen), 0, 0, 1, 1},
  {&__pyx_n_s_glycopeptidepy_structure_glycan, __pyx_k_glycopeptidepy_structure_glycan, sizeof(__pyx_k_glycopeptidepy_structure_glycan), 0, 0, 1, 1},
  {&__pyx_n_s_glycosaminoglycan, __pyx_k_glycosaminoglycan, sizeof(__pyx_k_glycosaminoglycan), 0, 0, 1, 1},
  {&__pyx_n_s_glycosylation_manager, __pyx_k_glycosylation_manager, sizeof(__pyx_k_glycosylation_manager), 0, 0, 1, 1},
  {&__pyx_n_s_glypy_structure_glycan_compositi, __pyx_k_glypy_structure_glycan_compositi, sizeof(__pyx_k_glypy_structure_glycan_compositi), 0, 0, 1, 1},
  {&__pyx_n_s_guess_query_mode, __pyx_k_guess_query_mode, sizeof(__pyx_k_guess_query_mode), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_init, __pyx_k_init, sizeof(__pyx_k_init), 0, 0, 1, 1},
  {&__pyx_n_s_is_extended, __pyx_k_is_extended, sizeof(__pyx_k_is_extended), 0, 0, 1, 1},
  {&__pyx_n_s_iteration_count, __pyx_k_iteration_count, sizeof(__pyx_k_iteration_count), 0, 0, 1, 1},
  {&__pyx_n_s_itertools, __pyx_k_itertools, sizeof(__pyx_k_itertools), 0, 0, 1, 1},
  {&__pyx_n_s_key, __pyx_k_key, sizeof(__pyx_k_key), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_mass, __pyx_k_mass, sizeof(__pyx_k_mass), 0, 0, 1, 1},
  {&__pyx_n_s_modified_increment, __pyx_k_modified_increment, sizeof(__pyx_k_modified_increment), 0, 0, 1, 1},
  {&__pyx_n_s_n_glycan_composition_fragments, __pyx_k_n_glycan_composition_fragments, sizeof(__pyx_k_n_glycan_composition_fragments), 0, 0, 1, 1},
  {&__pyx_n_s_n_glycan_stub_fragments, __pyx_k_n_glycan_stub_fragments, sizeof(__pyx_k_n_glycan_stub_fragments), 0, 0, 1, 1},
  {&__pyx_n_s_n_linked, __pyx_k_n_linked, sizeof(__pyx_k_n_linked), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_o_glycan_composition_fragments, __pyx_k_o_glycan_composition_fragments, sizeof(__pyx_k_o_glycan_composition_fragments), 0, 0, 1, 1},
  {&__pyx_n_s_o_glycan_stub_fragments, __pyx_k_o_glycan_stub_fragments, sizeof(__pyx_k_o_glycan_stub_fragments), 0, 0, 1, 1},
  {&__pyx_n_s_o_linked, __pyx_k_o_linked, sizeof(__pyx_k_o_linked), 0, 0, 1, 1},
  {&__pyx_n_s_obj, __pyx_k_obj, sizeof(__pyx_k_obj), 0, 0, 1, 1},
  {&__pyx_n_s_peptide, __pyx_k_peptide, sizeof(__pyx_k_peptide), 0, 0, 1, 1},
  {&__pyx_n_s_per_site_shifts, __pyx_k_per_site_shifts, sizeof(__pyx_k_per_site_shifts), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_product, __pyx_k_product, sizeof(__pyx_k_product), 0, 0, 1, 1},
  {&__pyx_n_s_product_2, __pyx_k_product_2, sizeof(__pyx_k_product_2), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_GlycanComposition, __pyx_k_pyx_unpickle_GlycanComposition, sizeof(__pyx_k_pyx_unpickle_GlycanComposition), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_GlycanComposition_2, __pyx_k_pyx_unpickle_GlycanComposition_2, sizeof(__pyx_k_pyx_unpickle_GlycanComposition_2), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_StubGlycopeptideS, __pyx_k_pyx_unpickle_StubGlycopeptideS, sizeof(__pyx_k_pyx_unpickle_StubGlycopeptideS), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle__CompositionTree, __pyx_k_pyx_unpickle__CompositionTree, sizeof(__pyx_k_pyx_unpickle__CompositionTree), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_query, __pyx_k_query, sizeof(__pyx_k_query), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_root, __pyx_k_root, sizeof(__pyx_k_root), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_kp_s_self___class_____name___self_ma, __pyx_k_self___class_____name___self_ma, sizeof(__pyx_k_self___class_____name___self_ma), 0, 0, 1, 0},
  {&__pyx_n_s_send, __pyx_k_send, sizeof(__pyx_k_send), 0, 0, 1, 1},
  {&__pyx_n_s_setitem_fast, __pyx_k_setitem_fast, sizeof(__pyx_k_setitem_fast), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_n_s_shift, __pyx_k_shift, sizeof(__pyx_k_shift), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_stub_fragments, __pyx_k_stub_fragments, sizeof(__pyx_k_stub_fragments), 0, 0, 1, 1},
  {&__pyx_n_s_stub_fragments_locals_genexpr, __pyx_k_stub_fragments_locals_genexpr, sizeof(__pyx_k_stub_fragments_locals_genexpr), 0, 0, 1, 1},
  {&__pyx_n_s_stub_glycopeptide, __pyx_k_stub_glycopeptide, sizeof(__pyx_k_stub_glycopeptide), 0, 0, 1, 1},
  {&__pyx_n_s_super, __pyx_k_super, sizeof(__pyx_k_super), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_throw, __pyx_k_throw, sizeof(__pyx_k_throw), 0, 0, 1, 1},
  {&__pyx_n_s_total_composition, __pyx_k_total_composition, sizeof(__pyx_k_total_composition), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {&__pyx_n_s_use_query, __pyx_k_use_query, sizeof(__pyx_k_use_query), 0, 0, 1, 1},
  {&__pyx_n_s_validate_glycan_composition, __pyx_k_validate_glycan_composition, sizeof(__pyx_k_validate_glycan_composition), 0, 0, 1, 1},
  {&__pyx_n_s_xylosylate_increment, __pyx_k_xylosylate_increment, sizeof(__pyx_k_xylosylate_increment), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_super = __Pyx_GetBuiltinName(__pyx_n_s_super); if (!__pyx_builtin_super) __PYX_ERR(0, 111, __pyx_L1_error)
  __pyx_builtin_KeyError = __Pyx_GetBuiltinName(__pyx_n_s_KeyError); if (!__pyx_builtin_KeyError) __PYX_ERR(0, 177, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 304, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 903, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":844
 *         xyl_in_aggregate = glycan[_XYL]
 *         if xyl_in_aggregate == 0:
 *             xyl_in_aggregate = glycan.query("Pen", exact=False)             # <<<<<<<<<<<<<<
 *         core_shifts = []
 *         for xyl_count in range(max(0, xyl_in_aggregate) + 1):
 */
  __pyx_tuple_ = PyTuple_Pack(1, __pyx_n_s_Pen); if (unlikely(!__pyx_tuple_)) __PYX_ERR(0, 844, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":903
 *         gag_linker = self.count_glycosylation_type(GlycosylationType_glycosaminoglycan) > 0
 *         if (n_glycan + o_glycan + gag_linker) > 2:
 *             raise ValueError(             # <<<<<<<<<<<<<<
 *                 "Does not support mixed-type glycan fragmentation (yet)")
 *         if n_glycan:
 */
  __pyx_tuple__2 = PyTuple_Pack(1, __pyx_kp_s_Does_not_support_mixed_type_glyc); if (unlikely(!__pyx_tuple__2)) __PYX_ERR(0, 903, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__2);
  __Pyx_GIVEREF(__pyx_tuple__2);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":913
 *         else:
 *             if (<GlycosylationManager>self.peptide.glycosylation_manager).get_size() > 0:
 *                 raise ValueError("Unknown Glycan Class Detected")             # <<<<<<<<<<<<<<
 *         return (a for a in [])
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_kp_s_Unknown_Glycan_Class_Detected); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 913, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":85
 * 
 * cdef:
 *     object _HEX = FrozenMonosaccharideResidue.from_iupac_lite("Hex")             # <<<<<<<<<<<<<<
 *     CComposition _HEX_composition = _HEX.total_composition()
 *     double _HEX_mass = _HEX.mass()
 */
  __pyx_tuple__4 = PyTuple_Pack(1, __pyx_n_s_Hex); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":88
 *     CComposition _HEX_composition = _HEX.total_composition()
 *     double _HEX_mass = _HEX.mass()
 *     object _HEXNAC = FrozenMonosaccharideResidue.from_iupac_lite("HexNAc")             # <<<<<<<<<<<<<<
 *     CComposition _HEXNAC_composition = _HEXNAC.total_composition()
 *     double _HEXNAC_mass = _HEXNAC.mass()
 */
  __pyx_tuple__5 = PyTuple_Pack(1, __pyx_n_s_HexNAc); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(0, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":91
 *     CComposition _HEXNAC_composition = _HEXNAC.total_composition()
 *     double _HEXNAC_mass = _HEXNAC.mass()
 *     object _FUC = FrozenMonosaccharideResidue.from_iupac_lite("Fuc")             # <<<<<<<<<<<<<<
 *     CComposition _FUC_composition = _FUC.total_composition()
 *     double _FUC_mass = _FUC.mass()
 */
  __pyx_tuple__6 = PyTuple_Pack(1, __pyx_n_s_Fuc); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":94
 *     CComposition _FUC_composition = _FUC.total_composition()
 *     double _FUC_mass = _FUC.mass()
 *     object _DHEX = FrozenMonosaccharideResidue.from_iupac_lite("dHex")             # <<<<<<<<<<<<<<
 *     CComposition _DHEX_composition = _DHEX.total_composition()
 *     double _DHEX_mass = _DHEX.mass()
 */
  __pyx_tuple__7 = PyTuple_Pack(1, __pyx_n_s_dHex); if (unlikely(!__pyx_tuple__7)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__7);
  __Pyx_GIVEREF(__pyx_tuple__7);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":97
 *     CComposition _DHEX_composition = _DHEX.total_composition()
 *     double _DHEX_mass = _DHEX.mass()
 *     object _XYL = FrozenMonosaccharideResidue.from_iupac_lite("Xyl")             # <<<<<<<<<<<<<<
 *     CComposition _XYL_composition = _XYL.total_composition()
 *     double _XYL_mass = _XYL.mass()
 */
  __pyx_tuple__8 = PyTuple_Pack(1, __pyx_n_s_Xyl); if (unlikely(!__pyx_tuple__8)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__8);
  __Pyx_GIVEREF(__pyx_tuple__8);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":100
 *     CComposition _XYL_composition = _XYL.total_composition()
 *     double _XYL_mass = _XYL.mass()
 *     object _AHEX = FrozenMonosaccharideResidue.from_iupac_lite("aHex")             # <<<<<<<<<<<<<<
 *     CComposition _AHEX_composition = _AHEX.total_composition()
 *     double _AHEX_mass = _AHEX.mass()
 */
  __pyx_tuple__9 = PyTuple_Pack(1, __pyx_n_s_aHex); if (unlikely(!__pyx_tuple__9)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__9);
  __Pyx_GIVEREF(__pyx_tuple__9);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":104
 *     double _AHEX_mass = _AHEX.mass()
 * 
 *     object _Hex2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Hex2NAc")             # <<<<<<<<<<<<<<
 *     object _Glc2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Glc2NAc")
 *     object _Gal2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Gal2NAc")
 */
  __pyx_tuple__10 = PyTuple_Pack(1, __pyx_n_s_Hex2NAc); if (unlikely(!__pyx_tuple__10)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__10);
  __Pyx_GIVEREF(__pyx_tuple__10);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":105
 * 
 *     object _Hex2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Hex2NAc")
 *     object _Glc2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Glc2NAc")             # <<<<<<<<<<<<<<
 *     object _Gal2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Gal2NAc")
 * 
 */
  __pyx_tuple__11 = PyTuple_Pack(1, __pyx_n_s_Glc2NAc); if (unlikely(!__pyx_tuple__11)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__11);
  __Pyx_GIVEREF(__pyx_tuple__11);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":106
 *     object _Hex2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Hex2NAc")
 *     object _Glc2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Glc2NAc")
 *     object _Gal2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Gal2NAc")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_tuple__12 = PyTuple_Pack(1, __pyx_n_s_Gal2NAc); if (unlikely(!__pyx_tuple__12)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__12);
  __Pyx_GIVEREF(__pyx_tuple__12);

  /* "(tree fragment)":1
 * def __pyx_unpickle__CompositionTree(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__13 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__13)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__13);
  __Pyx_GIVEREF(__pyx_tuple__13);
  __pyx_codeobj__14 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__13, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle__CompositionTree, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__14)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__15 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__15)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__15);
  __Pyx_GIVEREF(__pyx_tuple__15);
  __pyx_codeobj__16 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__15, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_GlycanComposition, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__16)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__17 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__17)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__17);
  __Pyx_GIVEREF(__pyx_tuple__17);
  __pyx_codeobj__18 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__17, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_GlycanComposition_2, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__18)) __PYX_ERR(1, 1, __pyx_L1_error)
  __pyx_tuple__19 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__19)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__19);
  __Pyx_GIVEREF(__pyx_tuple__19);
  __pyx_codeobj__20 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__19, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_StubGlycopeptideS, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__20)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_2 = PyInt_FromLong(2); if (unlikely(!__pyx_int_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_25957662 = PyInt_FromLong(25957662L); if (unlikely(!__pyx_int_25957662)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_93054799 = PyInt_FromLong(93054799L); if (unlikely(!__pyx_int_93054799)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_104505102 = PyInt_FromLong(104505102L); if (unlikely(!__pyx_int_104505102)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_257754233 = PyInt_FromLong(257754233L); if (unlikely(!__pyx_int_257754233)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_product = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_n_linked = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_o_linked = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_glycosaminoglycan = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_IonSeries_stub_glycopeptide = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase *)Py_None); Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__composition_tree_root = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *)Py_None); Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__composition_name_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__DHEX = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__DHEX_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__AHEX = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__AHEX_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Hex2NAc = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Glc2NAc = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Gal2NAc = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_hexnac_hex_composition_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment = &__pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment.copy = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment_copy;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment._get_glycan_composition = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__get_glycan_composition;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment._create = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *(*)(double, struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, int))__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_25GlycanCompositionFragment__create;
  if (PyType_Ready(&__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment) < 0) __PYX_ERR(0, 135, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment.tp_dictoffset && __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment.tp_dict, __pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment) < 0) __PYX_ERR(0, 135, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_GlycanCompositionFragment, (PyObject *)&__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment) < 0) __PYX_ERR(0, 135, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment) < 0) __PYX_ERR(0, 135, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment = &__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment;
  __pyx_t_1 = PyImport_ImportModule("glycopeptidepy._c.structure.fragmentation_strategy.base"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.fragmentation_strategy.base", "FragmentationStrategyBase", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase = &__pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase.__pyx_base = *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase.glycan_composition = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_glycan_composition;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase._guess_query_mode = (int (*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *, PyObject *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase__guess_query_mode;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase.count_glycosylation_type = (long (*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase *, PyObject *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_37GlycanCompositionFragmentStrategyBase_count_glycosylation_type;
  __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase.tp_base = __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_FragmentationStrategyBase;
  if (PyType_Ready(&__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase) < 0) __PYX_ERR(0, 109, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase.tp_dictoffset && __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase.tp_dict, __pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase) < 0) __PYX_ERR(0, 109, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_GlycanCompositionFragmentStrateg, (PyObject *)&__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase) < 0) __PYX_ERR(0, 109, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase) < 0) __PYX_ERR(0, 109, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase = &__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase;
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy = &__pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy.__pyx_base = *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy.fucosylate_increment = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_fucosylate_increment;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy.xylosylate_increment = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_xylosylate_increment;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy.modified_increment = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *, PyObject *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_modified_increment;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy.fucosylate_extended = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragment *, long, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_fucosylate_extended;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy._validate_glycan_composition = (int (*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *, PyObject *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy__validate_glycan_composition;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy._combinate_sites = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy__combinate_sites;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy.n_glycan_composition_fragments = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_n_glycan_composition_fragments *__pyx_optional_args))__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_n_glycan_composition_fragments;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy.o_glycan_composition_fragments = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_o_glycan_composition_fragments *__pyx_optional_args))__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_o_glycan_composition_fragments;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy.gag_linker_composition_fragments = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_gag_linker_composition_fragments *__pyx_optional_args))__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_24StubGlycopeptideStrategy_gag_linker_composition_fragments;
  __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy.tp_base = __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycanCompositionFragmentStrategyBase;
  if (PyType_Ready(&__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy) < 0) __PYX_ERR(0, 227, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy.tp_dictoffset && __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy.tp_dict, __pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy) < 0) __PYX_ERR(0, 227, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_StubGlycopeptideStrategy, (PyObject *)&__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy) < 0) __PYX_ERR(0, 227, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy) < 0) __PYX_ERR(0, 227, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy = &__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy;
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree = &__pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree.build = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *, PyObject *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_16_CompositionTree_build;
  if (PyType_Ready(&__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree) < 0) __PYX_ERR(0, 46, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree.tp_dictoffset && __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  if (__Pyx_SetVtable(__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree.tp_dict, __pyx_vtabptr_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree) < 0) __PYX_ERR(0, 46, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_CompositionTree, (PyObject *)&__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree) < 0) __PYX_ERR(0, 46, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree) < 0) __PYX_ERR(0, 46, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree = &__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree;
  if (PyType_Ready(&__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments) < 0) __PYX_ERR(0, 593, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments.tp_dictoffset && __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments = &__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct__n_glycan_stub_fragments;
  if (PyType_Ready(&__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments) < 0) __PYX_ERR(0, 640, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments.tp_dictoffset && __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments = &__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_1_o_glycan_stub_fragments;
  if (PyType_Ready(&__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments) < 0) __PYX_ERR(0, 792, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments.tp_dictoffset && __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments = &__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_2_gag_linker_stub_fragments;
  if (PyType_Ready(&__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr) < 0) __PYX_ERR(0, 914, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr.tp_dictoffset && __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr = &__pyx_type_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan___pyx_scope_struct_3_genexpr;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule("glypy.composition.ccomposition"); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5glypy_11composition_12ccomposition_CComposition = __Pyx_ImportType(__pyx_t_1, "glypy.composition.ccomposition", "CComposition", sizeof(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5glypy_11composition_12ccomposition_CComposition) __PYX_ERR(3, 2, __pyx_L1_error)
  __pyx_vtabptr_5glypy_11composition_12ccomposition_CComposition = (struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition*)__Pyx_GetVtable(__pyx_ptype_5glypy_11composition_12ccomposition_CComposition->tp_dict); if (unlikely(!__pyx_vtabptr_5glypy_11composition_12ccomposition_CComposition)) __PYX_ERR(3, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(4, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4bool_bool = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "bool", sizeof(PyBoolObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4bool_bool) __PYX_ERR(5, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_7complex_complex = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "complex", sizeof(PyComplexObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_7complex_complex) __PYX_ERR(6, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("glycopeptidepy._c.count_table"); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 51, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14glycopeptidepy_2_c_11count_table_CountTableIterator = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.count_table", "CountTableIterator", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTableIterator), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_11count_table_CountTableIterator) __PYX_ERR(7, 51, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTableIterator = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTableIterator*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_11count_table_CountTableIterator->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTableIterator)) __PYX_ERR(7, 51, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_11count_table_CountTable = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.count_table", "CountTable", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_11count_table_CountTable) __PYX_ERR(7, 68, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_11count_table_CountTable*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_11count_table_CountTable->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_11count_table_CountTable)) __PYX_ERR(7, 68, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("glycopeptidepy._c.structure.fragment"); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.fragment", "ChemicalShiftBase", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase) __PYX_ERR(8, 6, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_ChemicalShiftBase)) __PYX_ERR(8, 6, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.fragment", "IonSeriesBase", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase) __PYX_ERR(8, 15, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.fragment", "FragmentBase", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase) __PYX_ERR(8, 25, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_FragmentBase)) __PYX_ERR(8, 25, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_SimpleFragment = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.fragment", "SimpleFragment", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_SimpleFragment), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_SimpleFragment) __PYX_ERR(8, 60, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_SimpleFragment = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_SimpleFragment*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_SimpleFragment->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_SimpleFragment)) __PYX_ERR(8, 60, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_StubFragment = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.fragment", "StubFragment", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_StubFragment), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_StubFragment) __PYX_ERR(8, 69, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_StubFragment = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_StubFragment*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_StubFragment->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_StubFragment)) __PYX_ERR(8, 69, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.fragment", "PeptideFragment", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment) __PYX_ERR(8, 85, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment_PeptideFragment)) __PYX_ERR(8, 85, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment__NameTree = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.fragment", "_NameTree", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment__NameTree), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment__NameTree) __PYX_ERR(8, 113, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment__NameTree = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_8fragment__NameTree*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment__NameTree->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_8fragment__NameTree)) __PYX_ERR(8, 113, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("glycopeptidepy._c.structure.base"); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.base", "PeptideSequenceBase", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase) __PYX_ERR(9, 5, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase)) __PYX_ERR(9, 5, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.base", "TerminalGroup", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup) __PYX_ERR(9, 21, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_TerminalGroup = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_TerminalGroup)) __PYX_ERR(9, 21, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.base", "AminoAcidResidueBase", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase) __PYX_ERR(9, 35, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase)) __PYX_ERR(9, 35, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.base", "ModificationBase", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase) __PYX_ERR(9, 50, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_ModificationBase = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_ModificationBase)) __PYX_ERR(9, 50, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_SequencePosition = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.base", "SequencePosition", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_SequencePosition) __PYX_ERR(9, 69, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_SequencePosition = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_SequencePosition*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_SequencePosition->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_SequencePosition)) __PYX_ERR(9, 69, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("glycopeptidepy._c.structure.sequence_methods"); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.sequence_methods", "_PeptideSequenceCore", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore) __PYX_ERR(10, 15, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore)) __PYX_ERR(10, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("glycopeptidepy._c.structure.glycan"); if (unlikely(!__pyx_t_1)) __PYX_ERR(11, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14glycopeptidepy_2_c_9structure_6glycan_GlycanCompositionWithOffsetProxyBase = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.glycan", "GlycanCompositionWithOffsetProxyBase", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycanCompositionWithOffsetProxyBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_6glycan_GlycanCompositionWithOffsetProxyBase) __PYX_ERR(11, 5, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.glycan", "GlycosylationManager", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager) __PYX_ERR(11, 11, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager)) __PYX_ERR(11, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __pyx_t_1 = PyImport_ImportModule("glypy.composition.ccomposition"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_atom", (void **)&__pyx_vp_5glypy_11composition_12ccomposition__atom, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_formula", (void **)&__pyx_vp_5glypy_11composition_12ccomposition__formula, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_isotope_string", (void **)&__pyx_vp_5glypy_11composition_12ccomposition__isotope_string, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "isotope_pattern", (void **)&__pyx_vp_5glypy_11composition_12ccomposition_isotope_pattern, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "formula_pattern", (void **)&__pyx_vp_5glypy_11composition_12ccomposition_formula_pattern, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("glycopeptidepy._c.structure.fragmentation_strategy.base"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_n_glycosylation", (void **)&__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__n_glycosylation, "struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_o_glycosylation", (void **)&__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__o_glycosylation, "struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_gag_linker_glycosylation", (void **)&__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__gag_linker_glycosylation, "struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_modification_hexnac", (void **)&__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__modification_hexnac, "struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_modification_xylose", (void **)&__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base__modification_xylose, "struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "glycosylation_type_to_core", (void **)&__pyx_vp_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_4base_glycosylation_type_to_core, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __pyx_t_1 = PyImport_ImportModule("glycopeptidepy._c.structure.fragment"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportFunction(__pyx_t_1, "__pyx_fuse_0build_name_from_composition", (void (**)(void))&__pyx_fuse_0__pyx_f_14glycopeptidepy_2_c_9structure_8fragment_build_name_from_composition, "PyObject *(struct __pyx_obj_14glycopeptidepy_2_c_11count_table_CountTable *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "__pyx_fuse_1build_name_from_composition", (void (**)(void))&__pyx_fuse_1__pyx_f_14glycopeptidepy_2_c_9structure_8fragment_build_name_from_composition, "PyObject *(PyObject *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportFunction(__pyx_t_1, "__pyx_fuse_2build_name_from_composition", (void (**)(void))&__pyx_fuse_2__pyx_f_14glycopeptidepy_2_c_9structure_8fragment_build_name_from_composition, "PyObject *(PyObject *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initglycan(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initglycan(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_glycan(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_glycan(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_glycan(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  double __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'glycan' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_glycan(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("glycan", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_glycopeptidepy___c__structure__fragmentation_strategy__glycan) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name_2, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "glycopeptidepy._c.structure.fragmentation_strategy.glycan")) {
      if (unlikely(PyDict_SetItemString(modules, "glycopeptidepy._c.structure.fragmentation_strategy.glycan", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_variable_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_function_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":25
 * 
 * from glycopeptidepy.structure.fragment import (
 *     ChemicalShift, IonSeries, format_negative_composition)             # <<<<<<<<<<<<<<
 * 
 * from glycopeptidepy.structure.glycan import (
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_ChemicalShift);
  __Pyx_GIVEREF(__pyx_n_s_ChemicalShift);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_ChemicalShift);
  __Pyx_INCREF(__pyx_n_s_IonSeries);
  __Pyx_GIVEREF(__pyx_n_s_IonSeries);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_IonSeries);
  __Pyx_INCREF(__pyx_n_s_format_negative_composition);
  __Pyx_GIVEREF(__pyx_n_s_format_negative_composition);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_s_format_negative_composition);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":24
 * from glycopeptidepy._c.structure.glycan cimport GlycosylationManager
 * 
 * from glycopeptidepy.structure.fragment import (             # <<<<<<<<<<<<<<
 *     ChemicalShift, IonSeries, format_negative_composition)
 * 
 */
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_glycopeptidepy_structure_fragmen, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_ChemicalShift); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ChemicalShift, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_IonSeries); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_IonSeries, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_format_negative_composition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_format_negative_composition, __pyx_t_1) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":28
 * 
 * from glycopeptidepy.structure.glycan import (
 *     GlycosylationType,             # <<<<<<<<<<<<<<
 *     HashableGlycanComposition)
 * 
 */
  __pyx_t_2 = PyList_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_GlycosylationType);
  __Pyx_GIVEREF(__pyx_n_s_GlycosylationType);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_GlycosylationType);
  __Pyx_INCREF(__pyx_n_s_HashableGlycanComposition);
  __Pyx_GIVEREF(__pyx_n_s_HashableGlycanComposition);
  PyList_SET_ITEM(__pyx_t_2, 1, __pyx_n_s_HashableGlycanComposition);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":27
 *     ChemicalShift, IonSeries, format_negative_composition)
 * 
 * from glycopeptidepy.structure.glycan import (             # <<<<<<<<<<<<<<
 *     GlycosylationType,
 *     HashableGlycanComposition)
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_glycopeptidepy_structure_glycan, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_GlycosylationType); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_GlycosylationType, __pyx_t_2) < 0) __PYX_ERR(0, 28, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_HashableGlycanComposition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_HashableGlycanComposition, __pyx_t_2) < 0) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":31
 *     HashableGlycanComposition)
 * 
 * from itertools import product as _product             # <<<<<<<<<<<<<<
 * from glypy.structure.glycan_composition import (
 *     FrozenMonosaccharideResidue)
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_product);
  __Pyx_GIVEREF(__pyx_n_s_product);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_product);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_itertools, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_product); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_product_2, __pyx_t_1) < 0) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":33
 * from itertools import product as _product
 * from glypy.structure.glycan_composition import (
 *     FrozenMonosaccharideResidue)             # <<<<<<<<<<<<<<
 * 
 * cdef object product = _product
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_FrozenMonosaccharideResidue);
  __Pyx_GIVEREF(__pyx_n_s_FrozenMonosaccharideResidue);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_FrozenMonosaccharideResidue);

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":32
 * 
 * from itertools import product as _product
 * from glypy.structure.glycan_composition import (             # <<<<<<<<<<<<<<
 *     FrozenMonosaccharideResidue)
 * 
 */
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_glypy_structure_glycan_compositi, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_FrozenMonosaccharideResidue); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 32, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_FrozenMonosaccharideResidue, __pyx_t_2) < 0) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":35
 *     FrozenMonosaccharideResidue)
 * 
 * cdef object product = _product             # <<<<<<<<<<<<<<
 * 
 * cdef:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_product_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_product);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_product, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":38
 * 
 * cdef:
 *     object GlycosylationType_n_linked = GlycosylationType.n_linked             # <<<<<<<<<<<<<<
 *     object GlycosylationType_o_linked = GlycosylationType.o_linked
 *     object GlycosylationType_glycosaminoglycan = GlycosylationType.glycosaminoglycan
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_GlycosylationType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_n_linked); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_n_linked);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_n_linked, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":39
 * cdef:
 *     object GlycosylationType_n_linked = GlycosylationType.n_linked
 *     object GlycosylationType_o_linked = GlycosylationType.o_linked             # <<<<<<<<<<<<<<
 *     object GlycosylationType_glycosaminoglycan = GlycosylationType.glycosaminoglycan
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_GlycosylationType); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_o_linked); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_o_linked);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_o_linked, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":40
 *     object GlycosylationType_n_linked = GlycosylationType.n_linked
 *     object GlycosylationType_o_linked = GlycosylationType.o_linked
 *     object GlycosylationType_glycosaminoglycan = GlycosylationType.glycosaminoglycan             # <<<<<<<<<<<<<<
 * 
 * cdef IonSeriesBase IonSeries_stub_glycopeptide = IonSeries.stub_glycopeptide
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_GlycosylationType); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_glycosaminoglycan); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 40, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_glycosaminoglycan);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_GlycosylationType_glycosaminoglycan, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":42
 *     object GlycosylationType_glycosaminoglycan = GlycosylationType.glycosaminoglycan
 * 
 * cdef IonSeriesBase IonSeries_stub_glycopeptide = IonSeries.stub_glycopeptide             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_IonSeries); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_stub_glycopeptide); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase))))) __PYX_ERR(0, 42, __pyx_L1_error)
  __Pyx_XGOTREF(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_IonSeries_stub_glycopeptide));
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_IonSeries_stub_glycopeptide, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_8fragment_IonSeriesBase *)__pyx_t_1));
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":76
 * 
 * 
 * cdef _CompositionTree _composition_tree_root = _CompositionTree()             # <<<<<<<<<<<<<<
 * cdef dict _composition_name_cache = dict()
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XGOTREF(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__composition_tree_root));
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__composition_tree_root, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__CompositionTree *)__pyx_t_1));
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":77
 * 
 * cdef _CompositionTree _composition_tree_root = _CompositionTree()
 * cdef dict _composition_name_cache = dict()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__composition_name_cache);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__composition_name_cache, ((PyObject*)__pyx_t_1));
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":85
 * 
 * cdef:
 *     object _HEX = FrozenMonosaccharideResidue.from_iupac_lite("Hex")             # <<<<<<<<<<<<<<
 *     CComposition _HEX_composition = _HEX.total_composition()
 *     double _HEX_mass = _HEX.mass()
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FrozenMonosaccharideResidue); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_from_iupac_lite); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":86
 * cdef:
 *     object _HEX = FrozenMonosaccharideResidue.from_iupac_lite("Hex")
 *     CComposition _HEX_composition = _HEX.total_composition()             # <<<<<<<<<<<<<<
 *     double _HEX_mass = _HEX.mass()
 *     object _HEXNAC = FrozenMonosaccharideResidue.from_iupac_lite("HexNAc")
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX, __pyx_n_s_total_composition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 86, __pyx_L1_error)
  __Pyx_XGOTREF(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX_composition));
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX_composition, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_2));
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":87
 *     object _HEX = FrozenMonosaccharideResidue.from_iupac_lite("Hex")
 *     CComposition _HEX_composition = _HEX.total_composition()
 *     double _HEX_mass = _HEX.mass()             # <<<<<<<<<<<<<<
 *     object _HEXNAC = FrozenMonosaccharideResidue.from_iupac_lite("HexNAc")
 *     CComposition _HEXNAC_composition = _HEXNAC.total_composition()
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX, __pyx_n_s_mass); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 87, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 87, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEX_mass = __pyx_t_3;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":88
 *     CComposition _HEX_composition = _HEX.total_composition()
 *     double _HEX_mass = _HEX.mass()
 *     object _HEXNAC = FrozenMonosaccharideResidue.from_iupac_lite("HexNAc")             # <<<<<<<<<<<<<<
 *     CComposition _HEXNAC_composition = _HEXNAC.total_composition()
 *     double _HEXNAC_mass = _HEXNAC.mass()
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FrozenMonosaccharideResidue); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_from_iupac_lite); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__5, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 88, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":89
 *     double _HEX_mass = _HEX.mass()
 *     object _HEXNAC = FrozenMonosaccharideResidue.from_iupac_lite("HexNAc")
 *     CComposition _HEXNAC_composition = _HEXNAC.total_composition()             # <<<<<<<<<<<<<<
 *     double _HEXNAC_mass = _HEXNAC.mass()
 *     object _FUC = FrozenMonosaccharideResidue.from_iupac_lite("Fuc")
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC, __pyx_n_s_total_composition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 89, __pyx_L1_error)
  __Pyx_XGOTREF(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC_composition));
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC_composition, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_2));
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":90
 *     object _HEXNAC = FrozenMonosaccharideResidue.from_iupac_lite("HexNAc")
 *     CComposition _HEXNAC_composition = _HEXNAC.total_composition()
 *     double _HEXNAC_mass = _HEXNAC.mass()             # <<<<<<<<<<<<<<
 *     object _FUC = FrozenMonosaccharideResidue.from_iupac_lite("Fuc")
 *     CComposition _FUC_composition = _FUC.total_composition()
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC, __pyx_n_s_mass); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 90, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__HEXNAC_mass = __pyx_t_3;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":91
 *     CComposition _HEXNAC_composition = _HEXNAC.total_composition()
 *     double _HEXNAC_mass = _HEXNAC.mass()
 *     object _FUC = FrozenMonosaccharideResidue.from_iupac_lite("Fuc")             # <<<<<<<<<<<<<<
 *     CComposition _FUC_composition = _FUC.total_composition()
 *     double _FUC_mass = _FUC.mass()
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FrozenMonosaccharideResidue); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_from_iupac_lite); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 91, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":92
 *     double _HEXNAC_mass = _HEXNAC.mass()
 *     object _FUC = FrozenMonosaccharideResidue.from_iupac_lite("Fuc")
 *     CComposition _FUC_composition = _FUC.total_composition()             # <<<<<<<<<<<<<<
 *     double _FUC_mass = _FUC.mass()
 *     object _DHEX = FrozenMonosaccharideResidue.from_iupac_lite("dHex")
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC, __pyx_n_s_total_composition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 92, __pyx_L1_error)
  __Pyx_XGOTREF(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC_composition));
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC_composition, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_2));
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":93
 *     object _FUC = FrozenMonosaccharideResidue.from_iupac_lite("Fuc")
 *     CComposition _FUC_composition = _FUC.total_composition()
 *     double _FUC_mass = _FUC.mass()             # <<<<<<<<<<<<<<
 *     object _DHEX = FrozenMonosaccharideResidue.from_iupac_lite("dHex")
 *     CComposition _DHEX_composition = _DHEX.total_composition()
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC, __pyx_n_s_mass); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__FUC_mass = __pyx_t_3;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":94
 *     CComposition _FUC_composition = _FUC.total_composition()
 *     double _FUC_mass = _FUC.mass()
 *     object _DHEX = FrozenMonosaccharideResidue.from_iupac_lite("dHex")             # <<<<<<<<<<<<<<
 *     CComposition _DHEX_composition = _DHEX.total_composition()
 *     double _DHEX_mass = _DHEX.mass()
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FrozenMonosaccharideResidue); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_from_iupac_lite); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__7, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 94, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__DHEX);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__DHEX, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":95
 *     double _FUC_mass = _FUC.mass()
 *     object _DHEX = FrozenMonosaccharideResidue.from_iupac_lite("dHex")
 *     CComposition _DHEX_composition = _DHEX.total_composition()             # <<<<<<<<<<<<<<
 *     double _DHEX_mass = _DHEX.mass()
 *     object _XYL = FrozenMonosaccharideResidue.from_iupac_lite("Xyl")
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__DHEX, __pyx_n_s_total_composition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 95, __pyx_L1_error)
  __Pyx_XGOTREF(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__DHEX_composition));
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__DHEX_composition, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_2));
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":96
 *     object _DHEX = FrozenMonosaccharideResidue.from_iupac_lite("dHex")
 *     CComposition _DHEX_composition = _DHEX.total_composition()
 *     double _DHEX_mass = _DHEX.mass()             # <<<<<<<<<<<<<<
 *     object _XYL = FrozenMonosaccharideResidue.from_iupac_lite("Xyl")
 *     CComposition _XYL_composition = _XYL.total_composition()
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__DHEX, __pyx_n_s_mass); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 96, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__DHEX_mass = __pyx_t_3;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":97
 *     CComposition _DHEX_composition = _DHEX.total_composition()
 *     double _DHEX_mass = _DHEX.mass()
 *     object _XYL = FrozenMonosaccharideResidue.from_iupac_lite("Xyl")             # <<<<<<<<<<<<<<
 *     CComposition _XYL_composition = _XYL.total_composition()
 *     double _XYL_mass = _XYL.mass()
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FrozenMonosaccharideResidue); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_from_iupac_lite); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__8, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 97, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":98
 *     double _DHEX_mass = _DHEX.mass()
 *     object _XYL = FrozenMonosaccharideResidue.from_iupac_lite("Xyl")
 *     CComposition _XYL_composition = _XYL.total_composition()             # <<<<<<<<<<<<<<
 *     double _XYL_mass = _XYL.mass()
 *     object _AHEX = FrozenMonosaccharideResidue.from_iupac_lite("aHex")
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL, __pyx_n_s_total_composition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 98, __pyx_L1_error)
  __Pyx_XGOTREF(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL_composition));
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL_composition, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_2));
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":99
 *     object _XYL = FrozenMonosaccharideResidue.from_iupac_lite("Xyl")
 *     CComposition _XYL_composition = _XYL.total_composition()
 *     double _XYL_mass = _XYL.mass()             # <<<<<<<<<<<<<<
 *     object _AHEX = FrozenMonosaccharideResidue.from_iupac_lite("aHex")
 *     CComposition _AHEX_composition = _AHEX.total_composition()
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL, __pyx_n_s_mass); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 99, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__XYL_mass = __pyx_t_3;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":100
 *     CComposition _XYL_composition = _XYL.total_composition()
 *     double _XYL_mass = _XYL.mass()
 *     object _AHEX = FrozenMonosaccharideResidue.from_iupac_lite("aHex")             # <<<<<<<<<<<<<<
 *     CComposition _AHEX_composition = _AHEX.total_composition()
 *     double _AHEX_mass = _AHEX.mass()
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FrozenMonosaccharideResidue); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_from_iupac_lite); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__9, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 100, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__AHEX);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__AHEX, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":101
 *     double _XYL_mass = _XYL.mass()
 *     object _AHEX = FrozenMonosaccharideResidue.from_iupac_lite("aHex")
 *     CComposition _AHEX_composition = _AHEX.total_composition()             # <<<<<<<<<<<<<<
 *     double _AHEX_mass = _AHEX.mass()
 * 
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__AHEX, __pyx_n_s_total_composition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallNoArg(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 101, __pyx_L1_error)
  __Pyx_XGOTREF(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__AHEX_composition));
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__AHEX_composition, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_2));
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":102
 *     object _AHEX = FrozenMonosaccharideResidue.from_iupac_lite("aHex")
 *     CComposition _AHEX_composition = _AHEX.total_composition()
 *     double _AHEX_mass = _AHEX.mass()             # <<<<<<<<<<<<<<
 * 
 *     object _Hex2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Hex2NAc")
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__AHEX, __pyx_n_s_mass); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 102, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__AHEX_mass = __pyx_t_3;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":104
 *     double _AHEX_mass = _AHEX.mass()
 * 
 *     object _Hex2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Hex2NAc")             # <<<<<<<<<<<<<<
 *     object _Glc2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Glc2NAc")
 *     object _Gal2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Gal2NAc")
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FrozenMonosaccharideResidue); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_from_iupac_lite); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__10, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 104, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Hex2NAc);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Hex2NAc, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":105
 * 
 *     object _Hex2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Hex2NAc")
 *     object _Glc2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Glc2NAc")             # <<<<<<<<<<<<<<
 *     object _Gal2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Gal2NAc")
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FrozenMonosaccharideResidue); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_from_iupac_lite); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__11, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 105, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Glc2NAc);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Glc2NAc, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":106
 *     object _Hex2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Hex2NAc")
 *     object _Glc2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Glc2NAc")
 *     object _Gal2NAc = FrozenMonosaccharideResidue.from_iupac_lite("Gal2NAc")             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_FrozenMonosaccharideResidue); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_from_iupac_lite); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_tuple__12, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 106, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Gal2NAc);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__Gal2NAc, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":206
 * 
 * 
 * cdef dict hexnac_hex_composition_cache = dict()             # <<<<<<<<<<<<<<
 * 
 * cdef CComposition hexnac_hex_composition(long hexnac, long hexose):
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 206, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_hexnac_hex_composition_cache);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_hexnac_hex_composition_cache, ((PyObject*)__pyx_t_1));
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":241
 *     """
 * 
 *     _composition_tree = _composition_tree_root             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, peptide, extended=True, use_query=False, extended_fucosylation=False, **kwargs):
 */
  if (PyDict_SetItem((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy->tp_dict, __pyx_n_s_composition_tree, ((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan__composition_tree_root)) < 0) __PYX_ERR(0, 241, __pyx_L1_error)
  PyType_Modified(__pyx_ptype_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_StubGlycopeptideStrategy);

  /* "(tree fragment)":1
 * def __pyx_unpickle__CompositionTree(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_1__pyx_unpickle__CompositionTree, NULL, __pyx_n_s_glycopeptidepy__c_structure_frag); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle__CompositionTree, __pyx_t_1) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle__CompositionTree__set_state(<_CompositionTree> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle__CompositionTree__set_state(_CompositionTree __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.root = __pyx_state[0]
 *     if len(__pyx_state) > 1 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_3__pyx_unpickle_GlycanCompositionFragmentStrategyBase, NULL, __pyx_n_s_glycopeptidepy__c_structure_frag); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_GlycanComposition, __pyx_t_1) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_GlycanCompositionFragment(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_5__pyx_unpickle_GlycanCompositionFragment, NULL, __pyx_n_s_glycopeptidepy__c_structure_frag); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_GlycanComposition_2, __pyx_t_1) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_GlycanCompositionFragment__set_state(<GlycanCompositionFragment> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_GlycanCompositionFragment__set_state(GlycanCompositionFragment __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._hash_key = __pyx_state[0]; __pyx_result.composition = __pyx_state[1]; __pyx_result.is_extended = __pyx_state[2]; __pyx_result.key = __pyx_state[3]; __pyx_result.mass = __pyx_state[4]
 *     if len(__pyx_state) > 5 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_1 = PyCFunction_NewEx(&__pyx_mdef_14glycopeptidepy_2_c_9structure_22fragmentation_strategy_6glycan_7__pyx_unpickle_StubGlycopeptideStrategy, NULL, __pyx_n_s_glycopeptidepy__c_structure_frag); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_StubGlycopeptideS, __pyx_t_1) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/fragmentation_strategy/glycan.pyx":1
 * cimport cython             # <<<<<<<<<<<<<<
 * 
 * from cpython.ref cimport Py_INCREF
 */
  __pyx_t_1 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_1) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init glycopeptidepy._c.structure.fragmentation_strategy.glycan", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init glycopeptidepy._c.structure.fragmentation_strategy.glycan");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* IterFinish */
static CYTHON_INLINE int __Pyx_IterFinish(void) {
#if CYTHON_FAST_THREAD_STATE
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject* exc_type = tstate->curexc_type;
    if (unlikely(exc_type)) {
        if (likely(__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration))) {
            PyObject *exc_value, *exc_tb;
            exc_value = tstate->curexc_value;
            exc_tb = tstate->curexc_traceback;
            tstate->curexc_type = 0;
            tstate->curexc_value = 0;
            tstate->curexc_traceback = 0;
            Py_DECREF(exc_type);
            Py_XDECREF(exc_value);
            Py_XDECREF(exc_tb);
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#else
    if (unlikely(PyErr_Occurred())) {
        if (likely(PyErr_ExceptionMatches(PyExc_StopIteration))) {
            PyErr_Clear();
            return 0;
        } else {
            return -1;
        }
    }
    return 0;
#endif
}

/* UnpackItemEndCheck */
static int __Pyx_IternextUnpackEndCheck(PyObject *retval, Py_ssize_t expected) {
    if (unlikely(retval)) {
        Py_DECREF(retval);
        __Pyx_RaiseTooManyValuesError(expected);
        return -1;
    } else {
        return __Pyx_IterFinish();
    }
    return 0;
}

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* IterNext */
static PyObject *__Pyx_PyIter_Next2Default(PyObject* defval) {
    PyObject* exc_type;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    exc_type = __Pyx_PyErr_Occurred();
    if (unlikely(exc_type)) {
        if (!defval || unlikely(!__Pyx_PyErr_GivenExceptionMatches(exc_type, PyExc_StopIteration)))
            return NULL;
        __Pyx_PyErr_Clear();
        Py_INCREF(defval);
        return defval;
    }
    if (defval) {
        Py_INCREF(defval);
        return defval;
    }
    __Pyx_PyErr_SetNone(PyExc_StopIteration);
    return NULL;
}
static void __Pyx_PyIter_Next_ErrorNoIterator(PyObject *iterator) {
    PyErr_Format(PyExc_TypeError,
        "%.200s object is not an iterator", Py_TYPE(iterator)->tp_name);
}
static CYTHON_INLINE PyObject *__Pyx_PyIter_Next2(PyObject* iterator, PyObject* defval) {
    PyObject* next;
    iternextfunc iternext = Py_TYPE(iterator)->tp_iternext;
    if (likely(iternext)) {
#if CYTHON_USE_TYPE_SLOTS
        next = iternext(iterator);
        if (likely(next))
            return next;
        #if PY_VERSION_HEX >= 0x02070000
        if (unlikely(iternext == &_PyObject_NextNotImplemented))
            return NULL;
        #endif
#else
        next = PyIter_Next(iterator);
        if (likely(next))
            return next;
#endif
    } else if (CYTHON_USE_TYPE_SLOTS || unlikely(!PyIter_Check(iterator))) {
        __Pyx_PyIter_Next_ErrorNoIterator(iterator);
        return NULL;
    }
#if !CYTHON_USE_TYPE_SLOTS
    else {
        next = PyIter_Next(iterator);
        if (likely(next))
            return next;
    }
#endif
    return __Pyx_PyIter_Next2Default(defval);
}

/* SetItemInt */
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (!j) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return m->sq_ass_item(o, i, v);
        }
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (is_list || (PySequence_Check(o) && !PyDict_Check(o)))
#else
    if (is_list || PySequence_Check(o))
#endif
    {
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

/* None */
static CYTHON_INLINE void __Pyx_RaiseUnboundLocalError(const char *varname) {
    PyErr_Format(PyExc_UnboundLocalError, "local variable '%s' referenced before assignment", varname);
}

/* pyfrozenset_new */
static CYTHON_INLINE PyObject* __Pyx_PyFrozenSet_New(PyObject* it) {
    if (it) {
        PyObject* result;
#if CYTHON_COMPILING_IN_PYPY
        PyObject* args;
        args = PyTuple_Pack(1, it);
        if (unlikely(!args))
            return NULL;
        result = PyObject_Call((PyObject*)&PyFrozenSet_Type, args, NULL);
        Py_DECREF(args);
        return result;
#else
        if (PyFrozenSet_CheckExact(it)) {
            Py_INCREF(it);
            return it;
        }
        result = PyFrozenSet_New(it);
        if (unlikely(!result))
            return NULL;
        if ((PY_VERSION_HEX >= 0x031000A1) || likely(PySet_GET_SIZE(result)))
            return result;
        Py_DECREF(result);
#endif
    }
#if CYTHON_USE_TYPE_SLOTS
    return PyFrozenSet_Type.tp_new(&PyFrozenSet_Type, __pyx_empty_tuple, NULL);
#else
    return PyObject_Call((PyObject*)&PyFrozenSet_Type, __pyx_empty_tuple, NULL);
#endif
}

/* PySetContains */
static int __Pyx_PySet_ContainsUnhashable(PyObject *set, PyObject *key) {
    int result = -1;
    if (PySet_Check(key) && PyErr_ExceptionMatches(PyExc_TypeError)) {
        PyObject *tmpkey;
        PyErr_Clear();
        tmpkey = __Pyx_PyFrozenSet_New(key);
        if (tmpkey != NULL) {
            result = PySet_Contains(set, tmpkey);
            Py_DECREF(tmpkey);
        }
    }
    return result;
}
static CYTHON_INLINE int __Pyx_PySet_ContainsTF(PyObject* key, PyObject* set, int eq) {
    int result = PySet_Contains(set, key);
    if (unlikely(result < 0)) {
        result = __Pyx_PySet_ContainsUnhashable(set, key);
    }
    return unlikely(result < 0) ? result : (result == (eq == Py_EQ));
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* CallNextTpDealloc */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_dealloc != current_tp_dealloc)
        type = type->tp_base;
    while (type && type->tp_dealloc == current_tp_dealloc)
        type = type->tp_base;
    if (type)
        type->tp_dealloc(obj);
}

/* CallNextTpTraverse */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* FetchCommonType */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (descr != NULL) {
        *method = descr;
        return 0;
    }
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(name));
#endif
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod1 */
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
}

/* CoroutineBase */
#include <structmember.h>
#include <frameobject.h>
#define __Pyx_Coroutine_Undelegate(gen) Py_CLEAR((gen)->yieldfrom)
static int __Pyx_PyGen__FetchStopIterationValue(CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#if PY_VERSION_HEX >= 0x030300A0
        else if (Py_TYPE(ev) == (PyTypeObject*)PyExc_StopIteration) {
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            Py_DECREF(ev);
        }
#endif
        else if (unlikely(PyTuple_Check(ev))) {
            if (PyTuple_GET_SIZE(ev) >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#else
                value = PySequence_ITEM(ev, 0);
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if PY_VERSION_HEX >= 0x030300A0
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
    Py_DECREF(ev);
#else
    {
        PyObject* args = __Pyx_PyObject_GetAttrStr(ev, __pyx_n_s_args);
        Py_DECREF(ev);
        if (likely(args)) {
            value = PySequence_GetItem(args, 0);
            Py_DECREF(args);
        }
        if (unlikely(!value)) {
            __Pyx_ErrRestore(NULL, NULL, NULL);
            Py_INCREF(Py_None);
            value = Py_None;
        }
    }
#endif
    *pvalue = value;
    return 0;
}
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *t, *v, *tb;
    t = exc_state->exc_type;
    v = exc_state->exc_value;
    tb = exc_state->exc_traceback;
    exc_state->exc_type = NULL;
    exc_state->exc_value = NULL;
    exc_state->exc_traceback = NULL;
    Py_XDECREF(t);
    Py_XDECREF(v);
    Py_XDECREF(tb);
}
#define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyRunningError(CYTHON_UNUSED __pyx_CoroutineObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
        msg = "coroutine already executing";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
        msg = "async generator already executing";
    #endif
    } else {
        msg = "generator already executing";
    }
    PyErr_SetString(PyExc_ValueError, msg);
}
#define __Pyx_Coroutine_NotStartedError(gen)  (__Pyx__Coroutine_NotStartedError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_NotStartedError(CYTHON_UNUSED PyObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check(gen)) {
        msg = "can't send non-None value to a just-started coroutine";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact(gen)) {
        msg = "can't send non-None value to a just-started async generator";
    #endif
    } else {
        msg = "can't send non-None value to a just-started generator";
    }
    PyErr_SetString(PyExc_TypeError, msg);
}
#define __Pyx_Coroutine_AlreadyTerminatedError(gen, value, closing)  (__Pyx__Coroutine_AlreadyTerminatedError(gen, value, closing), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyTerminatedError(CYTHON_UNUSED PyObject *gen, PyObject *value, CYTHON_UNUSED int closing) {
    #ifdef __Pyx_Coroutine_USED
    if (!closing && __Pyx_Coroutine_Check(gen)) {
        PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
    } else
    #endif
    if (value) {
        #ifdef __Pyx_AsyncGen_USED
        if (__Pyx_AsyncGen_CheckExact(gen))
            PyErr_SetNone(__Pyx_PyExc_StopAsyncIteration);
        else
        #endif
        PyErr_SetNone(PyExc_StopIteration);
    }
}
static
PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, int closing) {
    __Pyx_PyThreadState_declare
    PyThreadState *tstate;
    __Pyx_ExcInfoStruct *exc_state;
    PyObject *retval;
    assert(!self->is_running);
    if (unlikely(self->resume_label == 0)) {
        if (unlikely(value && value != Py_None)) {
            return __Pyx_Coroutine_NotStartedError((PyObject*)self);
        }
    }
    if (unlikely(self->resume_label == -1)) {
        return __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
    }
#if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    tstate = __pyx_tstate;
#else
    tstate = __Pyx_PyThreadState_Current;
#endif
    exc_state = &self->gi_exc_state;
    if (exc_state->exc_type) {
        #if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
        #else
        if (exc_state->exc_traceback) {
            PyTracebackObject *tb = (PyTracebackObject *) exc_state->exc_traceback;
            PyFrameObject *f = tb->tb_frame;
            Py_XINCREF(tstate->frame);
            assert(f->f_back == NULL);
            f->f_back = tstate->frame;
        }
        #endif
    }
#if CYTHON_USE_EXC_INFO_STACK
    exc_state->previous_item = tstate->exc_info;
    tstate->exc_info = exc_state;
#else
    if (exc_state->exc_type) {
        __Pyx_ExceptionSwap(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(exc_state);
        __Pyx_ExceptionSave(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    }
#endif
    self->is_running = 1;
    retval = self->body((PyObject *) self, tstate, value);
    self->is_running = 0;
#if CYTHON_USE_EXC_INFO_STACK
    exc_state = &self->gi_exc_state;
    tstate->exc_info = exc_state->previous_item;
    exc_state->previous_item = NULL;
    __Pyx_Coroutine_ResetFrameBackpointer(exc_state);
#endif
    return retval;
}
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *exc_tb = exc_state->exc_traceback;
    if (likely(exc_tb)) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
#else
        PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
        PyFrameObject *f = tb->tb_frame;
        Py_CLEAR(f->f_back);
#endif
    }
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_MethodReturn(CYTHON_UNUSED PyObject* gen, PyObject *retval) {
    if (unlikely(!retval)) {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        if (!__Pyx_PyErr_Occurred()) {
            PyObject *exc = PyExc_StopIteration;
            #ifdef __Pyx_AsyncGen_USED
            if (__Pyx_AsyncGen_CheckExact(gen))
                exc = __Pyx_PyExc_StopAsyncIteration;
            #endif
            __Pyx_PyErr_SetNone(exc);
        }
    }
    return retval;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
static CYTHON_INLINE
PyObject *__Pyx_PyGen_Send(PyGenObject *gen, PyObject *arg) {
#if PY_VERSION_HEX <= 0x030A00A1
    return _PyGen_Send(gen, arg);
#else
    PyObject *result;
    if (PyIter_Send((PyObject*)gen, arg ? arg : Py_None, &result) == PYGEN_RETURN) {
        if (PyAsyncGen_CheckExact(gen)) {
            assert(result == Py_None);
            PyErr_SetNone(PyExc_StopAsyncIteration);
        }
        else if (result == Py_None) {
            PyErr_SetNone(PyExc_StopIteration);
        }
        else {
            _PyGen_SetStopIterationValue(result);
        }
        Py_CLEAR(result);
    }
    return result;
#endif
}
#endif
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen) {
    PyObject *ret;
    PyObject *val = NULL;
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
    ret = __Pyx_Coroutine_SendEx(gen, val, 0);
    Py_XDECREF(val);
    return ret;
}
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_AsyncGen_USED
        if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
            ret = __Pyx_async_gen_asend_send(yf, value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03050000 && defined(PyCoro_CheckExact) && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyCoro_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        {
            if (value == Py_None)
                ret = Py_TYPE(yf)->tp_iternext(yf);
            else
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_n_s_send, value);
        }
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        retval = __Pyx_Coroutine_FinishDelegation(gen);
    } else {
        retval = __Pyx_Coroutine_SendEx(gen, value, 0);
    }
    return __Pyx_Coroutine_MethodReturn(self, retval);
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    PyObject *retval = NULL;
    int err = 0;
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    if (__Pyx_CoroutineAwait_CheckExact(yf)) {
        retval = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf, NULL);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_AsyncGen_USED
    if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
        retval = __Pyx_async_gen_asend_close(yf, NULL);
    } else
    if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
        retval = __Pyx_async_gen_athrow_close(yf, NULL);
    } else
    #endif
    {
        PyObject *meth;
        gen->is_running = 1;
        meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_close);
        if (unlikely(!meth)) {
            if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                PyErr_WriteUnraisable(yf);
            }
            PyErr_Clear();
        } else {
            retval = PyObject_CallFunction(meth, NULL);
            Py_DECREF(meth);
            if (!retval)
                err = -1;
        }
        gen->is_running = 0;
    }
    Py_XDECREF(retval);
    return err;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, NULL);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, Py_None);
        } else
        #endif
            ret = Py_TYPE(yf)->tp_iternext(yf);
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        return __Pyx_Coroutine_FinishDelegation(gen);
    }
    return __Pyx_Coroutine_SendEx(gen, Py_None, 0);
}
static PyObject *__Pyx_Coroutine_Close_Method(PyObject *self, CYTHON_UNUSED PyObject *arg) {
    return __Pyx_Coroutine_Close(self);
}
static PyObject *__Pyx_Coroutine_Close(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *retval, *raised_exception;
    PyObject *yf = gen->yieldfrom;
    int err = 0;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    retval = __Pyx_Coroutine_SendEx(gen, NULL, 1);
    if (unlikely(retval)) {
        const char *msg;
        Py_DECREF(retval);
        if ((0)) {
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_Coroutine_Check(self)) {
            msg = "coroutine ignored GeneratorExit";
        #endif
        #ifdef __Pyx_AsyncGen_USED
        } else if (__Pyx_AsyncGen_CheckExact(self)) {
#if PY_VERSION_HEX < 0x03060000
            msg = "async generator ignored GeneratorExit - might require Python 3.6+ finalisation (PEP 525)";
#else
            msg = "async generator ignored GeneratorExit";
#endif
        #endif
        } else {
            msg = "generator ignored GeneratorExit";
        }
        PyErr_SetString(PyExc_RuntimeError, msg);
        return NULL;
    }
    raised_exception = PyErr_Occurred();
    if (likely(!raised_exception || __Pyx_PyErr_GivenExceptionMatches2(raised_exception, PyExc_GeneratorExit, PyExc_StopIteration))) {
        if (raised_exception) PyErr_Clear();
        Py_INCREF(Py_None);
        return Py_None;
    }
    return NULL;
}
static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                        PyObject *args, int close_on_genexit) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        Py_INCREF(yf);
        if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
            goto throw_here;
        }
        gen->is_running = 1;
        if (0
        #ifdef __Pyx_Generator_USED
            || __Pyx_Generator_CheckExact(yf)
        #endif
        #ifdef __Pyx_Coroutine_USED
            || __Pyx_Coroutine_Check(yf)
        #endif
            ) {
            ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
            ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
        #endif
        } else {
            PyObject *meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                    gen->is_running = 0;
                    return NULL;
                }
                PyErr_Clear();
                __Pyx_Coroutine_Undelegate(gen);
                gen->is_running = 0;
                goto throw_here;
            }
            if (likely(args)) {
                ret = PyObject_CallObject(meth, args);
            } else {
                ret = PyObject_CallFunctionObjArgs(meth, typ, val, tb, NULL);
            }
            Py_DECREF(meth);
        }
        gen->is_running = 0;
        Py_DECREF(yf);
        if (!ret) {
            ret = __Pyx_Coroutine_FinishDelegation(gen);
        }
        return __Pyx_Coroutine_MethodReturn(self, ret);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
    return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    PyObject *typ;
    PyObject *val = NULL;
    PyObject *tb = NULL;
    if (!PyArg_UnpackTuple(args, (char *)"throw", 1, 3, &typ, &val, &tb))
        return NULL;
    return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
}
static CYTHON_INLINE int __Pyx_Coroutine_traverse_excstate(__Pyx_ExcInfoStruct *exc_state, visitproc visit, void *arg) {
    Py_VISIT(exc_state->exc_type);
    Py_VISIT(exc_state->exc_value);
    Py_VISIT(exc_state->exc_traceback);
    return 0;
}
static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    return __Pyx_Coroutine_traverse_excstate(&gen->gi_exc_state, visit, arg);
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    Py_CLEAR(gen->yieldfrom);
    __Pyx_Coroutine_ExceptionClear(&gen->gi_exc_state);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
    }
#endif
    Py_CLEAR(gen->gi_code);
    Py_CLEAR(gen->gi_frame);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    Py_CLEAR(gen->gi_modulename);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label >= 0) {
        PyObject_GC_Track(self);
#if PY_VERSION_HEX >= 0x030400a1 && CYTHON_USE_TP_FINALIZE
        if (PyObject_CallFinalizerFromDealloc(self))
#else
        Py_TYPE(gen)->tp_del(self);
        if (Py_REFCNT(self) > 0)
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        /* We have to handle this case for asynchronous generators
           right here, because this code has to be between UNTRACK
           and GC_Del. */
        Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
    }
#endif
    __Pyx_Coroutine_clear(self);
    PyObject_GC_Del(gen);
}
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label < 0) {
        return;
    }
#if !CYTHON_USE_TP_FINALIZE
    assert(self->ob_refcnt == 0);
    __Pyx_SET_REFCNT(self, 1);
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
        PyObject *finalizer = agen->ag_finalizer;
        if (finalizer && !agen->ag_closed) {
            PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
            if (unlikely(!res)) {
                PyErr_WriteUnraisable(self);
            } else {
                Py_DECREF(res);
            }
            __Pyx_ErrRestore(error_type, error_value, error_traceback);
            return;
        }
    }
#endif
    if (unlikely(gen->resume_label == 0 && !error_value)) {
#ifdef __Pyx_Coroutine_USED
#ifdef __Pyx_Generator_USED
    if (!__Pyx_Generator_CheckExact(self))
#endif
        {
        PyObject_GC_UnTrack(self);
#if PY_MAJOR_VERSION >= 3  || defined(PyErr_WarnFormat)
        if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
            PyErr_WriteUnraisable(self);
#else
        {PyObject *msg;
        char *cmsg;
        #if CYTHON_COMPILING_IN_PYPY
        msg = NULL;
        cmsg = (char*) "coroutine was never awaited";
        #else
        char *cname;
        PyObject *qualname;
        qualname = gen->gi_qualname;
        cname = PyString_AS_STRING(qualname);
        msg = PyString_FromFormat("coroutine '%.50s' was never awaited", cname);
        if (unlikely(!msg)) {
            PyErr_Clear();
            cmsg = (char*) "coroutine was never awaited";
        } else {
            cmsg = PyString_AS_STRING(msg);
        }
        #endif
        if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, cmsg, 1) < 0))
            PyErr_WriteUnraisable(self);
        Py_XDECREF(msg);}
#endif
        PyObject_GC_Track(self);
        }
#endif
    } else {
        PyObject *res = __Pyx_Coroutine_Close(self);
        if (unlikely(!res)) {
            if (PyErr_Occurred())
                PyErr_WriteUnraisable(self);
        } else {
            Py_DECREF(res);
        }
    }
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
#if !CYTHON_USE_TP_FINALIZE
    assert(Py_REFCNT(self) > 0);
    if (--self->ob_refcnt == 0) {
        return;
    }
    {
        Py_ssize_t refcnt = Py_REFCNT(self);
        _Py_NewReference(self);
        __Pyx_SET_REFCNT(self, refcnt);
    }
#if CYTHON_COMPILING_IN_CPYTHON
    assert(PyType_IS_GC(Py_TYPE(self)) &&
           _Py_AS_GC(self)->gc.gc_refs != _PyGC_REFS_UNTRACKED);
    _Py_DEC_REFTOTAL;
#endif
#ifdef COUNT_ALLOCS
    --Py_TYPE(self)->tp_frees;
    --Py_TYPE(self)->tp_allocs;
#endif
#endif
}
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_name;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_name;
    Py_INCREF(value);
    self->gi_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_qualname;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_qualname;
    Py_INCREF(value);
    self->gi_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_frame(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *frame = self->gi_frame;
    if (!frame) {
        if (unlikely(!self->gi_code)) {
            Py_RETURN_NONE;
        }
        frame = (PyObject *) PyFrame_New(
            PyThreadState_Get(),            /*PyThreadState *tstate,*/
            (PyCodeObject*) self->gi_code,  /*PyCodeObject *code,*/
            __pyx_d,                 /*PyObject *globals,*/
            0                               /*PyObject *locals*/
        );
        if (unlikely(!frame))
            return NULL;
        self->gi_frame = frame;
    }
    Py_INCREF(frame);
    return frame;
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (unlikely(!gen))
        return NULL;
    return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->gi_exc_state.exc_type = NULL;
    gen->gi_exc_state.exc_value = NULL;
    gen->gi_exc_state.exc_traceback = NULL;
#if CYTHON_USE_EXC_INFO_STACK
    gen->gi_exc_state.previous_item = NULL;
#endif
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    Py_XINCREF(code);
    gen->gi_code = code;
    gen->gi_frame = NULL;
    PyObject_GC_Track(gen);
    return gen;
}

/* PatchModuleWithCoroutine */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

/* PatchGeneratorABC */
#ifndef CYTHON_REGISTER_ABCS
#define CYTHON_REGISTER_ABCS 1
#endif
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
static PyObject* __Pyx_patch_abc_module(PyObject *module);
static PyObject* __Pyx_patch_abc_module(PyObject *module) {
    module = __Pyx_Coroutine_patch_module(
        module, ""
"if _cython_generator_type is not None:\n"
"    try: Generator = _module.Generator\n"
"    except AttributeError: pass\n"
"    else: Generator.register(_cython_generator_type)\n"
"if _cython_coroutine_type is not None:\n"
"    try: Coroutine = _module.Coroutine\n"
"    except AttributeError: pass\n"
"    else: Coroutine.register(_cython_coroutine_type)\n"
    );
    return module;
}
#endif
static int __Pyx_patch_abc(void) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    static int abc_patched = 0;
    if (CYTHON_REGISTER_ABCS && !abc_patched) {
        PyObject *module;
        module = PyImport_ImportModule((PY_MAJOR_VERSION >= 3) ? "collections.abc" : "collections");
        if (!module) {
            PyErr_WriteUnraisable(NULL);
            if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning,
                    ((PY_MAJOR_VERSION >= 3) ?
                        "Cython module failed to register with collections.abc module" :
                        "Cython module failed to register with collections module"), 1) < 0)) {
                return -1;
            }
        } else {
            module = __Pyx_patch_abc_module(module);
            abc_patched = 1;
            if (unlikely(!module))
                return -1;
            Py_DECREF(module);
        }
        module = PyImport_ImportModule("backports_abc");
        if (module) {
            module = __Pyx_patch_abc_module(module);
            Py_XDECREF(module);
        }
        if (!module) {
            PyErr_Clear();
        }
    }
#else
    if ((0)) __Pyx_Coroutine_patch_module(NULL, NULL);
#endif
    return 0;
}

/* Generator */
static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     (char*) PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     (char*) PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS,
     (char*) PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {(char *) "gi_running", T_BOOL, offsetof(__pyx_CoroutineObject, is_running), READONLY, NULL},
    {(char*) "gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     (char*) PyDoc_STR("object being iterated by 'yield from', or None")},
    {(char*) "gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {(char *) "__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     (char*) PyDoc_STR("name of the generator"), 0},
    {(char *) "__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     (char*) PyDoc_STR("qualified name of the generator"), 0},
    {(char *) "gi_frame", (getter)__Pyx_Coroutine_get_frame, NULL,
     (char*) PyDoc_STR("Frame of the generator"), 0},
    {0, 0, 0, 0, 0}
};
static PyTypeObject __pyx_GeneratorType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    (destructor) __Pyx_Coroutine_dealloc,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    0,
    (traverseproc) __Pyx_Coroutine_traverse,
    0,
    0,
    offsetof(__pyx_CoroutineObject, gi_weakreflist),
    0,
    (iternextfunc) __Pyx_Generator_Next,
    __pyx_Generator_methods,
    __pyx_Generator_memberlist,
    __pyx_Generator_getsets,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if CYTHON_USE_TP_FINALIZE
    0,
#else
    __Pyx_Coroutine_del,
#endif
    0,
#if CYTHON_USE_TP_FINALIZE
    __Pyx_Coroutine_del,
#elif PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
};
static int __pyx_Generator_init(void) {
    __pyx_GeneratorType_type.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
    __pyx_GeneratorType_type.tp_iter = PyObject_SelfIter;
    __pyx_GeneratorType = __Pyx_FetchCommonType(&__pyx_GeneratorType_type);
    if (unlikely(!__pyx_GeneratorType)) {
        return -1;
    }
    return 0;
}

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* VoidPtrImport */
#ifndef __PYX_HAVE_RT_ImportVoidPtr
#define __PYX_HAVE_RT_ImportVoidPtr
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, desc);
        goto bad;
    }
    *p = PyCObject_AsVoidPtr(cobj);}
#endif
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* FunctionImport */
#ifndef __PYX_HAVE_RT_ImportFunction
#define __PYX_HAVE_RT_ImportFunction
static int __Pyx_ImportFunction(PyObject *module, const char *funcname, void (**f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, funcname);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C function %.200s",
                PyModule_GetName(module), funcname);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    tmp.p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C function %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), funcname, sig, desc);
        goto bad;
    }
    tmp.p = PyCObject_AsVoidPtr(cobj);}
#endif
    *f = tmp.fp;
    if (!(*f))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
