/* Generated by Cython 0.29.24 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [],
        "name": "glycopeptidepy._c.structure.sequence_methods",
        "sources": [
            "glycopeptidepy/_c/structure/sequence_methods.pyx"
        ]
    },
    "module_name": "glycopeptidepy._c.structure.sequence_methods"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_24"
#define CYTHON_HEX_VERSION 0x001D18F0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
#if PY_VERSION_HEX >= 0x030800A4 && PY_VERSION_HEX < 0x030800B2
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, 0, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__glycopeptidepy___c__structure__sequence_methods
#define __PYX_HAVE_API__glycopeptidepy___c__structure__sequence_methods
/* Early includes */
#include <string.h>
#include <stdio.h>
#include "pythread.h"
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "glycopeptidepy/_c/structure/sequence_methods.pyx",
  "stringsource",
  "ccomposition.pxd",
  "glycopeptidepy/_c/structure/base.pxd",
  "type.pxd",
  "bool.pxd",
  "complex.pxd",
  "cenum.pxd",
  "glycopeptidepy/_c/structure/modification/rule.pxd",
  "glycopeptidepy/_c/structure/modification/source.pxd",
  "glycopeptidepy/_c/structure/modification/modification.pxd",
  "glycopeptidepy/_c/structure/constants.pxd",
  "glycopeptidepy/_c/structure/glycan.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition;
struct __pyx_obj_5glypy_5utils_5cenum_EnumValue;
struct __pyx_obj_5glypy_5utils_5cenum_IntEnumValue;
struct __pyx_obj_5glypy_5utils_5cenum_EnumMeta;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_NeutralLossBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_12modification_ModificationInstanceBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_9constants_Configuration;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycanCompositionWithOffsetProxyBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore;
struct __pyx_opt_args_5glypy_11composition_12ccomposition_12CComposition_calc_mass;
struct __pyx_opt_args_5glypy_11composition_12ccomposition_calculate_mass;

/* "glypy/composition/ccomposition.pxd":6
 *     cdef tuple _mass_args
 *     cpdef CComposition clone(self)
 *     cpdef double calc_mass(self, int average=?, charge=?, dict mass_data=?) except -1             # <<<<<<<<<<<<<<
 *     cpdef _from_formula(self, str formula, dict mass_data)
 *     cpdef _from_dict(self, comp)
 */
struct __pyx_opt_args_5glypy_11composition_12ccomposition_12CComposition_calc_mass {
  int __pyx_n;
  int average;
  PyObject *charge;
  PyObject *mass_data;
};

/* "glypy/composition/ccomposition.pxd":31
 * 
 * 
 * cpdef double calculate_mass(CComposition composition=?, str formula=?, int average=?, charge=?, mass_data=?) except -1             # <<<<<<<<<<<<<<
 */
struct __pyx_opt_args_5glypy_11composition_12ccomposition_calculate_mass {
  int __pyx_n;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition;
  PyObject *formula;
  int average;
  PyObject *charge;
  PyObject *mass_data;
};
struct __pyx_opt_args_5glypy_5utils_5cenum_9EnumValue_add_name;

/* "glypy/utils/cenum.pxd":10
 *         public object _hash
 * 
 *     cpdef add_name(self, basestring name, bint force=*)             # <<<<<<<<<<<<<<
 *     cpdef int int_value(self) except -1
 * 
 */
struct __pyx_opt_args_5glypy_5utils_5cenum_9EnumValue_add_name {
  int __pyx_n;
  int force;
};
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_6glycan_20GlycosylationManager_pop;

/* "glycopeptidepy/_c/structure/glycan.pxd":25
 *     cpdef values(self)
 *     cpdef items(self)
 *     cpdef pop(self, key, default=*)             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t get_size(self)
 */
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_6glycan_20GlycosylationManager_pop {
  int __pyx_n;
  PyObject *__pyx_default;
};
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods__make_terminal_group;
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_parsed_string;
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_components;
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_sequence;

/* "glycopeptidepy/_c/structure/sequence_methods.pxd":12
 * cdef AminoAcidResidueBase _parse_residue(basestring residue_string)
 * 
 * cdef TerminalGroup _make_terminal_group(basestring base_composition_formula, ModificationBase modification=*)             # <<<<<<<<<<<<<<
 * 
 * 
 */
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods__make_terminal_group {
  int __pyx_n;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *modification;
};

/* "glycopeptidepy/_c/structure/sequence_methods.pxd":17
 * cdef class _PeptideSequenceCore(PeptideSequenceBase):
 * 
 *     cpdef _init_from_parsed_string(self, list seq_list, glycan=*, n_term=*, c_term=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef _init_from_string(self, sequence, parser_function)
 */
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_parsed_string {
  int __pyx_n;
  PyObject *glycan;
  PyObject *n_term;
  PyObject *c_term;
};

/* "glycopeptidepy/_c/structure/sequence_methods.pxd":21
 *     cpdef _init_from_string(self, sequence, parser_function)
 * 
 *     cpdef _init_from_components(self, list seq_list, glycan_composition=*, ModificationBase n_term=*, ModificationBase c_term=*)             # <<<<<<<<<<<<<<
 * 
 *     cpdef _init_termini(self, TerminalGroup n_term, TerminalGroup c_term)
 */
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_components {
  int __pyx_n;
  PyObject *glycan_composition;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *n_term;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *c_term;
};

/* "glycopeptidepy/_c/structure/sequence_methods.pxd":33
 *     cdef void set_c_term(self, _value)
 * 
 *     cpdef basestring get_sequence(self, bint include_glycan=*, bint include_termini=*, str implicit_n_term=*, str implicit_c_term=*)             # <<<<<<<<<<<<<<
 *     cpdef clone(self)
 * 
 */
struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_sequence {
  int __pyx_n;
  int include_glycan;
  int include_termini;
  PyObject *implicit_n_term;
  PyObject *implicit_c_term;
};

/* "glypy/composition/ccomposition.pxd":2
 * 
 * cdef class CComposition(dict):             # <<<<<<<<<<<<<<
 *     cdef object _mass
 *     cdef tuple _mass_args
 */
struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition {
  PyDictObject __pyx_base;
  struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *__pyx_vtab;
  PyObject *_mass;
  PyObject *_mass_args;
};


/* "glycopeptidepy/_c/structure/base.pxd":5
 * 
 * 
 * cdef class PeptideSequenceBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public list sequence
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_vtab;
  PyObject *sequence;
  double _mass;
  PyObject *_glycosylation_manager;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *_n_term;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *_c_term;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *_total_composition;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *_peptide_composition;
};


/* "glycopeptidepy/_c/structure/base.pxd":21
 * 
 * 
 * cdef class TerminalGroup(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public CComposition base_composition
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_vtab;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *base_composition;
  double mass;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *_modification;
};


/* "glycopeptidepy/_c/structure/base.pxd":35
 * 
 * 
 * cdef class AminoAcidResidueBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public basestring name
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_vtab;
  PyObject *name;
  PyObject *symbol;
  double mass;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition;
  PyObject *neutral_loss;
  Py_hash_t _hash;
};


/* "glycopeptidepy/_c/structure/base.pxd":50
 * 
 * 
 * cdef class ModificationBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public basestring name
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_vtab;
  PyObject *name;
  double mass;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition;
};


/* "glycopeptidepy/_c/structure/base.pxd":69
 * 
 * 
 * cdef class SequencePosition(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public AminoAcidResidueBase amino_acid
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_vtab;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *amino_acid;
  PyObject *modifications;
};


/* "glypy/utils/cenum.pxd":2
 * 
 * cdef class EnumValue(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public object group
 */
struct __pyx_obj_5glypy_5utils_5cenum_EnumValue {
  PyObject_HEAD
  struct __pyx_vtabstruct_5glypy_5utils_5cenum_EnumValue *__pyx_vtab;
  PyObject *group;
  PyObject *name;
  PyObject *value;
  PyObject *names;
  PyObject *_hash;
};


/* "glypy/utils/cenum.pxd":14
 * 
 * 
 * cdef class IntEnumValue(EnumValue):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public int _int_value
 */
struct __pyx_obj_5glypy_5utils_5cenum_IntEnumValue {
  struct __pyx_obj_5glypy_5utils_5cenum_EnumValue __pyx_base;
  int _int_value;
};


/* "glypy/utils/cenum.pxd":19
 * 
 * 
 * cdef class EnumMeta(type):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public dict name_map
 */
struct __pyx_obj_5glypy_5utils_5cenum_EnumMeta {
  PyHeapTypeObject __pyx_base;
  struct __pyx_vtabstruct_5glypy_5utils_5cenum_EnumMeta *__pyx_vtab;
  PyObject *name_map;
  PyObject *value_map;
};


/* "glycopeptidepy/_c/structure/modification/rule.pxd":5
 * from glypy.utils.cenum cimport EnumValue
 * 
 * cdef class ModificationRuleBase(ModificationBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public set names
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase __pyx_base;
  PyObject *names;
  PyObject *categories;
  PyObject *preferred_name;
  PyObject *title;
  PyObject *common_name;
  PyObject *unimod_name;
  Py_hash_t _hash;
};


/* "glycopeptidepy/_c/structure/modification/rule.pxd":22
 * 
 * 
 * cdef class NeutralLossBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public CComposition composition
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_NeutralLossBase {
  PyObject_HEAD
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition;
  double mass;
  PyObject *label;
};


/* "glycopeptidepy/_c/structure/modification/rule.pxd":32
 * 
 * 
 * cdef class GlycosylationBase(ModificationRuleBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public bint _is_composition
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase __pyx_base;
  int _is_composition;
  int _is_core;
  struct __pyx_obj_5glypy_5utils_5cenum_EnumValue *_glycosylation_type;
};


/* "glycopeptidepy/_c/structure/modification/source.pxd":3
 * from glycopeptidepy._c.structure.modification.rule cimport ModificationRuleBase
 * 
 * cdef class ModificationTableBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public dict store
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase *__pyx_vtab;
  PyObject *store;
  PyObject *_instance_custom_rules;
};


/* "glycopeptidepy/_c/structure/modification/modification.pxd":5
 * 
 * 
 * cdef class ModificationInstanceBase(ModificationBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public ModificationRuleBase rule
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_12modification_ModificationInstanceBase {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase __pyx_base;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase *rule;
  Py_hash_t _hash;
};


/* "glycopeptidepy/_c/structure/constants.pxd":2
 * 
 * cdef class Configuration(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public str N_TERM_DEFAULT
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_9constants_Configuration {
  PyObject_HEAD
  PyObject *N_TERM_DEFAULT;
  PyObject *C_TERM_DEFAULT;
  int FRAG_OFFSET;
  int ALLOW_MODIFIED_ASPARAGINE;
  int PARTIAL_HEXNAC_LOSS;
  int EXCLUDE_B1;
};


/* "glycopeptidepy/_c/structure/glycan.pxd":5
 * from glycopeptidepy._c.structure.base cimport PeptideSequenceBase
 * 
 * cdef class GlycanCompositionWithOffsetProxyBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public object obj
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycanCompositionWithOffsetProxyBase {
  PyObject_HEAD
  PyObject *obj;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition_offset;
};


/* "glycopeptidepy/_c/structure/glycan.pxd":11
 * 
 * 
 * cdef class GlycosylationManager(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public dict mapping
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *__pyx_vtab;
  PyObject *mapping;
  PyObject *parent;
  PyObject *_aggregate;
  PyObject *_proxy;
  PyObject *_type_track;
  int _total_glycosylation_size;
};


/* "glycopeptidepy/_c/structure/sequence_methods.pxd":15
 * 
 * 
 * cdef class _PeptideSequenceCore(PeptideSequenceBase):             # <<<<<<<<<<<<<<
 * 
 *     cpdef _init_from_parsed_string(self, list seq_list, glycan=*, n_term=*, c_term=*)
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase __pyx_base;
};



/* "glypy/composition/ccomposition.pxd":2
 * 
 * cdef class CComposition(dict):             # <<<<<<<<<<<<<<
 *     cdef object _mass
 *     cdef tuple _mass_args
 */

struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition {
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*clone)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, int __pyx_skip_dispatch);
  double (*calc_mass)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, int __pyx_skip_dispatch, struct __pyx_opt_args_5glypy_11composition_12ccomposition_12CComposition_calc_mass *__pyx_optional_args);
  PyObject *(*_from_formula)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*_from_dict)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *, int __pyx_skip_dispatch);
  long (*getitem)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *);
  void (*setitem)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *, long);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*add_from)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*subtract_from)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*_create)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *);
};
static struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *__pyx_vtabptr_5glypy_11composition_12ccomposition_CComposition;


/* "glycopeptidepy/_c/structure/base.pxd":5
 * 
 * 
 * cdef class PeptideSequenceBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public list sequence
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *(*get)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *, Py_ssize_t);
  PyObject *(*_invalidate)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase;


/* "glycopeptidepy/_c/structure/base.pxd":21
 * 
 * 
 * cdef class TerminalGroup(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public CComposition base_composition
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *(*get_modification)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*get_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *(*modify)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *, int __pyx_skip_dispatch);
  void (*set_modification)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *);
  int (*equal_to)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, PyObject *);
  PyObject *(*serialize)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_TerminalGroup;


/* "glycopeptidepy/_c/structure/base.pxd":35
 * 
 * 
 * cdef class AminoAcidResidueBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public basestring name
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *(*_create)(PyObject *, PyObject *, double, struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *);
  int (*equal_to)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase;


/* "glycopeptidepy/_c/structure/base.pxd":50
 * 
 * 
 * cdef class ModificationBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public basestring name
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase {
  int (*is_a)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*serialize)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *, int __pyx_skip_dispatch);
  int (*is_tracked_for)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_ModificationBase;


/* "glycopeptidepy/_c/structure/base.pxd":69
 * 
 * 
 * cdef class SequencePosition(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public AminoAcidResidueBase amino_acid
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_SequencePosition {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *(*_create)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *, PyObject *);
  double (*get_mass)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *);
  int (*has_modification)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *, PyObject *, int __pyx_skip_dispatch);
  int (*is_modified)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *, int __pyx_skip_dispatch);
  PyObject *(*drop_modification)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*add_modification)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_SequencePosition;


/* "glypy/utils/cenum.pxd":2
 * 
 * cdef class EnumValue(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public object group
 */

struct __pyx_vtabstruct_5glypy_5utils_5cenum_EnumValue {
  PyObject *(*add_name)(struct __pyx_obj_5glypy_5utils_5cenum_EnumValue *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_5glypy_5utils_5cenum_9EnumValue_add_name *__pyx_optional_args);
  int (*int_value)(struct __pyx_obj_5glypy_5utils_5cenum_EnumValue *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_5glypy_5utils_5cenum_EnumValue *__pyx_vtabptr_5glypy_5utils_5cenum_EnumValue;


/* "glypy/utils/cenum.pxd":14
 * 
 * 
 * cdef class IntEnumValue(EnumValue):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public int _int_value
 */

struct __pyx_vtabstruct_5glypy_5utils_5cenum_IntEnumValue {
  struct __pyx_vtabstruct_5glypy_5utils_5cenum_EnumValue __pyx_base;
};
static struct __pyx_vtabstruct_5glypy_5utils_5cenum_IntEnumValue *__pyx_vtabptr_5glypy_5utils_5cenum_IntEnumValue;


/* "glypy/utils/cenum.pxd":19
 * 
 * 
 * cdef class EnumMeta(type):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public dict name_map
 */

struct __pyx_vtabstruct_5glypy_5utils_5cenum_EnumMeta {
  struct __pyx_obj_5glypy_5utils_5cenum_EnumValue *(*name)(struct __pyx_obj_5glypy_5utils_5cenum_EnumMeta *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*translate)(struct __pyx_obj_5glypy_5utils_5cenum_EnumMeta *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_5glypy_5utils_5cenum_EnumValue *(*get)(struct __pyx_obj_5glypy_5utils_5cenum_EnumMeta *, PyObject *);
  int (*put)(struct __pyx_obj_5glypy_5utils_5cenum_EnumMeta *, PyObject *, PyObject *);
};
static struct __pyx_vtabstruct_5glypy_5utils_5cenum_EnumMeta *__pyx_vtabptr_5glypy_5utils_5cenum_EnumMeta;


/* "glycopeptidepy/_c/structure/modification/rule.pxd":5
 * from glypy.utils.cenum cimport EnumValue
 * 
 * cdef class ModificationRuleBase(ModificationBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public set names
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase {
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase __pyx_base;
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase;


/* "glycopeptidepy/_c/structure/modification/rule.pxd":32
 * 
 * 
 * cdef class GlycosylationBase(ModificationRuleBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public bint _is_composition
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase {
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase __pyx_base;
  int (*get_is_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase *);
  void (*set_is_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase *, int);
  int (*get_is_core)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase *);
  void (*set_is_core)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase *, int);
  struct __pyx_obj_5glypy_5utils_5cenum_EnumValue *(*get_glycosylation_type)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase *);
  void (*set_glycosylation_type)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase *, struct __pyx_obj_5glypy_5utils_5cenum_EnumValue *);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase;


/* "glycopeptidepy/_c/structure/modification/source.pxd":3
 * from glycopeptidepy._c.structure.modification.rule cimport ModificationRuleBase
 * 
 * cdef class ModificationTableBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public dict store
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase *(*resolve)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*get_modification)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase *, PyObject *, int __pyx_skip_dispatch);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase *(*_find_modification_rule)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase *, PyObject *);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase *(*_find_modification_rule_fuzzy)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase *, PyObject *);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase;


/* "glycopeptidepy/_c/structure/modification/modification.pxd":5
 * 
 * 
 * cdef class ModificationInstanceBase(ModificationBase):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public ModificationRuleBase rule
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_12modification_12modification_ModificationInstanceBase {
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase __pyx_base;
  PyObject *(*_init_from_rule)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_12modification_ModificationInstanceBase *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_12modification_12modification_ModificationInstanceBase *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_12modification_12modification_ModificationInstanceBase;


/* "glycopeptidepy/_c/structure/glycan.pxd":11
 * 
 * 
 * cdef class GlycosylationManager(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public dict mapping
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager {
  PyObject *(*update)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*clear)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  PyObject *(*keys)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  PyObject *(*values)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  PyObject *(*items)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  PyObject *(*pop)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_6glycan_20GlycosylationManager_pop *__pyx_optional_args);
  size_t (*get_size)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *(*copy)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *(*clone)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  PyObject *(*invalidate)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  PyObject *(*get_aggregate)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *);
  int (*set_aggregate)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, PyObject *);
  PyObject *(*_patch_aggregate)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  PyObject *(*get_glycosylation_types)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *);
  PyObject *(*_update_track)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*total_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  double (*mass)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  int (*is_fully_specified_topologies)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, int __pyx_skip_dispatch);
  PyObject *(*get_glycan_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *);
  PyObject *(*_make_glycan_composition_proxy)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *);
  int (*total_glycosylation_size)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *);
  int (*count_glycosylation_type)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *, PyObject *, int __pyx_skip_dispatch);
  void (*_init)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *(*_create)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *, PyObject *);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager;


/* "glycopeptidepy/_c/structure/sequence_methods.pyx":151
 * 
 * 
 * cdef class _PeptideSequenceCore(PeptideSequenceBase):             # <<<<<<<<<<<<<<
 *     def __init__(self, sequence=None, parser_function=None, **kwargs):
 *         if parser_function is None:
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore {
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase __pyx_base;
  PyObject *(*_init_from_parsed_string)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_parsed_string *__pyx_optional_args);
  PyObject *(*_init_from_string)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*_init_from_components)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_components *__pyx_optional_args);
  PyObject *(*_init_termini)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, int __pyx_skip_dispatch);
  PyObject *(*_initialize_fields)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch);
  size_t (*get_size)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *(*get_n_term)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *);
  void (*set_n_term)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *(*get_c_term)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *);
  void (*set_c_term)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *);
  PyObject *(*get_sequence)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_sequence *__pyx_optional_args);
  PyObject *(*clone)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch);
  int (*base_sequence_equality)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch);
  int (*modified_sequence_equality)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch);
  int (*full_structure_equality)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*peptide_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*total_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* SetItemInt.proto */
#define __Pyx_SetItemInt(o, i, v, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_SetItemInt_Fast(o, (Py_ssize_t)i, v, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list assignment index out of range"), -1) :\
               __Pyx_SetItemInt_Generic(o, to_py_func(i), v)))
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v);
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v,
                                               int is_list, int wraparound, int boundscheck);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* StrEquals.proto */
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyString_Equals __Pyx_PyUnicode_Equals
#else
#define __Pyx_PyString_Equals __Pyx_PyBytes_Equals
#endif

/* RaiseTooManyValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected);

/* RaiseNeedMoreValuesToUnpack.proto */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index);

/* RaiseNoneIterError.proto */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void);

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* ObjectGetItem.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key);
#else
#define __Pyx_PyObject_GetItem(obj, key)  PyObject_GetItem(obj, key)
#endif

/* StringJoin.proto */
#if PY_MAJOR_VERSION < 3
#define __Pyx_PyString_Join __Pyx_PyBytes_Join
#define __Pyx_PyBaseString_Join(s, v) (PyUnicode_CheckExact(s) ? PyUnicode_Join(s, v) : __Pyx_PyBytes_Join(s, v))
#else
#define __Pyx_PyString_Join PyUnicode_Join
#define __Pyx_PyBaseString_Join PyUnicode_Join
#endif
#if CYTHON_COMPILING_IN_CPYTHON
    #if PY_MAJOR_VERSION < 3
    #define __Pyx_PyBytes_Join _PyString_Join
    #else
    #define __Pyx_PyBytes_Join _PyBytes_Join
    #endif
#else
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values);
#endif

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* CallNextTpDealloc.proto */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc);

/* CallNextTpTraverse.proto */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse);

/* CallNextTpClear.proto */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_dealloc);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* CythonFunctionShared.proto */
#define __Pyx_CyFunction_USED 1
#define __Pyx_CYFUNCTION_STATICMETHOD  0x01
#define __Pyx_CYFUNCTION_CLASSMETHOD   0x02
#define __Pyx_CYFUNCTION_CCLASS        0x04
#define __Pyx_CyFunction_GetClosure(f)\
    (((__pyx_CyFunctionObject *) (f))->func_closure)
#define __Pyx_CyFunction_GetClassObj(f)\
    (((__pyx_CyFunctionObject *) (f))->func_classobj)
#define __Pyx_CyFunction_Defaults(type, f)\
    ((type *)(((__pyx_CyFunctionObject *) (f))->defaults))
#define __Pyx_CyFunction_SetDefaultsGetter(f, g)\
    ((__pyx_CyFunctionObject *) (f))->defaults_getter = (g)
typedef struct {
    PyCFunctionObject func;
#if PY_VERSION_HEX < 0x030500A0
    PyObject *func_weakreflist;
#endif
    PyObject *func_dict;
    PyObject *func_name;
    PyObject *func_qualname;
    PyObject *func_doc;
    PyObject *func_globals;
    PyObject *func_code;
    PyObject *func_closure;
    PyObject *func_classobj;
    void *defaults;
    int defaults_pyobjects;
    size_t defaults_size;  // used by FusedFunction for copying defaults
    int flags;
    PyObject *defaults_tuple;
    PyObject *defaults_kwdict;
    PyObject *(*defaults_getter)(PyObject *);
    PyObject *func_annotations;
} __pyx_CyFunctionObject;
static PyTypeObject *__pyx_CyFunctionType = 0;
#define __Pyx_CyFunction_Check(obj)  (__Pyx_TypeCheck(obj, __pyx_CyFunctionType))
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject* op, PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *self,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *m,
                                                         size_t size,
                                                         int pyobjects);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *m,
                                                            PyObject *tuple);
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *m,
                                                             PyObject *dict);
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *m,
                                                              PyObject *dict);
static int __pyx_CyFunction_init(void);

/* CythonFunction.proto */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml,
                                      int flags, PyObject* qualname,
                                      PyObject *closure,
                                      PyObject *module, PyObject *globals,
                                      PyObject* code);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* FunctionExport.proto */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_parsed_string(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_seq_list, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_parsed_string *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_string(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_sequence, PyObject *__pyx_v_parser_function, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_components(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_seq_list, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_components *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_termini(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_n_term, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_c_term, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__initialize_fields(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__invalidate(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_n_term(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto*/
static void __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_set_n_term(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v__value); /* proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_c_term(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto*/
static void __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_set_c_term(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v__value); /* proto*/
static struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_peptide_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_total_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static size_t __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_size(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_base_sequence_equality(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_other, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_modified_sequence_equality(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_other, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_full_structure_equality(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_other, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_sequence(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_sequence *__pyx_optional_args); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_clone(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/

/* Module declarations from 'cython' */

/* Module declarations from 'glypy.composition.ccomposition' */
static PyTypeObject *__pyx_ptype_5glypy_11composition_12ccomposition_CComposition = 0;
static PyObject **__pyx_vp_5glypy_11composition_12ccomposition__atom = 0;
#define __pyx_v_5glypy_11composition_12ccomposition__atom (*__pyx_vp_5glypy_11composition_12ccomposition__atom)
static PyObject **__pyx_vp_5glypy_11composition_12ccomposition__formula = 0;
#define __pyx_v_5glypy_11composition_12ccomposition__formula (*__pyx_vp_5glypy_11composition_12ccomposition__formula)
static PyObject **__pyx_vp_5glypy_11composition_12ccomposition__isotope_string = 0;
#define __pyx_v_5glypy_11composition_12ccomposition__isotope_string (*__pyx_vp_5glypy_11composition_12ccomposition__isotope_string)
static PyObject **__pyx_vp_5glypy_11composition_12ccomposition_isotope_pattern = 0;
#define __pyx_v_5glypy_11composition_12ccomposition_isotope_pattern (*__pyx_vp_5glypy_11composition_12ccomposition_isotope_pattern)
static PyObject **__pyx_vp_5glypy_11composition_12ccomposition_formula_pattern = 0;
#define __pyx_v_5glypy_11composition_12ccomposition_formula_pattern (*__pyx_vp_5glypy_11composition_12ccomposition_formula_pattern)

/* Module declarations from 'glycopeptidepy._c.structure.base' */
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_SequencePosition = 0;

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython.version' */

/* Module declarations from 'cpython.exc' */

/* Module declarations from 'cpython.module' */

/* Module declarations from 'cpython.mem' */

/* Module declarations from 'cpython.tuple' */

/* Module declarations from 'cpython.list' */

/* Module declarations from 'cpython.sequence' */

/* Module declarations from 'cpython.mapping' */

/* Module declarations from 'cpython.iterator' */

/* Module declarations from 'cpython.number' */

/* Module declarations from 'cpython.int' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.bool' */
static PyTypeObject *__pyx_ptype_7cpython_4bool_bool = 0;

/* Module declarations from 'cpython.long' */

/* Module declarations from 'cpython.float' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.complex' */
static PyTypeObject *__pyx_ptype_7cpython_7complex_complex = 0;

/* Module declarations from 'cpython.string' */

/* Module declarations from 'cpython.unicode' */

/* Module declarations from 'cpython.dict' */

/* Module declarations from 'cpython.instance' */

/* Module declarations from 'cpython.function' */

/* Module declarations from 'cpython.method' */

/* Module declarations from 'cpython.weakref' */

/* Module declarations from 'cpython.getargs' */

/* Module declarations from 'cpython.pythread' */

/* Module declarations from 'cpython.pystate' */

/* Module declarations from 'cpython.cobject' */

/* Module declarations from 'cpython.oldbuffer' */

/* Module declarations from 'cpython.set' */

/* Module declarations from 'cpython.buffer' */

/* Module declarations from 'cpython.bytes' */

/* Module declarations from 'cpython.pycapsule' */

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.ref' */

/* Module declarations from 'glypy.utils.cenum' */
static PyTypeObject *__pyx_ptype_5glypy_5utils_5cenum_EnumValue = 0;
static PyTypeObject *__pyx_ptype_5glypy_5utils_5cenum_IntEnumValue = 0;
static PyTypeObject *__pyx_ptype_5glypy_5utils_5cenum_EnumMeta = 0;
static PyObject **__pyx_vp_5glypy_5utils_5cenum_QMARK = 0;
#define __pyx_v_5glypy_5utils_5cenum_QMARK (*__pyx_vp_5glypy_5utils_5cenum_QMARK)

/* Module declarations from 'glycopeptidepy._c.structure.modification.rule' */
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_4rule_NeutralLossBase = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase = 0;

/* Module declarations from 'glycopeptidepy._c.structure.modification.source' */
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase = 0;

/* Module declarations from 'glycopeptidepy._c.structure.modification.modification' */
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_12modification_ModificationInstanceBase = 0;

/* Module declarations from 'glycopeptidepy._c.structure.constants' */
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_9constants_Configuration = 0;

/* Module declarations from 'glycopeptidepy._c.structure.glycan' */
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_6glycan_GlycanCompositionWithOffsetProxyBase = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager = 0;

/* Module declarations from 'glycopeptidepy._c.structure.sequence_methods' */
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore = 0;
static struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_WATER = 0;
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_9constants_Configuration *__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_structure_constants = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationImpl = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_AminoAcidResidueImpl = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_SequenceLocation_n_term = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_SequenceLocation_c_term = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationCategory_glycosylation = 0;
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase *__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_modification_table = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_terminal_group_cache = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_amino_acid_cache = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_implicit_terminal_cache = 0;
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__parse_residue(PyObject *); /*proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__make_terminal_group(PyObject *, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods__make_terminal_group *__pyx_optional_args); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_add_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *, PyObject *, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_get_modification_by_name(PyObject *, int __pyx_skip_dispatch); /*proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__get_implicit_terminal(PyObject *); /*proto*/
static struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__total_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods___pyx_unpickle__PeptideSequenceCore__set_state(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "glycopeptidepy._c.structure.sequence_methods"
extern int __pyx_module_is_main_glycopeptidepy___c__structure__sequence_methods;
int __pyx_module_is_main_glycopeptidepy___c__structure__sequence_methods = 0;

/* Implementation of 'glycopeptidepy._c.structure.sequence_methods' */
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_AttributeError;
static PyObject *__pyx_builtin_IndexError;
static const char __pyx_k_[] = "";
static const char __pyx_k_0[] = "({0})-";
static const char __pyx_k_s[] = "(%s)";
static const char __pyx_k__2[] = "|";
static const char __pyx_k_0_2[] = "-({0})";
static const char __pyx_k_H2O[] = "H2O";
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_s_2[] = "(%s)-";
static const char __pyx_k_s_3[] = "-(%s)";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_join[] = "join";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_mass[] = "_mass";
static const char __pyx_k_name[] = "__name__";
static const char __pyx_k_rule[] = "rule";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_class[] = "__class__";
static const char __pyx_k_clone[] = "clone";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_c_term[] = "c_term";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_glycan[] = "glycan";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_mass_2[] = "mass";
static const char __pyx_k_modify[] = "modify";
static const char __pyx_k_n_term[] = "n_term";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_position[] = "position";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_seq_list[] = "seq_list";
static const char __pyx_k_sequence[] = "sequence";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_aggregate[] = "aggregate";
static const char __pyx_k_constants[] = "constants";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_IndexError[] = "IndexError";
static const char __pyx_k_invalidate[] = "_invalidate";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_Modification[] = "Modification";
static const char __pyx_k_get_sequence[] = "get_sequence";
static const char __pyx_k_init_termini[] = "_init_termini";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_glycosylation[] = "glycosylation";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_AttributeError[] = "AttributeError";
static const char __pyx_k_include_glycan[] = "include_glycan";
static const char __pyx_k_implicit_c_term[] = "implicit_c_term";
static const char __pyx_k_implicit_n_term[] = "implicit_n_term";
static const char __pyx_k_include_termini[] = "include_termini";
static const char __pyx_k_parser_function[] = "parser_function";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_AminoAcidResidue[] = "AminoAcidResidue";
static const char __pyx_k_SequenceLocation[] = "SequenceLocation";
static const char __pyx_k_add_modification[] = "add_modification";
static const char __pyx_k_glycan_aggregate[] = "glycan_aggregate";
static const char __pyx_k_init_from_string[] = "_init_from_string";
static const char __pyx_k_initialize_fields[] = "_initialize_fields";
static const char __pyx_k_modification_type[] = "modification_type";
static const char __pyx_k_total_composition[] = "total_composition";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_composition_offset[] = "composition_offset";
static const char __pyx_k_glycan_composition[] = "glycan_composition";
static const char __pyx_k_modification_table[] = "modification_table";
static const char __pyx_k_sequence_tokenizer[] = "sequence_tokenizer";
static const char __pyx_k_PeptideSequenceCore[] = "_PeptideSequenceCore";
static const char __pyx_k_peptide_composition[] = "peptide_composition";
static const char __pyx_k_structure_constants[] = "_structure_constants";
static const char __pyx_k_ModificationCategory[] = "ModificationCategory";
static const char __pyx_k_init_from_components[] = "_init_from_components";
static const char __pyx_k_base_sequence_equality[] = "base_sequence_equality";
static const char __pyx_k_full_structure_equality[] = "full_structure_equality";
static const char __pyx_k_init_from_parsed_string[] = "_init_from_parsed_string";
static const char __pyx_k_glycopeptidepy__c_parser[] = "glycopeptidepy._c.parser";
static const char __pyx_k_glycopeptidepy_structure[] = "glycopeptidepy.structure";
static const char __pyx_k_peptide_base_composition[] = "_peptide_base_composition";
static const char __pyx_k_modified_sequence_equality[] = "modified_sequence_equality";
static const char __pyx_k_pyx_unpickle__PeptideSequenceC[] = "__pyx_unpickle__PeptideSequenceCore";
static const char __pyx_k_Invalid_modification_position_s[] = "Invalid modification position. %s, %s, %s";
static const char __pyx_k_n_term_sequence_c_term_glycan_a[] = "{n_term}{sequence}{c_term}{glycan_aggregate}[{_mass}]";
static const char __pyx_k_Incompatible_checksums_s_vs_0x9a[] = "Incompatible checksums (%s vs 0x9a27ba6 = (_c_term, _glycosylation_manager, _mass, _n_term, _peptide_composition, _total_composition, sequence))";
static const char __pyx_k_glycopeptidepy__c_structure_sequ[] = "glycopeptidepy/_c/structure/sequence_methods.pyx";
static const char __pyx_k_glycopeptidepy_structure_modific[] = "glycopeptidepy.structure.modification";
static const char __pyx_k_glycopeptidepy_structure_residue[] = "glycopeptidepy.structure.residue";
static const char __pyx_k_glycopeptidepy__c_structure_sequ_2[] = "glycopeptidepy._c.structure.sequence_methods";
static PyObject *__pyx_kp_s_;
static PyObject *__pyx_kp_s_0;
static PyObject *__pyx_kp_s_0_2;
static PyObject *__pyx_n_s_AminoAcidResidue;
static PyObject *__pyx_n_s_AttributeError;
static PyObject *__pyx_n_s_H2O;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x9a;
static PyObject *__pyx_n_s_IndexError;
static PyObject *__pyx_kp_s_Invalid_modification_position_s;
static PyObject *__pyx_n_s_Modification;
static PyObject *__pyx_n_s_ModificationCategory;
static PyObject *__pyx_n_s_PeptideSequenceCore;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_SequenceLocation;
static PyObject *__pyx_kp_s__2;
static PyObject *__pyx_n_s_add_modification;
static PyObject *__pyx_n_s_aggregate;
static PyObject *__pyx_n_s_base_sequence_equality;
static PyObject *__pyx_n_s_c_term;
static PyObject *__pyx_n_s_class;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_clone;
static PyObject *__pyx_n_s_composition_offset;
static PyObject *__pyx_n_s_constants;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_full_structure_equality;
static PyObject *__pyx_n_s_get_sequence;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_glycan;
static PyObject *__pyx_n_s_glycan_aggregate;
static PyObject *__pyx_n_s_glycan_composition;
static PyObject *__pyx_n_s_glycopeptidepy__c_parser;
static PyObject *__pyx_kp_s_glycopeptidepy__c_structure_sequ;
static PyObject *__pyx_n_s_glycopeptidepy__c_structure_sequ_2;
static PyObject *__pyx_n_s_glycopeptidepy_structure;
static PyObject *__pyx_n_s_glycopeptidepy_structure_modific;
static PyObject *__pyx_n_s_glycopeptidepy_structure_residue;
static PyObject *__pyx_n_s_glycosylation;
static PyObject *__pyx_n_s_implicit_c_term;
static PyObject *__pyx_n_s_implicit_n_term;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_include_glycan;
static PyObject *__pyx_n_s_include_termini;
static PyObject *__pyx_n_s_init_from_components;
static PyObject *__pyx_n_s_init_from_parsed_string;
static PyObject *__pyx_n_s_init_from_string;
static PyObject *__pyx_n_s_init_termini;
static PyObject *__pyx_n_s_initialize_fields;
static PyObject *__pyx_n_s_invalidate;
static PyObject *__pyx_n_s_join;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_mass;
static PyObject *__pyx_n_s_mass_2;
static PyObject *__pyx_n_s_modification_table;
static PyObject *__pyx_n_s_modification_type;
static PyObject *__pyx_n_s_modified_sequence_equality;
static PyObject *__pyx_n_s_modify;
static PyObject *__pyx_n_s_n_term;
static PyObject *__pyx_kp_s_n_term_sequence_c_term_glycan_a;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_parser_function;
static PyObject *__pyx_n_s_peptide_base_composition;
static PyObject *__pyx_n_s_peptide_composition;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_position;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle__PeptideSequenceC;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_rule;
static PyObject *__pyx_kp_s_s;
static PyObject *__pyx_kp_s_s_2;
static PyObject *__pyx_kp_s_s_3;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_n_s_seq_list;
static PyObject *__pyx_n_s_sequence;
static PyObject *__pyx_n_s_sequence_tokenizer;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_structure_constants;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_total_composition;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_get_modification_by_name(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore___init__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_sequence, PyObject *__pyx_v_parser_function, CYTHON_UNUSED PyObject *__pyx_v_kwargs); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_2_init_from_parsed_string(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_seq_list, PyObject *__pyx_v_glycan, PyObject *__pyx_v_n_term, PyObject *__pyx_v_c_term); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_4_init_from_string(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_sequence, PyObject *__pyx_v_parser_function); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6_init_from_components(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_seq_list, PyObject *__pyx_v_glycan_composition, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_n_term, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_c_term); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_8_init_termini(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_n_term, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_c_term); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_10_initialize_fields(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_12_invalidate(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_14__repr__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6n_term___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6n_term_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v__value); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6c_term___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6c_term_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v__value); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_4mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_4mass_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_21peptide_backbone_mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_10total_mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_16peptide_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_18total_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_20__iter__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_22__getitem__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_index); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_24__setitem__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /* proto */
static Py_ssize_t __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_26__len__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_28__eq__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_30base_sequence_equality(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_32modified_sequence_equality(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_34full_structure_equality(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_36__ne__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static Py_hash_t __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_38__hash__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_40get_sequence(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, int __pyx_v_include_glycan, int __pyx_v_include_termini, PyObject *__pyx_v_implicit_n_term, PyObject *__pyx_v_implicit_c_term); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_42__str__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_44clone(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_46__reduce_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_48__setstate_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_2add_modification(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_position, PyObject *__pyx_v_modification_type); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_4__pyx_unpickle__PeptideSequenceCore(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_161643430;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__4;
static PyObject *__pyx_tuple__6;
static PyObject *__pyx_codeobj__5;
static PyObject *__pyx_codeobj__7;
/* Late includes */

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":44
 * 
 * 
 * cpdef ModificationBase get_modification_by_name(basestring name):             # <<<<<<<<<<<<<<
 *     cdef:
 *         ModificationInstanceBase inst
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_1get_modification_by_name(PyObject *__pyx_self, PyObject *__pyx_v_name); /*proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_get_modification_by_name(PyObject *__pyx_v_name, CYTHON_UNUSED int __pyx_skip_dispatch) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_12modification_ModificationInstanceBase *__pyx_v_inst = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase *__pyx_v_rule = NULL;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_modification_by_name", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":48
 *         ModificationInstanceBase inst
 * 
 *     rule = modification_table.resolve(name)             # <<<<<<<<<<<<<<
 *     inst = <ModificationInstanceBase>ModificationImpl.__new__(ModificationImpl)
 *     inst._init_from_rule(rule)
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase *)__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_modification_table->__pyx_vtab)->resolve(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_modification_table, __pyx_v_name, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_rule = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":49
 * 
 *     rule = modification_table.resolve(name)
 *     inst = <ModificationInstanceBase>ModificationImpl.__new__(ModificationImpl)             # <<<<<<<<<<<<<<
 *     inst._init_from_rule(rule)
 *     return inst
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationImpl, __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationImpl) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationImpl);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 49, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __pyx_t_1;
  __Pyx_INCREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_inst = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_12modification_ModificationInstanceBase *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":50
 *     rule = modification_table.resolve(name)
 *     inst = <ModificationInstanceBase>ModificationImpl.__new__(ModificationImpl)
 *     inst._init_from_rule(rule)             # <<<<<<<<<<<<<<
 *     return inst
 * 
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_12modification_12modification_ModificationInstanceBase *)__pyx_v_inst->__pyx_base.__pyx_vtab)->_init_from_rule(__pyx_v_inst, __pyx_v_rule, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 50, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":51
 *     inst = <ModificationInstanceBase>ModificationImpl.__new__(ModificationImpl)
 *     inst._init_from_rule(rule)
 *     return inst             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_inst));
  __pyx_r = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_inst);
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":44
 * 
 * 
 * cpdef ModificationBase get_modification_by_name(basestring name):             # <<<<<<<<<<<<<<
 *     cdef:
 *         ModificationInstanceBase inst
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods.get_modification_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);
  __Pyx_XDECREF((PyObject *)__pyx_v_rule);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_1get_modification_by_name(PyObject *__pyx_self, PyObject *__pyx_v_name); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_get_modification_by_name[] = "get_modification_by_name(basestring name) -> ModificationBase";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_1get_modification_by_name(PyObject *__pyx_self, PyObject *__pyx_v_name) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_modification_by_name (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_name), (&PyBaseString_Type), 1, "name", 1))) __PYX_ERR(0, 44, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_get_modification_by_name(__pyx_self, ((PyObject*)__pyx_v_name));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_get_modification_by_name(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v_name) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_modification_by_name", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_get_modification_by_name(__pyx_v_name, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 44, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods.get_modification_by_name", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":58
 * # Helpers and Caches -----------------
 * 
 * cdef TerminalGroup _make_terminal_group(basestring base_composition_formula, ModificationBase modification=None):             # <<<<<<<<<<<<<<
 *     """A cache for creating :class:`~.TerminalGroup` instances from base formula and
 *     modification.
 */

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__make_terminal_group(PyObject *__pyx_v_base_composition_formula, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods__make_terminal_group *__pyx_optional_args) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_modification = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)Py_None);
  PyObject *__pyx_v_key = NULL;
  PyObject *__pyx_v_result;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_inst = NULL;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_make_terminal_group", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_modification = __pyx_optional_args->modification;
    }
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":66
 *     TerminalGroup
 *     """
 *     key = (base_composition_formula, modification)             # <<<<<<<<<<<<<<
 *     result = PyDict_GetItem(terminal_group_cache, key)
 *     if result == NULL:
 */
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 66, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_v_base_composition_formula);
  __Pyx_GIVEREF(__pyx_v_base_composition_formula);
  PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_base_composition_formula);
  __Pyx_INCREF(((PyObject *)__pyx_v_modification));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_modification));
  PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_modification));
  __pyx_v_key = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":67
 *     """
 *     key = (base_composition_formula, modification)
 *     result = PyDict_GetItem(terminal_group_cache, key)             # <<<<<<<<<<<<<<
 *     if result == NULL:
 *         inst = TerminalGroup(base_composition_formula, modification)
 */
  __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_terminal_group_cache;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_result = PyDict_GetItem(__pyx_t_1, __pyx_v_key);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":68
 *     key = (base_composition_formula, modification)
 *     result = PyDict_GetItem(terminal_group_cache, key)
 *     if result == NULL:             # <<<<<<<<<<<<<<
 *         inst = TerminalGroup(base_composition_formula, modification)
 *         PyDict_SetItem(terminal_group_cache, key, inst)
 */
  __pyx_t_2 = ((__pyx_v_result == NULL) != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":69
 *     result = PyDict_GetItem(terminal_group_cache, key)
 *     if result == NULL:
 *         inst = TerminalGroup(base_composition_formula, modification)             # <<<<<<<<<<<<<<
 *         PyDict_SetItem(terminal_group_cache, key, inst)
 *         return inst
 */
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 69, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_INCREF(__pyx_v_base_composition_formula);
    __Pyx_GIVEREF(__pyx_v_base_composition_formula);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_v_base_composition_formula);
    __Pyx_INCREF(((PyObject *)__pyx_v_modification));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_modification));
    PyTuple_SET_ITEM(__pyx_t_1, 1, ((PyObject *)__pyx_v_modification));
    __pyx_t_3 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup), __pyx_t_1, NULL); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 69, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_inst = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":70
 *     if result == NULL:
 *         inst = TerminalGroup(base_composition_formula, modification)
 *         PyDict_SetItem(terminal_group_cache, key, inst)             # <<<<<<<<<<<<<<
 *         return inst
 *     return <TerminalGroup>result
 */
    __pyx_t_3 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_terminal_group_cache;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_t_4 = PyDict_SetItem(__pyx_t_3, __pyx_v_key, ((PyObject *)__pyx_v_inst)); if (unlikely(__pyx_t_4 == ((int)-1))) __PYX_ERR(0, 70, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":71
 *         inst = TerminalGroup(base_composition_formula, modification)
 *         PyDict_SetItem(terminal_group_cache, key, inst)
 *         return inst             # <<<<<<<<<<<<<<
 *     return <TerminalGroup>result
 * 
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __Pyx_INCREF(((PyObject *)__pyx_v_inst));
    __pyx_r = __pyx_v_inst;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":68
 *     key = (base_composition_formula, modification)
 *     result = PyDict_GetItem(terminal_group_cache, key)
 *     if result == NULL:             # <<<<<<<<<<<<<<
 *         inst = TerminalGroup(base_composition_formula, modification)
 *         PyDict_SetItem(terminal_group_cache, key, inst)
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":72
 *         PyDict_SetItem(terminal_group_cache, key, inst)
 *         return inst
 *     return <TerminalGroup>result             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_result)));
  __pyx_r = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_result);
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":58
 * # Helpers and Caches -----------------
 * 
 * cdef TerminalGroup _make_terminal_group(basestring base_composition_formula, ModificationBase modification=None):             # <<<<<<<<<<<<<<
 *     """A cache for creating :class:`~.TerminalGroup` instances from base formula and
 *     modification.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._make_terminal_group", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_key);
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":78
 * 
 * 
 * cdef AminoAcidResidueBase _parse_residue(basestring residue_string):             # <<<<<<<<<<<<<<
 *     """A substitute cache for :class:`~.AminoAcidResidue` instantiation.
 * 
 */

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__parse_residue(PyObject *__pyx_v_residue_string) {
  PyObject *__pyx_v_result;
  PyObject *__pyx_v_inst = NULL;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_parse_residue", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":95
 *     AminoAcidResidueBase
 *     """
 *     result = PyDict_GetItem(amino_acid_cache, residue_string)             # <<<<<<<<<<<<<<
 *     if result == NULL:
 *         inst = AminoAcidResidueImpl(residue_string)
 */
  __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_amino_acid_cache;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_result = PyDict_GetItem(__pyx_t_1, __pyx_v_residue_string);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":96
 *     """
 *     result = PyDict_GetItem(amino_acid_cache, residue_string)
 *     if result == NULL:             # <<<<<<<<<<<<<<
 *         inst = AminoAcidResidueImpl(residue_string)
 *         PyDict_SetItem(amino_acid_cache, residue_string, inst)
 */
  __pyx_t_2 = ((__pyx_v_result == NULL) != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":97
 *     result = PyDict_GetItem(amino_acid_cache, residue_string)
 *     if result == NULL:
 *         inst = AminoAcidResidueImpl(residue_string)             # <<<<<<<<<<<<<<
 *         PyDict_SetItem(amino_acid_cache, residue_string, inst)
 *         return inst
 */
    __Pyx_INCREF(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_AminoAcidResidueImpl);
    __pyx_t_3 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_AminoAcidResidueImpl; __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_residue_string) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_residue_string);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 97, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_inst = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":98
 *     if result == NULL:
 *         inst = AminoAcidResidueImpl(residue_string)
 *         PyDict_SetItem(amino_acid_cache, residue_string, inst)             # <<<<<<<<<<<<<<
 *         return inst
 *     return <AminoAcidResidueBase>result
 */
    __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_amino_acid_cache;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_5 = PyDict_SetItem(__pyx_t_1, __pyx_v_residue_string, __pyx_v_inst); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 98, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":99
 *         inst = AminoAcidResidueImpl(residue_string)
 *         PyDict_SetItem(amino_acid_cache, residue_string, inst)
 *         return inst             # <<<<<<<<<<<<<<
 *     return <AminoAcidResidueBase>result
 * 
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    if (!(likely(((__pyx_v_inst) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_inst, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase))))) __PYX_ERR(0, 99, __pyx_L1_error)
    __Pyx_INCREF(__pyx_v_inst);
    __pyx_r = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_inst);
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":96
 *     """
 *     result = PyDict_GetItem(amino_acid_cache, residue_string)
 *     if result == NULL:             # <<<<<<<<<<<<<<
 *         inst = AminoAcidResidueImpl(residue_string)
 *         PyDict_SetItem(amino_acid_cache, residue_string, inst)
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":100
 *         PyDict_SetItem(amino_acid_cache, residue_string, inst)
 *         return inst
 *     return <AminoAcidResidueBase>result             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_result)));
  __pyx_r = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_result);
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":78
 * 
 * 
 * cdef AminoAcidResidueBase _parse_residue(basestring residue_string):             # <<<<<<<<<<<<<<
 *     """A substitute cache for :class:`~.AminoAcidResidue` instantiation.
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._parse_residue", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_inst);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":106
 * 
 * 
 * cdef ModificationBase _get_implicit_terminal(basestring terminal_name):             # <<<<<<<<<<<<<<
 *     """A cache for storing the Modification mapping for implicit N- and C-terminal
 *     groups
 */

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__get_implicit_terminal(PyObject *__pyx_v_terminal_name) {
  PyObject *__pyx_v_result;
  PyObject *__pyx_v_inst = NULL;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_get_implicit_terminal", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":114
 *     ModificationBase
 *     """
 *     result = PyDict_GetItem(implicit_terminal_cache, terminal_name)             # <<<<<<<<<<<<<<
 *     if result == NULL:
 *         inst = ModificationImpl(terminal_name)
 */
  __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_implicit_terminal_cache;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_result = PyDict_GetItem(__pyx_t_1, __pyx_v_terminal_name);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":115
 *     """
 *     result = PyDict_GetItem(implicit_terminal_cache, terminal_name)
 *     if result == NULL:             # <<<<<<<<<<<<<<
 *         inst = ModificationImpl(terminal_name)
 *         PyDict_SetItem(implicit_terminal_cache, terminal_name, inst)
 */
  __pyx_t_2 = ((__pyx_v_result == NULL) != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":116
 *     result = PyDict_GetItem(implicit_terminal_cache, terminal_name)
 *     if result == NULL:
 *         inst = ModificationImpl(terminal_name)             # <<<<<<<<<<<<<<
 *         PyDict_SetItem(implicit_terminal_cache, terminal_name, inst)
 *         return inst
 */
    __Pyx_INCREF(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationImpl);
    __pyx_t_3 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationImpl; __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_terminal_name) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_terminal_name);
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 116, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_inst = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":117
 *     if result == NULL:
 *         inst = ModificationImpl(terminal_name)
 *         PyDict_SetItem(implicit_terminal_cache, terminal_name, inst)             # <<<<<<<<<<<<<<
 *         return inst
 *     return <ModificationBase>result
 */
    __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_implicit_terminal_cache;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_5 = PyDict_SetItem(__pyx_t_1, __pyx_v_terminal_name, __pyx_v_inst); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 117, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":118
 *         inst = ModificationImpl(terminal_name)
 *         PyDict_SetItem(implicit_terminal_cache, terminal_name, inst)
 *         return inst             # <<<<<<<<<<<<<<
 *     return <ModificationBase>result
 * 
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    if (!(likely(((__pyx_v_inst) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_inst, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase))))) __PYX_ERR(0, 118, __pyx_L1_error)
    __Pyx_INCREF(__pyx_v_inst);
    __pyx_r = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_inst);
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":115
 *     """
 *     result = PyDict_GetItem(implicit_terminal_cache, terminal_name)
 *     if result == NULL:             # <<<<<<<<<<<<<<
 *         inst = ModificationImpl(terminal_name)
 *         PyDict_SetItem(implicit_terminal_cache, terminal_name, inst)
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":119
 *         PyDict_SetItem(implicit_terminal_cache, terminal_name, inst)
 *         return inst
 *     return <ModificationBase>result             # <<<<<<<<<<<<<<
 * 
 * # -------------------
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_result)));
  __pyx_r = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_result);
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":106
 * 
 * 
 * cdef ModificationBase _get_implicit_terminal(basestring terminal_name):             # <<<<<<<<<<<<<<
 *     """A cache for storing the Modification mapping for implicit N- and C-terminal
 *     groups
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._get_implicit_terminal", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_inst);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":124
 * 
 * 
 * cdef CComposition _total_composition(_PeptideSequenceCore sequence):             # <<<<<<<<<<<<<<
 *     cdef:
 *         CComposition total
 */

static struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__total_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_sequence) {
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_total = 0;
  size_t __pyx_v_i;
  size_t __pyx_v_j;
  size_t __pyx_v_m;
  size_t __pyx_v_n;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_position = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_mod = 0;
  PyObject *__pyx_v_gc = NULL;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  size_t __pyx_t_4;
  size_t __pyx_t_5;
  size_t __pyx_t_6;
  size_t __pyx_t_7;
  PyObject *__pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_total_composition", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":131
 *         ModificationBase mod
 * 
 *     n = sequence.get_size()             # <<<<<<<<<<<<<<
 *     total = CComposition()
 *     for i in range(n):
 */
  __pyx_v_n = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_sequence->__pyx_base.__pyx_vtab)->get_size(__pyx_v_sequence);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":132
 * 
 *     n = sequence.get_size()
 *     total = CComposition()             # <<<<<<<<<<<<<<
 *     for i in range(n):
 *         position = sequence.get(i)
 */
  __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_5glypy_11composition_12ccomposition_CComposition)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 132, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_total = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":133
 *     n = sequence.get_size()
 *     total = CComposition()
 *     for i in range(n):             # <<<<<<<<<<<<<<
 *         position = sequence.get(i)
 *         total += position.amino_acid.composition
 */
  __pyx_t_2 = __pyx_v_n;
  __pyx_t_3 = __pyx_t_2;
  for (__pyx_t_4 = 0; __pyx_t_4 < __pyx_t_3; __pyx_t_4+=1) {
    __pyx_v_i = __pyx_t_4;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":134
 *     total = CComposition()
 *     for i in range(n):
 *         position = sequence.get(i)             # <<<<<<<<<<<<<<
 *         total += position.amino_acid.composition
 *         m = PyList_GET_SIZE(position.modifications)
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_sequence->__pyx_base.__pyx_vtab)->__pyx_base.get(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_sequence), __pyx_v_i)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 134, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_position, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":135
 *     for i in range(n):
 *         position = sequence.get(i)
 *         total += position.amino_acid.composition             # <<<<<<<<<<<<<<
 *         m = PyList_GET_SIZE(position.modifications)
 *         for j in range(m):
 */
    __pyx_t_1 = PyNumber_InPlaceAdd(((PyObject *)__pyx_v_total), ((PyObject *)__pyx_v_position->amino_acid->composition)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 135, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 135, __pyx_L1_error)
    __Pyx_DECREF_SET(__pyx_v_total, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":136
 *         position = sequence.get(i)
 *         total += position.amino_acid.composition
 *         m = PyList_GET_SIZE(position.modifications)             # <<<<<<<<<<<<<<
 *         for j in range(m):
 *             mod = <ModificationBase>PyList_GetItem(position.modifications, j)
 */
    __pyx_t_1 = __pyx_v_position->modifications;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v_m = PyList_GET_SIZE(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":137
 *         total += position.amino_acid.composition
 *         m = PyList_GET_SIZE(position.modifications)
 *         for j in range(m):             # <<<<<<<<<<<<<<
 *             mod = <ModificationBase>PyList_GetItem(position.modifications, j)
 *             if mod.is_tracked_for(ModificationCategory_glycosylation):
 */
    __pyx_t_5 = __pyx_v_m;
    __pyx_t_6 = __pyx_t_5;
    for (__pyx_t_7 = 0; __pyx_t_7 < __pyx_t_6; __pyx_t_7+=1) {
      __pyx_v_j = __pyx_t_7;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":138
 *         m = PyList_GET_SIZE(position.modifications)
 *         for j in range(m):
 *             mod = <ModificationBase>PyList_GetItem(position.modifications, j)             # <<<<<<<<<<<<<<
 *             if mod.is_tracked_for(ModificationCategory_glycosylation):
 *                 continue
 */
      __pyx_t_1 = __pyx_v_position->modifications;
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_8 = PyList_GetItem(__pyx_t_1, __pyx_v_j); if (unlikely(__pyx_t_8 == ((PyObject *)NULL))) __PYX_ERR(0, 138, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = ((PyObject *)__pyx_t_8);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_mod, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":139
 *         for j in range(m):
 *             mod = <ModificationBase>PyList_GetItem(position.modifications, j)
 *             if mod.is_tracked_for(ModificationCategory_glycosylation):             # <<<<<<<<<<<<<<
 *                 continue
 *             total += mod.composition
 */
      __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationCategory_glycosylation;
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_9 = (((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_mod->__pyx_vtab)->is_tracked_for(__pyx_v_mod, __pyx_t_1, 0) != 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (__pyx_t_9) {

        /* "glycopeptidepy/_c/structure/sequence_methods.pyx":140
 *             mod = <ModificationBase>PyList_GetItem(position.modifications, j)
 *             if mod.is_tracked_for(ModificationCategory_glycosylation):
 *                 continue             # <<<<<<<<<<<<<<
 *             total += mod.composition
 * 
 */
        goto __pyx_L5_continue;

        /* "glycopeptidepy/_c/structure/sequence_methods.pyx":139
 *         for j in range(m):
 *             mod = <ModificationBase>PyList_GetItem(position.modifications, j)
 *             if mod.is_tracked_for(ModificationCategory_glycosylation):             # <<<<<<<<<<<<<<
 *                 continue
 *             total += mod.composition
 */
      }

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":141
 *             if mod.is_tracked_for(ModificationCategory_glycosylation):
 *                 continue
 *             total += mod.composition             # <<<<<<<<<<<<<<
 * 
 *     total += sequence._n_term.get_composition()
 */
      __pyx_t_1 = PyNumber_InPlaceAdd(((PyObject *)__pyx_v_total), ((PyObject *)__pyx_v_mod->composition)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 141, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 141, __pyx_L1_error)
      __Pyx_DECREF_SET(__pyx_v_total, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1));
      __pyx_t_1 = 0;
      __pyx_L5_continue:;
    }
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":143
 *             total += mod.composition
 * 
 *     total += sequence._n_term.get_composition()             # <<<<<<<<<<<<<<
 *     total += sequence._c_term.get_composition()
 *     gc = sequence.glycan_composition
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_sequence->__pyx_base._n_term->__pyx_vtab)->get_composition(__pyx_v_sequence->__pyx_base._n_term)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 143, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_10 = PyNumber_InPlaceAdd(((PyObject *)__pyx_v_total), __pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 143, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(((__pyx_t_10) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_10, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 143, __pyx_L1_error)
  __Pyx_DECREF_SET(__pyx_v_total, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_10));
  __pyx_t_10 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":144
 * 
 *     total += sequence._n_term.get_composition()
 *     total += sequence._c_term.get_composition()             # <<<<<<<<<<<<<<
 *     gc = sequence.glycan_composition
 *     if gc:
 */
  __pyx_t_10 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_sequence->__pyx_base._c_term->__pyx_vtab)->get_composition(__pyx_v_sequence->__pyx_base._c_term)); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_10);
  __pyx_t_1 = PyNumber_InPlaceAdd(((PyObject *)__pyx_v_total), __pyx_t_10); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 144, __pyx_L1_error)
  __Pyx_DECREF_SET(__pyx_v_total, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1));
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":145
 *     total += sequence._n_term.get_composition()
 *     total += sequence._c_term.get_composition()
 *     gc = sequence.glycan_composition             # <<<<<<<<<<<<<<
 *     if gc:
 *         total += gc.total_composition()
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_sequence), __pyx_n_s_glycan_composition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 145, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_gc = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":146
 *     total += sequence._c_term.get_composition()
 *     gc = sequence.glycan_composition
 *     if gc:             # <<<<<<<<<<<<<<
 *         total += gc.total_composition()
 *     return total
 */
  __pyx_t_9 = __Pyx_PyObject_IsTrue(__pyx_v_gc); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 146, __pyx_L1_error)
  if (__pyx_t_9) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":147
 *     gc = sequence.glycan_composition
 *     if gc:
 *         total += gc.total_composition()             # <<<<<<<<<<<<<<
 *     return total
 * 
 */
    __pyx_t_10 = __Pyx_PyObject_GetAttrStr(__pyx_v_gc, __pyx_n_s_total_composition); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __pyx_t_11 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_10))) {
      __pyx_t_11 = PyMethod_GET_SELF(__pyx_t_10);
      if (likely(__pyx_t_11)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_10);
        __Pyx_INCREF(__pyx_t_11);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_10, function);
      }
    }
    __pyx_t_1 = (__pyx_t_11) ? __Pyx_PyObject_CallOneArg(__pyx_t_10, __pyx_t_11) : __Pyx_PyObject_CallNoArg(__pyx_t_10);
    __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_10); __pyx_t_10 = 0;
    __pyx_t_10 = PyNumber_InPlaceAdd(((PyObject *)__pyx_v_total), __pyx_t_1); if (unlikely(!__pyx_t_10)) __PYX_ERR(0, 147, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_10);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (!(likely(((__pyx_t_10) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_10, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 147, __pyx_L1_error)
    __Pyx_DECREF_SET(__pyx_v_total, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_10));
    __pyx_t_10 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":146
 *     total += sequence._c_term.get_composition()
 *     gc = sequence.glycan_composition
 *     if gc:             # <<<<<<<<<<<<<<
 *         total += gc.total_composition()
 *     return total
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":148
 *     if gc:
 *         total += gc.total_composition()
 *     return total             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_total));
  __pyx_r = __pyx_v_total;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":124
 * 
 * 
 * cdef CComposition _total_composition(_PeptideSequenceCore sequence):             # <<<<<<<<<<<<<<
 *     cdef:
 *         CComposition total
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._total_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_total);
  __Pyx_XDECREF((PyObject *)__pyx_v_position);
  __Pyx_XDECREF((PyObject *)__pyx_v_mod);
  __Pyx_XDECREF(__pyx_v_gc);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":152
 * 
 * cdef class _PeptideSequenceCore(PeptideSequenceBase):
 *     def __init__(self, sequence=None, parser_function=None, **kwargs):             # <<<<<<<<<<<<<<
 *         if parser_function is None:
 *             parser_function = sequence_tokenizer
 */

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_sequence = 0;
  PyObject *__pyx_v_parser_function = 0;
  CYTHON_UNUSED PyObject *__pyx_v_kwargs = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  __pyx_v_kwargs = PyDict_New(); if (unlikely(!__pyx_v_kwargs)) return -1;
  __Pyx_GOTREF(__pyx_v_kwargs);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sequence,&__pyx_n_s_parser_function,0};
    PyObject* values[2] = {0,0};
    values[0] = ((PyObject *)Py_None);
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_sequence);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_parser_function);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, __pyx_v_kwargs, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 152, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_sequence = values[0];
    __pyx_v_parser_function = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 0, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 152, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_DECREF(__pyx_v_kwargs); __pyx_v_kwargs = 0;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore___init__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self), __pyx_v_sequence, __pyx_v_parser_function, __pyx_v_kwargs);

  /* function exit code */
  __Pyx_XDECREF(__pyx_v_kwargs);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore___init__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_sequence, PyObject *__pyx_v_parser_function, CYTHON_UNUSED PyObject *__pyx_v_kwargs) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);
  __Pyx_INCREF(__pyx_v_parser_function);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":153
 * cdef class _PeptideSequenceCore(PeptideSequenceBase):
 *     def __init__(self, sequence=None, parser_function=None, **kwargs):
 *         if parser_function is None:             # <<<<<<<<<<<<<<
 *             parser_function = sequence_tokenizer
 *         self._initialize_fields()
 */
  __pyx_t_1 = (__pyx_v_parser_function == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":154
 *     def __init__(self, sequence=None, parser_function=None, **kwargs):
 *         if parser_function is None:
 *             parser_function = sequence_tokenizer             # <<<<<<<<<<<<<<
 *         self._initialize_fields()
 *         if sequence is not None:
 */
    __Pyx_GetModuleGlobalName(__pyx_t_3, __pyx_n_s_sequence_tokenizer); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 154, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_parser_function, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":153
 * cdef class _PeptideSequenceCore(PeptideSequenceBase):
 *     def __init__(self, sequence=None, parser_function=None, **kwargs):
 *         if parser_function is None:             # <<<<<<<<<<<<<<
 *             parser_function = sequence_tokenizer
 *         self._initialize_fields()
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":155
 *         if parser_function is None:
 *             parser_function = sequence_tokenizer
 *         self._initialize_fields()             # <<<<<<<<<<<<<<
 *         if sequence is not None:
 *             self._init_from_string(sequence, parser_function)
 */
  __pyx_t_3 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->_initialize_fields(__pyx_v_self, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 155, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":156
 *             parser_function = sequence_tokenizer
 *         self._initialize_fields()
 *         if sequence is not None:             # <<<<<<<<<<<<<<
 *             self._init_from_string(sequence, parser_function)
 * 
 */
  __pyx_t_2 = (__pyx_v_sequence != Py_None);
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":157
 *         self._initialize_fields()
 *         if sequence is not None:
 *             self._init_from_string(sequence, parser_function)             # <<<<<<<<<<<<<<
 * 
 *     cpdef _init_from_parsed_string(self, list seq_list, glycan=None, n_term=None, c_term=None):
 */
    __pyx_t_3 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->_init_from_string(__pyx_v_self, __pyx_v_sequence, __pyx_v_parser_function, 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 157, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":156
 *             parser_function = sequence_tokenizer
 *         self._initialize_fields()
 *         if sequence is not None:             # <<<<<<<<<<<<<<
 *             self._init_from_string(sequence, parser_function)
 * 
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":152
 * 
 * cdef class _PeptideSequenceCore(PeptideSequenceBase):
 *     def __init__(self, sequence=None, parser_function=None, **kwargs):             # <<<<<<<<<<<<<<
 *         if parser_function is None:
 *             parser_function = sequence_tokenizer
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_parser_function);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":159
 *             self._init_from_string(sequence, parser_function)
 * 
 *     cpdef _init_from_parsed_string(self, list seq_list, glycan=None, n_term=None, c_term=None):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, j, n, m
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_3_init_from_parsed_string(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_parsed_string(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_seq_list, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_parsed_string *__pyx_optional_args) {
  PyObject *__pyx_v_glycan = ((PyObject *)Py_None);
  PyObject *__pyx_v_n_term = ((PyObject *)Py_None);
  PyObject *__pyx_v_c_term = ((PyObject *)Py_None);
  size_t __pyx_v_i;
  size_t __pyx_v_j;
  size_t __pyx_v_n;
  size_t __pyx_v_m;
  PyObject *__pyx_v_sequence = 0;
  PyObject *__pyx_v_item = 0;
  PyObject *__pyx_v_mods = 0;
  PyObject *__pyx_v_mod_strs = 0;
  double __pyx_v_mass;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_mod = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_res = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_position = 0;
  int __pyx_v_has_glycan;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *__pyx_v_glycosylation_manager = 0;
  PyObject *__pyx_v_mod_str = NULL;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_n_term_group = NULL;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_c_term_group = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  size_t __pyx_t_9;
  PyObject *__pyx_t_10;
  size_t __pyx_t_11;
  size_t __pyx_t_12;
  size_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  int __pyx_t_16;
  struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods__make_terminal_group __pyx_t_17;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_init_from_parsed_string", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_glycan = __pyx_optional_args->glycan;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_n_term = __pyx_optional_args->n_term;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_c_term = __pyx_optional_args->c_term;
        }
      }
    }
  }
  __Pyx_INCREF(__pyx_v_n_term);
  __Pyx_INCREF(__pyx_v_c_term);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_init_from_parsed_string); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 159, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_3_init_from_parsed_string)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[5] = {__pyx_t_4, __pyx_v_seq_list, __pyx_v_glycan, __pyx_v_n_term, __pyx_v_c_term};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 159, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[5] = {__pyx_t_4, __pyx_v_seq_list, __pyx_v_glycan, __pyx_v_n_term, __pyx_v_c_term};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 159, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(4+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 159, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(__pyx_v_seq_list);
          __Pyx_GIVEREF(__pyx_v_seq_list);
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_seq_list);
          __Pyx_INCREF(__pyx_v_glycan);
          __Pyx_GIVEREF(__pyx_v_glycan);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_glycan);
          __Pyx_INCREF(__pyx_v_n_term);
          __Pyx_GIVEREF(__pyx_v_n_term);
          PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, __pyx_v_n_term);
          __Pyx_INCREF(__pyx_v_c_term);
          __Pyx_GIVEREF(__pyx_v_c_term);
          PyTuple_SET_ITEM(__pyx_t_6, 3+__pyx_t_5, __pyx_v_c_term);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 159, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":170
 *             GlycosylationManager glycosylation_manager
 * 
 *         i = 0             # <<<<<<<<<<<<<<
 *         n = PyList_GET_SIZE(seq_list)
 *         self.sequence = sequence = PyList_New(n)
 */
  __pyx_v_i = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":171
 * 
 *         i = 0
 *         n = PyList_GET_SIZE(seq_list)             # <<<<<<<<<<<<<<
 *         self.sequence = sequence = PyList_New(n)
 *         glycosylation_manager = <GlycosylationManager>self._glycosylation_manager
 */
  __pyx_v_n = PyList_GET_SIZE(__pyx_v_seq_list);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":172
 *         i = 0
 *         n = PyList_GET_SIZE(seq_list)
 *         self.sequence = sequence = PyList_New(n)             # <<<<<<<<<<<<<<
 *         glycosylation_manager = <GlycosylationManager>self._glycosylation_manager
 *         mass = 0
 */
  __pyx_t_1 = PyList_New(__pyx_v_n); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 172, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.sequence);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.sequence);
  __pyx_v_self->__pyx_base.sequence = ((PyObject*)__pyx_t_1);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_sequence = ((PyObject*)__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":173
 *         n = PyList_GET_SIZE(seq_list)
 *         self.sequence = sequence = PyList_New(n)
 *         glycosylation_manager = <GlycosylationManager>self._glycosylation_manager             # <<<<<<<<<<<<<<
 *         mass = 0
 *         for i in range(n):
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base._glycosylation_manager;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_glycosylation_manager = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":174
 *         self.sequence = sequence = PyList_New(n)
 *         glycosylation_manager = <GlycosylationManager>self._glycosylation_manager
 *         mass = 0             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             item = <list>PyList_GET_ITEM(seq_list, i)
 */
  __pyx_v_mass = 0.0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":175
 *         glycosylation_manager = <GlycosylationManager>self._glycosylation_manager
 *         mass = 0
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             item = <list>PyList_GET_ITEM(seq_list, i)
 *             res = _parse_residue(<basestring>PyList_GET_ITEM(item, 0))
 */
  __pyx_t_7 = __pyx_v_n;
  __pyx_t_8 = __pyx_t_7;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":176
 *         mass = 0
 *         for i in range(n):
 *             item = <list>PyList_GET_ITEM(seq_list, i)             # <<<<<<<<<<<<<<
 *             res = _parse_residue(<basestring>PyList_GET_ITEM(item, 0))
 *             mass += res.mass
 */
    __pyx_t_10 = PyList_GET_ITEM(__pyx_v_seq_list, __pyx_v_i);
    __pyx_t_1 = ((PyObject *)__pyx_t_10);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_item, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":177
 *         for i in range(n):
 *             item = <list>PyList_GET_ITEM(seq_list, i)
 *             res = _parse_residue(<basestring>PyList_GET_ITEM(item, 0))             # <<<<<<<<<<<<<<
 *             mass += res.mass
 *             mods = []
 */
    __pyx_t_10 = PyList_GET_ITEM(__pyx_v_item, 0);
    __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__parse_residue(((PyObject*)__pyx_t_10))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 177, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_res, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":178
 *             item = <list>PyList_GET_ITEM(seq_list, i)
 *             res = _parse_residue(<basestring>PyList_GET_ITEM(item, 0))
 *             mass += res.mass             # <<<<<<<<<<<<<<
 *             mods = []
 *             mod_strs = <list>PyList_GET_ITEM(item, 1)
 */
    __pyx_v_mass = (__pyx_v_mass + __pyx_v_res->mass);

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":179
 *             res = _parse_residue(<basestring>PyList_GET_ITEM(item, 0))
 *             mass += res.mass
 *             mods = []             # <<<<<<<<<<<<<<
 *             mod_strs = <list>PyList_GET_ITEM(item, 1)
 *             m = PyList_GET_SIZE(mod_strs)
 */
    __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 179, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_mods, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":180
 *             mass += res.mass
 *             mods = []
 *             mod_strs = <list>PyList_GET_ITEM(item, 1)             # <<<<<<<<<<<<<<
 *             m = PyList_GET_SIZE(mod_strs)
 *             for j in range(m):
 */
    __pyx_t_10 = PyList_GET_ITEM(__pyx_v_item, 1);
    __pyx_t_1 = ((PyObject *)__pyx_t_10);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_mod_strs, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":181
 *             mods = []
 *             mod_strs = <list>PyList_GET_ITEM(item, 1)
 *             m = PyList_GET_SIZE(mod_strs)             # <<<<<<<<<<<<<<
 *             for j in range(m):
 *                 mod_str = <basestring>PyList_GET_ITEM(mod_strs, j)
 */
    __pyx_v_m = PyList_GET_SIZE(__pyx_v_mod_strs);

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":182
 *             mod_strs = <list>PyList_GET_ITEM(item, 1)
 *             m = PyList_GET_SIZE(mod_strs)
 *             for j in range(m):             # <<<<<<<<<<<<<<
 *                 mod_str = <basestring>PyList_GET_ITEM(mod_strs, j)
 *                 if mod_str != '':
 */
    __pyx_t_11 = __pyx_v_m;
    __pyx_t_12 = __pyx_t_11;
    for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
      __pyx_v_j = __pyx_t_13;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":183
 *             m = PyList_GET_SIZE(mod_strs)
 *             for j in range(m):
 *                 mod_str = <basestring>PyList_GET_ITEM(mod_strs, j)             # <<<<<<<<<<<<<<
 *                 if mod_str != '':
 *                     mod = get_modification_by_name(mod_str)
 */
      __pyx_t_10 = PyList_GET_ITEM(__pyx_v_mod_strs, __pyx_v_j);
      __pyx_t_1 = ((PyObject *)__pyx_t_10);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_mod_str, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":184
 *             for j in range(m):
 *                 mod_str = <basestring>PyList_GET_ITEM(mod_strs, j)
 *                 if mod_str != '':             # <<<<<<<<<<<<<<
 *                     mod = get_modification_by_name(mod_str)
 *                     if mod.is_tracked_for(ModificationCategory_glycosylation):
 */
      __pyx_t_14 = (__Pyx_PyUnicode_Equals(__pyx_v_mod_str, __pyx_kp_s_, Py_NE)); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 184, __pyx_L1_error)
      __pyx_t_15 = (__pyx_t_14 != 0);
      if (__pyx_t_15) {

        /* "glycopeptidepy/_c/structure/sequence_methods.pyx":185
 *                 mod_str = <basestring>PyList_GET_ITEM(mod_strs, j)
 *                 if mod_str != '':
 *                     mod = get_modification_by_name(mod_str)             # <<<<<<<<<<<<<<
 *                     if mod.is_tracked_for(ModificationCategory_glycosylation):
 *                         glycosylation_manager[i] = mod
 */
        __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_get_modification_by_name(__pyx_v_mod_str, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 185, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __Pyx_XDECREF_SET(__pyx_v_mod, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "glycopeptidepy/_c/structure/sequence_methods.pyx":186
 *                 if mod_str != '':
 *                     mod = get_modification_by_name(mod_str)
 *                     if mod.is_tracked_for(ModificationCategory_glycosylation):             # <<<<<<<<<<<<<<
 *                         glycosylation_manager[i] = mod
 *                     mods.append(mod)
 */
        __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationCategory_glycosylation;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_15 = (((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_mod->__pyx_vtab)->is_tracked_for(__pyx_v_mod, __pyx_t_1, 0) != 0);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        if (__pyx_t_15) {

          /* "glycopeptidepy/_c/structure/sequence_methods.pyx":187
 *                     mod = get_modification_by_name(mod_str)
 *                     if mod.is_tracked_for(ModificationCategory_glycosylation):
 *                         glycosylation_manager[i] = mod             # <<<<<<<<<<<<<<
 *                     mods.append(mod)
 *                     mass += mod.mass
 */
          if (unlikely(__Pyx_SetItemInt(((PyObject *)__pyx_v_glycosylation_manager), __pyx_v_i, ((PyObject *)__pyx_v_mod), size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1) < 0)) __PYX_ERR(0, 187, __pyx_L1_error)

          /* "glycopeptidepy/_c/structure/sequence_methods.pyx":186
 *                 if mod_str != '':
 *                     mod = get_modification_by_name(mod_str)
 *                     if mod.is_tracked_for(ModificationCategory_glycosylation):             # <<<<<<<<<<<<<<
 *                         glycosylation_manager[i] = mod
 *                     mods.append(mod)
 */
        }

        /* "glycopeptidepy/_c/structure/sequence_methods.pyx":188
 *                     if mod.is_tracked_for(ModificationCategory_glycosylation):
 *                         glycosylation_manager[i] = mod
 *                     mods.append(mod)             # <<<<<<<<<<<<<<
 *                     mass += mod.mass
 * 
 */
        __pyx_t_16 = __Pyx_PyList_Append(__pyx_v_mods, ((PyObject *)__pyx_v_mod)); if (unlikely(__pyx_t_16 == ((int)-1))) __PYX_ERR(0, 188, __pyx_L1_error)

        /* "glycopeptidepy/_c/structure/sequence_methods.pyx":189
 *                         glycosylation_manager[i] = mod
 *                     mods.append(mod)
 *                     mass += mod.mass             # <<<<<<<<<<<<<<
 * 
 *             position = SequencePosition._create(res, mods)
 */
        __pyx_v_mass = (__pyx_v_mass + __pyx_v_mod->mass);

        /* "glycopeptidepy/_c/structure/sequence_methods.pyx":184
 *             for j in range(m):
 *                 mod_str = <basestring>PyList_GET_ITEM(mod_strs, j)
 *                 if mod_str != '':             # <<<<<<<<<<<<<<
 *                     mod = get_modification_by_name(mod_str)
 *                     if mod.is_tracked_for(ModificationCategory_glycosylation):
 */
      }
    }

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":191
 *                     mass += mod.mass
 * 
 *             position = SequencePosition._create(res, mods)             # <<<<<<<<<<<<<<
 *             Py_INCREF(position)
 *             PyList_SetItem(sequence, i, position)
 */
    __pyx_t_1 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_SequencePosition->_create(__pyx_v_res, __pyx_v_mods)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 191, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_position, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":192
 * 
 *             position = SequencePosition._create(res, mods)
 *             Py_INCREF(position)             # <<<<<<<<<<<<<<
 *             PyList_SetItem(sequence, i, position)
 * 
 */
    Py_INCREF(((PyObject *)__pyx_v_position));

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":193
 *             position = SequencePosition._create(res, mods)
 *             Py_INCREF(position)
 *             PyList_SetItem(sequence, i, position)             # <<<<<<<<<<<<<<
 * 
 *         self._mass = mass
 */
    __pyx_t_5 = PyList_SetItem(__pyx_v_sequence, __pyx_v_i, ((PyObject *)__pyx_v_position)); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 193, __pyx_L1_error)
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":195
 *             PyList_SetItem(sequence, i, position)
 * 
 *         self._mass = mass             # <<<<<<<<<<<<<<
 *         has_glycan = glycan is not None
 *         if has_glycan:
 */
  __pyx_v_self->__pyx_base._mass = __pyx_v_mass;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":196
 * 
 *         self._mass = mass
 *         has_glycan = glycan is not None             # <<<<<<<<<<<<<<
 *         if has_glycan:
 *             glycosylation_manager.set_aggregate(glycan)
 */
  __pyx_t_15 = (__pyx_v_glycan != Py_None);
  __pyx_v_has_glycan = __pyx_t_15;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":197
 *         self._mass = mass
 *         has_glycan = glycan is not None
 *         if has_glycan:             # <<<<<<<<<<<<<<
 *             glycosylation_manager.set_aggregate(glycan)
 * 
 */
  __pyx_t_15 = (__pyx_v_has_glycan != 0);
  if (__pyx_t_15) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":198
 *         has_glycan = glycan is not None
 *         if has_glycan:
 *             glycosylation_manager.set_aggregate(glycan)             # <<<<<<<<<<<<<<
 * 
 *         if isinstance(n_term, basestring):
 */
    (void)(((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *)__pyx_v_glycosylation_manager->__pyx_vtab)->set_aggregate(__pyx_v_glycosylation_manager, __pyx_v_glycan));

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":197
 *         self._mass = mass
 *         has_glycan = glycan is not None
 *         if has_glycan:             # <<<<<<<<<<<<<<
 *             glycosylation_manager.set_aggregate(glycan)
 * 
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":200
 *             glycosylation_manager.set_aggregate(glycan)
 * 
 *         if isinstance(n_term, basestring):             # <<<<<<<<<<<<<<
 *             if n_term != structure_constants.N_TERM_DEFAULT:
 *                 n_term = get_modification_by_name(n_term)
 */
  __pyx_t_15 = __Pyx_PyBaseString_Check(__pyx_v_n_term); 
  __pyx_t_14 = (__pyx_t_15 != 0);
  if (__pyx_t_14) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":201
 * 
 *         if isinstance(n_term, basestring):
 *             if n_term != structure_constants.N_TERM_DEFAULT:             # <<<<<<<<<<<<<<
 *                 n_term = get_modification_by_name(n_term)
 *             else:
 */
    __pyx_t_14 = (__Pyx_PyString_Equals(__pyx_v_n_term, __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_structure_constants->N_TERM_DEFAULT, Py_NE)); if (unlikely(__pyx_t_14 < 0)) __PYX_ERR(0, 201, __pyx_L1_error)
    if (__pyx_t_14) {

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":202
 *         if isinstance(n_term, basestring):
 *             if n_term != structure_constants.N_TERM_DEFAULT:
 *                 n_term = get_modification_by_name(n_term)             # <<<<<<<<<<<<<<
 *             else:
 *                 n_term = None
 */
      if (!(likely(__Pyx_PyBaseString_CheckExact(__pyx_v_n_term))||((__pyx_v_n_term) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", (PY_MAJOR_VERSION < 3 ? "basestring" : "str"), Py_TYPE(__pyx_v_n_term)->tp_name), 0))) __PYX_ERR(0, 202, __pyx_L1_error)
      __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_get_modification_by_name(((PyObject*)__pyx_v_n_term), 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 202, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF_SET(__pyx_v_n_term, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":201
 * 
 *         if isinstance(n_term, basestring):
 *             if n_term != structure_constants.N_TERM_DEFAULT:             # <<<<<<<<<<<<<<
 *                 n_term = get_modification_by_name(n_term)
 *             else:
 */
      goto __pyx_L11;
    }

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":204
 *                 n_term = get_modification_by_name(n_term)
 *             else:
 *                 n_term = None             # <<<<<<<<<<<<<<
 *         if isinstance(c_term, basestring):
 *             if c_term != structure_constants.C_TERM_DEFAULT:
 */
    /*else*/ {
      __Pyx_INCREF(Py_None);
      __Pyx_DECREF_SET(__pyx_v_n_term, Py_None);
    }
    __pyx_L11:;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":200
 *             glycosylation_manager.set_aggregate(glycan)
 * 
 *         if isinstance(n_term, basestring):             # <<<<<<<<<<<<<<
 *             if n_term != structure_constants.N_TERM_DEFAULT:
 *                 n_term = get_modification_by_name(n_term)
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":205
 *             else:
 *                 n_term = None
 *         if isinstance(c_term, basestring):             # <<<<<<<<<<<<<<
 *             if c_term != structure_constants.C_TERM_DEFAULT:
 *                 c_term = get_modification_by_name(c_term)
 */
  __pyx_t_14 = __Pyx_PyBaseString_Check(__pyx_v_c_term); 
  __pyx_t_15 = (__pyx_t_14 != 0);
  if (__pyx_t_15) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":206
 *                 n_term = None
 *         if isinstance(c_term, basestring):
 *             if c_term != structure_constants.C_TERM_DEFAULT:             # <<<<<<<<<<<<<<
 *                 c_term = get_modification_by_name(c_term)
 *             else:
 */
    __pyx_t_15 = (__Pyx_PyString_Equals(__pyx_v_c_term, __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_structure_constants->C_TERM_DEFAULT, Py_NE)); if (unlikely(__pyx_t_15 < 0)) __PYX_ERR(0, 206, __pyx_L1_error)
    if (__pyx_t_15) {

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":207
 *         if isinstance(c_term, basestring):
 *             if c_term != structure_constants.C_TERM_DEFAULT:
 *                 c_term = get_modification_by_name(c_term)             # <<<<<<<<<<<<<<
 *             else:
 *                 c_term = None
 */
      if (!(likely(__Pyx_PyBaseString_CheckExact(__pyx_v_c_term))||((__pyx_v_c_term) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", (PY_MAJOR_VERSION < 3 ? "basestring" : "str"), Py_TYPE(__pyx_v_c_term)->tp_name), 0))) __PYX_ERR(0, 207, __pyx_L1_error)
      __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_get_modification_by_name(((PyObject*)__pyx_v_c_term), 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 207, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF_SET(__pyx_v_c_term, __pyx_t_1);
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":206
 *                 n_term = None
 *         if isinstance(c_term, basestring):
 *             if c_term != structure_constants.C_TERM_DEFAULT:             # <<<<<<<<<<<<<<
 *                 c_term = get_modification_by_name(c_term)
 *             else:
 */
      goto __pyx_L13;
    }

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":209
 *                 c_term = get_modification_by_name(c_term)
 *             else:
 *                 c_term = None             # <<<<<<<<<<<<<<
 *         n_term_group = _make_terminal_group(structure_constants.N_TERM_DEFAULT, n_term)
 *         c_term_group = _make_terminal_group(structure_constants.C_TERM_DEFAULT, c_term)
 */
    /*else*/ {
      __Pyx_INCREF(Py_None);
      __Pyx_DECREF_SET(__pyx_v_c_term, Py_None);
    }
    __pyx_L13:;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":205
 *             else:
 *                 n_term = None
 *         if isinstance(c_term, basestring):             # <<<<<<<<<<<<<<
 *             if c_term != structure_constants.C_TERM_DEFAULT:
 *                 c_term = get_modification_by_name(c_term)
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":210
 *             else:
 *                 c_term = None
 *         n_term_group = _make_terminal_group(structure_constants.N_TERM_DEFAULT, n_term)             # <<<<<<<<<<<<<<
 *         c_term_group = _make_terminal_group(structure_constants.C_TERM_DEFAULT, c_term)
 *         self._init_termini(
 */
  __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_structure_constants->N_TERM_DEFAULT;
  __Pyx_INCREF(__pyx_t_1);
  if (!(likely(((__pyx_v_n_term) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_n_term, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase))))) __PYX_ERR(0, 210, __pyx_L1_error)
  __pyx_t_17.__pyx_n = 1;
  __pyx_t_17.modification = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_n_term);
  __pyx_t_2 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__make_terminal_group(((PyObject*)__pyx_t_1), &__pyx_t_17)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 210, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_n_term_group = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":211
 *                 c_term = None
 *         n_term_group = _make_terminal_group(structure_constants.N_TERM_DEFAULT, n_term)
 *         c_term_group = _make_terminal_group(structure_constants.C_TERM_DEFAULT, c_term)             # <<<<<<<<<<<<<<
 *         self._init_termini(
 *             n_term_group,
 */
  __pyx_t_2 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_structure_constants->C_TERM_DEFAULT;
  __Pyx_INCREF(__pyx_t_2);
  if (!(likely(((__pyx_v_c_term) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_c_term, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase))))) __PYX_ERR(0, 211, __pyx_L1_error)
  __pyx_t_17.__pyx_n = 1;
  __pyx_t_17.modification = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_c_term);
  __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__make_terminal_group(((PyObject*)__pyx_t_2), &__pyx_t_17)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 211, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v_c_term_group = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":212
 *         n_term_group = _make_terminal_group(structure_constants.N_TERM_DEFAULT, n_term)
 *         c_term_group = _make_terminal_group(structure_constants.C_TERM_DEFAULT, c_term)
 *         self._init_termini(             # <<<<<<<<<<<<<<
 *             n_term_group,
 *             c_term_group)
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->_init_termini(__pyx_v_self, __pyx_v_n_term_group, __pyx_v_c_term_group, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 212, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":159
 *             self._init_from_string(sequence, parser_function)
 * 
 *     cpdef _init_from_parsed_string(self, list seq_list, glycan=None, n_term=None, c_term=None):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, j, n, m
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore._init_from_parsed_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_sequence);
  __Pyx_XDECREF(__pyx_v_item);
  __Pyx_XDECREF(__pyx_v_mods);
  __Pyx_XDECREF(__pyx_v_mod_strs);
  __Pyx_XDECREF((PyObject *)__pyx_v_mod);
  __Pyx_XDECREF((PyObject *)__pyx_v_res);
  __Pyx_XDECREF((PyObject *)__pyx_v_position);
  __Pyx_XDECREF((PyObject *)__pyx_v_glycosylation_manager);
  __Pyx_XDECREF(__pyx_v_mod_str);
  __Pyx_XDECREF((PyObject *)__pyx_v_n_term_group);
  __Pyx_XDECREF((PyObject *)__pyx_v_c_term_group);
  __Pyx_XDECREF(__pyx_v_n_term);
  __Pyx_XDECREF(__pyx_v_c_term);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_3_init_from_parsed_string(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_2_init_from_parsed_string[] = "_PeptideSequenceCore._init_from_parsed_string(self, list seq_list, glycan=None, n_term=None, c_term=None)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_3_init_from_parsed_string(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_seq_list = 0;
  PyObject *__pyx_v_glycan = 0;
  PyObject *__pyx_v_n_term = 0;
  PyObject *__pyx_v_c_term = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_init_from_parsed_string (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_seq_list,&__pyx_n_s_glycan,&__pyx_n_s_n_term,&__pyx_n_s_c_term,0};
    PyObject* values[4] = {0,0,0,0};
    values[1] = ((PyObject *)Py_None);
    values[2] = ((PyObject *)Py_None);
    values[3] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_seq_list)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_glycan);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_n_term);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_c_term);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_init_from_parsed_string") < 0)) __PYX_ERR(0, 159, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_seq_list = ((PyObject*)values[0]);
    __pyx_v_glycan = values[1];
    __pyx_v_n_term = values[2];
    __pyx_v_c_term = values[3];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_init_from_parsed_string", 0, 1, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 159, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore._init_from_parsed_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_seq_list), (&PyList_Type), 1, "seq_list", 1))) __PYX_ERR(0, 159, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_2_init_from_parsed_string(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self), __pyx_v_seq_list, __pyx_v_glycan, __pyx_v_n_term, __pyx_v_c_term);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_2_init_from_parsed_string(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_seq_list, PyObject *__pyx_v_glycan, PyObject *__pyx_v_n_term, PyObject *__pyx_v_c_term) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_parsed_string __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_init_from_parsed_string", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 3;
  __pyx_t_2.glycan = __pyx_v_glycan;
  __pyx_t_2.n_term = __pyx_v_n_term;
  __pyx_t_2.c_term = __pyx_v_c_term;
  __pyx_t_1 = __pyx_vtabptr_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore->_init_from_parsed_string(__pyx_v_self, __pyx_v_seq_list, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 159, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore._init_from_parsed_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":216
 *             c_term_group)
 * 
 *     cpdef _init_from_string(self, sequence, parser_function):             # <<<<<<<<<<<<<<
 *         """Initialize a :class:`PeptideSequence` from a parse-able string.
 * 
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_5_init_from_string(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_string(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_sequence, PyObject *__pyx_v_parser_function, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_seq_list = NULL;
  CYTHON_UNUSED PyObject *__pyx_v_modifications = NULL;
  PyObject *__pyx_v_glycan = NULL;
  PyObject *__pyx_v_n_term = NULL;
  PyObject *__pyx_v_c_term = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  int __pyx_t_8;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_parsed_string __pyx_t_11;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_init_from_string", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_init_from_string); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 216, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_5_init_from_string)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_sequence, __pyx_v_parser_function};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 216, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, __pyx_v_sequence, __pyx_v_parser_function};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 216, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 216, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(__pyx_v_sequence);
          __Pyx_GIVEREF(__pyx_v_sequence);
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_sequence);
          __Pyx_INCREF(__pyx_v_parser_function);
          __Pyx_GIVEREF(__pyx_v_parser_function);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_parser_function);
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 216, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":226
 *             The parsing function to use
 *         """
 *         if sequence is None or sequence == "":             # <<<<<<<<<<<<<<
 *             pass
 *         else:
 */
  __pyx_t_8 = (__pyx_v_sequence == Py_None);
  __pyx_t_9 = (__pyx_t_8 != 0);
  if (!__pyx_t_9) {
  } else {
    __pyx_t_7 = __pyx_t_9;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_9 = (__Pyx_PyString_Equals(__pyx_v_sequence, __pyx_kp_s_, Py_EQ)); if (unlikely(__pyx_t_9 < 0)) __PYX_ERR(0, 226, __pyx_L1_error)
  __pyx_t_7 = __pyx_t_9;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_7) {
    goto __pyx_L3;
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":229
 *             pass
 *         else:
 *             seq_list, modifications, glycan, n_term, c_term = <tuple>parser_function(             # <<<<<<<<<<<<<<
 *                 sequence)
 *             self._init_from_parsed_string(seq_list, glycan, n_term, c_term)
 */
  /*else*/ {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":230
 *         else:
 *             seq_list, modifications, glycan, n_term, c_term = <tuple>parser_function(
 *                 sequence)             # <<<<<<<<<<<<<<
 *             self._init_from_parsed_string(seq_list, glycan, n_term, c_term)
 * 
 */
    __Pyx_INCREF(__pyx_v_parser_function);
    __pyx_t_2 = __pyx_v_parser_function; __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_3, __pyx_v_sequence) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v_sequence);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 229, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":229
 *             pass
 *         else:
 *             seq_list, modifications, glycan, n_term, c_term = <tuple>parser_function(             # <<<<<<<<<<<<<<
 *                 sequence)
 *             self._init_from_parsed_string(seq_list, glycan, n_term, c_term)
 */
    __pyx_t_2 = __pyx_t_1;
    __Pyx_INCREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (likely(__pyx_t_2 != Py_None)) {
      PyObject* sequence = __pyx_t_2;
      Py_ssize_t size = __Pyx_PySequence_SIZE(sequence);
      if (unlikely(size != 5)) {
        if (size > 5) __Pyx_RaiseTooManyValuesError(5);
        else if (size >= 0) __Pyx_RaiseNeedMoreValuesError(size);
        __PYX_ERR(0, 229, __pyx_L1_error)
      }
      #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
      __pyx_t_1 = PyTuple_GET_ITEM(sequence, 0); 
      __pyx_t_3 = PyTuple_GET_ITEM(sequence, 1); 
      __pyx_t_6 = PyTuple_GET_ITEM(sequence, 2); 
      __pyx_t_4 = PyTuple_GET_ITEM(sequence, 3); 
      __pyx_t_10 = PyTuple_GET_ITEM(sequence, 4); 
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_6);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_10);
      #else
      {
        Py_ssize_t i;
        PyObject** temps[5] = {&__pyx_t_1,&__pyx_t_3,&__pyx_t_6,&__pyx_t_4,&__pyx_t_10};
        for (i=0; i < 5; i++) {
          PyObject* item = PySequence_ITEM(sequence, i); if (unlikely(!item)) __PYX_ERR(0, 229, __pyx_L1_error)
          __Pyx_GOTREF(item);
          *(temps[i]) = item;
        }
      }
      #endif
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    } else {
      __Pyx_RaiseNoneNotIterableError(); __PYX_ERR(0, 229, __pyx_L1_error)
    }
    __pyx_v_seq_list = __pyx_t_1;
    __pyx_t_1 = 0;
    __pyx_v_modifications = __pyx_t_3;
    __pyx_t_3 = 0;
    __pyx_v_glycan = __pyx_t_6;
    __pyx_t_6 = 0;
    __pyx_v_n_term = __pyx_t_4;
    __pyx_t_4 = 0;
    __pyx_v_c_term = __pyx_t_10;
    __pyx_t_10 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":231
 *             seq_list, modifications, glycan, n_term, c_term = <tuple>parser_function(
 *                 sequence)
 *             self._init_from_parsed_string(seq_list, glycan, n_term, c_term)             # <<<<<<<<<<<<<<
 * 
 *     cpdef _init_from_components(self, list seq_list, glycan_composition=None, ModificationBase n_term=None, ModificationBase c_term=None):
 */
    if (!(likely(PyList_CheckExact(__pyx_v_seq_list))||((__pyx_v_seq_list) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_seq_list)->tp_name), 0))) __PYX_ERR(0, 231, __pyx_L1_error)
    __pyx_t_11.__pyx_n = 3;
    __pyx_t_11.glycan = __pyx_v_glycan;
    __pyx_t_11.n_term = __pyx_v_n_term;
    __pyx_t_11.c_term = __pyx_v_c_term;
    __pyx_t_2 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->_init_from_parsed_string(__pyx_v_self, ((PyObject*)__pyx_v_seq_list), 0, &__pyx_t_11); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 231, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  }
  __pyx_L3:;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":216
 *             c_term_group)
 * 
 *     cpdef _init_from_string(self, sequence, parser_function):             # <<<<<<<<<<<<<<
 *         """Initialize a :class:`PeptideSequence` from a parse-able string.
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore._init_from_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_seq_list);
  __Pyx_XDECREF(__pyx_v_modifications);
  __Pyx_XDECREF(__pyx_v_glycan);
  __Pyx_XDECREF(__pyx_v_n_term);
  __Pyx_XDECREF(__pyx_v_c_term);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_5_init_from_string(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_4_init_from_string[] = "_PeptideSequenceCore._init_from_string(self, sequence, parser_function)\nInitialize a :class:`PeptideSequence` from a parse-able string.\n\n        Parameters\n        ----------\n        sequence : :class:`str`\n            The string to parse\n        parser_function : :class:`Callable`\n            The parsing function to use\n        ";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_5_init_from_string(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_sequence = 0;
  PyObject *__pyx_v_parser_function = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_init_from_string (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_sequence,&__pyx_n_s_parser_function,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_sequence)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_parser_function)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_init_from_string", 1, 2, 2, 1); __PYX_ERR(0, 216, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_init_from_string") < 0)) __PYX_ERR(0, 216, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_sequence = values[0];
    __pyx_v_parser_function = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_init_from_string", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 216, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore._init_from_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_4_init_from_string(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self), __pyx_v_sequence, __pyx_v_parser_function);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_4_init_from_string(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_sequence, PyObject *__pyx_v_parser_function) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_init_from_string", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_string(__pyx_v_self, __pyx_v_sequence, __pyx_v_parser_function, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 216, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore._init_from_string", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":233
 *             self._init_from_parsed_string(seq_list, glycan, n_term, c_term)
 * 
 *     cpdef _init_from_components(self, list seq_list, glycan_composition=None, ModificationBase n_term=None, ModificationBase c_term=None):             # <<<<<<<<<<<<<<
 *         """Initialize a :class:`PeptideSequence`'s state from a sequence of (
 *         :class:`~.AminoAcidResidue`, [:class:`~.Modification`, ]) pairs, with optional extra information
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_7_init_from_components(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_components(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_seq_list, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_components *__pyx_optional_args) {
  PyObject *__pyx_v_glycan_composition = ((PyObject *)Py_None);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_n_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)Py_None);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_c_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)Py_None);
  size_t __pyx_v_i;
  size_t __pyx_v_j;
  size_t __pyx_v_n;
  size_t __pyx_v_m;
  PyObject *__pyx_v_sequence = 0;
  PyObject *__pyx_v_mods = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_item = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_position = 0;
  double __pyx_v_mass;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_mod = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *__pyx_v_glycosylation_manager = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  size_t __pyx_t_9;
  PyObject *__pyx_t_10;
  size_t __pyx_t_11;
  size_t __pyx_t_12;
  size_t __pyx_t_13;
  int __pyx_t_14;
  int __pyx_t_15;
  struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods__make_terminal_group __pyx_t_16;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_init_from_components", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_glycan_composition = __pyx_optional_args->glycan_composition;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_n_term = __pyx_optional_args->n_term;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_c_term = __pyx_optional_args->c_term;
        }
      }
    }
  }
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_init_from_components); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 233, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_7_init_from_components)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[5] = {__pyx_t_4, __pyx_v_seq_list, __pyx_v_glycan_composition, ((PyObject *)__pyx_v_n_term), ((PyObject *)__pyx_v_c_term)};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 233, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[5] = {__pyx_t_4, __pyx_v_seq_list, __pyx_v_glycan_composition, ((PyObject *)__pyx_v_n_term), ((PyObject *)__pyx_v_c_term)};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 4+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 233, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(4+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 233, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(__pyx_v_seq_list);
          __Pyx_GIVEREF(__pyx_v_seq_list);
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, __pyx_v_seq_list);
          __Pyx_INCREF(__pyx_v_glycan_composition);
          __Pyx_GIVEREF(__pyx_v_glycan_composition);
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_v_glycan_composition);
          __Pyx_INCREF(((PyObject *)__pyx_v_n_term));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_n_term));
          PyTuple_SET_ITEM(__pyx_t_6, 2+__pyx_t_5, ((PyObject *)__pyx_v_n_term));
          __Pyx_INCREF(((PyObject *)__pyx_v_c_term));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_c_term));
          PyTuple_SET_ITEM(__pyx_t_6, 3+__pyx_t_5, ((PyObject *)__pyx_v_c_term));
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 233, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":259
 *             GlycosylationManager glycosylation_manager
 * 
 *         i = 0             # <<<<<<<<<<<<<<
 *         n = PyList_GET_SIZE(seq_list)
 *         self.sequence = sequence = PyList_New(n)
 */
  __pyx_v_i = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":260
 * 
 *         i = 0
 *         n = PyList_GET_SIZE(seq_list)             # <<<<<<<<<<<<<<
 *         self.sequence = sequence = PyList_New(n)
 *         mass = 0
 */
  __pyx_v_n = PyList_GET_SIZE(__pyx_v_seq_list);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":261
 *         i = 0
 *         n = PyList_GET_SIZE(seq_list)
 *         self.sequence = sequence = PyList_New(n)             # <<<<<<<<<<<<<<
 *         mass = 0
 *         glycosylation_manager = <GlycosylationManager>self._glycosylation_manager
 */
  __pyx_t_1 = PyList_New(__pyx_v_n); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.sequence);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.sequence);
  __pyx_v_self->__pyx_base.sequence = ((PyObject*)__pyx_t_1);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_sequence = ((PyObject*)__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":262
 *         n = PyList_GET_SIZE(seq_list)
 *         self.sequence = sequence = PyList_New(n)
 *         mass = 0             # <<<<<<<<<<<<<<
 *         glycosylation_manager = <GlycosylationManager>self._glycosylation_manager
 *         for i in range(n):
 */
  __pyx_v_mass = 0.0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":263
 *         self.sequence = sequence = PyList_New(n)
 *         mass = 0
 *         glycosylation_manager = <GlycosylationManager>self._glycosylation_manager             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             item = <SequencePosition>PyList_GET_ITEM(seq_list, i)
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base._glycosylation_manager;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_glycosylation_manager = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":264
 *         mass = 0
 *         glycosylation_manager = <GlycosylationManager>self._glycosylation_manager
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             item = <SequencePosition>PyList_GET_ITEM(seq_list, i)
 *             mass += item.amino_acid.mass
 */
  __pyx_t_7 = __pyx_v_n;
  __pyx_t_8 = __pyx_t_7;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":265
 *         glycosylation_manager = <GlycosylationManager>self._glycosylation_manager
 *         for i in range(n):
 *             item = <SequencePosition>PyList_GET_ITEM(seq_list, i)             # <<<<<<<<<<<<<<
 *             mass += item.amino_acid.mass
 *             m = PyList_GET_SIZE(item.modifications)
 */
    __pyx_t_10 = PyList_GET_ITEM(__pyx_v_seq_list, __pyx_v_i);
    __pyx_t_1 = ((PyObject *)__pyx_t_10);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_item, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":266
 *         for i in range(n):
 *             item = <SequencePosition>PyList_GET_ITEM(seq_list, i)
 *             mass += item.amino_acid.mass             # <<<<<<<<<<<<<<
 *             m = PyList_GET_SIZE(item.modifications)
 *             mods = PyList_New(m)
 */
    __pyx_v_mass = (__pyx_v_mass + __pyx_v_item->amino_acid->mass);

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":267
 *             item = <SequencePosition>PyList_GET_ITEM(seq_list, i)
 *             mass += item.amino_acid.mass
 *             m = PyList_GET_SIZE(item.modifications)             # <<<<<<<<<<<<<<
 *             mods = PyList_New(m)
 *             for j in range(m):
 */
    __pyx_t_1 = __pyx_v_item->modifications;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v_m = PyList_GET_SIZE(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":268
 *             mass += item.amino_acid.mass
 *             m = PyList_GET_SIZE(item.modifications)
 *             mods = PyList_New(m)             # <<<<<<<<<<<<<<
 *             for j in range(m):
 *                 mod = <ModificationBase>PyList_GET_ITEM(item.modifications, j)
 */
    __pyx_t_1 = PyList_New(__pyx_v_m); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 268, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_mods, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":269
 *             m = PyList_GET_SIZE(item.modifications)
 *             mods = PyList_New(m)
 *             for j in range(m):             # <<<<<<<<<<<<<<
 *                 mod = <ModificationBase>PyList_GET_ITEM(item.modifications, j)
 *                 Py_INCREF(mod)
 */
    __pyx_t_11 = __pyx_v_m;
    __pyx_t_12 = __pyx_t_11;
    for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
      __pyx_v_j = __pyx_t_13;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":270
 *             mods = PyList_New(m)
 *             for j in range(m):
 *                 mod = <ModificationBase>PyList_GET_ITEM(item.modifications, j)             # <<<<<<<<<<<<<<
 *                 Py_INCREF(mod)
 *                 PyList_SetItem(mods, j, mod)
 */
      __pyx_t_1 = __pyx_v_item->modifications;
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_10 = PyList_GET_ITEM(__pyx_t_1, __pyx_v_j);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = ((PyObject *)__pyx_t_10);
      __Pyx_INCREF(__pyx_t_1);
      __Pyx_XDECREF_SET(__pyx_v_mod, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":271
 *             for j in range(m):
 *                 mod = <ModificationBase>PyList_GET_ITEM(item.modifications, j)
 *                 Py_INCREF(mod)             # <<<<<<<<<<<<<<
 *                 PyList_SetItem(mods, j, mod)
 *                 if mod.is_tracked_for(ModificationCategory_glycosylation):
 */
      Py_INCREF(((PyObject *)__pyx_v_mod));

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":272
 *                 mod = <ModificationBase>PyList_GET_ITEM(item.modifications, j)
 *                 Py_INCREF(mod)
 *                 PyList_SetItem(mods, j, mod)             # <<<<<<<<<<<<<<
 *                 if mod.is_tracked_for(ModificationCategory_glycosylation):
 *                     glycosylation_manager[i] = mod
 */
      __pyx_t_5 = PyList_SetItem(__pyx_v_mods, __pyx_v_j, ((PyObject *)__pyx_v_mod)); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 272, __pyx_L1_error)

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":273
 *                 Py_INCREF(mod)
 *                 PyList_SetItem(mods, j, mod)
 *                 if mod.is_tracked_for(ModificationCategory_glycosylation):             # <<<<<<<<<<<<<<
 *                     glycosylation_manager[i] = mod
 *                 mass += mod.mass
 */
      __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationCategory_glycosylation;
      __Pyx_INCREF(__pyx_t_1);
      __pyx_t_14 = (((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_mod->__pyx_vtab)->is_tracked_for(__pyx_v_mod, __pyx_t_1, 0) != 0);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      if (__pyx_t_14) {

        /* "glycopeptidepy/_c/structure/sequence_methods.pyx":274
 *                 PyList_SetItem(mods, j, mod)
 *                 if mod.is_tracked_for(ModificationCategory_glycosylation):
 *                     glycosylation_manager[i] = mod             # <<<<<<<<<<<<<<
 *                 mass += mod.mass
 *             position = SequencePosition._create(item.amino_acid, mods)
 */
        if (unlikely(__Pyx_SetItemInt(((PyObject *)__pyx_v_glycosylation_manager), __pyx_v_i, ((PyObject *)__pyx_v_mod), size_t, 0, __Pyx_PyInt_FromSize_t, 0, 0, 1) < 0)) __PYX_ERR(0, 274, __pyx_L1_error)

        /* "glycopeptidepy/_c/structure/sequence_methods.pyx":273
 *                 Py_INCREF(mod)
 *                 PyList_SetItem(mods, j, mod)
 *                 if mod.is_tracked_for(ModificationCategory_glycosylation):             # <<<<<<<<<<<<<<
 *                     glycosylation_manager[i] = mod
 *                 mass += mod.mass
 */
      }

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":275
 *                 if mod.is_tracked_for(ModificationCategory_glycosylation):
 *                     glycosylation_manager[i] = mod
 *                 mass += mod.mass             # <<<<<<<<<<<<<<
 *             position = SequencePosition._create(item.amino_acid, mods)
 *             Py_INCREF(position)
 */
      __pyx_v_mass = (__pyx_v_mass + __pyx_v_mod->mass);
    }

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":276
 *                     glycosylation_manager[i] = mod
 *                 mass += mod.mass
 *             position = SequencePosition._create(item.amino_acid, mods)             # <<<<<<<<<<<<<<
 *             Py_INCREF(position)
 *             PyList_SetItem(sequence, i, position)
 */
    __pyx_t_1 = ((PyObject *)__pyx_v_item->amino_acid);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_2 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_SequencePosition->_create(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_t_1), __pyx_v_mods)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 276, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_XDECREF_SET(__pyx_v_position, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":277
 *                 mass += mod.mass
 *             position = SequencePosition._create(item.amino_acid, mods)
 *             Py_INCREF(position)             # <<<<<<<<<<<<<<
 *             PyList_SetItem(sequence, i, position)
 * 
 */
    Py_INCREF(((PyObject *)__pyx_v_position));

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":278
 *             position = SequencePosition._create(item.amino_acid, mods)
 *             Py_INCREF(position)
 *             PyList_SetItem(sequence, i, position)             # <<<<<<<<<<<<<<
 * 
 *         self._mass = mass
 */
    __pyx_t_5 = PyList_SetItem(__pyx_v_sequence, __pyx_v_i, ((PyObject *)__pyx_v_position)); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 278, __pyx_L1_error)
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":280
 *             PyList_SetItem(sequence, i, position)
 * 
 *         self._mass = mass             # <<<<<<<<<<<<<<
 *         if glycan_composition is not None:
 *             glycosylation_manager.set_aggregate(glycan_composition.clone())
 */
  __pyx_v_self->__pyx_base._mass = __pyx_v_mass;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":281
 * 
 *         self._mass = mass
 *         if glycan_composition is not None:             # <<<<<<<<<<<<<<
 *             glycosylation_manager.set_aggregate(glycan_composition.clone())
 * 
 */
  __pyx_t_14 = (__pyx_v_glycan_composition != Py_None);
  __pyx_t_15 = (__pyx_t_14 != 0);
  if (__pyx_t_15) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":282
 *         self._mass = mass
 *         if glycan_composition is not None:
 *             glycosylation_manager.set_aggregate(glycan_composition.clone())             # <<<<<<<<<<<<<<
 * 
 *         self._init_termini(
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_glycan_composition, __pyx_n_s_clone); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 282, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_3)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_3);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_1, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_1);
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 282, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    (void)(((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager *)__pyx_v_glycosylation_manager->__pyx_vtab)->set_aggregate(__pyx_v_glycosylation_manager, __pyx_t_2));
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":281
 * 
 *         self._mass = mass
 *         if glycan_composition is not None:             # <<<<<<<<<<<<<<
 *             glycosylation_manager.set_aggregate(glycan_composition.clone())
 * 
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":285
 * 
 *         self._init_termini(
 *             _make_terminal_group(structure_constants.N_TERM_DEFAULT, n_term),             # <<<<<<<<<<<<<<
 *             _make_terminal_group(structure_constants.C_TERM_DEFAULT, c_term))
 * 
 */
  __pyx_t_2 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_structure_constants->N_TERM_DEFAULT;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_16.__pyx_n = 1;
  __pyx_t_16.modification = __pyx_v_n_term;
  __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__make_terminal_group(((PyObject*)__pyx_t_2), &__pyx_t_16)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 285, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":286
 *         self._init_termini(
 *             _make_terminal_group(structure_constants.N_TERM_DEFAULT, n_term),
 *             _make_terminal_group(structure_constants.C_TERM_DEFAULT, c_term))             # <<<<<<<<<<<<<<
 * 
 *     cpdef _init_termini(self, TerminalGroup n_term, TerminalGroup c_term):
 */
  __pyx_t_2 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_structure_constants->C_TERM_DEFAULT;
  __Pyx_INCREF(__pyx_t_2);
  __pyx_t_16.__pyx_n = 1;
  __pyx_t_16.modification = __pyx_v_c_term;
  __pyx_t_3 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__make_terminal_group(((PyObject*)__pyx_t_2), &__pyx_t_16)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 286, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":284
 *             glycosylation_manager.set_aggregate(glycan_composition.clone())
 * 
 *         self._init_termini(             # <<<<<<<<<<<<<<
 *             _make_terminal_group(structure_constants.N_TERM_DEFAULT, n_term),
 *             _make_terminal_group(structure_constants.C_TERM_DEFAULT, c_term))
 */
  __pyx_t_2 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->_init_termini(__pyx_v_self, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_1), ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_3), 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 284, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":233
 *             self._init_from_parsed_string(seq_list, glycan, n_term, c_term)
 * 
 *     cpdef _init_from_components(self, list seq_list, glycan_composition=None, ModificationBase n_term=None, ModificationBase c_term=None):             # <<<<<<<<<<<<<<
 *         """Initialize a :class:`PeptideSequence`'s state from a sequence of (
 *         :class:`~.AminoAcidResidue`, [:class:`~.Modification`, ]) pairs, with optional extra information
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore._init_from_components", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_sequence);
  __Pyx_XDECREF(__pyx_v_mods);
  __Pyx_XDECREF((PyObject *)__pyx_v_item);
  __Pyx_XDECREF((PyObject *)__pyx_v_position);
  __Pyx_XDECREF((PyObject *)__pyx_v_mod);
  __Pyx_XDECREF((PyObject *)__pyx_v_glycosylation_manager);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_7_init_from_components(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6_init_from_components[] = "_PeptideSequenceCore._init_from_components(self, list seq_list, glycan_composition=None, ModificationBase n_term=None, ModificationBase c_term=None)\nInitialize a :class:`PeptideSequence`'s state from a sequence of (\n        :class:`~.AminoAcidResidue`, [:class:`~.Modification`, ]) pairs, with optional extra information\n        defining the glycan composition aggregate and the terminal groups\n\n        Parameters\n        ----------\n        seq_list : :class:`Sequence`\n            The sequence of (:class:`~.AminoAcidResidue`, [:class:`~.Modification`, ])\n        glycan_composition : :class:`~.GlycanComposition`, optional\n            The aggregate glycan composition associated with the sequence not fully specified\n            by the modifications in ``seq_list``.\n        n_term : :class:`~.Modification`, optional\n            The N-terminal modification, if any\n        c_term : :class:`~.Modification`, optional\n            The C-terminal modification, if any\n        ";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_7_init_from_components(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_seq_list = 0;
  PyObject *__pyx_v_glycan_composition = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_n_term = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_c_term = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_init_from_components (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_seq_list,&__pyx_n_s_glycan_composition,&__pyx_n_s_n_term,&__pyx_n_s_c_term,0};
    PyObject* values[4] = {0,0,0,0};
    values[1] = ((PyObject *)Py_None);
    values[2] = (PyObject *)((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)Py_None);
    values[3] = (PyObject *)((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_seq_list)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_glycan_composition);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_n_term);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_c_term);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_init_from_components") < 0)) __PYX_ERR(0, 233, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_seq_list = ((PyObject*)values[0]);
    __pyx_v_glycan_composition = values[1];
    __pyx_v_n_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)values[2]);
    __pyx_v_c_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_init_from_components", 0, 1, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 233, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore._init_from_components", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_seq_list), (&PyList_Type), 1, "seq_list", 1))) __PYX_ERR(0, 233, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_n_term), __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase, 1, "n_term", 0))) __PYX_ERR(0, 233, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_c_term), __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase, 1, "c_term", 0))) __PYX_ERR(0, 233, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6_init_from_components(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self), __pyx_v_seq_list, __pyx_v_glycan_composition, __pyx_v_n_term, __pyx_v_c_term);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6_init_from_components(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_seq_list, PyObject *__pyx_v_glycan_composition, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_n_term, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_c_term) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_components __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_init_from_components", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 3;
  __pyx_t_2.glycan_composition = __pyx_v_glycan_composition;
  __pyx_t_2.n_term = __pyx_v_n_term;
  __pyx_t_2.c_term = __pyx_v_c_term;
  __pyx_t_1 = __pyx_vtabptr_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore->_init_from_components(__pyx_v_self, __pyx_v_seq_list, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 233, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore._init_from_components", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":288
 *             _make_terminal_group(structure_constants.C_TERM_DEFAULT, c_term))
 * 
 *     cpdef _init_termini(self, TerminalGroup n_term, TerminalGroup c_term):             # <<<<<<<<<<<<<<
 *         '''This method assumes that the terminal groups are not yet initialized,
 *         and that their mass contributions are not reflected in :attr:`_mass`
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_9_init_termini(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_termini(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_n_term, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_c_term, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_init_termini", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_init_termini); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 288, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_9_init_termini)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        __pyx_t_5 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
            __pyx_t_5 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_n_term), ((PyObject *)__pyx_v_c_term)};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 288, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_3)) {
          PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_n_term), ((PyObject *)__pyx_v_c_term)};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_3, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 288, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_GOTREF(__pyx_t_2);
        } else
        #endif
        {
          __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 288, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_6);
          if (__pyx_t_4) {
            __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
          }
          __Pyx_INCREF(((PyObject *)__pyx_v_n_term));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_n_term));
          PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, ((PyObject *)__pyx_v_n_term));
          __Pyx_INCREF(((PyObject *)__pyx_v_c_term));
          __Pyx_GIVEREF(((PyObject *)__pyx_v_c_term));
          PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, ((PyObject *)__pyx_v_c_term));
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_3, __pyx_t_6, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 288, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        }
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":292
 *         and that their mass contributions are not reflected in :attr:`_mass`
 *         '''
 *         self._n_term = n_term             # <<<<<<<<<<<<<<
 *         self._c_term = c_term
 *         self._mass += n_term.mass + c_term.mass
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_n_term));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_n_term));
  __Pyx_GOTREF(__pyx_v_self->__pyx_base._n_term);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base._n_term));
  __pyx_v_self->__pyx_base._n_term = __pyx_v_n_term;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":293
 *         '''
 *         self._n_term = n_term
 *         self._c_term = c_term             # <<<<<<<<<<<<<<
 *         self._mass += n_term.mass + c_term.mass
 * 
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_c_term));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_c_term));
  __Pyx_GOTREF(__pyx_v_self->__pyx_base._c_term);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base._c_term));
  __pyx_v_self->__pyx_base._c_term = __pyx_v_c_term;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":294
 *         self._n_term = n_term
 *         self._c_term = c_term
 *         self._mass += n_term.mass + c_term.mass             # <<<<<<<<<<<<<<
 * 
 *     cpdef _initialize_fields(self):
 */
  __pyx_v_self->__pyx_base._mass = (__pyx_v_self->__pyx_base._mass + (__pyx_v_n_term->mass + __pyx_v_c_term->mass));

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":288
 *             _make_terminal_group(structure_constants.C_TERM_DEFAULT, c_term))
 * 
 *     cpdef _init_termini(self, TerminalGroup n_term, TerminalGroup c_term):             # <<<<<<<<<<<<<<
 *         '''This method assumes that the terminal groups are not yet initialized,
 *         and that their mass contributions are not reflected in :attr:`_mass`
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore._init_termini", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_9_init_termini(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_8_init_termini[] = "_PeptideSequenceCore._init_termini(self, TerminalGroup n_term, TerminalGroup c_term)\nThis method assumes that the terminal groups are not yet initialized,\n        and that their mass contributions are not reflected in :attr:`_mass`\n        ";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_9_init_termini(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_n_term = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_c_term = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_init_termini (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_n_term,&__pyx_n_s_c_term,0};
    PyObject* values[2] = {0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_n_term)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_c_term)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("_init_termini", 1, 2, 2, 1); __PYX_ERR(0, 288, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "_init_termini") < 0)) __PYX_ERR(0, 288, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 2) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
    }
    __pyx_v_n_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)values[0]);
    __pyx_v_c_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)values[1]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("_init_termini", 1, 2, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 288, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore._init_termini", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_n_term), __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup, 1, "n_term", 0))) __PYX_ERR(0, 288, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_c_term), __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup, 1, "c_term", 0))) __PYX_ERR(0, 288, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_8_init_termini(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self), __pyx_v_n_term, __pyx_v_c_term);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_8_init_termini(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_n_term, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_c_term) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_init_termini", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_termini(__pyx_v_self, __pyx_v_n_term, __pyx_v_c_term, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 288, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore._init_termini", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":296
 *         self._mass += n_term.mass + c_term.mass
 * 
 *     cpdef _initialize_fields(self):             # <<<<<<<<<<<<<<
 *         """Initialize all mutable fields to their default, empty values.
 *         """
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_11_initialize_fields(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__initialize_fields(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_initialize_fields", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_initialize_fields); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 296, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_11_initialize_fields)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 296, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":299
 *         """Initialize all mutable fields to their default, empty values.
 *         """
 *         self._mass = 0.0             # <<<<<<<<<<<<<<
 *         self.sequence = []
 * 
 */
  __pyx_v_self->__pyx_base._mass = 0.0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":300
 *         """
 *         self._mass = 0.0
 *         self.sequence = []             # <<<<<<<<<<<<<<
 * 
 *         self._glycosylation_manager = GlycosylationManager._create(self, None)
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 300, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base.sequence);
  __Pyx_DECREF(__pyx_v_self->__pyx_base.sequence);
  __pyx_v_self->__pyx_base.sequence = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":302
 *         self.sequence = []
 * 
 *         self._glycosylation_manager = GlycosylationManager._create(self, None)             # <<<<<<<<<<<<<<
 * 
 *         self._n_term = None
 */
  __pyx_t_1 = ((PyObject *)__pyx_vtabptr_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager->_create(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), Py_None)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 302, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base._glycosylation_manager);
  __Pyx_DECREF(__pyx_v_self->__pyx_base._glycosylation_manager);
  __pyx_v_self->__pyx_base._glycosylation_manager = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":304
 *         self._glycosylation_manager = GlycosylationManager._create(self, None)
 * 
 *         self._n_term = None             # <<<<<<<<<<<<<<
 *         self._c_term = None
 * 
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base._n_term);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base._n_term));
  __pyx_v_self->__pyx_base._n_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)Py_None);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":305
 * 
 *         self._n_term = None
 *         self._c_term = None             # <<<<<<<<<<<<<<
 * 
 *         self._total_composition = None
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base._c_term);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base._c_term));
  __pyx_v_self->__pyx_base._c_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)Py_None);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":307
 *         self._c_term = None
 * 
 *         self._total_composition = None             # <<<<<<<<<<<<<<
 *         self._peptide_composition = None
 * 
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base._total_composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base._total_composition));
  __pyx_v_self->__pyx_base._total_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":308
 * 
 *         self._total_composition = None
 *         self._peptide_composition = None             # <<<<<<<<<<<<<<
 * 
 *     cpdef _invalidate(self):
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base._peptide_composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base._peptide_composition));
  __pyx_v_self->__pyx_base._peptide_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":296
 *         self._mass += n_term.mass + c_term.mass
 * 
 *     cpdef _initialize_fields(self):             # <<<<<<<<<<<<<<
 *         """Initialize all mutable fields to their default, empty values.
 *         """
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore._initialize_fields", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_11_initialize_fields(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_10_initialize_fields[] = "_PeptideSequenceCore._initialize_fields(self)\nInitialize all mutable fields to their default, empty values.\n        ";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_11_initialize_fields(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_initialize_fields (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_10_initialize_fields(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_10_initialize_fields(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_initialize_fields", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__initialize_fields(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 296, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore._initialize_fields", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":310
 *         self._peptide_composition = None
 * 
 *     cpdef _invalidate(self):             # <<<<<<<<<<<<<<
 *         self._total_composition = None
 *         self._peptide_composition = None
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_13_invalidate(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__invalidate(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_invalidate", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_invalidate); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 310, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_13_invalidate)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 310, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":311
 * 
 *     cpdef _invalidate(self):
 *         self._total_composition = None             # <<<<<<<<<<<<<<
 *         self._peptide_composition = None
 * 
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base._total_composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base._total_composition));
  __pyx_v_self->__pyx_base._total_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":312
 *     cpdef _invalidate(self):
 *         self._total_composition = None
 *         self._peptide_composition = None             # <<<<<<<<<<<<<<
 * 
 *     cdef TerminalGroup get_n_term(self):
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->__pyx_base._peptide_composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base._peptide_composition));
  __pyx_v_self->__pyx_base._peptide_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":310
 *         self._peptide_composition = None
 * 
 *     cpdef _invalidate(self):             # <<<<<<<<<<<<<<
 *         self._total_composition = None
 *         self._peptide_composition = None
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore._invalidate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_13_invalidate(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_12_invalidate[] = "_PeptideSequenceCore._invalidate(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_13_invalidate(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_invalidate (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_12_invalidate(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_12_invalidate(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_invalidate", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__invalidate(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 310, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore._invalidate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":314
 *         self._peptide_composition = None
 * 
 *     cdef TerminalGroup get_n_term(self):             # <<<<<<<<<<<<<<
 *         return self._n_term
 * 
 */

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_n_term(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_n_term", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":315
 * 
 *     cdef TerminalGroup get_n_term(self):
 *         return self._n_term             # <<<<<<<<<<<<<<
 * 
 *     cdef void set_n_term(self, _value):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._n_term));
  __pyx_r = __pyx_v_self->__pyx_base._n_term;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":314
 *         self._peptide_composition = None
 * 
 *     cdef TerminalGroup get_n_term(self):             # <<<<<<<<<<<<<<
 *         return self._n_term
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":317
 *         return self._n_term
 * 
 *     cdef void set_n_term(self, _value):             # <<<<<<<<<<<<<<
 *         cdef:
 *             TerminalGroup value
 */

static void __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_set_n_term(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v__value) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_value = 0;
  double __pyx_v_reset_mass;
  double __pyx_v_new_mass;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  double __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_n_term", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":321
 *             TerminalGroup value
 *             double reset_mass, new_mass
 *         if isinstance(_value, ModificationBase):             # <<<<<<<<<<<<<<
 *             value = self._n_term.modify(_value)
 *         else:
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v__value, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":322
 *             double reset_mass, new_mass
 *         if isinstance(_value, ModificationBase):
 *             value = self._n_term.modify(_value)             # <<<<<<<<<<<<<<
 *         else:
 *             value = _value
 */
    if (!(likely(((__pyx_v__value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v__value, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase))))) __PYX_ERR(0, 322, __pyx_L1_error)
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self->__pyx_base._n_term->__pyx_vtab)->modify(__pyx_v_self->__pyx_base._n_term, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v__value), 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 322, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_value = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":321
 *             TerminalGroup value
 *             double reset_mass, new_mass
 *         if isinstance(_value, ModificationBase):             # <<<<<<<<<<<<<<
 *             value = self._n_term.modify(_value)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":324
 *             value = self._n_term.modify(_value)
 *         else:
 *             value = _value             # <<<<<<<<<<<<<<
 * 
 *         self._invalidate()
 */
  /*else*/ {
    if (!(likely(((__pyx_v__value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v__value, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup))))) __PYX_ERR(0, 324, __pyx_L1_error)
    __pyx_t_3 = __pyx_v__value;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_value = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":326
 *             value = _value
 * 
 *         self._invalidate()             # <<<<<<<<<<<<<<
 *         reset_mass = 0
 *         try:
 */
  __pyx_t_3 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._invalidate(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 326, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":327
 * 
 *         self._invalidate()
 *         reset_mass = 0             # <<<<<<<<<<<<<<
 *         try:
 *             reset_mass = self._n_term.mass
 */
  __pyx_v_reset_mass = 0.0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":328
 *         self._invalidate()
 *         reset_mass = 0
 *         try:             # <<<<<<<<<<<<<<
 *             reset_mass = self._n_term.mass
 *         except AttributeError:
 */
  {
    (void)__pyx_t_4; (void)__pyx_t_5; (void)__pyx_t_6; /* mark used */
    /*try:*/ {

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":329
 *         reset_mass = 0
 *         try:
 *             reset_mass = self._n_term.mass             # <<<<<<<<<<<<<<
 *         except AttributeError:
 *             pass
 */
      __pyx_t_7 = __pyx_v_self->__pyx_base._n_term->mass;
      __pyx_v_reset_mass = __pyx_t_7;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":328
 *         self._invalidate()
 *         reset_mass = 0
 *         try:             # <<<<<<<<<<<<<<
 *             reset_mass = self._n_term.mass
 *         except AttributeError:
 */
    }
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":332
 *         except AttributeError:
 *             pass
 *         self._n_term = value             # <<<<<<<<<<<<<<
 *         new_mass = 0
 *         try:
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_value));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_value));
  __Pyx_GOTREF(__pyx_v_self->__pyx_base._n_term);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base._n_term));
  __pyx_v_self->__pyx_base._n_term = __pyx_v_value;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":333
 *             pass
 *         self._n_term = value
 *         new_mass = 0             # <<<<<<<<<<<<<<
 *         try:
 *             new_mass = value.mass
 */
  __pyx_v_new_mass = 0.0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":334
 *         self._n_term = value
 *         new_mass = 0
 *         try:             # <<<<<<<<<<<<<<
 *             new_mass = value.mass
 *         except AttributeError:
 */
  {
    (void)__pyx_t_6; (void)__pyx_t_5; (void)__pyx_t_4; /* mark used */
    /*try:*/ {

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":335
 *         new_mass = 0
 *         try:
 *             new_mass = value.mass             # <<<<<<<<<<<<<<
 *         except AttributeError:
 *             pass
 */
      __pyx_t_7 = __pyx_v_value->mass;
      __pyx_v_new_mass = __pyx_t_7;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":334
 *         self._n_term = value
 *         new_mass = 0
 *         try:             # <<<<<<<<<<<<<<
 *             new_mass = value.mass
 *         except AttributeError:
 */
    }
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":338
 *         except AttributeError:
 *             pass
 *         self._mass += new_mass - reset_mass             # <<<<<<<<<<<<<<
 * 
 *     cdef TerminalGroup get_c_term(self):
 */
  __pyx_v_self->__pyx_base._mass = (__pyx_v_self->__pyx_base._mass + (__pyx_v_new_mass - __pyx_v_reset_mass));

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":317
 *         return self._n_term
 * 
 *     cdef void set_n_term(self, _value):             # <<<<<<<<<<<<<<
 *         cdef:
 *             TerminalGroup value
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_WriteUnraisable("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.set_n_term", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_value);
  __Pyx_RefNannyFinishContext();
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":340
 *         self._mass += new_mass - reset_mass
 * 
 *     cdef TerminalGroup get_c_term(self):             # <<<<<<<<<<<<<<
 *         return self._c_term
 * 
 */

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_c_term(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_c_term", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":341
 * 
 *     cdef TerminalGroup get_c_term(self):
 *         return self._c_term             # <<<<<<<<<<<<<<
 * 
 *     cdef void set_c_term(self, _value):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._c_term));
  __pyx_r = __pyx_v_self->__pyx_base._c_term;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":340
 *         self._mass += new_mass - reset_mass
 * 
 *     cdef TerminalGroup get_c_term(self):             # <<<<<<<<<<<<<<
 *         return self._c_term
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":343
 *         return self._c_term
 * 
 *     cdef void set_c_term(self, _value):             # <<<<<<<<<<<<<<
 *         cdef:
 *             TerminalGroup value
 */

static void __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_set_c_term(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v__value) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_value = 0;
  double __pyx_v_reset_mass;
  double __pyx_v_new_mass;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  double __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("set_c_term", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":347
 *             TerminalGroup value
 *             double reset_mass, new_mass
 *         if isinstance(_value, ModificationBase):             # <<<<<<<<<<<<<<
 *             value = self._c_term.modify(_value)
 *         else:
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v__value, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":348
 *             double reset_mass, new_mass
 *         if isinstance(_value, ModificationBase):
 *             value = self._c_term.modify(_value)             # <<<<<<<<<<<<<<
 *         else:
 *             value = _value
 */
    if (!(likely(((__pyx_v__value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v__value, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase))))) __PYX_ERR(0, 348, __pyx_L1_error)
    __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self->__pyx_base._c_term->__pyx_vtab)->modify(__pyx_v_self->__pyx_base._c_term, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v__value), 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 348, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_v_value = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":347
 *             TerminalGroup value
 *             double reset_mass, new_mass
 *         if isinstance(_value, ModificationBase):             # <<<<<<<<<<<<<<
 *             value = self._c_term.modify(_value)
 *         else:
 */
    goto __pyx_L3;
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":350
 *             value = self._c_term.modify(_value)
 *         else:
 *             value = _value             # <<<<<<<<<<<<<<
 *         self._invalidate()
 *         reset_mass = 0
 */
  /*else*/ {
    if (!(likely(((__pyx_v__value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v__value, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup))))) __PYX_ERR(0, 350, __pyx_L1_error)
    __pyx_t_3 = __pyx_v__value;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_value = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_3);
    __pyx_t_3 = 0;
  }
  __pyx_L3:;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":351
 *         else:
 *             value = _value
 *         self._invalidate()             # <<<<<<<<<<<<<<
 *         reset_mass = 0
 *         try:
 */
  __pyx_t_3 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._invalidate(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), 0); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 351, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":352
 *             value = _value
 *         self._invalidate()
 *         reset_mass = 0             # <<<<<<<<<<<<<<
 *         try:
 *             reset_mass = self._c_term.mass
 */
  __pyx_v_reset_mass = 0.0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":353
 *         self._invalidate()
 *         reset_mass = 0
 *         try:             # <<<<<<<<<<<<<<
 *             reset_mass = self._c_term.mass
 *         except AttributeError:
 */
  {
    (void)__pyx_t_4; (void)__pyx_t_5; (void)__pyx_t_6; /* mark used */
    /*try:*/ {

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":354
 *         reset_mass = 0
 *         try:
 *             reset_mass = self._c_term.mass             # <<<<<<<<<<<<<<
 *         except AttributeError:
 *             pass
 */
      __pyx_t_7 = __pyx_v_self->__pyx_base._c_term->mass;
      __pyx_v_reset_mass = __pyx_t_7;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":353
 *         self._invalidate()
 *         reset_mass = 0
 *         try:             # <<<<<<<<<<<<<<
 *             reset_mass = self._c_term.mass
 *         except AttributeError:
 */
    }
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":357
 *         except AttributeError:
 *             pass
 *         self._c_term = value             # <<<<<<<<<<<<<<
 *         new_mass = 0
 *         try:
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_value));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_value));
  __Pyx_GOTREF(__pyx_v_self->__pyx_base._c_term);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base._c_term));
  __pyx_v_self->__pyx_base._c_term = __pyx_v_value;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":358
 *             pass
 *         self._c_term = value
 *         new_mass = 0             # <<<<<<<<<<<<<<
 *         try:
 *             new_mass = value.mass
 */
  __pyx_v_new_mass = 0.0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":359
 *         self._c_term = value
 *         new_mass = 0
 *         try:             # <<<<<<<<<<<<<<
 *             new_mass = value.mass
 *         except AttributeError:
 */
  {
    (void)__pyx_t_6; (void)__pyx_t_5; (void)__pyx_t_4; /* mark used */
    /*try:*/ {

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":360
 *         new_mass = 0
 *         try:
 *             new_mass = value.mass             # <<<<<<<<<<<<<<
 *         except AttributeError:
 *             pass
 */
      __pyx_t_7 = __pyx_v_value->mass;
      __pyx_v_new_mass = __pyx_t_7;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":359
 *         self._c_term = value
 *         new_mass = 0
 *         try:             # <<<<<<<<<<<<<<
 *             new_mass = value.mass
 *         except AttributeError:
 */
    }
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":363
 *         except AttributeError:
 *             pass
 *         self._mass += new_mass - reset_mass             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __pyx_v_self->__pyx_base._mass = (__pyx_v_self->__pyx_base._mass + (__pyx_v_new_mass - __pyx_v_reset_mass));

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":343
 *         return self._c_term
 * 
 *     cdef void set_c_term(self, _value):             # <<<<<<<<<<<<<<
 *         cdef:
 *             TerminalGroup value
 */

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_WriteUnraisable("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.set_c_term", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_value);
  __Pyx_RefNannyFinishContext();
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":365
 *         self._mass += new_mass - reset_mass
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         n_term = ""
 *         if self.n_term is not None:
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_15__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_15__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_14__repr__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_14__repr__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  PyObject *__pyx_v_n_term = NULL;
  PyObject *__pyx_v_c_term = NULL;
  PyObject *__pyx_v_aggregate = NULL;
  PyObject *__pyx_v_rep = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":366
 * 
 *     def __repr__(self):
 *         n_term = ""             # <<<<<<<<<<<<<<
 *         if self.n_term is not None:
 *             n_term = "({0})-".format(self.n_term)
 */
  __Pyx_INCREF(__pyx_kp_s_);
  __pyx_v_n_term = __pyx_kp_s_;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":367
 *     def __repr__(self):
 *         n_term = ""
 *         if self.n_term is not None:             # <<<<<<<<<<<<<<
 *             n_term = "({0})-".format(self.n_term)
 *         c_term = ""
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_n_term); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 367, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = (__pyx_t_1 != Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":368
 *         n_term = ""
 *         if self.n_term is not None:
 *             n_term = "({0})-".format(self.n_term)             # <<<<<<<<<<<<<<
 *         c_term = ""
 *         if self.c_term is not None:
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_0, __pyx_n_s_format); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 368, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_n_term); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 368, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 368, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_n_term, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":367
 *     def __repr__(self):
 *         n_term = ""
 *         if self.n_term is not None:             # <<<<<<<<<<<<<<
 *             n_term = "({0})-".format(self.n_term)
 *         c_term = ""
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":369
 *         if self.n_term is not None:
 *             n_term = "({0})-".format(self.n_term)
 *         c_term = ""             # <<<<<<<<<<<<<<
 *         if self.c_term is not None:
 *             c_term = "-({0})".format(self.c_term)
 */
  __Pyx_INCREF(__pyx_kp_s_);
  __pyx_v_c_term = __pyx_kp_s_;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":370
 *             n_term = "({0})-".format(self.n_term)
 *         c_term = ""
 *         if self.c_term is not None:             # <<<<<<<<<<<<<<
 *             c_term = "-({0})".format(self.c_term)
 *         aggregate = self._glycosylation_manager.aggregate if self._glycosylation_manager.aggregate is not None else ""
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_c_term); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 370, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = (__pyx_t_1 != Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":371
 *         c_term = ""
 *         if self.c_term is not None:
 *             c_term = "-({0})".format(self.c_term)             # <<<<<<<<<<<<<<
 *         aggregate = self._glycosylation_manager.aggregate if self._glycosylation_manager.aggregate is not None else ""
 *         rep = "{n_term}{sequence}{c_term}{glycan_aggregate}[{_mass}]".format(
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_0_2, __pyx_n_s_format); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 371, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_c_term); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 371, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_6 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_6)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_6);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_1 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_6, __pyx_t_5) : __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5);
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 371, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_DECREF_SET(__pyx_v_c_term, __pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":370
 *             n_term = "({0})-".format(self.n_term)
 *         c_term = ""
 *         if self.c_term is not None:             # <<<<<<<<<<<<<<
 *             c_term = "-({0})".format(self.c_term)
 *         aggregate = self._glycosylation_manager.aggregate if self._glycosylation_manager.aggregate is not None else ""
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":372
 *         if self.c_term is not None:
 *             c_term = "-({0})".format(self.c_term)
 *         aggregate = self._glycosylation_manager.aggregate if self._glycosylation_manager.aggregate is not None else ""             # <<<<<<<<<<<<<<
 *         rep = "{n_term}{sequence}{c_term}{glycan_aggregate}[{_mass}]".format(
 *             n_term=n_term, c_term=c_term,
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base._glycosylation_manager, __pyx_n_s_aggregate); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 372, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_2 = (__pyx_t_4 != Py_None);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  if ((__pyx_t_2 != 0)) {
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base._glycosylation_manager, __pyx_n_s_aggregate); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 372, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __pyx_t_4;
    __pyx_t_4 = 0;
  } else {
    __Pyx_INCREF(__pyx_kp_s_);
    __pyx_t_1 = __pyx_kp_s_;
  }
  __pyx_v_aggregate = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":373
 *             c_term = "-({0})".format(self.c_term)
 *         aggregate = self._glycosylation_manager.aggregate if self._glycosylation_manager.aggregate is not None else ""
 *         rep = "{n_term}{sequence}{c_term}{glycan_aggregate}[{_mass}]".format(             # <<<<<<<<<<<<<<
 *             n_term=n_term, c_term=c_term,
 *             glycan_aggregate=aggregate,
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_kp_s_n_term_sequence_c_term_glycan_a, __pyx_n_s_format); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":374
 *         aggregate = self._glycosylation_manager.aggregate if self._glycosylation_manager.aggregate is not None else ""
 *         rep = "{n_term}{sequence}{c_term}{glycan_aggregate}[{_mass}]".format(
 *             n_term=n_term, c_term=c_term,             # <<<<<<<<<<<<<<
 *             glycan_aggregate=aggregate,
 *             sequence=self.sequence,
 */
  __pyx_t_4 = __Pyx_PyDict_NewPresized(5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 374, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_n_term, __pyx_v_n_term) < 0) __PYX_ERR(0, 374, __pyx_L1_error)
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_c_term, __pyx_v_c_term) < 0) __PYX_ERR(0, 374, __pyx_L1_error)

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":375
 *         rep = "{n_term}{sequence}{c_term}{glycan_aggregate}[{_mass}]".format(
 *             n_term=n_term, c_term=c_term,
 *             glycan_aggregate=aggregate,             # <<<<<<<<<<<<<<
 *             sequence=self.sequence,
 *             _mass=self._mass)
 */
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_glycan_aggregate, __pyx_v_aggregate) < 0) __PYX_ERR(0, 374, __pyx_L1_error)

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":376
 *             n_term=n_term, c_term=c_term,
 *             glycan_aggregate=aggregate,
 *             sequence=self.sequence,             # <<<<<<<<<<<<<<
 *             _mass=self._mass)
 *         return rep
 */
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_sequence, __pyx_v_self->__pyx_base.sequence) < 0) __PYX_ERR(0, 374, __pyx_L1_error)

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":377
 *             glycan_aggregate=aggregate,
 *             sequence=self.sequence,
 *             _mass=self._mass)             # <<<<<<<<<<<<<<
 *         return rep
 * 
 */
  __pyx_t_5 = PyFloat_FromDouble(__pyx_v_self->__pyx_base._mass); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 377, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_mass, __pyx_t_5) < 0) __PYX_ERR(0, 374, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":373
 *             c_term = "-({0})".format(self.c_term)
 *         aggregate = self._glycosylation_manager.aggregate if self._glycosylation_manager.aggregate is not None else ""
 *         rep = "{n_term}{sequence}{c_term}{glycan_aggregate}[{_mass}]".format(             # <<<<<<<<<<<<<<
 *             n_term=n_term, c_term=c_term,
 *             glycan_aggregate=aggregate,
 */
  __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 373, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_5);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_v_rep = __pyx_t_5;
  __pyx_t_5 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":378
 *             sequence=self.sequence,
 *             _mass=self._mass)
 *         return rep             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_rep);
  __pyx_r = __pyx_v_rep;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":365
 *         self._mass += new_mass - reset_mass
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         n_term = ""
 *         if self.n_term is not None:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_n_term);
  __Pyx_XDECREF(__pyx_v_c_term);
  __Pyx_XDECREF(__pyx_v_aggregate);
  __Pyx_XDECREF(__pyx_v_rep);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":381
 * 
 *     @property
 *     def n_term(self):             # <<<<<<<<<<<<<<
 *         return self.get_n_term()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6n_term_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6n_term_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6n_term___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6n_term___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":382
 *     @property
 *     def n_term(self):
 *         return self.get_n_term()             # <<<<<<<<<<<<<<
 * 
 *     @n_term.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_n_term(__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 382, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":381
 * 
 *     @property
 *     def n_term(self):             # <<<<<<<<<<<<<<
 *         return self.get_n_term()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.n_term.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":385
 * 
 *     @n_term.setter
 *     def n_term(self, _value):             # <<<<<<<<<<<<<<
 *         self.set_n_term(_value)
 * 
 */

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6n_term_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v__value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6n_term_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v__value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6n_term_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self), ((PyObject *)__pyx_v__value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6n_term_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v__value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":386
 *     @n_term.setter
 *     def n_term(self, _value):
 *         self.set_n_term(_value)             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->set_n_term(__pyx_v_self, __pyx_v__value);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":385
 * 
 *     @n_term.setter
 *     def n_term(self, _value):             # <<<<<<<<<<<<<<
 *         self.set_n_term(_value)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":389
 * 
 *     @property
 *     def c_term(self):             # <<<<<<<<<<<<<<
 *         return self.get_c_term()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6c_term_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6c_term_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6c_term___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6c_term___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":390
 *     @property
 *     def c_term(self):
 *         return self.get_c_term()             # <<<<<<<<<<<<<<
 * 
 *     @c_term.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_c_term(__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 390, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":389
 * 
 *     @property
 *     def c_term(self):             # <<<<<<<<<<<<<<
 *         return self.get_c_term()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.c_term.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":393
 * 
 *     @c_term.setter
 *     def c_term(self, _value):             # <<<<<<<<<<<<<<
 *         self.set_c_term(_value)
 * 
 */

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6c_term_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v__value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6c_term_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v__value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6c_term_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self), ((PyObject *)__pyx_v__value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6c_term_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v__value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":394
 *     @c_term.setter
 *     def c_term(self, _value):
 *         self.set_c_term(_value)             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->set_c_term(__pyx_v_self, __pyx_v__value);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":393
 * 
 *     @c_term.setter
 *     def c_term(self, _value):             # <<<<<<<<<<<<<<
 *         self.set_c_term(_value)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":397
 * 
 *     @property
 *     def mass(self):             # <<<<<<<<<<<<<<
 *         return self._mass
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_4mass_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_4mass_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_4mass___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_4mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":398
 *     @property
 *     def mass(self):
 *         return self._mass             # <<<<<<<<<<<<<<
 * 
 *     @mass.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->__pyx_base._mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 398, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":397
 * 
 *     @property
 *     def mass(self):             # <<<<<<<<<<<<<<
 *         return self._mass
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.mass.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":401
 * 
 *     @mass.setter
 *     def mass(self, value):             # <<<<<<<<<<<<<<
 *         self._mass = value
 * 
 */

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_4mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_4mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_4mass_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_4mass_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":402
 *     @mass.setter
 *     def mass(self, value):
 *         self._mass = value             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 402, __pyx_L1_error)
  __pyx_v_self->__pyx_base._mass = __pyx_t_1;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":401
 * 
 *     @mass.setter
 *     def mass(self, value):             # <<<<<<<<<<<<<<
 *         self._mass = value
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.mass.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":405
 * 
 *     @property
 *     def peptide_backbone_mass(self):             # <<<<<<<<<<<<<<
 *         return self.peptide_composition().mass
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_21peptide_backbone_mass_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_21peptide_backbone_mass_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_21peptide_backbone_mass___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_21peptide_backbone_mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":406
 *     @property
 *     def peptide_backbone_mass(self):
 *         return self.peptide_composition().mass             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->peptide_composition(__pyx_v_self, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_mass_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 406, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":405
 * 
 *     @property
 *     def peptide_backbone_mass(self):             # <<<<<<<<<<<<<<
 *         return self.peptide_composition().mass
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.peptide_backbone_mass.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":409
 * 
 *     @property
 *     def total_mass(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             double total
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_10total_mass_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_10total_mass_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_10total_mass___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_10total_mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  double __pyx_v_total;
  size_t __pyx_v_i;
  size_t __pyx_v_j;
  size_t __pyx_v_m;
  size_t __pyx_v_n;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_position = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_mod = 0;
  PyObject *__pyx_v_gc = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  size_t __pyx_t_1;
  size_t __pyx_t_2;
  size_t __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  size_t __pyx_t_6;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  PyObject *__pyx_t_9;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  double __pyx_t_13;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":416
 *             ModificationBase mod
 * 
 *         n = self.get_size()             # <<<<<<<<<<<<<<
 *         total = 0
 *         for i in range(n):
 */
  __pyx_v_n = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_size(__pyx_v_self);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":417
 * 
 *         n = self.get_size()
 *         total = 0             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             position = self.get(i)
 */
  __pyx_v_total = 0.0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":418
 *         n = self.get_size()
 *         total = 0
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             position = self.get(i)
 *             total += position.amino_acid.mass
 */
  __pyx_t_1 = __pyx_v_n;
  __pyx_t_2 = __pyx_t_1;
  for (__pyx_t_3 = 0; __pyx_t_3 < __pyx_t_2; __pyx_t_3+=1) {
    __pyx_v_i = __pyx_t_3;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":419
 *         total = 0
 *         for i in range(n):
 *             position = self.get(i)             # <<<<<<<<<<<<<<
 *             total += position.amino_acid.mass
 *             m = PyList_Size(position.modifications)
 */
    __pyx_t_4 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.get(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), __pyx_v_i)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 419, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_XDECREF_SET(__pyx_v_position, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_t_4));
    __pyx_t_4 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":420
 *         for i in range(n):
 *             position = self.get(i)
 *             total += position.amino_acid.mass             # <<<<<<<<<<<<<<
 *             m = PyList_Size(position.modifications)
 *             for j in range(m):
 */
    __pyx_v_total = (__pyx_v_total + __pyx_v_position->amino_acid->mass);

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":421
 *             position = self.get(i)
 *             total += position.amino_acid.mass
 *             m = PyList_Size(position.modifications)             # <<<<<<<<<<<<<<
 *             for j in range(m):
 *                 mod = <ModificationBase>PyList_GetItem(position.modifications, j)
 */
    __pyx_t_4 = __pyx_v_position->modifications;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_5 = PyList_Size(__pyx_t_4); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1L))) __PYX_ERR(0, 421, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_m = __pyx_t_5;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":422
 *             total += position.amino_acid.mass
 *             m = PyList_Size(position.modifications)
 *             for j in range(m):             # <<<<<<<<<<<<<<
 *                 mod = <ModificationBase>PyList_GetItem(position.modifications, j)
 *                 if mod.is_tracked_for(ModificationCategory_glycosylation):
 */
    __pyx_t_6 = __pyx_v_m;
    __pyx_t_7 = __pyx_t_6;
    for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
      __pyx_v_j = __pyx_t_8;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":423
 *             m = PyList_Size(position.modifications)
 *             for j in range(m):
 *                 mod = <ModificationBase>PyList_GetItem(position.modifications, j)             # <<<<<<<<<<<<<<
 *                 if mod.is_tracked_for(ModificationCategory_glycosylation):
 *                     continue
 */
      __pyx_t_4 = __pyx_v_position->modifications;
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_9 = PyList_GetItem(__pyx_t_4, __pyx_v_j); if (unlikely(__pyx_t_9 == ((PyObject *)NULL))) __PYX_ERR(0, 423, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __pyx_t_4 = ((PyObject *)__pyx_t_9);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_XDECREF_SET(__pyx_v_mod, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_t_4));
      __pyx_t_4 = 0;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":424
 *             for j in range(m):
 *                 mod = <ModificationBase>PyList_GetItem(position.modifications, j)
 *                 if mod.is_tracked_for(ModificationCategory_glycosylation):             # <<<<<<<<<<<<<<
 *                     continue
 *                 total += mod.mass
 */
      __pyx_t_4 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationCategory_glycosylation;
      __Pyx_INCREF(__pyx_t_4);
      __pyx_t_10 = (((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_mod->__pyx_vtab)->is_tracked_for(__pyx_v_mod, __pyx_t_4, 0) != 0);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      if (__pyx_t_10) {

        /* "glycopeptidepy/_c/structure/sequence_methods.pyx":425
 *                 mod = <ModificationBase>PyList_GetItem(position.modifications, j)
 *                 if mod.is_tracked_for(ModificationCategory_glycosylation):
 *                     continue             # <<<<<<<<<<<<<<
 *                 total += mod.mass
 * 
 */
        goto __pyx_L5_continue;

        /* "glycopeptidepy/_c/structure/sequence_methods.pyx":424
 *             for j in range(m):
 *                 mod = <ModificationBase>PyList_GetItem(position.modifications, j)
 *                 if mod.is_tracked_for(ModificationCategory_glycosylation):             # <<<<<<<<<<<<<<
 *                     continue
 *                 total += mod.mass
 */
      }

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":426
 *                 if mod.is_tracked_for(ModificationCategory_glycosylation):
 *                     continue
 *                 total += mod.mass             # <<<<<<<<<<<<<<
 * 
 *         total += self._n_term.mass
 */
      __pyx_v_total = (__pyx_v_total + __pyx_v_mod->mass);
      __pyx_L5_continue:;
    }
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":428
 *                 total += mod.mass
 * 
 *         total += self._n_term.mass             # <<<<<<<<<<<<<<
 *         total += self._c_term.mass
 *         gc = self.glycan_composition
 */
  __pyx_v_total = (__pyx_v_total + __pyx_v_self->__pyx_base._n_term->mass);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":429
 * 
 *         total += self._n_term.mass
 *         total += self._c_term.mass             # <<<<<<<<<<<<<<
 *         gc = self.glycan_composition
 *         if gc:
 */
  __pyx_v_total = (__pyx_v_total + __pyx_v_self->__pyx_base._c_term->mass);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":430
 *         total += self._n_term.mass
 *         total += self._c_term.mass
 *         gc = self.glycan_composition             # <<<<<<<<<<<<<<
 *         if gc:
 *             total += PyFloat_AsDouble(gc.mass())
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_glycan_composition); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 430, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_v_gc = __pyx_t_4;
  __pyx_t_4 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":431
 *         total += self._c_term.mass
 *         gc = self.glycan_composition
 *         if gc:             # <<<<<<<<<<<<<<
 *             total += PyFloat_AsDouble(gc.mass())
 *         return total
 */
  __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_v_gc); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 431, __pyx_L1_error)
  if (__pyx_t_10) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":432
 *         gc = self.glycan_composition
 *         if gc:
 *             total += PyFloat_AsDouble(gc.mass())             # <<<<<<<<<<<<<<
 *         return total
 * 
 */
    __pyx_t_11 = __Pyx_PyObject_GetAttrStr(__pyx_v_gc, __pyx_n_s_mass_2); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 432, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_11);
    __pyx_t_12 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_11))) {
      __pyx_t_12 = PyMethod_GET_SELF(__pyx_t_11);
      if (likely(__pyx_t_12)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_11);
        __Pyx_INCREF(__pyx_t_12);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_11, function);
      }
    }
    __pyx_t_4 = (__pyx_t_12) ? __Pyx_PyObject_CallOneArg(__pyx_t_11, __pyx_t_12) : __Pyx_PyObject_CallNoArg(__pyx_t_11);
    __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
    if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 432, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
    __pyx_t_13 = PyFloat_AsDouble(__pyx_t_4); if (unlikely(__pyx_t_13 == ((double)-1.0) && PyErr_Occurred())) __PYX_ERR(0, 432, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_v_total = (__pyx_v_total + __pyx_t_13);

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":431
 *         total += self._c_term.mass
 *         gc = self.glycan_composition
 *         if gc:             # <<<<<<<<<<<<<<
 *             total += PyFloat_AsDouble(gc.mass())
 *         return total
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":433
 *         if gc:
 *             total += PyFloat_AsDouble(gc.mass())
 *         return total             # <<<<<<<<<<<<<<
 * 
 *     cpdef CComposition peptide_composition(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_4 = PyFloat_FromDouble(__pyx_v_total); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 433, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_r = __pyx_t_4;
  __pyx_t_4 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":409
 * 
 *     @property
 *     def total_mass(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             double total
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.total_mass.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_position);
  __Pyx_XDECREF((PyObject *)__pyx_v_mod);
  __Pyx_XDECREF(__pyx_v_gc);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":435
 *         return total
 * 
 *     cpdef CComposition peptide_composition(self):             # <<<<<<<<<<<<<<
 *         if self._peptide_composition is None:
 *             if self.glycan is None:
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_17peptide_composition(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_peptide_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_v_glycan_composition = NULL;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("peptide_composition", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_peptide_composition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 435, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_17peptide_composition)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 435, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 435, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":436
 * 
 *     cpdef CComposition peptide_composition(self):
 *         if self._peptide_composition is None:             # <<<<<<<<<<<<<<
 *             if self.glycan is None:
 *                 glycan_composition = CComposition()
 */
  __pyx_t_5 = (((PyObject *)__pyx_v_self->__pyx_base._peptide_composition) == Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":437
 *     cpdef CComposition peptide_composition(self):
 *         if self._peptide_composition is None:
 *             if self.glycan is None:             # <<<<<<<<<<<<<<
 *                 glycan_composition = CComposition()
 *             else:
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_glycan); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 437, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_6 = (__pyx_t_1 == Py_None);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_5 = (__pyx_t_6 != 0);
    if (__pyx_t_5) {

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":438
 *         if self._peptide_composition is None:
 *             if self.glycan is None:
 *                 glycan_composition = CComposition()             # <<<<<<<<<<<<<<
 *             else:
 *                 glycan_composition = self.glycan_composition.total_composition()
 */
      __pyx_t_1 = __Pyx_PyObject_CallNoArg(((PyObject *)__pyx_ptype_5glypy_11composition_12ccomposition_CComposition)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 438, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_v_glycan_composition = __pyx_t_1;
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":437
 *     cpdef CComposition peptide_composition(self):
 *         if self._peptide_composition is None:
 *             if self.glycan is None:             # <<<<<<<<<<<<<<
 *                 glycan_composition = CComposition()
 *             else:
 */
      goto __pyx_L4;
    }

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":440
 *                 glycan_composition = CComposition()
 *             else:
 *                 glycan_composition = self.glycan_composition.total_composition()             # <<<<<<<<<<<<<<
 *             self._peptide_base_composition = self.total_composition() - glycan_composition
 *         return self._peptide_base_composition
 */
    /*else*/ {
      __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_glycan_composition); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 440, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_total_composition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 440, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __pyx_t_2 = NULL;
      if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_2)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
      __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 440, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_v_glycan_composition = __pyx_t_1;
      __pyx_t_1 = 0;
    }
    __pyx_L4:;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":441
 *             else:
 *                 glycan_composition = self.glycan_composition.total_composition()
 *             self._peptide_base_composition = self.total_composition() - glycan_composition             # <<<<<<<<<<<<<<
 *         return self._peptide_base_composition
 * 
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->total_composition(__pyx_v_self, 0)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 441, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyNumber_Subtract(__pyx_t_1, __pyx_v_glycan_composition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 441, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_peptide_base_composition, __pyx_t_3) < 0) __PYX_ERR(0, 441, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":436
 * 
 *     cpdef CComposition peptide_composition(self):
 *         if self._peptide_composition is None:             # <<<<<<<<<<<<<<
 *             if self.glycan is None:
 *                 glycan_composition = CComposition()
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":442
 *                 glycan_composition = self.glycan_composition.total_composition()
 *             self._peptide_base_composition = self.total_composition() - glycan_composition
 *         return self._peptide_base_composition             # <<<<<<<<<<<<<<
 * 
 *     cpdef CComposition total_composition(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_peptide_base_composition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 442, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  if (!(likely(((__pyx_t_3) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_3, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 442, __pyx_L1_error)
  __pyx_r = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_3);
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":435
 *         return total
 * 
 *     cpdef CComposition peptide_composition(self):             # <<<<<<<<<<<<<<
 *         if self._peptide_composition is None:
 *             if self.glycan is None:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.peptide_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_glycan_composition);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_17peptide_composition(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_16peptide_composition[] = "_PeptideSequenceCore.peptide_composition(self) -> CComposition";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_17peptide_composition(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("peptide_composition (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_16peptide_composition(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_16peptide_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("peptide_composition", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_peptide_composition(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 435, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.peptide_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":444
 *         return self._peptide_base_composition
 * 
 *     cpdef CComposition total_composition(self):             # <<<<<<<<<<<<<<
 *         if self._total_composition is None:
 *             self._total_composition = _total_composition(self)
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_19total_composition(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_total_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("total_composition", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_total_composition); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 444, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_19total_composition)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 444, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 444, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":445
 * 
 *     cpdef CComposition total_composition(self):
 *         if self._total_composition is None:             # <<<<<<<<<<<<<<
 *             self._total_composition = _total_composition(self)
 *         return self._total_composition
 */
  __pyx_t_5 = (((PyObject *)__pyx_v_self->__pyx_base._total_composition) == Py_None);
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":446
 *     cpdef CComposition total_composition(self):
 *         if self._total_composition is None:
 *             self._total_composition = _total_composition(self)             # <<<<<<<<<<<<<<
 *         return self._total_composition
 * 
 */
    __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__total_composition(__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 446, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->__pyx_base._total_composition);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->__pyx_base._total_composition));
    __pyx_v_self->__pyx_base._total_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":445
 * 
 *     cpdef CComposition total_composition(self):
 *         if self._total_composition is None:             # <<<<<<<<<<<<<<
 *             self._total_composition = _total_composition(self)
 *         return self._total_composition
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":447
 *         if self._total_composition is None:
 *             self._total_composition = _total_composition(self)
 *         return self._total_composition             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._total_composition));
  __pyx_r = __pyx_v_self->__pyx_base._total_composition;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":444
 *         return self._peptide_base_composition
 * 
 *     cpdef CComposition total_composition(self):             # <<<<<<<<<<<<<<
 *         if self._total_composition is None:
 *             self._total_composition = _total_composition(self)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.total_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_19total_composition(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_18total_composition[] = "_PeptideSequenceCore.total_composition(self) -> CComposition";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_19total_composition(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("total_composition (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_18total_composition(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_18total_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("total_composition", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_total_composition(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 444, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.total_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":449
 *         return self._total_composition
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.sequence)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_21__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_21__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_20__iter__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_20__iter__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iter__", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":450
 * 
 *     def __iter__(self):
 *         return iter(self.sequence)             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, index):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_v_self->__pyx_base.sequence;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyObject_GetIter(__pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 450, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":449
 *         return self._total_composition
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         return iter(self.sequence)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":452
 *         return iter(self.sequence)
 * 
 *     def __getitem__(self, index):             # <<<<<<<<<<<<<<
 *         if isinstance(index, slice):
 *             return self.sequence[index]
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_23__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_23__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_22__getitem__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self), ((PyObject *)__pyx_v_index));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_22__getitem__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_index) {
  PyObject *__pyx_v_c_index = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_sub = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  long __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":453
 * 
 *     def __getitem__(self, index):
 *         if isinstance(index, slice):             # <<<<<<<<<<<<<<
 *             return self.sequence[index]
 *         cdef c_index = PyInt_AsLong(index)
 */
  __pyx_t_1 = PySlice_Check(__pyx_v_index); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":454
 *     def __getitem__(self, index):
 *         if isinstance(index, slice):
 *             return self.sequence[index]             # <<<<<<<<<<<<<<
 *         cdef c_index = PyInt_AsLong(index)
 *         if c_index < 0:
 */
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_self->__pyx_base.sequence == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 454, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_v_self->__pyx_base.sequence, __pyx_v_index); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 454, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":453
 * 
 *     def __getitem__(self, index):
 *         if isinstance(index, slice):             # <<<<<<<<<<<<<<
 *             return self.sequence[index]
 *         cdef c_index = PyInt_AsLong(index)
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":455
 *         if isinstance(index, slice):
 *             return self.sequence[index]
 *         cdef c_index = PyInt_AsLong(index)             # <<<<<<<<<<<<<<
 *         if c_index < 0:
 *             return self.sequence[index]
 */
  __pyx_t_4 = PyInt_AsLong(__pyx_v_index); if (unlikely(__pyx_t_4 == ((long)-1L) && PyErr_Occurred())) __PYX_ERR(0, 455, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyInt_From_long(__pyx_t_4); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 455, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_c_index = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":456
 *             return self.sequence[index]
 *         cdef c_index = PyInt_AsLong(index)
 *         if c_index < 0:             # <<<<<<<<<<<<<<
 *             return self.sequence[index]
 *         sub = self.get(c_index)
 */
  __pyx_t_3 = PyObject_RichCompare(__pyx_v_c_index, __pyx_int_0, Py_LT); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 456, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 456, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":457
 *         cdef c_index = PyInt_AsLong(index)
 *         if c_index < 0:
 *             return self.sequence[index]             # <<<<<<<<<<<<<<
 *         sub = self.get(c_index)
 *         return sub
 */
    __Pyx_XDECREF(__pyx_r);
    if (unlikely(__pyx_v_self->__pyx_base.sequence == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(0, 457, __pyx_L1_error)
    }
    __pyx_t_3 = __Pyx_PyObject_GetItem(__pyx_v_self->__pyx_base.sequence, __pyx_v_index); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 457, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":456
 *             return self.sequence[index]
 *         cdef c_index = PyInt_AsLong(index)
 *         if c_index < 0:             # <<<<<<<<<<<<<<
 *             return self.sequence[index]
 *         sub = self.get(c_index)
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":458
 *         if c_index < 0:
 *             return self.sequence[index]
 *         sub = self.get(c_index)             # <<<<<<<<<<<<<<
 *         return sub
 * 
 */
  __pyx_t_5 = PyInt_AsSsize_t(__pyx_v_c_index); if (unlikely((__pyx_t_5 == (Py_ssize_t)-1) && PyErr_Occurred())) __PYX_ERR(0, 458, __pyx_L1_error)
  __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.get(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), __pyx_t_5)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 458, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v_sub = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":459
 *             return self.sequence[index]
 *         sub = self.get(c_index)
 *         return sub             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, index, value):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_sub));
  __pyx_r = ((PyObject *)__pyx_v_sub);
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":452
 *         return iter(self.sequence)
 * 
 *     def __getitem__(self, index):             # <<<<<<<<<<<<<<
 *         if isinstance(index, slice):
 *             return self.sequence[index]
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_c_index);
  __Pyx_XDECREF((PyObject *)__pyx_v_sub);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":461
 *         return sub
 * 
 *     def __setitem__(self, index, value):             # <<<<<<<<<<<<<<
 *         self._invalidate()
 *         self.sequence[index] = value
 */

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_25__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_25__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_24__setitem__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self), ((PyObject *)__pyx_v_index), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_24__setitem__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_index, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":462
 * 
 *     def __setitem__(self, index, value):
 *         self._invalidate()             # <<<<<<<<<<<<<<
 *         self.sequence[index] = value
 * 
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._invalidate(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 462, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":463
 *     def __setitem__(self, index, value):
 *         self._invalidate()
 *         self.sequence[index] = value             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  if (unlikely(__pyx_v_self->__pyx_base.sequence == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(0, 463, __pyx_L1_error)
  }
  if (unlikely(PyObject_SetItem(__pyx_v_self->__pyx_base.sequence, __pyx_v_index, __pyx_v_value) < 0)) __PYX_ERR(0, 463, __pyx_L1_error)

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":461
 *         return sub
 * 
 *     def __setitem__(self, index, value):             # <<<<<<<<<<<<<<
 *         self._invalidate()
 *         self.sequence[index] = value
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":465
 *         self.sequence[index] = value
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return PyList_Size(self.sequence)
 * 
 */

/* Python wrapper */
static Py_ssize_t __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_27__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_27__len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_26__len__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_26__len__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":466
 * 
 *     def __len__(self):
 *         return PyList_Size(self.sequence)             # <<<<<<<<<<<<<<
 * 
 *     cdef size_t get_size(self):
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.sequence;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyList_Size(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1L))) __PYX_ERR(0, 466, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":465
 *         self.sequence[index] = value
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return PyList_Size(self.sequence)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.__len__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":468
 *         return PyList_Size(self.sequence)
 * 
 *     cdef size_t get_size(self):             # <<<<<<<<<<<<<<
 *         return PyList_Size(self.sequence)
 * 
 */

static size_t __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_size(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  size_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_size", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":469
 * 
 *     cdef size_t get_size(self):
 *         return PyList_Size(self.sequence)             # <<<<<<<<<<<<<<
 * 
 *     # equality methods
 */
  __pyx_t_1 = __pyx_v_self->__pyx_base.sequence;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyList_Size(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1L))) __PYX_ERR(0, 469, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":468
 *         return PyList_Size(self.sequence)
 * 
 *     cdef size_t get_size(self):             # <<<<<<<<<<<<<<
 *         return PyList_Size(self.sequence)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.get_size", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":473
 *     # equality methods
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         if isinstance(other, _PeptideSequenceCore):
 *             return self.full_structure_equality(<_PeptideSequenceCore>other)
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_29__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_29__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__eq__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_28__eq__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_28__eq__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__eq__", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":474
 * 
 *     def __eq__(self, other):
 *         if isinstance(other, _PeptideSequenceCore):             # <<<<<<<<<<<<<<
 *             return self.full_structure_equality(<_PeptideSequenceCore>other)
 *         else:
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_other, __pyx_ptype_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":475
 *     def __eq__(self, other):
 *         if isinstance(other, _PeptideSequenceCore):
 *             return self.full_structure_equality(<_PeptideSequenceCore>other)             # <<<<<<<<<<<<<<
 *         else:
 *             return str(self) == str(other)
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyBool_FromLong(((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->full_structure_equality(__pyx_v_self, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_other), 0)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 475, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":474
 * 
 *     def __eq__(self, other):
 *         if isinstance(other, _PeptideSequenceCore):             # <<<<<<<<<<<<<<
 *             return self.full_structure_equality(<_PeptideSequenceCore>other)
 *         else:
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":477
 *             return self.full_structure_equality(<_PeptideSequenceCore>other)
 *         else:
 *             return str(self) == str(other)             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint base_sequence_equality(self, _PeptideSequenceCore other):
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 477, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_other); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 477, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyObject_RichCompare(__pyx_t_3, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 477, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_r = __pyx_t_5;
    __pyx_t_5 = 0;
    goto __pyx_L0;
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":473
 *     # equality methods
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         if isinstance(other, _PeptideSequenceCore):
 *             return self.full_structure_equality(<_PeptideSequenceCore>other)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":479
 *             return str(self) == str(other)
 * 
 *     cpdef bint base_sequence_equality(self, _PeptideSequenceCore other):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t n, m, i
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_31base_sequence_equality(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_base_sequence_equality(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_other, int __pyx_skip_dispatch) {
  size_t __pyx_v_n;
  size_t __pyx_v_m;
  size_t __pyx_v_i;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  size_t __pyx_t_6;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("base_sequence_equality", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_base_sequence_equality); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 479, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_31base_sequence_equality)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_other)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_other));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 479, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 479, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":482
 *         cdef:
 *             size_t n, m, i
 *         n = self.get_size()             # <<<<<<<<<<<<<<
 *         m = other.get_size()
 *         if n != m:
 */
  __pyx_v_n = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_size(__pyx_v_self);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":483
 *             size_t n, m, i
 *         n = self.get_size()
 *         m = other.get_size()             # <<<<<<<<<<<<<<
 *         if n != m:
 *             return False
 */
  __pyx_v_m = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_other->__pyx_base.__pyx_vtab)->get_size(__pyx_v_other);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":484
 *         n = self.get_size()
 *         m = other.get_size()
 *         if n != m:             # <<<<<<<<<<<<<<
 *             return False
 *         for i in range(n):
 */
  __pyx_t_5 = ((__pyx_v_n != __pyx_v_m) != 0);
  if (__pyx_t_5) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":485
 *         m = other.get_size()
 *         if n != m:
 *             return False             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             if self.get(i).amino_acid != other.get(i).amino_acid:
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":484
 *         n = self.get_size()
 *         m = other.get_size()
 *         if n != m:             # <<<<<<<<<<<<<<
 *             return False
 *         for i in range(n):
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":486
 *         if n != m:
 *             return False
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             if self.get(i).amino_acid != other.get(i).amino_acid:
 *                 return False
 */
  __pyx_t_6 = __pyx_v_n;
  __pyx_t_7 = __pyx_t_6;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
    __pyx_v_i = __pyx_t_8;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":487
 *             return False
 *         for i in range(n):
 *             if self.get(i).amino_acid != other.get(i).amino_acid:             # <<<<<<<<<<<<<<
 *                 return False
 *         return True
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.get(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), __pyx_v_i)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 487, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_other->__pyx_base.__pyx_vtab)->__pyx_base.get(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_other), __pyx_v_i)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 487, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyObject_RichCompare(((PyObject *)((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_t_1)->amino_acid), ((PyObject *)((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_t_2)->amino_acid), Py_NE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 487, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 487, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_5) {

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":488
 *         for i in range(n):
 *             if self.get(i).amino_acid != other.get(i).amino_acid:
 *                 return False             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
      __pyx_r = 0;
      goto __pyx_L0;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":487
 *             return False
 *         for i in range(n):
 *             if self.get(i).amino_acid != other.get(i).amino_acid:             # <<<<<<<<<<<<<<
 *                 return False
 *         return True
 */
    }
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":489
 *             if self.get(i).amino_acid != other.get(i).amino_acid:
 *                 return False
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint modified_sequence_equality(self, _PeptideSequenceCore other):
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":479
 *             return str(self) == str(other)
 * 
 *     cpdef bint base_sequence_equality(self, _PeptideSequenceCore other):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t n, m, i
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.base_sequence_equality", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_31base_sequence_equality(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_30base_sequence_equality[] = "_PeptideSequenceCore.base_sequence_equality(self, _PeptideSequenceCore other) -> bool";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_31base_sequence_equality(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("base_sequence_equality (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore, 1, "other", 0))) __PYX_ERR(0, 479, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_30base_sequence_equality(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self), ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_other));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_30base_sequence_equality(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("base_sequence_equality", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_base_sequence_equality(__pyx_v_self, __pyx_v_other, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 479, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.base_sequence_equality", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":491
 *         return True
 * 
 *     cpdef bint modified_sequence_equality(self, _PeptideSequenceCore other):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t n, m, i
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_33modified_sequence_equality(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_modified_sequence_equality(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_other, int __pyx_skip_dispatch) {
  size_t __pyx_v_n;
  size_t __pyx_v_m;
  size_t __pyx_v_i;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_a = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_b = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  size_t __pyx_t_6;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("modified_sequence_equality", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_modified_sequence_equality); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 491, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_33modified_sequence_equality)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_other)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_other));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 491, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 491, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":495
 *             size_t n, m, i
 *             SequencePosition a, b
 *         n = self.get_size()             # <<<<<<<<<<<<<<
 *         m = other.get_size()
 *         if n != m:
 */
  __pyx_v_n = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_size(__pyx_v_self);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":496
 *             SequencePosition a, b
 *         n = self.get_size()
 *         m = other.get_size()             # <<<<<<<<<<<<<<
 *         if n != m:
 *             return False
 */
  __pyx_v_m = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_other->__pyx_base.__pyx_vtab)->get_size(__pyx_v_other);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":497
 *         n = self.get_size()
 *         m = other.get_size()
 *         if n != m:             # <<<<<<<<<<<<<<
 *             return False
 *         for i in range(n):
 */
  __pyx_t_5 = ((__pyx_v_n != __pyx_v_m) != 0);
  if (__pyx_t_5) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":498
 *         m = other.get_size()
 *         if n != m:
 *             return False             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             a = self.get(i)
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":497
 *         n = self.get_size()
 *         m = other.get_size()
 *         if n != m:             # <<<<<<<<<<<<<<
 *             return False
 *         for i in range(n):
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":499
 *         if n != m:
 *             return False
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             a = self.get(i)
 *             b = other.get(i)
 */
  __pyx_t_6 = __pyx_v_n;
  __pyx_t_7 = __pyx_t_6;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
    __pyx_v_i = __pyx_t_8;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":500
 *             return False
 *         for i in range(n):
 *             a = self.get(i)             # <<<<<<<<<<<<<<
 *             b = other.get(i)
 *             if a != b:
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.get(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), __pyx_v_i)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 500, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_a, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":501
 *         for i in range(n):
 *             a = self.get(i)
 *             b = other.get(i)             # <<<<<<<<<<<<<<
 *             if a != b:
 *                 return False
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_other->__pyx_base.__pyx_vtab)->__pyx_base.get(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_other), __pyx_v_i)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 501, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_b, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":502
 *             a = self.get(i)
 *             b = other.get(i)
 *             if a != b:             # <<<<<<<<<<<<<<
 *                 return False
 *         if self.get_n_term() != other.get_n_term():
 */
    __pyx_t_1 = PyObject_RichCompare(((PyObject *)__pyx_v_a), ((PyObject *)__pyx_v_b), Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 502, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 502, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_5) {

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":503
 *             b = other.get(i)
 *             if a != b:
 *                 return False             # <<<<<<<<<<<<<<
 *         if self.get_n_term() != other.get_n_term():
 *             return  False
 */
      __pyx_r = 0;
      goto __pyx_L0;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":502
 *             a = self.get(i)
 *             b = other.get(i)
 *             if a != b:             # <<<<<<<<<<<<<<
 *                 return False
 *         if self.get_n_term() != other.get_n_term():
 */
    }
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":504
 *             if a != b:
 *                 return False
 *         if self.get_n_term() != other.get_n_term():             # <<<<<<<<<<<<<<
 *             return  False
 *         if self.get_c_term() != other.get_c_term():
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_n_term(__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 504, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_other->__pyx_base.__pyx_vtab)->get_n_term(__pyx_v_other)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 504, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_NE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 504, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 504, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (__pyx_t_5) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":505
 *                 return False
 *         if self.get_n_term() != other.get_n_term():
 *             return  False             # <<<<<<<<<<<<<<
 *         if self.get_c_term() != other.get_c_term():
 *             return False
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":504
 *             if a != b:
 *                 return False
 *         if self.get_n_term() != other.get_n_term():             # <<<<<<<<<<<<<<
 *             return  False
 *         if self.get_c_term() != other.get_c_term():
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":506
 *         if self.get_n_term() != other.get_n_term():
 *             return  False
 *         if self.get_c_term() != other.get_c_term():             # <<<<<<<<<<<<<<
 *             return False
 *         return True
 */
  __pyx_t_3 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_c_term(__pyx_v_self)); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_other->__pyx_base.__pyx_vtab)->get_c_term(__pyx_v_other)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = PyObject_RichCompare(__pyx_t_3, __pyx_t_2, Py_NE); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 506, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_5) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":507
 *             return  False
 *         if self.get_c_term() != other.get_c_term():
 *             return False             # <<<<<<<<<<<<<<
 *         return True
 * 
 */
    __pyx_r = 0;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":506
 *         if self.get_n_term() != other.get_n_term():
 *             return  False
 *         if self.get_c_term() != other.get_c_term():             # <<<<<<<<<<<<<<
 *             return False
 *         return True
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":508
 *         if self.get_c_term() != other.get_c_term():
 *             return False
 *         return True             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint full_structure_equality(self, _PeptideSequenceCore other):
 */
  __pyx_r = 1;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":491
 *         return True
 * 
 *     cpdef bint modified_sequence_equality(self, _PeptideSequenceCore other):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t n, m, i
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.modified_sequence_equality", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_a);
  __Pyx_XDECREF((PyObject *)__pyx_v_b);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_33modified_sequence_equality(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_32modified_sequence_equality[] = "_PeptideSequenceCore.modified_sequence_equality(self, _PeptideSequenceCore other) -> bool";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_33modified_sequence_equality(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("modified_sequence_equality (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore, 1, "other", 0))) __PYX_ERR(0, 491, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_32modified_sequence_equality(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self), ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_other));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_32modified_sequence_equality(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("modified_sequence_equality", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_modified_sequence_equality(__pyx_v_self, __pyx_v_other, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 491, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.modified_sequence_equality", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":510
 *         return True
 * 
 *     cpdef bint full_structure_equality(self, _PeptideSequenceCore other):             # <<<<<<<<<<<<<<
 *         sequences_equal = self.modified_sequence_equality(other)
 *         if sequences_equal:
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_35full_structure_equality(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_full_structure_equality(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_other, int __pyx_skip_dispatch) {
  int __pyx_v_sequences_equal;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("full_structure_equality", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_full_structure_equality); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 510, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_35full_structure_equality)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_other)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_other));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 510, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 510, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":511
 * 
 *     cpdef bint full_structure_equality(self, _PeptideSequenceCore other):
 *         sequences_equal = self.modified_sequence_equality(other)             # <<<<<<<<<<<<<<
 *         if sequences_equal:
 *             return self.glycan == other.glycan
 */
  __pyx_v_sequences_equal = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->modified_sequence_equality(__pyx_v_self, __pyx_v_other, 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":512
 *     cpdef bint full_structure_equality(self, _PeptideSequenceCore other):
 *         sequences_equal = self.modified_sequence_equality(other)
 *         if sequences_equal:             # <<<<<<<<<<<<<<
 *             return self.glycan == other.glycan
 *         else:
 */
  __pyx_t_5 = (__pyx_v_sequences_equal != 0);
  if (__pyx_t_5) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":513
 *         sequences_equal = self.modified_sequence_equality(other)
 *         if sequences_equal:
 *             return self.glycan == other.glycan             # <<<<<<<<<<<<<<
 *         else:
 *             return False
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_glycan); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 513, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_other), __pyx_n_s_glycan); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 513, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 513, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 513, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_r = __pyx_t_5;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":512
 *     cpdef bint full_structure_equality(self, _PeptideSequenceCore other):
 *         sequences_equal = self.modified_sequence_equality(other)
 *         if sequences_equal:             # <<<<<<<<<<<<<<
 *             return self.glycan == other.glycan
 *         else:
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":515
 *             return self.glycan == other.glycan
 *         else:
 *             return False             # <<<<<<<<<<<<<<
 * 
 *     def __ne__(self, other):
 */
  /*else*/ {
    __pyx_r = 0;
    goto __pyx_L0;
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":510
 *         return True
 * 
 *     cpdef bint full_structure_equality(self, _PeptideSequenceCore other):             # <<<<<<<<<<<<<<
 *         sequences_equal = self.modified_sequence_equality(other)
 *         if sequences_equal:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.full_structure_equality", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_35full_structure_equality(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_34full_structure_equality[] = "_PeptideSequenceCore.full_structure_equality(self, _PeptideSequenceCore other) -> bool";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_35full_structure_equality(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("full_structure_equality (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_other), __pyx_ptype_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore, 1, "other", 0))) __PYX_ERR(0, 510, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_34full_structure_equality(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self), ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_other));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_34full_structure_equality(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("full_structure_equality", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_full_structure_equality(__pyx_v_self, __pyx_v_other, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 510, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.full_structure_equality", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":517
 *             return False
 * 
 *     def __ne__(self, other):             # <<<<<<<<<<<<<<
 *         return str(self) != str(other)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_37__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_37__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__ne__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_36__ne__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_36__ne__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__ne__", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":518
 * 
 *     def __ne__(self, other):
 *         return str(self) != str(other)             # <<<<<<<<<<<<<<
 * 
 *     def __hash__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), ((PyObject *)__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 518, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_other); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 518, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyObject_RichCompare(__pyx_t_1, __pyx_t_2, Py_NE); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 518, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":517
 *             return False
 * 
 *     def __ne__(self, other):             # <<<<<<<<<<<<<<
 *         return str(self) != str(other)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.__ne__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":520
 *         return str(self) != str(other)
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         return hash(self.get_sequence())
 * 
 */

/* Python wrapper */
static Py_hash_t __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_39__hash__(PyObject *__pyx_v_self); /*proto*/
static Py_hash_t __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_39__hash__(PyObject *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__hash__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_38__hash__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_hash_t __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_38__hash__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_hash_t __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__hash__", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":521
 * 
 *     def __hash__(self):
 *         return hash(self.get_sequence())             # <<<<<<<<<<<<<<
 * 
 *     cpdef basestring get_sequence(self, bint include_glycan=True, bint include_termini=True, str implicit_n_term=None, str implicit_c_term=None):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_sequence(__pyx_v_self, 0, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 521, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_Hash(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_hash_t)-1))) __PYX_ERR(0, 521, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":520
 *         return str(self) != str(other)
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         return hash(self.get_sequence())
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.__hash__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  if (unlikely(__pyx_r == -1) && !PyErr_Occurred()) __pyx_r = -2;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":523
 *         return hash(self.get_sequence())
 * 
 *     cpdef basestring get_sequence(self, bint include_glycan=True, bint include_termini=True, str implicit_n_term=None, str implicit_c_term=None):             # <<<<<<<<<<<<<<
 *         """
 *         Generate human readable sequence string. Called by :meth:`__str__`
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_41get_sequence(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_sequence(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_sequence *__pyx_optional_args) {
  int __pyx_v_include_glycan = ((int)1);
  int __pyx_v_include_termini = ((int)1);
  PyObject *__pyx_v_implicit_n_term = ((PyObject*)Py_None);
  PyObject *__pyx_v_implicit_c_term = ((PyObject*)Py_None);
  PyObject *__pyx_v_seq_list = 0;
  PyObject *__pyx_v_mod_strs = 0;
  size_t __pyx_v_i;
  size_t __pyx_v_j;
  size_t __pyx_v_n;
  size_t __pyx_v_m;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_position = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_mod = 0;
  PyObject *__pyx_v_mod_str = 0;
  PyObject *__pyx_v_n_term = 0;
  PyObject *__pyx_v_c_term = 0;
  PyObject *__pyx_v_rep = 0;
  int __pyx_v_needs_terminals;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_terminal = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_implicit_terminal = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  int __pyx_t_10;
  size_t __pyx_t_11;
  size_t __pyx_t_12;
  size_t __pyx_t_13;
  Py_ssize_t __pyx_t_14;
  PyObject *__pyx_t_15;
  size_t __pyx_t_16;
  size_t __pyx_t_17;
  size_t __pyx_t_18;
  int __pyx_t_19;
  int __pyx_t_20;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_sequence", 0);
  if (__pyx_optional_args) {
    if (__pyx_optional_args->__pyx_n > 0) {
      __pyx_v_include_glycan = __pyx_optional_args->include_glycan;
      if (__pyx_optional_args->__pyx_n > 1) {
        __pyx_v_include_termini = __pyx_optional_args->include_termini;
        if (__pyx_optional_args->__pyx_n > 2) {
          __pyx_v_implicit_n_term = __pyx_optional_args->implicit_n_term;
          if (__pyx_optional_args->__pyx_n > 3) {
            __pyx_v_implicit_c_term = __pyx_optional_args->implicit_c_term;
          }
        }
      }
    }
  }
  __Pyx_INCREF(__pyx_v_implicit_n_term);
  __Pyx_INCREF(__pyx_v_implicit_c_term);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_get_sequence); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 523, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_41get_sequence)) {
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_3 = __Pyx_PyBool_FromLong(__pyx_v_include_glycan); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 523, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_3);
        __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_v_include_termini); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 523, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_5 = __pyx_t_1; __pyx_t_6 = NULL;
        __pyx_t_7 = 0;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_5))) {
          __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_5);
          if (likely(__pyx_t_6)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_5);
            __Pyx_INCREF(__pyx_t_6);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_5, function);
            __pyx_t_7 = 1;
          }
        }
        #if CYTHON_FAST_PYCALL
        if (PyFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[5] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_v_implicit_n_term, __pyx_v_implicit_c_term};
          __pyx_t_2 = __Pyx_PyFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 4+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 523, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        #if CYTHON_FAST_PYCCALL
        if (__Pyx_PyFastCFunction_Check(__pyx_t_5)) {
          PyObject *__pyx_temp[5] = {__pyx_t_6, __pyx_t_3, __pyx_t_4, __pyx_v_implicit_n_term, __pyx_v_implicit_c_term};
          __pyx_t_2 = __Pyx_PyCFunction_FastCall(__pyx_t_5, __pyx_temp+1-__pyx_t_7, 4+__pyx_t_7); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 523, __pyx_L1_error)
          __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
        } else
        #endif
        {
          __pyx_t_8 = PyTuple_New(4+__pyx_t_7); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 523, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_8);
          if (__pyx_t_6) {
            __Pyx_GIVEREF(__pyx_t_6); PyTuple_SET_ITEM(__pyx_t_8, 0, __pyx_t_6); __pyx_t_6 = NULL;
          }
          __Pyx_GIVEREF(__pyx_t_3);
          PyTuple_SET_ITEM(__pyx_t_8, 0+__pyx_t_7, __pyx_t_3);
          __Pyx_GIVEREF(__pyx_t_4);
          PyTuple_SET_ITEM(__pyx_t_8, 1+__pyx_t_7, __pyx_t_4);
          __Pyx_INCREF(__pyx_v_implicit_n_term);
          __Pyx_GIVEREF(__pyx_v_implicit_n_term);
          PyTuple_SET_ITEM(__pyx_t_8, 2+__pyx_t_7, __pyx_v_implicit_n_term);
          __Pyx_INCREF(__pyx_v_implicit_c_term);
          __Pyx_GIVEREF(__pyx_v_implicit_c_term);
          PyTuple_SET_ITEM(__pyx_t_8, 3+__pyx_t_7, __pyx_v_implicit_c_term);
          __pyx_t_3 = 0;
          __pyx_t_4 = 0;
          __pyx_t_2 = __Pyx_PyObject_Call(__pyx_t_5, __pyx_t_8, NULL); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 523, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        }
        __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
        if (!(likely(__Pyx_PyBaseString_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", (PY_MAJOR_VERSION < 3 ? "basestring" : "str"), Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 523, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":540
 *         str
 *         """
 *         if implicit_n_term is None:             # <<<<<<<<<<<<<<
 *             implicit_n_term = structure_constants.N_TERM_DEFAULT
 *         if implicit_c_term is None:
 */
  __pyx_t_9 = (__pyx_v_implicit_n_term == ((PyObject*)Py_None));
  __pyx_t_10 = (__pyx_t_9 != 0);
  if (__pyx_t_10) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":541
 *         """
 *         if implicit_n_term is None:
 *             implicit_n_term = structure_constants.N_TERM_DEFAULT             # <<<<<<<<<<<<<<
 *         if implicit_c_term is None:
 *             implicit_c_term = structure_constants.C_TERM_DEFAULT
 */
    __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_structure_constants->N_TERM_DEFAULT;
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_implicit_n_term, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":540
 *         str
 *         """
 *         if implicit_n_term is None:             # <<<<<<<<<<<<<<
 *             implicit_n_term = structure_constants.N_TERM_DEFAULT
 *         if implicit_c_term is None:
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":542
 *         if implicit_n_term is None:
 *             implicit_n_term = structure_constants.N_TERM_DEFAULT
 *         if implicit_c_term is None:             # <<<<<<<<<<<<<<
 *             implicit_c_term = structure_constants.C_TERM_DEFAULT
 * 
 */
  __pyx_t_10 = (__pyx_v_implicit_c_term == ((PyObject*)Py_None));
  __pyx_t_9 = (__pyx_t_10 != 0);
  if (__pyx_t_9) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":543
 *             implicit_n_term = structure_constants.N_TERM_DEFAULT
 *         if implicit_c_term is None:
 *             implicit_c_term = structure_constants.C_TERM_DEFAULT             # <<<<<<<<<<<<<<
 * 
 *         cdef:
 */
    __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_structure_constants->C_TERM_DEFAULT;
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_implicit_c_term, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":542
 *         if implicit_n_term is None:
 *             implicit_n_term = structure_constants.N_TERM_DEFAULT
 *         if implicit_c_term is None:             # <<<<<<<<<<<<<<
 *             implicit_c_term = structure_constants.C_TERM_DEFAULT
 * 
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":555
 *             ModificationBase implicit_terminal
 * 
 *         seq_list = []             # <<<<<<<<<<<<<<
 *         n = self.get_size()
 *         for i in range(n):
 */
  __pyx_t_1 = PyList_New(0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 555, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_seq_list = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":556
 * 
 *         seq_list = []
 *         n = self.get_size()             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             position = self.get(i)
 */
  __pyx_v_n = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_size(__pyx_v_self);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":557
 *         seq_list = []
 *         n = self.get_size()
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             position = self.get(i)
 *             m = PyList_Size(position.modifications)
 */
  __pyx_t_11 = __pyx_v_n;
  __pyx_t_12 = __pyx_t_11;
  for (__pyx_t_13 = 0; __pyx_t_13 < __pyx_t_12; __pyx_t_13+=1) {
    __pyx_v_i = __pyx_t_13;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":558
 *         n = self.get_size()
 *         for i in range(n):
 *             position = self.get(i)             # <<<<<<<<<<<<<<
 *             m = PyList_Size(position.modifications)
 *             if m > 0:
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.get(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), __pyx_v_i)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 558, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_position, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":559
 *         for i in range(n):
 *             position = self.get(i)
 *             m = PyList_Size(position.modifications)             # <<<<<<<<<<<<<<
 *             if m > 0:
 *                 if m == 1:
 */
    __pyx_t_1 = __pyx_v_position->modifications;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_14 = PyList_Size(__pyx_t_1); if (unlikely(__pyx_t_14 == ((Py_ssize_t)-1L))) __PYX_ERR(0, 559, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_v_m = __pyx_t_14;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":560
 *             position = self.get(i)
 *             m = PyList_Size(position.modifications)
 *             if m > 0:             # <<<<<<<<<<<<<<
 *                 if m == 1:
 *                     mod = <ModificationBase>PyList_GetItem(position.modifications, 0)
 */
    __pyx_t_9 = ((__pyx_v_m > 0) != 0);
    if (__pyx_t_9) {

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":561
 *             m = PyList_Size(position.modifications)
 *             if m > 0:
 *                 if m == 1:             # <<<<<<<<<<<<<<
 *                     mod = <ModificationBase>PyList_GetItem(position.modifications, 0)
 *                     mod_str = "(%s)" % mod.serialize()
 */
      __pyx_t_9 = ((__pyx_v_m == 1) != 0);
      if (__pyx_t_9) {

        /* "glycopeptidepy/_c/structure/sequence_methods.pyx":562
 *             if m > 0:
 *                 if m == 1:
 *                     mod = <ModificationBase>PyList_GetItem(position.modifications, 0)             # <<<<<<<<<<<<<<
 *                     mod_str = "(%s)" % mod.serialize()
 *                 else:
 */
        __pyx_t_1 = __pyx_v_position->modifications;
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_15 = PyList_GetItem(__pyx_t_1, 0); if (unlikely(__pyx_t_15 == ((PyObject *)NULL))) __PYX_ERR(0, 562, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __pyx_t_1 = ((PyObject *)__pyx_t_15);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_XDECREF_SET(__pyx_v_mod, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_t_1));
        __pyx_t_1 = 0;

        /* "glycopeptidepy/_c/structure/sequence_methods.pyx":563
 *                 if m == 1:
 *                     mod = <ModificationBase>PyList_GetItem(position.modifications, 0)
 *                     mod_str = "(%s)" % mod.serialize()             # <<<<<<<<<<<<<<
 *                 else:
 *                     mod_strs = []
 */
        __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_mod->__pyx_vtab)->serialize(__pyx_v_mod, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 563, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_1);
        __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_s, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 563, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        __Pyx_XDECREF_SET(__pyx_v_mod_str, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "glycopeptidepy/_c/structure/sequence_methods.pyx":561
 *             m = PyList_Size(position.modifications)
 *             if m > 0:
 *                 if m == 1:             # <<<<<<<<<<<<<<
 *                     mod = <ModificationBase>PyList_GetItem(position.modifications, 0)
 *                     mod_str = "(%s)" % mod.serialize()
 */
        goto __pyx_L8;
      }

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":565
 *                     mod_str = "(%s)" % mod.serialize()
 *                 else:
 *                     mod_strs = []             # <<<<<<<<<<<<<<
 *                     for j in range(m):
 *                         mod = <ModificationBase>PyList_GetItem(position.modifications, j)
 */
      /*else*/ {
        __pyx_t_2 = PyList_New(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 565, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_mod_strs, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "glycopeptidepy/_c/structure/sequence_methods.pyx":566
 *                 else:
 *                     mod_strs = []
 *                     for j in range(m):             # <<<<<<<<<<<<<<
 *                         mod = <ModificationBase>PyList_GetItem(position.modifications, j)
 *                         mod_strs.append(mod.serialize())
 */
        __pyx_t_16 = __pyx_v_m;
        __pyx_t_17 = __pyx_t_16;
        for (__pyx_t_18 = 0; __pyx_t_18 < __pyx_t_17; __pyx_t_18+=1) {
          __pyx_v_j = __pyx_t_18;

          /* "glycopeptidepy/_c/structure/sequence_methods.pyx":567
 *                     mod_strs = []
 *                     for j in range(m):
 *                         mod = <ModificationBase>PyList_GetItem(position.modifications, j)             # <<<<<<<<<<<<<<
 *                         mod_strs.append(mod.serialize())
 *                     mod_str = '|'.join(mod_strs)
 */
          __pyx_t_2 = __pyx_v_position->modifications;
          __Pyx_INCREF(__pyx_t_2);
          __pyx_t_15 = PyList_GetItem(__pyx_t_2, __pyx_v_j); if (unlikely(__pyx_t_15 == ((PyObject *)NULL))) __PYX_ERR(0, 567, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
          __pyx_t_2 = ((PyObject *)__pyx_t_15);
          __Pyx_INCREF(__pyx_t_2);
          __Pyx_XDECREF_SET(__pyx_v_mod, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_t_2));
          __pyx_t_2 = 0;

          /* "glycopeptidepy/_c/structure/sequence_methods.pyx":568
 *                     for j in range(m):
 *                         mod = <ModificationBase>PyList_GetItem(position.modifications, j)
 *                         mod_strs.append(mod.serialize())             # <<<<<<<<<<<<<<
 *                     mod_str = '|'.join(mod_strs)
 *                     mod_str = "(%s)" % mod_str
 */
          __pyx_t_2 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_mod->__pyx_vtab)->serialize(__pyx_v_mod, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 568, __pyx_L1_error)
          __Pyx_GOTREF(__pyx_t_2);
          __pyx_t_19 = __Pyx_PyList_Append(__pyx_v_mod_strs, __pyx_t_2); if (unlikely(__pyx_t_19 == ((int)-1))) __PYX_ERR(0, 568, __pyx_L1_error)
          __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        }

        /* "glycopeptidepy/_c/structure/sequence_methods.pyx":569
 *                         mod = <ModificationBase>PyList_GetItem(position.modifications, j)
 *                         mod_strs.append(mod.serialize())
 *                     mod_str = '|'.join(mod_strs)             # <<<<<<<<<<<<<<
 *                     mod_str = "(%s)" % mod_str
 *                 seq_list.append(position.amino_acid.symbol + mod_str)
 */
        __pyx_t_2 = __Pyx_PyString_Join(__pyx_kp_s__2, __pyx_v_mod_strs); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 569, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_XDECREF_SET(__pyx_v_mod_str, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;

        /* "glycopeptidepy/_c/structure/sequence_methods.pyx":570
 *                         mod_strs.append(mod.serialize())
 *                     mod_str = '|'.join(mod_strs)
 *                     mod_str = "(%s)" % mod_str             # <<<<<<<<<<<<<<
 *                 seq_list.append(position.amino_acid.symbol + mod_str)
 *             else:
 */
        __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_s, __pyx_v_mod_str); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 570, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF_SET(__pyx_v_mod_str, ((PyObject*)__pyx_t_2));
        __pyx_t_2 = 0;
      }
      __pyx_L8:;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":571
 *                     mod_str = '|'.join(mod_strs)
 *                     mod_str = "(%s)" % mod_str
 *                 seq_list.append(position.amino_acid.symbol + mod_str)             # <<<<<<<<<<<<<<
 *             else:
 *                 seq_list.append(position.amino_acid.symbol)
 */
      __pyx_t_2 = PyNumber_Add(__pyx_v_position->amino_acid->symbol, __pyx_v_mod_str); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 571, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_19 = __Pyx_PyList_Append(__pyx_v_seq_list, __pyx_t_2); if (unlikely(__pyx_t_19 == ((int)-1))) __PYX_ERR(0, 571, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":560
 *             position = self.get(i)
 *             m = PyList_Size(position.modifications)
 *             if m > 0:             # <<<<<<<<<<<<<<
 *                 if m == 1:
 *                     mod = <ModificationBase>PyList_GetItem(position.modifications, 0)
 */
      goto __pyx_L7;
    }

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":573
 *                 seq_list.append(position.amino_acid.symbol + mod_str)
 *             else:
 *                 seq_list.append(position.amino_acid.symbol)             # <<<<<<<<<<<<<<
 * 
 *         rep = ''.join(seq_list)
 */
    /*else*/ {
      __pyx_t_2 = __pyx_v_position->amino_acid->symbol;
      __Pyx_INCREF(__pyx_t_2);
      __pyx_t_19 = __Pyx_PyList_Append(__pyx_v_seq_list, __pyx_t_2); if (unlikely(__pyx_t_19 == ((int)-1))) __PYX_ERR(0, 573, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    }
    __pyx_L7:;
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":575
 *                 seq_list.append(position.amino_acid.symbol)
 * 
 *         rep = ''.join(seq_list)             # <<<<<<<<<<<<<<
 *         if include_termini:
 *             needs_terminals = False
 */
  __pyx_t_2 = __Pyx_PyString_Join(__pyx_kp_s_, __pyx_v_seq_list); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 575, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v_rep = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":576
 * 
 *         rep = ''.join(seq_list)
 *         if include_termini:             # <<<<<<<<<<<<<<
 *             needs_terminals = False
 *             # Handle N-terminal
 */
  __pyx_t_9 = (__pyx_v_include_termini != 0);
  if (__pyx_t_9) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":577
 *         rep = ''.join(seq_list)
 *         if include_termini:
 *             needs_terminals = False             # <<<<<<<<<<<<<<
 *             # Handle N-terminal
 *             n_term = ""
 */
    __pyx_v_needs_terminals = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":579
 *             needs_terminals = False
 *             # Handle N-terminal
 *             n_term = ""             # <<<<<<<<<<<<<<
 *             terminal = self.get_n_term()
 *             implicit_terminal = _get_implicit_terminal(implicit_n_term)
 */
    __Pyx_INCREF(__pyx_kp_s_);
    __pyx_v_n_term = ((PyObject*)__pyx_kp_s_);

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":580
 *             # Handle N-terminal
 *             n_term = ""
 *             terminal = self.get_n_term()             # <<<<<<<<<<<<<<
 *             implicit_terminal = _get_implicit_terminal(implicit_n_term)
 *             if terminal is not None and not terminal.equal_to(implicit_terminal):
 */
    __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_n_term(__pyx_v_self)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 580, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_terminal = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":581
 *             n_term = ""
 *             terminal = self.get_n_term()
 *             implicit_terminal = _get_implicit_terminal(implicit_n_term)             # <<<<<<<<<<<<<<
 *             if terminal is not None and not terminal.equal_to(implicit_terminal):
 *                 n_term = "(%s)-" % terminal.serialize()
 */
    __pyx_t_2 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__get_implicit_terminal(((PyObject*)__pyx_v_implicit_n_term))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 581, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_v_implicit_terminal = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_t_2);
    __pyx_t_2 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":582
 *             terminal = self.get_n_term()
 *             implicit_terminal = _get_implicit_terminal(implicit_n_term)
 *             if terminal is not None and not terminal.equal_to(implicit_terminal):             # <<<<<<<<<<<<<<
 *                 n_term = "(%s)-" % terminal.serialize()
 *                 needs_terminals = True
 */
    __pyx_t_10 = (((PyObject *)__pyx_v_terminal) != Py_None);
    __pyx_t_20 = (__pyx_t_10 != 0);
    if (__pyx_t_20) {
    } else {
      __pyx_t_9 = __pyx_t_20;
      goto __pyx_L13_bool_binop_done;
    }
    __pyx_t_20 = ((!(((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_terminal->__pyx_vtab)->equal_to(__pyx_v_terminal, ((PyObject *)__pyx_v_implicit_terminal)) != 0)) != 0);
    __pyx_t_9 = __pyx_t_20;
    __pyx_L13_bool_binop_done:;
    if (__pyx_t_9) {

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":583
 *             implicit_terminal = _get_implicit_terminal(implicit_n_term)
 *             if terminal is not None and not terminal.equal_to(implicit_terminal):
 *                 n_term = "(%s)-" % terminal.serialize()             # <<<<<<<<<<<<<<
 *                 needs_terminals = True
 * 
 */
      __pyx_t_2 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_terminal->__pyx_vtab)->serialize(__pyx_v_terminal, 0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 583, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __pyx_t_1 = __Pyx_PyString_Format(__pyx_kp_s_s_2, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 583, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF_SET(__pyx_v_n_term, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":584
 *             if terminal is not None and not terminal.equal_to(implicit_terminal):
 *                 n_term = "(%s)-" % terminal.serialize()
 *                 needs_terminals = True             # <<<<<<<<<<<<<<
 * 
 *             # Handle C-terminal
 */
      __pyx_v_needs_terminals = 1;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":582
 *             terminal = self.get_n_term()
 *             implicit_terminal = _get_implicit_terminal(implicit_n_term)
 *             if terminal is not None and not terminal.equal_to(implicit_terminal):             # <<<<<<<<<<<<<<
 *                 n_term = "(%s)-" % terminal.serialize()
 *                 needs_terminals = True
 */
    }

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":587
 * 
 *             # Handle C-terminal
 *             c_term = ""             # <<<<<<<<<<<<<<
 *             terminal = self.get_c_term()
 *             implicit_terminal = _get_implicit_terminal(implicit_c_term)
 */
    __Pyx_INCREF(__pyx_kp_s_);
    __pyx_v_c_term = ((PyObject*)__pyx_kp_s_);

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":588
 *             # Handle C-terminal
 *             c_term = ""
 *             terminal = self.get_c_term()             # <<<<<<<<<<<<<<
 *             implicit_terminal = _get_implicit_terminal(implicit_c_term)
 *             if terminal is not None and not terminal.equal_to(implicit_terminal):
 */
    __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_c_term(__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 588, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_terminal, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":589
 *             c_term = ""
 *             terminal = self.get_c_term()
 *             implicit_terminal = _get_implicit_terminal(implicit_c_term)             # <<<<<<<<<<<<<<
 *             if terminal is not None and not terminal.equal_to(implicit_terminal):
 *                 c_term = "-(%s)" % terminal.serialize()
 */
    __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__get_implicit_terminal(((PyObject*)__pyx_v_implicit_c_term))); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 589, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF_SET(__pyx_v_implicit_terminal, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":590
 *             terminal = self.get_c_term()
 *             implicit_terminal = _get_implicit_terminal(implicit_c_term)
 *             if terminal is not None and not terminal.equal_to(implicit_terminal):             # <<<<<<<<<<<<<<
 *                 c_term = "-(%s)" % terminal.serialize()
 *                 needs_terminals = True
 */
    __pyx_t_20 = (((PyObject *)__pyx_v_terminal) != Py_None);
    __pyx_t_10 = (__pyx_t_20 != 0);
    if (__pyx_t_10) {
    } else {
      __pyx_t_9 = __pyx_t_10;
      goto __pyx_L16_bool_binop_done;
    }
    __pyx_t_10 = ((!(((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_terminal->__pyx_vtab)->equal_to(__pyx_v_terminal, ((PyObject *)__pyx_v_implicit_terminal)) != 0)) != 0);
    __pyx_t_9 = __pyx_t_10;
    __pyx_L16_bool_binop_done:;
    if (__pyx_t_9) {

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":591
 *             implicit_terminal = _get_implicit_terminal(implicit_c_term)
 *             if terminal is not None and not terminal.equal_to(implicit_terminal):
 *                 c_term = "-(%s)" % terminal.serialize()             # <<<<<<<<<<<<<<
 *                 needs_terminals = True
 *             if needs_terminals:
 */
      __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_terminal->__pyx_vtab)->serialize(__pyx_v_terminal, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 591, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_s_3, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 591, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __Pyx_DECREF_SET(__pyx_v_c_term, ((PyObject*)__pyx_t_2));
      __pyx_t_2 = 0;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":592
 *             if terminal is not None and not terminal.equal_to(implicit_terminal):
 *                 c_term = "-(%s)" % terminal.serialize()
 *                 needs_terminals = True             # <<<<<<<<<<<<<<
 *             if needs_terminals:
 *                 rep = "".join((n_term, rep, c_term))
 */
      __pyx_v_needs_terminals = 1;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":590
 *             terminal = self.get_c_term()
 *             implicit_terminal = _get_implicit_terminal(implicit_c_term)
 *             if terminal is not None and not terminal.equal_to(implicit_terminal):             # <<<<<<<<<<<<<<
 *                 c_term = "-(%s)" % terminal.serialize()
 *                 needs_terminals = True
 */
    }

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":593
 *                 c_term = "-(%s)" % terminal.serialize()
 *                 needs_terminals = True
 *             if needs_terminals:             # <<<<<<<<<<<<<<
 *                 rep = "".join((n_term, rep, c_term))
 *         if include_glycan:
 */
    __pyx_t_9 = (__pyx_v_needs_terminals != 0);
    if (__pyx_t_9) {

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":594
 *                 needs_terminals = True
 *             if needs_terminals:
 *                 rep = "".join((n_term, rep, c_term))             # <<<<<<<<<<<<<<
 *         if include_glycan:
 *             if self._glycosylation_manager.aggregate is not None:
 */
      __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 594, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_INCREF(__pyx_v_n_term);
      __Pyx_GIVEREF(__pyx_v_n_term);
      PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v_n_term);
      __Pyx_INCREF(__pyx_v_rep);
      __Pyx_GIVEREF(__pyx_v_rep);
      PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_rep);
      __Pyx_INCREF(__pyx_v_c_term);
      __Pyx_GIVEREF(__pyx_v_c_term);
      PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_c_term);
      __pyx_t_1 = __Pyx_PyString_Join(__pyx_kp_s_, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 594, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      __Pyx_DECREF_SET(__pyx_v_rep, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":593
 *                 c_term = "-(%s)" % terminal.serialize()
 *                 needs_terminals = True
 *             if needs_terminals:             # <<<<<<<<<<<<<<
 *                 rep = "".join((n_term, rep, c_term))
 *         if include_glycan:
 */
    }

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":576
 * 
 *         rep = ''.join(seq_list)
 *         if include_termini:             # <<<<<<<<<<<<<<
 *             needs_terminals = False
 *             # Handle N-terminal
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":595
 *             if needs_terminals:
 *                 rep = "".join((n_term, rep, c_term))
 *         if include_glycan:             # <<<<<<<<<<<<<<
 *             if self._glycosylation_manager.aggregate is not None:
 *                 rep += str(self._glycosylation_manager.aggregate)
 */
  __pyx_t_9 = (__pyx_v_include_glycan != 0);
  if (__pyx_t_9) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":596
 *                 rep = "".join((n_term, rep, c_term))
 *         if include_glycan:
 *             if self._glycosylation_manager.aggregate is not None:             # <<<<<<<<<<<<<<
 *                 rep += str(self._glycosylation_manager.aggregate)
 *         return rep
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base._glycosylation_manager, __pyx_n_s_aggregate); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 596, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_9 = (__pyx_t_1 != Py_None);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_10 = (__pyx_t_9 != 0);
    if (__pyx_t_10) {

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":597
 *         if include_glycan:
 *             if self._glycosylation_manager.aggregate is not None:
 *                 rep += str(self._glycosylation_manager.aggregate)             # <<<<<<<<<<<<<<
 *         return rep
 * 
 */
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base._glycosylation_manager, __pyx_n_s_aggregate); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 597, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 597, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_2);
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_rep, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 597, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
      if (!(likely(__Pyx_PyBaseString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", (PY_MAJOR_VERSION < 3 ? "basestring" : "str"), Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 597, __pyx_L1_error)
      __Pyx_DECREF_SET(__pyx_v_rep, ((PyObject*)__pyx_t_1));
      __pyx_t_1 = 0;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":596
 *                 rep = "".join((n_term, rep, c_term))
 *         if include_glycan:
 *             if self._glycosylation_manager.aggregate is not None:             # <<<<<<<<<<<<<<
 *                 rep += str(self._glycosylation_manager.aggregate)
 *         return rep
 */
    }

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":595
 *             if needs_terminals:
 *                 rep = "".join((n_term, rep, c_term))
 *         if include_glycan:             # <<<<<<<<<<<<<<
 *             if self._glycosylation_manager.aggregate is not None:
 *                 rep += str(self._glycosylation_manager.aggregate)
 */
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":598
 *             if self._glycosylation_manager.aggregate is not None:
 *                 rep += str(self._glycosylation_manager.aggregate)
 *         return rep             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_rep);
  __pyx_r = __pyx_v_rep;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":523
 *         return hash(self.get_sequence())
 * 
 *     cpdef basestring get_sequence(self, bint include_glycan=True, bint include_termini=True, str implicit_n_term=None, str implicit_c_term=None):             # <<<<<<<<<<<<<<
 *         """
 *         Generate human readable sequence string. Called by :meth:`__str__`
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.get_sequence", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_seq_list);
  __Pyx_XDECREF(__pyx_v_mod_strs);
  __Pyx_XDECREF((PyObject *)__pyx_v_position);
  __Pyx_XDECREF((PyObject *)__pyx_v_mod);
  __Pyx_XDECREF(__pyx_v_mod_str);
  __Pyx_XDECREF(__pyx_v_n_term);
  __Pyx_XDECREF(__pyx_v_c_term);
  __Pyx_XDECREF(__pyx_v_rep);
  __Pyx_XDECREF((PyObject *)__pyx_v_terminal);
  __Pyx_XDECREF((PyObject *)__pyx_v_implicit_terminal);
  __Pyx_XDECREF(__pyx_v_implicit_n_term);
  __Pyx_XDECREF(__pyx_v_implicit_c_term);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_41get_sequence(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_40get_sequence[] = "_PeptideSequenceCore.get_sequence(self, bool include_glycan=True, bool include_termini=True, str implicit_n_term=None, str implicit_c_term=None) -> basestring\n\n        Generate human readable sequence string. Called by :meth:`__str__`\n\n        Parameters\n        ----------\n        include_glycan: bool\n            Whether to include the glycan in the resulting string. Defaults to `True`\n        include_termini: bool\n            Whether to include the N- and C-termini. Make sure this is `True` if you want non-standard\n            termini to be properly propagated.\n\n\n        Returns\n        -------\n        str\n        ";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_41get_sequence(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  int __pyx_v_include_glycan;
  int __pyx_v_include_termini;
  PyObject *__pyx_v_implicit_n_term = 0;
  PyObject *__pyx_v_implicit_c_term = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_sequence (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_include_glycan,&__pyx_n_s_include_termini,&__pyx_n_s_implicit_n_term,&__pyx_n_s_implicit_c_term,0};
    PyObject* values[4] = {0,0,0,0};
    values[2] = ((PyObject*)Py_None);
    values[3] = ((PyObject*)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_include_glycan);
          if (value) { values[0] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_include_termini);
          if (value) { values[1] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_implicit_n_term);
          if (value) { values[2] = value; kw_args--; }
        }
        CYTHON_FALLTHROUGH;
        case  3:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_implicit_c_term);
          if (value) { values[3] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "get_sequence") < 0)) __PYX_ERR(0, 523, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  4: values[3] = PyTuple_GET_ITEM(__pyx_args, 3);
        CYTHON_FALLTHROUGH;
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    if (values[0]) {
      __pyx_v_include_glycan = __Pyx_PyObject_IsTrue(values[0]); if (unlikely((__pyx_v_include_glycan == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 523, __pyx_L3_error)
    } else {
      __pyx_v_include_glycan = ((int)1);
    }
    if (values[1]) {
      __pyx_v_include_termini = __Pyx_PyObject_IsTrue(values[1]); if (unlikely((__pyx_v_include_termini == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 523, __pyx_L3_error)
    } else {
      __pyx_v_include_termini = ((int)1);
    }
    __pyx_v_implicit_n_term = ((PyObject*)values[2]);
    __pyx_v_implicit_c_term = ((PyObject*)values[3]);
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("get_sequence", 0, 0, 4, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 523, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.get_sequence", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_implicit_n_term), (&PyString_Type), 1, "implicit_n_term", 1))) __PYX_ERR(0, 523, __pyx_L1_error)
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_implicit_c_term), (&PyString_Type), 1, "implicit_c_term", 1))) __PYX_ERR(0, 523, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_40get_sequence(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self), __pyx_v_include_glycan, __pyx_v_include_termini, __pyx_v_implicit_n_term, __pyx_v_implicit_c_term);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_40get_sequence(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, int __pyx_v_include_glycan, int __pyx_v_include_termini, PyObject *__pyx_v_implicit_n_term, PyObject *__pyx_v_implicit_c_term) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_sequence __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_sequence", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2.__pyx_n = 4;
  __pyx_t_2.include_glycan = __pyx_v_include_glycan;
  __pyx_t_2.include_termini = __pyx_v_include_termini;
  __pyx_t_2.implicit_n_term = __pyx_v_implicit_n_term;
  __pyx_t_2.implicit_c_term = __pyx_v_implicit_c_term;
  __pyx_t_1 = __pyx_vtabptr_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore->get_sequence(__pyx_v_self, 1, &__pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 523, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.get_sequence", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":600
 *         return rep
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.get_sequence()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_43__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_43__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_42__str__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_42__str__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":601
 * 
 *     def __str__(self):
 *         return self.get_sequence()             # <<<<<<<<<<<<<<
 * 
 *     cpdef clone(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_sequence(__pyx_v_self, 0, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 601, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":600
 *         return rep
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.get_sequence()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":603
 *         return self.get_sequence()
 * 
 *     cpdef clone(self):             # <<<<<<<<<<<<<<
 *         cdef _PeptideSequenceCore inst = self.__class__()
 *         if self._glycosylation_manager.aggregate is not None:
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_45clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_clone(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, int __pyx_skip_dispatch) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_inst = 0;
  PyObject *__pyx_v_glycan = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_components __pyx_t_7;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_clone); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 603, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_45clone)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 603, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":604
 * 
 *     cpdef clone(self):
 *         cdef _PeptideSequenceCore inst = self.__class__()             # <<<<<<<<<<<<<<
 *         if self._glycosylation_manager.aggregate is not None:
 *             glycan = self._glycosylation_manager.aggregate.clone()
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 604, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 604, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore))))) __PYX_ERR(0, 604, __pyx_L1_error)
  __pyx_v_inst = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":605
 *     cpdef clone(self):
 *         cdef _PeptideSequenceCore inst = self.__class__()
 *         if self._glycosylation_manager.aggregate is not None:             # <<<<<<<<<<<<<<
 *             glycan = self._glycosylation_manager.aggregate.clone()
 *             glycan.composition_offset = CComposition._create(WATER)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base._glycosylation_manager, __pyx_n_s_aggregate); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 605, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = (__pyx_t_1 != Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":606
 *         cdef _PeptideSequenceCore inst = self.__class__()
 *         if self._glycosylation_manager.aggregate is not None:
 *             glycan = self._glycosylation_manager.aggregate.clone()             # <<<<<<<<<<<<<<
 *             glycan.composition_offset = CComposition._create(WATER)
 *         else:
 */
    __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_self->__pyx_base._glycosylation_manager, __pyx_n_s_aggregate); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_clone); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_3))) {
      __pyx_t_2 = PyMethod_GET_SELF(__pyx_t_3);
      if (likely(__pyx_t_2)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
        __Pyx_INCREF(__pyx_t_2);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_3, function);
      }
    }
    __pyx_t_1 = (__pyx_t_2) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
    __Pyx_XDECREF(__pyx_t_2); __pyx_t_2 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 606, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __pyx_v_glycan = __pyx_t_1;
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":607
 *         if self._glycosylation_manager.aggregate is not None:
 *             glycan = self._glycosylation_manager.aggregate.clone()
 *             glycan.composition_offset = CComposition._create(WATER)             # <<<<<<<<<<<<<<
 *         else:
 *             glycan = None
 */
    __pyx_t_1 = ((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_WATER);
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_3 = ((PyObject *)__pyx_vtabptr_5glypy_11composition_12ccomposition_CComposition->_create(((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1))); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 607, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__Pyx_PyObject_SetAttrStr(__pyx_v_glycan, __pyx_n_s_composition_offset, __pyx_t_3) < 0) __PYX_ERR(0, 607, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":605
 *     cpdef clone(self):
 *         cdef _PeptideSequenceCore inst = self.__class__()
 *         if self._glycosylation_manager.aggregate is not None:             # <<<<<<<<<<<<<<
 *             glycan = self._glycosylation_manager.aggregate.clone()
 *             glycan.composition_offset = CComposition._create(WATER)
 */
    goto __pyx_L3;
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":609
 *             glycan.composition_offset = CComposition._create(WATER)
 *         else:
 *             glycan = None             # <<<<<<<<<<<<<<
 *         inst._init_from_components(
 *             self.sequence, glycan,
 */
  /*else*/ {
    __Pyx_INCREF(Py_None);
    __pyx_v_glycan = Py_None;
  }
  __pyx_L3:;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":611
 *             glycan = None
 *         inst._init_from_components(
 *             self.sequence, glycan,             # <<<<<<<<<<<<<<
 *             self.get_n_term().get_modification(),
 *             self.get_c_term().get_modification())
 */
  __pyx_t_3 = __pyx_v_self->__pyx_base.sequence;
  __Pyx_INCREF(__pyx_t_3);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":612
 *         inst._init_from_components(
 *             self.sequence, glycan,
 *             self.get_n_term().get_modification(),             # <<<<<<<<<<<<<<
 *             self.get_c_term().get_modification())
 *         return inst
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_n_term(__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 612, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_1)->__pyx_vtab)->get_modification(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_1))); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 612, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":613
 *             self.sequence, glycan,
 *             self.get_n_term().get_modification(),
 *             self.get_c_term().get_modification())             # <<<<<<<<<<<<<<
 *         return inst
 * 
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_c_term(__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 613, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_4 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_1)->__pyx_vtab)->get_modification(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_1))); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 613, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":610
 *         else:
 *             glycan = None
 *         inst._init_from_components(             # <<<<<<<<<<<<<<
 *             self.sequence, glycan,
 *             self.get_n_term().get_modification(),
 */
  __pyx_t_7.__pyx_n = 3;
  __pyx_t_7.glycan_composition = __pyx_v_glycan;
  __pyx_t_7.n_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_t_2);
  __pyx_t_7.c_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_t_4);
  __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_inst->__pyx_base.__pyx_vtab)->_init_from_components(__pyx_v_inst, ((PyObject*)__pyx_t_3), 0, &__pyx_t_7); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 610, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":614
 *             self.get_n_term().get_modification(),
 *             self.get_c_term().get_modification())
 *         return inst             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_inst));
  __pyx_r = ((PyObject *)__pyx_v_inst);
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":603
 *         return self.get_sequence()
 * 
 *     cpdef clone(self):             # <<<<<<<<<<<<<<
 *         cdef _PeptideSequenceCore inst = self.__class__()
 *         if self._glycosylation_manager.aggregate is not None:
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);
  __Pyx_XDECREF(__pyx_v_glycan);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_45clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_44clone[] = "_PeptideSequenceCore.clone(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_45clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("clone (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_44clone(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_44clone(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_clone(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 603, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_47__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_46__reduce_cython__[] = "_PeptideSequenceCore.__reduce_cython__(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_47__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_46__reduce_cython__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_46__reduce_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._c_term, self._glycosylation_manager, self._mass, self._n_term, self._peptide_composition, self._total_composition, self.sequence)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->__pyx_base._mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(7); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._c_term));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._c_term));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self->__pyx_base._c_term));
  __Pyx_INCREF(__pyx_v_self->__pyx_base._glycosylation_manager);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base._glycosylation_manager);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_self->__pyx_base._glycosylation_manager);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._n_term));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._n_term));
  PyTuple_SET_ITEM(__pyx_t_2, 3, ((PyObject *)__pyx_v_self->__pyx_base._n_term));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._peptide_composition));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._peptide_composition));
  PyTuple_SET_ITEM(__pyx_t_2, 4, ((PyObject *)__pyx_v_self->__pyx_base._peptide_composition));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->__pyx_base._total_composition));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->__pyx_base._total_composition));
  PyTuple_SET_ITEM(__pyx_t_2, 5, ((PyObject *)__pyx_v_self->__pyx_base._total_composition));
  __Pyx_INCREF(__pyx_v_self->__pyx_base.sequence);
  __Pyx_GIVEREF(__pyx_v_self->__pyx_base.sequence);
  PyTuple_SET_ITEM(__pyx_t_2, 6, __pyx_v_self->__pyx_base.sequence);
  __pyx_t_1 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._c_term, self._glycosylation_manager, self._mass, self._n_term, self._peptide_composition, self._total_composition, self.sequence)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_2 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__dict = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":7
 *     state = (self._c_term, self._glycosylation_manager, self._mass, self._n_term, self._peptide_composition, self._total_composition, self.sequence)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_3 = (__pyx_v__dict != Py_None);
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v__dict);
    __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._c_term is not None or self._glycosylation_manager is not None or self._n_term is not None or self._peptide_composition is not None or self._total_composition is not None or self.sequence is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._c_term, self._glycosylation_manager, self._mass, self._n_term, self._peptide_composition, self._total_composition, self.sequence)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._c_term is not None or self._glycosylation_manager is not None or self._n_term is not None or self._peptide_composition is not None or self._total_composition is not None or self.sequence is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle__PeptideSequenceCore, (type(self), 0x9a27ba6, None), state
 */
  /*else*/ {
    __pyx_t_3 = (((PyObject *)__pyx_v_self->__pyx_base._c_term) != Py_None);
    __pyx_t_5 = (__pyx_t_3 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->__pyx_base._glycosylation_manager != Py_None);
    __pyx_t_3 = (__pyx_t_5 != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_4 = __pyx_t_3;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_3 = (((PyObject *)__pyx_v_self->__pyx_base._n_term) != Py_None);
    __pyx_t_5 = (__pyx_t_3 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->__pyx_base._peptide_composition) != Py_None);
    __pyx_t_3 = (__pyx_t_5 != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_4 = __pyx_t_3;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_3 = (((PyObject *)__pyx_v_self->__pyx_base._total_composition) != Py_None);
    __pyx_t_5 = (__pyx_t_3 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->__pyx_base.sequence != ((PyObject*)Py_None));
    __pyx_t_3 = (__pyx_t_5 != 0);
    __pyx_t_4 = __pyx_t_3;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_4;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._c_term is not None or self._glycosylation_manager is not None or self._n_term is not None or self._peptide_composition is not None or self._total_composition is not None or self.sequence is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle__PeptideSequenceCore, (type(self), 0x9a27ba6, None), state
 *     else:
 */
  __pyx_t_4 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_4) {

    /* "(tree fragment)":13
 *         use_setstate = self._c_term is not None or self._glycosylation_manager is not None or self._n_term is not None or self._peptide_composition is not None or self._total_composition is not None or self.sequence is not None
 *     if use_setstate:
 *         return __pyx_unpickle__PeptideSequenceCore, (type(self), 0x9a27ba6, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle__PeptideSequenceCore, (type(self), 0x9a27ba6, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle__PeptideSequenceC); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_161643430);
    __Pyx_GIVEREF(__pyx_int_161643430);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_int_161643430);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_2, 2, Py_None);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_2);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._c_term is not None or self._glycosylation_manager is not None or self._n_term is not None or self._peptide_composition is not None or self._total_composition is not None or self.sequence is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle__PeptideSequenceCore, (type(self), 0x9a27ba6, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle__PeptideSequenceCore, (type(self), 0x9a27ba6, None), state
 *     else:
 *         return __pyx_unpickle__PeptideSequenceCore, (type(self), 0x9a27ba6, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle__PeptideSequenceCore__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle__PeptideSequenceC); if (unlikely(!__pyx_t_6)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_161643430);
    __Pyx_GIVEREF(__pyx_int_161643430);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_int_161643430);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
    __pyx_t_6 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle__PeptideSequenceCore, (type(self), 0x9a27ba6, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle__PeptideSequenceCore__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_49__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_48__setstate_cython__[] = "_PeptideSequenceCore.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_49__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_48__setstate_cython__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_48__setstate_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle__PeptideSequenceCore, (type(self), 0x9a27ba6, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle__PeptideSequenceCore__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods___pyx_unpickle__PeptideSequenceCore__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle__PeptideSequenceCore, (type(self), 0x9a27ba6, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle__PeptideSequenceCore__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/sequence_methods.pyx":618
 * 
 * @cython.binding(True)
 * cpdef add_modification(_PeptideSequenceCore self, position, modification_type):             # <<<<<<<<<<<<<<
 *     '''
 *     Add a modification by name to a specific residue. If the
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_3add_modification(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_add_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_position, PyObject *__pyx_v_modification_type, CYTHON_UNUSED int __pyx_skip_dispatch) {
  long __pyx_v_position_;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_mod = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_pos = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  long __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_modification", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":635
 *         ModificationBase mod
 *         SequencePosition pos
 *     self._invalidate()             # <<<<<<<<<<<<<<
 * 
 *     if isinstance(modification_type, Modification):
 */
  __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base._invalidate(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 635, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":637
 *     self._invalidate()
 * 
 *     if isinstance(modification_type, Modification):             # <<<<<<<<<<<<<<
 *         mod = modification_type
 *     else:
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_Modification); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 637, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyObject_IsInstance(__pyx_v_modification_type, __pyx_t_1); if (unlikely(__pyx_t_2 == ((int)-1))) __PYX_ERR(0, 637, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":638
 * 
 *     if isinstance(modification_type, Modification):
 *         mod = modification_type             # <<<<<<<<<<<<<<
 *     else:
 *         mod = Modification(rule=modification_type)
 */
    if (!(likely(((__pyx_v_modification_type) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_modification_type, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase))))) __PYX_ERR(0, 638, __pyx_L1_error)
    __pyx_t_1 = __pyx_v_modification_type;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_v_mod = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":637
 *     self._invalidate()
 * 
 *     if isinstance(modification_type, Modification):             # <<<<<<<<<<<<<<
 *         mod = modification_type
 *     else:
 */
    goto __pyx_L3;
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":640
 *         mod = modification_type
 *     else:
 *         mod = Modification(rule=modification_type)             # <<<<<<<<<<<<<<
 * 
 *     if position is SequenceLocation_n_term:
 */
  /*else*/ {
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_Modification); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 640, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 640, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (PyDict_SetItem(__pyx_t_4, __pyx_n_s_rule, __pyx_v_modification_type) < 0) __PYX_ERR(0, 640, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 640, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (!(likely(((__pyx_t_5) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_5, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase))))) __PYX_ERR(0, 640, __pyx_L1_error)
    __pyx_v_mod = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_t_5);
    __pyx_t_5 = 0;
  }
  __pyx_L3:;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":642
 *         mod = Modification(rule=modification_type)
 * 
 *     if position is SequenceLocation_n_term:             # <<<<<<<<<<<<<<
 *         self.n_term = self.n_term.modify(mod)
 *     elif position is SequenceLocation_c_term:
 */
  __pyx_t_3 = (__pyx_v_position == __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_SequenceLocation_n_term);
  __pyx_t_2 = (__pyx_t_3 != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":643
 * 
 *     if position is SequenceLocation_n_term:
 *         self.n_term = self.n_term.modify(mod)             # <<<<<<<<<<<<<<
 *     elif position is SequenceLocation_c_term:
 *         self.c_term = self.c_term.modify(mod)
 */
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_n_term); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 643, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_4, __pyx_n_s_modify); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 643, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_4 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_1))) {
      __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_1);
      if (likely(__pyx_t_4)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
        __Pyx_INCREF(__pyx_t_4);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_1, function);
      }
    }
    __pyx_t_5 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_4, ((PyObject *)__pyx_v_mod)) : __Pyx_PyObject_CallOneArg(__pyx_t_1, ((PyObject *)__pyx_v_mod));
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 643, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_n_term, __pyx_t_5) < 0) __PYX_ERR(0, 643, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":642
 *         mod = Modification(rule=modification_type)
 * 
 *     if position is SequenceLocation_n_term:             # <<<<<<<<<<<<<<
 *         self.n_term = self.n_term.modify(mod)
 *     elif position is SequenceLocation_c_term:
 */
    goto __pyx_L4;
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":644
 *     if position is SequenceLocation_n_term:
 *         self.n_term = self.n_term.modify(mod)
 *     elif position is SequenceLocation_c_term:             # <<<<<<<<<<<<<<
 *         self.c_term = self.c_term.modify(mod)
 *     else:
 */
  __pyx_t_2 = (__pyx_v_position == __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_SequenceLocation_c_term);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":645
 *         self.n_term = self.n_term.modify(mod)
 *     elif position is SequenceLocation_c_term:
 *         self.c_term = self.c_term.modify(mod)             # <<<<<<<<<<<<<<
 *     else:
 *         position_ = PyInt_AsLong(position)
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_c_term); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 645, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_modify); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 645, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
      __pyx_t_1 = PyMethod_GET_SELF(__pyx_t_4);
      if (likely(__pyx_t_1)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
        __Pyx_INCREF(__pyx_t_1);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_4, function);
      }
    }
    __pyx_t_5 = (__pyx_t_1) ? __Pyx_PyObject_Call2Args(__pyx_t_4, __pyx_t_1, ((PyObject *)__pyx_v_mod)) : __Pyx_PyObject_CallOneArg(__pyx_t_4, ((PyObject *)__pyx_v_mod));
    __Pyx_XDECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 645, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_c_term, __pyx_t_5) < 0) __PYX_ERR(0, 645, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":644
 *     if position is SequenceLocation_n_term:
 *         self.n_term = self.n_term.modify(mod)
 *     elif position is SequenceLocation_c_term:             # <<<<<<<<<<<<<<
 *         self.c_term = self.c_term.modify(mod)
 *     else:
 */
    goto __pyx_L4;
  }

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":647
 *         self.c_term = self.c_term.modify(mod)
 *     else:
 *         position_ = PyInt_AsLong(position)             # <<<<<<<<<<<<<<
 *         if (position_ == -1) or (position_ >= self.get_size()):
 *             raise IndexError(
 */
  /*else*/ {
    __pyx_t_6 = PyInt_AsLong(__pyx_v_position); if (unlikely(__pyx_t_6 == ((long)-1L) && PyErr_Occurred())) __PYX_ERR(0, 647, __pyx_L1_error)
    __pyx_v_position_ = __pyx_t_6;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":648
 *     else:
 *         position_ = PyInt_AsLong(position)
 *         if (position_ == -1) or (position_ >= self.get_size()):             # <<<<<<<<<<<<<<
 *             raise IndexError(
 *                 "Invalid modification position. %s, %s, %s" %
 */
    __pyx_t_2 = ((__pyx_v_position_ == -1L) != 0);
    if (!__pyx_t_2) {
    } else {
      __pyx_t_3 = __pyx_t_2;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_2 = ((__pyx_v_position_ >= ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->get_size(__pyx_v_self)) != 0);
    __pyx_t_3 = __pyx_t_2;
    __pyx_L6_bool_binop_done:;
    if (unlikely(__pyx_t_3)) {

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":651
 *             raise IndexError(
 *                 "Invalid modification position. %s, %s, %s" %
 *                 (position, str(self.sequence), modification_type))             # <<<<<<<<<<<<<<
 *         pos = self.get(position_)
 *         pos.add_modification(mod)
 */
      __pyx_t_5 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_v_self->__pyx_base.sequence); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 651, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_4 = PyTuple_New(3); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 651, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_v_position);
      __Pyx_GIVEREF(__pyx_v_position);
      PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_v_position);
      __Pyx_GIVEREF(__pyx_t_5);
      PyTuple_SET_ITEM(__pyx_t_4, 1, __pyx_t_5);
      __Pyx_INCREF(__pyx_v_modification_type);
      __Pyx_GIVEREF(__pyx_v_modification_type);
      PyTuple_SET_ITEM(__pyx_t_4, 2, __pyx_v_modification_type);
      __pyx_t_5 = 0;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":650
 *         if (position_ == -1) or (position_ >= self.get_size()):
 *             raise IndexError(
 *                 "Invalid modification position. %s, %s, %s" %             # <<<<<<<<<<<<<<
 *                 (position, str(self.sequence), modification_type))
 *         pos = self.get(position_)
 */
      __pyx_t_5 = __Pyx_PyString_Format(__pyx_kp_s_Invalid_modification_position_s, __pyx_t_4); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 650, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":649
 *         position_ = PyInt_AsLong(position)
 *         if (position_ == -1) or (position_ >= self.get_size()):
 *             raise IndexError(             # <<<<<<<<<<<<<<
 *                 "Invalid modification position. %s, %s, %s" %
 *                 (position, str(self.sequence), modification_type))
 */
      __pyx_t_4 = __Pyx_PyObject_CallOneArg(__pyx_builtin_IndexError, __pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 649, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_Raise(__pyx_t_4, 0, 0, 0);
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __PYX_ERR(0, 649, __pyx_L1_error)

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":648
 *     else:
 *         position_ = PyInt_AsLong(position)
 *         if (position_ == -1) or (position_ >= self.get_size()):             # <<<<<<<<<<<<<<
 *             raise IndexError(
 *                 "Invalid modification position. %s, %s, %s" %
 */
    }

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":652
 *                 "Invalid modification position. %s, %s, %s" %
 *                 (position, str(self.sequence), modification_type))
 *         pos = self.get(position_)             # <<<<<<<<<<<<<<
 *         pos.add_modification(mod)
 *         self._mass += mod.mass
 */
    __pyx_t_4 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v_self->__pyx_base.__pyx_vtab)->__pyx_base.get(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), __pyx_v_position_)); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 652, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_v_pos = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_t_4);
    __pyx_t_4 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":653
 *                 (position, str(self.sequence), modification_type))
 *         pos = self.get(position_)
 *         pos.add_modification(mod)             # <<<<<<<<<<<<<<
 *         self._mass += mod.mass
 *         if mod.is_tracked_for(ModificationCategory_glycosylation):
 */
    __pyx_t_4 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_pos->__pyx_vtab)->add_modification(__pyx_v_pos, ((PyObject *)__pyx_v_mod), 0); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 653, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":654
 *         pos = self.get(position_)
 *         pos.add_modification(mod)
 *         self._mass += mod.mass             # <<<<<<<<<<<<<<
 *         if mod.is_tracked_for(ModificationCategory_glycosylation):
 *             self._glycosylation_manager[position] = mod
 */
    __pyx_v_self->__pyx_base._mass = (__pyx_v_self->__pyx_base._mass + __pyx_v_mod->mass);

    /* "glycopeptidepy/_c/structure/sequence_methods.pyx":655
 *         pos.add_modification(mod)
 *         self._mass += mod.mass
 *         if mod.is_tracked_for(ModificationCategory_glycosylation):             # <<<<<<<<<<<<<<
 *             self._glycosylation_manager[position] = mod
 */
    __pyx_t_4 = __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationCategory_glycosylation;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_3 = (((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_mod->__pyx_vtab)->is_tracked_for(__pyx_v_mod, __pyx_t_4, 0) != 0);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__pyx_t_3) {

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":656
 *         self._mass += mod.mass
 *         if mod.is_tracked_for(ModificationCategory_glycosylation):
 *             self._glycosylation_manager[position] = mod             # <<<<<<<<<<<<<<
 */
      if (unlikely(PyObject_SetItem(__pyx_v_self->__pyx_base._glycosylation_manager, __pyx_v_position, ((PyObject *)__pyx_v_mod)) < 0)) __PYX_ERR(0, 656, __pyx_L1_error)

      /* "glycopeptidepy/_c/structure/sequence_methods.pyx":655
 *         pos.add_modification(mod)
 *         self._mass += mod.mass
 *         if mod.is_tracked_for(ModificationCategory_glycosylation):             # <<<<<<<<<<<<<<
 *             self._glycosylation_manager[position] = mod
 */
    }
  }
  __pyx_L4:;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":618
 * 
 * @cython.binding(True)
 * cpdef add_modification(_PeptideSequenceCore self, position, modification_type):             # <<<<<<<<<<<<<<
 *     '''
 *     Add a modification by name to a specific residue. If the
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods.add_modification", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mod);
  __Pyx_XDECREF((PyObject *)__pyx_v_pos);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_3add_modification(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_2add_modification[] = "add_modification(_PeptideSequenceCore self, position, modification_type)\n\n    Add a modification by name to a specific residue. If the\n    position is the N-term or the C-term, the terminal modification will\n    be replaced.\n\n    Parameters\n    ----------\n    position: int\n        The position of the modification to add\n    modification_type: str or Modification\n        The modification to add\n    ";
static PyMethodDef __pyx_mdef_14glycopeptidepy_2_c_9structure_16sequence_methods_3add_modification = {"add_modification", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_3add_modification, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_2add_modification};
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_3add_modification(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self = 0;
  PyObject *__pyx_v_position = 0;
  PyObject *__pyx_v_modification_type = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_modification (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_self,&__pyx_n_s_position,&__pyx_n_s_modification_type,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_self)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_position)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_modification", 1, 3, 3, 1); __PYX_ERR(0, 618, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_modification_type)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("add_modification", 1, 3, 3, 2); __PYX_ERR(0, 618, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "add_modification") < 0)) __PYX_ERR(0, 618, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v_self = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)values[0]);
    __pyx_v_position = values[1];
    __pyx_v_modification_type = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("add_modification", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 618, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods.add_modification", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_self), __pyx_ptype_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore, 1, "self", 0))) __PYX_ERR(0, 618, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_2add_modification(__pyx_self, __pyx_v_self, __pyx_v_position, __pyx_v_modification_type);

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_2add_modification(CYTHON_UNUSED PyObject *__pyx_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v_self, PyObject *__pyx_v_position, PyObject *__pyx_v_modification_type) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_modification", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_add_modification(__pyx_v_self, __pyx_v_position, __pyx_v_modification_type, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods.add_modification", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle__PeptideSequenceCore(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_5__pyx_unpickle__PeptideSequenceCore(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_4__pyx_unpickle__PeptideSequenceCore[] = "__pyx_unpickle__PeptideSequenceCore(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_14glycopeptidepy_2_c_9structure_16sequence_methods_5__pyx_unpickle__PeptideSequenceCore = {"__pyx_unpickle__PeptideSequenceCore", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_5__pyx_unpickle__PeptideSequenceCore, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_4__pyx_unpickle__PeptideSequenceCore};
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_5__pyx_unpickle__PeptideSequenceCore(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle__PeptideSequenceCore (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__PeptideSequenceCore", 1, 3, 3, 1); __PYX_ERR(1, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__PeptideSequenceCore", 1, 3, 3, 2); __PYX_ERR(1, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle__PeptideSequenceCore") < 0)) __PYX_ERR(1, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(1, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle__PeptideSequenceCore", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(1, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods.__pyx_unpickle__PeptideSequenceCore", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_4__pyx_unpickle__PeptideSequenceCore(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_16sequence_methods_4__pyx_unpickle__PeptideSequenceCore(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle__PeptideSequenceCore", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x9a27ba6:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9a27ba6 = (_c_term, _glycosylation_manager, _mass, _n_term, _peptide_composition, _total_composition, sequence))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x9a27ba6) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x9a27ba6:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9a27ba6 = (_c_term, _glycosylation_manager, _mass, _n_term, _peptide_composition, _total_composition, sequence))" % __pyx_checksum)
 *     __pyx_result = _PeptideSequenceCore.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x9a27ba6:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9a27ba6 = (_c_term, _glycosylation_manager, _mass, _n_term, _peptide_composition, _total_composition, sequence))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = _PeptideSequenceCore.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x9a, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(1, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x9a27ba6:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9a27ba6 = (_c_term, _glycosylation_manager, _mass, _n_term, _peptide_composition, _total_composition, sequence))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9a27ba6 = (_c_term, _glycosylation_manager, _mass, _n_term, _peptide_composition, _total_composition, sequence))" % __pyx_checksum)
 *     __pyx_result = _PeptideSequenceCore.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle__PeptideSequenceCore__set_state(<_PeptideSequenceCore> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9a27ba6 = (_c_term, _glycosylation_manager, _mass, _n_term, _peptide_composition, _total_composition, sequence))" % __pyx_checksum)
 *     __pyx_result = _PeptideSequenceCore.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle__PeptideSequenceCore__set_state(<_PeptideSequenceCore> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = _PeptideSequenceCore.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle__PeptideSequenceCore__set_state(<_PeptideSequenceCore> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle__PeptideSequenceCore__set_state(_PeptideSequenceCore __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(1, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods___pyx_unpickle__PeptideSequenceCore__set_state(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(1, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9a27ba6 = (_c_term, _glycosylation_manager, _mass, _n_term, _peptide_composition, _total_composition, sequence))" % __pyx_checksum)
 *     __pyx_result = _PeptideSequenceCore.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle__PeptideSequenceCore__set_state(<_PeptideSequenceCore> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle__PeptideSequenceCore__set_state(<_PeptideSequenceCore> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle__PeptideSequenceCore__set_state(_PeptideSequenceCore __pyx_result, tuple __pyx_state):
 *     __pyx_result._c_term = __pyx_state[0]; __pyx_result._glycosylation_manager = __pyx_state[1]; __pyx_result._mass = __pyx_state[2]; __pyx_result._n_term = __pyx_state[3]; __pyx_result._peptide_composition = __pyx_state[4]; __pyx_result._total_composition = __pyx_state[5]; __pyx_result.sequence = __pyx_state[6]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle__PeptideSequenceCore(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods.__pyx_unpickle__PeptideSequenceCore", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle__PeptideSequenceCore__set_state(<_PeptideSequenceCore> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle__PeptideSequenceCore__set_state(_PeptideSequenceCore __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._c_term = __pyx_state[0]; __pyx_result._glycosylation_manager = __pyx_state[1]; __pyx_result._mass = __pyx_state[2]; __pyx_result._n_term = __pyx_state[3]; __pyx_result._peptide_composition = __pyx_state[4]; __pyx_result._total_composition = __pyx_state[5]; __pyx_result.sequence = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods___pyx_unpickle__PeptideSequenceCore__set_state(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle__PeptideSequenceCore__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle__PeptideSequenceCore__set_state(_PeptideSequenceCore __pyx_result, tuple __pyx_state):
 *     __pyx_result._c_term = __pyx_state[0]; __pyx_result._glycosylation_manager = __pyx_state[1]; __pyx_result._mass = __pyx_state[2]; __pyx_result._n_term = __pyx_state[3]; __pyx_result._peptide_composition = __pyx_state[4]; __pyx_result._total_composition = __pyx_state[5]; __pyx_result.sequence = __pyx_state[6]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._c_term);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._c_term));
  __pyx_v___pyx_result->__pyx_base._c_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._glycosylation_manager);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base._glycosylation_manager);
  __pyx_v___pyx_result->__pyx_base._glycosylation_manager = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->__pyx_base._mass = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._n_term);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._n_term));
  __pyx_v___pyx_result->__pyx_base._n_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._peptide_composition);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._peptide_composition));
  __pyx_v___pyx_result->__pyx_base._peptide_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base._total_composition);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->__pyx_base._total_composition));
  __pyx_v___pyx_result->__pyx_base._total_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(1, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(1, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->__pyx_base.sequence);
  __Pyx_DECREF(__pyx_v___pyx_result->__pyx_base.sequence);
  __pyx_v___pyx_result->__pyx_base.sequence = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle__PeptideSequenceCore__set_state(_PeptideSequenceCore __pyx_result, tuple __pyx_state):
 *     __pyx_result._c_term = __pyx_state[0]; __pyx_result._glycosylation_manager = __pyx_state[1]; __pyx_result._mass = __pyx_state[2]; __pyx_result._n_term = __pyx_state[3]; __pyx_result._peptide_composition = __pyx_state[4]; __pyx_result._total_composition = __pyx_state[5]; __pyx_result.sequence = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(1, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 7) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._c_term = __pyx_state[0]; __pyx_result._glycosylation_manager = __pyx_state[1]; __pyx_result._mass = __pyx_state[2]; __pyx_result._n_term = __pyx_state[3]; __pyx_result._peptide_composition = __pyx_state[4]; __pyx_result._total_composition = __pyx_state[5]; __pyx_result.sequence = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(1, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle__PeptideSequenceCore__set_state(_PeptideSequenceCore __pyx_result, tuple __pyx_state):
 *     __pyx_result._c_term = __pyx_state[0]; __pyx_result._glycosylation_manager = __pyx_state[1]; __pyx_result._mass = __pyx_state[2]; __pyx_result._n_term = __pyx_state[3]; __pyx_result._peptide_composition = __pyx_state[4]; __pyx_result._total_composition = __pyx_state[5]; __pyx_result.sequence = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle__PeptideSequenceCore__set_state(<_PeptideSequenceCore> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle__PeptideSequenceCore__set_state(_PeptideSequenceCore __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._c_term = __pyx_state[0]; __pyx_result._glycosylation_manager = __pyx_state[1]; __pyx_result._mass = __pyx_state[2]; __pyx_result._n_term = __pyx_state[3]; __pyx_result._peptide_composition = __pyx_state[4]; __pyx_result._total_composition = __pyx_state[5]; __pyx_result.sequence = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.sequence_methods.__pyx_unpickle__PeptideSequenceCore__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore;

static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore(PyTypeObject *t, PyObject *a, PyObject *k) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *p;
  PyObject *o = __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase->tp_new(t, a, k);
  if (unlikely(!o)) return 0;
  p = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *)o);
  p->__pyx_base.__pyx_vtab = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase*)__pyx_vtabptr_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore;
  return o;
}

static void __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore(PyObject *o) {
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  PyObject_GC_Track(o);
  if (likely(__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase)) __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase->tp_dealloc(o); else __Pyx_call_next_tp_dealloc(o, __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore);
}

static int __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore(PyObject *o, visitproc v, void *a) {
  int e;
  e = ((likely(__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase)) ? ((__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase->tp_traverse) ? __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase->tp_traverse(o, v, a) : 0) : __Pyx_call_next_tp_traverse(o, v, a, __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore)); if (e) return e;
  return 0;
}

static int __pyx_tp_clear_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore(PyObject *o) {
  if (likely(__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase)) { if (__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase->tp_clear) __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase->tp_clear(o); } else __Pyx_call_next_tp_clear(o, __pyx_tp_clear_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore);
  return 0;
}
static PyObject *__pyx_sq_item_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_25__setitem__(o, i, v);
  }
  else {
    if (__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase->tp_as_mapping && __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase->tp_as_mapping->mp_ass_subscript)
      return __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase->tp_as_mapping->mp_ass_subscript(o, i, v);
    PyErr_Format(PyExc_NotImplementedError,
      "Subscript deletion not supported by %.200s", Py_TYPE(o)->tp_name);
    return -1;
  }
}

static PyObject *__pyx_tp_richcompare_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore(PyObject *o1, PyObject *o2, int op) {
  switch (op) {
    case Py_EQ: {
      return __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_29__eq__(o1, o2);
    }
    case Py_NE: {
      return __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_37__ne__(o1, o2);
    }
    default: {
      return __Pyx_NewRef(Py_NotImplemented);
    }
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_n_term(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6n_term_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_n_term(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6n_term_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_c_term(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6c_term_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_c_term(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6c_term_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_mass(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_4mass_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_mass(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_4mass_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_peptide_backbone_mass(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_21peptide_backbone_mass_1__get__(o);
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_total_mass(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_10total_mass_1__get__(o);
}

static PyMethodDef __pyx_methods_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore[] = {
  {"_init_from_parsed_string", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_3_init_from_parsed_string, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_2_init_from_parsed_string},
  {"_init_from_string", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_5_init_from_string, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_4_init_from_string},
  {"_init_from_components", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_7_init_from_components, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_6_init_from_components},
  {"_init_termini", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_9_init_termini, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_8_init_termini},
  {"_initialize_fields", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_11_initialize_fields, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_10_initialize_fields},
  {"_invalidate", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_13_invalidate, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_12_invalidate},
  {"peptide_composition", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_17peptide_composition, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_16peptide_composition},
  {"total_composition", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_19total_composition, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_18total_composition},
  {"base_sequence_equality", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_31base_sequence_equality, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_30base_sequence_equality},
  {"modified_sequence_equality", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_33modified_sequence_equality, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_32modified_sequence_equality},
  {"full_structure_equality", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_35full_structure_equality, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_34full_structure_equality},
  {"get_sequence", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_41get_sequence, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_40get_sequence},
  {"clone", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_45clone, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_44clone},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_47__reduce_cython__, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_46__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_49__setstate_cython__, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_48__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore[] = {
  {(char *)"n_term", __pyx_getprop_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_n_term, __pyx_setprop_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_n_term, (char *)0, 0},
  {(char *)"c_term", __pyx_getprop_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_c_term, __pyx_setprop_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_c_term, (char *)0, 0},
  {(char *)"mass", __pyx_getprop_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_mass, __pyx_setprop_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_mass, (char *)0, 0},
  {(char *)"peptide_backbone_mass", __pyx_getprop_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_peptide_backbone_mass, 0, (char *)0, 0},
  {(char *)"total_mass", __pyx_getprop_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_total_mass, 0, (char *)0, 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence__PeptideSequenceCore = {
  __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_27__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping__PeptideSequenceCore = {
  __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_27__len__, /*mp_length*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_23__getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore = {
  PyVarObject_HEAD_INIT(0, 0)
  "glycopeptidepy._c.structure.sequence_methods._PeptideSequenceCore", /*tp_name*/
  sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_15__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence__PeptideSequenceCore, /*tp_as_sequence*/
  &__pyx_tp_as_mapping__PeptideSequenceCore, /*tp_as_mapping*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_39__hash__, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_43__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "_PeptideSequenceCore(sequence=None, parser_function=None, **kwargs)", /*tp_doc*/
  __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore, /*tp_traverse*/
  __pyx_tp_clear_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore, /*tp_clear*/
  __pyx_tp_richcompare_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_21__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {"get_modification_by_name", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_1get_modification_by_name, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_get_modification_by_name},
  {"add_modification", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_16sequence_methods_3add_modification, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_16sequence_methods_2add_modification},
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_sequence_methods(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_sequence_methods},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "sequence_methods",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_kp_s_, __pyx_k_, sizeof(__pyx_k_), 0, 0, 1, 0},
  {&__pyx_kp_s_0, __pyx_k_0, sizeof(__pyx_k_0), 0, 0, 1, 0},
  {&__pyx_kp_s_0_2, __pyx_k_0_2, sizeof(__pyx_k_0_2), 0, 0, 1, 0},
  {&__pyx_n_s_AminoAcidResidue, __pyx_k_AminoAcidResidue, sizeof(__pyx_k_AminoAcidResidue), 0, 0, 1, 1},
  {&__pyx_n_s_AttributeError, __pyx_k_AttributeError, sizeof(__pyx_k_AttributeError), 0, 0, 1, 1},
  {&__pyx_n_s_H2O, __pyx_k_H2O, sizeof(__pyx_k_H2O), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x9a, __pyx_k_Incompatible_checksums_s_vs_0x9a, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x9a), 0, 0, 1, 0},
  {&__pyx_n_s_IndexError, __pyx_k_IndexError, sizeof(__pyx_k_IndexError), 0, 0, 1, 1},
  {&__pyx_kp_s_Invalid_modification_position_s, __pyx_k_Invalid_modification_position_s, sizeof(__pyx_k_Invalid_modification_position_s), 0, 0, 1, 0},
  {&__pyx_n_s_Modification, __pyx_k_Modification, sizeof(__pyx_k_Modification), 0, 0, 1, 1},
  {&__pyx_n_s_ModificationCategory, __pyx_k_ModificationCategory, sizeof(__pyx_k_ModificationCategory), 0, 0, 1, 1},
  {&__pyx_n_s_PeptideSequenceCore, __pyx_k_PeptideSequenceCore, sizeof(__pyx_k_PeptideSequenceCore), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_SequenceLocation, __pyx_k_SequenceLocation, sizeof(__pyx_k_SequenceLocation), 0, 0, 1, 1},
  {&__pyx_kp_s__2, __pyx_k__2, sizeof(__pyx_k__2), 0, 0, 1, 0},
  {&__pyx_n_s_add_modification, __pyx_k_add_modification, sizeof(__pyx_k_add_modification), 0, 0, 1, 1},
  {&__pyx_n_s_aggregate, __pyx_k_aggregate, sizeof(__pyx_k_aggregate), 0, 0, 1, 1},
  {&__pyx_n_s_base_sequence_equality, __pyx_k_base_sequence_equality, sizeof(__pyx_k_base_sequence_equality), 0, 0, 1, 1},
  {&__pyx_n_s_c_term, __pyx_k_c_term, sizeof(__pyx_k_c_term), 0, 0, 1, 1},
  {&__pyx_n_s_class, __pyx_k_class, sizeof(__pyx_k_class), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_clone, __pyx_k_clone, sizeof(__pyx_k_clone), 0, 0, 1, 1},
  {&__pyx_n_s_composition_offset, __pyx_k_composition_offset, sizeof(__pyx_k_composition_offset), 0, 0, 1, 1},
  {&__pyx_n_s_constants, __pyx_k_constants, sizeof(__pyx_k_constants), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_full_structure_equality, __pyx_k_full_structure_equality, sizeof(__pyx_k_full_structure_equality), 0, 0, 1, 1},
  {&__pyx_n_s_get_sequence, __pyx_k_get_sequence, sizeof(__pyx_k_get_sequence), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_glycan, __pyx_k_glycan, sizeof(__pyx_k_glycan), 0, 0, 1, 1},
  {&__pyx_n_s_glycan_aggregate, __pyx_k_glycan_aggregate, sizeof(__pyx_k_glycan_aggregate), 0, 0, 1, 1},
  {&__pyx_n_s_glycan_composition, __pyx_k_glycan_composition, sizeof(__pyx_k_glycan_composition), 0, 0, 1, 1},
  {&__pyx_n_s_glycopeptidepy__c_parser, __pyx_k_glycopeptidepy__c_parser, sizeof(__pyx_k_glycopeptidepy__c_parser), 0, 0, 1, 1},
  {&__pyx_kp_s_glycopeptidepy__c_structure_sequ, __pyx_k_glycopeptidepy__c_structure_sequ, sizeof(__pyx_k_glycopeptidepy__c_structure_sequ), 0, 0, 1, 0},
  {&__pyx_n_s_glycopeptidepy__c_structure_sequ_2, __pyx_k_glycopeptidepy__c_structure_sequ_2, sizeof(__pyx_k_glycopeptidepy__c_structure_sequ_2), 0, 0, 1, 1},
  {&__pyx_n_s_glycopeptidepy_structure, __pyx_k_glycopeptidepy_structure, sizeof(__pyx_k_glycopeptidepy_structure), 0, 0, 1, 1},
  {&__pyx_n_s_glycopeptidepy_structure_modific, __pyx_k_glycopeptidepy_structure_modific, sizeof(__pyx_k_glycopeptidepy_structure_modific), 0, 0, 1, 1},
  {&__pyx_n_s_glycopeptidepy_structure_residue, __pyx_k_glycopeptidepy_structure_residue, sizeof(__pyx_k_glycopeptidepy_structure_residue), 0, 0, 1, 1},
  {&__pyx_n_s_glycosylation, __pyx_k_glycosylation, sizeof(__pyx_k_glycosylation), 0, 0, 1, 1},
  {&__pyx_n_s_implicit_c_term, __pyx_k_implicit_c_term, sizeof(__pyx_k_implicit_c_term), 0, 0, 1, 1},
  {&__pyx_n_s_implicit_n_term, __pyx_k_implicit_n_term, sizeof(__pyx_k_implicit_n_term), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_include_glycan, __pyx_k_include_glycan, sizeof(__pyx_k_include_glycan), 0, 0, 1, 1},
  {&__pyx_n_s_include_termini, __pyx_k_include_termini, sizeof(__pyx_k_include_termini), 0, 0, 1, 1},
  {&__pyx_n_s_init_from_components, __pyx_k_init_from_components, sizeof(__pyx_k_init_from_components), 0, 0, 1, 1},
  {&__pyx_n_s_init_from_parsed_string, __pyx_k_init_from_parsed_string, sizeof(__pyx_k_init_from_parsed_string), 0, 0, 1, 1},
  {&__pyx_n_s_init_from_string, __pyx_k_init_from_string, sizeof(__pyx_k_init_from_string), 0, 0, 1, 1},
  {&__pyx_n_s_init_termini, __pyx_k_init_termini, sizeof(__pyx_k_init_termini), 0, 0, 1, 1},
  {&__pyx_n_s_initialize_fields, __pyx_k_initialize_fields, sizeof(__pyx_k_initialize_fields), 0, 0, 1, 1},
  {&__pyx_n_s_invalidate, __pyx_k_invalidate, sizeof(__pyx_k_invalidate), 0, 0, 1, 1},
  {&__pyx_n_s_join, __pyx_k_join, sizeof(__pyx_k_join), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_mass, __pyx_k_mass, sizeof(__pyx_k_mass), 0, 0, 1, 1},
  {&__pyx_n_s_mass_2, __pyx_k_mass_2, sizeof(__pyx_k_mass_2), 0, 0, 1, 1},
  {&__pyx_n_s_modification_table, __pyx_k_modification_table, sizeof(__pyx_k_modification_table), 0, 0, 1, 1},
  {&__pyx_n_s_modification_type, __pyx_k_modification_type, sizeof(__pyx_k_modification_type), 0, 0, 1, 1},
  {&__pyx_n_s_modified_sequence_equality, __pyx_k_modified_sequence_equality, sizeof(__pyx_k_modified_sequence_equality), 0, 0, 1, 1},
  {&__pyx_n_s_modify, __pyx_k_modify, sizeof(__pyx_k_modify), 0, 0, 1, 1},
  {&__pyx_n_s_n_term, __pyx_k_n_term, sizeof(__pyx_k_n_term), 0, 0, 1, 1},
  {&__pyx_kp_s_n_term_sequence_c_term_glycan_a, __pyx_k_n_term_sequence_c_term_glycan_a, sizeof(__pyx_k_n_term_sequence_c_term_glycan_a), 0, 0, 1, 0},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_parser_function, __pyx_k_parser_function, sizeof(__pyx_k_parser_function), 0, 0, 1, 1},
  {&__pyx_n_s_peptide_base_composition, __pyx_k_peptide_base_composition, sizeof(__pyx_k_peptide_base_composition), 0, 0, 1, 1},
  {&__pyx_n_s_peptide_composition, __pyx_k_peptide_composition, sizeof(__pyx_k_peptide_composition), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_position, __pyx_k_position, sizeof(__pyx_k_position), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle__PeptideSequenceC, __pyx_k_pyx_unpickle__PeptideSequenceC, sizeof(__pyx_k_pyx_unpickle__PeptideSequenceC), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_rule, __pyx_k_rule, sizeof(__pyx_k_rule), 0, 0, 1, 1},
  {&__pyx_kp_s_s, __pyx_k_s, sizeof(__pyx_k_s), 0, 0, 1, 0},
  {&__pyx_kp_s_s_2, __pyx_k_s_2, sizeof(__pyx_k_s_2), 0, 0, 1, 0},
  {&__pyx_kp_s_s_3, __pyx_k_s_3, sizeof(__pyx_k_s_3), 0, 0, 1, 0},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_n_s_seq_list, __pyx_k_seq_list, sizeof(__pyx_k_seq_list), 0, 0, 1, 1},
  {&__pyx_n_s_sequence, __pyx_k_sequence, sizeof(__pyx_k_sequence), 0, 0, 1, 1},
  {&__pyx_n_s_sequence_tokenizer, __pyx_k_sequence_tokenizer, sizeof(__pyx_k_sequence_tokenizer), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_structure_constants, __pyx_k_structure_constants, sizeof(__pyx_k_structure_constants), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_total_composition, __pyx_k_total_composition, sizeof(__pyx_k_total_composition), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 133, __pyx_L1_error)
  __pyx_builtin_AttributeError = __Pyx_GetBuiltinName(__pyx_n_s_AttributeError); if (!__pyx_builtin_AttributeError) __PYX_ERR(0, 330, __pyx_L1_error)
  __pyx_builtin_IndexError = __Pyx_GetBuiltinName(__pyx_n_s_IndexError); if (!__pyx_builtin_IndexError) __PYX_ERR(0, 649, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":29
 * from glycopeptidepy.structure import constants as _structure_constants
 * 
 * cdef CComposition WATER = CComposition("H2O")             # <<<<<<<<<<<<<<
 * 
 * cdef Configuration structure_constants = _structure_constants
 */
  __pyx_tuple__3 = PyTuple_Pack(1, __pyx_n_s_H2O); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":618
 * 
 * @cython.binding(True)
 * cpdef add_modification(_PeptideSequenceCore self, position, modification_type):             # <<<<<<<<<<<<<<
 *     '''
 *     Add a modification by name to a specific residue. If the
 */
  __pyx_tuple__4 = PyTuple_Pack(3, __pyx_n_s_self, __pyx_n_s_position, __pyx_n_s_modification_type); if (unlikely(!__pyx_tuple__4)) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__4);
  __Pyx_GIVEREF(__pyx_tuple__4);
  __pyx_codeobj__5 = (PyObject*)__Pyx_PyCode_New(3, 0, 3, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__4, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_glycopeptidepy__c_structure_sequ, __pyx_n_s_add_modification, 618, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__5)) __PYX_ERR(0, 618, __pyx_L1_error)

  /* "(tree fragment)":1
 * def __pyx_unpickle__PeptideSequenceCore(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple__6 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__6)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__6);
  __Pyx_GIVEREF(__pyx_tuple__6);
  __pyx_codeobj__7 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__6, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle__PeptideSequenceC, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__7)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_161643430 = PyInt_FromLong(161643430L); if (unlikely(!__pyx_int_161643430)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_WATER = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_structure_constants = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_9constants_Configuration *)Py_None); Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationImpl = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_AminoAcidResidueImpl = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_SequenceLocation_n_term = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_SequenceLocation_c_term = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationCategory_glycosylation = Py_None; Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_modification_table = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase *)Py_None); Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_terminal_group_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_amino_acid_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_implicit_terminal_cache = ((PyObject*)Py_None); Py_INCREF(Py_None);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  if (__Pyx_ExportFunction("_parse_residue", (void (*)(void))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__parse_residue, "struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *(PyObject *)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("_make_terminal_group", (void (*)(void))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods__make_terminal_group, "struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *(PyObject *, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods__make_terminal_group *__pyx_optional_args)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ExportFunction("add_modification", (void (*)(void))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_add_modification, "PyObject *(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *, PyObject *, int __pyx_skip_dispatch)") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_t_1 = PyImport_ImportModule("glycopeptidepy._c.structure.base"); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.base", "PeptideSequenceBase", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore = &__pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.__pyx_base = *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.__pyx_base._invalidate = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__invalidate;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore._init_from_parsed_string = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_parsed_string *__pyx_optional_args))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_parsed_string;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore._init_from_string = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *, PyObject *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_string;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore._init_from_components = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_components *__pyx_optional_args))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_from_components;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore._init_termini = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__init_termini;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore._initialize_fields = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore__initialize_fields;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.get_size = (size_t (*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_size;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.get_n_term = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_n_term;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.set_n_term = (void (*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_set_n_term;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.get_c_term = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_c_term;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.set_c_term = (void (*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, PyObject *))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_set_c_term;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.get_sequence = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch, struct __pyx_opt_args_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_sequence *__pyx_optional_args))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_get_sequence;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.clone = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_clone;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.base_sequence_equality = (int (*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_base_sequence_equality;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.modified_sequence_equality = (int (*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_modified_sequence_equality;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.full_structure_equality = (int (*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_full_structure_equality;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.peptide_composition = (struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_peptide_composition;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.total_composition = (struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_16sequence_methods_20_PeptideSequenceCore_total_composition;
  __pyx_type_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.tp_base = __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase;
  if (PyType_Ready(&__pyx_type_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore) < 0) __PYX_ERR(0, 151, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.tp_dictoffset && __pyx_type_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore.tp_dict, __pyx_vtabptr_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore) < 0) __PYX_ERR(0, 151, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_PeptideSequenceCore, (PyObject *)&__pyx_type_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore) < 0) __PYX_ERR(0, 151, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore) < 0) __PYX_ERR(0, 151, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore = &__pyx_type_14glycopeptidepy_2_c_9structure_16sequence_methods__PeptideSequenceCore;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule("glypy.composition.ccomposition"); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5glypy_11composition_12ccomposition_CComposition = __Pyx_ImportType(__pyx_t_1, "glypy.composition.ccomposition", "CComposition", sizeof(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5glypy_11composition_12ccomposition_CComposition) __PYX_ERR(2, 2, __pyx_L1_error)
  __pyx_vtabptr_5glypy_11composition_12ccomposition_CComposition = (struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition*)__Pyx_GetVtable(__pyx_ptype_5glypy_11composition_12ccomposition_CComposition->tp_dict); if (unlikely(!__pyx_vtabptr_5glypy_11composition_12ccomposition_CComposition)) __PYX_ERR(2, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("glycopeptidepy._c.structure.base"); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 21, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.base", "TerminalGroup", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup) __PYX_ERR(3, 21, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_TerminalGroup = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_TerminalGroup)) __PYX_ERR(3, 21, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.base", "AminoAcidResidueBase", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase) __PYX_ERR(3, 35, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase)) __PYX_ERR(3, 35, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.base", "ModificationBase", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase) __PYX_ERR(3, 50, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_ModificationBase = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_ModificationBase)) __PYX_ERR(3, 50, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_SequencePosition = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.base", "SequencePosition", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_SequencePosition) __PYX_ERR(3, 69, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_SequencePosition = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_SequencePosition*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_SequencePosition->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_SequencePosition)) __PYX_ERR(3, 69, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(4, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(5, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4bool_bool = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "bool", sizeof(PyBoolObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4bool_bool) __PYX_ERR(5, 8, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(6, 15, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_7complex_complex = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "complex", sizeof(PyComplexObject), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_7complex_complex) __PYX_ERR(6, 15, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("glypy.utils.cenum"); if (unlikely(!__pyx_t_1)) __PYX_ERR(7, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5glypy_5utils_5cenum_EnumValue = __Pyx_ImportType(__pyx_t_1, "glypy.utils.cenum", "EnumValue", sizeof(struct __pyx_obj_5glypy_5utils_5cenum_EnumValue), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5glypy_5utils_5cenum_EnumValue) __PYX_ERR(7, 2, __pyx_L1_error)
  __pyx_vtabptr_5glypy_5utils_5cenum_EnumValue = (struct __pyx_vtabstruct_5glypy_5utils_5cenum_EnumValue*)__Pyx_GetVtable(__pyx_ptype_5glypy_5utils_5cenum_EnumValue->tp_dict); if (unlikely(!__pyx_vtabptr_5glypy_5utils_5cenum_EnumValue)) __PYX_ERR(7, 2, __pyx_L1_error)
  __pyx_ptype_5glypy_5utils_5cenum_IntEnumValue = __Pyx_ImportType(__pyx_t_1, "glypy.utils.cenum", "IntEnumValue", sizeof(struct __pyx_obj_5glypy_5utils_5cenum_IntEnumValue), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5glypy_5utils_5cenum_IntEnumValue) __PYX_ERR(7, 14, __pyx_L1_error)
  __pyx_vtabptr_5glypy_5utils_5cenum_IntEnumValue = (struct __pyx_vtabstruct_5glypy_5utils_5cenum_IntEnumValue*)__Pyx_GetVtable(__pyx_ptype_5glypy_5utils_5cenum_IntEnumValue->tp_dict); if (unlikely(!__pyx_vtabptr_5glypy_5utils_5cenum_IntEnumValue)) __PYX_ERR(7, 14, __pyx_L1_error)
  __pyx_ptype_5glypy_5utils_5cenum_EnumMeta = __Pyx_ImportType(__pyx_t_1, "glypy.utils.cenum", "EnumMeta", sizeof(struct __pyx_obj_5glypy_5utils_5cenum_EnumMeta), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5glypy_5utils_5cenum_EnumMeta) __PYX_ERR(7, 19, __pyx_L1_error)
  __pyx_vtabptr_5glypy_5utils_5cenum_EnumMeta = (struct __pyx_vtabstruct_5glypy_5utils_5cenum_EnumMeta*)__Pyx_GetVtable(__pyx_ptype_5glypy_5utils_5cenum_EnumMeta->tp_dict); if (unlikely(!__pyx_vtabptr_5glypy_5utils_5cenum_EnumMeta)) __PYX_ERR(7, 19, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("glycopeptidepy._c.structure.modification.rule"); if (unlikely(!__pyx_t_1)) __PYX_ERR(8, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.modification.rule", "ModificationRuleBase", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase) __PYX_ERR(8, 5, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_12modification_4rule_ModificationRuleBase)) __PYX_ERR(8, 5, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_4rule_NeutralLossBase = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.modification.rule", "NeutralLossBase", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_NeutralLossBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_4rule_NeutralLossBase) __PYX_ERR(8, 22, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.modification.rule", "GlycosylationBase", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase) __PYX_ERR(8, 32, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_12modification_4rule_GlycosylationBase)) __PYX_ERR(8, 32, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("glycopeptidepy._c.structure.modification.source"); if (unlikely(!__pyx_t_1)) __PYX_ERR(9, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.modification.source", "ModificationTableBase", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase) __PYX_ERR(9, 3, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase)) __PYX_ERR(9, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("glycopeptidepy._c.structure.modification.modification"); if (unlikely(!__pyx_t_1)) __PYX_ERR(10, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_12modification_ModificationInstanceBase = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.modification.modification", "ModificationInstanceBase", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_12modification_ModificationInstanceBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_12modification_ModificationInstanceBase) __PYX_ERR(10, 5, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_12modification_12modification_ModificationInstanceBase = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_12modification_12modification_ModificationInstanceBase*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_12modification_ModificationInstanceBase->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_12modification_12modification_ModificationInstanceBase)) __PYX_ERR(10, 5, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("glycopeptidepy._c.structure.constants"); if (unlikely(!__pyx_t_1)) __PYX_ERR(11, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14glycopeptidepy_2_c_9structure_9constants_Configuration = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.constants", "Configuration", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_9constants_Configuration), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_9constants_Configuration) __PYX_ERR(11, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("glycopeptidepy._c.structure.glycan"); if (unlikely(!__pyx_t_1)) __PYX_ERR(12, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_14glycopeptidepy_2_c_9structure_6glycan_GlycanCompositionWithOffsetProxyBase = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.glycan", "GlycanCompositionWithOffsetProxyBase", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycanCompositionWithOffsetProxyBase), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_6glycan_GlycanCompositionWithOffsetProxyBase) __PYX_ERR(12, 5, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager = __Pyx_ImportType(__pyx_t_1, "glycopeptidepy._c.structure.glycan", "GlycosylationManager", sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager) __PYX_ERR(12, 11, __pyx_L1_error)
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager = (struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager*)__Pyx_GetVtable(__pyx_ptype_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager->tp_dict); if (unlikely(!__pyx_vtabptr_14glycopeptidepy_2_c_9structure_6glycan_GlycosylationManager)) __PYX_ERR(12, 11, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __pyx_t_1 = PyImport_ImportModule("glypy.composition.ccomposition"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_atom", (void **)&__pyx_vp_5glypy_11composition_12ccomposition__atom, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_formula", (void **)&__pyx_vp_5glypy_11composition_12ccomposition__formula, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_isotope_string", (void **)&__pyx_vp_5glypy_11composition_12ccomposition__isotope_string, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "isotope_pattern", (void **)&__pyx_vp_5glypy_11composition_12ccomposition_isotope_pattern, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "formula_pattern", (void **)&__pyx_vp_5glypy_11composition_12ccomposition_formula_pattern, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule("glypy.utils.cenum"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "QMARK", (void **)&__pyx_vp_5glypy_5utils_5cenum_QMARK, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initsequence_methods(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initsequence_methods(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_sequence_methods(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_sequence_methods(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_sequence_methods(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'sequence_methods' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_sequence_methods(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("sequence_methods", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_glycopeptidepy___c__structure__sequence_methods) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "glycopeptidepy._c.structure.sequence_methods")) {
      if (unlikely(PyDict_SetItemString(modules, "glycopeptidepy._c.structure.sequence_methods", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  if (unlikely(__Pyx_modinit_function_export_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_variable_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":18
 * from glycopeptidepy._c.structure.modification.modification cimport ModificationInstanceBase
 * 
 * from glycopeptidepy._c.parser import sequence_tokenizer             # <<<<<<<<<<<<<<
 * 
 * from glypy.composition.ccomposition cimport CComposition
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_sequence_tokenizer);
  __Pyx_GIVEREF(__pyx_n_s_sequence_tokenizer);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_sequence_tokenizer);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_glycopeptidepy__c_parser, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_sequence_tokenizer); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_sequence_tokenizer, __pyx_t_1) < 0) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":25
 * from glycopeptidepy._c.structure.glycan cimport GlycosylationManager
 * 
 * from glycopeptidepy.structure.residue import AminoAcidResidue             # <<<<<<<<<<<<<<
 * from glycopeptidepy.structure.modification import Modification, ModificationCategory, SequenceLocation
 * from glycopeptidepy.structure import constants as _structure_constants
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_AminoAcidResidue);
  __Pyx_GIVEREF(__pyx_n_s_AminoAcidResidue);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_AminoAcidResidue);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_glycopeptidepy_structure_residue, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_AminoAcidResidue); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_AminoAcidResidue, __pyx_t_2) < 0) __PYX_ERR(0, 25, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":26
 * 
 * from glycopeptidepy.structure.residue import AminoAcidResidue
 * from glycopeptidepy.structure.modification import Modification, ModificationCategory, SequenceLocation             # <<<<<<<<<<<<<<
 * from glycopeptidepy.structure import constants as _structure_constants
 * 
 */
  __pyx_t_1 = PyList_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_Modification);
  __Pyx_GIVEREF(__pyx_n_s_Modification);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_Modification);
  __Pyx_INCREF(__pyx_n_s_ModificationCategory);
  __Pyx_GIVEREF(__pyx_n_s_ModificationCategory);
  PyList_SET_ITEM(__pyx_t_1, 1, __pyx_n_s_ModificationCategory);
  __Pyx_INCREF(__pyx_n_s_SequenceLocation);
  __Pyx_GIVEREF(__pyx_n_s_SequenceLocation);
  PyList_SET_ITEM(__pyx_t_1, 2, __pyx_n_s_SequenceLocation);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_glycopeptidepy_structure_modific, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_Modification); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_Modification, __pyx_t_1) < 0) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_ModificationCategory); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_ModificationCategory, __pyx_t_1) < 0) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_SequenceLocation); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_SequenceLocation, __pyx_t_1) < 0) __PYX_ERR(0, 26, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":27
 * from glycopeptidepy.structure.residue import AminoAcidResidue
 * from glycopeptidepy.structure.modification import Modification, ModificationCategory, SequenceLocation
 * from glycopeptidepy.structure import constants as _structure_constants             # <<<<<<<<<<<<<<
 * 
 * cdef CComposition WATER = CComposition("H2O")
 */
  __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_n_s_constants);
  __Pyx_GIVEREF(__pyx_n_s_constants);
  PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_constants);
  __pyx_t_1 = __Pyx_Import(__pyx_n_s_glycopeptidepy_structure, __pyx_t_2, -1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_1, __pyx_n_s_constants); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_structure_constants, __pyx_t_2) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":29
 * from glycopeptidepy.structure import constants as _structure_constants
 * 
 * cdef CComposition WATER = CComposition("H2O")             # <<<<<<<<<<<<<<
 * 
 * cdef Configuration structure_constants = _structure_constants
 */
  __pyx_t_1 = __Pyx_PyObject_Call(((PyObject *)__pyx_ptype_5glypy_11composition_12ccomposition_CComposition), __pyx_tuple__3, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 29, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XGOTREF(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_WATER));
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_WATER, ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1));
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":31
 * cdef CComposition WATER = CComposition("H2O")
 * 
 * cdef Configuration structure_constants = _structure_constants             # <<<<<<<<<<<<<<
 * 
 * cdef object ModificationImpl = Modification
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_structure_constants); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_14glycopeptidepy_2_c_9structure_9constants_Configuration))))) __PYX_ERR(0, 31, __pyx_L1_error)
  __Pyx_XGOTREF(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_structure_constants));
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_structure_constants, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_9constants_Configuration *)__pyx_t_1));
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":33
 * cdef Configuration structure_constants = _structure_constants
 * 
 * cdef object ModificationImpl = Modification             # <<<<<<<<<<<<<<
 * cdef object AminoAcidResidueImpl = AminoAcidResidue
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_Modification); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 33, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationImpl);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationImpl, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":34
 * 
 * cdef object ModificationImpl = Modification
 * cdef object AminoAcidResidueImpl = AminoAcidResidue             # <<<<<<<<<<<<<<
 * 
 * cdef object SequenceLocation_n_term = SequenceLocation.n_term
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_AminoAcidResidue); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 34, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_AminoAcidResidueImpl);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_AminoAcidResidueImpl, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":36
 * cdef object AminoAcidResidueImpl = AminoAcidResidue
 * 
 * cdef object SequenceLocation_n_term = SequenceLocation.n_term             # <<<<<<<<<<<<<<
 * cdef object SequenceLocation_c_term = SequenceLocation.c_term
 * 
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_SequenceLocation); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_n_term); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 36, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_SequenceLocation_n_term);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_SequenceLocation_n_term, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":37
 * 
 * cdef object SequenceLocation_n_term = SequenceLocation.n_term
 * cdef object SequenceLocation_c_term = SequenceLocation.c_term             # <<<<<<<<<<<<<<
 * 
 * cdef object ModificationCategory_glycosylation = ModificationCategory.glycosylation
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_SequenceLocation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_t_2, __pyx_n_s_c_term); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 37, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_SequenceLocation_c_term);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_SequenceLocation_c_term, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":39
 * cdef object SequenceLocation_c_term = SequenceLocation.c_term
 * 
 * cdef object ModificationCategory_glycosylation = ModificationCategory.glycosylation             # <<<<<<<<<<<<<<
 * 
 * cdef ModificationTableBase modification_table = ModificationImpl.modification_table
 */
  __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_ModificationCategory); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_t_1, __pyx_n_s_glycosylation); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationCategory_glycosylation);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationCategory_glycosylation, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":41
 * cdef object ModificationCategory_glycosylation = ModificationCategory.glycosylation
 * 
 * cdef ModificationTableBase modification_table = ModificationImpl.modification_table             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_ModificationImpl, __pyx_n_s_modification_table); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase))))) __PYX_ERR(0, 41, __pyx_L1_error)
  __Pyx_XGOTREF(((PyObject *)__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_modification_table));
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_modification_table, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_12modification_6source_ModificationTableBase *)__pyx_t_2));
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":54
 * 
 * 
 * cdef dict terminal_group_cache = dict()             # <<<<<<<<<<<<<<
 * 
 * # Helpers and Caches -----------------
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 54, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_terminal_group_cache);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_terminal_group_cache, ((PyObject*)__pyx_t_2));
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":75
 * 
 * 
 * cdef dict amino_acid_cache = dict()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 75, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_amino_acid_cache);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_amino_acid_cache, ((PyObject*)__pyx_t_2));
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":103
 * 
 * 
 * cdef dict implicit_terminal_cache = dict()             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 103, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_implicit_terminal_cache);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_16sequence_methods_implicit_terminal_cache, ((PyObject*)__pyx_t_2));
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":618
 * 
 * @cython.binding(True)
 * cpdef add_modification(_PeptideSequenceCore self, position, modification_type):             # <<<<<<<<<<<<<<
 *     '''
 *     Add a modification by name to a specific residue. If the
 */
  __pyx_t_2 = __Pyx_CyFunction_New(&__pyx_mdef_14glycopeptidepy_2_c_9structure_16sequence_methods_3add_modification, 0, __pyx_n_s_add_modification, NULL, __pyx_n_s_glycopeptidepy__c_structure_sequ_2, __pyx_d, ((PyObject *)__pyx_codeobj__5)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_add_modification, __pyx_t_2) < 0) __PYX_ERR(0, 618, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle__PeptideSequenceCore(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_14glycopeptidepy_2_c_9structure_16sequence_methods_5__pyx_unpickle__PeptideSequenceCore, NULL, __pyx_n_s_glycopeptidepy__c_structure_sequ_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle__PeptideSequenceC, __pyx_t_2) < 0) __PYX_ERR(1, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/sequence_methods.pyx":1
 * cimport cython             # <<<<<<<<<<<<<<
 * 
 * from cpython.ref cimport Py_INCREF
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init glycopeptidepy._c.structure.sequence_methods", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init glycopeptidepy._c.structure.sequence_methods");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* SetItemInt */
static int __Pyx_SetItemInt_Generic(PyObject *o, PyObject *j, PyObject *v) {
    int r;
    if (!j) return -1;
    r = PyObject_SetItem(o, j, v);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE int __Pyx_SetItemInt_Fast(PyObject *o, Py_ssize_t i, PyObject *v, int is_list,
                                               CYTHON_NCP_UNUSED int wraparound, CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = (!wraparound) ? i : ((likely(i >= 0)) ? i : i + PyList_GET_SIZE(o));
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o)))) {
            PyObject* old = PyList_GET_ITEM(o, n);
            Py_INCREF(v);
            PyList_SET_ITEM(o, n, v);
            Py_DECREF(old);
            return 1;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_ass_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return -1;
                    PyErr_Clear();
                }
            }
            return m->sq_ass_item(o, i, v);
        }
    }
#else
#if CYTHON_COMPILING_IN_PYPY
    if (is_list || (PySequence_Check(o) && !PyDict_Check(o)))
#else
    if (is_list || PySequence_Check(o))
#endif
    {
        return PySequence_SetItem(o, i, v);
    }
#endif
    return __Pyx_SetItemInt_Generic(o, PyInt_FromSsize_t(i), v);
}

/* RaiseTooManyValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseTooManyValuesError(Py_ssize_t expected) {
    PyErr_Format(PyExc_ValueError,
                 "too many values to unpack (expected %" CYTHON_FORMAT_SSIZE_T "d)", expected);
}

/* RaiseNeedMoreValuesToUnpack */
static CYTHON_INLINE void __Pyx_RaiseNeedMoreValuesError(Py_ssize_t index) {
    PyErr_Format(PyExc_ValueError,
                 "need more than %" CYTHON_FORMAT_SSIZE_T "d value%.1s to unpack",
                 index, (index == 1) ? "" : "s");
}

/* RaiseNoneIterError */
static CYTHON_INLINE void __Pyx_RaiseNoneNotIterableError(void) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not iterable");
}

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* ObjectGetItem */
#if CYTHON_USE_TYPE_SLOTS
static PyObject *__Pyx_PyObject_GetIndex(PyObject *obj, PyObject* index) {
    PyObject *runerr;
    Py_ssize_t key_value;
    PySequenceMethods *m = Py_TYPE(obj)->tp_as_sequence;
    if (unlikely(!(m && m->sq_item))) {
        PyErr_Format(PyExc_TypeError, "'%.200s' object is not subscriptable", Py_TYPE(obj)->tp_name);
        return NULL;
    }
    key_value = __Pyx_PyIndex_AsSsize_t(index);
    if (likely(key_value != -1 || !(runerr = PyErr_Occurred()))) {
        return __Pyx_GetItemInt_Fast(obj, key_value, 0, 1, 1);
    }
    if (PyErr_GivenExceptionMatches(runerr, PyExc_OverflowError)) {
        PyErr_Clear();
        PyErr_Format(PyExc_IndexError, "cannot fit '%.200s' into an index-sized integer", Py_TYPE(index)->tp_name);
    }
    return NULL;
}
static PyObject *__Pyx_PyObject_GetItem(PyObject *obj, PyObject* key) {
    PyMappingMethods *m = Py_TYPE(obj)->tp_as_mapping;
    if (likely(m && m->mp_subscript)) {
        return m->mp_subscript(obj, key);
    }
    return __Pyx_PyObject_GetIndex(obj, key);
}
#endif

/* StringJoin */
#if !CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyBytes_Join(PyObject* sep, PyObject* values) {
    return PyObject_CallMethodObjArgs(sep, __pyx_n_s_join, values, NULL);
}
#endif

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* CallNextTpDealloc */
static void __Pyx_call_next_tp_dealloc(PyObject* obj, destructor current_tp_dealloc) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_dealloc != current_tp_dealloc)
        type = type->tp_base;
    while (type && type->tp_dealloc == current_tp_dealloc)
        type = type->tp_base;
    if (type)
        type->tp_dealloc(obj);
}

/* CallNextTpTraverse */
static int __Pyx_call_next_tp_traverse(PyObject* obj, visitproc v, void *a, traverseproc current_tp_traverse) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_traverse != current_tp_traverse)
        type = type->tp_base;
    while (type && type->tp_traverse == current_tp_traverse)
        type = type->tp_base;
    if (type && type->tp_traverse)
        return type->tp_traverse(obj, v, a);
    return 0;
}

/* CallNextTpClear */
static void __Pyx_call_next_tp_clear(PyObject* obj, inquiry current_tp_clear) {
    PyTypeObject* type = Py_TYPE(obj);
    while (type && type->tp_clear != current_tp_clear)
        type = type->tp_base;
    while (type && type->tp_clear == current_tp_clear)
        type = type->tp_base;
    if (type && type->tp_clear)
        type->tp_clear(obj);
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* FetchCommonType */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* CythonFunctionShared */
#include <structmember.h>
static PyObject *
__Pyx_CyFunction_get_doc(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *closure)
{
    if (unlikely(op->func_doc == NULL)) {
        if (op->func.m_ml->ml_doc) {
#if PY_MAJOR_VERSION >= 3
            op->func_doc = PyUnicode_FromString(op->func.m_ml->ml_doc);
#else
            op->func_doc = PyString_FromString(op->func.m_ml->ml_doc);
#endif
            if (unlikely(op->func_doc == NULL))
                return NULL;
        } else {
            Py_INCREF(Py_None);
            return Py_None;
        }
    }
    Py_INCREF(op->func_doc);
    return op->func_doc;
}
static int
__Pyx_CyFunction_set_doc(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp = op->func_doc;
    if (value == NULL) {
        value = Py_None;
    }
    Py_INCREF(value);
    op->func_doc = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_name(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_name == NULL)) {
#if PY_MAJOR_VERSION >= 3
        op->func_name = PyUnicode_InternFromString(op->func.m_ml->ml_name);
#else
        op->func_name = PyString_InternFromString(op->func.m_ml->ml_name);
#endif
        if (unlikely(op->func_name == NULL))
            return NULL;
    }
    Py_INCREF(op->func_name);
    return op->func_name;
}
static int
__Pyx_CyFunction_set_name(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = op->func_name;
    Py_INCREF(value);
    op->func_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_qualname(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_qualname);
    return op->func_qualname;
}
static int
__Pyx_CyFunction_set_qualname(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = op->func_qualname;
    Py_INCREF(value);
    op->func_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_self(__pyx_CyFunctionObject *m, CYTHON_UNUSED void *closure)
{
    PyObject *self;
    self = m->func_closure;
    if (self == NULL)
        self = Py_None;
    Py_INCREF(self);
    return self;
}
static PyObject *
__Pyx_CyFunction_get_dict(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    if (unlikely(op->func_dict == NULL)) {
        op->func_dict = PyDict_New();
        if (unlikely(op->func_dict == NULL))
            return NULL;
    }
    Py_INCREF(op->func_dict);
    return op->func_dict;
}
static int
__Pyx_CyFunction_set_dict(__pyx_CyFunctionObject *op, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
    if (unlikely(value == NULL)) {
        PyErr_SetString(PyExc_TypeError,
               "function's dictionary may not be deleted");
        return -1;
    }
    if (unlikely(!PyDict_Check(value))) {
        PyErr_SetString(PyExc_TypeError,
               "setting function's dictionary to a non-dict");
        return -1;
    }
    tmp = op->func_dict;
    Py_INCREF(value);
    op->func_dict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_globals(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(op->func_globals);
    return op->func_globals;
}
static PyObject *
__Pyx_CyFunction_get_closure(CYTHON_UNUSED __pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    Py_INCREF(Py_None);
    return Py_None;
}
static PyObject *
__Pyx_CyFunction_get_code(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context)
{
    PyObject* result = (op->func_code) ? op->func_code : Py_None;
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_init_defaults(__pyx_CyFunctionObject *op) {
    int result = 0;
    PyObject *res = op->defaults_getter((PyObject *) op);
    if (unlikely(!res))
        return -1;
    #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    op->defaults_tuple = PyTuple_GET_ITEM(res, 0);
    Py_INCREF(op->defaults_tuple);
    op->defaults_kwdict = PyTuple_GET_ITEM(res, 1);
    Py_INCREF(op->defaults_kwdict);
    #else
    op->defaults_tuple = PySequence_ITEM(res, 0);
    if (unlikely(!op->defaults_tuple)) result = -1;
    else {
        op->defaults_kwdict = PySequence_ITEM(res, 1);
        if (unlikely(!op->defaults_kwdict)) result = -1;
    }
    #endif
    Py_DECREF(res);
    return result;
}
static int
__Pyx_CyFunction_set_defaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyTuple_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__defaults__ must be set to a tuple object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_tuple;
    op->defaults_tuple = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_defaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_tuple;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_tuple;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_kwdefaults(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value) {
        value = Py_None;
    } else if (value != Py_None && !PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__kwdefaults__ must be set to a dict object");
        return -1;
    }
    Py_INCREF(value);
    tmp = op->defaults_kwdict;
    op->defaults_kwdict = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_kwdefaults(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->defaults_kwdict;
    if (unlikely(!result)) {
        if (op->defaults_getter) {
            if (__Pyx_CyFunction_init_defaults(op) < 0) return NULL;
            result = op->defaults_kwdict;
        } else {
            result = Py_None;
        }
    }
    Py_INCREF(result);
    return result;
}
static int
__Pyx_CyFunction_set_annotations(__pyx_CyFunctionObject *op, PyObject* value, CYTHON_UNUSED void *context) {
    PyObject* tmp;
    if (!value || value == Py_None) {
        value = NULL;
    } else if (!PyDict_Check(value)) {
        PyErr_SetString(PyExc_TypeError,
                        "__annotations__ must be set to a dict object");
        return -1;
    }
    Py_XINCREF(value);
    tmp = op->func_annotations;
    op->func_annotations = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_CyFunction_get_annotations(__pyx_CyFunctionObject *op, CYTHON_UNUSED void *context) {
    PyObject* result = op->func_annotations;
    if (unlikely(!result)) {
        result = PyDict_New();
        if (unlikely(!result)) return NULL;
        op->func_annotations = result;
    }
    Py_INCREF(result);
    return result;
}
static PyGetSetDef __pyx_CyFunction_getsets[] = {
    {(char *) "func_doc", (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "__doc__",  (getter)__Pyx_CyFunction_get_doc, (setter)__Pyx_CyFunction_set_doc, 0, 0},
    {(char *) "func_name", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__name__", (getter)__Pyx_CyFunction_get_name, (setter)__Pyx_CyFunction_set_name, 0, 0},
    {(char *) "__qualname__", (getter)__Pyx_CyFunction_get_qualname, (setter)__Pyx_CyFunction_set_qualname, 0, 0},
    {(char *) "__self__", (getter)__Pyx_CyFunction_get_self, 0, 0, 0},
    {(char *) "func_dict", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "__dict__", (getter)__Pyx_CyFunction_get_dict, (setter)__Pyx_CyFunction_set_dict, 0, 0},
    {(char *) "func_globals", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "__globals__", (getter)__Pyx_CyFunction_get_globals, 0, 0, 0},
    {(char *) "func_closure", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "__closure__", (getter)__Pyx_CyFunction_get_closure, 0, 0, 0},
    {(char *) "func_code", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "__code__", (getter)__Pyx_CyFunction_get_code, 0, 0, 0},
    {(char *) "func_defaults", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__defaults__", (getter)__Pyx_CyFunction_get_defaults, (setter)__Pyx_CyFunction_set_defaults, 0, 0},
    {(char *) "__kwdefaults__", (getter)__Pyx_CyFunction_get_kwdefaults, (setter)__Pyx_CyFunction_set_kwdefaults, 0, 0},
    {(char *) "__annotations__", (getter)__Pyx_CyFunction_get_annotations, (setter)__Pyx_CyFunction_set_annotations, 0, 0},
    {0, 0, 0, 0, 0}
};
static PyMemberDef __pyx_CyFunction_members[] = {
    {(char *) "__module__", T_OBJECT, offsetof(PyCFunctionObject, m_module), PY_WRITE_RESTRICTED, 0},
    {0, 0, 0,  0, 0}
};
static PyObject *
__Pyx_CyFunction_reduce(__pyx_CyFunctionObject *m, CYTHON_UNUSED PyObject *args)
{
#if PY_MAJOR_VERSION >= 3
    Py_INCREF(m->func_qualname);
    return m->func_qualname;
#else
    return PyString_FromString(m->func.m_ml->ml_name);
#endif
}
static PyMethodDef __pyx_CyFunction_methods[] = {
    {"__reduce__", (PyCFunction)__Pyx_CyFunction_reduce, METH_VARARGS, 0},
    {0, 0, 0, 0}
};
#if PY_VERSION_HEX < 0x030500A0
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func_weakreflist)
#else
#define __Pyx_CyFunction_weakreflist(cyfunc) ((cyfunc)->func.m_weakreflist)
#endif
static PyObject *__Pyx_CyFunction_Init(__pyx_CyFunctionObject *op, PyMethodDef *ml, int flags, PyObject* qualname,
                                       PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    if (unlikely(op == NULL))
        return NULL;
    op->flags = flags;
    __Pyx_CyFunction_weakreflist(op) = NULL;
    op->func.m_ml = ml;
    op->func.m_self = (PyObject *) op;
    Py_XINCREF(closure);
    op->func_closure = closure;
    Py_XINCREF(module);
    op->func.m_module = module;
    op->func_dict = NULL;
    op->func_name = NULL;
    Py_INCREF(qualname);
    op->func_qualname = qualname;
    op->func_doc = NULL;
    op->func_classobj = NULL;
    op->func_globals = globals;
    Py_INCREF(op->func_globals);
    Py_XINCREF(code);
    op->func_code = code;
    op->defaults_pyobjects = 0;
    op->defaults_size = 0;
    op->defaults = NULL;
    op->defaults_tuple = NULL;
    op->defaults_kwdict = NULL;
    op->defaults_getter = NULL;
    op->func_annotations = NULL;
    return (PyObject *) op;
}
static int
__Pyx_CyFunction_clear(__pyx_CyFunctionObject *m)
{
    Py_CLEAR(m->func_closure);
    Py_CLEAR(m->func.m_module);
    Py_CLEAR(m->func_dict);
    Py_CLEAR(m->func_name);
    Py_CLEAR(m->func_qualname);
    Py_CLEAR(m->func_doc);
    Py_CLEAR(m->func_globals);
    Py_CLEAR(m->func_code);
    Py_CLEAR(m->func_classobj);
    Py_CLEAR(m->defaults_tuple);
    Py_CLEAR(m->defaults_kwdict);
    Py_CLEAR(m->func_annotations);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_XDECREF(pydefaults[i]);
        PyObject_Free(m->defaults);
        m->defaults = NULL;
    }
    return 0;
}
static void __Pyx__CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    if (__Pyx_CyFunction_weakreflist(m) != NULL)
        PyObject_ClearWeakRefs((PyObject *) m);
    __Pyx_CyFunction_clear(m);
    PyObject_GC_Del(m);
}
static void __Pyx_CyFunction_dealloc(__pyx_CyFunctionObject *m)
{
    PyObject_GC_UnTrack(m);
    __Pyx__CyFunction_dealloc(m);
}
static int __Pyx_CyFunction_traverse(__pyx_CyFunctionObject *m, visitproc visit, void *arg)
{
    Py_VISIT(m->func_closure);
    Py_VISIT(m->func.m_module);
    Py_VISIT(m->func_dict);
    Py_VISIT(m->func_name);
    Py_VISIT(m->func_qualname);
    Py_VISIT(m->func_doc);
    Py_VISIT(m->func_globals);
    Py_VISIT(m->func_code);
    Py_VISIT(m->func_classobj);
    Py_VISIT(m->defaults_tuple);
    Py_VISIT(m->defaults_kwdict);
    if (m->defaults) {
        PyObject **pydefaults = __Pyx_CyFunction_Defaults(PyObject *, m);
        int i;
        for (i = 0; i < m->defaults_pyobjects; i++)
            Py_VISIT(pydefaults[i]);
    }
    return 0;
}
static PyObject *__Pyx_CyFunction_descr_get(PyObject *func, PyObject *obj, PyObject *type)
{
#if PY_MAJOR_VERSION < 3
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    if (m->flags & __Pyx_CYFUNCTION_STATICMETHOD) {
        Py_INCREF(func);
        return func;
    }
    if (m->flags & __Pyx_CYFUNCTION_CLASSMETHOD) {
        if (type == NULL)
            type = (PyObject *)(Py_TYPE(obj));
        return __Pyx_PyMethod_New(func, type, (PyObject *)(Py_TYPE(type)));
    }
    if (obj == Py_None)
        obj = NULL;
#endif
    return __Pyx_PyMethod_New(func, obj, type);
}
static PyObject*
__Pyx_CyFunction_repr(__pyx_CyFunctionObject *op)
{
#if PY_MAJOR_VERSION >= 3
    return PyUnicode_FromFormat("<cyfunction %U at %p>",
                                op->func_qualname, (void *)op);
#else
    return PyString_FromFormat("<cyfunction %s at %p>",
                               PyString_AsString(op->func_qualname), (void *)op);
#endif
}
static PyObject * __Pyx_CyFunction_CallMethod(PyObject *func, PyObject *self, PyObject *arg, PyObject *kw) {
    PyCFunctionObject* f = (PyCFunctionObject*)func;
    PyCFunction meth = f->m_ml->ml_meth;
    Py_ssize_t size;
    switch (f->m_ml->ml_flags & (METH_VARARGS | METH_KEYWORDS | METH_NOARGS | METH_O)) {
    case METH_VARARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0))
            return (*meth)(self, arg);
        break;
    case METH_VARARGS | METH_KEYWORDS:
        return (*(PyCFunctionWithKeywords)(void*)meth)(self, arg, kw);
    case METH_NOARGS:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 0))
                return (*meth)(self, NULL);
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes no arguments (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    case METH_O:
        if (likely(kw == NULL || PyDict_Size(kw) == 0)) {
            size = PyTuple_GET_SIZE(arg);
            if (likely(size == 1)) {
                PyObject *result, *arg0;
                #if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                arg0 = PyTuple_GET_ITEM(arg, 0);
                #else
                arg0 = PySequence_ITEM(arg, 0); if (unlikely(!arg0)) return NULL;
                #endif
                result = (*meth)(self, arg0);
                #if !(CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS)
                Py_DECREF(arg0);
                #endif
                return result;
            }
            PyErr_Format(PyExc_TypeError,
                "%.200s() takes exactly one argument (%" CYTHON_FORMAT_SSIZE_T "d given)",
                f->m_ml->ml_name, size);
            return NULL;
        }
        break;
    default:
        PyErr_SetString(PyExc_SystemError, "Bad call flags in "
                        "__Pyx_CyFunction_Call. METH_OLDARGS is no "
                        "longer supported!");
        return NULL;
    }
    PyErr_Format(PyExc_TypeError, "%.200s() takes no keyword arguments",
                 f->m_ml->ml_name);
    return NULL;
}
static CYTHON_INLINE PyObject *__Pyx_CyFunction_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    return __Pyx_CyFunction_CallMethod(func, ((PyCFunctionObject*)func)->m_self, arg, kw);
}
static PyObject *__Pyx_CyFunction_CallAsMethod(PyObject *func, PyObject *args, PyObject *kw) {
    PyObject *result;
    __pyx_CyFunctionObject *cyfunc = (__pyx_CyFunctionObject *) func;
    if ((cyfunc->flags & __Pyx_CYFUNCTION_CCLASS) && !(cyfunc->flags & __Pyx_CYFUNCTION_STATICMETHOD)) {
        Py_ssize_t argc;
        PyObject *new_args;
        PyObject *self;
        argc = PyTuple_GET_SIZE(args);
        new_args = PyTuple_GetSlice(args, 1, argc);
        if (unlikely(!new_args))
            return NULL;
        self = PyTuple_GetItem(args, 0);
        if (unlikely(!self)) {
            Py_DECREF(new_args);
            return NULL;
        }
        result = __Pyx_CyFunction_CallMethod(func, self, new_args, kw);
        Py_DECREF(new_args);
    } else {
        result = __Pyx_CyFunction_Call(func, args, kw);
    }
    return result;
}
static PyTypeObject __pyx_CyFunctionType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "cython_function_or_method",
    sizeof(__pyx_CyFunctionObject),
    0,
    (destructor) __Pyx_CyFunction_dealloc,
    0,
    0,
    0,
#if PY_MAJOR_VERSION < 3
    0,
#else
    0,
#endif
    (reprfunc) __Pyx_CyFunction_repr,
    0,
    0,
    0,
    0,
    __Pyx_CyFunction_CallAsMethod,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC,
    0,
    (traverseproc) __Pyx_CyFunction_traverse,
    (inquiry) __Pyx_CyFunction_clear,
    0,
#if PY_VERSION_HEX < 0x030500A0
    offsetof(__pyx_CyFunctionObject, func_weakreflist),
#else
    offsetof(PyCFunctionObject, m_weakreflist),
#endif
    0,
    0,
    __pyx_CyFunction_methods,
    __pyx_CyFunction_members,
    __pyx_CyFunction_getsets,
    0,
    0,
    __Pyx_CyFunction_descr_get,
    0,
    offsetof(__pyx_CyFunctionObject, func_dict),
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
};
static int __pyx_CyFunction_init(void) {
    __pyx_CyFunctionType = __Pyx_FetchCommonType(&__pyx_CyFunctionType_type);
    if (unlikely(__pyx_CyFunctionType == NULL)) {
        return -1;
    }
    return 0;
}
static CYTHON_INLINE void *__Pyx_CyFunction_InitDefaults(PyObject *func, size_t size, int pyobjects) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults = PyObject_Malloc(size);
    if (unlikely(!m->defaults))
        return PyErr_NoMemory();
    memset(m->defaults, 0, size);
    m->defaults_pyobjects = pyobjects;
    m->defaults_size = size;
    return m->defaults;
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsTuple(PyObject *func, PyObject *tuple) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_tuple = tuple;
    Py_INCREF(tuple);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetDefaultsKwDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->defaults_kwdict = dict;
    Py_INCREF(dict);
}
static CYTHON_INLINE void __Pyx_CyFunction_SetAnnotationsDict(PyObject *func, PyObject *dict) {
    __pyx_CyFunctionObject *m = (__pyx_CyFunctionObject *) func;
    m->func_annotations = dict;
    Py_INCREF(dict);
}

/* CythonFunction */
static PyObject *__Pyx_CyFunction_New(PyMethodDef *ml, int flags, PyObject* qualname,
                                      PyObject *closure, PyObject *module, PyObject* globals, PyObject* code) {
    PyObject *op = __Pyx_CyFunction_Init(
        PyObject_GC_New(__pyx_CyFunctionObject, __pyx_CyFunctionType),
        ml, flags, qualname, closure, module, globals, code
    );
    if (likely(op)) {
        PyObject_GC_Track(op);
    }
    return op;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* FunctionExport */
static int __Pyx_ExportFunction(const char *name, void (*f)(void), const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    union {
        void (*fp)(void);
        void *p;
    } tmp;
    d = PyObject_GetAttrString(__pyx_m, (char *)"__pyx_capi__");
    if (!d) {
        PyErr_Clear();
        d = PyDict_New();
        if (!d)
            goto bad;
        Py_INCREF(d);
        if (PyModule_AddObject(__pyx_m, (char *)"__pyx_capi__", d) < 0)
            goto bad;
    }
    tmp.fp = f;
#if PY_VERSION_HEX >= 0x02070000
    cobj = PyCapsule_New(tmp.p, sig, 0);
#else
    cobj = PyCObject_FromVoidPtrAndDesc(tmp.p, (void *)sig, 0);
#endif
    if (!cobj)
        goto bad;
    if (PyDict_SetItemString(d, name, cobj) < 0)
        goto bad;
    Py_DECREF(cobj);
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(cobj);
    Py_XDECREF(d);
    return -1;
}

/* VoidPtrImport */
#ifndef __PYX_HAVE_RT_ImportVoidPtr
#define __PYX_HAVE_RT_ImportVoidPtr
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, desc);
        goto bad;
    }
    *p = PyCObject_AsVoidPtr(cobj);}
#endif
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
