/* Generated by Cython 0.29.24 */

/* BEGIN: Cython Metadata
{
    "distutils": {
        "depends": [],
        "name": "glycopeptidepy._c.structure.base",
        "sources": [
            "glycopeptidepy/_c/structure/base.pyx"
        ]
    },
    "module_name": "glycopeptidepy._c.structure.base"
}
END: Cython Metadata */

#ifndef PY_SSIZE_T_CLEAN
#define PY_SSIZE_T_CLEAN
#endif /* PY_SSIZE_T_CLEAN */
#include "Python.h"
#ifndef Py_PYTHON_H
    #error Python headers needed to compile C extensions, please install development version of Python.
#elif PY_VERSION_HEX < 0x02060000 || (0x03000000 <= PY_VERSION_HEX && PY_VERSION_HEX < 0x03030000)
    #error Cython requires Python 2.6+ or Python 3.3+.
#else
#define CYTHON_ABI "0_29_24"
#define CYTHON_HEX_VERSION 0x001D18F0
#define CYTHON_FUTURE_DIVISION 0
#include <stddef.h>
#ifndef offsetof
  #define offsetof(type, member) ( (size_t) & ((type*)0) -> member )
#endif
#if !defined(WIN32) && !defined(MS_WINDOWS)
  #ifndef __stdcall
    #define __stdcall
  #endif
  #ifndef __cdecl
    #define __cdecl
  #endif
  #ifndef __fastcall
    #define __fastcall
  #endif
#endif
#ifndef DL_IMPORT
  #define DL_IMPORT(t) t
#endif
#ifndef DL_EXPORT
  #define DL_EXPORT(t) t
#endif
#define __PYX_COMMA ,
#ifndef HAVE_LONG_LONG
  #if PY_VERSION_HEX >= 0x02070000
    #define HAVE_LONG_LONG
  #endif
#endif
#ifndef PY_LONG_LONG
  #define PY_LONG_LONG LONG_LONG
#endif
#ifndef Py_HUGE_VAL
  #define Py_HUGE_VAL HUGE_VAL
#endif
#ifdef PYPY_VERSION
  #define CYTHON_COMPILING_IN_PYPY 1
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #undef CYTHON_USE_TYPE_SLOTS
  #define CYTHON_USE_TYPE_SLOTS 0
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #if PY_VERSION_HEX < 0x03050000
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #undef CYTHON_USE_UNICODE_INTERNALS
  #define CYTHON_USE_UNICODE_INTERNALS 0
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #undef CYTHON_AVOID_BORROWED_REFS
  #define CYTHON_AVOID_BORROWED_REFS 1
  #undef CYTHON_ASSUME_SAFE_MACROS
  #define CYTHON_ASSUME_SAFE_MACROS 0
  #undef CYTHON_UNPACK_METHODS
  #define CYTHON_UNPACK_METHODS 0
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#elif defined(PYSTON_VERSION)
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 1
  #define CYTHON_COMPILING_IN_CPYTHON 0
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #undef CYTHON_USE_PYTYPE_LOOKUP
  #define CYTHON_USE_PYTYPE_LOOKUP 0
  #undef CYTHON_USE_ASYNC_SLOTS
  #define CYTHON_USE_ASYNC_SLOTS 0
  #undef CYTHON_USE_PYLIST_INTERNALS
  #define CYTHON_USE_PYLIST_INTERNALS 0
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #undef CYTHON_USE_UNICODE_WRITER
  #define CYTHON_USE_UNICODE_WRITER 0
  #undef CYTHON_USE_PYLONG_INTERNALS
  #define CYTHON_USE_PYLONG_INTERNALS 0
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #undef CYTHON_FAST_THREAD_STATE
  #define CYTHON_FAST_THREAD_STATE 0
  #undef CYTHON_FAST_PYCALL
  #define CYTHON_FAST_PYCALL 0
  #undef CYTHON_PEP489_MULTI_PHASE_INIT
  #define CYTHON_PEP489_MULTI_PHASE_INIT 0
  #undef CYTHON_USE_TP_FINALIZE
  #define CYTHON_USE_TP_FINALIZE 0
  #undef CYTHON_USE_DICT_VERSIONS
  #define CYTHON_USE_DICT_VERSIONS 0
  #undef CYTHON_USE_EXC_INFO_STACK
  #define CYTHON_USE_EXC_INFO_STACK 0
#else
  #define CYTHON_COMPILING_IN_PYPY 0
  #define CYTHON_COMPILING_IN_PYSTON 0
  #define CYTHON_COMPILING_IN_CPYTHON 1
  #ifndef CYTHON_USE_TYPE_SLOTS
    #define CYTHON_USE_TYPE_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYTYPE_LOOKUP
    #define CYTHON_USE_PYTYPE_LOOKUP 0
  #elif !defined(CYTHON_USE_PYTYPE_LOOKUP)
    #define CYTHON_USE_PYTYPE_LOOKUP 1
  #endif
  #if PY_MAJOR_VERSION < 3
    #undef CYTHON_USE_ASYNC_SLOTS
    #define CYTHON_USE_ASYNC_SLOTS 0
  #elif !defined(CYTHON_USE_ASYNC_SLOTS)
    #define CYTHON_USE_ASYNC_SLOTS 1
  #endif
  #if PY_VERSION_HEX < 0x02070000
    #undef CYTHON_USE_PYLONG_INTERNALS
    #define CYTHON_USE_PYLONG_INTERNALS 0
  #elif !defined(CYTHON_USE_PYLONG_INTERNALS)
    #define CYTHON_USE_PYLONG_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_PYLIST_INTERNALS
    #define CYTHON_USE_PYLIST_INTERNALS 1
  #endif
  #ifndef CYTHON_USE_UNICODE_INTERNALS
    #define CYTHON_USE_UNICODE_INTERNALS 1
  #endif
  #if PY_VERSION_HEX < 0x030300F0
    #undef CYTHON_USE_UNICODE_WRITER
    #define CYTHON_USE_UNICODE_WRITER 0
  #elif !defined(CYTHON_USE_UNICODE_WRITER)
    #define CYTHON_USE_UNICODE_WRITER 1
  #endif
  #ifndef CYTHON_AVOID_BORROWED_REFS
    #define CYTHON_AVOID_BORROWED_REFS 0
  #endif
  #ifndef CYTHON_ASSUME_SAFE_MACROS
    #define CYTHON_ASSUME_SAFE_MACROS 1
  #endif
  #ifndef CYTHON_UNPACK_METHODS
    #define CYTHON_UNPACK_METHODS 1
  #endif
  #ifndef CYTHON_FAST_THREAD_STATE
    #define CYTHON_FAST_THREAD_STATE 1
  #endif
  #ifndef CYTHON_FAST_PYCALL
    #define CYTHON_FAST_PYCALL 1
  #endif
  #ifndef CYTHON_PEP489_MULTI_PHASE_INIT
    #define CYTHON_PEP489_MULTI_PHASE_INIT (PY_VERSION_HEX >= 0x03050000)
  #endif
  #ifndef CYTHON_USE_TP_FINALIZE
    #define CYTHON_USE_TP_FINALIZE (PY_VERSION_HEX >= 0x030400a1)
  #endif
  #ifndef CYTHON_USE_DICT_VERSIONS
    #define CYTHON_USE_DICT_VERSIONS (PY_VERSION_HEX >= 0x030600B1)
  #endif
  #ifndef CYTHON_USE_EXC_INFO_STACK
    #define CYTHON_USE_EXC_INFO_STACK (PY_VERSION_HEX >= 0x030700A3)
  #endif
#endif
#if !defined(CYTHON_FAST_PYCCALL)
#define CYTHON_FAST_PYCCALL  (CYTHON_FAST_PYCALL && PY_VERSION_HEX >= 0x030600B1)
#endif
#if CYTHON_USE_PYLONG_INTERNALS
  #include "longintrepr.h"
  #undef SHIFT
  #undef BASE
  #undef MASK
  #ifdef SIZEOF_VOID_P
    enum { __pyx_check_sizeof_voidp = 1 / (int)(SIZEOF_VOID_P == sizeof(void*)) };
  #endif
#endif
#ifndef __has_attribute
  #define __has_attribute(x) 0
#endif
#ifndef __has_cpp_attribute
  #define __has_cpp_attribute(x) 0
#endif
#ifndef CYTHON_RESTRICT
  #if defined(__GNUC__)
    #define CYTHON_RESTRICT __restrict__
  #elif defined(_MSC_VER) && _MSC_VER >= 1400
    #define CYTHON_RESTRICT __restrict
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_RESTRICT restrict
  #else
    #define CYTHON_RESTRICT
  #endif
#endif
#ifndef CYTHON_UNUSED
# if defined(__GNUC__)
#   if !(defined(__cplusplus)) || (__GNUC__ > 3 || (__GNUC__ == 3 && __GNUC_MINOR__ >= 4))
#     define CYTHON_UNUSED __attribute__ ((__unused__))
#   else
#     define CYTHON_UNUSED
#   endif
# elif defined(__ICC) || (defined(__INTEL_COMPILER) && !defined(_MSC_VER))
#   define CYTHON_UNUSED __attribute__ ((__unused__))
# else
#   define CYTHON_UNUSED
# endif
#endif
#ifndef CYTHON_MAYBE_UNUSED_VAR
#  if defined(__cplusplus)
     template<class T> void CYTHON_MAYBE_UNUSED_VAR( const T& ) { }
#  else
#    define CYTHON_MAYBE_UNUSED_VAR(x) (void)(x)
#  endif
#endif
#ifndef CYTHON_NCP_UNUSED
# if CYTHON_COMPILING_IN_CPYTHON
#  define CYTHON_NCP_UNUSED
# else
#  define CYTHON_NCP_UNUSED CYTHON_UNUSED
# endif
#endif
#define __Pyx_void_to_None(void_result) ((void)(void_result), Py_INCREF(Py_None), Py_None)
#ifdef _MSC_VER
    #ifndef _MSC_STDINT_H_
        #if _MSC_VER < 1300
           typedef unsigned char     uint8_t;
           typedef unsigned int      uint32_t;
        #else
           typedef unsigned __int8   uint8_t;
           typedef unsigned __int32  uint32_t;
        #endif
    #endif
#else
   #include <stdint.h>
#endif
#ifndef CYTHON_FALLTHROUGH
  #if defined(__cplusplus) && __cplusplus >= 201103L
    #if __has_cpp_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH [[fallthrough]]
    #elif __has_cpp_attribute(clang::fallthrough)
      #define CYTHON_FALLTHROUGH [[clang::fallthrough]]
    #elif __has_cpp_attribute(gnu::fallthrough)
      #define CYTHON_FALLTHROUGH [[gnu::fallthrough]]
    #endif
  #endif
  #ifndef CYTHON_FALLTHROUGH
    #if __has_attribute(fallthrough)
      #define CYTHON_FALLTHROUGH __attribute__((fallthrough))
    #else
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
  #if defined(__clang__ ) && defined(__apple_build_version__)
    #if __apple_build_version__ < 7000000
      #undef  CYTHON_FALLTHROUGH
      #define CYTHON_FALLTHROUGH
    #endif
  #endif
#endif

#ifndef CYTHON_INLINE
  #if defined(__clang__)
    #define CYTHON_INLINE __inline__ __attribute__ ((__unused__))
  #elif defined(__GNUC__)
    #define CYTHON_INLINE __inline__
  #elif defined(_MSC_VER)
    #define CYTHON_INLINE __inline
  #elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define CYTHON_INLINE inline
  #else
    #define CYTHON_INLINE
  #endif
#endif

#if CYTHON_COMPILING_IN_PYPY && PY_VERSION_HEX < 0x02070600 && !defined(Py_OptimizeFlag)
  #define Py_OptimizeFlag 0
#endif
#define __PYX_BUILD_PY_SSIZE_T "n"
#define CYTHON_FORMAT_SSIZE_T "z"
#if PY_MAJOR_VERSION < 3
  #define __Pyx_BUILTIN_MODULE_NAME "__builtin__"
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a+k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
  #define __Pyx_DefaultClassType PyClass_Type
#else
  #define __Pyx_BUILTIN_MODULE_NAME "builtins"
#if PY_VERSION_HEX >= 0x030800A4 && PY_VERSION_HEX < 0x030800B2
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, 0, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#else
  #define __Pyx_PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)\
          PyCode_New(a, k, l, s, f, code, c, n, v, fv, cell, fn, name, fline, lnos)
#endif
  #define __Pyx_DefaultClassType PyType_Type
#endif
#ifndef Py_TPFLAGS_CHECKTYPES
  #define Py_TPFLAGS_CHECKTYPES 0
#endif
#ifndef Py_TPFLAGS_HAVE_INDEX
  #define Py_TPFLAGS_HAVE_INDEX 0
#endif
#ifndef Py_TPFLAGS_HAVE_NEWBUFFER
  #define Py_TPFLAGS_HAVE_NEWBUFFER 0
#endif
#ifndef Py_TPFLAGS_HAVE_FINALIZE
  #define Py_TPFLAGS_HAVE_FINALIZE 0
#endif
#ifndef METH_STACKLESS
  #define METH_STACKLESS 0
#endif
#if PY_VERSION_HEX <= 0x030700A3 || !defined(METH_FASTCALL)
  #ifndef METH_FASTCALL
     #define METH_FASTCALL 0x80
  #endif
  typedef PyObject *(*__Pyx_PyCFunctionFast) (PyObject *self, PyObject *const *args, Py_ssize_t nargs);
  typedef PyObject *(*__Pyx_PyCFunctionFastWithKeywords) (PyObject *self, PyObject *const *args,
                                                          Py_ssize_t nargs, PyObject *kwnames);
#else
  #define __Pyx_PyCFunctionFast _PyCFunctionFast
  #define __Pyx_PyCFunctionFastWithKeywords _PyCFunctionFastWithKeywords
#endif
#if CYTHON_FAST_PYCCALL
#define __Pyx_PyFastCFunction_Check(func)\
    ((PyCFunction_Check(func) && (METH_FASTCALL == (PyCFunction_GET_FLAGS(func) & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)))))
#else
#define __Pyx_PyFastCFunction_Check(func) 0
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Malloc)
  #define PyObject_Malloc(s)   PyMem_Malloc(s)
  #define PyObject_Free(p)     PyMem_Free(p)
  #define PyObject_Realloc(p)  PyMem_Realloc(p)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX < 0x030400A1
  #define PyMem_RawMalloc(n)           PyMem_Malloc(n)
  #define PyMem_RawRealloc(p, n)       PyMem_Realloc(p, n)
  #define PyMem_RawFree(p)             PyMem_Free(p)
#endif
#if CYTHON_COMPILING_IN_PYSTON
  #define __Pyx_PyCode_HasFreeVars(co)  PyCode_HasFreeVars(co)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno) PyFrame_SetLineNumber(frame, lineno)
#else
  #define __Pyx_PyCode_HasFreeVars(co)  (PyCode_GetNumFree(co) > 0)
  #define __Pyx_PyFrame_SetLineNumber(frame, lineno)  (frame)->f_lineno = (lineno)
#endif
#if !CYTHON_FAST_THREAD_STATE || PY_VERSION_HEX < 0x02070000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#elif PY_VERSION_HEX >= 0x03060000
  #define __Pyx_PyThreadState_Current _PyThreadState_UncheckedGet()
#elif PY_VERSION_HEX >= 0x03000000
  #define __Pyx_PyThreadState_Current PyThreadState_GET()
#else
  #define __Pyx_PyThreadState_Current _PyThreadState_Current
#endif
#if PY_VERSION_HEX < 0x030700A2 && !defined(PyThread_tss_create) && !defined(Py_tss_NEEDS_INIT)
#include "pythread.h"
#define Py_tss_NEEDS_INIT 0
typedef int Py_tss_t;
static CYTHON_INLINE int PyThread_tss_create(Py_tss_t *key) {
  *key = PyThread_create_key();
  return 0;
}
static CYTHON_INLINE Py_tss_t * PyThread_tss_alloc(void) {
  Py_tss_t *key = (Py_tss_t *)PyObject_Malloc(sizeof(Py_tss_t));
  *key = Py_tss_NEEDS_INIT;
  return key;
}
static CYTHON_INLINE void PyThread_tss_free(Py_tss_t *key) {
  PyObject_Free(key);
}
static CYTHON_INLINE int PyThread_tss_is_created(Py_tss_t *key) {
  return *key != Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE void PyThread_tss_delete(Py_tss_t *key) {
  PyThread_delete_key(*key);
  *key = Py_tss_NEEDS_INIT;
}
static CYTHON_INLINE int PyThread_tss_set(Py_tss_t *key, void *value) {
  return PyThread_set_key_value(*key, value);
}
static CYTHON_INLINE void * PyThread_tss_get(Py_tss_t *key) {
  return PyThread_get_key_value(*key);
}
#endif
#if CYTHON_COMPILING_IN_CPYTHON || defined(_PyDict_NewPresized)
#define __Pyx_PyDict_NewPresized(n)  ((n <= 8) ? PyDict_New() : _PyDict_NewPresized(n))
#else
#define __Pyx_PyDict_NewPresized(n)  PyDict_New()
#endif
#if PY_MAJOR_VERSION >= 3 || CYTHON_FUTURE_DIVISION
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_TrueDivide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceTrueDivide(x,y)
#else
  #define __Pyx_PyNumber_Divide(x,y)         PyNumber_Divide(x,y)
  #define __Pyx_PyNumber_InPlaceDivide(x,y)  PyNumber_InPlaceDivide(x,y)
#endif
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1 && CYTHON_USE_UNICODE_INTERNALS
#define __Pyx_PyDict_GetItemStr(dict, name)  _PyDict_GetItem_KnownHash(dict, name, ((PyASCIIObject *) name)->hash)
#else
#define __Pyx_PyDict_GetItemStr(dict, name)  PyDict_GetItem(dict, name)
#endif
#if PY_VERSION_HEX > 0x03030000 && defined(PyUnicode_KIND)
  #define CYTHON_PEP393_ENABLED 1
  #if defined(PyUnicode_IS_READY)
  #define __Pyx_PyUnicode_READY(op)       (likely(PyUnicode_IS_READY(op)) ?\
                                              0 : _PyUnicode_Ready((PyObject *)(op)))
  #else
  #define __Pyx_PyUnicode_READY(op)       (0)
  #endif
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_LENGTH(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) PyUnicode_READ_CHAR(u, i)
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   PyUnicode_MAX_CHAR_VALUE(u)
  #define __Pyx_PyUnicode_KIND(u)         PyUnicode_KIND(u)
  #define __Pyx_PyUnicode_DATA(u)         PyUnicode_DATA(u)
  #define __Pyx_PyUnicode_READ(k, d, i)   PyUnicode_READ(k, d, i)
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  PyUnicode_WRITE(k, d, i, ch)
  #if defined(PyUnicode_IS_READY) && defined(PyUnicode_GET_SIZE)
  #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03090000
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : ((PyCompactUnicodeObject *)(u))->wstr_length))
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != (likely(PyUnicode_IS_READY(u)) ? PyUnicode_GET_LENGTH(u) : PyUnicode_GET_SIZE(u)))
  #endif
  #else
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_LENGTH(u))
  #endif
#else
  #define CYTHON_PEP393_ENABLED 0
  #define PyUnicode_1BYTE_KIND  1
  #define PyUnicode_2BYTE_KIND  2
  #define PyUnicode_4BYTE_KIND  4
  #define __Pyx_PyUnicode_READY(op)       (0)
  #define __Pyx_PyUnicode_GET_LENGTH(u)   PyUnicode_GET_SIZE(u)
  #define __Pyx_PyUnicode_READ_CHAR(u, i) ((Py_UCS4)(PyUnicode_AS_UNICODE(u)[i]))
  #define __Pyx_PyUnicode_MAX_CHAR_VALUE(u)   ((sizeof(Py_UNICODE) == 2) ? 65535 : 1114111)
  #define __Pyx_PyUnicode_KIND(u)         (sizeof(Py_UNICODE))
  #define __Pyx_PyUnicode_DATA(u)         ((void*)PyUnicode_AS_UNICODE(u))
  #define __Pyx_PyUnicode_READ(k, d, i)   ((void)(k), (Py_UCS4)(((Py_UNICODE*)d)[i]))
  #define __Pyx_PyUnicode_WRITE(k, d, i, ch)  (((void)(k)), ((Py_UNICODE*)d)[i] = ch)
  #define __Pyx_PyUnicode_IS_TRUE(u)      (0 != PyUnicode_GET_SIZE(u))
#endif
#if CYTHON_COMPILING_IN_PYPY
  #define __Pyx_PyUnicode_Concat(a, b)      PyNumber_Add(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  PyNumber_Add(a, b)
#else
  #define __Pyx_PyUnicode_Concat(a, b)      PyUnicode_Concat(a, b)
  #define __Pyx_PyUnicode_ConcatSafe(a, b)  ((unlikely((a) == Py_None) || unlikely((b) == Py_None)) ?\
      PyNumber_Add(a, b) : __Pyx_PyUnicode_Concat(a, b))
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyUnicode_Contains)
  #define PyUnicode_Contains(u, s)  PySequence_Contains(u, s)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyByteArray_Check)
  #define PyByteArray_Check(obj)  PyObject_TypeCheck(obj, &PyByteArray_Type)
#endif
#if CYTHON_COMPILING_IN_PYPY && !defined(PyObject_Format)
  #define PyObject_Format(obj, fmt)  PyObject_CallMethod(obj, "__format__", "O", fmt)
#endif
#define __Pyx_PyString_FormatSafe(a, b)   ((unlikely((a) == Py_None || (PyString_Check(b) && !PyString_CheckExact(b)))) ? PyNumber_Remainder(a, b) : __Pyx_PyString_Format(a, b))
#define __Pyx_PyUnicode_FormatSafe(a, b)  ((unlikely((a) == Py_None || (PyUnicode_Check(b) && !PyUnicode_CheckExact(b)))) ? PyNumber_Remainder(a, b) : PyUnicode_Format(a, b))
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyString_Format(a, b)  PyUnicode_Format(a, b)
#else
  #define __Pyx_PyString_Format(a, b)  PyString_Format(a, b)
#endif
#if PY_MAJOR_VERSION < 3 && !defined(PyObject_ASCII)
  #define PyObject_ASCII(o)            PyObject_Repr(o)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBaseString_Type            PyUnicode_Type
  #define PyStringObject               PyUnicodeObject
  #define PyString_Type                PyUnicode_Type
  #define PyString_Check               PyUnicode_Check
  #define PyString_CheckExact          PyUnicode_CheckExact
#ifndef PyObject_Unicode
  #define PyObject_Unicode             PyObject_Str
#endif
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyBaseString_Check(obj) PyUnicode_Check(obj)
  #define __Pyx_PyBaseString_CheckExact(obj) PyUnicode_CheckExact(obj)
#else
  #define __Pyx_PyBaseString_Check(obj) (PyString_Check(obj) || PyUnicode_Check(obj))
  #define __Pyx_PyBaseString_CheckExact(obj) (PyString_CheckExact(obj) || PyUnicode_CheckExact(obj))
#endif
#ifndef PySet_CheckExact
  #define PySet_CheckExact(obj)        (Py_TYPE(obj) == &PySet_Type)
#endif
#if PY_VERSION_HEX >= 0x030900A4
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_SET_REFCNT(obj, refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SET_SIZE(obj, size)
#else
  #define __Pyx_SET_REFCNT(obj, refcnt) Py_REFCNT(obj) = (refcnt)
  #define __Pyx_SET_SIZE(obj, size) Py_SIZE(obj) = (size)
#endif
#if CYTHON_ASSUME_SAFE_MACROS
  #define __Pyx_PySequence_SIZE(seq)  Py_SIZE(seq)
#else
  #define __Pyx_PySequence_SIZE(seq)  PySequence_Size(seq)
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyIntObject                  PyLongObject
  #define PyInt_Type                   PyLong_Type
  #define PyInt_Check(op)              PyLong_Check(op)
  #define PyInt_CheckExact(op)         PyLong_CheckExact(op)
  #define PyInt_FromString             PyLong_FromString
  #define PyInt_FromUnicode            PyLong_FromUnicode
  #define PyInt_FromLong               PyLong_FromLong
  #define PyInt_FromSize_t             PyLong_FromSize_t
  #define PyInt_FromSsize_t            PyLong_FromSsize_t
  #define PyInt_AsLong                 PyLong_AsLong
  #define PyInt_AS_LONG                PyLong_AS_LONG
  #define PyInt_AsSsize_t              PyLong_AsSsize_t
  #define PyInt_AsUnsignedLongMask     PyLong_AsUnsignedLongMask
  #define PyInt_AsUnsignedLongLongMask PyLong_AsUnsignedLongLongMask
  #define PyNumber_Int                 PyNumber_Long
#endif
#if PY_MAJOR_VERSION >= 3
  #define PyBoolObject                 PyLongObject
#endif
#if PY_MAJOR_VERSION >= 3 && CYTHON_COMPILING_IN_PYPY
  #ifndef PyUnicode_InternFromString
    #define PyUnicode_InternFromString(s) PyUnicode_FromString(s)
  #endif
#endif
#if PY_VERSION_HEX < 0x030200A4
  typedef long Py_hash_t;
  #define __Pyx_PyInt_FromHash_t PyInt_FromLong
  #define __Pyx_PyInt_AsHash_t   PyInt_AsLong
#else
  #define __Pyx_PyInt_FromHash_t PyInt_FromSsize_t
  #define __Pyx_PyInt_AsHash_t   PyInt_AsSsize_t
#endif
#if PY_MAJOR_VERSION >= 3
  #define __Pyx_PyMethod_New(func, self, klass) ((self) ? ((void)(klass), PyMethod_New(func, self)) : __Pyx_NewRef(func))
#else
  #define __Pyx_PyMethod_New(func, self, klass) PyMethod_New(func, self, klass)
#endif
#if CYTHON_USE_ASYNC_SLOTS
  #if PY_VERSION_HEX >= 0x030500B1
    #define __Pyx_PyAsyncMethodsStruct PyAsyncMethods
    #define __Pyx_PyType_AsAsync(obj) (Py_TYPE(obj)->tp_as_async)
  #else
    #define __Pyx_PyType_AsAsync(obj) ((__Pyx_PyAsyncMethodsStruct*) (Py_TYPE(obj)->tp_reserved))
  #endif
#else
  #define __Pyx_PyType_AsAsync(obj) NULL
#endif
#ifndef __Pyx_PyAsyncMethodsStruct
    typedef struct {
        unaryfunc am_await;
        unaryfunc am_aiter;
        unaryfunc am_anext;
    } __Pyx_PyAsyncMethodsStruct;
#endif

#if defined(WIN32) || defined(MS_WINDOWS)
  #define _USE_MATH_DEFINES
#endif
#include <math.h>
#ifdef NAN
#define __PYX_NAN() ((float) NAN)
#else
static CYTHON_INLINE float __PYX_NAN() {
  float value;
  memset(&value, 0xFF, sizeof(value));
  return value;
}
#endif
#if defined(__CYGWIN__) && defined(_LDBL_EQ_DBL)
#define __Pyx_truncl trunc
#else
#define __Pyx_truncl truncl
#endif

#define __PYX_MARK_ERR_POS(f_index, lineno) \
    { __pyx_filename = __pyx_f[f_index]; (void)__pyx_filename; __pyx_lineno = lineno; (void)__pyx_lineno; __pyx_clineno = __LINE__; (void)__pyx_clineno; }
#define __PYX_ERR(f_index, lineno, Ln_error) \
    { __PYX_MARK_ERR_POS(f_index, lineno) goto Ln_error; }

#ifndef __PYX_EXTERN_C
  #ifdef __cplusplus
    #define __PYX_EXTERN_C extern "C"
  #else
    #define __PYX_EXTERN_C extern
  #endif
#endif

#define __PYX_HAVE__glycopeptidepy___c__structure__base
#define __PYX_HAVE_API__glycopeptidepy___c__structure__base
/* Early includes */
#include <string.h>
#include <stdio.h>
#ifdef _OPENMP
#include <omp.h>
#endif /* _OPENMP */

#if defined(PYREX_WITHOUT_ASSERTIONS) && !defined(CYTHON_WITHOUT_ASSERTIONS)
#define CYTHON_WITHOUT_ASSERTIONS
#endif

typedef struct {PyObject **p; const char *s; const Py_ssize_t n; const char* encoding;
                const char is_unicode; const char is_str; const char intern; } __Pyx_StringTabEntry;

#define __PYX_DEFAULT_STRING_ENCODING_IS_ASCII 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_UTF8 0
#define __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT (PY_MAJOR_VERSION >= 3 && __PYX_DEFAULT_STRING_ENCODING_IS_UTF8)
#define __PYX_DEFAULT_STRING_ENCODING ""
#define __Pyx_PyObject_FromString __Pyx_PyBytes_FromString
#define __Pyx_PyObject_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#define __Pyx_uchar_cast(c) ((unsigned char)c)
#define __Pyx_long_cast(x) ((long)x)
#define __Pyx_fits_Py_ssize_t(v, type, is_signed)  (\
    (sizeof(type) < sizeof(Py_ssize_t))  ||\
    (sizeof(type) > sizeof(Py_ssize_t) &&\
          likely(v < (type)PY_SSIZE_T_MAX ||\
                 v == (type)PY_SSIZE_T_MAX)  &&\
          (!is_signed || likely(v > (type)PY_SSIZE_T_MIN ||\
                                v == (type)PY_SSIZE_T_MIN)))  ||\
    (sizeof(type) == sizeof(Py_ssize_t) &&\
          (is_signed || likely(v < (type)PY_SSIZE_T_MAX ||\
                               v == (type)PY_SSIZE_T_MAX)))  )
static CYTHON_INLINE int __Pyx_is_valid_index(Py_ssize_t i, Py_ssize_t limit) {
    return (size_t) i < (size_t) limit;
}
#if defined (__cplusplus) && __cplusplus >= 201103L
    #include <cstdlib>
    #define __Pyx_sst_abs(value) std::abs(value)
#elif SIZEOF_INT >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) abs(value)
#elif SIZEOF_LONG >= SIZEOF_SIZE_T
    #define __Pyx_sst_abs(value) labs(value)
#elif defined (_MSC_VER)
    #define __Pyx_sst_abs(value) ((Py_ssize_t)_abs64(value))
#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 199901L
    #define __Pyx_sst_abs(value) llabs(value)
#elif defined (__GNUC__)
    #define __Pyx_sst_abs(value) __builtin_llabs(value)
#else
    #define __Pyx_sst_abs(value) ((value<0) ? -value : value)
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject*);
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject*, Py_ssize_t* length);
#define __Pyx_PyByteArray_FromString(s) PyByteArray_FromStringAndSize((const char*)s, strlen((const char*)s))
#define __Pyx_PyByteArray_FromStringAndSize(s, l) PyByteArray_FromStringAndSize((const char*)s, l)
#define __Pyx_PyBytes_FromString        PyBytes_FromString
#define __Pyx_PyBytes_FromStringAndSize PyBytes_FromStringAndSize
static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char*);
#if PY_MAJOR_VERSION < 3
    #define __Pyx_PyStr_FromString        __Pyx_PyBytes_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyBytes_FromStringAndSize
#else
    #define __Pyx_PyStr_FromString        __Pyx_PyUnicode_FromString
    #define __Pyx_PyStr_FromStringAndSize __Pyx_PyUnicode_FromStringAndSize
#endif
#define __Pyx_PyBytes_AsWritableString(s)     ((char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableSString(s)    ((signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsWritableUString(s)    ((unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsString(s)     ((const char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsSString(s)    ((const signed char*) PyBytes_AS_STRING(s))
#define __Pyx_PyBytes_AsUString(s)    ((const unsigned char*) PyBytes_AS_STRING(s))
#define __Pyx_PyObject_AsWritableString(s)    ((char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableSString(s)    ((signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsWritableUString(s)    ((unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsSString(s)    ((const signed char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_AsUString(s)    ((const unsigned char*) __Pyx_PyObject_AsString(s))
#define __Pyx_PyObject_FromCString(s)  __Pyx_PyObject_FromString((const char*)s)
#define __Pyx_PyBytes_FromCString(s)   __Pyx_PyBytes_FromString((const char*)s)
#define __Pyx_PyByteArray_FromCString(s)   __Pyx_PyByteArray_FromString((const char*)s)
#define __Pyx_PyStr_FromCString(s)     __Pyx_PyStr_FromString((const char*)s)
#define __Pyx_PyUnicode_FromCString(s) __Pyx_PyUnicode_FromString((const char*)s)
static CYTHON_INLINE size_t __Pyx_Py_UNICODE_strlen(const Py_UNICODE *u) {
    const Py_UNICODE *u_end = u;
    while (*u_end++) ;
    return (size_t)(u_end - u - 1);
}
#define __Pyx_PyUnicode_FromUnicode(u)       PyUnicode_FromUnicode(u, __Pyx_Py_UNICODE_strlen(u))
#define __Pyx_PyUnicode_FromUnicodeAndLength PyUnicode_FromUnicode
#define __Pyx_PyUnicode_AsUnicode            PyUnicode_AsUnicode
#define __Pyx_NewRef(obj) (Py_INCREF(obj), obj)
#define __Pyx_Owned_Py_None(b) __Pyx_NewRef(Py_None)
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b);
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject*);
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject*);
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x);
#define __Pyx_PySequence_Tuple(obj)\
    (likely(PyTuple_CheckExact(obj)) ? __Pyx_NewRef(obj) : PySequence_Tuple(obj))
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject*);
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t);
#if CYTHON_ASSUME_SAFE_MACROS
#define __pyx_PyFloat_AsDouble(x) (PyFloat_CheckExact(x) ? PyFloat_AS_DOUBLE(x) : PyFloat_AsDouble(x))
#else
#define __pyx_PyFloat_AsDouble(x) PyFloat_AsDouble(x)
#endif
#define __pyx_PyFloat_AsFloat(x) ((float) __pyx_PyFloat_AsDouble(x))
#if PY_MAJOR_VERSION >= 3
#define __Pyx_PyNumber_Int(x) (PyLong_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Long(x))
#else
#define __Pyx_PyNumber_Int(x) (PyInt_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Int(x))
#endif
#define __Pyx_PyNumber_Float(x) (PyFloat_CheckExact(x) ? __Pyx_NewRef(x) : PyNumber_Float(x))
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
static int __Pyx_sys_getdefaultencoding_not_ascii;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    PyObject* ascii_chars_u = NULL;
    PyObject* ascii_chars_b = NULL;
    const char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    if (strcmp(default_encoding_c, "ascii") == 0) {
        __Pyx_sys_getdefaultencoding_not_ascii = 0;
    } else {
        char ascii_chars[128];
        int c;
        for (c = 0; c < 128; c++) {
            ascii_chars[c] = c;
        }
        __Pyx_sys_getdefaultencoding_not_ascii = 1;
        ascii_chars_u = PyUnicode_DecodeASCII(ascii_chars, 128, NULL);
        if (!ascii_chars_u) goto bad;
        ascii_chars_b = PyUnicode_AsEncodedString(ascii_chars_u, default_encoding_c, NULL);
        if (!ascii_chars_b || !PyBytes_Check(ascii_chars_b) || memcmp(ascii_chars, PyBytes_AS_STRING(ascii_chars_b), 128) != 0) {
            PyErr_Format(
                PyExc_ValueError,
                "This module compiled with c_string_encoding=ascii, but default encoding '%.200s' is not a superset of ascii.",
                default_encoding_c);
            goto bad;
        }
        Py_DECREF(ascii_chars_u);
        Py_DECREF(ascii_chars_b);
    }
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    Py_XDECREF(ascii_chars_u);
    Py_XDECREF(ascii_chars_b);
    return -1;
}
#endif
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT && PY_MAJOR_VERSION >= 3
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_DecodeUTF8(c_str, size, NULL)
#else
#define __Pyx_PyUnicode_FromStringAndSize(c_str, size) PyUnicode_Decode(c_str, size, __PYX_DEFAULT_STRING_ENCODING, NULL)
#if __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
static char* __PYX_DEFAULT_STRING_ENCODING;
static int __Pyx_init_sys_getdefaultencoding_params(void) {
    PyObject* sys;
    PyObject* default_encoding = NULL;
    char* default_encoding_c;
    sys = PyImport_ImportModule("sys");
    if (!sys) goto bad;
    default_encoding = PyObject_CallMethod(sys, (char*) (const char*) "getdefaultencoding", NULL);
    Py_DECREF(sys);
    if (!default_encoding) goto bad;
    default_encoding_c = PyBytes_AsString(default_encoding);
    if (!default_encoding_c) goto bad;
    __PYX_DEFAULT_STRING_ENCODING = (char*) malloc(strlen(default_encoding_c) + 1);
    if (!__PYX_DEFAULT_STRING_ENCODING) goto bad;
    strcpy(__PYX_DEFAULT_STRING_ENCODING, default_encoding_c);
    Py_DECREF(default_encoding);
    return 0;
bad:
    Py_XDECREF(default_encoding);
    return -1;
}
#endif
#endif


/* Test for GCC > 2.95 */
#if defined(__GNUC__)     && (__GNUC__ > 2 || (__GNUC__ == 2 && (__GNUC_MINOR__ > 95)))
  #define likely(x)   __builtin_expect(!!(x), 1)
  #define unlikely(x) __builtin_expect(!!(x), 0)
#else /* !__GNUC__ or GCC < 2.95 */
  #define likely(x)   (x)
  #define unlikely(x) (x)
#endif /* __GNUC__ */
static CYTHON_INLINE void __Pyx_pretend_to_initialize(void* ptr) { (void)ptr; }

static PyObject *__pyx_m = NULL;
static PyObject *__pyx_d;
static PyObject *__pyx_b;
static PyObject *__pyx_cython_runtime = NULL;
static PyObject *__pyx_empty_tuple;
static PyObject *__pyx_empty_bytes;
static PyObject *__pyx_empty_unicode;
static int __pyx_lineno;
static int __pyx_clineno = 0;
static const char * __pyx_cfilenm= __FILE__;
static const char *__pyx_filename;


static const char *__pyx_f[] = {
  "glycopeptidepy/_c/structure/base.pyx",
  "glycopeptidepy/_c/structure/base.pxd",
  "stringsource",
  "ccomposition.pxd",
  "type.pxd",
};

/*--- Type declarations ---*/
struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition;
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__;
struct __pyx_opt_args_5glypy_11composition_12ccomposition_12CComposition_calc_mass;
struct __pyx_opt_args_5glypy_11composition_12ccomposition_calculate_mass;

/* "glypy/composition/ccomposition.pxd":6
 *     cdef tuple _mass_args
 *     cpdef CComposition clone(self)
 *     cpdef double calc_mass(self, int average=?, charge=?, dict mass_data=?) except -1             # <<<<<<<<<<<<<<
 *     cpdef _from_formula(self, str formula, dict mass_data)
 *     cpdef _from_dict(self, comp)
 */
struct __pyx_opt_args_5glypy_11composition_12ccomposition_12CComposition_calc_mass {
  int __pyx_n;
  int average;
  PyObject *charge;
  PyObject *mass_data;
};

/* "glypy/composition/ccomposition.pxd":31
 * 
 * 
 * cpdef double calculate_mass(CComposition composition=?, str formula=?, int average=?, charge=?, mass_data=?) except -1             # <<<<<<<<<<<<<<
 */
struct __pyx_opt_args_5glypy_11composition_12ccomposition_calculate_mass {
  int __pyx_n;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition;
  PyObject *formula;
  int average;
  PyObject *charge;
  PyObject *mass_data;
};

/* "glypy/composition/ccomposition.pxd":2
 * 
 * cdef class CComposition(dict):             # <<<<<<<<<<<<<<
 *     cdef object _mass
 *     cdef tuple _mass_args
 */
struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition {
  PyDictObject __pyx_base;
  struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *__pyx_vtab;
  PyObject *_mass;
  PyObject *_mass_args;
};


/* "glycopeptidepy/_c/structure/base.pxd":5
 * 
 * 
 * cdef class PeptideSequenceBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public list sequence
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_vtab;
  PyObject *sequence;
  double _mass;
  PyObject *_glycosylation_manager;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *_n_term;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *_c_term;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *_total_composition;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *_peptide_composition;
};


/* "glycopeptidepy/_c/structure/base.pxd":21
 * 
 * 
 * cdef class TerminalGroup(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public CComposition base_composition
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_vtab;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *base_composition;
  double mass;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *_modification;
};


/* "glycopeptidepy/_c/structure/base.pxd":35
 * 
 * 
 * cdef class AminoAcidResidueBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public basestring name
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_vtab;
  PyObject *name;
  PyObject *symbol;
  double mass;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition;
  PyObject *neutral_loss;
  Py_hash_t _hash;
};


/* "glycopeptidepy/_c/structure/base.pxd":50
 * 
 * 
 * cdef class ModificationBase(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public basestring name
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_vtab;
  PyObject *name;
  double mass;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *composition;
};


/* "glycopeptidepy/_c/structure/base.pxd":69
 * 
 * 
 * cdef class SequencePosition(object):             # <<<<<<<<<<<<<<
 *     cdef:
 *         public AminoAcidResidueBase amino_acid
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition {
  PyObject_HEAD
  struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_vtab;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *amino_acid;
  PyObject *modifications;
};


/* "glycopeptidepy/_c/structure/base.pyx":231
 *         self.modifications = PySequence_GetItem(parts, 1)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         yield self.amino_acid
 *         yield self.modifications
 */
struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ {
  PyObject_HEAD
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self;
};



/* "glypy/composition/ccomposition.pxd":2
 * 
 * cdef class CComposition(dict):             # <<<<<<<<<<<<<<
 *     cdef object _mass
 *     cdef tuple _mass_args
 */

struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition {
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*clone)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, int __pyx_skip_dispatch);
  double (*calc_mass)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, int __pyx_skip_dispatch, struct __pyx_opt_args_5glypy_11composition_12ccomposition_12CComposition_calc_mass *__pyx_optional_args);
  PyObject *(*_from_formula)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*_from_dict)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *, int __pyx_skip_dispatch);
  long (*getitem)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *);
  void (*setitem)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *, long);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*add_from)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*subtract_from)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *, PyObject *);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*_create)(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *);
};
static struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition *__pyx_vtabptr_5glypy_11composition_12ccomposition_CComposition;


/* "glycopeptidepy/_c/structure/base.pyx":14
 * 
 * @cython.freelist(10000)
 * cdef class PeptideSequenceBase(object):             # <<<<<<<<<<<<<<
 * 
 *     cdef SequencePosition get(self, ssize_t i):
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *(*get)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *, Py_ssize_t);
  PyObject *(*_invalidate)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase;


/* "glycopeptidepy/_c/structure/base.pyx":27
 * 
 * @cython.freelist(100)
 * cdef class TerminalGroup(object):             # <<<<<<<<<<<<<<
 *     def __init__(self, base_composition, modification=None):
 *         if not isinstance(base_composition, CComposition):
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *(*get_modification)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *);
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*get_composition)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *);
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *(*modify)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *, int __pyx_skip_dispatch);
  void (*set_modification)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *);
  int (*equal_to)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, PyObject *);
  PyObject *(*serialize)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_TerminalGroup;


/* "glycopeptidepy/_c/structure/base.pyx":145
 * 
 * @cython.freelist(100)
 * cdef class AminoAcidResidueBase(object):             # <<<<<<<<<<<<<<
 *     '''
 *     A base type for classes describing amino acid residues
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *(*_create)(PyObject *, PyObject *, double, struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *);
  int (*equal_to)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase;


/* "glycopeptidepy/_c/structure/base.pyx":186
 * 
 * @cython.freelist(100000)
 * cdef class ModificationBase(object):             # <<<<<<<<<<<<<<
 * 
 *     def __copy__(self):
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase {
  int (*is_a)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*serialize)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *, int __pyx_skip_dispatch);
  int (*is_tracked_for)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_ModificationBase;


/* "glycopeptidepy/_c/structure/base.pyx":225
 * 
 * @cython.freelist(10000000)
 * cdef class SequencePosition(object):             # <<<<<<<<<<<<<<
 * 
 *     def __init__(self, parts):
 */

struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_SequencePosition {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *(*_create)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *, PyObject *);
  double (*get_mass)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *);
  int (*has_modification)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *, PyObject *, int __pyx_skip_dispatch);
  int (*is_modified)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *, int __pyx_skip_dispatch);
  PyObject *(*drop_modification)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *, PyObject *, int __pyx_skip_dispatch);
  PyObject *(*add_modification)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *, PyObject *, int __pyx_skip_dispatch);
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_SequencePosition;

/* --- Runtime support code (head) --- */
/* Refnanny.proto */
#ifndef CYTHON_REFNANNY
  #define CYTHON_REFNANNY 0
#endif
#if CYTHON_REFNANNY
  typedef struct {
    void (*INCREF)(void*, PyObject*, int);
    void (*DECREF)(void*, PyObject*, int);
    void (*GOTREF)(void*, PyObject*, int);
    void (*GIVEREF)(void*, PyObject*, int);
    void* (*SetupContext)(const char*, int, const char*);
    void (*FinishContext)(void**);
  } __Pyx_RefNannyAPIStruct;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNanny = NULL;
  static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname);
  #define __Pyx_RefNannyDeclarations void *__pyx_refnanny = NULL;
#ifdef WITH_THREAD
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          if (acquire_gil) {\
              PyGILState_STATE __pyx_gilstate_save = PyGILState_Ensure();\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
              PyGILState_Release(__pyx_gilstate_save);\
          } else {\
              __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__);\
          }
#else
  #define __Pyx_RefNannySetupContext(name, acquire_gil)\
          __pyx_refnanny = __Pyx_RefNanny->SetupContext((name), __LINE__, __FILE__)
#endif
  #define __Pyx_RefNannyFinishContext()\
          __Pyx_RefNanny->FinishContext(&__pyx_refnanny)
  #define __Pyx_INCREF(r)  __Pyx_RefNanny->INCREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_DECREF(r)  __Pyx_RefNanny->DECREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GOTREF(r)  __Pyx_RefNanny->GOTREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_GIVEREF(r) __Pyx_RefNanny->GIVEREF(__pyx_refnanny, (PyObject *)(r), __LINE__)
  #define __Pyx_XINCREF(r)  do { if((r) != NULL) {__Pyx_INCREF(r); }} while(0)
  #define __Pyx_XDECREF(r)  do { if((r) != NULL) {__Pyx_DECREF(r); }} while(0)
  #define __Pyx_XGOTREF(r)  do { if((r) != NULL) {__Pyx_GOTREF(r); }} while(0)
  #define __Pyx_XGIVEREF(r) do { if((r) != NULL) {__Pyx_GIVEREF(r);}} while(0)
#else
  #define __Pyx_RefNannyDeclarations
  #define __Pyx_RefNannySetupContext(name, acquire_gil)
  #define __Pyx_RefNannyFinishContext()
  #define __Pyx_INCREF(r) Py_INCREF(r)
  #define __Pyx_DECREF(r) Py_DECREF(r)
  #define __Pyx_GOTREF(r)
  #define __Pyx_GIVEREF(r)
  #define __Pyx_XINCREF(r) Py_XINCREF(r)
  #define __Pyx_XDECREF(r) Py_XDECREF(r)
  #define __Pyx_XGOTREF(r)
  #define __Pyx_XGIVEREF(r)
#endif
#define __Pyx_XDECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_XDECREF(tmp);\
    } while (0)
#define __Pyx_DECREF_SET(r, v) do {\
        PyObject *tmp = (PyObject *) r;\
        r = v; __Pyx_DECREF(tmp);\
    } while (0)
#define __Pyx_CLEAR(r)    do { PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);} while(0)
#define __Pyx_XCLEAR(r)   do { if((r) != NULL) {PyObject* tmp = ((PyObject*)(r)); r = NULL; __Pyx_DECREF(tmp);}} while(0)

/* PyObjectGetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GetAttrStr(o,n) PyObject_GetAttr(o,n)
#endif

/* GetBuiltinName.proto */
static PyObject *__Pyx_GetBuiltinName(PyObject *name);

/* PyDictVersioning.proto */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
#define __PYX_DICT_VERSION_INIT  ((PY_UINT64_T) -1)
#define __PYX_GET_DICT_VERSION(dict)  (((PyDictObject*)(dict))->ma_version_tag)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)\
    (version_var) = __PYX_GET_DICT_VERSION(dict);\
    (cache_var) = (value);
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP) {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    if (likely(__PYX_GET_DICT_VERSION(DICT) == __pyx_dict_version)) {\
        (VAR) = __pyx_dict_cached_value;\
    } else {\
        (VAR) = __pyx_dict_cached_value = (LOOKUP);\
        __pyx_dict_version = __PYX_GET_DICT_VERSION(DICT);\
    }\
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj);
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj);
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version);
#else
#define __PYX_GET_DICT_VERSION(dict)  (0)
#define __PYX_UPDATE_DICT_CACHE(dict, value, cache_var, version_var)
#define __PYX_PY_DICT_LOOKUP_IF_MODIFIED(VAR, DICT, LOOKUP)  (VAR) = (LOOKUP);
#endif

/* PyFunctionFastCall.proto */
#if CYTHON_FAST_PYCALL
#define __Pyx_PyFunction_FastCall(func, args, nargs)\
    __Pyx_PyFunction_FastCallDict((func), (args), (nargs), NULL)
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs);
#else
#define __Pyx_PyFunction_FastCallDict(func, args, nargs, kwargs) _PyFunction_FastCallDict(func, args, nargs, kwargs)
#endif
#define __Pyx_BUILD_ASSERT_EXPR(cond)\
    (sizeof(char [1 - 2*!(cond)]) - 1)
#ifndef Py_MEMBER_SIZE
#define Py_MEMBER_SIZE(type, member) sizeof(((type *)0)->member)
#endif
  static size_t __pyx_pyframe_localsplus_offset = 0;
  #include "frameobject.h"
  #define __Pxy_PyFrame_Initialize_Offsets()\
    ((void)__Pyx_BUILD_ASSERT_EXPR(sizeof(PyFrameObject) == offsetof(PyFrameObject, f_localsplus) + Py_MEMBER_SIZE(PyFrameObject, f_localsplus)),\
     (void)(__pyx_pyframe_localsplus_offset = ((size_t)PyFrame_Type.tp_basicsize) - Py_MEMBER_SIZE(PyFrameObject, f_localsplus)))
  #define __Pyx_PyFrame_GetLocalsplus(frame)\
    (assert(__pyx_pyframe_localsplus_offset), (PyObject **)(((char *)(frame)) + __pyx_pyframe_localsplus_offset))
#endif

/* PyObjectCall.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw);
#else
#define __Pyx_PyObject_Call(func, arg, kw) PyObject_Call(func, arg, kw)
#endif

/* PyObjectCallMethO.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg);
#endif

/* PyObjectCallNoArg.proto */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func);
#else
#define __Pyx_PyObject_CallNoArg(func) __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL)
#endif

/* PyCFunctionFastCall.proto */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject *__Pyx_PyCFunction_FastCall(PyObject *func, PyObject **args, Py_ssize_t nargs);
#else
#define __Pyx_PyCFunction_FastCall(func, args, nargs)  (assert(0), NULL)
#endif

/* PyObjectCallOneArg.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg);

/* ExtTypeTest.proto */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type);

/* PyErrExceptionMatches.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_ExceptionMatches(err) __Pyx_PyErr_ExceptionMatchesInState(__pyx_tstate, err)
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err);
#else
#define __Pyx_PyErr_ExceptionMatches(err)  PyErr_ExceptionMatches(err)
#endif

/* PyThreadStateGet.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyThreadState_declare  PyThreadState *__pyx_tstate;
#define __Pyx_PyThreadState_assign  __pyx_tstate = __Pyx_PyThreadState_Current;
#define __Pyx_PyErr_Occurred()  __pyx_tstate->curexc_type
#else
#define __Pyx_PyThreadState_declare
#define __Pyx_PyThreadState_assign
#define __Pyx_PyErr_Occurred()  PyErr_Occurred()
#endif

/* PyErrFetchRestore.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyErr_Clear() __Pyx_ErrRestore(NULL, NULL, NULL)
#define __Pyx_ErrRestoreWithState(type, value, tb)  __Pyx_ErrRestoreInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)    __Pyx_ErrFetchInState(PyThreadState_GET(), type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  __Pyx_ErrRestoreInState(__pyx_tstate, type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)    __Pyx_ErrFetchInState(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_PyErr_SetNone(exc) (Py_INCREF(exc), __Pyx_ErrRestore((exc), NULL, NULL))
#else
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#endif
#else
#define __Pyx_PyErr_Clear() PyErr_Clear()
#define __Pyx_PyErr_SetNone(exc) PyErr_SetNone(exc)
#define __Pyx_ErrRestoreWithState(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchWithState(type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestoreInState(tstate, type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetchInState(tstate, type, value, tb)  PyErr_Fetch(type, value, tb)
#define __Pyx_ErrRestore(type, value, tb)  PyErr_Restore(type, value, tb)
#define __Pyx_ErrFetch(type, value, tb)  PyErr_Fetch(type, value, tb)
#endif

/* GetAttr.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *, PyObject *);

/* GetAttr3.proto */
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *, PyObject *, PyObject *);

/* GetModuleGlobalName.proto */
#if CYTHON_USE_DICT_VERSIONS
#define __Pyx_GetModuleGlobalName(var, name)  {\
    static PY_UINT64_T __pyx_dict_version = 0;\
    static PyObject *__pyx_dict_cached_value = NULL;\
    (var) = (likely(__pyx_dict_version == __PYX_GET_DICT_VERSION(__pyx_d))) ?\
        (likely(__pyx_dict_cached_value) ? __Pyx_NewRef(__pyx_dict_cached_value) : __Pyx_GetBuiltinName(name)) :\
        __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
#define __Pyx_GetModuleGlobalNameUncached(var, name)  {\
    PY_UINT64_T __pyx_dict_version;\
    PyObject *__pyx_dict_cached_value;\
    (var) = __Pyx__GetModuleGlobalName(name, &__pyx_dict_version, &__pyx_dict_cached_value);\
}
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value);
#else
#define __Pyx_GetModuleGlobalName(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
#define __Pyx_GetModuleGlobalNameUncached(var, name)  (var) = __Pyx__GetModuleGlobalName(name)
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name);
#endif

/* RaiseDoubleKeywords.proto */
static void __Pyx_RaiseDoubleKeywordsError(const char* func_name, PyObject* kw_name);

/* ParseKeywords.proto */
static int __Pyx_ParseOptionalKeywords(PyObject *kwds, PyObject **argnames[],\
    PyObject *kwds2, PyObject *values[], Py_ssize_t num_pos_args,\
    const char* function_name);

/* RaiseArgTupleInvalid.proto */
static void __Pyx_RaiseArgtupleInvalid(const char* func_name, int exact,
    Py_ssize_t num_min, Py_ssize_t num_max, Py_ssize_t num_found);

/* PyObjectSetAttrStr.proto */
#if CYTHON_USE_TYPE_SLOTS
#define __Pyx_PyObject_DelAttrStr(o,n) __Pyx_PyObject_SetAttrStr(o, n, NULL)
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value);
#else
#define __Pyx_PyObject_DelAttrStr(o,n)   PyObject_DelAttr(o,n)
#define __Pyx_PyObject_SetAttrStr(o,n,v) PyObject_SetAttr(o,n,v)
#endif

/* ArgTypeTest.proto */
#define __Pyx_ArgTypeTest(obj, type, none_allowed, name, exact)\
    ((likely((Py_TYPE(obj) == type) | (none_allowed && (obj == Py_None)))) ? 1 :\
        __Pyx__ArgTypeTest(obj, type, name, exact))
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact);

/* PyObjectCall2Args.proto */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2);

/* GetTopmostException.proto */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem * __Pyx_PyErr_GetTopmostException(PyThreadState *tstate);
#endif

/* SaveResetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSave(type, value, tb)  __Pyx__ExceptionSave(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#define __Pyx_ExceptionReset(type, value, tb)  __Pyx__ExceptionReset(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb);
#else
#define __Pyx_ExceptionSave(type, value, tb)   PyErr_GetExcInfo(type, value, tb)
#define __Pyx_ExceptionReset(type, value, tb)  PyErr_SetExcInfo(type, value, tb)
#endif

/* GetException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_GetException(type, value, tb)  __Pyx__GetException(__pyx_tstate, type, value, tb)
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* Import.proto */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level);

/* ImportFrom.proto */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name);

/* WriteUnraisableException.proto */
static void __Pyx_WriteUnraisable(const char *name, int clineno,
                                  int lineno, const char *filename,
                                  int full_traceback, int nogil);

/* IncludeStringH.proto */
#include <string.h>

/* BytesEquals.proto */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals);

/* UnicodeEquals.proto */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals);

/* PyIntCompare.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, long intval, long inplace);

/* RaiseException.proto */
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause);

/* ListAppend.proto */
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static CYTHON_INLINE int __Pyx_PyList_Append(PyObject* list, PyObject* x) {
    PyListObject* L = (PyListObject*) list;
    Py_ssize_t len = Py_SIZE(list);
    if (likely(L->allocated > len) & likely(len > (L->allocated >> 1))) {
        Py_INCREF(x);
        PyList_SET_ITEM(list, len, x);
        __Pyx_SET_SIZE(list, len + 1);
        return 0;
    }
    return PyList_Append(list, x);
}
#else
#define __Pyx_PyList_Append(L,x) PyList_Append(L,x)
#endif

/* PyObjectGetMethod.proto */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method);

/* PyObjectCallMethod1.proto */
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg);

/* pop_index.proto */
static PyObject* __Pyx__PyObject_PopNewIndex(PyObject* L, PyObject* py_ix);
static PyObject* __Pyx__PyObject_PopIndex(PyObject* L, PyObject* py_ix);
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static PyObject* __Pyx__PyList_PopIndex(PyObject* L, PyObject* py_ix, Py_ssize_t ix);
#define __Pyx_PyObject_PopIndex(L, py_ix, ix, is_signed, type, to_py_func) (\
    (likely(PyList_CheckExact(L) && __Pyx_fits_Py_ssize_t(ix, type, is_signed))) ?\
        __Pyx__PyList_PopIndex(L, py_ix, ix) : (\
        (unlikely((py_ix) == Py_None)) ? __Pyx__PyObject_PopNewIndex(L, to_py_func(ix)) :\
            __Pyx__PyObject_PopIndex(L, py_ix)))
#define __Pyx_PyList_PopIndex(L, py_ix, ix, is_signed, type, to_py_func) (\
    __Pyx_fits_Py_ssize_t(ix, type, is_signed) ?\
        __Pyx__PyList_PopIndex(L, py_ix, ix) : (\
        (unlikely((py_ix) == Py_None)) ? __Pyx__PyObject_PopNewIndex(L, to_py_func(ix)) :\
            __Pyx__PyObject_PopIndex(L, py_ix)))
#else
#define __Pyx_PyList_PopIndex(L, py_ix, ix, is_signed, type, to_py_func)\
    __Pyx_PyObject_PopIndex(L, py_ix, ix, is_signed, type, to_py_func)
#define __Pyx_PyObject_PopIndex(L, py_ix, ix, is_signed, type, to_py_func) (\
    (unlikely((py_ix) == Py_None)) ? __Pyx__PyObject_PopNewIndex(L, to_py_func(ix)) :\
        __Pyx__PyObject_PopIndex(L, py_ix))
#endif

/* GetItemInt.proto */
#define __Pyx_GetItemInt(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Fast(o, (Py_ssize_t)i, is_list, wraparound, boundscheck) :\
    (is_list ? (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL) :\
               __Pyx_GetItemInt_Generic(o, to_py_func(i))))
#define __Pyx_GetItemInt_List(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_List_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "list index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
#define __Pyx_GetItemInt_Tuple(o, i, type, is_signed, to_py_func, is_list, wraparound, boundscheck)\
    (__Pyx_fits_Py_ssize_t(i, type, is_signed) ?\
    __Pyx_GetItemInt_Tuple_Fast(o, (Py_ssize_t)i, wraparound, boundscheck) :\
    (PyErr_SetString(PyExc_IndexError, "tuple index out of range"), (PyObject*)NULL))
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              int wraparound, int boundscheck);
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j);
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i,
                                                     int is_list, int wraparound, int boundscheck);

/* HasAttr.proto */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *, PyObject *);

/* PyObject_GenericGetAttrNoDict.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttrNoDict PyObject_GenericGetAttr
#endif

/* PyObject_GenericGetAttr.proto */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name);
#else
#define __Pyx_PyObject_GenericGetAttr PyObject_GenericGetAttr
#endif

/* SetVTable.proto */
static int __Pyx_SetVtable(PyObject *dict, void *vtable);

/* PyObjectGetAttrStrNoError.proto */
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name);

/* SetupReduce.proto */
static int __Pyx_setup_reduce(PyObject* type_obj);

/* TypeImport.proto */
#ifndef __PYX_HAVE_RT_ImportType_proto
#define __PYX_HAVE_RT_ImportType_proto
enum __Pyx_ImportType_CheckSize {
   __Pyx_ImportType_CheckSize_Error = 0,
   __Pyx_ImportType_CheckSize_Warn = 1,
   __Pyx_ImportType_CheckSize_Ignore = 2
};
static PyTypeObject *__Pyx_ImportType(PyObject* module, const char *module_name, const char *class_name, size_t size, enum __Pyx_ImportType_CheckSize check_size);
#endif

/* GetVTable.proto */
static void* __Pyx_GetVtable(PyObject *dict);

/* CLineInTraceback.proto */
#ifdef CYTHON_CLINE_IN_TRACEBACK
#define __Pyx_CLineForTraceback(tstate, c_line)  (((CYTHON_CLINE_IN_TRACEBACK)) ? c_line : 0)
#else
static int __Pyx_CLineForTraceback(PyThreadState *tstate, int c_line);
#endif

/* CodeObjectCache.proto */
typedef struct {
    PyCodeObject* code_object;
    int code_line;
} __Pyx_CodeObjectCacheEntry;
struct __Pyx_CodeObjectCache {
    int count;
    int max_count;
    __Pyx_CodeObjectCacheEntry* entries;
};
static struct __Pyx_CodeObjectCache __pyx_code_cache = {0,0,NULL};
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line);
static PyCodeObject *__pyx_find_code_object(int code_line);
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object);

/* AddTraceback.proto */
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename);

/* GCCDiagnostics.proto */
#if defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 6))
#define __Pyx_HAS_GCC_DIAGNOSTIC
#endif

/* CIntFromPy.proto */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *);

/* CIntFromPy.proto */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value);

/* CIntToPy.proto */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value);

/* CIntFromPy.proto */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *);

/* FastTypeChecks.proto */
#if CYTHON_COMPILING_IN_CPYTHON
#define __Pyx_TypeCheck(obj, type) __Pyx_IsSubtype(Py_TYPE(obj), (PyTypeObject *)type)
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject *type);
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *type1, PyObject *type2);
#else
#define __Pyx_TypeCheck(obj, type) PyObject_TypeCheck(obj, (PyTypeObject *)type)
#define __Pyx_PyErr_GivenExceptionMatches(err, type) PyErr_GivenExceptionMatches(err, type)
#define __Pyx_PyErr_GivenExceptionMatches2(err, type1, type2) (PyErr_GivenExceptionMatches(err, type1) || PyErr_GivenExceptionMatches(err, type2))
#endif
#define __Pyx_PyException_Check(obj) __Pyx_TypeCheck(obj, PyExc_Exception)

/* FetchCommonType.proto */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type);

/* SwapException.proto */
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_ExceptionSwap(type, value, tb)  __Pyx__ExceptionSwap(__pyx_tstate, type, value, tb)
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb);
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb);
#endif

/* CoroutineBase.proto */
typedef PyObject *(*__pyx_coroutine_body_t)(PyObject *, PyThreadState *, PyObject *);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_ExcInfoStruct  _PyErr_StackItem
#else
typedef struct {
    PyObject *exc_type;
    PyObject *exc_value;
    PyObject *exc_traceback;
} __Pyx_ExcInfoStruct;
#endif
typedef struct {
    PyObject_HEAD
    __pyx_coroutine_body_t body;
    PyObject *closure;
    __Pyx_ExcInfoStruct gi_exc_state;
    PyObject *gi_weakreflist;
    PyObject *classobj;
    PyObject *yieldfrom;
    PyObject *gi_name;
    PyObject *gi_qualname;
    PyObject *gi_modulename;
    PyObject *gi_code;
    PyObject *gi_frame;
    int resume_label;
    char is_running;
} __pyx_CoroutineObject;
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
    PyTypeObject *type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
    PyObject *name, PyObject *qualname, PyObject *module_name);
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name);
static CYTHON_INLINE void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *self);
static int __Pyx_Coroutine_clear(PyObject *self);
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value);
static PyObject *__Pyx_Coroutine_Close(PyObject *self);
static PyObject *__Pyx_Coroutine_Throw(PyObject *gen, PyObject *args);
#if CYTHON_USE_EXC_INFO_STACK
#define __Pyx_Coroutine_SwapException(self)
#define __Pyx_Coroutine_ResetAndClearException(self)  __Pyx_Coroutine_ExceptionClear(&(self)->gi_exc_state)
#else
#define __Pyx_Coroutine_SwapException(self) {\
    __Pyx_ExceptionSwap(&(self)->gi_exc_state.exc_type, &(self)->gi_exc_state.exc_value, &(self)->gi_exc_state.exc_traceback);\
    __Pyx_Coroutine_ResetFrameBackpointer(&(self)->gi_exc_state);\
    }
#define __Pyx_Coroutine_ResetAndClearException(self) {\
    __Pyx_ExceptionReset((self)->gi_exc_state.exc_type, (self)->gi_exc_state.exc_value, (self)->gi_exc_state.exc_traceback);\
    (self)->gi_exc_state.exc_type = (self)->gi_exc_state.exc_value = (self)->gi_exc_state.exc_traceback = NULL;\
    }
#endif
#if CYTHON_FAST_THREAD_STATE
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__pyx_tstate, pvalue)
#else
#define __Pyx_PyGen_FetchStopIterationValue(pvalue)\
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, pvalue)
#endif
static int __Pyx_PyGen__FetchStopIterationValue(PyThreadState *tstate, PyObject **pvalue);
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state);

/* PatchModuleWithCoroutine.proto */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code);

/* PatchGeneratorABC.proto */
static int __Pyx_patch_abc(void);

/* Generator.proto */
#define __Pyx_Generator_USED
static PyTypeObject *__pyx_GeneratorType = 0;
#define __Pyx_Generator_CheckExact(obj) (Py_TYPE(obj) == __pyx_GeneratorType)
#define __Pyx_Generator_New(body, code, closure, name, qualname, module_name)\
    __Pyx__Coroutine_New(__pyx_GeneratorType, body, code, closure, name, qualname, module_name)
static PyObject *__Pyx_Generator_Next(PyObject *self);
static int __pyx_Generator_init(void);

/* CheckBinaryVersion.proto */
static int __Pyx_check_binary_version(void);

/* VoidPtrImport.proto */
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig);

/* InitStrings.proto */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t);

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_f_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_get(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, Py_ssize_t __pyx_v_i); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__invalidate(CYTHON_UNUSED struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_get_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self); /* proto*/
static void __pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_set_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_value); /* proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_modify(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_modification, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_get_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self); /* proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_equal_to(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, PyObject *__pyx_v_other); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_serialize(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_f_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase__create(PyObject *__pyx_v_name, PyObject *__pyx_v_symbol, double __pyx_v_mass, struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_composition); /* proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_equal_to(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_other); /* proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_is_a(CYTHON_UNUSED struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_category, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_serialize(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_is_tracked_for(CYTHON_UNUSED struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_category, int __pyx_skip_dispatch); /* proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition__create(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_amino_acid, PyObject *__pyx_v_modifications); /* proto*/
static double __pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_get_mass(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self); /* proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_has_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_modification, int __pyx_skip_dispatch); /* proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_is_modified(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_add_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_modification, int __pyx_skip_dispatch); /* proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_drop_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_modification, int __pyx_skip_dispatch); /* proto*/

/* Module declarations from 'cython' */

/* Module declarations from 'glypy.composition.ccomposition' */
static PyTypeObject *__pyx_ptype_5glypy_11composition_12ccomposition_CComposition = 0;
static PyObject **__pyx_vp_5glypy_11composition_12ccomposition__atom = 0;
#define __pyx_v_5glypy_11composition_12ccomposition__atom (*__pyx_vp_5glypy_11composition_12ccomposition__atom)
static PyObject **__pyx_vp_5glypy_11composition_12ccomposition__formula = 0;
#define __pyx_v_5glypy_11composition_12ccomposition__formula (*__pyx_vp_5glypy_11composition_12ccomposition__formula)
static PyObject **__pyx_vp_5glypy_11composition_12ccomposition__isotope_string = 0;
#define __pyx_v_5glypy_11composition_12ccomposition__isotope_string (*__pyx_vp_5glypy_11composition_12ccomposition__isotope_string)
static PyObject **__pyx_vp_5glypy_11composition_12ccomposition_isotope_pattern = 0;
#define __pyx_v_5glypy_11composition_12ccomposition_isotope_pattern (*__pyx_vp_5glypy_11composition_12ccomposition_isotope_pattern)
static PyObject **__pyx_vp_5glypy_11composition_12ccomposition_formula_pattern = 0;
#define __pyx_v_5glypy_11composition_12ccomposition_formula_pattern (*__pyx_vp_5glypy_11composition_12ccomposition_formula_pattern)

/* Module declarations from 'libc.string' */

/* Module declarations from 'libc.stdio' */

/* Module declarations from '__builtin__' */

/* Module declarations from 'cpython.type' */
static PyTypeObject *__pyx_ptype_7cpython_4type_type = 0;

/* Module declarations from 'cpython' */

/* Module declarations from 'cpython.object' */

/* Module declarations from 'cpython.list' */

/* Module declarations from 'cpython.sequence' */

/* Module declarations from 'glycopeptidepy._c.structure.base' */
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_SequencePosition = 0;
static PyTypeObject *__pyx_ptype_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ = 0;
static PyObject *__pyx_v_14glycopeptidepy_2_c_9structure_4base_formula = 0;
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_4base___pyx_unpickle_PeptideSequenceBase__set_state(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *, PyObject *); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_4base___pyx_unpickle_AminoAcidResidueBase__set_state(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *, PyObject *); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_4base___pyx_unpickle_ModificationBase__set_state(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *, PyObject *); /*proto*/
#define __Pyx_MODULE_NAME "glycopeptidepy._c.structure.base"
extern int __pyx_module_is_main_glycopeptidepy___c__structure__base;
int __pyx_module_is_main_glycopeptidepy___c__structure__base = 0;

/* Implementation of 'glycopeptidepy._c.structure.base' */
static PyObject *__pyx_builtin_AttributeError;
static PyObject *__pyx_builtin_NotImplemented;
static PyObject *__pyx_builtin_IndexError;
static PyObject *__pyx_builtin_range;
static PyObject *__pyx_builtin_ValueError;
static const char __pyx_k_new[] = "__new__";
static const char __pyx_k_pop[] = "pop";
static const char __pyx_k_r_r[] = "[%r, %r]";
static const char __pyx_k_args[] = "args";
static const char __pyx_k_dict[] = "__dict__";
static const char __pyx_k_is_a[] = "is_a";
static const char __pyx_k_iter[] = "__iter__";
static const char __pyx_k_main[] = "__main__";
static const char __pyx_k_mass[] = "mass";
static const char __pyx_k_name[] = "name";
static const char __pyx_k_self[] = "self";
static const char __pyx_k_send[] = "send";
static const char __pyx_k_test[] = "__test__";
static const char __pyx_k_class[] = "__class__";
static const char __pyx_k_clone[] = "clone";
static const char __pyx_k_close[] = "close";
static const char __pyx_k_parts[] = "parts";
static const char __pyx_k_range[] = "range";
static const char __pyx_k_throw[] = "throw";
static const char __pyx_k_format[] = "format";
static const char __pyx_k_import[] = "__import__";
static const char __pyx_k_modify[] = "modify";
static const char __pyx_k_name_2[] = "__name__";
static const char __pyx_k_pickle[] = "pickle";
static const char __pyx_k_reduce[] = "__reduce__";
static const char __pyx_k_symbol[] = "symbol";
static const char __pyx_k_update[] = "update";
static const char __pyx_k_formula[] = "formula";
static const char __pyx_k_getstate[] = "__getstate__";
static const char __pyx_k_pyx_type[] = "__pyx_type";
static const char __pyx_k_setstate[] = "__setstate__";
static const char __pyx_k_formula_2[] = "_formula";
static const char __pyx_k_pyx_state[] = "__pyx_state";
static const char __pyx_k_reduce_ex[] = "__reduce_ex__";
static const char __pyx_k_serialize[] = "serialize";
static const char __pyx_k_IndexError[] = "IndexError";
static const char __pyx_k_ValueError[] = "ValueError";
static const char __pyx_k_amino_acid[] = "amino_acid";
static const char __pyx_k_invalidate[] = "_invalidate";
static const char __pyx_k_pyx_result[] = "__pyx_result";
static const char __pyx_k_pyx_vtable[] = "__pyx_vtable__";
static const char __pyx_k_PickleError[] = "PickleError";
static const char __pyx_k_composition[] = "composition";
static const char __pyx_k_is_modified[] = "is_modified";
static const char __pyx_k_Modification[] = "Modification";
static const char __pyx_k_modification[] = "modification";
static const char __pyx_k_pyx_checksum[] = "__pyx_checksum";
static const char __pyx_k_stringsource[] = "stringsource";
static const char __pyx_k_TerminalGroup[] = "TerminalGroup";
static const char __pyx_k_modifications[] = "modifications";
static const char __pyx_k_reduce_cython[] = "__reduce_cython__";
static const char __pyx_k_AttributeError[] = "AttributeError";
static const char __pyx_k_NotImplemented[] = "NotImplemented";
static const char __pyx_k_calculate_mass[] = "_calculate_mass";
static const char __pyx_k_is_tracked_for[] = "is_tracked_for";
static const char __pyx_k_pyx_PickleError[] = "__pyx_PickleError";
static const char __pyx_k_setstate_cython[] = "__setstate_cython__";
static const char __pyx_k_ModificationBase[] = "ModificationBase";
static const char __pyx_k_SequencePosition[] = "SequencePosition";
static const char __pyx_k_add_modification[] = "add_modification";
static const char __pyx_k_base_composition[] = "base_composition";
static const char __pyx_k_has_modification[] = "has_modification";
static const char __pyx_k_drop_modification[] = "drop_modification";
static const char __pyx_k_glypy_composition[] = "glypy.composition";
static const char __pyx_k_cline_in_traceback[] = "cline_in_traceback";
static const char __pyx_k_PeptideSequenceBase[] = "PeptideSequenceBase";
static const char __pyx_k_AminoAcidResidueBase[] = "AminoAcidResidueBase";
static const char __pyx_k_SequencePosition___iter[] = "SequencePosition.__iter__";
static const char __pyx_k_Modification_s_not_found[] = "Modification %s not found";
static const char __pyx_k_pyx_unpickle_ModificationBase[] = "__pyx_unpickle_ModificationBase";
static const char __pyx_k_pyx_unpickle_AminoAcidResidueB[] = "__pyx_unpickle_AminoAcidResidueBase";
static const char __pyx_k_pyx_unpickle_PeptideSequenceBa[] = "__pyx_unpickle_PeptideSequenceBase";
static const char __pyx_k_self___class_____name___self_ba[] = "{self.__class__.__name__}({self.base_composition}, {self.modification})";
static const char __pyx_k_Incompatible_checksums_s_vs_0x54[] = "Incompatible checksums (%s vs 0x54d98d7 = (composition, mass, name))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x5b[] = "Incompatible checksums (%s vs 0x5b0643e = (_hash, composition, mass, name, neutral_loss, symbol))";
static const char __pyx_k_Incompatible_checksums_s_vs_0x9a[] = "Incompatible checksums (%s vs 0x9a27ba6 = (_c_term, _glycosylation_manager, _mass, _n_term, _peptide_composition, _total_composition, sequence))";
static const char __pyx_k_glycopeptidepy__c_structure_base[] = "glycopeptidepy._c.structure.base";
static const char __pyx_k_glycopeptidepy_structure_modific[] = "glycopeptidepy.structure.modification";
static PyObject *__pyx_n_s_AminoAcidResidueBase;
static PyObject *__pyx_n_s_AttributeError;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x54;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x5b;
static PyObject *__pyx_kp_s_Incompatible_checksums_s_vs_0x9a;
static PyObject *__pyx_n_s_IndexError;
static PyObject *__pyx_n_s_Modification;
static PyObject *__pyx_n_s_ModificationBase;
static PyObject *__pyx_kp_s_Modification_s_not_found;
static PyObject *__pyx_n_s_NotImplemented;
static PyObject *__pyx_n_s_PeptideSequenceBase;
static PyObject *__pyx_n_s_PickleError;
static PyObject *__pyx_n_s_SequencePosition;
static PyObject *__pyx_n_s_SequencePosition___iter;
static PyObject *__pyx_n_s_TerminalGroup;
static PyObject *__pyx_n_s_ValueError;
static PyObject *__pyx_n_s_add_modification;
static PyObject *__pyx_n_s_amino_acid;
static PyObject *__pyx_n_s_args;
static PyObject *__pyx_n_s_base_composition;
static PyObject *__pyx_n_s_calculate_mass;
static PyObject *__pyx_n_s_class;
static PyObject *__pyx_n_s_cline_in_traceback;
static PyObject *__pyx_n_s_clone;
static PyObject *__pyx_n_s_close;
static PyObject *__pyx_n_s_composition;
static PyObject *__pyx_n_s_dict;
static PyObject *__pyx_n_s_drop_modification;
static PyObject *__pyx_n_s_format;
static PyObject *__pyx_n_s_formula;
static PyObject *__pyx_n_s_formula_2;
static PyObject *__pyx_n_s_getstate;
static PyObject *__pyx_n_s_glycopeptidepy__c_structure_base;
static PyObject *__pyx_n_s_glycopeptidepy_structure_modific;
static PyObject *__pyx_n_s_glypy_composition;
static PyObject *__pyx_n_s_has_modification;
static PyObject *__pyx_n_s_import;
static PyObject *__pyx_n_s_invalidate;
static PyObject *__pyx_n_s_is_a;
static PyObject *__pyx_n_s_is_modified;
static PyObject *__pyx_n_s_is_tracked_for;
static PyObject *__pyx_n_s_iter;
static PyObject *__pyx_n_s_main;
static PyObject *__pyx_n_s_mass;
static PyObject *__pyx_n_s_modification;
static PyObject *__pyx_n_s_modifications;
static PyObject *__pyx_n_s_modify;
static PyObject *__pyx_n_s_name;
static PyObject *__pyx_n_s_name_2;
static PyObject *__pyx_n_s_new;
static PyObject *__pyx_n_s_parts;
static PyObject *__pyx_n_s_pickle;
static PyObject *__pyx_n_s_pop;
static PyObject *__pyx_n_s_pyx_PickleError;
static PyObject *__pyx_n_s_pyx_checksum;
static PyObject *__pyx_n_s_pyx_result;
static PyObject *__pyx_n_s_pyx_state;
static PyObject *__pyx_n_s_pyx_type;
static PyObject *__pyx_n_s_pyx_unpickle_AminoAcidResidueB;
static PyObject *__pyx_n_s_pyx_unpickle_ModificationBase;
static PyObject *__pyx_n_s_pyx_unpickle_PeptideSequenceBa;
static PyObject *__pyx_n_s_pyx_vtable;
static PyObject *__pyx_kp_s_r_r;
static PyObject *__pyx_n_s_range;
static PyObject *__pyx_n_s_reduce;
static PyObject *__pyx_n_s_reduce_cython;
static PyObject *__pyx_n_s_reduce_ex;
static PyObject *__pyx_n_s_self;
static PyObject *__pyx_kp_s_self___class_____name___self_ba;
static PyObject *__pyx_n_s_send;
static PyObject *__pyx_n_s_serialize;
static PyObject *__pyx_n_s_setstate;
static PyObject *__pyx_n_s_setstate_cython;
static PyObject *__pyx_kp_s_stringsource;
static PyObject *__pyx_n_s_symbol;
static PyObject *__pyx_n_s_test;
static PyObject *__pyx_n_s_throw;
static PyObject *__pyx_n_s_update;
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__invalidate(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_5_mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_5_mass_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_2__reduce_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_4__setstate_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup___init__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, PyObject *__pyx_v_base_composition, PyObject *__pyx_v_modification); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_2_calculate_mass(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_4clone(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_6__reduce__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_12modification___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_12modification_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_8modify(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_modification); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_11composition___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_10__repr__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_12__str__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_14__eq__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16__ne__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static Py_hash_t __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_18__hash__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_20serialize(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_4mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_4mass_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase___copy__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self); /* proto */
static Py_hash_t __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_2__hash__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4__eq__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6__ne__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4mass_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_5_hash___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_5_hash_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_8__reduce_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_10__setstate_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase___copy__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_2is_a(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, PyObject *__pyx_v_category); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4serialize(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_6is_tracked_for(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, PyObject *__pyx_v_category); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4mass_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_8__reduce_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_10__setstate_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, PyObject *__pyx_v___pyx_state); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition___init__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_parts); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_2__iter__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_5__getitem__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_i); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_7__setitem__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_i, PyObject *__pyx_v_value); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_9__eq__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_11__ne__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_other); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13__reduce__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self); /* proto */
static Py_ssize_t __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_15__len__(CYTHON_UNUSED struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_17__repr__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_19has_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_modification); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_21is_modified(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_23add_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_modification); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_25drop_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_modification); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_4mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_value); /* proto */
static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base___pyx_unpickle_PeptideSequenceBase(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_2__pyx_unpickle_AminoAcidResidueBase(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_4__pyx_unpickle_ModificationBase(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state); /* proto */
static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_4base_TerminalGroup(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_4base_ModificationBase(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_4base_SequencePosition(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__(PyTypeObject *t, PyObject *a, PyObject *k); /*proto*/
static PyObject *__pyx_int_0;
static PyObject *__pyx_int_1;
static PyObject *__pyx_int_88971479;
static PyObject *__pyx_int_95446078;
static PyObject *__pyx_int_161643430;
static PyObject *__pyx_tuple_;
static PyObject *__pyx_tuple__3;
static PyObject *__pyx_tuple__5;
static PyObject *__pyx_codeobj__2;
static PyObject *__pyx_codeobj__4;
static PyObject *__pyx_codeobj__6;
/* Late includes */

/* "glycopeptidepy/_c/structure/base.pyx":16
 * cdef class PeptideSequenceBase(object):
 * 
 *     cdef SequencePosition get(self, ssize_t i):             # <<<<<<<<<<<<<<
 *         IF SAFE_GET:
 *             return <SequencePosition>PyList_GetItem(self.sequence, i)
 */

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_f_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_get(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, Py_ssize_t __pyx_v_i) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":18
 *     cdef SequencePosition get(self, ssize_t i):
 *         IF SAFE_GET:
 *             return <SequencePosition>PyList_GetItem(self.sequence, i)             # <<<<<<<<<<<<<<
 *         ELSE:
 *             return <SequencePosition>PyList_GET_ITEM(self.sequence, i)
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_1 = __pyx_v_self->sequence;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyList_GetItem(__pyx_t_1, __pyx_v_i); if (unlikely(__pyx_t_2 == ((PyObject *)NULL))) __PYX_ERR(0, 18, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_INCREF(((PyObject *)((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_t_2)));
  __pyx_r = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_t_2);
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":16
 * cdef class PeptideSequenceBase(object):
 * 
 *     cdef SequencePosition get(self, ssize_t i):             # <<<<<<<<<<<<<<
 *         IF SAFE_GET:
 *             return <SequencePosition>PyList_GetItem(self.sequence, i)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.PeptideSequenceBase.get", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":22
 *             return <SequencePosition>PyList_GET_ITEM(self.sequence, i)
 * 
 *     cpdef _invalidate(self):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_1_invalidate(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__invalidate(CYTHON_UNUSED struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_invalidate", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_invalidate); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 22, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_1_invalidate)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 22, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/base.pyx":23
 * 
 *     cpdef _invalidate(self):
 *         pass             # <<<<<<<<<<<<<<
 * 
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.PeptideSequenceBase._invalidate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":22
 *             return <SequencePosition>PyList_GET_ITEM(self.sequence, i)
 * 
 *     cpdef _invalidate(self):             # <<<<<<<<<<<<<<
 *         pass
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_1_invalidate(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__invalidate[] = "PeptideSequenceBase._invalidate(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_1_invalidate(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_invalidate (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__invalidate(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__invalidate(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_invalidate", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__invalidate(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 22, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.PeptideSequenceBase._invalidate", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":7
 * cdef class PeptideSequenceBase(object):
 *     cdef:
 *         public list sequence             # <<<<<<<<<<<<<<
 *         public double _mass
 *         public object _glycosylation_manager
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->sequence);
  __pyx_r = __pyx_v_self->sequence;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 7, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->sequence);
  __Pyx_DECREF(__pyx_v_self->sequence);
  __pyx_v_self->sequence = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.PeptideSequenceBase.sequence.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->sequence);
  __Pyx_DECREF(__pyx_v_self->sequence);
  __pyx_v_self->sequence = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":8
 *     cdef:
 *         public list sequence
 *         public double _mass             # <<<<<<<<<<<<<<
 *         public object _glycosylation_manager
 *         public TerminalGroup _n_term
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_5_mass_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_5_mass_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_5_mass___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_5_mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 8, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.PeptideSequenceBase._mass.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_5_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_5_mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_5_mass_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_5_mass_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 8, __pyx_L1_error)
  __pyx_v_self->_mass = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.PeptideSequenceBase._mass.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":9
 *         public list sequence
 *         public double _mass
 *         public object _glycosylation_manager             # <<<<<<<<<<<<<<
 *         public TerminalGroup _n_term
 *         public TerminalGroup _c_term
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->_glycosylation_manager);
  __pyx_r = __pyx_v_self->_glycosylation_manager;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->_glycosylation_manager);
  __Pyx_DECREF(__pyx_v_self->_glycosylation_manager);
  __pyx_v_self->_glycosylation_manager = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_glycosylation_manager);
  __Pyx_DECREF(__pyx_v_self->_glycosylation_manager);
  __pyx_v_self->_glycosylation_manager = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":10
 *         public double _mass
 *         public object _glycosylation_manager
 *         public TerminalGroup _n_term             # <<<<<<<<<<<<<<
 *         public TerminalGroup _c_term
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_n_term));
  __pyx_r = ((PyObject *)__pyx_v_self->_n_term);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup))))) __PYX_ERR(1, 10, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_n_term);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_n_term));
  __pyx_v_self->_n_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.PeptideSequenceBase._n_term.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_n_term);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_n_term));
  __pyx_v_self->_n_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":11
 *         public object _glycosylation_manager
 *         public TerminalGroup _n_term
 *         public TerminalGroup _c_term             # <<<<<<<<<<<<<<
 * 
 *         public CComposition _total_composition
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_c_term));
  __pyx_r = ((PyObject *)__pyx_v_self->_c_term);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup))))) __PYX_ERR(1, 11, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_c_term);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_c_term));
  __pyx_v_self->_c_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.PeptideSequenceBase._c_term.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_c_term);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_c_term));
  __pyx_v_self->_c_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":13
 *         public TerminalGroup _c_term
 * 
 *         public CComposition _total_composition             # <<<<<<<<<<<<<<
 *         public CComposition _peptide_composition
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_total_composition));
  __pyx_r = ((PyObject *)__pyx_v_self->_total_composition);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(1, 13, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_total_composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_total_composition));
  __pyx_v_self->_total_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.PeptideSequenceBase._total_composition.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_total_composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_total_composition));
  __pyx_v_self->_total_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":14
 * 
 *         public CComposition _total_composition
 *         public CComposition _peptide_composition             # <<<<<<<<<<<<<<
 * 
 *     cdef SequencePosition get(self, ssize_t i)
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_peptide_composition));
  __pyx_r = ((PyObject *)__pyx_v_self->_peptide_composition);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(1, 14, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_peptide_composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_peptide_composition));
  __pyx_v_self->_peptide_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.PeptideSequenceBase._peptide_composition.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_peptide_composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_peptide_composition));
  __pyx_v_self->_peptide_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_2__reduce_cython__[] = "PeptideSequenceBase.__reduce_cython__(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_3__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_2__reduce_cython__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_2__reduce_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._c_term, self._glycosylation_manager, self._mass, self._n_term, self._peptide_composition, self._total_composition, self.sequence)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(7); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_c_term));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_c_term));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self->_c_term));
  __Pyx_INCREF(__pyx_v_self->_glycosylation_manager);
  __Pyx_GIVEREF(__pyx_v_self->_glycosylation_manager);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_self->_glycosylation_manager);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_n_term));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_n_term));
  PyTuple_SET_ITEM(__pyx_t_2, 3, ((PyObject *)__pyx_v_self->_n_term));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_peptide_composition));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_peptide_composition));
  PyTuple_SET_ITEM(__pyx_t_2, 4, ((PyObject *)__pyx_v_self->_peptide_composition));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_total_composition));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->_total_composition));
  PyTuple_SET_ITEM(__pyx_t_2, 5, ((PyObject *)__pyx_v_self->_total_composition));
  __Pyx_INCREF(__pyx_v_self->sequence);
  __Pyx_GIVEREF(__pyx_v_self->sequence);
  PyTuple_SET_ITEM(__pyx_t_2, 6, __pyx_v_self->sequence);
  __pyx_t_1 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._c_term, self._glycosylation_manager, self._mass, self._n_term, self._peptide_composition, self._total_composition, self.sequence)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_2 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__dict = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":7
 *     state = (self._c_term, self._glycosylation_manager, self._mass, self._n_term, self._peptide_composition, self._total_composition, self.sequence)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_3 = (__pyx_v__dict != Py_None);
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v__dict);
    __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self._c_term is not None or self._glycosylation_manager is not None or self._n_term is not None or self._peptide_composition is not None or self._total_composition is not None or self.sequence is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._c_term, self._glycosylation_manager, self._mass, self._n_term, self._peptide_composition, self._total_composition, self.sequence)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self._c_term is not None or self._glycosylation_manager is not None or self._n_term is not None or self._peptide_composition is not None or self._total_composition is not None or self.sequence is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_PeptideSequenceBase, (type(self), 0x9a27ba6, None), state
 */
  /*else*/ {
    __pyx_t_3 = (((PyObject *)__pyx_v_self->_c_term) != Py_None);
    __pyx_t_5 = (__pyx_t_3 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->_glycosylation_manager != Py_None);
    __pyx_t_3 = (__pyx_t_5 != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_4 = __pyx_t_3;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_3 = (((PyObject *)__pyx_v_self->_n_term) != Py_None);
    __pyx_t_5 = (__pyx_t_3 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (((PyObject *)__pyx_v_self->_peptide_composition) != Py_None);
    __pyx_t_3 = (__pyx_t_5 != 0);
    if (!__pyx_t_3) {
    } else {
      __pyx_t_4 = __pyx_t_3;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_3 = (((PyObject *)__pyx_v_self->_total_composition) != Py_None);
    __pyx_t_5 = (__pyx_t_3 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->sequence != ((PyObject*)Py_None));
    __pyx_t_3 = (__pyx_t_5 != 0);
    __pyx_t_4 = __pyx_t_3;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_4;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._c_term is not None or self._glycosylation_manager is not None or self._n_term is not None or self._peptide_composition is not None or self._total_composition is not None or self.sequence is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PeptideSequenceBase, (type(self), 0x9a27ba6, None), state
 *     else:
 */
  __pyx_t_4 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_4) {

    /* "(tree fragment)":13
 *         use_setstate = self._c_term is not None or self._glycosylation_manager is not None or self._n_term is not None or self._peptide_composition is not None or self._total_composition is not None or self.sequence is not None
 *     if use_setstate:
 *         return __pyx_unpickle_PeptideSequenceBase, (type(self), 0x9a27ba6, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_PeptideSequenceBase, (type(self), 0x9a27ba6, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_PeptideSequenceBa); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_161643430);
    __Pyx_GIVEREF(__pyx_int_161643430);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_int_161643430);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_2, 2, Py_None);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_2);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self._c_term is not None or self._glycosylation_manager is not None or self._n_term is not None or self._peptide_composition is not None or self._total_composition is not None or self.sequence is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_PeptideSequenceBase, (type(self), 0x9a27ba6, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_PeptideSequenceBase, (type(self), 0x9a27ba6, None), state
 *     else:
 *         return __pyx_unpickle_PeptideSequenceBase, (type(self), 0x9a27ba6, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PeptideSequenceBase__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle_PeptideSequenceBa); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_161643430);
    __Pyx_GIVEREF(__pyx_int_161643430);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_int_161643430);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
    __pyx_t_6 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.PeptideSequenceBase.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PeptideSequenceBase, (type(self), 0x9a27ba6, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PeptideSequenceBase__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_4__setstate_cython__[] = "PeptideSequenceBase.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_5__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_4__setstate_cython__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_4__setstate_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_PeptideSequenceBase, (type(self), 0x9a27ba6, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_PeptideSequenceBase__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_4base___pyx_unpickle_PeptideSequenceBase__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_PeptideSequenceBase, (type(self), 0x9a27ba6, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_PeptideSequenceBase__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.PeptideSequenceBase.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":28
 * @cython.freelist(100)
 * cdef class TerminalGroup(object):
 *     def __init__(self, base_composition, modification=None):             # <<<<<<<<<<<<<<
 *         if not isinstance(base_composition, CComposition):
 *             base_composition = CComposition(base_composition)
 */

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_base_composition = 0;
  PyObject *__pyx_v_modification = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_base_composition,&__pyx_n_s_modification,0};
    PyObject* values[2] = {0,0};
    values[1] = ((PyObject *)Py_None);
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_base_composition)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (kw_args > 0) {
          PyObject* value = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_modification);
          if (value) { values[1] = value; kw_args--; }
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 28, __pyx_L3_error)
      }
    } else {
      switch (PyTuple_GET_SIZE(__pyx_args)) {
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        break;
        default: goto __pyx_L5_argtuple_error;
      }
    }
    __pyx_v_base_composition = values[0];
    __pyx_v_modification = values[1];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 0, 1, 2, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 28, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup___init__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self), __pyx_v_base_composition, __pyx_v_modification);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup___init__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, PyObject *__pyx_v_base_composition, PyObject *__pyx_v_modification) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  double __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);
  __Pyx_INCREF(__pyx_v_base_composition);

  /* "glycopeptidepy/_c/structure/base.pyx":29
 * cdef class TerminalGroup(object):
 *     def __init__(self, base_composition, modification=None):
 *         if not isinstance(base_composition, CComposition):             # <<<<<<<<<<<<<<
 *             base_composition = CComposition(base_composition)
 *         self.base_composition = base_composition
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_base_composition, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition); 
  __pyx_t_2 = ((!(__pyx_t_1 != 0)) != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/base.pyx":30
 *     def __init__(self, base_composition, modification=None):
 *         if not isinstance(base_composition, CComposition):
 *             base_composition = CComposition(base_composition)             # <<<<<<<<<<<<<<
 *         self.base_composition = base_composition
 *         self._modification = None
 */
    __pyx_t_3 = __Pyx_PyObject_CallOneArg(((PyObject *)__pyx_ptype_5glypy_11composition_12ccomposition_CComposition), __pyx_v_base_composition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 30, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF_SET(__pyx_v_base_composition, __pyx_t_3);
    __pyx_t_3 = 0;

    /* "glycopeptidepy/_c/structure/base.pyx":29
 * cdef class TerminalGroup(object):
 *     def __init__(self, base_composition, modification=None):
 *         if not isinstance(base_composition, CComposition):             # <<<<<<<<<<<<<<
 *             base_composition = CComposition(base_composition)
 *         self.base_composition = base_composition
 */
  }

  /* "glycopeptidepy/_c/structure/base.pyx":31
 *         if not isinstance(base_composition, CComposition):
 *             base_composition = CComposition(base_composition)
 *         self.base_composition = base_composition             # <<<<<<<<<<<<<<
 *         self._modification = None
 *         if modification is not None:
 */
  if (!(likely(((__pyx_v_base_composition) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_base_composition, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 31, __pyx_L1_error)
  __pyx_t_3 = __pyx_v_base_composition;
  __Pyx_INCREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_3);
  __Pyx_GOTREF(__pyx_v_self->base_composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->base_composition));
  __pyx_v_self->base_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "glycopeptidepy/_c/structure/base.pyx":32
 *             base_composition = CComposition(base_composition)
 *         self.base_composition = base_composition
 *         self._modification = None             # <<<<<<<<<<<<<<
 *         if modification is not None:
 *             self.modification = modification
 */
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_modification);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_modification));
  __pyx_v_self->_modification = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)Py_None);

  /* "glycopeptidepy/_c/structure/base.pyx":33
 *         self.base_composition = base_composition
 *         self._modification = None
 *         if modification is not None:             # <<<<<<<<<<<<<<
 *             self.modification = modification
 *         self.mass = self._calculate_mass()
 */
  __pyx_t_2 = (__pyx_v_modification != Py_None);
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "glycopeptidepy/_c/structure/base.pyx":34
 *         self._modification = None
 *         if modification is not None:
 *             self.modification = modification             # <<<<<<<<<<<<<<
 *         self.mass = self._calculate_mass()
 * 
 */
    if (__Pyx_PyObject_SetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_modification, __pyx_v_modification) < 0) __PYX_ERR(0, 34, __pyx_L1_error)

    /* "glycopeptidepy/_c/structure/base.pyx":33
 *         self.base_composition = base_composition
 *         self._modification = None
 *         if modification is not None:             # <<<<<<<<<<<<<<
 *             self.modification = modification
 *         self.mass = self._calculate_mass()
 */
  }

  /* "glycopeptidepy/_c/structure/base.pyx":35
 *         if modification is not None:
 *             self.modification = modification
 *         self.mass = self._calculate_mass()             # <<<<<<<<<<<<<<
 * 
 *     def _calculate_mass(self):
 */
  __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_calculate_mass); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_4);
  __pyx_t_5 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_4))) {
    __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_4);
    if (likely(__pyx_t_5)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_4);
      __Pyx_INCREF(__pyx_t_5);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_4, function);
    }
  }
  __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_CallOneArg(__pyx_t_4, __pyx_t_5) : __Pyx_PyObject_CallNoArg(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  __pyx_t_6 = __pyx_PyFloat_AsDouble(__pyx_t_3); if (unlikely((__pyx_t_6 == (double)-1) && PyErr_Occurred())) __PYX_ERR(0, 35, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_v_self->mass = __pyx_t_6;

  /* "glycopeptidepy/_c/structure/base.pyx":28
 * @cython.freelist(100)
 * cdef class TerminalGroup(object):
 *     def __init__(self, base_composition, modification=None):             # <<<<<<<<<<<<<<
 *         if not isinstance(base_composition, CComposition):
 *             base_composition = CComposition(base_composition)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_base_composition);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":37
 *         self.mass = self._calculate_mass()
 * 
 *     def _calculate_mass(self):             # <<<<<<<<<<<<<<
 *         base_mass = self.base_composition.mass
 *         mod = self.modification
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_3_calculate_mass(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_2_calculate_mass[] = "TerminalGroup._calculate_mass(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_3_calculate_mass(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("_calculate_mass (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_2_calculate_mass(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_2_calculate_mass(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self) {
  PyObject *__pyx_v_base_mass = NULL;
  PyObject *__pyx_v_mod = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_calculate_mass", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":38
 * 
 *     def _calculate_mass(self):
 *         base_mass = self.base_composition.mass             # <<<<<<<<<<<<<<
 *         mod = self.modification
 *         if mod is not None:
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self->base_composition), __pyx_n_s_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 38, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_base_mass = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/base.pyx":39
 *     def _calculate_mass(self):
 *         base_mass = self.base_composition.mass
 *         mod = self.modification             # <<<<<<<<<<<<<<
 *         if mod is not None:
 *             base_mass += mod.mass
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_modification); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_mod = __pyx_t_1;
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/base.pyx":40
 *         base_mass = self.base_composition.mass
 *         mod = self.modification
 *         if mod is not None:             # <<<<<<<<<<<<<<
 *             base_mass += mod.mass
 *         return base_mass
 */
  __pyx_t_2 = (__pyx_v_mod != Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "glycopeptidepy/_c/structure/base.pyx":41
 *         mod = self.modification
 *         if mod is not None:
 *             base_mass += mod.mass             # <<<<<<<<<<<<<<
 *         return base_mass
 * 
 */
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_mod, __pyx_n_s_mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 41, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_4 = PyNumber_InPlaceAdd(__pyx_v_base_mass, __pyx_t_1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 41, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_DECREF_SET(__pyx_v_base_mass, __pyx_t_4);
    __pyx_t_4 = 0;

    /* "glycopeptidepy/_c/structure/base.pyx":40
 *         base_mass = self.base_composition.mass
 *         mod = self.modification
 *         if mod is not None:             # <<<<<<<<<<<<<<
 *             base_mass += mod.mass
 *         return base_mass
 */
  }

  /* "glycopeptidepy/_c/structure/base.pyx":42
 *         if mod is not None:
 *             base_mass += mod.mass
 *         return base_mass             # <<<<<<<<<<<<<<
 * 
 *     def clone(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_base_mass);
  __pyx_r = __pyx_v_base_mass;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":37
 *         self.mass = self._calculate_mass()
 * 
 *     def _calculate_mass(self):             # <<<<<<<<<<<<<<
 *         base_mass = self.base_composition.mass
 *         mod = self.modification
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup._calculate_mass", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_base_mass);
  __Pyx_XDECREF(__pyx_v_mod);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":44
 *         return base_mass
 * 
 *     def clone(self):             # <<<<<<<<<<<<<<
 *         return self.__class__(self.base_composition, self.modification)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_5clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_4clone[] = "TerminalGroup.clone(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_5clone(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("clone (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_4clone(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_4clone(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("clone", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":45
 * 
 *     def clone(self):
 *         return self.__class__(self.base_composition, self.modification)             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_modification); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 45, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_t_4 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_self->base_composition), __pyx_t_3};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 45, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_4, ((PyObject *)__pyx_v_self->base_composition), __pyx_t_3};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 45, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  } else
  #endif
  {
    __pyx_t_6 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_6)) __PYX_ERR(0, 45, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    if (__pyx_t_4) {
      __Pyx_GIVEREF(__pyx_t_4); PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_4); __pyx_t_4 = NULL;
    }
    __Pyx_INCREF(((PyObject *)__pyx_v_self->base_composition));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self->base_composition));
    PyTuple_SET_ITEM(__pyx_t_6, 0+__pyx_t_5, ((PyObject *)__pyx_v_self->base_composition));
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_6, 1+__pyx_t_5, __pyx_t_3);
    __pyx_t_3 = 0;
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_6, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 45, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":44
 *         return base_mass
 * 
 *     def clone(self):             # <<<<<<<<<<<<<<
 *         return self.__class__(self.base_composition, self.modification)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.clone", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":47
 *         return self.__class__(self.base_composition, self.modification)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return self.__class__, (self.base_composition, self.modification)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_7__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_6__reduce__[] = "TerminalGroup.__reduce__(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_7__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_6__reduce__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_6__reduce__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":48
 * 
 *     def __reduce__(self):
 *         return self.__class__, (self.base_composition, self.modification)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_modification); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->base_composition));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->base_composition));
  PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)__pyx_v_self->base_composition));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 48, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":47
 *         return self.__class__(self.base_composition, self.modification)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return self.__class__, (self.base_composition, self.modification)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":51
 * 
 * 
 *     cdef ModificationBase get_modification(self):             # <<<<<<<<<<<<<<
 *         return self._modification
 * 
 */

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_get_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("get_modification", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":52
 * 
 *     cdef ModificationBase get_modification(self):
 *         return self._modification             # <<<<<<<<<<<<<<
 * 
 *     cdef void set_modification(self, ModificationBase value):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_modification));
  __pyx_r = __pyx_v_self->_modification;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":51
 * 
 * 
 *     cdef ModificationBase get_modification(self):             # <<<<<<<<<<<<<<
 *         return self._modification
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":54
 *         return self._modification
 * 
 *     cdef void set_modification(self, ModificationBase value):             # <<<<<<<<<<<<<<
 *         cdef:
 *             double new_mass, old_mass
 */

static void __pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_set_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_value) {
  double __pyx_v_new_mass;
  double __pyx_v_old_mass;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  double __pyx_t_3;
  __Pyx_RefNannySetupContext("set_modification", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":57
 *         cdef:
 *             double new_mass, old_mass
 *         if value is not None:             # <<<<<<<<<<<<<<
 *             new_mass = value.mass
 *         else:
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_value) != Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/base.pyx":58
 *             double new_mass, old_mass
 *         if value is not None:
 *             new_mass = value.mass             # <<<<<<<<<<<<<<
 *         else:
 *             new_mass = 0
 */
    __pyx_t_3 = __pyx_v_value->mass;
    __pyx_v_new_mass = __pyx_t_3;

    /* "glycopeptidepy/_c/structure/base.pyx":57
 *         cdef:
 *             double new_mass, old_mass
 *         if value is not None:             # <<<<<<<<<<<<<<
 *             new_mass = value.mass
 *         else:
 */
    goto __pyx_L3;
  }

  /* "glycopeptidepy/_c/structure/base.pyx":60
 *             new_mass = value.mass
 *         else:
 *             new_mass = 0             # <<<<<<<<<<<<<<
 *         if self._modification is not None:
 *             old_mass = self._modification.mass
 */
  /*else*/ {
    __pyx_v_new_mass = 0.0;
  }
  __pyx_L3:;

  /* "glycopeptidepy/_c/structure/base.pyx":61
 *         else:
 *             new_mass = 0
 *         if self._modification is not None:             # <<<<<<<<<<<<<<
 *             old_mass = self._modification.mass
 *         else:
 */
  __pyx_t_2 = (((PyObject *)__pyx_v_self->_modification) != Py_None);
  __pyx_t_1 = (__pyx_t_2 != 0);
  if (__pyx_t_1) {

    /* "glycopeptidepy/_c/structure/base.pyx":62
 *             new_mass = 0
 *         if self._modification is not None:
 *             old_mass = self._modification.mass             # <<<<<<<<<<<<<<
 *         else:
 *             old_mass = 0
 */
    __pyx_t_3 = __pyx_v_self->_modification->mass;
    __pyx_v_old_mass = __pyx_t_3;

    /* "glycopeptidepy/_c/structure/base.pyx":61
 *         else:
 *             new_mass = 0
 *         if self._modification is not None:             # <<<<<<<<<<<<<<
 *             old_mass = self._modification.mass
 *         else:
 */
    goto __pyx_L4;
  }

  /* "glycopeptidepy/_c/structure/base.pyx":64
 *             old_mass = self._modification.mass
 *         else:
 *             old_mass = 0             # <<<<<<<<<<<<<<
 *         self.mass += new_mass - old_mass
 *         self._modification = value
 */
  /*else*/ {
    __pyx_v_old_mass = 0.0;
  }
  __pyx_L4:;

  /* "glycopeptidepy/_c/structure/base.pyx":65
 *         else:
 *             old_mass = 0
 *         self.mass += new_mass - old_mass             # <<<<<<<<<<<<<<
 *         self._modification = value
 * 
 */
  __pyx_v_self->mass = (__pyx_v_self->mass + (__pyx_v_new_mass - __pyx_v_old_mass));

  /* "glycopeptidepy/_c/structure/base.pyx":66
 *             old_mass = 0
 *         self.mass += new_mass - old_mass
 *         self._modification = value             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_value));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_value));
  __Pyx_GOTREF(__pyx_v_self->_modification);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_modification));
  __pyx_v_self->_modification = __pyx_v_value;

  /* "glycopeptidepy/_c/structure/base.pyx":54
 *         return self._modification
 * 
 *     cdef void set_modification(self, ModificationBase value):             # <<<<<<<<<<<<<<
 *         cdef:
 *             double new_mass, old_mass
 */

  /* function exit code */
  __Pyx_RefNannyFinishContext();
}

/* "glycopeptidepy/_c/structure/base.pyx":69
 * 
 *     @property
 *     def modification(self):             # <<<<<<<<<<<<<<
 *         return self.get_modification()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_12modification_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_12modification_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_12modification___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_12modification___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":70
 *     @property
 *     def modification(self):
 *         return self.get_modification()             # <<<<<<<<<<<<<<
 * 
 *     @modification.setter
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self->__pyx_vtab)->get_modification(__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 70, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":69
 * 
 *     @property
 *     def modification(self):             # <<<<<<<<<<<<<<
 *         return self.get_modification()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.modification.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":73
 * 
 *     @modification.setter
 *     def modification(self, ModificationBase value):             # <<<<<<<<<<<<<<
 *         self.set_modification(value)
 * 
 */

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_12modification_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_12modification_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_value), __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase, 1, "value", 0))) __PYX_ERR(0, 73, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_12modification_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self), ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_value));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_12modification_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":74
 *     @modification.setter
 *     def modification(self, ModificationBase value):
 *         self.set_modification(value)             # <<<<<<<<<<<<<<
 * 
 *     cpdef TerminalGroup modify(self, ModificationBase modification):
 */
  ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self->__pyx_vtab)->set_modification(__pyx_v_self, __pyx_v_value);

  /* "glycopeptidepy/_c/structure/base.pyx":73
 * 
 *     @modification.setter
 *     def modification(self, ModificationBase value):             # <<<<<<<<<<<<<<
 *         self.set_modification(value)
 * 
 */

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":76
 *         self.set_modification(value)
 * 
 *     cpdef TerminalGroup modify(self, ModificationBase modification):             # <<<<<<<<<<<<<<
 *         return self.__class__(self.base_composition, modification)
 * 
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_9modify(PyObject *__pyx_v_self, PyObject *__pyx_v_modification); /*proto*/
static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_modify(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_modification, int __pyx_skip_dispatch) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("modify", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_modify); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 76, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_9modify)) {
        __Pyx_XDECREF(((PyObject *)__pyx_r));
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, ((PyObject *)__pyx_v_modification)) : __Pyx_PyObject_CallOneArg(__pyx_t_3, ((PyObject *)__pyx_v_modification));
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 76, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(((__pyx_t_2) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_2, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup))))) __PYX_ERR(0, 76, __pyx_L1_error)
        __pyx_r = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/base.pyx":77
 * 
 *     cpdef TerminalGroup modify(self, ModificationBase modification):
 *         return self.__class__(self.base_composition, modification)             # <<<<<<<<<<<<<<
 * 
 *     cdef CComposition get_composition(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 77, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  __pyx_t_5 = 0;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
      __pyx_t_5 = 1;
    }
  }
  #if CYTHON_FAST_PYCALL
  if (PyFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, ((PyObject *)__pyx_v_self->base_composition), ((PyObject *)__pyx_v_modification)};
    __pyx_t_1 = __Pyx_PyFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 77, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  #if CYTHON_FAST_PYCCALL
  if (__Pyx_PyFastCFunction_Check(__pyx_t_2)) {
    PyObject *__pyx_temp[3] = {__pyx_t_3, ((PyObject *)__pyx_v_self->base_composition), ((PyObject *)__pyx_v_modification)};
    __pyx_t_1 = __Pyx_PyCFunction_FastCall(__pyx_t_2, __pyx_temp+1-__pyx_t_5, 2+__pyx_t_5); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 77, __pyx_L1_error)
    __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_GOTREF(__pyx_t_1);
  } else
  #endif
  {
    __pyx_t_4 = PyTuple_New(2+__pyx_t_5); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 77, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    if (__pyx_t_3) {
      __Pyx_GIVEREF(__pyx_t_3); PyTuple_SET_ITEM(__pyx_t_4, 0, __pyx_t_3); __pyx_t_3 = NULL;
    }
    __Pyx_INCREF(((PyObject *)__pyx_v_self->base_composition));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_self->base_composition));
    PyTuple_SET_ITEM(__pyx_t_4, 0+__pyx_t_5, ((PyObject *)__pyx_v_self->base_composition));
    __Pyx_INCREF(((PyObject *)__pyx_v_modification));
    __Pyx_GIVEREF(((PyObject *)__pyx_v_modification));
    PyTuple_SET_ITEM(__pyx_t_4, 1+__pyx_t_5, ((PyObject *)__pyx_v_modification));
    __pyx_t_1 = __Pyx_PyObject_Call(__pyx_t_2, __pyx_t_4, NULL); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 77, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
  }
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup))))) __PYX_ERR(0, 77, __pyx_L1_error)
  __pyx_r = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":76
 *         self.set_modification(value)
 * 
 *     cpdef TerminalGroup modify(self, ModificationBase modification):             # <<<<<<<<<<<<<<
 *         return self.__class__(self.base_composition, modification)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.modify", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_9modify(PyObject *__pyx_v_self, PyObject *__pyx_v_modification); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_8modify[] = "TerminalGroup.modify(self, ModificationBase modification) -> TerminalGroup";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_9modify(PyObject *__pyx_v_self, PyObject *__pyx_v_modification) {
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("modify (wrapper)", 0);
  if (unlikely(!__Pyx_ArgTypeTest(((PyObject *)__pyx_v_modification), __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase, 1, "modification", 0))) __PYX_ERR(0, 76, __pyx_L1_error)
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_8modify(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self), ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_modification));

  /* function exit code */
  goto __pyx_L0;
  __pyx_L1_error:;
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_8modify(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_modification) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("modify", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)__pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_modify(__pyx_v_self, __pyx_v_modification, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 76, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.modify", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":79
 *         return self.__class__(self.base_composition, modification)
 * 
 *     cdef CComposition get_composition(self):             # <<<<<<<<<<<<<<
 *         cdef ModificationBase modification = self.get_modification()
 * 
 */

static struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_get_composition(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_modification = 0;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_mod_comp = NULL;
  struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_composition", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":80
 * 
 *     cdef CComposition get_composition(self):
 *         cdef ModificationBase modification = self.get_modification()             # <<<<<<<<<<<<<<
 * 
 *         if modification is None:
 */
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self->__pyx_vtab)->get_modification(__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 80, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_v_modification = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/base.pyx":82
 *         cdef ModificationBase modification = self.get_modification()
 * 
 *         if modification is None:             # <<<<<<<<<<<<<<
 *             return self.base_composition
 *         mod_comp = modification.composition
 */
  __pyx_t_2 = (((PyObject *)__pyx_v_modification) == Py_None);
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (__pyx_t_3) {

    /* "glycopeptidepy/_c/structure/base.pyx":83
 * 
 *         if modification is None:
 *             return self.base_composition             # <<<<<<<<<<<<<<
 *         mod_comp = modification.composition
 *         return self.base_composition + mod_comp
 */
    __Pyx_XDECREF(((PyObject *)__pyx_r));
    __Pyx_INCREF(((PyObject *)__pyx_v_self->base_composition));
    __pyx_r = __pyx_v_self->base_composition;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/base.pyx":82
 *         cdef ModificationBase modification = self.get_modification()
 * 
 *         if modification is None:             # <<<<<<<<<<<<<<
 *             return self.base_composition
 *         mod_comp = modification.composition
 */
  }

  /* "glycopeptidepy/_c/structure/base.pyx":84
 *         if modification is None:
 *             return self.base_composition
 *         mod_comp = modification.composition             # <<<<<<<<<<<<<<
 *         return self.base_composition + mod_comp
 * 
 */
  __pyx_t_1 = ((PyObject *)__pyx_v_modification->composition);
  __Pyx_INCREF(__pyx_t_1);
  __pyx_v_mod_comp = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/base.pyx":85
 *             return self.base_composition
 *         mod_comp = modification.composition
 *         return self.base_composition + mod_comp             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __pyx_t_1 = PyNumber_Add(((PyObject *)__pyx_v_self->base_composition), ((PyObject *)__pyx_v_mod_comp)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 85, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(0, 85, __pyx_L1_error)
  __pyx_r = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":79
 *         return self.__class__(self.base_composition, modification)
 * 
 *     cdef CComposition get_composition(self):             # <<<<<<<<<<<<<<
 *         cdef ModificationBase modification = self.get_modification()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.get_composition", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_modification);
  __Pyx_XDECREF((PyObject *)__pyx_v_mod_comp);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":88
 * 
 *     @property
 *     def composition(self):             # <<<<<<<<<<<<<<
 *         return self.get_composition()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_11composition_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_11composition_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_11composition___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_11composition___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":89
 *     @property
 *     def composition(self):
 *         return self.get_composition()             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self->__pyx_vtab)->get_composition(__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 89, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":88
 * 
 *     @property
 *     def composition(self):             # <<<<<<<<<<<<<<
 *         return self.get_composition()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.composition.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":91
 *         return self.get_composition()
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         template = "{self.__class__.__name__}({self.base_composition}, {self.modification})"
 *         return template.format(self=self)
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_11__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_11__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_10__repr__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_10__repr__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self) {
  PyObject *__pyx_v_template = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":92
 * 
 *     def __repr__(self):
 *         template = "{self.__class__.__name__}({self.base_composition}, {self.modification})"             # <<<<<<<<<<<<<<
 *         return template.format(self=self)
 * 
 */
  __Pyx_INCREF(__pyx_kp_s_self___class_____name___self_ba);
  __pyx_v_template = __pyx_kp_s_self___class_____name___self_ba;

  /* "glycopeptidepy/_c/structure/base.pyx":93
 *     def __repr__(self):
 *         template = "{self.__class__.__name__}({self.base_composition}, {self.modification})"
 *         return template.format(self=self)             # <<<<<<<<<<<<<<
 * 
 *     def __str__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(__pyx_v_template, __pyx_n_s_format); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyDict_NewPresized(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_t_2, __pyx_n_s_self, ((PyObject *)__pyx_v_self)) < 0) __PYX_ERR(0, 93, __pyx_L1_error)
  __pyx_t_3 = __Pyx_PyObject_Call(__pyx_t_1, __pyx_empty_tuple, __pyx_t_2); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 93, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_3;
  __pyx_t_3 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":91
 *         return self.get_composition()
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         template = "{self.__class__.__name__}({self.base_composition}, {self.modification})"
 *         return template.format(self=self)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_template);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":95
 *         return template.format(self=self)
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.serialize()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13__str__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13__str__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__str__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_12__str__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_12__str__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__str__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":96
 * 
 *     def __str__(self):
 *         return self.serialize()             # <<<<<<<<<<<<<<
 * 
 *     def __eq__(self, other):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self->__pyx_vtab)->serialize(__pyx_v_self, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 96, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":95
 *         return template.format(self=self)
 * 
 *     def __str__(self):             # <<<<<<<<<<<<<<
 *         return self.serialize()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.__str__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":98
 *         return self.serialize()
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         if other is None:
 *             return False
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_15__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_15__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__eq__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_14__eq__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_14__eq__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_v_Modification = NULL;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  PyObject *__pyx_t_11 = NULL;
  PyObject *__pyx_t_12 = NULL;
  PyObject *__pyx_t_13 = NULL;
  PyObject *__pyx_t_14 = NULL;
  PyObject *__pyx_t_15 = NULL;
  PyObject *__pyx_t_16 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__eq__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":99
 * 
 *     def __eq__(self, other):
 *         if other is None:             # <<<<<<<<<<<<<<
 *             return False
 *         try:
 */
  __pyx_t_1 = (__pyx_v_other == Py_None);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/base.pyx":100
 *     def __eq__(self, other):
 *         if other is None:
 *             return False             # <<<<<<<<<<<<<<
 *         try:
 *             return (self.base_composition == other.base_composition) and (self.modification == other.modification)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/base.pyx":99
 * 
 *     def __eq__(self, other):
 *         if other is None:             # <<<<<<<<<<<<<<
 *             return False
 *         try:
 */
  }

  /* "glycopeptidepy/_c/structure/base.pyx":101
 *         if other is None:
 *             return False
 *         try:             # <<<<<<<<<<<<<<
 *             return (self.base_composition == other.base_composition) and (self.modification == other.modification)
 *         except AttributeError:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    /*try:*/ {

      /* "glycopeptidepy/_c/structure/base.pyx":102
 *             return False
 *         try:
 *             return (self.base_composition == other.base_composition) and (self.modification == other.modification)             # <<<<<<<<<<<<<<
 *         except AttributeError:
 *             if isinstance(other, basestring):
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_n_s_base_composition); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 102, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = PyObject_RichCompare(((PyObject *)__pyx_v_self->base_composition), __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 102, __pyx_L4_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 102, __pyx_L4_error)
      if (__pyx_t_2) {
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else {
        __Pyx_INCREF(__pyx_t_8);
        __pyx_t_6 = __pyx_t_8;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L10_bool_binop_done;
      }
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_modification); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 102, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_n_s_modification); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 102, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_9 = PyObject_RichCompare(__pyx_t_8, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_9); if (unlikely(!__pyx_t_9)) __PYX_ERR(0, 102, __pyx_L4_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_INCREF(__pyx_t_9);
      __pyx_t_6 = __pyx_t_9;
      __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
      __pyx_L10_bool_binop_done:;
      __pyx_r = __pyx_t_6;
      __pyx_t_6 = 0;
      goto __pyx_L8_try_return;

      /* "glycopeptidepy/_c/structure/base.pyx":101
 *         if other is None:
 *             return False
 *         try:             # <<<<<<<<<<<<<<
 *             return (self.base_composition == other.base_composition) and (self.modification == other.modification)
 *         except AttributeError:
 */
    }
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;

    /* "glycopeptidepy/_c/structure/base.pyx":103
 *         try:
 *             return (self.base_composition == other.base_composition) and (self.modification == other.modification)
 *         except AttributeError:             # <<<<<<<<<<<<<<
 *             if isinstance(other, basestring):
 *                 from glycopeptidepy.structure.modification import Modification
 */
    __pyx_t_10 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_AttributeError);
    if (__pyx_t_10) {
      __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_9, &__pyx_t_7) < 0) __PYX_ERR(0, 103, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_9);
      __Pyx_GOTREF(__pyx_t_7);

      /* "glycopeptidepy/_c/structure/base.pyx":104
 *             return (self.base_composition == other.base_composition) and (self.modification == other.modification)
 *         except AttributeError:
 *             if isinstance(other, basestring):             # <<<<<<<<<<<<<<
 *                 from glycopeptidepy.structure.modification import Modification
 *                 return self.composition == Modification(other).composition
 */
      __pyx_t_2 = __Pyx_PyBaseString_Check(__pyx_v_other); 
      __pyx_t_1 = (__pyx_t_2 != 0);
      if (__pyx_t_1) {

        /* "glycopeptidepy/_c/structure/base.pyx":105
 *         except AttributeError:
 *             if isinstance(other, basestring):
 *                 from glycopeptidepy.structure.modification import Modification             # <<<<<<<<<<<<<<
 *                 return self.composition == Modification(other).composition
 *             else:
 */
        __pyx_t_8 = PyList_New(1); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 105, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_INCREF(__pyx_n_s_Modification);
        __Pyx_GIVEREF(__pyx_n_s_Modification);
        PyList_SET_ITEM(__pyx_t_8, 0, __pyx_n_s_Modification);
        __pyx_t_11 = __Pyx_Import(__pyx_n_s_glycopeptidepy_structure_modific, __pyx_t_8, -1); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 105, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_8 = __Pyx_ImportFrom(__pyx_t_11, __pyx_n_s_Modification); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 105, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_8);
        __pyx_v_Modification = __pyx_t_8;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;

        /* "glycopeptidepy/_c/structure/base.pyx":106
 *             if isinstance(other, basestring):
 *                 from glycopeptidepy.structure.modification import Modification
 *                 return self.composition == Modification(other).composition             # <<<<<<<<<<<<<<
 *             else:
 *                 try:
 */
        __Pyx_XDECREF(__pyx_r);
        __pyx_t_11 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_composition); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 106, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_11);
        __Pyx_INCREF(__pyx_v_Modification);
        __pyx_t_12 = __pyx_v_Modification; __pyx_t_13 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_12))) {
          __pyx_t_13 = PyMethod_GET_SELF(__pyx_t_12);
          if (likely(__pyx_t_13)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_12);
            __Pyx_INCREF(__pyx_t_13);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_12, function);
          }
        }
        __pyx_t_8 = (__pyx_t_13) ? __Pyx_PyObject_Call2Args(__pyx_t_12, __pyx_t_13, __pyx_v_other) : __Pyx_PyObject_CallOneArg(__pyx_t_12, __pyx_v_other);
        __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
        if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 106, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_8);
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_composition); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 106, __pyx_L6_except_error)
        __Pyx_GOTREF(__pyx_t_12);
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        __pyx_t_8 = PyObject_RichCompare(__pyx_t_11, __pyx_t_12, Py_EQ); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 106, __pyx_L6_except_error)
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
        __pyx_r = __pyx_t_8;
        __pyx_t_8 = 0;
        __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
        __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
        __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
        goto __pyx_L7_except_return;

        /* "glycopeptidepy/_c/structure/base.pyx":104
 *             return (self.base_composition == other.base_composition) and (self.modification == other.modification)
 *         except AttributeError:
 *             if isinstance(other, basestring):             # <<<<<<<<<<<<<<
 *                 from glycopeptidepy.structure.modification import Modification
 *                 return self.composition == Modification(other).composition
 */
      }

      /* "glycopeptidepy/_c/structure/base.pyx":108
 *                 return self.composition == Modification(other).composition
 *             else:
 *                 try:             # <<<<<<<<<<<<<<
 *                     return self.composition == other.composition
 *                 except AttributeError:
 */
      /*else*/ {
        {
          __Pyx_PyThreadState_declare
          __Pyx_PyThreadState_assign
          __Pyx_ExceptionSave(&__pyx_t_14, &__pyx_t_15, &__pyx_t_16);
          __Pyx_XGOTREF(__pyx_t_14);
          __Pyx_XGOTREF(__pyx_t_15);
          __Pyx_XGOTREF(__pyx_t_16);
          /*try:*/ {

            /* "glycopeptidepy/_c/structure/base.pyx":109
 *             else:
 *                 try:
 *                     return self.composition == other.composition             # <<<<<<<<<<<<<<
 *                 except AttributeError:
 *                     return NotImplemented
 */
            __Pyx_XDECREF(__pyx_r);
            __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_composition); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 109, __pyx_L15_error)
            __Pyx_GOTREF(__pyx_t_8);
            __pyx_t_12 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_n_s_composition); if (unlikely(!__pyx_t_12)) __PYX_ERR(0, 109, __pyx_L15_error)
            __Pyx_GOTREF(__pyx_t_12);
            __pyx_t_11 = PyObject_RichCompare(__pyx_t_8, __pyx_t_12, Py_EQ); __Pyx_XGOTREF(__pyx_t_11); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 109, __pyx_L15_error)
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            __pyx_r = __pyx_t_11;
            __pyx_t_11 = 0;
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
            goto __pyx_L19_try_return;

            /* "glycopeptidepy/_c/structure/base.pyx":108
 *                 return self.composition == Modification(other).composition
 *             else:
 *                 try:             # <<<<<<<<<<<<<<
 *                     return self.composition == other.composition
 *                 except AttributeError:
 */
          }
          __pyx_L15_error:;
          __Pyx_XDECREF(__pyx_t_11); __pyx_t_11 = 0;
          __Pyx_XDECREF(__pyx_t_12); __pyx_t_12 = 0;
          __Pyx_XDECREF(__pyx_t_13); __pyx_t_13 = 0;
          __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

          /* "glycopeptidepy/_c/structure/base.pyx":110
 *                 try:
 *                     return self.composition == other.composition
 *                 except AttributeError:             # <<<<<<<<<<<<<<
 *                     return NotImplemented
 * 
 */
          __pyx_t_10 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_AttributeError);
          if (__pyx_t_10) {
            __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
            if (__Pyx_GetException(&__pyx_t_11, &__pyx_t_12, &__pyx_t_8) < 0) __PYX_ERR(0, 110, __pyx_L17_except_error)
            __Pyx_GOTREF(__pyx_t_11);
            __Pyx_GOTREF(__pyx_t_12);
            __Pyx_GOTREF(__pyx_t_8);

            /* "glycopeptidepy/_c/structure/base.pyx":111
 *                     return self.composition == other.composition
 *                 except AttributeError:
 *                     return NotImplemented             # <<<<<<<<<<<<<<
 * 
 *     cdef bint equal_to(self, object other):
 */
            __Pyx_XDECREF(__pyx_r);
            __Pyx_INCREF(__pyx_builtin_NotImplemented);
            __pyx_r = __pyx_builtin_NotImplemented;
            __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
            __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
            __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
            __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
            __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
            __Pyx_DECREF(__pyx_t_12); __pyx_t_12 = 0;
            goto __pyx_L18_except_return;
          }
          goto __pyx_L17_except_error;
          __pyx_L17_except_error:;

          /* "glycopeptidepy/_c/structure/base.pyx":108
 *                 return self.composition == Modification(other).composition
 *             else:
 *                 try:             # <<<<<<<<<<<<<<
 *                     return self.composition == other.composition
 *                 except AttributeError:
 */
          __Pyx_XGIVEREF(__pyx_t_14);
          __Pyx_XGIVEREF(__pyx_t_15);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
          goto __pyx_L6_except_error;
          __pyx_L19_try_return:;
          __Pyx_XGIVEREF(__pyx_t_14);
          __Pyx_XGIVEREF(__pyx_t_15);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
          goto __pyx_L7_except_return;
          __pyx_L18_except_return:;
          __Pyx_XGIVEREF(__pyx_t_14);
          __Pyx_XGIVEREF(__pyx_t_15);
          __Pyx_XGIVEREF(__pyx_t_16);
          __Pyx_ExceptionReset(__pyx_t_14, __pyx_t_15, __pyx_t_16);
          goto __pyx_L7_except_return;
        }
      }
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;

    /* "glycopeptidepy/_c/structure/base.pyx":101
 *         if other is None:
 *             return False
 *         try:             # <<<<<<<<<<<<<<
 *             return (self.base_composition == other.base_composition) and (self.modification == other.modification)
 *         except AttributeError:
 */
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L1_error;
    __pyx_L8_try_return:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L0;
    __pyx_L7_except_return:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L0;
  }

  /* "glycopeptidepy/_c/structure/base.pyx":98
 *         return self.serialize()
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         if other is None:
 *             return False
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_XDECREF(__pyx_t_12);
  __Pyx_XDECREF(__pyx_t_13);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_Modification);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":113
 *                     return NotImplemented
 * 
 *     cdef bint equal_to(self, object other):             # <<<<<<<<<<<<<<
 *         cdef:
 *             TerminalGroup other_terminal
 */

static int __pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_equal_to(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, PyObject *__pyx_v_other) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_other_terminal = 0;
  PyObject *__pyx_v_Modification = NULL;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("equal_to", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":116
 *         cdef:
 *             TerminalGroup other_terminal
 *         if isinstance(other, TerminalGroup):             # <<<<<<<<<<<<<<
 *             other_terminal = <TerminalGroup>other
 *             return (self.base_composition == other_terminal.base_composition) and (
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_other, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/base.pyx":117
 *             TerminalGroup other_terminal
 *         if isinstance(other, TerminalGroup):
 *             other_terminal = <TerminalGroup>other             # <<<<<<<<<<<<<<
 *             return (self.base_composition == other_terminal.base_composition) and (
 *                 self.modification == other_terminal.modification)
 */
    __pyx_t_3 = __pyx_v_other;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_other_terminal = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "glycopeptidepy/_c/structure/base.pyx":118
 *         if isinstance(other, TerminalGroup):
 *             other_terminal = <TerminalGroup>other
 *             return (self.base_composition == other_terminal.base_composition) and (             # <<<<<<<<<<<<<<
 *                 self.modification == other_terminal.modification)
 *         else:
 */
    __pyx_t_3 = PyObject_RichCompare(((PyObject *)__pyx_v_self->base_composition), ((PyObject *)__pyx_v_other_terminal->base_composition), Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 118, __pyx_L1_error)
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 118, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    if (__pyx_t_1) {
    } else {
      __pyx_t_2 = __pyx_t_1;
      goto __pyx_L4_bool_binop_done;
    }

    /* "glycopeptidepy/_c/structure/base.pyx":119
 *             other_terminal = <TerminalGroup>other
 *             return (self.base_composition == other_terminal.base_composition) and (
 *                 self.modification == other_terminal.modification)             # <<<<<<<<<<<<<<
 *         else:
 *             if isinstance(other, basestring):
 */
    __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_modification); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 119, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_other_terminal), __pyx_n_s_modification); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 119, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyObject_RichCompare(__pyx_t_3, __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 119, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_1 < 0)) __PYX_ERR(0, 119, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __pyx_t_2 = __pyx_t_1;
    __pyx_L4_bool_binop_done:;
    __pyx_r = __pyx_t_2;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/base.pyx":116
 *         cdef:
 *             TerminalGroup other_terminal
 *         if isinstance(other, TerminalGroup):             # <<<<<<<<<<<<<<
 *             other_terminal = <TerminalGroup>other
 *             return (self.base_composition == other_terminal.base_composition) and (
 */
  }

  /* "glycopeptidepy/_c/structure/base.pyx":121
 *                 self.modification == other_terminal.modification)
 *         else:
 *             if isinstance(other, basestring):             # <<<<<<<<<<<<<<
 *                 from glycopeptidepy.structure.modification import Modification
 *                 return self.composition == Modification(other).composition
 */
  /*else*/ {
    __pyx_t_2 = __Pyx_PyBaseString_Check(__pyx_v_other); 
    __pyx_t_1 = (__pyx_t_2 != 0);
    if (__pyx_t_1) {

      /* "glycopeptidepy/_c/structure/base.pyx":122
 *         else:
 *             if isinstance(other, basestring):
 *                 from glycopeptidepy.structure.modification import Modification             # <<<<<<<<<<<<<<
 *                 return self.composition == Modification(other).composition
 *             elif isinstance(other, ModificationBase):
 */
      __pyx_t_5 = PyList_New(1); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 122, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_n_s_Modification);
      __Pyx_GIVEREF(__pyx_n_s_Modification);
      PyList_SET_ITEM(__pyx_t_5, 0, __pyx_n_s_Modification);
      __pyx_t_4 = __Pyx_Import(__pyx_n_s_glycopeptidepy_structure_modific, __pyx_t_5, -1); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 122, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = __Pyx_ImportFrom(__pyx_t_4, __pyx_n_s_Modification); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 122, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_INCREF(__pyx_t_5);
      __pyx_v_Modification = __pyx_t_5;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;

      /* "glycopeptidepy/_c/structure/base.pyx":123
 *             if isinstance(other, basestring):
 *                 from glycopeptidepy.structure.modification import Modification
 *                 return self.composition == Modification(other).composition             # <<<<<<<<<<<<<<
 *             elif isinstance(other, ModificationBase):
 *                 return self.composition == (<ModificationBase>other).composition
 */
      __pyx_t_4 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_composition); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __Pyx_INCREF(__pyx_v_Modification);
      __pyx_t_3 = __pyx_v_Modification; __pyx_t_6 = NULL;
      if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
        __pyx_t_6 = PyMethod_GET_SELF(__pyx_t_3);
        if (likely(__pyx_t_6)) {
          PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
          __Pyx_INCREF(__pyx_t_6);
          __Pyx_INCREF(function);
          __Pyx_DECREF_SET(__pyx_t_3, function);
        }
      }
      __pyx_t_5 = (__pyx_t_6) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_6, __pyx_v_other) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_other);
      __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
      if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_3 = __Pyx_PyObject_GetAttrStr(__pyx_t_5, __pyx_n_s_composition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 123, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_3);
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_5 = PyObject_RichCompare(__pyx_t_4, __pyx_t_3, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 123, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_t_1 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely((__pyx_t_1 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 123, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_r = __pyx_t_1;
      goto __pyx_L0;

      /* "glycopeptidepy/_c/structure/base.pyx":121
 *                 self.modification == other_terminal.modification)
 *         else:
 *             if isinstance(other, basestring):             # <<<<<<<<<<<<<<
 *                 from glycopeptidepy.structure.modification import Modification
 *                 return self.composition == Modification(other).composition
 */
    }

    /* "glycopeptidepy/_c/structure/base.pyx":124
 *                 from glycopeptidepy.structure.modification import Modification
 *                 return self.composition == Modification(other).composition
 *             elif isinstance(other, ModificationBase):             # <<<<<<<<<<<<<<
 *                 return self.composition == (<ModificationBase>other).composition
 *             else:
 */
    __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_other, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase); 
    __pyx_t_2 = (__pyx_t_1 != 0);
    if (__pyx_t_2) {

      /* "glycopeptidepy/_c/structure/base.pyx":125
 *                 return self.composition == Modification(other).composition
 *             elif isinstance(other, ModificationBase):
 *                 return self.composition == (<ModificationBase>other).composition             # <<<<<<<<<<<<<<
 *             else:
 *                 try:
 */
      __pyx_t_5 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_composition); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 125, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_5);
      __pyx_t_3 = PyObject_RichCompare(__pyx_t_5, ((PyObject *)((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_other)->composition), Py_EQ); __Pyx_XGOTREF(__pyx_t_3); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 125, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_3); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 125, __pyx_L1_error)
      __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
      __pyx_r = __pyx_t_2;
      goto __pyx_L0;

      /* "glycopeptidepy/_c/structure/base.pyx":124
 *                 from glycopeptidepy.structure.modification import Modification
 *                 return self.composition == Modification(other).composition
 *             elif isinstance(other, ModificationBase):             # <<<<<<<<<<<<<<
 *                 return self.composition == (<ModificationBase>other).composition
 *             else:
 */
    }

    /* "glycopeptidepy/_c/structure/base.pyx":127
 *                 return self.composition == (<ModificationBase>other).composition
 *             else:
 *                 try:             # <<<<<<<<<<<<<<
 *                     return self.composition == other.composition
 *                 except AttributeError:
 */
    /*else*/ {
      {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        __Pyx_ExceptionSave(&__pyx_t_7, &__pyx_t_8, &__pyx_t_9);
        __Pyx_XGOTREF(__pyx_t_7);
        __Pyx_XGOTREF(__pyx_t_8);
        __Pyx_XGOTREF(__pyx_t_9);
        /*try:*/ {

          /* "glycopeptidepy/_c/structure/base.pyx":128
 *             else:
 *                 try:
 *                     return self.composition == other.composition             # <<<<<<<<<<<<<<
 *                 except AttributeError:
 *                     return False
 */
          __pyx_t_3 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_composition); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 128, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_3);
          __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_n_s_composition); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 128, __pyx_L7_error)
          __Pyx_GOTREF(__pyx_t_5);
          __pyx_t_4 = PyObject_RichCompare(__pyx_t_3, __pyx_t_5, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 128, __pyx_L7_error)
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_4); if (unlikely((__pyx_t_2 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 128, __pyx_L7_error)
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __pyx_r = __pyx_t_2;
          goto __pyx_L11_try_return;

          /* "glycopeptidepy/_c/structure/base.pyx":127
 *                 return self.composition == (<ModificationBase>other).composition
 *             else:
 *                 try:             # <<<<<<<<<<<<<<
 *                     return self.composition == other.composition
 *                 except AttributeError:
 */
        }
        __pyx_L7_error:;
        __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
        __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;

        /* "glycopeptidepy/_c/structure/base.pyx":129
 *                 try:
 *                     return self.composition == other.composition
 *                 except AttributeError:             # <<<<<<<<<<<<<<
 *                     return False
 * 
 */
        __pyx_t_10 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_AttributeError);
        if (__pyx_t_10) {
          __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.equal_to", __pyx_clineno, __pyx_lineno, __pyx_filename);
          if (__Pyx_GetException(&__pyx_t_4, &__pyx_t_5, &__pyx_t_3) < 0) __PYX_ERR(0, 129, __pyx_L9_except_error)
          __Pyx_GOTREF(__pyx_t_4);
          __Pyx_GOTREF(__pyx_t_5);
          __Pyx_GOTREF(__pyx_t_3);

          /* "glycopeptidepy/_c/structure/base.pyx":130
 *                     return self.composition == other.composition
 *                 except AttributeError:
 *                     return False             # <<<<<<<<<<<<<<
 * 
 *     def __ne__(self, other):
 */
          __pyx_r = 0;
          __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
          __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
          __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
          goto __pyx_L10_except_return;
        }
        goto __pyx_L9_except_error;
        __pyx_L9_except_error:;

        /* "glycopeptidepy/_c/structure/base.pyx":127
 *                 return self.composition == (<ModificationBase>other).composition
 *             else:
 *                 try:             # <<<<<<<<<<<<<<
 *                     return self.composition == other.composition
 *                 except AttributeError:
 */
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
        goto __pyx_L1_error;
        __pyx_L11_try_return:;
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
        goto __pyx_L0;
        __pyx_L10_except_return:;
        __Pyx_XGIVEREF(__pyx_t_7);
        __Pyx_XGIVEREF(__pyx_t_8);
        __Pyx_XGIVEREF(__pyx_t_9);
        __Pyx_ExceptionReset(__pyx_t_7, __pyx_t_8, __pyx_t_9);
        goto __pyx_L0;
      }
    }
  }

  /* "glycopeptidepy/_c/structure/base.pyx":113
 *                     return NotImplemented
 * 
 *     cdef bint equal_to(self, object other):             # <<<<<<<<<<<<<<
 *         cdef:
 *             TerminalGroup other_terminal
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_WriteUnraisable("glycopeptidepy._c.structure.base.TerminalGroup.equal_to", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_other_terminal);
  __Pyx_XDECREF(__pyx_v_Modification);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":132
 *                     return False
 * 
 *     def __ne__(self, other):             # <<<<<<<<<<<<<<
 *         return not (self == other)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_17__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_17__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__ne__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16__ne__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16__ne__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__ne__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":133
 * 
 *     def __ne__(self, other):
 *         return not (self == other)             # <<<<<<<<<<<<<<
 * 
 *     def __hash__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyObject_RichCompare(((PyObject *)__pyx_v_self), __pyx_v_other, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 133, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 133, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyBool_FromLong((!__pyx_t_2)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 133, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":132
 *                     return False
 * 
 *     def __ne__(self, other):             # <<<<<<<<<<<<<<
 *         return not (self == other)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.__ne__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":135
 *         return not (self == other)
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         return hash(formula(self.get_composition()))
 * 
 */

/* Python wrapper */
static Py_hash_t __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_19__hash__(PyObject *__pyx_v_self); /*proto*/
static Py_hash_t __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_19__hash__(PyObject *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__hash__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_18__hash__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_hash_t __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_18__hash__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_hash_t __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__hash__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":136
 * 
 *     def __hash__(self):
 *         return hash(formula(self.get_composition()))             # <<<<<<<<<<<<<<
 * 
 *     cpdef str serialize(self):
 */
  __pyx_t_2 = ((PyObject *)((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self->__pyx_vtab)->get_composition(__pyx_v_self)); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(__pyx_v_14glycopeptidepy_2_c_9structure_4base_formula);
  __pyx_t_3 = __pyx_v_14glycopeptidepy_2_c_9structure_4base_formula; __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_3, function);
    }
  }
  __pyx_t_1 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_t_2) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_2);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
  __pyx_t_5 = PyObject_Hash(__pyx_t_1); if (unlikely(__pyx_t_5 == ((Py_hash_t)-1))) __PYX_ERR(0, 136, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_5;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":135
 *         return not (self == other)
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         return hash(formula(self.get_composition()))
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.__hash__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  if (unlikely(__pyx_r == -1) && !PyErr_Occurred()) __pyx_r = -2;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":138
 *         return hash(formula(self.get_composition()))
 * 
 *     cpdef str serialize(self):             # <<<<<<<<<<<<<<
 *         if self.modification is not None:
 *             return str(self.modification)
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_21serialize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_serialize(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_serialize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 138, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_21serialize)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 138, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(PyString_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 138, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/base.pyx":139
 * 
 *     cpdef str serialize(self):
 *         if self.modification is not None:             # <<<<<<<<<<<<<<
 *             return str(self.modification)
 *         return formula(self.base_composition)
 */
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_modification); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 139, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_5 = (__pyx_t_1 != Py_None);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_6 = (__pyx_t_5 != 0);
  if (__pyx_t_6) {

    /* "glycopeptidepy/_c/structure/base.pyx":140
 *     cpdef str serialize(self):
 *         if self.modification is not None:
 *             return str(self.modification)             # <<<<<<<<<<<<<<
 *         return formula(self.base_composition)
 * 
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_modification); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(((PyObject *)(&PyString_Type)), __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 140, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (!(likely(PyString_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 140, __pyx_L1_error)
    __pyx_r = ((PyObject*)__pyx_t_2);
    __pyx_t_2 = 0;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/base.pyx":139
 * 
 *     cpdef str serialize(self):
 *         if self.modification is not None:             # <<<<<<<<<<<<<<
 *             return str(self.modification)
 *         return formula(self.base_composition)
 */
  }

  /* "glycopeptidepy/_c/structure/base.pyx":141
 *         if self.modification is not None:
 *             return str(self.modification)
 *         return formula(self.base_composition)             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_14glycopeptidepy_2_c_9structure_4base_formula);
  __pyx_t_1 = __pyx_v_14glycopeptidepy_2_c_9structure_4base_formula; __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_1))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_1);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_1);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_1, function);
    }
  }
  __pyx_t_2 = (__pyx_t_3) ? __Pyx_PyObject_Call2Args(__pyx_t_1, __pyx_t_3, ((PyObject *)__pyx_v_self->base_composition)) : __Pyx_PyObject_CallOneArg(__pyx_t_1, ((PyObject *)__pyx_v_self->base_composition));
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 141, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (!(likely(PyString_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "str", Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 141, __pyx_L1_error)
  __pyx_r = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":138
 *         return hash(formula(self.get_composition()))
 * 
 *     cpdef str serialize(self):             # <<<<<<<<<<<<<<
 *         if self.modification is not None:
 *             return str(self.modification)
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_21serialize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_20serialize[] = "TerminalGroup.serialize(self) -> str";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_21serialize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_20serialize(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_20serialize(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_serialize(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 138, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":23
 * cdef class TerminalGroup(object):
 *     cdef:
 *         public CComposition base_composition             # <<<<<<<<<<<<<<
 *         public double mass
 *         public ModificationBase _modification
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->base_composition));
  __pyx_r = ((PyObject *)__pyx_v_self->base_composition);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(1, 23, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->base_composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->base_composition));
  __pyx_v_self->base_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.base_composition.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->base_composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->base_composition));
  __pyx_v_self->base_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":24
 *     cdef:
 *         public CComposition base_composition
 *         public double mass             # <<<<<<<<<<<<<<
 *         public ModificationBase _modification
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_4mass_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_4mass_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_4mass___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_4mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 24, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.mass.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_4mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_4mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_4mass_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_4mass_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 24, __pyx_L1_error)
  __pyx_v_self->mass = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup.mass.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":25
 *         public CComposition base_composition
 *         public double mass
 *         public ModificationBase _modification             # <<<<<<<<<<<<<<
 * 
 *     cdef ModificationBase get_modification(self)
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->_modification));
  __pyx_r = ((PyObject *)__pyx_v_self->_modification);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase))))) __PYX_ERR(1, 25, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->_modification);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_modification));
  __pyx_v_self->_modification = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.TerminalGroup._modification.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->_modification);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->_modification));
  __pyx_v_self->_modification = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":150
 *     '''
 * 
 *     def __copy__(self):             # <<<<<<<<<<<<<<
 *         return self.clone()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_1__copy__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase___copy__[] = "AminoAcidResidueBase.__copy__(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_1__copy__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__copy__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase___copy__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase___copy__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__copy__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":151
 * 
 *     def __copy__(self):
 *         return self.clone()             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_clone); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 151, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":150
 *     '''
 * 
 *     def __copy__(self):             # <<<<<<<<<<<<<<
 *         return self.clone()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.AminoAcidResidueBase.__copy__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":154
 * 
 *     @staticmethod
 *     cdef AminoAcidResidueBase _create(str name, str symbol, double mass, CComposition composition):             # <<<<<<<<<<<<<<
 *         cdef AminoAcidResidueBase inst = AminoAcidResidueBase.__new__(AminoAcidResidueBase)
 *         inst.name = name
 */

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_f_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase__create(PyObject *__pyx_v_name, PyObject *__pyx_v_symbol, double __pyx_v_mass, struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *__pyx_v_composition) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_inst = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_create", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":155
 *     @staticmethod
 *     cdef AminoAcidResidueBase _create(str name, str symbol, double mass, CComposition composition):
 *         cdef AminoAcidResidueBase inst = AminoAcidResidueBase.__new__(AminoAcidResidueBase)             # <<<<<<<<<<<<<<
 *         inst.name = name
 *         inst.symbol = symbol
 */
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase(((PyTypeObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase), __pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 155, __pyx_L1_error)
  __Pyx_GOTREF(((PyObject *)__pyx_t_1));
  __pyx_v_inst = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/base.pyx":156
 *     cdef AminoAcidResidueBase _create(str name, str symbol, double mass, CComposition composition):
 *         cdef AminoAcidResidueBase inst = AminoAcidResidueBase.__new__(AminoAcidResidueBase)
 *         inst.name = name             # <<<<<<<<<<<<<<
 *         inst.symbol = symbol
 *         inst.mass = mass
 */
  __Pyx_INCREF(__pyx_v_name);
  __Pyx_GIVEREF(__pyx_v_name);
  __Pyx_GOTREF(__pyx_v_inst->name);
  __Pyx_DECREF(__pyx_v_inst->name);
  __pyx_v_inst->name = ((PyObject*)__pyx_v_name);

  /* "glycopeptidepy/_c/structure/base.pyx":157
 *         cdef AminoAcidResidueBase inst = AminoAcidResidueBase.__new__(AminoAcidResidueBase)
 *         inst.name = name
 *         inst.symbol = symbol             # <<<<<<<<<<<<<<
 *         inst.mass = mass
 *         inst.composition = composition
 */
  __Pyx_INCREF(__pyx_v_symbol);
  __Pyx_GIVEREF(__pyx_v_symbol);
  __Pyx_GOTREF(__pyx_v_inst->symbol);
  __Pyx_DECREF(__pyx_v_inst->symbol);
  __pyx_v_inst->symbol = ((PyObject*)__pyx_v_symbol);

  /* "glycopeptidepy/_c/structure/base.pyx":158
 *         inst.name = name
 *         inst.symbol = symbol
 *         inst.mass = mass             # <<<<<<<<<<<<<<
 *         inst.composition = composition
 *         return inst
 */
  __pyx_v_inst->mass = __pyx_v_mass;

  /* "glycopeptidepy/_c/structure/base.pyx":159
 *         inst.symbol = symbol
 *         inst.mass = mass
 *         inst.composition = composition             # <<<<<<<<<<<<<<
 *         return inst
 * 
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_composition));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_composition));
  __Pyx_GOTREF(__pyx_v_inst->composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_inst->composition));
  __pyx_v_inst->composition = __pyx_v_composition;

  /* "glycopeptidepy/_c/structure/base.pyx":160
 *         inst.mass = mass
 *         inst.composition = composition
 *         return inst             # <<<<<<<<<<<<<<
 * 
 *     def __hash__(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_inst));
  __pyx_r = __pyx_v_inst;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":154
 * 
 *     @staticmethod
 *     cdef AminoAcidResidueBase _create(str name, str symbol, double mass, CComposition composition):             # <<<<<<<<<<<<<<
 *         cdef AminoAcidResidueBase inst = AminoAcidResidueBase.__new__(AminoAcidResidueBase)
 *         inst.name = name
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.AminoAcidResidueBase._create", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":162
 *         return inst
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         return self._hash
 * 
 */

/* Python wrapper */
static Py_hash_t __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_3__hash__(PyObject *__pyx_v_self); /*proto*/
static Py_hash_t __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_3__hash__(PyObject *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__hash__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_2__hash__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_hash_t __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_2__hash__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self) {
  Py_hash_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__hash__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":163
 * 
 *     def __hash__(self):
 *         return self._hash             # <<<<<<<<<<<<<<
 * 
 *     def __eq__(self, other):
 */
  __pyx_r = __pyx_v_self->_hash;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":162
 *         return inst
 * 
 *     def __hash__(self):             # <<<<<<<<<<<<<<
 *         return self._hash
 * 
 */

  /* function exit code */
  __pyx_L0:;
  if (unlikely(__pyx_r == -1) && !PyErr_Occurred()) __pyx_r = -2;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":165
 *         return self._hash
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         if self is other:
 *             return True
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_5__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_5__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__eq__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4__eq__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4__eq__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__eq__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":166
 * 
 *     def __eq__(self, other):
 *         if self is other:             # <<<<<<<<<<<<<<
 *             return True
 *         try:
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_self) == __pyx_v_other);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/base.pyx":167
 *     def __eq__(self, other):
 *         if self is other:
 *             return True             # <<<<<<<<<<<<<<
 *         try:
 *             return self.name == other.name or self.symbol == other.symbol
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_True);
    __pyx_r = Py_True;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/base.pyx":166
 * 
 *     def __eq__(self, other):
 *         if self is other:             # <<<<<<<<<<<<<<
 *             return True
 *         try:
 */
  }

  /* "glycopeptidepy/_c/structure/base.pyx":168
 *         if self is other:
 *             return True
 *         try:             # <<<<<<<<<<<<<<
 *             return self.name == other.name or self.symbol == other.symbol
 *         except AttributeError:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    /*try:*/ {

      /* "glycopeptidepy/_c/structure/base.pyx":169
 *             return True
 *         try:
 *             return self.name == other.name or self.symbol == other.symbol             # <<<<<<<<<<<<<<
 *         except AttributeError:
 *             return self.name == other or self.symbol == other
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_n_s_name); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 169, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = PyObject_RichCompare(__pyx_v_self->name, __pyx_t_7, Py_EQ); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 169, __pyx_L4_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 169, __pyx_L4_error)
      if (!__pyx_t_2) {
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else {
        __Pyx_INCREF(__pyx_t_8);
        __pyx_t_6 = __pyx_t_8;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L10_bool_binop_done;
      }
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_n_s_symbol); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 169, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_7 = PyObject_RichCompare(__pyx_v_self->symbol, __pyx_t_8, Py_EQ); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 169, __pyx_L4_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_INCREF(__pyx_t_7);
      __pyx_t_6 = __pyx_t_7;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_L10_bool_binop_done:;
      __pyx_r = __pyx_t_6;
      __pyx_t_6 = 0;
      goto __pyx_L8_try_return;

      /* "glycopeptidepy/_c/structure/base.pyx":168
 *         if self is other:
 *             return True
 *         try:             # <<<<<<<<<<<<<<
 *             return self.name == other.name or self.symbol == other.symbol
 *         except AttributeError:
 */
    }
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "glycopeptidepy/_c/structure/base.pyx":170
 *         try:
 *             return self.name == other.name or self.symbol == other.symbol
 *         except AttributeError:             # <<<<<<<<<<<<<<
 *             return self.name == other or self.symbol == other
 * 
 */
    __pyx_t_9 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_AttributeError);
    if (__pyx_t_9) {
      __Pyx_AddTraceback("glycopeptidepy._c.structure.base.AminoAcidResidueBase.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8) < 0) __PYX_ERR(0, 170, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GOTREF(__pyx_t_8);

      /* "glycopeptidepy/_c/structure/base.pyx":171
 *             return self.name == other.name or self.symbol == other.symbol
 *         except AttributeError:
 *             return self.name == other or self.symbol == other             # <<<<<<<<<<<<<<
 * 
 *     cdef bint equal_to(self, AminoAcidResidueBase other):
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_11 = PyObject_RichCompare(__pyx_v_self->name, __pyx_v_other, Py_EQ); __Pyx_XGOTREF(__pyx_t_11); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 171, __pyx_L6_except_error)
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_11); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 171, __pyx_L6_except_error)
      if (!__pyx_t_2) {
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      } else {
        __Pyx_INCREF(__pyx_t_11);
        __pyx_t_10 = __pyx_t_11;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        goto __pyx_L14_bool_binop_done;
      }
      __pyx_t_11 = PyObject_RichCompare(__pyx_v_self->symbol, __pyx_v_other, Py_EQ); __Pyx_XGOTREF(__pyx_t_11); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 171, __pyx_L6_except_error)
      __Pyx_INCREF(__pyx_t_11);
      __pyx_t_10 = __pyx_t_11;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_L14_bool_binop_done:;
      __pyx_r = __pyx_t_10;
      __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      goto __pyx_L7_except_return;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;

    /* "glycopeptidepy/_c/structure/base.pyx":168
 *         if self is other:
 *             return True
 *         try:             # <<<<<<<<<<<<<<
 *             return self.name == other.name or self.symbol == other.symbol
 *         except AttributeError:
 */
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L1_error;
    __pyx_L8_try_return:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L0;
    __pyx_L7_except_return:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L0;
  }

  /* "glycopeptidepy/_c/structure/base.pyx":165
 *         return self._hash
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         if self is other:
 *             return True
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.AminoAcidResidueBase.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":173
 *             return self.name == other or self.symbol == other
 * 
 *     cdef bint equal_to(self, AminoAcidResidueBase other):             # <<<<<<<<<<<<<<
 *         return self.name == other.name or self.symbol == other.symbol
 * 
 */

static int __pyx_f_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_equal_to(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_other) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  int __pyx_t_3;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("equal_to", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":174
 * 
 *     cdef bint equal_to(self, AminoAcidResidueBase other):
 *         return self.name == other.name or self.symbol == other.symbol             # <<<<<<<<<<<<<<
 * 
 *     def __ne__(self, other):
 */
  __pyx_t_2 = (__Pyx_PyUnicode_Equals(__pyx_v_self->name, __pyx_v_other->name, Py_EQ)); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 174, __pyx_L1_error)
  __pyx_t_3 = (__pyx_t_2 != 0);
  if (!__pyx_t_3) {
  } else {
    __pyx_t_1 = __pyx_t_3;
    goto __pyx_L3_bool_binop_done;
  }
  __pyx_t_3 = (__Pyx_PyUnicode_Equals(__pyx_v_self->symbol, __pyx_v_other->symbol, Py_EQ)); if (unlikely(__pyx_t_3 < 0)) __PYX_ERR(0, 174, __pyx_L1_error)
  __pyx_t_2 = (__pyx_t_3 != 0);
  __pyx_t_1 = __pyx_t_2;
  __pyx_L3_bool_binop_done:;
  __pyx_r = __pyx_t_1;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":173
 *             return self.name == other or self.symbol == other
 * 
 *     cdef bint equal_to(self, AminoAcidResidueBase other):             # <<<<<<<<<<<<<<
 *         return self.name == other.name or self.symbol == other.symbol
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_WriteUnraisable("glycopeptidepy._c.structure.base.AminoAcidResidueBase.equal_to", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":176
 *         return self.name == other.name or self.symbol == other.symbol
 * 
 *     def __ne__(self, other):             # <<<<<<<<<<<<<<
 *         if self is other:
 *             return False
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_7__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_7__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__ne__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6__ne__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6__ne__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  PyObject *__pyx_t_6 = NULL;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  int __pyx_t_9;
  PyObject *__pyx_t_10 = NULL;
  PyObject *__pyx_t_11 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__ne__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":177
 * 
 *     def __ne__(self, other):
 *         if self is other:             # <<<<<<<<<<<<<<
 *             return False
 *         try:
 */
  __pyx_t_1 = (((PyObject *)__pyx_v_self) == __pyx_v_other);
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/base.pyx":178
 *     def __ne__(self, other):
 *         if self is other:
 *             return False             # <<<<<<<<<<<<<<
 *         try:
 *             return self.name != other.name and self.symbol != other.symbol
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(Py_False);
    __pyx_r = Py_False;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/base.pyx":177
 * 
 *     def __ne__(self, other):
 *         if self is other:             # <<<<<<<<<<<<<<
 *             return False
 *         try:
 */
  }

  /* "glycopeptidepy/_c/structure/base.pyx":179
 *         if self is other:
 *             return False
 *         try:             # <<<<<<<<<<<<<<
 *             return self.name != other.name and self.symbol != other.symbol
 *         except AttributeError:
 */
  {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ExceptionSave(&__pyx_t_3, &__pyx_t_4, &__pyx_t_5);
    __Pyx_XGOTREF(__pyx_t_3);
    __Pyx_XGOTREF(__pyx_t_4);
    __Pyx_XGOTREF(__pyx_t_5);
    /*try:*/ {

      /* "glycopeptidepy/_c/structure/base.pyx":180
 *             return False
 *         try:
 *             return self.name != other.name and self.symbol != other.symbol             # <<<<<<<<<<<<<<
 *         except AttributeError:
 *             return self.name != other and self.symbol != other
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_7 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_n_s_name); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 180, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_7);
      __pyx_t_8 = PyObject_RichCompare(__pyx_v_self->name, __pyx_t_7, Py_NE); __Pyx_XGOTREF(__pyx_t_8); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 180, __pyx_L4_error)
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_8); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 180, __pyx_L4_error)
      if (__pyx_t_2) {
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      } else {
        __Pyx_INCREF(__pyx_t_8);
        __pyx_t_6 = __pyx_t_8;
        __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
        goto __pyx_L10_bool_binop_done;
      }
      __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_n_s_symbol); if (unlikely(!__pyx_t_8)) __PYX_ERR(0, 180, __pyx_L4_error)
      __Pyx_GOTREF(__pyx_t_8);
      __pyx_t_7 = PyObject_RichCompare(__pyx_v_self->symbol, __pyx_t_8, Py_NE); __Pyx_XGOTREF(__pyx_t_7); if (unlikely(!__pyx_t_7)) __PYX_ERR(0, 180, __pyx_L4_error)
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      __Pyx_INCREF(__pyx_t_7);
      __pyx_t_6 = __pyx_t_7;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __pyx_L10_bool_binop_done:;
      __pyx_r = __pyx_t_6;
      __pyx_t_6 = 0;
      goto __pyx_L8_try_return;

      /* "glycopeptidepy/_c/structure/base.pyx":179
 *         if self is other:
 *             return False
 *         try:             # <<<<<<<<<<<<<<
 *             return self.name != other.name and self.symbol != other.symbol
 *         except AttributeError:
 */
    }
    __pyx_L4_error:;
    __Pyx_XDECREF(__pyx_t_6); __pyx_t_6 = 0;
    __Pyx_XDECREF(__pyx_t_7); __pyx_t_7 = 0;
    __Pyx_XDECREF(__pyx_t_8); __pyx_t_8 = 0;

    /* "glycopeptidepy/_c/structure/base.pyx":181
 *         try:
 *             return self.name != other.name and self.symbol != other.symbol
 *         except AttributeError:             # <<<<<<<<<<<<<<
 *             return self.name != other and self.symbol != other
 * 
 */
    __pyx_t_9 = __Pyx_PyErr_ExceptionMatches(__pyx_builtin_AttributeError);
    if (__pyx_t_9) {
      __Pyx_AddTraceback("glycopeptidepy._c.structure.base.AminoAcidResidueBase.__ne__", __pyx_clineno, __pyx_lineno, __pyx_filename);
      if (__Pyx_GetException(&__pyx_t_6, &__pyx_t_7, &__pyx_t_8) < 0) __PYX_ERR(0, 181, __pyx_L6_except_error)
      __Pyx_GOTREF(__pyx_t_6);
      __Pyx_GOTREF(__pyx_t_7);
      __Pyx_GOTREF(__pyx_t_8);

      /* "glycopeptidepy/_c/structure/base.pyx":182
 *             return self.name != other.name and self.symbol != other.symbol
 *         except AttributeError:
 *             return self.name != other and self.symbol != other             # <<<<<<<<<<<<<<
 * 
 * 
 */
      __Pyx_XDECREF(__pyx_r);
      __pyx_t_11 = PyObject_RichCompare(__pyx_v_self->name, __pyx_v_other, Py_NE); __Pyx_XGOTREF(__pyx_t_11); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 182, __pyx_L6_except_error)
      __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_11); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 182, __pyx_L6_except_error)
      if (__pyx_t_2) {
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      } else {
        __Pyx_INCREF(__pyx_t_11);
        __pyx_t_10 = __pyx_t_11;
        __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
        goto __pyx_L14_bool_binop_done;
      }
      __pyx_t_11 = PyObject_RichCompare(__pyx_v_self->symbol, __pyx_v_other, Py_NE); __Pyx_XGOTREF(__pyx_t_11); if (unlikely(!__pyx_t_11)) __PYX_ERR(0, 182, __pyx_L6_except_error)
      __Pyx_INCREF(__pyx_t_11);
      __pyx_t_10 = __pyx_t_11;
      __Pyx_DECREF(__pyx_t_11); __pyx_t_11 = 0;
      __pyx_L14_bool_binop_done:;
      __pyx_r = __pyx_t_10;
      __pyx_t_10 = 0;
      __Pyx_DECREF(__pyx_t_6); __pyx_t_6 = 0;
      __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
      __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
      goto __pyx_L7_except_return;
    }
    goto __pyx_L6_except_error;
    __pyx_L6_except_error:;

    /* "glycopeptidepy/_c/structure/base.pyx":179
 *         if self is other:
 *             return False
 *         try:             # <<<<<<<<<<<<<<
 *             return self.name != other.name and self.symbol != other.symbol
 *         except AttributeError:
 */
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L1_error;
    __pyx_L8_try_return:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L0;
    __pyx_L7_except_return:;
    __Pyx_XGIVEREF(__pyx_t_3);
    __Pyx_XGIVEREF(__pyx_t_4);
    __Pyx_XGIVEREF(__pyx_t_5);
    __Pyx_ExceptionReset(__pyx_t_3, __pyx_t_4, __pyx_t_5);
    goto __pyx_L0;
  }

  /* "glycopeptidepy/_c/structure/base.pyx":176
 *         return self.name == other.name or self.symbol == other.symbol
 * 
 *     def __ne__(self, other):             # <<<<<<<<<<<<<<
 *         if self is other:
 *             return False
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_XDECREF(__pyx_t_11);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.AminoAcidResidueBase.__ne__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":37
 * cdef class AminoAcidResidueBase(object):
 *     cdef:
 *         public basestring name             # <<<<<<<<<<<<<<
 *         public basestring symbol
 *         public double mass
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->name);
  __pyx_r = __pyx_v_self->name;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(__Pyx_PyBaseString_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", (PY_MAJOR_VERSION < 3 ? "basestring" : "str"), Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 37, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.AminoAcidResidueBase.name.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":38
 *     cdef:
 *         public basestring name
 *         public basestring symbol             # <<<<<<<<<<<<<<
 *         public double mass
 *         public CComposition composition
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->symbol);
  __pyx_r = __pyx_v_self->symbol;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(__Pyx_PyBaseString_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", (PY_MAJOR_VERSION < 3 ? "basestring" : "str"), Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 38, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->symbol);
  __Pyx_DECREF(__pyx_v_self->symbol);
  __pyx_v_self->symbol = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.AminoAcidResidueBase.symbol.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->symbol);
  __Pyx_DECREF(__pyx_v_self->symbol);
  __pyx_v_self->symbol = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":39
 *         public basestring name
 *         public basestring symbol
 *         public double mass             # <<<<<<<<<<<<<<
 *         public CComposition composition
 *         public object neutral_loss
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4mass_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4mass_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4mass___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 39, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.AminoAcidResidueBase.mass.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4mass_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4mass_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 39, __pyx_L1_error)
  __pyx_v_self->mass = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.AminoAcidResidueBase.mass.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":40
 *         public basestring symbol
 *         public double mass
 *         public CComposition composition             # <<<<<<<<<<<<<<
 *         public object neutral_loss
 *         public Py_hash_t _hash
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->composition));
  __pyx_r = ((PyObject *)__pyx_v_self->composition);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(1, 40, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->composition));
  __pyx_v_self->composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.AminoAcidResidueBase.composition.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->composition));
  __pyx_v_self->composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":41
 *         public double mass
 *         public CComposition composition
 *         public object neutral_loss             # <<<<<<<<<<<<<<
 *         public Py_hash_t _hash
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->neutral_loss);
  __pyx_r = __pyx_v_self->neutral_loss;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__", 0);
  __Pyx_INCREF(__pyx_v_value);
  __Pyx_GIVEREF(__pyx_v_value);
  __Pyx_GOTREF(__pyx_v_self->neutral_loss);
  __Pyx_DECREF(__pyx_v_self->neutral_loss);
  __pyx_v_self->neutral_loss = __pyx_v_value;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->neutral_loss);
  __Pyx_DECREF(__pyx_v_self->neutral_loss);
  __pyx_v_self->neutral_loss = Py_None;

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":42
 *         public CComposition composition
 *         public object neutral_loss
 *         public Py_hash_t _hash             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_5_hash_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_5_hash_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_5_hash___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_5_hash___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyInt_FromHash_t(__pyx_v_self->_hash); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 42, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.AminoAcidResidueBase._hash.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_5_hash_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_5_hash_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_5_hash_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_5_hash_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  Py_hash_t __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __Pyx_PyInt_AsHash_t(__pyx_v_value); if (unlikely((__pyx_t_1 == (Py_hash_t)-1) && PyErr_Occurred())) __PYX_ERR(1, 42, __pyx_L1_error)
  __pyx_v_self->_hash = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.AminoAcidResidueBase._hash.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_8__reduce_cython__[] = "AminoAcidResidueBase.__reduce_cython__(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_8__reduce_cython__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_8__reduce_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self._hash, self.composition, self.mass, self.name, self.neutral_loss, self.symbol)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = __Pyx_PyInt_FromHash_t(__pyx_v_self->_hash); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyFloat_FromDouble(__pyx_v_self->mass); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = PyTuple_New(6); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->composition));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->composition));
  PyTuple_SET_ITEM(__pyx_t_3, 1, ((PyObject *)__pyx_v_self->composition));
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_t_2);
  __Pyx_INCREF(__pyx_v_self->name);
  __Pyx_GIVEREF(__pyx_v_self->name);
  PyTuple_SET_ITEM(__pyx_t_3, 3, __pyx_v_self->name);
  __Pyx_INCREF(__pyx_v_self->neutral_loss);
  __Pyx_GIVEREF(__pyx_v_self->neutral_loss);
  PyTuple_SET_ITEM(__pyx_t_3, 4, __pyx_v_self->neutral_loss);
  __Pyx_INCREF(__pyx_v_self->symbol);
  __Pyx_GIVEREF(__pyx_v_self->symbol);
  PyTuple_SET_ITEM(__pyx_t_3, 5, __pyx_v_self->symbol);
  __pyx_t_1 = 0;
  __pyx_t_2 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_3);
  __pyx_t_3 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self._hash, self.composition, self.mass, self.name, self.neutral_loss, self.symbol)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_3 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __pyx_v__dict = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":7
 *     state = (self._hash, self.composition, self.mass, self.name, self.neutral_loss, self.symbol)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_4 = (__pyx_v__dict != Py_None);
  __pyx_t_5 = (__pyx_t_4 != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_v__dict);
    __pyx_t_2 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_2));
    __pyx_t_2 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.composition is not None or self.name is not None or self.neutral_loss is not None or self.symbol is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self._hash, self.composition, self.mass, self.name, self.neutral_loss, self.symbol)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.composition is not None or self.name is not None or self.neutral_loss is not None or self.symbol is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_AminoAcidResidueBase, (type(self), 0x5b0643e, None), state
 */
  /*else*/ {
    __pyx_t_4 = (((PyObject *)__pyx_v_self->composition) != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->name != ((PyObject*)Py_None));
    __pyx_t_4 = (__pyx_t_6 != 0);
    if (!__pyx_t_4) {
    } else {
      __pyx_t_5 = __pyx_t_4;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = (__pyx_v_self->neutral_loss != Py_None);
    __pyx_t_6 = (__pyx_t_4 != 0);
    if (!__pyx_t_6) {
    } else {
      __pyx_t_5 = __pyx_t_6;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_6 = (__pyx_v_self->symbol != ((PyObject*)Py_None));
    __pyx_t_4 = (__pyx_t_6 != 0);
    __pyx_t_5 = __pyx_t_4;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_5;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.composition is not None or self.name is not None or self.neutral_loss is not None or self.symbol is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_AminoAcidResidueBase, (type(self), 0x5b0643e, None), state
 *     else:
 */
  __pyx_t_5 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_5) {

    /* "(tree fragment)":13
 *         use_setstate = self.composition is not None or self.name is not None or self.neutral_loss is not None or self.symbol is not None
 *     if use_setstate:
 *         return __pyx_unpickle_AminoAcidResidueBase, (type(self), 0x5b0643e, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_AminoAcidResidueBase, (type(self), 0x5b0643e, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_pyx_unpickle_AminoAcidResidueB); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_95446078);
    __Pyx_GIVEREF(__pyx_int_95446078);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_95446078);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_3, 2, Py_None);
    __pyx_t_1 = PyTuple_New(3); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_3);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_1, 2, __pyx_v_state);
    __pyx_t_2 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.composition is not None or self.name is not None or self.neutral_loss is not None or self.symbol is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_AminoAcidResidueBase, (type(self), 0x5b0643e, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_AminoAcidResidueBase, (type(self), 0x5b0643e, None), state
 *     else:
 *         return __pyx_unpickle_AminoAcidResidueBase, (type(self), 0x5b0643e, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_AminoAcidResidueBase__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_AminoAcidResidueB); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_3 = PyTuple_New(3); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_3, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_95446078);
    __Pyx_GIVEREF(__pyx_int_95446078);
    PyTuple_SET_ITEM(__pyx_t_3, 1, __pyx_int_95446078);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_3, 2, __pyx_v_state);
    __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_3);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
    __pyx_t_1 = 0;
    __pyx_t_3 = 0;
    __pyx_r = __pyx_t_2;
    __pyx_t_2 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.AminoAcidResidueBase.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_AminoAcidResidueBase, (type(self), 0x5b0643e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_AminoAcidResidueBase__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_10__setstate_cython__[] = "AminoAcidResidueBase.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_10__setstate_cython__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_10__setstate_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_AminoAcidResidueBase, (type(self), 0x5b0643e, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_AminoAcidResidueBase__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_4base___pyx_unpickle_AminoAcidResidueBase__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_AminoAcidResidueBase, (type(self), 0x5b0643e, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_AminoAcidResidueBase__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.AminoAcidResidueBase.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":188
 * cdef class ModificationBase(object):
 * 
 *     def __copy__(self):             # <<<<<<<<<<<<<<
 *         return self.clone()
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_1__copy__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16ModificationBase___copy__[] = "ModificationBase.__copy__(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_1__copy__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__copy__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase___copy__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase___copy__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__copy__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":189
 * 
 *     def __copy__(self):
 *         return self.clone()             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint is_a(self, object category):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_clone); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_3 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_3 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_3)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_3);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_1 = (__pyx_t_3) ? __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_3) : __Pyx_PyObject_CallNoArg(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3); __pyx_t_3 = 0;
  if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 189, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":188
 * cdef class ModificationBase(object):
 * 
 *     def __copy__(self):             # <<<<<<<<<<<<<<
 *         return self.clone()
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.ModificationBase.__copy__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":191
 *         return self.clone()
 * 
 *     cpdef bint is_a(self, object category):             # <<<<<<<<<<<<<<
 *         '''Returns whether or not this :class:`ModificationBase` object belongs to
 *         the specified :class:`~.ModificationCategory`.
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_3is_a(PyObject *__pyx_v_self, PyObject *__pyx_v_category); /*proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_is_a(CYTHON_UNUSED struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_category, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_a", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_a); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 191, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_3is_a)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_category) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_category);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 191, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 191, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/base.pyx":199
 *         bool
 *         '''
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     cpdef basestring serialize(self):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":191
 *         return self.clone()
 * 
 *     cpdef bint is_a(self, object category):             # <<<<<<<<<<<<<<
 *         '''Returns whether or not this :class:`ModificationBase` object belongs to
 *         the specified :class:`~.ModificationCategory`.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("glycopeptidepy._c.structure.base.ModificationBase.is_a", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_3is_a(PyObject *__pyx_v_self, PyObject *__pyx_v_category); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_2is_a[] = "ModificationBase.is_a(self, category) -> bool\nReturns whether or not this :class:`ModificationBase` object belongs to\n        the specified :class:`~.ModificationCategory`.\n\n        Returns\n        -------\n        bool\n        ";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_3is_a(PyObject *__pyx_v_self, PyObject *__pyx_v_category) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_a (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_2is_a(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_self), ((PyObject *)__pyx_v_category));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_2is_a(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, PyObject *__pyx_v_category) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_a", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_is_a(__pyx_v_self, __pyx_v_category, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 191, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.ModificationBase.is_a", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":201
 *         return False
 * 
 *     cpdef basestring serialize(self):             # <<<<<<<<<<<<<<
 *         '''A string representation for inclusion in sequences'''
 *         return self.name
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_5serialize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_serialize(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_serialize); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 201, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_5serialize)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 201, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        if (!(likely(__Pyx_PyBaseString_CheckExact(__pyx_t_2))||((__pyx_t_2) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", (PY_MAJOR_VERSION < 3 ? "basestring" : "str"), Py_TYPE(__pyx_t_2)->tp_name), 0))) __PYX_ERR(0, 201, __pyx_L1_error)
        __pyx_r = ((PyObject*)__pyx_t_2);
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/base.pyx":203
 *     cpdef basestring serialize(self):
 *         '''A string representation for inclusion in sequences'''
 *         return self.name             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint is_tracked_for(self, object category):
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->name);
  __pyx_r = __pyx_v_self->name;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":201
 *         return False
 * 
 *     cpdef basestring serialize(self):             # <<<<<<<<<<<<<<
 *         '''A string representation for inclusion in sequences'''
 *         return self.name
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.ModificationBase.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_5serialize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4serialize[] = "ModificationBase.serialize(self) -> basestring\nA string representation for inclusion in sequences";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_5serialize(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("serialize (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4serialize(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4serialize(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("serialize", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_serialize(__pyx_v_self, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 201, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.ModificationBase.serialize", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":205
 *         return self.name
 * 
 *     cpdef bint is_tracked_for(self, object category):             # <<<<<<<<<<<<<<
 *         """Determine if this :class:`ModificationBase` is tracked by a particular
 *         behavioral pattern associated with a :class:`~.ModificationCategory`.
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_7is_tracked_for(PyObject *__pyx_v_self, PyObject *__pyx_v_category); /*proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_is_tracked_for(CYTHON_UNUSED struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, CYTHON_UNUSED PyObject *__pyx_v_category, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_tracked_for", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_tracked_for); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 205, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_7is_tracked_for)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_category) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_category);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 205, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 205, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/base.pyx":221
 *         bool
 *         """
 *         return False             # <<<<<<<<<<<<<<
 * 
 * 
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":205
 *         return self.name
 * 
 *     cpdef bint is_tracked_for(self, object category):             # <<<<<<<<<<<<<<
 *         """Determine if this :class:`ModificationBase` is tracked by a particular
 *         behavioral pattern associated with a :class:`~.ModificationCategory`.
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("glycopeptidepy._c.structure.base.ModificationBase.is_tracked_for", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_7is_tracked_for(PyObject *__pyx_v_self, PyObject *__pyx_v_category); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_6is_tracked_for[] = "ModificationBase.is_tracked_for(self, category) -> bool\nDetermine if this :class:`ModificationBase` is tracked by a particular\n        behavioral pattern associated with a :class:`~.ModificationCategory`.\n\n        This relationship is distinct from :meth:`is_a` which merely observes that\n        the semantic relationship holds, not that any actual behavior is available.\n\n        Parameters\n        ----------\n        category : :class:`~.ModificationCategory`\n            The category to check\n\n        Returns\n        -------\n        bool\n        ";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_7is_tracked_for(PyObject *__pyx_v_self, PyObject *__pyx_v_category) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_tracked_for (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_6is_tracked_for(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_self), ((PyObject *)__pyx_v_category));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_6is_tracked_for(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, PyObject *__pyx_v_category) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_tracked_for", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_is_tracked_for(__pyx_v_self, __pyx_v_category, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 205, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.ModificationBase.is_tracked_for", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":52
 * cdef class ModificationBase(object):
 *     cdef:
 *         public basestring name             # <<<<<<<<<<<<<<
 *         public double mass
 *         public CComposition composition
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->name);
  __pyx_r = __pyx_v_self->name;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(__Pyx_PyBaseString_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", (PY_MAJOR_VERSION < 3 ? "basestring" : "str"), Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 52, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.ModificationBase.name.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->name);
  __Pyx_DECREF(__pyx_v_self->name);
  __pyx_v_self->name = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":53
 *     cdef:
 *         public basestring name
 *         public double mass             # <<<<<<<<<<<<<<
 *         public CComposition composition
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4mass_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4mass_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4mass___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(1, 53, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.ModificationBase.mass.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4mass_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4mass_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4mass_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  double __pyx_t_1;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  __pyx_t_1 = __pyx_PyFloat_AsDouble(__pyx_v_value); if (unlikely((__pyx_t_1 == (double)-1) && PyErr_Occurred())) __PYX_ERR(1, 53, __pyx_L1_error)
  __pyx_v_self->mass = __pyx_t_1;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.ModificationBase.mass.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":54
 *         public basestring name
 *         public double mass
 *         public CComposition composition             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint is_a(self, object category)
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->composition));
  __pyx_r = ((PyObject *)__pyx_v_self->composition);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(1, 54, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->composition));
  __pyx_v_self->composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.ModificationBase.composition.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->composition);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->composition));
  __pyx_v_self->composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_8__reduce_cython__[] = "ModificationBase.__reduce_cython__(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_9__reduce_cython__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_8__reduce_cython__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_8__reduce_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self) {
  PyObject *__pyx_v_state = 0;
  PyObject *__pyx_v__dict = 0;
  int __pyx_v_use_setstate;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_t_3;
  int __pyx_t_4;
  int __pyx_t_5;
  PyObject *__pyx_t_6 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce_cython__", 0);

  /* "(tree fragment)":5
 *     cdef object _dict
 *     cdef bint use_setstate
 *     state = (self.composition, self.mass, self.name)             # <<<<<<<<<<<<<<
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 */
  __pyx_t_1 = PyFloat_FromDouble(__pyx_v_self->mass); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->composition));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->composition));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self->composition));
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_1);
  __Pyx_INCREF(__pyx_v_self->name);
  __Pyx_GIVEREF(__pyx_v_self->name);
  PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_self->name);
  __pyx_t_1 = 0;
  __pyx_v_state = ((PyObject*)__pyx_t_2);
  __pyx_t_2 = 0;

  /* "(tree fragment)":6
 *     cdef bint use_setstate
 *     state = (self.composition, self.mass, self.name)
 *     _dict = getattr(self, '__dict__', None)             # <<<<<<<<<<<<<<
 *     if _dict is not None:
 *         state += (_dict,)
 */
  __pyx_t_2 = __Pyx_GetAttr3(((PyObject *)__pyx_v_self), __pyx_n_s_dict, Py_None); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_v__dict = __pyx_t_2;
  __pyx_t_2 = 0;

  /* "(tree fragment)":7
 *     state = (self.composition, self.mass, self.name)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
  __pyx_t_3 = (__pyx_v__dict != Py_None);
  __pyx_t_4 = (__pyx_t_3 != 0);
  if (__pyx_t_4) {

    /* "(tree fragment)":8
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:
 *         state += (_dict,)             # <<<<<<<<<<<<<<
 *         use_setstate = True
 *     else:
 */
    __pyx_t_2 = PyTuple_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_v__dict);
    __Pyx_GIVEREF(__pyx_v__dict);
    PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_v__dict);
    __pyx_t_1 = PyNumber_InPlaceAdd(__pyx_v_state, __pyx_t_2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 8, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF_SET(__pyx_v_state, ((PyObject*)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "(tree fragment)":9
 *     if _dict is not None:
 *         state += (_dict,)
 *         use_setstate = True             # <<<<<<<<<<<<<<
 *     else:
 *         use_setstate = self.composition is not None or self.name is not None
 */
    __pyx_v_use_setstate = 1;

    /* "(tree fragment)":7
 *     state = (self.composition, self.mass, self.name)
 *     _dict = getattr(self, '__dict__', None)
 *     if _dict is not None:             # <<<<<<<<<<<<<<
 *         state += (_dict,)
 *         use_setstate = True
 */
    goto __pyx_L3;
  }

  /* "(tree fragment)":11
 *         use_setstate = True
 *     else:
 *         use_setstate = self.composition is not None or self.name is not None             # <<<<<<<<<<<<<<
 *     if use_setstate:
 *         return __pyx_unpickle_ModificationBase, (type(self), 0x54d98d7, None), state
 */
  /*else*/ {
    __pyx_t_3 = (((PyObject *)__pyx_v_self->composition) != Py_None);
    __pyx_t_5 = (__pyx_t_3 != 0);
    if (!__pyx_t_5) {
    } else {
      __pyx_t_4 = __pyx_t_5;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_5 = (__pyx_v_self->name != ((PyObject*)Py_None));
    __pyx_t_3 = (__pyx_t_5 != 0);
    __pyx_t_4 = __pyx_t_3;
    __pyx_L4_bool_binop_done:;
    __pyx_v_use_setstate = __pyx_t_4;
  }
  __pyx_L3:;

  /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.composition is not None or self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ModificationBase, (type(self), 0x54d98d7, None), state
 *     else:
 */
  __pyx_t_4 = (__pyx_v_use_setstate != 0);
  if (__pyx_t_4) {

    /* "(tree fragment)":13
 *         use_setstate = self.composition is not None or self.name is not None
 *     if use_setstate:
 *         return __pyx_unpickle_ModificationBase, (type(self), 0x54d98d7, None), state             # <<<<<<<<<<<<<<
 *     else:
 *         return __pyx_unpickle_ModificationBase, (type(self), 0x54d98d7, state)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_1, __pyx_n_s_pyx_unpickle_ModificationBase); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_88971479);
    __Pyx_GIVEREF(__pyx_int_88971479);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_int_88971479);
    __Pyx_INCREF(Py_None);
    __Pyx_GIVEREF(Py_None);
    PyTuple_SET_ITEM(__pyx_t_2, 2, Py_None);
    __pyx_t_6 = PyTuple_New(3); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 13, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_1);
    PyTuple_SET_ITEM(__pyx_t_6, 0, __pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_6, 1, __pyx_t_2);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_6, 2, __pyx_v_state);
    __pyx_t_1 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_6;
    __pyx_t_6 = 0;
    goto __pyx_L0;

    /* "(tree fragment)":12
 *     else:
 *         use_setstate = self.composition is not None or self.name is not None
 *     if use_setstate:             # <<<<<<<<<<<<<<
 *         return __pyx_unpickle_ModificationBase, (type(self), 0x54d98d7, None), state
 *     else:
 */
  }

  /* "(tree fragment)":15
 *         return __pyx_unpickle_ModificationBase, (type(self), 0x54d98d7, None), state
 *     else:
 *         return __pyx_unpickle_ModificationBase, (type(self), 0x54d98d7, state)             # <<<<<<<<<<<<<<
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ModificationBase__set_state(self, __pyx_state)
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __Pyx_GetModuleGlobalName(__pyx_t_6, __pyx_n_s_pyx_unpickle_ModificationBase); if (unlikely(!__pyx_t_6)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_6);
    __pyx_t_2 = PyTuple_New(3); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_GIVEREF(((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)Py_TYPE(((PyObject *)__pyx_v_self))));
    __Pyx_INCREF(__pyx_int_88971479);
    __Pyx_GIVEREF(__pyx_int_88971479);
    PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_int_88971479);
    __Pyx_INCREF(__pyx_v_state);
    __Pyx_GIVEREF(__pyx_v_state);
    PyTuple_SET_ITEM(__pyx_t_2, 2, __pyx_v_state);
    __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 15, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_6);
    PyTuple_SET_ITEM(__pyx_t_1, 0, __pyx_t_6);
    __Pyx_GIVEREF(__pyx_t_2);
    PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_t_2);
    __pyx_t_6 = 0;
    __pyx_t_2 = 0;
    __pyx_r = __pyx_t_1;
    __pyx_t_1 = 0;
    goto __pyx_L0;
  }

  /* "(tree fragment)":1
 * def __reduce_cython__(self):             # <<<<<<<<<<<<<<
 *     cdef tuple state
 *     cdef object _dict
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_6);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.ModificationBase.__reduce_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v_state);
  __Pyx_XDECREF(__pyx_v__dict);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ModificationBase, (type(self), 0x54d98d7, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ModificationBase__set_state(self, __pyx_state)
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_10__setstate_cython__[] = "ModificationBase.__setstate_cython__(self, __pyx_state)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11__setstate_cython__(PyObject *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setstate_cython__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_10__setstate_cython__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v_self), ((PyObject *)__pyx_v___pyx_state));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_10__setstate_cython__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_self, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setstate_cython__", 0);

  /* "(tree fragment)":17
 *         return __pyx_unpickle_ModificationBase, (type(self), 0x54d98d7, state)
 * def __setstate_cython__(self, __pyx_state):
 *     __pyx_unpickle_ModificationBase__set_state(self, __pyx_state)             # <<<<<<<<<<<<<<
 */
  if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 17, __pyx_L1_error)
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_4base___pyx_unpickle_ModificationBase__set_state(__pyx_v_self, ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 17, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

  /* "(tree fragment)":16
 *     else:
 *         return __pyx_unpickle_ModificationBase, (type(self), 0x54d98d7, state)
 * def __setstate_cython__(self, __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_unpickle_ModificationBase__set_state(self, __pyx_state)
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.ModificationBase.__setstate_cython__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":227
 * cdef class SequencePosition(object):
 * 
 *     def __init__(self, parts):             # <<<<<<<<<<<<<<
 *         self.amino_acid = PySequence_GetItem(parts, 0)
 *         self.modifications = PySequence_GetItem(parts, 1)
 */

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_1__init__(PyObject *__pyx_v_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v_parts = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__init__ (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_parts,0};
    PyObject* values[1] = {0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_parts)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__init__") < 0)) __PYX_ERR(0, 227, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 1) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
    }
    __pyx_v_parts = values[0];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__init__", 1, 1, 1, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(0, 227, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return -1;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition___init__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self), __pyx_v_parts);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition___init__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_parts) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__init__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":228
 * 
 *     def __init__(self, parts):
 *         self.amino_acid = PySequence_GetItem(parts, 0)             # <<<<<<<<<<<<<<
 *         self.modifications = PySequence_GetItem(parts, 1)
 * 
 */
  __pyx_t_1 = PySequence_GetItem(__pyx_v_parts, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 228, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase))))) __PYX_ERR(0, 228, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->amino_acid);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->amino_acid));
  __pyx_v_self->amino_acid = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/base.pyx":229
 *     def __init__(self, parts):
 *         self.amino_acid = PySequence_GetItem(parts, 0)
 *         self.modifications = PySequence_GetItem(parts, 1)             # <<<<<<<<<<<<<<
 * 
 *     def __iter__(self):
 */
  __pyx_t_1 = PySequence_GetItem(__pyx_v_parts, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 229, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(0, 229, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->modifications);
  __Pyx_DECREF(__pyx_v_self->modifications);
  __pyx_v_self->modifications = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/base.pyx":227
 * cdef class SequencePosition(object):
 * 
 *     def __init__(self, parts):             # <<<<<<<<<<<<<<
 *         self.amino_acid = PySequence_GetItem(parts, 0)
 *         self.modifications = PySequence_GetItem(parts, 1)
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.__init__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static PyObject *__pyx_gb_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_4generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value); /* proto */

/* "glycopeptidepy/_c/structure/base.pyx":231
 *         self.modifications = PySequence_GetItem(parts, 1)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         yield self.amino_acid
 *         yield self.modifications
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_3__iter__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_3__iter__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_2__iter__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_2__iter__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ *__pyx_cur_scope;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__iter__", 0);
  __pyx_cur_scope = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ *)__pyx_tp_new_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__(__pyx_ptype_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__, __pyx_empty_tuple, NULL);
  if (unlikely(!__pyx_cur_scope)) {
    __pyx_cur_scope = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ *)Py_None);
    __Pyx_INCREF(Py_None);
    __PYX_ERR(0, 231, __pyx_L1_error)
  } else {
    __Pyx_GOTREF(__pyx_cur_scope);
  }
  __pyx_cur_scope->__pyx_v_self = __pyx_v_self;
  __Pyx_INCREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  __Pyx_GIVEREF((PyObject *)__pyx_cur_scope->__pyx_v_self);
  {
    __pyx_CoroutineObject *gen = __Pyx_Generator_New((__pyx_coroutine_body_t) __pyx_gb_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_4generator, NULL, (PyObject *) __pyx_cur_scope, __pyx_n_s_iter, __pyx_n_s_SequencePosition___iter, __pyx_n_s_glycopeptidepy__c_structure_base); if (unlikely(!gen)) __PYX_ERR(0, 231, __pyx_L1_error)
    __Pyx_DECREF(__pyx_cur_scope);
    __Pyx_RefNannyFinishContext();
    return (PyObject *) gen;
  }

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __Pyx_DECREF(((PyObject *)__pyx_cur_scope));
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_gb_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_4generator(__pyx_CoroutineObject *__pyx_generator, CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject *__pyx_sent_value) /* generator body */
{
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ *__pyx_cur_scope = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ *)__pyx_generator->closure);
  PyObject *__pyx_r = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__iter__", 0);
  switch (__pyx_generator->resume_label) {
    case 0: goto __pyx_L3_first_run;
    case 1: goto __pyx_L4_resume_from_yield;
    case 2: goto __pyx_L5_resume_from_yield;
    default: /* CPython raises the right error here */
    __Pyx_RefNannyFinishContext();
    return NULL;
  }
  __pyx_L3_first_run:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 231, __pyx_L1_error)

  /* "glycopeptidepy/_c/structure/base.pyx":232
 * 
 *     def __iter__(self):
 *         yield self.amino_acid             # <<<<<<<<<<<<<<
 *         yield self.modifications
 * 
 */
  __Pyx_INCREF(((PyObject *)__pyx_cur_scope->__pyx_v_self->amino_acid));
  __pyx_r = ((PyObject *)__pyx_cur_scope->__pyx_v_self->amino_acid);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  /* return from generator, yielding value */
  __pyx_generator->resume_label = 1;
  return __pyx_r;
  __pyx_L4_resume_from_yield:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 232, __pyx_L1_error)

  /* "glycopeptidepy/_c/structure/base.pyx":233
 *     def __iter__(self):
 *         yield self.amino_acid
 *         yield self.modifications             # <<<<<<<<<<<<<<
 * 
 *     def __getitem__(self, i):
 */
  __Pyx_INCREF(__pyx_cur_scope->__pyx_v_self->modifications);
  __pyx_r = __pyx_cur_scope->__pyx_v_self->modifications;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  /* return from generator, yielding value */
  __pyx_generator->resume_label = 2;
  return __pyx_r;
  __pyx_L5_resume_from_yield:;
  if (unlikely(!__pyx_sent_value)) __PYX_ERR(0, 233, __pyx_L1_error)
  CYTHON_MAYBE_UNUSED_VAR(__pyx_cur_scope);

  /* "glycopeptidepy/_c/structure/base.pyx":231
 *         self.modifications = PySequence_GetItem(parts, 1)
 * 
 *     def __iter__(self):             # <<<<<<<<<<<<<<
 *         yield self.amino_acid
 *         yield self.modifications
 */

  /* function exit code */
  PyErr_SetNone(PyExc_StopIteration);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_AddTraceback("__iter__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_r); __pyx_r = 0;
  #if !CYTHON_USE_EXC_INFO_STACK
  __Pyx_Coroutine_ResetAndClearException(__pyx_generator);
  #endif
  __pyx_generator->resume_label = -1;
  __Pyx_Coroutine_clear((PyObject*)__pyx_generator);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":235
 *         yield self.modifications
 * 
 *     def __getitem__(self, i):             # <<<<<<<<<<<<<<
 *         if i == 0:
 *             return self.amino_acid
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_6__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_i); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_6__getitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_i) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__getitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_5__getitem__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self), ((PyObject *)__pyx_v_i));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_5__getitem__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_i) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__getitem__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":236
 * 
 *     def __getitem__(self, i):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *             return self.amino_acid
 *         elif i == 1:
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_i, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 236, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 236, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/base.pyx":237
 *     def __getitem__(self, i):
 *         if i == 0:
 *             return self.amino_acid             # <<<<<<<<<<<<<<
 *         elif i == 1:
 *             return self.modifications
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(((PyObject *)__pyx_v_self->amino_acid));
    __pyx_r = ((PyObject *)__pyx_v_self->amino_acid);
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/base.pyx":236
 * 
 *     def __getitem__(self, i):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *             return self.amino_acid
 *         elif i == 1:
 */
  }

  /* "glycopeptidepy/_c/structure/base.pyx":238
 *         if i == 0:
 *             return self.amino_acid
 *         elif i == 1:             # <<<<<<<<<<<<<<
 *             return self.modifications
 *         else:
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_i, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 238, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 238, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(__pyx_t_2)) {

    /* "glycopeptidepy/_c/structure/base.pyx":239
 *             return self.amino_acid
 *         elif i == 1:
 *             return self.modifications             # <<<<<<<<<<<<<<
 *         else:
 *             raise IndexError(i)
 */
    __Pyx_XDECREF(__pyx_r);
    __Pyx_INCREF(__pyx_v_self->modifications);
    __pyx_r = __pyx_v_self->modifications;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/base.pyx":238
 *         if i == 0:
 *             return self.amino_acid
 *         elif i == 1:             # <<<<<<<<<<<<<<
 *             return self.modifications
 *         else:
 */
  }

  /* "glycopeptidepy/_c/structure/base.pyx":241
 *             return self.modifications
 *         else:
 *             raise IndexError(i)             # <<<<<<<<<<<<<<
 * 
 *     def __setitem__(self, i, value):
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_IndexError, __pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 241, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 241, __pyx_L1_error)
  }

  /* "glycopeptidepy/_c/structure/base.pyx":235
 *         yield self.modifications
 * 
 *     def __getitem__(self, i):             # <<<<<<<<<<<<<<
 *         if i == 0:
 *             return self.amino_acid
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.__getitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":243
 *             raise IndexError(i)
 * 
 *     def __setitem__(self, i, value):             # <<<<<<<<<<<<<<
 *         if i == 0:
 *             self.amino_acid = value
 */

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_8__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_i, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_8__setitem__(PyObject *__pyx_v_self, PyObject *__pyx_v_i, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__setitem__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_7__setitem__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self), ((PyObject *)__pyx_v_i), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_7__setitem__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_i, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__setitem__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":244
 * 
 *     def __setitem__(self, i, value):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *             self.amino_acid = value
 *         elif i == 1:
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_i, __pyx_int_0, 0, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 244, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/base.pyx":245
 *     def __setitem__(self, i, value):
 *         if i == 0:
 *             self.amino_acid = value             # <<<<<<<<<<<<<<
 *         elif i == 1:
 *             self.modifications = value
 */
    if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase))))) __PYX_ERR(0, 245, __pyx_L1_error)
    __pyx_t_1 = __pyx_v_value;
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->amino_acid);
    __Pyx_DECREF(((PyObject *)__pyx_v_self->amino_acid));
    __pyx_v_self->amino_acid = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/base.pyx":244
 * 
 *     def __setitem__(self, i, value):
 *         if i == 0:             # <<<<<<<<<<<<<<
 *             self.amino_acid = value
 *         elif i == 1:
 */
    goto __pyx_L3;
  }

  /* "glycopeptidepy/_c/structure/base.pyx":246
 *         if i == 0:
 *             self.amino_acid = value
 *         elif i == 1:             # <<<<<<<<<<<<<<
 *             self.modifications = value
 *         else:
 */
  __pyx_t_1 = __Pyx_PyInt_EqObjC(__pyx_v_i, __pyx_int_1, 1, 0); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 246, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  if (likely(__pyx_t_2)) {

    /* "glycopeptidepy/_c/structure/base.pyx":247
 *             self.amino_acid = value
 *         elif i == 1:
 *             self.modifications = value             # <<<<<<<<<<<<<<
 *         else:
 *             raise IndexError(i)
 */
    if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(0, 247, __pyx_L1_error)
    __pyx_t_1 = __pyx_v_value;
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_GIVEREF(__pyx_t_1);
    __Pyx_GOTREF(__pyx_v_self->modifications);
    __Pyx_DECREF(__pyx_v_self->modifications);
    __pyx_v_self->modifications = ((PyObject*)__pyx_t_1);
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/base.pyx":246
 *         if i == 0:
 *             self.amino_acid = value
 *         elif i == 1:             # <<<<<<<<<<<<<<
 *             self.modifications = value
 *         else:
 */
    goto __pyx_L3;
  }

  /* "glycopeptidepy/_c/structure/base.pyx":249
 *             self.modifications = value
 *         else:
 *             raise IndexError(i)             # <<<<<<<<<<<<<<
 * 
 *     def __eq__(self, other):
 */
  /*else*/ {
    __pyx_t_1 = __Pyx_PyObject_CallOneArg(__pyx_builtin_IndexError, __pyx_v_i); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 249, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_Raise(__pyx_t_1, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __PYX_ERR(0, 249, __pyx_L1_error)
  }
  __pyx_L3:;

  /* "glycopeptidepy/_c/structure/base.pyx":243
 *             raise IndexError(i)
 * 
 *     def __setitem__(self, i, value):             # <<<<<<<<<<<<<<
 *         if i == 0:
 *             self.amino_acid = value
 */

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.__setitem__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":251
 *             raise IndexError(i)
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         cdef:
 *             SequencePosition other_t
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10__eq__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__eq__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_9__eq__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_9__eq__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_other) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_other_t = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  int __pyx_t_2;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__eq__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":254
 *         cdef:
 *             SequencePosition other_t
 *         if isinstance(other, SequencePosition):             # <<<<<<<<<<<<<<
 *             other_t = other
 *             return self.amino_acid.equal_to(other_t.amino_acid) and self.modifications == other_t.modifications
 */
  __pyx_t_1 = __Pyx_TypeCheck(__pyx_v_other, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_SequencePosition); 
  __pyx_t_2 = (__pyx_t_1 != 0);
  if (__pyx_t_2) {

    /* "glycopeptidepy/_c/structure/base.pyx":255
 *             SequencePosition other_t
 *         if isinstance(other, SequencePosition):
 *             other_t = other             # <<<<<<<<<<<<<<
 *             return self.amino_acid.equal_to(other_t.amino_acid) and self.modifications == other_t.modifications
 *         else:
 */
    if (!(likely(((__pyx_v_other) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_other, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_SequencePosition))))) __PYX_ERR(0, 255, __pyx_L1_error)
    __pyx_t_3 = __pyx_v_other;
    __Pyx_INCREF(__pyx_t_3);
    __pyx_v_other_t = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_t_3);
    __pyx_t_3 = 0;

    /* "glycopeptidepy/_c/structure/base.pyx":256
 *         if isinstance(other, SequencePosition):
 *             other_t = other
 *             return self.amino_acid.equal_to(other_t.amino_acid) and self.modifications == other_t.modifications             # <<<<<<<<<<<<<<
 *         else:
 *             return self.amino_acid == other.amino_acid and self.modifications == other.modifications
 */
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = ((PyObject *)__pyx_v_other_t->amino_acid);
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_2 = ((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v_self->amino_acid->__pyx_vtab)->equal_to(__pyx_v_self->amino_acid, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_t_4));
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (__pyx_t_2) {
    } else {
      __pyx_t_4 = __Pyx_PyBool_FromLong(__pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 256, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_4);
      __pyx_t_3 = __pyx_t_4;
      __pyx_t_4 = 0;
      goto __pyx_L4_bool_binop_done;
    }
    __pyx_t_4 = PyObject_RichCompare(__pyx_v_self->modifications, __pyx_v_other_t->modifications, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 256, __pyx_L1_error)
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_3 = __pyx_t_4;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_L4_bool_binop_done:;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;

    /* "glycopeptidepy/_c/structure/base.pyx":254
 *         cdef:
 *             SequencePosition other_t
 *         if isinstance(other, SequencePosition):             # <<<<<<<<<<<<<<
 *             other_t = other
 *             return self.amino_acid.equal_to(other_t.amino_acid) and self.modifications == other_t.modifications
 */
  }

  /* "glycopeptidepy/_c/structure/base.pyx":258
 *             return self.amino_acid.equal_to(other_t.amino_acid) and self.modifications == other_t.modifications
 *         else:
 *             return self.amino_acid == other.amino_acid and self.modifications == other.modifications             # <<<<<<<<<<<<<<
 * 
 *     def __ne__(self, other):
 */
  /*else*/ {
    __Pyx_XDECREF(__pyx_r);
    __pyx_t_4 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_n_s_amino_acid); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 258, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __pyx_t_5 = PyObject_RichCompare(((PyObject *)__pyx_v_self->amino_acid), __pyx_t_4, Py_EQ); __Pyx_XGOTREF(__pyx_t_5); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 258, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_5); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 258, __pyx_L1_error)
    if (__pyx_t_2) {
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    } else {
      __Pyx_INCREF(__pyx_t_5);
      __pyx_t_3 = __pyx_t_5;
      __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
      goto __pyx_L6_bool_binop_done;
    }
    __pyx_t_5 = __Pyx_PyObject_GetAttrStr(__pyx_v_other, __pyx_n_s_modifications); if (unlikely(!__pyx_t_5)) __PYX_ERR(0, 258, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_5);
    __pyx_t_4 = PyObject_RichCompare(__pyx_v_self->modifications, __pyx_t_5, Py_EQ); __Pyx_XGOTREF(__pyx_t_4); if (unlikely(!__pyx_t_4)) __PYX_ERR(0, 258, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_INCREF(__pyx_t_4);
    __pyx_t_3 = __pyx_t_4;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    __pyx_L6_bool_binop_done:;
    __pyx_r = __pyx_t_3;
    __pyx_t_3 = 0;
    goto __pyx_L0;
  }

  /* "glycopeptidepy/_c/structure/base.pyx":251
 *             raise IndexError(i)
 * 
 *     def __eq__(self, other):             # <<<<<<<<<<<<<<
 *         cdef:
 *             SequencePosition other_t
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.__eq__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_other_t);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":260
 *             return self.amino_acid == other.amino_acid and self.modifications == other.modifications
 * 
 *     def __ne__(self, other):             # <<<<<<<<<<<<<<
 *         return not (self == other)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_12__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_other); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_12__ne__(PyObject *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__ne__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_11__ne__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self), ((PyObject *)__pyx_v_other));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_11__ne__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_other) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_t_2;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__ne__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":261
 * 
 *     def __ne__(self, other):
 *         return not (self == other)             # <<<<<<<<<<<<<<
 * 
 *     def __reduce__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyObject_RichCompare(((PyObject *)__pyx_v_self), __pyx_v_other, Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 261, __pyx_L1_error)
  __pyx_t_2 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_2 < 0)) __PYX_ERR(0, 261, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_PyBool_FromLong((!__pyx_t_2)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 261, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":260
 *             return self.amino_acid == other.amino_acid and self.modifications == other.modifications
 * 
 *     def __ne__(self, other):             # <<<<<<<<<<<<<<
 *         return not (self == other)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.__ne__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":263
 *         return not (self == other)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return self.__class__, ((self.amino_acid, self.modifications),)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_14__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13__reduce__[] = "SequencePosition.__reduce__(self)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_14__reduce__(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__reduce__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13__reduce__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13__reduce__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__reduce__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":264
 * 
 *     def __reduce__(self):
 *         return self.__class__, ((self.amino_acid, self.modifications),)             # <<<<<<<<<<<<<<
 * 
 *     def __len__(self):
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_class); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->amino_acid));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->amino_acid));
  PyTuple_SET_ITEM(__pyx_t_2, 0, ((PyObject *)__pyx_v_self->amino_acid));
  __Pyx_INCREF(__pyx_v_self->modifications);
  __Pyx_GIVEREF(__pyx_v_self->modifications);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_v_self->modifications);
  __pyx_t_3 = PyTuple_New(1); if (unlikely(!__pyx_t_3)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_GIVEREF(__pyx_t_2);
  PyTuple_SET_ITEM(__pyx_t_3, 0, __pyx_t_2);
  __pyx_t_2 = 0;
  __pyx_t_2 = PyTuple_New(2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 264, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_1);
  PyTuple_SET_ITEM(__pyx_t_2, 0, __pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_3);
  PyTuple_SET_ITEM(__pyx_t_2, 1, __pyx_t_3);
  __pyx_t_1 = 0;
  __pyx_t_3 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":263
 *         return not (self == other)
 * 
 *     def __reduce__(self):             # <<<<<<<<<<<<<<
 *         return self.__class__, ((self.amino_acid, self.modifications),)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.__reduce__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":266
 *         return self.__class__, ((self.amino_acid, self.modifications),)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return 2
 * 
 */

/* Python wrapper */
static Py_ssize_t __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_16__len__(PyObject *__pyx_v_self); /*proto*/
static Py_ssize_t __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_16__len__(PyObject *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_15__len__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static Py_ssize_t __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_15__len__(CYTHON_UNUSED struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self) {
  Py_ssize_t __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__len__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":267
 * 
 *     def __len__(self):
 *         return 2             # <<<<<<<<<<<<<<
 * 
 *     def __repr__(self):
 */
  __pyx_r = 2;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":266
 *         return self.__class__, ((self.amino_acid, self.modifications),)
 * 
 *     def __len__(self):             # <<<<<<<<<<<<<<
 *         return 2
 * 
 */

  /* function exit code */
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":269
 *         return 2
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "[%r, %r]" % (self.amino_acid, self.modifications)
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_18__repr__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_18__repr__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__repr__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_17__repr__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_17__repr__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__repr__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":270
 * 
 *     def __repr__(self):
 *         return "[%r, %r]" % (self.amino_acid, self.modifications)             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyTuple_New(2); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->amino_acid));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_self->amino_acid));
  PyTuple_SET_ITEM(__pyx_t_1, 0, ((PyObject *)__pyx_v_self->amino_acid));
  __Pyx_INCREF(__pyx_v_self->modifications);
  __Pyx_GIVEREF(__pyx_v_self->modifications);
  PyTuple_SET_ITEM(__pyx_t_1, 1, __pyx_v_self->modifications);
  __pyx_t_2 = __Pyx_PyString_Format(__pyx_kp_s_r_r, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 270, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":269
 *         return 2
 * 
 *     def __repr__(self):             # <<<<<<<<<<<<<<
 *         return "[%r, %r]" % (self.amino_acid, self.modifications)
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.__repr__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":273
 * 
 *     @staticmethod
 *     cdef SequencePosition _create(AminoAcidResidueBase amino_acid, list modifications):             # <<<<<<<<<<<<<<
 *         cdef SequencePosition inst = SequencePosition.__new__(SequencePosition)
 *         inst.amino_acid = amino_acid
 */

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition__create(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v_amino_acid, PyObject *__pyx_v_modifications) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_inst = 0;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("_create", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":274
 *     @staticmethod
 *     cdef SequencePosition _create(AminoAcidResidueBase amino_acid, list modifications):
 *         cdef SequencePosition inst = SequencePosition.__new__(SequencePosition)             # <<<<<<<<<<<<<<
 *         inst.amino_acid = amino_acid
 *         inst.modifications = modifications
 */
  __pyx_t_1 = ((PyObject *)__pyx_tp_new_14glycopeptidepy_2_c_9structure_4base_SequencePosition(((PyTypeObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_SequencePosition), __pyx_empty_tuple, NULL)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 274, __pyx_L1_error)
  __Pyx_GOTREF(((PyObject *)__pyx_t_1));
  __pyx_v_inst = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "glycopeptidepy/_c/structure/base.pyx":275
 *     cdef SequencePosition _create(AminoAcidResidueBase amino_acid, list modifications):
 *         cdef SequencePosition inst = SequencePosition.__new__(SequencePosition)
 *         inst.amino_acid = amino_acid             # <<<<<<<<<<<<<<
 *         inst.modifications = modifications
 *         return inst
 */
  __Pyx_INCREF(((PyObject *)__pyx_v_amino_acid));
  __Pyx_GIVEREF(((PyObject *)__pyx_v_amino_acid));
  __Pyx_GOTREF(__pyx_v_inst->amino_acid);
  __Pyx_DECREF(((PyObject *)__pyx_v_inst->amino_acid));
  __pyx_v_inst->amino_acid = __pyx_v_amino_acid;

  /* "glycopeptidepy/_c/structure/base.pyx":276
 *         cdef SequencePosition inst = SequencePosition.__new__(SequencePosition)
 *         inst.amino_acid = amino_acid
 *         inst.modifications = modifications             # <<<<<<<<<<<<<<
 *         return inst
 * 
 */
  __Pyx_INCREF(__pyx_v_modifications);
  __Pyx_GIVEREF(__pyx_v_modifications);
  __Pyx_GOTREF(__pyx_v_inst->modifications);
  __Pyx_DECREF(__pyx_v_inst->modifications);
  __pyx_v_inst->modifications = __pyx_v_modifications;

  /* "glycopeptidepy/_c/structure/base.pyx":277
 *         inst.amino_acid = amino_acid
 *         inst.modifications = modifications
 *         return inst             # <<<<<<<<<<<<<<
 * 
 *     cdef double get_mass(self):
 */
  __Pyx_XDECREF(((PyObject *)__pyx_r));
  __Pyx_INCREF(((PyObject *)__pyx_v_inst));
  __pyx_r = __pyx_v_inst;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":273
 * 
 *     @staticmethod
 *     cdef SequencePosition _create(AminoAcidResidueBase amino_acid, list modifications):             # <<<<<<<<<<<<<<
 *         cdef SequencePosition inst = SequencePosition.__new__(SequencePosition)
 *         inst.amino_acid = amino_acid
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition._create", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_inst);
  __Pyx_XGIVEREF((PyObject *)__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":279
 *         return inst
 * 
 *     cdef double get_mass(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             double mass
 */

static double __pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_get_mass(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self) {
  double __pyx_v_mass;
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_mod = 0;
  double __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_ssize_t __pyx_t_2;
  size_t __pyx_t_3;
  size_t __pyx_t_4;
  size_t __pyx_t_5;
  PyObject *__pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("get_mass", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":285
 *             ModificationBase mod
 * 
 *         mass = 0             # <<<<<<<<<<<<<<
 *         mass += self.amino_acid.mass
 *         n = PyList_Size(self.modifications)
 */
  __pyx_v_mass = 0.0;

  /* "glycopeptidepy/_c/structure/base.pyx":286
 * 
 *         mass = 0
 *         mass += self.amino_acid.mass             # <<<<<<<<<<<<<<
 *         n = PyList_Size(self.modifications)
 *         for i in range(n):
 */
  __pyx_v_mass = (__pyx_v_mass + __pyx_v_self->amino_acid->mass);

  /* "glycopeptidepy/_c/structure/base.pyx":287
 *         mass = 0
 *         mass += self.amino_acid.mass
 *         n = PyList_Size(self.modifications)             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             mod = <ModificationBase>PyList_GET_ITEM(self.modifications, i)
 */
  __pyx_t_1 = __pyx_v_self->modifications;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_2 = PyList_Size(__pyx_t_1); if (unlikely(__pyx_t_2 == ((Py_ssize_t)-1L))) __PYX_ERR(0, 287, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_n = __pyx_t_2;

  /* "glycopeptidepy/_c/structure/base.pyx":288
 *         mass += self.amino_acid.mass
 *         n = PyList_Size(self.modifications)
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             mod = <ModificationBase>PyList_GET_ITEM(self.modifications, i)
 *             mass += mod.mass
 */
  __pyx_t_3 = __pyx_v_n;
  __pyx_t_4 = __pyx_t_3;
  for (__pyx_t_5 = 0; __pyx_t_5 < __pyx_t_4; __pyx_t_5+=1) {
    __pyx_v_i = __pyx_t_5;

    /* "glycopeptidepy/_c/structure/base.pyx":289
 *         n = PyList_Size(self.modifications)
 *         for i in range(n):
 *             mod = <ModificationBase>PyList_GET_ITEM(self.modifications, i)             # <<<<<<<<<<<<<<
 *             mass += mod.mass
 *         return mass
 */
    __pyx_t_1 = __pyx_v_self->modifications;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_6 = PyList_GET_ITEM(__pyx_t_1, __pyx_v_i);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((PyObject *)__pyx_t_6);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_mod, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/base.pyx":290
 *         for i in range(n):
 *             mod = <ModificationBase>PyList_GET_ITEM(self.modifications, i)
 *             mass += mod.mass             # <<<<<<<<<<<<<<
 *         return mass
 * 
 */
    __pyx_v_mass = (__pyx_v_mass + __pyx_v_mod->mass);
  }

  /* "glycopeptidepy/_c/structure/base.pyx":291
 *             mod = <ModificationBase>PyList_GET_ITEM(self.modifications, i)
 *             mass += mod.mass
 *         return mass             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint has_modification(self, modification):
 */
  __pyx_r = __pyx_v_mass;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":279
 *         return inst
 * 
 *     cdef double get_mass(self):             # <<<<<<<<<<<<<<
 *         cdef:
 *             double mass
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_WriteUnraisable("glycopeptidepy._c.structure.base.SequencePosition.get_mass", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mod);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":293
 *         return mass
 * 
 *     cpdef bint has_modification(self, modification):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_20has_modification(PyObject *__pyx_v_self, PyObject *__pyx_v_modification); /*proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_has_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_modification, int __pyx_skip_dispatch) {
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_mod = 0;
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  size_t __pyx_t_9;
  PyObject *__pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("has_modification", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_has_modification); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 293, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_20has_modification)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_modification) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_modification);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 293, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 293, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/base.pyx":298
 *             ModificationBase mod
 * 
 *         n = PyList_Size(self.modifications)             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             mod = <ModificationBase>PyList_GET_ITEM(self.modifications, i)
 */
  __pyx_t_1 = __pyx_v_self->modifications;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_6 = PyList_Size(__pyx_t_1); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1L))) __PYX_ERR(0, 298, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_n = __pyx_t_6;

  /* "glycopeptidepy/_c/structure/base.pyx":299
 * 
 *         n = PyList_Size(self.modifications)
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             mod = <ModificationBase>PyList_GET_ITEM(self.modifications, i)
 *             if modification == mod:
 */
  __pyx_t_7 = __pyx_v_n;
  __pyx_t_8 = __pyx_t_7;
  for (__pyx_t_9 = 0; __pyx_t_9 < __pyx_t_8; __pyx_t_9+=1) {
    __pyx_v_i = __pyx_t_9;

    /* "glycopeptidepy/_c/structure/base.pyx":300
 *         n = PyList_Size(self.modifications)
 *         for i in range(n):
 *             mod = <ModificationBase>PyList_GET_ITEM(self.modifications, i)             # <<<<<<<<<<<<<<
 *             if modification == mod:
 *                 return True
 */
    __pyx_t_1 = __pyx_v_self->modifications;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_10 = PyList_GET_ITEM(__pyx_t_1, __pyx_v_i);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((PyObject *)__pyx_t_10);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_mod, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/base.pyx":301
 *         for i in range(n):
 *             mod = <ModificationBase>PyList_GET_ITEM(self.modifications, i)
 *             if modification == mod:             # <<<<<<<<<<<<<<
 *                 return True
 *         return False
 */
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_modification, ((PyObject *)__pyx_v_mod), Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 301, __pyx_L1_error)
    __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_5 < 0)) __PYX_ERR(0, 301, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_5) {

      /* "glycopeptidepy/_c/structure/base.pyx":302
 *             mod = <ModificationBase>PyList_GET_ITEM(self.modifications, i)
 *             if modification == mod:
 *                 return True             # <<<<<<<<<<<<<<
 *         return False
 * 
 */
      __pyx_r = 1;
      goto __pyx_L0;

      /* "glycopeptidepy/_c/structure/base.pyx":301
 *         for i in range(n):
 *             mod = <ModificationBase>PyList_GET_ITEM(self.modifications, i)
 *             if modification == mod:             # <<<<<<<<<<<<<<
 *                 return True
 *         return False
 */
    }
  }

  /* "glycopeptidepy/_c/structure/base.pyx":303
 *             if modification == mod:
 *                 return True
 *         return False             # <<<<<<<<<<<<<<
 * 
 *     cpdef bint is_modified(self):
 */
  __pyx_r = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":293
 *         return mass
 * 
 *     cpdef bint has_modification(self, modification):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("glycopeptidepy._c.structure.base.SequencePosition.has_modification", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mod);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_20has_modification(PyObject *__pyx_v_self, PyObject *__pyx_v_modification); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_19has_modification[] = "SequencePosition.has_modification(self, modification) -> bool";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_20has_modification(PyObject *__pyx_v_self, PyObject *__pyx_v_modification) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("has_modification (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_19has_modification(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self), ((PyObject *)__pyx_v_modification));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_19has_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_modification) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("has_modification", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_has_modification(__pyx_v_self, __pyx_v_modification, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 293, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.has_modification", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":305
 *         return False
 * 
 *     cpdef bint is_modified(self):             # <<<<<<<<<<<<<<
 *         return PyList_Size(self.modifications) > 0
 * 
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_22is_modified(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static int __pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_is_modified(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, int __pyx_skip_dispatch) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  Py_ssize_t __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_modified", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_is_modified); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 305, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_22is_modified)) {
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_t_4) : __Pyx_PyObject_CallNoArg(__pyx_t_3);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 305, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_t_5 = __Pyx_PyObject_IsTrue(__pyx_t_2); if (unlikely((__pyx_t_5 == (int)-1) && PyErr_Occurred())) __PYX_ERR(0, 305, __pyx_L1_error)
        __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
        __pyx_r = __pyx_t_5;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/base.pyx":306
 * 
 *     cpdef bint is_modified(self):
 *         return PyList_Size(self.modifications) > 0             # <<<<<<<<<<<<<<
 * 
 *     cpdef add_modification(self, modification):
 */
  __pyx_t_1 = __pyx_v_self->modifications;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_6 = PyList_Size(__pyx_t_1); if (unlikely(__pyx_t_6 == ((Py_ssize_t)-1L))) __PYX_ERR(0, 306, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_r = (__pyx_t_6 > 0);
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":305
 *         return False
 * 
 *     cpdef bint is_modified(self):             # <<<<<<<<<<<<<<
 *         return PyList_Size(self.modifications) > 0
 * 
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_WriteUnraisable("glycopeptidepy._c.structure.base.SequencePosition.is_modified", __pyx_clineno, __pyx_lineno, __pyx_filename, 1, 0);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_22is_modified(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_21is_modified[] = "SequencePosition.is_modified(self) -> bool";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_22is_modified(PyObject *__pyx_v_self, CYTHON_UNUSED PyObject *unused) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("is_modified (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_21is_modified(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_21is_modified(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("is_modified", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __Pyx_PyBool_FromLong(__pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_is_modified(__pyx_v_self, 1)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 305, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.is_modified", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":308
 *         return PyList_Size(self.modifications) > 0
 * 
 *     cpdef add_modification(self, modification):             # <<<<<<<<<<<<<<
 *         self.modifications.append(modification)
 * 
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_24add_modification(PyObject *__pyx_v_self, PyObject *__pyx_v_modification); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_add_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_modification, int __pyx_skip_dispatch) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  int __pyx_t_5;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_modification", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_add_modification); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 308, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_24add_modification)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_modification) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_modification);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 308, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/base.pyx":309
 * 
 *     cpdef add_modification(self, modification):
 *         self.modifications.append(modification)             # <<<<<<<<<<<<<<
 * 
 *     cpdef drop_modification(self, modification):
 */
  if (unlikely(__pyx_v_self->modifications == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "append");
    __PYX_ERR(0, 309, __pyx_L1_error)
  }
  __pyx_t_5 = __Pyx_PyList_Append(__pyx_v_self->modifications, __pyx_v_modification); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(0, 309, __pyx_L1_error)

  /* "glycopeptidepy/_c/structure/base.pyx":308
 *         return PyList_Size(self.modifications) > 0
 * 
 *     cpdef add_modification(self, modification):             # <<<<<<<<<<<<<<
 *         self.modifications.append(modification)
 * 
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.add_modification", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_24add_modification(PyObject *__pyx_v_self, PyObject *__pyx_v_modification); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_23add_modification[] = "SequencePosition.add_modification(self, modification)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_24add_modification(PyObject *__pyx_v_self, PyObject *__pyx_v_modification) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("add_modification (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_23add_modification(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self), ((PyObject *)__pyx_v_modification));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_23add_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_modification) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("add_modification", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_add_modification(__pyx_v_self, __pyx_v_modification, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 308, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.add_modification", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":311
 *         self.modifications.append(modification)
 * 
 *     cpdef drop_modification(self, modification):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n,
 */

static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_26drop_modification(PyObject *__pyx_v_self, PyObject *__pyx_v_modification); /*proto*/
static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_drop_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_modification, int __pyx_skip_dispatch) {
  size_t __pyx_v_i;
  size_t __pyx_v_n;
  int __pyx_v_index;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v_mod = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  Py_ssize_t __pyx_t_5;
  size_t __pyx_t_6;
  size_t __pyx_t_7;
  size_t __pyx_t_8;
  PyObject *__pyx_t_9;
  int __pyx_t_10;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("drop_modification", 0);
  /* Check if called by wrapper */
  if (unlikely(__pyx_skip_dispatch)) ;
  /* Check if overridden in Python */
  else if (unlikely((Py_TYPE(((PyObject *)__pyx_v_self))->tp_dictoffset != 0) || (Py_TYPE(((PyObject *)__pyx_v_self))->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)))) {
    #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    static PY_UINT64_T __pyx_tp_dict_version = __PYX_DICT_VERSION_INIT, __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
    if (unlikely(!__Pyx_object_dict_version_matches(((PyObject *)__pyx_v_self), __pyx_tp_dict_version, __pyx_obj_dict_version))) {
      PY_UINT64_T __pyx_type_dict_guard = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      #endif
      __pyx_t_1 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v_self), __pyx_n_s_drop_modification); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 311, __pyx_L1_error)
      __Pyx_GOTREF(__pyx_t_1);
      if (!PyCFunction_Check(__pyx_t_1) || (PyCFunction_GET_FUNCTION(__pyx_t_1) != (PyCFunction)(void*)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_26drop_modification)) {
        __Pyx_XDECREF(__pyx_r);
        __Pyx_INCREF(__pyx_t_1);
        __pyx_t_3 = __pyx_t_1; __pyx_t_4 = NULL;
        if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_3))) {
          __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_3);
          if (likely(__pyx_t_4)) {
            PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_3);
            __Pyx_INCREF(__pyx_t_4);
            __Pyx_INCREF(function);
            __Pyx_DECREF_SET(__pyx_t_3, function);
          }
        }
        __pyx_t_2 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_3, __pyx_t_4, __pyx_v_modification) : __Pyx_PyObject_CallOneArg(__pyx_t_3, __pyx_v_modification);
        __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
        if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 311, __pyx_L1_error)
        __Pyx_GOTREF(__pyx_t_2);
        __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
        __pyx_r = __pyx_t_2;
        __pyx_t_2 = 0;
        __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
        goto __pyx_L0;
      }
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
      __pyx_tp_dict_version = __Pyx_get_tp_dict_version(((PyObject *)__pyx_v_self));
      __pyx_obj_dict_version = __Pyx_get_object_dict_version(((PyObject *)__pyx_v_self));
      if (unlikely(__pyx_type_dict_guard != __pyx_tp_dict_version)) {
        __pyx_tp_dict_version = __pyx_obj_dict_version = __PYX_DICT_VERSION_INIT;
      }
      #endif
      __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
      #if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_PYTYPE_LOOKUP && CYTHON_USE_TYPE_SLOTS
    }
    #endif
  }

  /* "glycopeptidepy/_c/structure/base.pyx":317
 *             ModificationBase mod
 * 
 *         n = PyList_Size(self.modifications)             # <<<<<<<<<<<<<<
 *         index = -1
 *         for i in range(n):
 */
  __pyx_t_1 = __pyx_v_self->modifications;
  __Pyx_INCREF(__pyx_t_1);
  __pyx_t_5 = PyList_Size(__pyx_t_1); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1L))) __PYX_ERR(0, 317, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v_n = __pyx_t_5;

  /* "glycopeptidepy/_c/structure/base.pyx":318
 * 
 *         n = PyList_Size(self.modifications)
 *         index = -1             # <<<<<<<<<<<<<<
 *         for i in range(n):
 *             mod = <ModificationBase>PyList_GET_ITEM(self.modifications, i)
 */
  __pyx_v_index = -1;

  /* "glycopeptidepy/_c/structure/base.pyx":319
 *         n = PyList_Size(self.modifications)
 *         index = -1
 *         for i in range(n):             # <<<<<<<<<<<<<<
 *             mod = <ModificationBase>PyList_GET_ITEM(self.modifications, i)
 *             if modification == mod:
 */
  __pyx_t_6 = __pyx_v_n;
  __pyx_t_7 = __pyx_t_6;
  for (__pyx_t_8 = 0; __pyx_t_8 < __pyx_t_7; __pyx_t_8+=1) {
    __pyx_v_i = __pyx_t_8;

    /* "glycopeptidepy/_c/structure/base.pyx":320
 *         index = -1
 *         for i in range(n):
 *             mod = <ModificationBase>PyList_GET_ITEM(self.modifications, i)             # <<<<<<<<<<<<<<
 *             if modification == mod:
 *                 index = i
 */
    __pyx_t_1 = __pyx_v_self->modifications;
    __Pyx_INCREF(__pyx_t_1);
    __pyx_t_9 = PyList_GET_ITEM(__pyx_t_1, __pyx_v_i);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __pyx_t_1 = ((PyObject *)__pyx_t_9);
    __Pyx_INCREF(__pyx_t_1);
    __Pyx_XDECREF_SET(__pyx_v_mod, ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_t_1));
    __pyx_t_1 = 0;

    /* "glycopeptidepy/_c/structure/base.pyx":321
 *         for i in range(n):
 *             mod = <ModificationBase>PyList_GET_ITEM(self.modifications, i)
 *             if modification == mod:             # <<<<<<<<<<<<<<
 *                 index = i
 *                 break
 */
    __pyx_t_1 = PyObject_RichCompare(__pyx_v_modification, ((PyObject *)__pyx_v_mod), Py_EQ); __Pyx_XGOTREF(__pyx_t_1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 321, __pyx_L1_error)
    __pyx_t_10 = __Pyx_PyObject_IsTrue(__pyx_t_1); if (unlikely(__pyx_t_10 < 0)) __PYX_ERR(0, 321, __pyx_L1_error)
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    if (__pyx_t_10) {

      /* "glycopeptidepy/_c/structure/base.pyx":322
 *             mod = <ModificationBase>PyList_GET_ITEM(self.modifications, i)
 *             if modification == mod:
 *                 index = i             # <<<<<<<<<<<<<<
 *                 break
 *         if index == -1:
 */
      __pyx_v_index = __pyx_v_i;

      /* "glycopeptidepy/_c/structure/base.pyx":323
 *             if modification == mod:
 *                 index = i
 *                 break             # <<<<<<<<<<<<<<
 *         if index == -1:
 *             raise ValueError("Modification %s not found" % modification)
 */
      goto __pyx_L4_break;

      /* "glycopeptidepy/_c/structure/base.pyx":321
 *         for i in range(n):
 *             mod = <ModificationBase>PyList_GET_ITEM(self.modifications, i)
 *             if modification == mod:             # <<<<<<<<<<<<<<
 *                 index = i
 *                 break
 */
    }
  }
  __pyx_L4_break:;

  /* "glycopeptidepy/_c/structure/base.pyx":324
 *                 index = i
 *                 break
 *         if index == -1:             # <<<<<<<<<<<<<<
 *             raise ValueError("Modification %s not found" % modification)
 *         return self.modifications.pop(index)
 */
  __pyx_t_10 = ((__pyx_v_index == -1L) != 0);
  if (unlikely(__pyx_t_10)) {

    /* "glycopeptidepy/_c/structure/base.pyx":325
 *                 break
 *         if index == -1:
 *             raise ValueError("Modification %s not found" % modification)             # <<<<<<<<<<<<<<
 *         return self.modifications.pop(index)
 * 
 */
    __pyx_t_1 = __Pyx_PyString_FormatSafe(__pyx_kp_s_Modification_s_not_found, __pyx_v_modification); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 325, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __pyx_t_2 = __Pyx_PyObject_CallOneArg(__pyx_builtin_ValueError, __pyx_t_1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 325, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
    __Pyx_Raise(__pyx_t_2, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __PYX_ERR(0, 325, __pyx_L1_error)

    /* "glycopeptidepy/_c/structure/base.pyx":324
 *                 index = i
 *                 break
 *         if index == -1:             # <<<<<<<<<<<<<<
 *             raise ValueError("Modification %s not found" % modification)
 *         return self.modifications.pop(index)
 */
  }

  /* "glycopeptidepy/_c/structure/base.pyx":326
 *         if index == -1:
 *             raise ValueError("Modification %s not found" % modification)
 *         return self.modifications.pop(index)             # <<<<<<<<<<<<<<
 * 
 *     @property
 */
  __Pyx_XDECREF(__pyx_r);
  if (unlikely(__pyx_v_self->modifications == Py_None)) {
    PyErr_Format(PyExc_AttributeError, "'NoneType' object has no attribute '%.30s'", "pop");
    __PYX_ERR(0, 326, __pyx_L1_error)
  }
  __pyx_t_2 = __Pyx_PyList_PopIndex(__pyx_v_self->modifications, Py_None, __pyx_v_index, 1, int, __Pyx_PyInt_From_int); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 326, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_r = __pyx_t_2;
  __pyx_t_2 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":311
 *         self.modifications.append(modification)
 * 
 *     cpdef drop_modification(self, modification):             # <<<<<<<<<<<<<<
 *         cdef:
 *             size_t i, n,
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.drop_modification", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XDECREF((PyObject *)__pyx_v_mod);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_26drop_modification(PyObject *__pyx_v_self, PyObject *__pyx_v_modification); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_25drop_modification[] = "SequencePosition.drop_modification(self, modification)";
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_26drop_modification(PyObject *__pyx_v_self, PyObject *__pyx_v_modification) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("drop_modification (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_25drop_modification(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self), ((PyObject *)__pyx_v_modification));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_25drop_modification(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_modification) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("drop_modification", 0);
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = __pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_drop_modification(__pyx_v_self, __pyx_v_modification, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 311, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.drop_modification", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pyx":329
 * 
 *     @property
 *     def mass(self):             # <<<<<<<<<<<<<<
 *         return self.get_mass()
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_4mass_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_4mass_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_4mass___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_4mass___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__get__", 0);

  /* "glycopeptidepy/_c/structure/base.pyx":330
 *     @property
 *     def mass(self):
 *         return self.get_mass()             # <<<<<<<<<<<<<<
 */
  __Pyx_XDECREF(__pyx_r);
  __pyx_t_1 = PyFloat_FromDouble(((struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self->__pyx_vtab)->get_mass(__pyx_v_self)); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 330, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_r = __pyx_t_1;
  __pyx_t_1 = 0;
  goto __pyx_L0;

  /* "glycopeptidepy/_c/structure/base.pyx":329
 * 
 *     @property
 *     def mass(self):             # <<<<<<<<<<<<<<
 *         return self.get_mass()
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.mass.__get__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":71
 * cdef class SequencePosition(object):
 *     cdef:
 *         public AminoAcidResidueBase amino_acid             # <<<<<<<<<<<<<<
 *         public list modifications
 * 
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(((PyObject *)__pyx_v_self->amino_acid));
  __pyx_r = ((PyObject *)__pyx_v_self->amino_acid);
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(((__pyx_v_value) == Py_None) || likely(__Pyx_TypeTest(__pyx_v_value, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase))))) __PYX_ERR(1, 71, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->amino_acid);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->amino_acid));
  __pyx_v_self->amino_acid = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.amino_acid.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->amino_acid);
  __Pyx_DECREF(((PyObject *)__pyx_v_self->amino_acid));
  __pyx_v_self->amino_acid = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "glycopeptidepy/_c/structure/base.pxd":72
 *     cdef:
 *         public AminoAcidResidueBase amino_acid
 *         public list modifications             # <<<<<<<<<<<<<<
 * 
 *     @staticmethod
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications_1__get__(PyObject *__pyx_v_self); /*proto*/
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications_1__get__(PyObject *__pyx_v_self) {
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications___get__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications___get__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__get__", 0);
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v_self->modifications);
  __pyx_r = __pyx_v_self->modifications;
  goto __pyx_L0;

  /* function exit code */
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications_3__set__(PyObject *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__set__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications_2__set__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self), ((PyObject *)__pyx_v_value));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications_2__set__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self, PyObject *__pyx_v_value) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__set__", 0);
  if (!(likely(PyList_CheckExact(__pyx_v_value))||((__pyx_v_value) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_v_value)->tp_name), 0))) __PYX_ERR(1, 72, __pyx_L1_error)
  __pyx_t_1 = __pyx_v_value;
  __Pyx_INCREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v_self->modifications);
  __Pyx_DECREF(__pyx_v_self->modifications);
  __pyx_v_self->modifications = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* function exit code */
  __pyx_r = 0;
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.SequencePosition.modifications.__set__", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = -1;
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* Python wrapper */
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications_5__del__(PyObject *__pyx_v_self); /*proto*/
static int __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications_5__del__(PyObject *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__ (wrapper)", 0);
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications_4__del__(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)__pyx_v_self));

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static int __pyx_pf_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications_4__del__(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_v_self) {
  int __pyx_r;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__del__", 0);
  __Pyx_INCREF(Py_None);
  __Pyx_GIVEREF(Py_None);
  __Pyx_GOTREF(__pyx_v_self->modifications);
  __Pyx_DECREF(__pyx_v_self->modifications);
  __pyx_v_self->modifications = ((PyObject*)Py_None);

  /* function exit code */
  __pyx_r = 0;
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_PeptideSequenceBase(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_1__pyx_unpickle_PeptideSequenceBase(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base___pyx_unpickle_PeptideSequenceBase[] = "__pyx_unpickle_PeptideSequenceBase(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_14glycopeptidepy_2_c_9structure_4base_1__pyx_unpickle_PeptideSequenceBase = {"__pyx_unpickle_PeptideSequenceBase", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_1__pyx_unpickle_PeptideSequenceBase, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_4base___pyx_unpickle_PeptideSequenceBase};
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_1__pyx_unpickle_PeptideSequenceBase(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_PeptideSequenceBase (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PeptideSequenceBase", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PeptideSequenceBase", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_PeptideSequenceBase") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_PeptideSequenceBase", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.__pyx_unpickle_PeptideSequenceBase", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base___pyx_unpickle_PeptideSequenceBase(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base___pyx_unpickle_PeptideSequenceBase(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PeptideSequenceBase", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x9a27ba6:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9a27ba6 = (_c_term, _glycosylation_manager, _mass, _n_term, _peptide_composition, _total_composition, sequence))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x9a27ba6) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x9a27ba6:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9a27ba6 = (_c_term, _glycosylation_manager, _mass, _n_term, _peptide_composition, _total_composition, sequence))" % __pyx_checksum)
 *     __pyx_result = PeptideSequenceBase.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x9a27ba6:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9a27ba6 = (_c_term, _glycosylation_manager, _mass, _n_term, _peptide_composition, _total_composition, sequence))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = PeptideSequenceBase.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x9a, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x9a27ba6:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9a27ba6 = (_c_term, _glycosylation_manager, _mass, _n_term, _peptide_composition, _total_composition, sequence))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9a27ba6 = (_c_term, _glycosylation_manager, _mass, _n_term, _peptide_composition, _total_composition, sequence))" % __pyx_checksum)
 *     __pyx_result = PeptideSequenceBase.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PeptideSequenceBase__set_state(<PeptideSequenceBase> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9a27ba6 = (_c_term, _glycosylation_manager, _mass, _n_term, _peptide_composition, _total_composition, sequence))" % __pyx_checksum)
 *     __pyx_result = PeptideSequenceBase.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PeptideSequenceBase__set_state(<PeptideSequenceBase> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = PeptideSequenceBase.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PeptideSequenceBase__set_state(<PeptideSequenceBase> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_PeptideSequenceBase__set_state(PeptideSequenceBase __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_14glycopeptidepy_2_c_9structure_4base___pyx_unpickle_PeptideSequenceBase__set_state(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x9a27ba6 = (_c_term, _glycosylation_manager, _mass, _n_term, _peptide_composition, _total_composition, sequence))" % __pyx_checksum)
 *     __pyx_result = PeptideSequenceBase.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_PeptideSequenceBase__set_state(<PeptideSequenceBase> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_PeptideSequenceBase__set_state(<PeptideSequenceBase> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_PeptideSequenceBase__set_state(PeptideSequenceBase __pyx_result, tuple __pyx_state):
 *     __pyx_result._c_term = __pyx_state[0]; __pyx_result._glycosylation_manager = __pyx_state[1]; __pyx_result._mass = __pyx_state[2]; __pyx_result._n_term = __pyx_state[3]; __pyx_result._peptide_composition = __pyx_state[4]; __pyx_result._total_composition = __pyx_state[5]; __pyx_result.sequence = __pyx_state[6]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_PeptideSequenceBase(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.__pyx_unpickle_PeptideSequenceBase", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_PeptideSequenceBase__set_state(<PeptideSequenceBase> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PeptideSequenceBase__set_state(PeptideSequenceBase __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._c_term = __pyx_state[0]; __pyx_result._glycosylation_manager = __pyx_state[1]; __pyx_result._mass = __pyx_state[2]; __pyx_result._n_term = __pyx_state[3]; __pyx_result._peptide_composition = __pyx_state[4]; __pyx_result._total_composition = __pyx_state[5]; __pyx_result.sequence = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_4base___pyx_unpickle_PeptideSequenceBase__set_state(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_PeptideSequenceBase__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_PeptideSequenceBase__set_state(PeptideSequenceBase __pyx_result, tuple __pyx_state):
 *     __pyx_result._c_term = __pyx_state[0]; __pyx_result._glycosylation_manager = __pyx_state[1]; __pyx_result._mass = __pyx_state[2]; __pyx_result._n_term = __pyx_state[3]; __pyx_result._peptide_composition = __pyx_state[4]; __pyx_result._total_composition = __pyx_state[5]; __pyx_result.sequence = __pyx_state[6]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_c_term);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_c_term));
  __pyx_v___pyx_result->_c_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_glycosylation_manager);
  __Pyx_DECREF(__pyx_v___pyx_result->_glycosylation_manager);
  __pyx_v___pyx_result->_glycosylation_manager = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_mass = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_n_term);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_n_term));
  __pyx_v___pyx_result->_n_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_peptide_composition);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_peptide_composition));
  __pyx_v___pyx_result->_peptide_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->_total_composition);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->_total_composition));
  __pyx_v___pyx_result->_total_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(PyList_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "list", Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->sequence);
  __Pyx_DECREF(__pyx_v___pyx_result->sequence);
  __pyx_v___pyx_result->sequence = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_PeptideSequenceBase__set_state(PeptideSequenceBase __pyx_result, tuple __pyx_state):
 *     __pyx_result._c_term = __pyx_state[0]; __pyx_result._glycosylation_manager = __pyx_state[1]; __pyx_result._mass = __pyx_state[2]; __pyx_result._n_term = __pyx_state[3]; __pyx_result._peptide_composition = __pyx_state[4]; __pyx_result._total_composition = __pyx_state[5]; __pyx_result.sequence = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 7) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result._c_term = __pyx_state[0]; __pyx_result._glycosylation_manager = __pyx_state[1]; __pyx_result._mass = __pyx_state[2]; __pyx_result._n_term = __pyx_state[3]; __pyx_result._peptide_composition = __pyx_state[4]; __pyx_result._total_composition = __pyx_state[5]; __pyx_result.sequence = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[7])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 7, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_PeptideSequenceBase__set_state(PeptideSequenceBase __pyx_result, tuple __pyx_state):
 *     __pyx_result._c_term = __pyx_state[0]; __pyx_result._glycosylation_manager = __pyx_state[1]; __pyx_result._mass = __pyx_state[2]; __pyx_result._n_term = __pyx_state[3]; __pyx_result._peptide_composition = __pyx_state[4]; __pyx_result._total_composition = __pyx_state[5]; __pyx_result.sequence = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[7])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_PeptideSequenceBase__set_state(<PeptideSequenceBase> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PeptideSequenceBase__set_state(PeptideSequenceBase __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._c_term = __pyx_state[0]; __pyx_result._glycosylation_manager = __pyx_state[1]; __pyx_result._mass = __pyx_state[2]; __pyx_result._n_term = __pyx_state[3]; __pyx_result._peptide_composition = __pyx_state[4]; __pyx_result._total_composition = __pyx_state[5]; __pyx_result.sequence = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.__pyx_unpickle_PeptideSequenceBase__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_AminoAcidResidueBase(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_3__pyx_unpickle_AminoAcidResidueBase(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_2__pyx_unpickle_AminoAcidResidueBase[] = "__pyx_unpickle_AminoAcidResidueBase(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_14glycopeptidepy_2_c_9structure_4base_3__pyx_unpickle_AminoAcidResidueBase = {"__pyx_unpickle_AminoAcidResidueBase", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_3__pyx_unpickle_AminoAcidResidueBase, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_2__pyx_unpickle_AminoAcidResidueBase};
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_3__pyx_unpickle_AminoAcidResidueBase(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_AminoAcidResidueBase (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_AminoAcidResidueBase", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_AminoAcidResidueBase", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_AminoAcidResidueBase") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_AminoAcidResidueBase", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.__pyx_unpickle_AminoAcidResidueBase", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_2__pyx_unpickle_AminoAcidResidueBase(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_2__pyx_unpickle_AminoAcidResidueBase(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_AminoAcidResidueBase", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x5b0643e:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5b0643e = (_hash, composition, mass, name, neutral_loss, symbol))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x5b0643e) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x5b0643e:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5b0643e = (_hash, composition, mass, name, neutral_loss, symbol))" % __pyx_checksum)
 *     __pyx_result = AminoAcidResidueBase.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x5b0643e:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5b0643e = (_hash, composition, mass, name, neutral_loss, symbol))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = AminoAcidResidueBase.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x5b, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x5b0643e:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5b0643e = (_hash, composition, mass, name, neutral_loss, symbol))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5b0643e = (_hash, composition, mass, name, neutral_loss, symbol))" % __pyx_checksum)
 *     __pyx_result = AminoAcidResidueBase.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_AminoAcidResidueBase__set_state(<AminoAcidResidueBase> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5b0643e = (_hash, composition, mass, name, neutral_loss, symbol))" % __pyx_checksum)
 *     __pyx_result = AminoAcidResidueBase.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_AminoAcidResidueBase__set_state(<AminoAcidResidueBase> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = AminoAcidResidueBase.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_AminoAcidResidueBase__set_state(<AminoAcidResidueBase> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_AminoAcidResidueBase__set_state(AminoAcidResidueBase __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_14glycopeptidepy_2_c_9structure_4base___pyx_unpickle_AminoAcidResidueBase__set_state(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x5b0643e = (_hash, composition, mass, name, neutral_loss, symbol))" % __pyx_checksum)
 *     __pyx_result = AminoAcidResidueBase.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_AminoAcidResidueBase__set_state(<AminoAcidResidueBase> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_AminoAcidResidueBase__set_state(<AminoAcidResidueBase> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_AminoAcidResidueBase__set_state(AminoAcidResidueBase __pyx_result, tuple __pyx_state):
 *     __pyx_result._hash = __pyx_state[0]; __pyx_result.composition = __pyx_state[1]; __pyx_result.mass = __pyx_state[2]; __pyx_result.name = __pyx_state[3]; __pyx_result.neutral_loss = __pyx_state[4]; __pyx_result.symbol = __pyx_state[5]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_AminoAcidResidueBase(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.__pyx_unpickle_AminoAcidResidueBase", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_AminoAcidResidueBase__set_state(<AminoAcidResidueBase> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_AminoAcidResidueBase__set_state(AminoAcidResidueBase __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._hash = __pyx_state[0]; __pyx_result.composition = __pyx_state[1]; __pyx_result.mass = __pyx_state[2]; __pyx_result.name = __pyx_state[3]; __pyx_result.neutral_loss = __pyx_state[4]; __pyx_result.symbol = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_4base___pyx_unpickle_AminoAcidResidueBase__set_state(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  Py_hash_t __pyx_t_2;
  double __pyx_t_3;
  int __pyx_t_4;
  Py_ssize_t __pyx_t_5;
  int __pyx_t_6;
  int __pyx_t_7;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  PyObject *__pyx_t_10 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_AminoAcidResidueBase__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_AminoAcidResidueBase__set_state(AminoAcidResidueBase __pyx_result, tuple __pyx_state):
 *     __pyx_result._hash = __pyx_state[0]; __pyx_result.composition = __pyx_state[1]; __pyx_result.mass = __pyx_state[2]; __pyx_result.name = __pyx_state[3]; __pyx_result.neutral_loss = __pyx_state[4]; __pyx_result.symbol = __pyx_state[5]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __Pyx_PyInt_AsHash_t(__pyx_t_1); if (unlikely((__pyx_t_2 == (Py_hash_t)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->_hash = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->composition);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->composition));
  __pyx_v___pyx_result->composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_3 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_3 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->mass = __pyx_t_3;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(__Pyx_PyBaseString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", (PY_MAJOR_VERSION < 3 ? "basestring" : "str"), Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->name);
  __Pyx_DECREF(__pyx_v___pyx_result->name);
  __pyx_v___pyx_result->name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 4, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->neutral_loss);
  __Pyx_DECREF(__pyx_v___pyx_result->neutral_loss);
  __pyx_v___pyx_result->neutral_loss = __pyx_t_1;
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 5, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(__Pyx_PyBaseString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", (PY_MAJOR_VERSION < 3 ? "basestring" : "str"), Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->symbol);
  __Pyx_DECREF(__pyx_v___pyx_result->symbol);
  __pyx_v___pyx_result->symbol = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_AminoAcidResidueBase__set_state(AminoAcidResidueBase __pyx_result, tuple __pyx_state):
 *     __pyx_result._hash = __pyx_state[0]; __pyx_result.composition = __pyx_state[1]; __pyx_result.mass = __pyx_state[2]; __pyx_result.name = __pyx_state[3]; __pyx_result.neutral_loss = __pyx_state[4]; __pyx_result.symbol = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_5 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_5 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = ((__pyx_t_5 > 6) != 0);
  if (__pyx_t_6) {
  } else {
    __pyx_t_4 = __pyx_t_6;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_6 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_6 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_7 = (__pyx_t_6 != 0);
  __pyx_t_4 = __pyx_t_7;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_4) {

    /* "(tree fragment)":14
 *     __pyx_result._hash = __pyx_state[0]; __pyx_result.composition = __pyx_state[1]; __pyx_result.mass = __pyx_state[2]; __pyx_result.name = __pyx_state[3]; __pyx_result.neutral_loss = __pyx_state[4]; __pyx_result.symbol = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[6])             # <<<<<<<<<<<<<<
 */
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_9 = __Pyx_PyObject_GetAttrStr(__pyx_t_8, __pyx_n_s_update); if (unlikely(!__pyx_t_9)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_9);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_8 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 6, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __pyx_t_10 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_9))) {
      __pyx_t_10 = PyMethod_GET_SELF(__pyx_t_9);
      if (likely(__pyx_t_10)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_9);
        __Pyx_INCREF(__pyx_t_10);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_9, function);
      }
    }
    __pyx_t_1 = (__pyx_t_10) ? __Pyx_PyObject_Call2Args(__pyx_t_9, __pyx_t_10, __pyx_t_8) : __Pyx_PyObject_CallOneArg(__pyx_t_9, __pyx_t_8);
    __Pyx_XDECREF(__pyx_t_10); __pyx_t_10 = 0;
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_AminoAcidResidueBase__set_state(AminoAcidResidueBase __pyx_result, tuple __pyx_state):
 *     __pyx_result._hash = __pyx_state[0]; __pyx_result.composition = __pyx_state[1]; __pyx_result.mass = __pyx_state[2]; __pyx_result.name = __pyx_state[3]; __pyx_result.neutral_loss = __pyx_state[4]; __pyx_result.symbol = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[6])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_AminoAcidResidueBase__set_state(<AminoAcidResidueBase> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_AminoAcidResidueBase__set_state(AminoAcidResidueBase __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._hash = __pyx_state[0]; __pyx_result.composition = __pyx_state[1]; __pyx_result.mass = __pyx_state[2]; __pyx_result.name = __pyx_state[3]; __pyx_result.neutral_loss = __pyx_state[4]; __pyx_result.symbol = __pyx_state[5]
 *     if len(__pyx_state) > 6 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_XDECREF(__pyx_t_10);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.__pyx_unpickle_AminoAcidResidueBase__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":1
 * def __pyx_unpickle_ModificationBase(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

/* Python wrapper */
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_5__pyx_unpickle_ModificationBase(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds); /*proto*/
static char __pyx_doc_14glycopeptidepy_2_c_9structure_4base_4__pyx_unpickle_ModificationBase[] = "__pyx_unpickle_ModificationBase(__pyx_type, long __pyx_checksum, __pyx_state)";
static PyMethodDef __pyx_mdef_14glycopeptidepy_2_c_9structure_4base_5__pyx_unpickle_ModificationBase = {"__pyx_unpickle_ModificationBase", (PyCFunction)(void*)(PyCFunctionWithKeywords)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_5__pyx_unpickle_ModificationBase, METH_VARARGS|METH_KEYWORDS, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_4__pyx_unpickle_ModificationBase};
static PyObject *__pyx_pw_14glycopeptidepy_2_c_9structure_4base_5__pyx_unpickle_ModificationBase(PyObject *__pyx_self, PyObject *__pyx_args, PyObject *__pyx_kwds) {
  PyObject *__pyx_v___pyx_type = 0;
  long __pyx_v___pyx_checksum;
  PyObject *__pyx_v___pyx_state = 0;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  PyObject *__pyx_r = 0;
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__pyx_unpickle_ModificationBase (wrapper)", 0);
  {
    static PyObject **__pyx_pyargnames[] = {&__pyx_n_s_pyx_type,&__pyx_n_s_pyx_checksum,&__pyx_n_s_pyx_state,0};
    PyObject* values[3] = {0,0,0};
    if (unlikely(__pyx_kwds)) {
      Py_ssize_t kw_args;
      const Py_ssize_t pos_args = PyTuple_GET_SIZE(__pyx_args);
      switch (pos_args) {
        case  3: values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
        CYTHON_FALLTHROUGH;
        case  2: values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
        CYTHON_FALLTHROUGH;
        case  1: values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
        CYTHON_FALLTHROUGH;
        case  0: break;
        default: goto __pyx_L5_argtuple_error;
      }
      kw_args = PyDict_Size(__pyx_kwds);
      switch (pos_args) {
        case  0:
        if (likely((values[0] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_type)) != 0)) kw_args--;
        else goto __pyx_L5_argtuple_error;
        CYTHON_FALLTHROUGH;
        case  1:
        if (likely((values[1] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_checksum)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ModificationBase", 1, 3, 3, 1); __PYX_ERR(2, 1, __pyx_L3_error)
        }
        CYTHON_FALLTHROUGH;
        case  2:
        if (likely((values[2] = __Pyx_PyDict_GetItemStr(__pyx_kwds, __pyx_n_s_pyx_state)) != 0)) kw_args--;
        else {
          __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ModificationBase", 1, 3, 3, 2); __PYX_ERR(2, 1, __pyx_L3_error)
        }
      }
      if (unlikely(kw_args > 0)) {
        if (unlikely(__Pyx_ParseOptionalKeywords(__pyx_kwds, __pyx_pyargnames, 0, values, pos_args, "__pyx_unpickle_ModificationBase") < 0)) __PYX_ERR(2, 1, __pyx_L3_error)
      }
    } else if (PyTuple_GET_SIZE(__pyx_args) != 3) {
      goto __pyx_L5_argtuple_error;
    } else {
      values[0] = PyTuple_GET_ITEM(__pyx_args, 0);
      values[1] = PyTuple_GET_ITEM(__pyx_args, 1);
      values[2] = PyTuple_GET_ITEM(__pyx_args, 2);
    }
    __pyx_v___pyx_type = values[0];
    __pyx_v___pyx_checksum = __Pyx_PyInt_As_long(values[1]); if (unlikely((__pyx_v___pyx_checksum == (long)-1) && PyErr_Occurred())) __PYX_ERR(2, 1, __pyx_L3_error)
    __pyx_v___pyx_state = values[2];
  }
  goto __pyx_L4_argument_unpacking_done;
  __pyx_L5_argtuple_error:;
  __Pyx_RaiseArgtupleInvalid("__pyx_unpickle_ModificationBase", 1, 3, 3, PyTuple_GET_SIZE(__pyx_args)); __PYX_ERR(2, 1, __pyx_L3_error)
  __pyx_L3_error:;
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.__pyx_unpickle_ModificationBase", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __Pyx_RefNannyFinishContext();
  return NULL;
  __pyx_L4_argument_unpacking_done:;
  __pyx_r = __pyx_pf_14glycopeptidepy_2_c_9structure_4base_4__pyx_unpickle_ModificationBase(__pyx_self, __pyx_v___pyx_type, __pyx_v___pyx_checksum, __pyx_v___pyx_state);

  /* function exit code */
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

static PyObject *__pyx_pf_14glycopeptidepy_2_c_9structure_4base_4__pyx_unpickle_ModificationBase(CYTHON_UNUSED PyObject *__pyx_self, PyObject *__pyx_v___pyx_type, long __pyx_v___pyx_checksum, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_v___pyx_PickleError = 0;
  PyObject *__pyx_v___pyx_result = 0;
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  int __pyx_t_1;
  PyObject *__pyx_t_2 = NULL;
  PyObject *__pyx_t_3 = NULL;
  PyObject *__pyx_t_4 = NULL;
  PyObject *__pyx_t_5 = NULL;
  int __pyx_t_6;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ModificationBase", 0);

  /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x54d98d7:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x54d98d7 = (composition, mass, name))" % __pyx_checksum)
 */
  __pyx_t_1 = ((__pyx_v___pyx_checksum != 0x54d98d7) != 0);
  if (__pyx_t_1) {

    /* "(tree fragment)":5
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x54d98d7:
 *         from pickle import PickleError as __pyx_PickleError             # <<<<<<<<<<<<<<
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x54d98d7 = (composition, mass, name))" % __pyx_checksum)
 *     __pyx_result = ModificationBase.__new__(__pyx_type)
 */
    __pyx_t_2 = PyList_New(1); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_n_s_PickleError);
    __Pyx_GIVEREF(__pyx_n_s_PickleError);
    PyList_SET_ITEM(__pyx_t_2, 0, __pyx_n_s_PickleError);
    __pyx_t_3 = __Pyx_Import(__pyx_n_s_pickle, __pyx_t_2, -1); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __pyx_t_2 = __Pyx_ImportFrom(__pyx_t_3, __pyx_n_s_PickleError); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 5, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __Pyx_INCREF(__pyx_t_2);
    __pyx_v___pyx_PickleError = __pyx_t_2;
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":6
 *     if __pyx_checksum != 0x54d98d7:
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x54d98d7 = (composition, mass, name))" % __pyx_checksum)             # <<<<<<<<<<<<<<
 *     __pyx_result = ModificationBase.__new__(__pyx_type)
 *     if __pyx_state is not None:
 */
    __pyx_t_2 = __Pyx_PyInt_From_long(__pyx_v___pyx_checksum); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_2);
    __pyx_t_4 = __Pyx_PyString_Format(__pyx_kp_s_Incompatible_checksums_s_vs_0x54, __pyx_t_2); if (unlikely(!__pyx_t_4)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_4);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_INCREF(__pyx_v___pyx_PickleError);
    __pyx_t_2 = __pyx_v___pyx_PickleError; __pyx_t_5 = NULL;
    if (CYTHON_UNPACK_METHODS && unlikely(PyMethod_Check(__pyx_t_2))) {
      __pyx_t_5 = PyMethod_GET_SELF(__pyx_t_2);
      if (likely(__pyx_t_5)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
        __Pyx_INCREF(__pyx_t_5);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_2, function);
      }
    }
    __pyx_t_3 = (__pyx_t_5) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_5, __pyx_t_4) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_t_4);
    __Pyx_XDECREF(__pyx_t_5); __pyx_t_5 = 0;
    __Pyx_DECREF(__pyx_t_4); __pyx_t_4 = 0;
    if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 6, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
    __Pyx_Raise(__pyx_t_3, 0, 0, 0);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;
    __PYX_ERR(2, 6, __pyx_L1_error)

    /* "(tree fragment)":4
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 *     if __pyx_checksum != 0x54d98d7:             # <<<<<<<<<<<<<<
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x54d98d7 = (composition, mass, name))" % __pyx_checksum)
 */
  }

  /* "(tree fragment)":7
 *         from pickle import PickleError as __pyx_PickleError
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x54d98d7 = (composition, mass, name))" % __pyx_checksum)
 *     __pyx_result = ModificationBase.__new__(__pyx_type)             # <<<<<<<<<<<<<<
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ModificationBase__set_state(<ModificationBase> __pyx_result, __pyx_state)
 */
  __pyx_t_2 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase), __pyx_n_s_new); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __pyx_t_4 = NULL;
  if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_2))) {
    __pyx_t_4 = PyMethod_GET_SELF(__pyx_t_2);
    if (likely(__pyx_t_4)) {
      PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_2);
      __Pyx_INCREF(__pyx_t_4);
      __Pyx_INCREF(function);
      __Pyx_DECREF_SET(__pyx_t_2, function);
    }
  }
  __pyx_t_3 = (__pyx_t_4) ? __Pyx_PyObject_Call2Args(__pyx_t_2, __pyx_t_4, __pyx_v___pyx_type) : __Pyx_PyObject_CallOneArg(__pyx_t_2, __pyx_v___pyx_type);
  __Pyx_XDECREF(__pyx_t_4); __pyx_t_4 = 0;
  if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 7, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_3);
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;
  __pyx_v___pyx_result = __pyx_t_3;
  __pyx_t_3 = 0;

  /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x54d98d7 = (composition, mass, name))" % __pyx_checksum)
 *     __pyx_result = ModificationBase.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ModificationBase__set_state(<ModificationBase> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  __pyx_t_1 = (__pyx_v___pyx_state != Py_None);
  __pyx_t_6 = (__pyx_t_1 != 0);
  if (__pyx_t_6) {

    /* "(tree fragment)":9
 *     __pyx_result = ModificationBase.__new__(__pyx_type)
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ModificationBase__set_state(<ModificationBase> __pyx_result, __pyx_state)             # <<<<<<<<<<<<<<
 *     return __pyx_result
 * cdef __pyx_unpickle_ModificationBase__set_state(ModificationBase __pyx_result, tuple __pyx_state):
 */
    if (!(likely(PyTuple_CheckExact(__pyx_v___pyx_state))||((__pyx_v___pyx_state) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", "tuple", Py_TYPE(__pyx_v___pyx_state)->tp_name), 0))) __PYX_ERR(2, 9, __pyx_L1_error)
    __pyx_t_3 = __pyx_f_14glycopeptidepy_2_c_9structure_4base___pyx_unpickle_ModificationBase__set_state(((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)__pyx_v___pyx_result), ((PyObject*)__pyx_v___pyx_state)); if (unlikely(!__pyx_t_3)) __PYX_ERR(2, 9, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_3);
    __Pyx_DECREF(__pyx_t_3); __pyx_t_3 = 0;

    /* "(tree fragment)":8
 *         raise __pyx_PickleError("Incompatible checksums (%s vs 0x54d98d7 = (composition, mass, name))" % __pyx_checksum)
 *     __pyx_result = ModificationBase.__new__(__pyx_type)
 *     if __pyx_state is not None:             # <<<<<<<<<<<<<<
 *         __pyx_unpickle_ModificationBase__set_state(<ModificationBase> __pyx_result, __pyx_state)
 *     return __pyx_result
 */
  }

  /* "(tree fragment)":10
 *     if __pyx_state is not None:
 *         __pyx_unpickle_ModificationBase__set_state(<ModificationBase> __pyx_result, __pyx_state)
 *     return __pyx_result             # <<<<<<<<<<<<<<
 * cdef __pyx_unpickle_ModificationBase__set_state(ModificationBase __pyx_result, tuple __pyx_state):
 *     __pyx_result.composition = __pyx_state[0]; __pyx_result.mass = __pyx_state[1]; __pyx_result.name = __pyx_state[2]
 */
  __Pyx_XDECREF(__pyx_r);
  __Pyx_INCREF(__pyx_v___pyx_result);
  __pyx_r = __pyx_v___pyx_result;
  goto __pyx_L0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ModificationBase(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */

  /* function exit code */
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_2);
  __Pyx_XDECREF(__pyx_t_3);
  __Pyx_XDECREF(__pyx_t_4);
  __Pyx_XDECREF(__pyx_t_5);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.__pyx_unpickle_ModificationBase", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = NULL;
  __pyx_L0:;
  __Pyx_XDECREF(__pyx_v___pyx_PickleError);
  __Pyx_XDECREF(__pyx_v___pyx_result);
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}

/* "(tree fragment)":11
 *         __pyx_unpickle_ModificationBase__set_state(<ModificationBase> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ModificationBase__set_state(ModificationBase __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.composition = __pyx_state[0]; __pyx_result.mass = __pyx_state[1]; __pyx_result.name = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 */

static PyObject *__pyx_f_14glycopeptidepy_2_c_9structure_4base___pyx_unpickle_ModificationBase__set_state(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_v___pyx_result, PyObject *__pyx_v___pyx_state) {
  PyObject *__pyx_r = NULL;
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  double __pyx_t_2;
  int __pyx_t_3;
  Py_ssize_t __pyx_t_4;
  int __pyx_t_5;
  int __pyx_t_6;
  PyObject *__pyx_t_7 = NULL;
  PyObject *__pyx_t_8 = NULL;
  PyObject *__pyx_t_9 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__pyx_unpickle_ModificationBase__set_state", 0);

  /* "(tree fragment)":12
 *     return __pyx_result
 * cdef __pyx_unpickle_ModificationBase__set_state(ModificationBase __pyx_result, tuple __pyx_state):
 *     __pyx_result.composition = __pyx_state[0]; __pyx_result.mass = __pyx_state[1]; __pyx_result.name = __pyx_state[2]             # <<<<<<<<<<<<<<
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 0, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(((__pyx_t_1) == Py_None) || likely(__Pyx_TypeTest(__pyx_t_1, __pyx_ptype_5glypy_11composition_12ccomposition_CComposition))))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->composition);
  __Pyx_DECREF(((PyObject *)__pyx_v___pyx_result->composition));
  __pyx_v___pyx_result->composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)__pyx_t_1);
  __pyx_t_1 = 0;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 1, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_t_2 = __pyx_PyFloat_AsDouble(__pyx_t_1); if (unlikely((__pyx_t_2 == (double)-1) && PyErr_Occurred())) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_v___pyx_result->mass = __pyx_t_2;
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
    __PYX_ERR(2, 12, __pyx_L1_error)
  }
  __pyx_t_1 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 2, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (!(likely(__Pyx_PyBaseString_CheckExact(__pyx_t_1))||((__pyx_t_1) == Py_None)||(PyErr_Format(PyExc_TypeError, "Expected %.16s, got %.200s", (PY_MAJOR_VERSION < 3 ? "basestring" : "str"), Py_TYPE(__pyx_t_1)->tp_name), 0))) __PYX_ERR(2, 12, __pyx_L1_error)
  __Pyx_GIVEREF(__pyx_t_1);
  __Pyx_GOTREF(__pyx_v___pyx_result->name);
  __Pyx_DECREF(__pyx_v___pyx_result->name);
  __pyx_v___pyx_result->name = ((PyObject*)__pyx_t_1);
  __pyx_t_1 = 0;

  /* "(tree fragment)":13
 * cdef __pyx_unpickle_ModificationBase__set_state(ModificationBase __pyx_result, tuple __pyx_state):
 *     __pyx_result.composition = __pyx_state[0]; __pyx_result.mass = __pyx_state[1]; __pyx_result.name = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  if (unlikely(__pyx_v___pyx_state == Py_None)) {
    PyErr_SetString(PyExc_TypeError, "object of type 'NoneType' has no len()");
    __PYX_ERR(2, 13, __pyx_L1_error)
  }
  __pyx_t_4 = PyTuple_GET_SIZE(__pyx_v___pyx_state); if (unlikely(__pyx_t_4 == ((Py_ssize_t)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_5 = ((__pyx_t_4 > 3) != 0);
  if (__pyx_t_5) {
  } else {
    __pyx_t_3 = __pyx_t_5;
    goto __pyx_L4_bool_binop_done;
  }
  __pyx_t_5 = __Pyx_HasAttr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(__pyx_t_5 == ((int)-1))) __PYX_ERR(2, 13, __pyx_L1_error)
  __pyx_t_6 = (__pyx_t_5 != 0);
  __pyx_t_3 = __pyx_t_6;
  __pyx_L4_bool_binop_done:;
  if (__pyx_t_3) {

    /* "(tree fragment)":14
 *     __pyx_result.composition = __pyx_state[0]; __pyx_result.mass = __pyx_state[1]; __pyx_result.name = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 *         __pyx_result.__dict__.update(__pyx_state[3])             # <<<<<<<<<<<<<<
 */
    __pyx_t_7 = __Pyx_PyObject_GetAttrStr(((PyObject *)__pyx_v___pyx_result), __pyx_n_s_dict); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_8 = __Pyx_PyObject_GetAttrStr(__pyx_t_7, __pyx_n_s_update); if (unlikely(!__pyx_t_8)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_8);
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(__pyx_v___pyx_state == Py_None)) {
      PyErr_SetString(PyExc_TypeError, "'NoneType' object is not subscriptable");
      __PYX_ERR(2, 14, __pyx_L1_error)
    }
    __pyx_t_7 = __Pyx_GetItemInt_Tuple(__pyx_v___pyx_state, 3, long, 1, __Pyx_PyInt_From_long, 0, 0, 1); if (unlikely(!__pyx_t_7)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_7);
    __pyx_t_9 = NULL;
    if (CYTHON_UNPACK_METHODS && likely(PyMethod_Check(__pyx_t_8))) {
      __pyx_t_9 = PyMethod_GET_SELF(__pyx_t_8);
      if (likely(__pyx_t_9)) {
        PyObject* function = PyMethod_GET_FUNCTION(__pyx_t_8);
        __Pyx_INCREF(__pyx_t_9);
        __Pyx_INCREF(function);
        __Pyx_DECREF_SET(__pyx_t_8, function);
      }
    }
    __pyx_t_1 = (__pyx_t_9) ? __Pyx_PyObject_Call2Args(__pyx_t_8, __pyx_t_9, __pyx_t_7) : __Pyx_PyObject_CallOneArg(__pyx_t_8, __pyx_t_7);
    __Pyx_XDECREF(__pyx_t_9); __pyx_t_9 = 0;
    __Pyx_DECREF(__pyx_t_7); __pyx_t_7 = 0;
    if (unlikely(!__pyx_t_1)) __PYX_ERR(2, 14, __pyx_L1_error)
    __Pyx_GOTREF(__pyx_t_1);
    __Pyx_DECREF(__pyx_t_8); __pyx_t_8 = 0;
    __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;

    /* "(tree fragment)":13
 * cdef __pyx_unpickle_ModificationBase__set_state(ModificationBase __pyx_result, tuple __pyx_state):
 *     __pyx_result.composition = __pyx_state[0]; __pyx_result.mass = __pyx_state[1]; __pyx_result.name = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):             # <<<<<<<<<<<<<<
 *         __pyx_result.__dict__.update(__pyx_state[3])
 */
  }

  /* "(tree fragment)":11
 *         __pyx_unpickle_ModificationBase__set_state(<ModificationBase> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_ModificationBase__set_state(ModificationBase __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result.composition = __pyx_state[0]; __pyx_result.mass = __pyx_state[1]; __pyx_result.name = __pyx_state[2]
 *     if len(__pyx_state) > 3 and hasattr(__pyx_result, '__dict__'):
 */

  /* function exit code */
  __pyx_r = Py_None; __Pyx_INCREF(Py_None);
  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_7);
  __Pyx_XDECREF(__pyx_t_8);
  __Pyx_XDECREF(__pyx_t_9);
  __Pyx_AddTraceback("glycopeptidepy._c.structure.base.__pyx_unpickle_ModificationBase__set_state", __pyx_clineno, __pyx_lineno, __pyx_filename);
  __pyx_r = 0;
  __pyx_L0:;
  __Pyx_XGIVEREF(__pyx_r);
  __Pyx_RefNannyFinishContext();
  return __pyx_r;
}
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase;

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *__pyx_freelist_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase[10000];
static int __pyx_freecount_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase = 0;

static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *p;
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase)) & ((t->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    o = (PyObject*)__pyx_freelist_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase[--__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase];
    memset(o, 0, sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
      o = (*t->tp_alloc)(t, 0);
    } else {
      o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
    }
    if (unlikely(!o)) return 0;
  }
  p = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)o);
  p->__pyx_vtab = __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase;
  p->sequence = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->_glycosylation_manager = Py_None; Py_INCREF(Py_None);
  p->_n_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)Py_None); Py_INCREF(Py_None);
  p->_c_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)Py_None); Py_INCREF(Py_None);
  p->_total_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  p->_peptide_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase(PyObject *o) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->sequence);
  Py_CLEAR(p->_glycosylation_manager);
  Py_CLEAR(p->_n_term);
  Py_CLEAR(p->_c_term);
  Py_CLEAR(p->_total_composition);
  Py_CLEAR(p->_peptide_composition);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase < 10000) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase)) & ((Py_TYPE(o)->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    __pyx_freelist_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase[__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase++] = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)o;
  if (p->sequence) {
    e = (*v)(p->sequence, a); if (e) return e;
  }
  if (p->_glycosylation_manager) {
    e = (*v)(p->_glycosylation_manager, a); if (e) return e;
  }
  if (p->_n_term) {
    e = (*v)(((PyObject *)p->_n_term), a); if (e) return e;
  }
  if (p->_c_term) {
    e = (*v)(((PyObject *)p->_c_term), a); if (e) return e;
  }
  if (p->_total_composition) {
    e = (*v)(((PyObject *)p->_total_composition), a); if (e) return e;
  }
  if (p->_peptide_composition) {
    e = (*v)(((PyObject *)p->_peptide_composition), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *)o;
  tmp = ((PyObject*)p->sequence);
  p->sequence = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_glycosylation_manager);
  p->_glycosylation_manager = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_n_term);
  p->_n_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_c_term);
  p->_c_term = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_total_composition);
  p->_total_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_peptide_composition);
  p->_peptide_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_sequence(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_sequence(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_8sequence_5__del__(o);
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__mass(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_5_mass_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__mass(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_5_mass_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__glycosylation_manager(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__glycosylation_manager(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_22_glycosylation_manager_5__del__(o);
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__n_term(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__n_term(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_n_term_5__del__(o);
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__c_term(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__c_term(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_7_c_term_5__del__(o);
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__total_composition(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__total_composition(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_18_total_composition_5__del__(o);
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__peptide_composition(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__peptide_composition(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_20_peptide_composition_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase[] = {
  {"_invalidate", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_1_invalidate, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__invalidate},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_3__reduce_cython__, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_2__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_5__setstate_cython__, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_4__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase[] = {
  {(char *)"sequence", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_sequence, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_sequence, (char *)"sequence: list", 0},
  {(char *)"_mass", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__mass, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__mass, (char *)"_mass: 'double'", 0},
  {(char *)"_glycosylation_manager", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__glycosylation_manager, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__glycosylation_manager, (char *)"_glycosylation_manager: object", 0},
  {(char *)"_n_term", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__n_term, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__n_term, (char *)"_n_term: glycopeptidepy._c.structure.base.TerminalGroup", 0},
  {(char *)"_c_term", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__c_term, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__c_term, (char *)"_c_term: glycopeptidepy._c.structure.base.TerminalGroup", 0},
  {(char *)"_total_composition", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__total_composition, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__total_composition, (char *)"_total_composition: glypy.composition.ccomposition.CComposition", 0},
  {(char *)"_peptide_composition", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__peptide_composition, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__peptide_composition, (char *)"_peptide_composition: glypy.composition.ccomposition.CComposition", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase = {
  PyVarObject_HEAD_INIT(0, 0)
  "glycopeptidepy._c.structure.base.PeptideSequenceBase", /*tp_name*/
  sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase, /*tp_traverse*/
  __pyx_tp_clear_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_TerminalGroup __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_TerminalGroup;

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *__pyx_freelist_14glycopeptidepy_2_c_9structure_4base_TerminalGroup[100];
static int __pyx_freecount_14glycopeptidepy_2_c_9structure_4base_TerminalGroup = 0;

static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_4base_TerminalGroup(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *p;
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_TerminalGroup > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup)) & ((t->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    o = (PyObject*)__pyx_freelist_14glycopeptidepy_2_c_9structure_4base_TerminalGroup[--__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_TerminalGroup];
    memset(o, 0, sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
      o = (*t->tp_alloc)(t, 0);
    } else {
      o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
    }
    if (unlikely(!o)) return 0;
  }
  p = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)o);
  p->__pyx_vtab = __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_TerminalGroup;
  p->base_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  p->_modification = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_4base_TerminalGroup(PyObject *o) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->base_composition);
  Py_CLEAR(p->_modification);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_TerminalGroup < 100) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup)) & ((Py_TYPE(o)->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    __pyx_freelist_14glycopeptidepy_2_c_9structure_4base_TerminalGroup[__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_TerminalGroup++] = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_4base_TerminalGroup(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)o;
  if (p->base_composition) {
    e = (*v)(((PyObject *)p->base_composition), a); if (e) return e;
  }
  if (p->_modification) {
    e = (*v)(((PyObject *)p->_modification), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_14glycopeptidepy_2_c_9structure_4base_TerminalGroup(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *)o;
  tmp = ((PyObject*)p->base_composition);
  p->base_composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->_modification);
  p->_modification = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_tp_richcompare_14glycopeptidepy_2_c_9structure_4base_TerminalGroup(PyObject *o1, PyObject *o2, int op) {
  switch (op) {
    case Py_EQ: {
      return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_15__eq__(o1, o2);
    }
    case Py_NE: {
      return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_17__ne__(o1, o2);
    }
    default: {
      return __Pyx_NewRef(Py_NotImplemented);
    }
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_modification(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_12modification_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_modification(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_12modification_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_composition(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_11composition_1__get__(o);
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_base_composition(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_base_composition(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_16base_composition_5__del__(o);
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_mass(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_4mass_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_mass(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_4mass_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup__modification(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup__modification(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13_modification_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_14glycopeptidepy_2_c_9structure_4base_TerminalGroup[] = {
  {"_calculate_mass", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_3_calculate_mass, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_2_calculate_mass},
  {"clone", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_5clone, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_4clone},
  {"__reduce__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_7__reduce__, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_6__reduce__},
  {"modify", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_9modify, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_8modify},
  {"serialize", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_21serialize, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_20serialize},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_14glycopeptidepy_2_c_9structure_4base_TerminalGroup[] = {
  {(char *)"modification", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_modification, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_modification, (char *)0, 0},
  {(char *)"composition", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_composition, 0, (char *)0, 0},
  {(char *)"base_composition", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_base_composition, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_base_composition, (char *)"base_composition: glypy.composition.ccomposition.CComposition", 0},
  {(char *)"mass", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_mass, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_mass, (char *)"mass: 'double'", 0},
  {(char *)"_modification", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup__modification, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup__modification, (char *)"_modification: glycopeptidepy._c.structure.base.ModificationBase", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_14glycopeptidepy_2_c_9structure_4base_TerminalGroup = {
  PyVarObject_HEAD_INIT(0, 0)
  "glycopeptidepy._c.structure.base.TerminalGroup", /*tp_name*/
  sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_4base_TerminalGroup, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_11__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_19__hash__, /*tp_hash*/
  0, /*tp_call*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_13__str__, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "TerminalGroup(base_composition, modification=None)", /*tp_doc*/
  __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_4base_TerminalGroup, /*tp_traverse*/
  __pyx_tp_clear_14glycopeptidepy_2_c_9structure_4base_TerminalGroup, /*tp_clear*/
  __pyx_tp_richcompare_14glycopeptidepy_2_c_9structure_4base_TerminalGroup, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_14glycopeptidepy_2_c_9structure_4base_TerminalGroup, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_14glycopeptidepy_2_c_9structure_4base_TerminalGroup, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_14glycopeptidepy_2_c_9structure_4base_TerminalGroup, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase;

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *__pyx_freelist_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase[100];
static int __pyx_freecount_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase = 0;

static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *p;
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase)) & ((t->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    o = (PyObject*)__pyx_freelist_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase[--__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase];
    memset(o, 0, sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
      o = (*t->tp_alloc)(t, 0);
    } else {
      o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
    }
    if (unlikely(!o)) return 0;
  }
  p = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)o);
  p->__pyx_vtab = __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase;
  p->name = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->symbol = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  p->neutral_loss = Py_None; Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase(PyObject *o) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->name);
  Py_CLEAR(p->symbol);
  Py_CLEAR(p->composition);
  Py_CLEAR(p->neutral_loss);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase < 100) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase)) & ((Py_TYPE(o)->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    __pyx_freelist_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase[__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase++] = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)o;
  if (p->composition) {
    e = (*v)(((PyObject *)p->composition), a); if (e) return e;
  }
  if (p->neutral_loss) {
    e = (*v)(p->neutral_loss, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)o;
  tmp = ((PyObject*)p->composition);
  p->composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->neutral_loss);
  p->neutral_loss = Py_None; Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_tp_richcompare_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase(PyObject *o1, PyObject *o2, int op) {
  switch (op) {
    case Py_EQ: {
      return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_5__eq__(o1, o2);
    }
    case Py_NE: {
      return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_7__ne__(o1, o2);
    }
    default: {
      return __Pyx_NewRef(Py_NotImplemented);
    }
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_name(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4name_5__del__(o);
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_symbol(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_symbol(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_6symbol_5__del__(o);
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_mass(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4mass_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_mass(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_4mass_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_composition(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_composition(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11composition_5__del__(o);
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_neutral_loss(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_neutral_loss(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_12neutral_loss_5__del__(o);
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase__hash(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_5_hash_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase__hash(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_5_hash_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyMethodDef __pyx_methods_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase[] = {
  {"__copy__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_1__copy__, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase___copy__},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_9__reduce_cython__, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_11__setstate_cython__, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase[] = {
  {(char *)"name", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_name, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_name, (char *)"name: basestring", 0},
  {(char *)"symbol", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_symbol, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_symbol, (char *)"symbol: basestring", 0},
  {(char *)"mass", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_mass, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_mass, (char *)"mass: 'double'", 0},
  {(char *)"composition", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_composition, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_composition, (char *)"composition: glypy.composition.ccomposition.CComposition", 0},
  {(char *)"neutral_loss", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_neutral_loss, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_neutral_loss, (char *)"neutral_loss: object", 0},
  {(char *)"_hash", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase__hash, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase__hash, (char *)"_hash: 'Py_hash_t'", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase = {
  PyVarObject_HEAD_INIT(0, 0)
  "glycopeptidepy._c.structure.base.AminoAcidResidueBase", /*tp_name*/
  sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_3__hash__, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "\n    A base type for classes describing amino acid residues\n    ", /*tp_doc*/
  __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase, /*tp_traverse*/
  __pyx_tp_clear_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase, /*tp_clear*/
  __pyx_tp_richcompare_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_ModificationBase __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_ModificationBase;

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *__pyx_freelist_14glycopeptidepy_2_c_9structure_4base_ModificationBase[100000];
static int __pyx_freecount_14glycopeptidepy_2_c_9structure_4base_ModificationBase = 0;

static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_4base_ModificationBase(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *p;
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_ModificationBase > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase)) & ((t->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    o = (PyObject*)__pyx_freelist_14glycopeptidepy_2_c_9structure_4base_ModificationBase[--__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_ModificationBase];
    memset(o, 0, sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
      o = (*t->tp_alloc)(t, 0);
    } else {
      o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
    }
    if (unlikely(!o)) return 0;
  }
  p = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)o);
  p->__pyx_vtab = __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_ModificationBase;
  p->name = ((PyObject*)Py_None); Py_INCREF(Py_None);
  p->composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_4base_ModificationBase(PyObject *o) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->name);
  Py_CLEAR(p->composition);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_ModificationBase < 100000) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase)) & ((Py_TYPE(o)->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    __pyx_freelist_14glycopeptidepy_2_c_9structure_4base_ModificationBase[__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_ModificationBase++] = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_4base_ModificationBase(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)o;
  if (p->composition) {
    e = (*v)(((PyObject *)p->composition), a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_14glycopeptidepy_2_c_9structure_4base_ModificationBase(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *)o;
  tmp = ((PyObject*)p->composition);
  p->composition = ((struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_name(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_name(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4name_5__del__(o);
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_mass(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4mass_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_mass(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4mass_3__set__(o, v);
  }
  else {
    PyErr_SetString(PyExc_NotImplementedError, "__del__");
    return -1;
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_composition(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_composition(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11composition_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_14glycopeptidepy_2_c_9structure_4base_ModificationBase[] = {
  {"__copy__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_1__copy__, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16ModificationBase___copy__},
  {"is_a", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_3is_a, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_2is_a},
  {"serialize", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_5serialize, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_4serialize},
  {"is_tracked_for", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_7is_tracked_for, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_6is_tracked_for},
  {"__reduce_cython__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_9__reduce_cython__, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_8__reduce_cython__},
  {"__setstate_cython__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_11__setstate_cython__, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_10__setstate_cython__},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_14glycopeptidepy_2_c_9structure_4base_ModificationBase[] = {
  {(char *)"name", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_name, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_name, (char *)"name: basestring", 0},
  {(char *)"mass", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_mass, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_mass, (char *)"mass: 'double'", 0},
  {(char *)"composition", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_composition, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_composition, (char *)"composition: glypy.composition.ccomposition.CComposition", 0},
  {0, 0, 0, 0, 0}
};

static PyTypeObject __pyx_type_14glycopeptidepy_2_c_9structure_4base_ModificationBase = {
  PyVarObject_HEAD_INIT(0, 0)
  "glycopeptidepy._c.structure.base.ModificationBase", /*tp_name*/
  sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_4base_ModificationBase, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_4base_ModificationBase, /*tp_traverse*/
  __pyx_tp_clear_14glycopeptidepy_2_c_9structure_4base_ModificationBase, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_14glycopeptidepy_2_c_9structure_4base_ModificationBase, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_14glycopeptidepy_2_c_9structure_4base_ModificationBase, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_14glycopeptidepy_2_c_9structure_4base_ModificationBase, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};
static struct __pyx_vtabstruct_14glycopeptidepy_2_c_9structure_4base_SequencePosition __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_SequencePosition;

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *__pyx_freelist_14glycopeptidepy_2_c_9structure_4base_SequencePosition[10000000];
static int __pyx_freecount_14glycopeptidepy_2_c_9structure_4base_SequencePosition = 0;

static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_4base_SequencePosition(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *p;
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_SequencePosition > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition)) & ((t->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    o = (PyObject*)__pyx_freelist_14glycopeptidepy_2_c_9structure_4base_SequencePosition[--__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_SequencePosition];
    memset(o, 0, sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    if (likely((t->tp_flags & Py_TPFLAGS_IS_ABSTRACT) == 0)) {
      o = (*t->tp_alloc)(t, 0);
    } else {
      o = (PyObject *) PyBaseObject_Type.tp_new(t, __pyx_empty_tuple, 0);
    }
    if (unlikely(!o)) return 0;
  }
  p = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)o);
  p->__pyx_vtab = __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_SequencePosition;
  p->amino_acid = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)Py_None); Py_INCREF(Py_None);
  p->modifications = ((PyObject*)Py_None); Py_INCREF(Py_None);
  return o;
}

static void __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_4base_SequencePosition(PyObject *o) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)o;
  #if CYTHON_USE_TP_FINALIZE
  if (unlikely(PyType_HasFeature(Py_TYPE(o), Py_TPFLAGS_HAVE_FINALIZE) && Py_TYPE(o)->tp_finalize) && !_PyGC_FINALIZED(o)) {
    if (PyObject_CallFinalizerFromDealloc(o)) return;
  }
  #endif
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->amino_acid);
  Py_CLEAR(p->modifications);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_SequencePosition < 10000000) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition)) & ((Py_TYPE(o)->tp_flags & (Py_TPFLAGS_IS_ABSTRACT | Py_TPFLAGS_HEAPTYPE)) == 0))) {
    __pyx_freelist_14glycopeptidepy_2_c_9structure_4base_SequencePosition[__pyx_freecount_14glycopeptidepy_2_c_9structure_4base_SequencePosition++] = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_4base_SequencePosition(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)o;
  if (p->amino_acid) {
    e = (*v)(((PyObject *)p->amino_acid), a); if (e) return e;
  }
  if (p->modifications) {
    e = (*v)(p->modifications, a); if (e) return e;
  }
  return 0;
}

static int __pyx_tp_clear_14glycopeptidepy_2_c_9structure_4base_SequencePosition(PyObject *o) {
  PyObject* tmp;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *)o;
  tmp = ((PyObject*)p->amino_acid);
  p->amino_acid = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  tmp = ((PyObject*)p->modifications);
  p->modifications = ((PyObject*)Py_None); Py_INCREF(Py_None);
  Py_XDECREF(tmp);
  return 0;
}
static PyObject *__pyx_sq_item_14glycopeptidepy_2_c_9structure_4base_SequencePosition(PyObject *o, Py_ssize_t i) {
  PyObject *r;
  PyObject *x = PyInt_FromSsize_t(i); if(!x) return 0;
  r = Py_TYPE(o)->tp_as_mapping->mp_subscript(o, x);
  Py_DECREF(x);
  return r;
}

static int __pyx_mp_ass_subscript_14glycopeptidepy_2_c_9structure_4base_SequencePosition(PyObject *o, PyObject *i, PyObject *v) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_8__setitem__(o, i, v);
  }
  else {
    PyErr_Format(PyExc_NotImplementedError,
      "Subscript deletion not supported by %.200s", Py_TYPE(o)->tp_name);
    return -1;
  }
}

static PyObject *__pyx_tp_richcompare_14glycopeptidepy_2_c_9structure_4base_SequencePosition(PyObject *o1, PyObject *o2, int op) {
  switch (op) {
    case Py_EQ: {
      return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10__eq__(o1, o2);
    }
    case Py_NE: {
      return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_12__ne__(o1, o2);
    }
    default: {
      return __Pyx_NewRef(Py_NotImplemented);
    }
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_mass(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_4mass_1__get__(o);
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_amino_acid(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_amino_acid(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_10amino_acid_5__del__(o);
  }
}

static PyObject *__pyx_getprop_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_modifications(PyObject *o, CYTHON_UNUSED void *x) {
  return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications_1__get__(o);
}

static int __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_modifications(PyObject *o, PyObject *v, CYTHON_UNUSED void *x) {
  if (v) {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications_3__set__(o, v);
  }
  else {
    return __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13modifications_5__del__(o);
  }
}

static PyMethodDef __pyx_methods_14glycopeptidepy_2_c_9structure_4base_SequencePosition[] = {
  {"__reduce__", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_14__reduce__, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_13__reduce__},
  {"has_modification", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_20has_modification, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_19has_modification},
  {"is_modified", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_22is_modified, METH_NOARGS, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_21is_modified},
  {"add_modification", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_24add_modification, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_23add_modification},
  {"drop_modification", (PyCFunction)__pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_26drop_modification, METH_O, __pyx_doc_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_25drop_modification},
  {0, 0, 0, 0}
};

static struct PyGetSetDef __pyx_getsets_14glycopeptidepy_2_c_9structure_4base_SequencePosition[] = {
  {(char *)"mass", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_mass, 0, (char *)0, 0},
  {(char *)"amino_acid", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_amino_acid, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_amino_acid, (char *)"amino_acid: glycopeptidepy._c.structure.base.AminoAcidResidueBase", 0},
  {(char *)"modifications", __pyx_getprop_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_modifications, __pyx_setprop_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_modifications, (char *)"modifications: list", 0},
  {0, 0, 0, 0, 0}
};

static PySequenceMethods __pyx_tp_as_sequence_SequencePosition = {
  __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_16__len__, /*sq_length*/
  0, /*sq_concat*/
  0, /*sq_repeat*/
  __pyx_sq_item_14glycopeptidepy_2_c_9structure_4base_SequencePosition, /*sq_item*/
  0, /*sq_slice*/
  0, /*sq_ass_item*/
  0, /*sq_ass_slice*/
  0, /*sq_contains*/
  0, /*sq_inplace_concat*/
  0, /*sq_inplace_repeat*/
};

static PyMappingMethods __pyx_tp_as_mapping_SequencePosition = {
  __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_16__len__, /*mp_length*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_6__getitem__, /*mp_subscript*/
  __pyx_mp_ass_subscript_14glycopeptidepy_2_c_9structure_4base_SequencePosition, /*mp_ass_subscript*/
};

static PyTypeObject __pyx_type_14glycopeptidepy_2_c_9structure_4base_SequencePosition = {
  PyVarObject_HEAD_INIT(0, 0)
  "glycopeptidepy._c.structure.base.SequencePosition", /*tp_name*/
  sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_4base_SequencePosition, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_18__repr__, /*tp_repr*/
  0, /*tp_as_number*/
  &__pyx_tp_as_sequence_SequencePosition, /*tp_as_sequence*/
  &__pyx_tp_as_mapping_SequencePosition, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_BASETYPE|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  "SequencePosition(parts)", /*tp_doc*/
  __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_4base_SequencePosition, /*tp_traverse*/
  __pyx_tp_clear_14glycopeptidepy_2_c_9structure_4base_SequencePosition, /*tp_clear*/
  __pyx_tp_richcompare_14glycopeptidepy_2_c_9structure_4base_SequencePosition, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_3__iter__, /*tp_iter*/
  0, /*tp_iternext*/
  __pyx_methods_14glycopeptidepy_2_c_9structure_4base_SequencePosition, /*tp_methods*/
  0, /*tp_members*/
  __pyx_getsets_14glycopeptidepy_2_c_9structure_4base_SequencePosition, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  __pyx_pw_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_1__init__, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_14glycopeptidepy_2_c_9structure_4base_SequencePosition, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ *__pyx_freelist_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__[8];
static int __pyx_freecount_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ = 0;

static PyObject *__pyx_tp_new_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__(PyTypeObject *t, CYTHON_UNUSED PyObject *a, CYTHON_UNUSED PyObject *k) {
  PyObject *o;
  if (CYTHON_COMPILING_IN_CPYTHON && likely((__pyx_freecount_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ > 0) & (t->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__)))) {
    o = (PyObject*)__pyx_freelist_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__[--__pyx_freecount_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__];
    memset(o, 0, sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__));
    (void) PyObject_INIT(o, t);
    PyObject_GC_Track(o);
  } else {
    o = (*t->tp_alloc)(t, 0);
    if (unlikely(!o)) return 0;
  }
  return o;
}

static void __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__(PyObject *o) {
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ *)o;
  PyObject_GC_UnTrack(o);
  Py_CLEAR(p->__pyx_v_self);
  if (CYTHON_COMPILING_IN_CPYTHON && ((__pyx_freecount_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ < 8) & (Py_TYPE(o)->tp_basicsize == sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__)))) {
    __pyx_freelist_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__[__pyx_freecount_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__++] = ((struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ *)o);
  } else {
    (*Py_TYPE(o)->tp_free)(o);
  }
}

static int __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__(PyObject *o, visitproc v, void *a) {
  int e;
  struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ *p = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ *)o;
  if (p->__pyx_v_self) {
    e = (*v)(((PyObject *)p->__pyx_v_self), a); if (e) return e;
  }
  return 0;
}

static PyTypeObject __pyx_type_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ = {
  PyVarObject_HEAD_INIT(0, 0)
  "glycopeptidepy._c.structure.base.__pyx_scope_struct____iter__", /*tp_name*/
  sizeof(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__), /*tp_basicsize*/
  0, /*tp_itemsize*/
  __pyx_tp_dealloc_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__, /*tp_dealloc*/
  #if PY_VERSION_HEX < 0x030800b4
  0, /*tp_print*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4
  0, /*tp_vectorcall_offset*/
  #endif
  0, /*tp_getattr*/
  0, /*tp_setattr*/
  #if PY_MAJOR_VERSION < 3
  0, /*tp_compare*/
  #endif
  #if PY_MAJOR_VERSION >= 3
  0, /*tp_as_async*/
  #endif
  0, /*tp_repr*/
  0, /*tp_as_number*/
  0, /*tp_as_sequence*/
  0, /*tp_as_mapping*/
  0, /*tp_hash*/
  0, /*tp_call*/
  0, /*tp_str*/
  0, /*tp_getattro*/
  0, /*tp_setattro*/
  0, /*tp_as_buffer*/
  Py_TPFLAGS_DEFAULT|Py_TPFLAGS_HAVE_VERSION_TAG|Py_TPFLAGS_CHECKTYPES|Py_TPFLAGS_HAVE_NEWBUFFER|Py_TPFLAGS_HAVE_GC, /*tp_flags*/
  0, /*tp_doc*/
  __pyx_tp_traverse_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__, /*tp_traverse*/
  0, /*tp_clear*/
  0, /*tp_richcompare*/
  0, /*tp_weaklistoffset*/
  0, /*tp_iter*/
  0, /*tp_iternext*/
  0, /*tp_methods*/
  0, /*tp_members*/
  0, /*tp_getset*/
  0, /*tp_base*/
  0, /*tp_dict*/
  0, /*tp_descr_get*/
  0, /*tp_descr_set*/
  0, /*tp_dictoffset*/
  0, /*tp_init*/
  0, /*tp_alloc*/
  __pyx_tp_new_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__, /*tp_new*/
  0, /*tp_free*/
  0, /*tp_is_gc*/
  0, /*tp_bases*/
  0, /*tp_mro*/
  0, /*tp_cache*/
  0, /*tp_subclasses*/
  0, /*tp_weaklist*/
  0, /*tp_del*/
  0, /*tp_version_tag*/
  #if PY_VERSION_HEX >= 0x030400a1
  0, /*tp_finalize*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b1
  0, /*tp_vectorcall*/
  #endif
  #if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
  0, /*tp_print*/
  #endif
};

static PyMethodDef __pyx_methods[] = {
  {0, 0, 0, 0}
};

#if PY_MAJOR_VERSION >= 3
#if CYTHON_PEP489_MULTI_PHASE_INIT
static PyObject* __pyx_pymod_create(PyObject *spec, PyModuleDef *def); /*proto*/
static int __pyx_pymod_exec_base(PyObject* module); /*proto*/
static PyModuleDef_Slot __pyx_moduledef_slots[] = {
  {Py_mod_create, (void*)__pyx_pymod_create},
  {Py_mod_exec, (void*)__pyx_pymod_exec_base},
  {0, NULL}
};
#endif

static struct PyModuleDef __pyx_moduledef = {
    PyModuleDef_HEAD_INIT,
    "base",
    0, /* m_doc */
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    0, /* m_size */
  #else
    -1, /* m_size */
  #endif
    __pyx_methods /* m_methods */,
  #if CYTHON_PEP489_MULTI_PHASE_INIT
    __pyx_moduledef_slots, /* m_slots */
  #else
    NULL, /* m_reload */
  #endif
    NULL, /* m_traverse */
    NULL, /* m_clear */
    NULL /* m_free */
};
#endif
#ifndef CYTHON_SMALL_CODE
#if defined(__clang__)
    #define CYTHON_SMALL_CODE
#elif defined(__GNUC__) && (__GNUC__ > 4 || (__GNUC__ == 4 && __GNUC_MINOR__ >= 3))
    #define CYTHON_SMALL_CODE __attribute__((cold))
#else
    #define CYTHON_SMALL_CODE
#endif
#endif

static __Pyx_StringTabEntry __pyx_string_tab[] = {
  {&__pyx_n_s_AminoAcidResidueBase, __pyx_k_AminoAcidResidueBase, sizeof(__pyx_k_AminoAcidResidueBase), 0, 0, 1, 1},
  {&__pyx_n_s_AttributeError, __pyx_k_AttributeError, sizeof(__pyx_k_AttributeError), 0, 0, 1, 1},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x54, __pyx_k_Incompatible_checksums_s_vs_0x54, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x54), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x5b, __pyx_k_Incompatible_checksums_s_vs_0x5b, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x5b), 0, 0, 1, 0},
  {&__pyx_kp_s_Incompatible_checksums_s_vs_0x9a, __pyx_k_Incompatible_checksums_s_vs_0x9a, sizeof(__pyx_k_Incompatible_checksums_s_vs_0x9a), 0, 0, 1, 0},
  {&__pyx_n_s_IndexError, __pyx_k_IndexError, sizeof(__pyx_k_IndexError), 0, 0, 1, 1},
  {&__pyx_n_s_Modification, __pyx_k_Modification, sizeof(__pyx_k_Modification), 0, 0, 1, 1},
  {&__pyx_n_s_ModificationBase, __pyx_k_ModificationBase, sizeof(__pyx_k_ModificationBase), 0, 0, 1, 1},
  {&__pyx_kp_s_Modification_s_not_found, __pyx_k_Modification_s_not_found, sizeof(__pyx_k_Modification_s_not_found), 0, 0, 1, 0},
  {&__pyx_n_s_NotImplemented, __pyx_k_NotImplemented, sizeof(__pyx_k_NotImplemented), 0, 0, 1, 1},
  {&__pyx_n_s_PeptideSequenceBase, __pyx_k_PeptideSequenceBase, sizeof(__pyx_k_PeptideSequenceBase), 0, 0, 1, 1},
  {&__pyx_n_s_PickleError, __pyx_k_PickleError, sizeof(__pyx_k_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_SequencePosition, __pyx_k_SequencePosition, sizeof(__pyx_k_SequencePosition), 0, 0, 1, 1},
  {&__pyx_n_s_SequencePosition___iter, __pyx_k_SequencePosition___iter, sizeof(__pyx_k_SequencePosition___iter), 0, 0, 1, 1},
  {&__pyx_n_s_TerminalGroup, __pyx_k_TerminalGroup, sizeof(__pyx_k_TerminalGroup), 0, 0, 1, 1},
  {&__pyx_n_s_ValueError, __pyx_k_ValueError, sizeof(__pyx_k_ValueError), 0, 0, 1, 1},
  {&__pyx_n_s_add_modification, __pyx_k_add_modification, sizeof(__pyx_k_add_modification), 0, 0, 1, 1},
  {&__pyx_n_s_amino_acid, __pyx_k_amino_acid, sizeof(__pyx_k_amino_acid), 0, 0, 1, 1},
  {&__pyx_n_s_args, __pyx_k_args, sizeof(__pyx_k_args), 0, 0, 1, 1},
  {&__pyx_n_s_base_composition, __pyx_k_base_composition, sizeof(__pyx_k_base_composition), 0, 0, 1, 1},
  {&__pyx_n_s_calculate_mass, __pyx_k_calculate_mass, sizeof(__pyx_k_calculate_mass), 0, 0, 1, 1},
  {&__pyx_n_s_class, __pyx_k_class, sizeof(__pyx_k_class), 0, 0, 1, 1},
  {&__pyx_n_s_cline_in_traceback, __pyx_k_cline_in_traceback, sizeof(__pyx_k_cline_in_traceback), 0, 0, 1, 1},
  {&__pyx_n_s_clone, __pyx_k_clone, sizeof(__pyx_k_clone), 0, 0, 1, 1},
  {&__pyx_n_s_close, __pyx_k_close, sizeof(__pyx_k_close), 0, 0, 1, 1},
  {&__pyx_n_s_composition, __pyx_k_composition, sizeof(__pyx_k_composition), 0, 0, 1, 1},
  {&__pyx_n_s_dict, __pyx_k_dict, sizeof(__pyx_k_dict), 0, 0, 1, 1},
  {&__pyx_n_s_drop_modification, __pyx_k_drop_modification, sizeof(__pyx_k_drop_modification), 0, 0, 1, 1},
  {&__pyx_n_s_format, __pyx_k_format, sizeof(__pyx_k_format), 0, 0, 1, 1},
  {&__pyx_n_s_formula, __pyx_k_formula, sizeof(__pyx_k_formula), 0, 0, 1, 1},
  {&__pyx_n_s_formula_2, __pyx_k_formula_2, sizeof(__pyx_k_formula_2), 0, 0, 1, 1},
  {&__pyx_n_s_getstate, __pyx_k_getstate, sizeof(__pyx_k_getstate), 0, 0, 1, 1},
  {&__pyx_n_s_glycopeptidepy__c_structure_base, __pyx_k_glycopeptidepy__c_structure_base, sizeof(__pyx_k_glycopeptidepy__c_structure_base), 0, 0, 1, 1},
  {&__pyx_n_s_glycopeptidepy_structure_modific, __pyx_k_glycopeptidepy_structure_modific, sizeof(__pyx_k_glycopeptidepy_structure_modific), 0, 0, 1, 1},
  {&__pyx_n_s_glypy_composition, __pyx_k_glypy_composition, sizeof(__pyx_k_glypy_composition), 0, 0, 1, 1},
  {&__pyx_n_s_has_modification, __pyx_k_has_modification, sizeof(__pyx_k_has_modification), 0, 0, 1, 1},
  {&__pyx_n_s_import, __pyx_k_import, sizeof(__pyx_k_import), 0, 0, 1, 1},
  {&__pyx_n_s_invalidate, __pyx_k_invalidate, sizeof(__pyx_k_invalidate), 0, 0, 1, 1},
  {&__pyx_n_s_is_a, __pyx_k_is_a, sizeof(__pyx_k_is_a), 0, 0, 1, 1},
  {&__pyx_n_s_is_modified, __pyx_k_is_modified, sizeof(__pyx_k_is_modified), 0, 0, 1, 1},
  {&__pyx_n_s_is_tracked_for, __pyx_k_is_tracked_for, sizeof(__pyx_k_is_tracked_for), 0, 0, 1, 1},
  {&__pyx_n_s_iter, __pyx_k_iter, sizeof(__pyx_k_iter), 0, 0, 1, 1},
  {&__pyx_n_s_main, __pyx_k_main, sizeof(__pyx_k_main), 0, 0, 1, 1},
  {&__pyx_n_s_mass, __pyx_k_mass, sizeof(__pyx_k_mass), 0, 0, 1, 1},
  {&__pyx_n_s_modification, __pyx_k_modification, sizeof(__pyx_k_modification), 0, 0, 1, 1},
  {&__pyx_n_s_modifications, __pyx_k_modifications, sizeof(__pyx_k_modifications), 0, 0, 1, 1},
  {&__pyx_n_s_modify, __pyx_k_modify, sizeof(__pyx_k_modify), 0, 0, 1, 1},
  {&__pyx_n_s_name, __pyx_k_name, sizeof(__pyx_k_name), 0, 0, 1, 1},
  {&__pyx_n_s_name_2, __pyx_k_name_2, sizeof(__pyx_k_name_2), 0, 0, 1, 1},
  {&__pyx_n_s_new, __pyx_k_new, sizeof(__pyx_k_new), 0, 0, 1, 1},
  {&__pyx_n_s_parts, __pyx_k_parts, sizeof(__pyx_k_parts), 0, 0, 1, 1},
  {&__pyx_n_s_pickle, __pyx_k_pickle, sizeof(__pyx_k_pickle), 0, 0, 1, 1},
  {&__pyx_n_s_pop, __pyx_k_pop, sizeof(__pyx_k_pop), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_PickleError, __pyx_k_pyx_PickleError, sizeof(__pyx_k_pyx_PickleError), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_checksum, __pyx_k_pyx_checksum, sizeof(__pyx_k_pyx_checksum), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_result, __pyx_k_pyx_result, sizeof(__pyx_k_pyx_result), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_state, __pyx_k_pyx_state, sizeof(__pyx_k_pyx_state), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_type, __pyx_k_pyx_type, sizeof(__pyx_k_pyx_type), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_AminoAcidResidueB, __pyx_k_pyx_unpickle_AminoAcidResidueB, sizeof(__pyx_k_pyx_unpickle_AminoAcidResidueB), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_ModificationBase, __pyx_k_pyx_unpickle_ModificationBase, sizeof(__pyx_k_pyx_unpickle_ModificationBase), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_unpickle_PeptideSequenceBa, __pyx_k_pyx_unpickle_PeptideSequenceBa, sizeof(__pyx_k_pyx_unpickle_PeptideSequenceBa), 0, 0, 1, 1},
  {&__pyx_n_s_pyx_vtable, __pyx_k_pyx_vtable, sizeof(__pyx_k_pyx_vtable), 0, 0, 1, 1},
  {&__pyx_kp_s_r_r, __pyx_k_r_r, sizeof(__pyx_k_r_r), 0, 0, 1, 0},
  {&__pyx_n_s_range, __pyx_k_range, sizeof(__pyx_k_range), 0, 0, 1, 1},
  {&__pyx_n_s_reduce, __pyx_k_reduce, sizeof(__pyx_k_reduce), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_cython, __pyx_k_reduce_cython, sizeof(__pyx_k_reduce_cython), 0, 0, 1, 1},
  {&__pyx_n_s_reduce_ex, __pyx_k_reduce_ex, sizeof(__pyx_k_reduce_ex), 0, 0, 1, 1},
  {&__pyx_n_s_self, __pyx_k_self, sizeof(__pyx_k_self), 0, 0, 1, 1},
  {&__pyx_kp_s_self___class_____name___self_ba, __pyx_k_self___class_____name___self_ba, sizeof(__pyx_k_self___class_____name___self_ba), 0, 0, 1, 0},
  {&__pyx_n_s_send, __pyx_k_send, sizeof(__pyx_k_send), 0, 0, 1, 1},
  {&__pyx_n_s_serialize, __pyx_k_serialize, sizeof(__pyx_k_serialize), 0, 0, 1, 1},
  {&__pyx_n_s_setstate, __pyx_k_setstate, sizeof(__pyx_k_setstate), 0, 0, 1, 1},
  {&__pyx_n_s_setstate_cython, __pyx_k_setstate_cython, sizeof(__pyx_k_setstate_cython), 0, 0, 1, 1},
  {&__pyx_kp_s_stringsource, __pyx_k_stringsource, sizeof(__pyx_k_stringsource), 0, 0, 1, 0},
  {&__pyx_n_s_symbol, __pyx_k_symbol, sizeof(__pyx_k_symbol), 0, 0, 1, 1},
  {&__pyx_n_s_test, __pyx_k_test, sizeof(__pyx_k_test), 0, 0, 1, 1},
  {&__pyx_n_s_throw, __pyx_k_throw, sizeof(__pyx_k_throw), 0, 0, 1, 1},
  {&__pyx_n_s_update, __pyx_k_update, sizeof(__pyx_k_update), 0, 0, 1, 1},
  {0, 0, 0, 0, 0, 0, 0}
};
static CYTHON_SMALL_CODE int __Pyx_InitCachedBuiltins(void) {
  __pyx_builtin_AttributeError = __Pyx_GetBuiltinName(__pyx_n_s_AttributeError); if (!__pyx_builtin_AttributeError) __PYX_ERR(0, 103, __pyx_L1_error)
  __pyx_builtin_NotImplemented = __Pyx_GetBuiltinName(__pyx_n_s_NotImplemented); if (!__pyx_builtin_NotImplemented) __PYX_ERR(0, 111, __pyx_L1_error)
  __pyx_builtin_IndexError = __Pyx_GetBuiltinName(__pyx_n_s_IndexError); if (!__pyx_builtin_IndexError) __PYX_ERR(0, 241, __pyx_L1_error)
  __pyx_builtin_range = __Pyx_GetBuiltinName(__pyx_n_s_range); if (!__pyx_builtin_range) __PYX_ERR(0, 288, __pyx_L1_error)
  __pyx_builtin_ValueError = __Pyx_GetBuiltinName(__pyx_n_s_ValueError); if (!__pyx_builtin_ValueError) __PYX_ERR(0, 325, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitCachedConstants(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_InitCachedConstants", 0);

  /* "(tree fragment)":1
 * def __pyx_unpickle_PeptideSequenceBase(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_tuple_ = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple_)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple_);
  __Pyx_GIVEREF(__pyx_tuple_);
  __pyx_codeobj__2 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple_, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_PeptideSequenceBa, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__3 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__3)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__3);
  __Pyx_GIVEREF(__pyx_tuple__3);
  __pyx_codeobj__4 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__3, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_AminoAcidResidueB, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__4)) __PYX_ERR(2, 1, __pyx_L1_error)
  __pyx_tuple__5 = PyTuple_Pack(5, __pyx_n_s_pyx_type, __pyx_n_s_pyx_checksum, __pyx_n_s_pyx_state, __pyx_n_s_pyx_PickleError, __pyx_n_s_pyx_result); if (unlikely(!__pyx_tuple__5)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_tuple__5);
  __Pyx_GIVEREF(__pyx_tuple__5);
  __pyx_codeobj__6 = (PyObject*)__Pyx_PyCode_New(3, 0, 5, 0, CO_OPTIMIZED|CO_NEWLOCALS, __pyx_empty_bytes, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_tuple__5, __pyx_empty_tuple, __pyx_empty_tuple, __pyx_kp_s_stringsource, __pyx_n_s_pyx_unpickle_ModificationBase, 1, __pyx_empty_bytes); if (unlikely(!__pyx_codeobj__6)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_InitGlobals(void) {
  if (__Pyx_InitStrings(__pyx_string_tab) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  __pyx_int_0 = PyInt_FromLong(0); if (unlikely(!__pyx_int_0)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_1 = PyInt_FromLong(1); if (unlikely(!__pyx_int_1)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_88971479 = PyInt_FromLong(88971479L); if (unlikely(!__pyx_int_88971479)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_95446078 = PyInt_FromLong(95446078L); if (unlikely(!__pyx_int_95446078)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_int_161643430 = PyInt_FromLong(161643430L); if (unlikely(!__pyx_int_161643430)) __PYX_ERR(0, 1, __pyx_L1_error)
  return 0;
  __pyx_L1_error:;
  return -1;
}

static CYTHON_SMALL_CODE int __Pyx_modinit_global_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_export_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_init_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_type_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_variable_import_code(void); /*proto*/
static CYTHON_SMALL_CODE int __Pyx_modinit_function_import_code(void); /*proto*/

static int __Pyx_modinit_global_init_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_global_init_code", 0);
  /*--- Global init code ---*/
  __pyx_v_14glycopeptidepy_2_c_9structure_4base_formula = Py_None; Py_INCREF(Py_None);
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_variable_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_export_code", 0);
  /*--- Variable export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_function_export_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_export_code", 0);
  /*--- Function export code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}

static int __Pyx_modinit_type_init_code(void) {
  __Pyx_RefNannyDeclarations
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_init_code", 0);
  /*--- Type init code ---*/
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase = &__pyx_vtable_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase.get = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *, Py_ssize_t))__pyx_f_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase_get;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase._invalidate = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_4base_19PeptideSequenceBase__invalidate;
  if (PyType_Ready(&__pyx_type_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase) < 0) __PYX_ERR(0, 14, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase.tp_dictoffset && __pyx_type_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase.tp_dict, __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase) < 0) __PYX_ERR(0, 14, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_PeptideSequenceBase, (PyObject *)&__pyx_type_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase) < 0) __PYX_ERR(0, 14, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase) < 0) __PYX_ERR(0, 14, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase = &__pyx_type_14glycopeptidepy_2_c_9structure_4base_PeptideSequenceBase;
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_TerminalGroup = &__pyx_vtable_14glycopeptidepy_2_c_9structure_4base_TerminalGroup;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_TerminalGroup.get_modification = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *))__pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_get_modification;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_TerminalGroup.get_composition = (struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *))__pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_get_composition;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_TerminalGroup.modify = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_modify;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_TerminalGroup.set_modification = (void (*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *))__pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_set_modification;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_TerminalGroup.equal_to = (int (*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, PyObject *))__pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_equal_to;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_TerminalGroup.serialize = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_TerminalGroup *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_4base_13TerminalGroup_serialize;
  if (PyType_Ready(&__pyx_type_14glycopeptidepy_2_c_9structure_4base_TerminalGroup) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_14glycopeptidepy_2_c_9structure_4base_TerminalGroup.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_14glycopeptidepy_2_c_9structure_4base_TerminalGroup.tp_dictoffset && __pyx_type_14glycopeptidepy_2_c_9structure_4base_TerminalGroup.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_14glycopeptidepy_2_c_9structure_4base_TerminalGroup.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_14glycopeptidepy_2_c_9structure_4base_TerminalGroup.tp_dict, __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_TerminalGroup) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_TerminalGroup, (PyObject *)&__pyx_type_14glycopeptidepy_2_c_9structure_4base_TerminalGroup) < 0) __PYX_ERR(0, 27, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_TerminalGroup = &__pyx_type_14glycopeptidepy_2_c_9structure_4base_TerminalGroup;
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase = &__pyx_vtable_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase._create = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *(*)(PyObject *, PyObject *, double, struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition *))__pyx_f_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase__create;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase.equal_to = (int (*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *, struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *))__pyx_f_14glycopeptidepy_2_c_9structure_4base_20AminoAcidResidueBase_equal_to;
  if (PyType_Ready(&__pyx_type_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase) < 0) __PYX_ERR(0, 145, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase.tp_dictoffset && __pyx_type_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase.tp_dict, __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase) < 0) __PYX_ERR(0, 145, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_AminoAcidResidueBase, (PyObject *)&__pyx_type_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase) < 0) __PYX_ERR(0, 145, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase) < 0) __PYX_ERR(0, 145, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase = &__pyx_type_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase;
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_ModificationBase = &__pyx_vtable_14glycopeptidepy_2_c_9structure_4base_ModificationBase;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_ModificationBase.is_a = (int (*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *, PyObject *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_is_a;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_ModificationBase.serialize = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_serialize;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_ModificationBase.is_tracked_for = (int (*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_ModificationBase *, PyObject *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_4base_16ModificationBase_is_tracked_for;
  if (PyType_Ready(&__pyx_type_14glycopeptidepy_2_c_9structure_4base_ModificationBase) < 0) __PYX_ERR(0, 186, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_14glycopeptidepy_2_c_9structure_4base_ModificationBase.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_14glycopeptidepy_2_c_9structure_4base_ModificationBase.tp_dictoffset && __pyx_type_14glycopeptidepy_2_c_9structure_4base_ModificationBase.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_14glycopeptidepy_2_c_9structure_4base_ModificationBase.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_14glycopeptidepy_2_c_9structure_4base_ModificationBase.tp_dict, __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_ModificationBase) < 0) __PYX_ERR(0, 186, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_ModificationBase, (PyObject *)&__pyx_type_14glycopeptidepy_2_c_9structure_4base_ModificationBase) < 0) __PYX_ERR(0, 186, __pyx_L1_error)
  if (__Pyx_setup_reduce((PyObject*)&__pyx_type_14glycopeptidepy_2_c_9structure_4base_ModificationBase) < 0) __PYX_ERR(0, 186, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_ModificationBase = &__pyx_type_14glycopeptidepy_2_c_9structure_4base_ModificationBase;
  __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_SequencePosition = &__pyx_vtable_14glycopeptidepy_2_c_9structure_4base_SequencePosition;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_SequencePosition._create = (struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_AminoAcidResidueBase *, PyObject *))__pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition__create;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_SequencePosition.get_mass = (double (*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *))__pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_get_mass;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_SequencePosition.has_modification = (int (*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *, PyObject *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_has_modification;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_SequencePosition.is_modified = (int (*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_is_modified;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_SequencePosition.drop_modification = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *, PyObject *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_drop_modification;
  __pyx_vtable_14glycopeptidepy_2_c_9structure_4base_SequencePosition.add_modification = (PyObject *(*)(struct __pyx_obj_14glycopeptidepy_2_c_9structure_4base_SequencePosition *, PyObject *, int __pyx_skip_dispatch))__pyx_f_14glycopeptidepy_2_c_9structure_4base_16SequencePosition_add_modification;
  if (PyType_Ready(&__pyx_type_14glycopeptidepy_2_c_9structure_4base_SequencePosition) < 0) __PYX_ERR(0, 225, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_14glycopeptidepy_2_c_9structure_4base_SequencePosition.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_14glycopeptidepy_2_c_9structure_4base_SequencePosition.tp_dictoffset && __pyx_type_14glycopeptidepy_2_c_9structure_4base_SequencePosition.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_14glycopeptidepy_2_c_9structure_4base_SequencePosition.tp_getattro = __Pyx_PyObject_GenericGetAttr;
  }
  if (__Pyx_SetVtable(__pyx_type_14glycopeptidepy_2_c_9structure_4base_SequencePosition.tp_dict, __pyx_vtabptr_14glycopeptidepy_2_c_9structure_4base_SequencePosition) < 0) __PYX_ERR(0, 225, __pyx_L1_error)
  if (PyObject_SetAttr(__pyx_m, __pyx_n_s_SequencePosition, (PyObject *)&__pyx_type_14glycopeptidepy_2_c_9structure_4base_SequencePosition) < 0) __PYX_ERR(0, 225, __pyx_L1_error)
  __pyx_ptype_14glycopeptidepy_2_c_9structure_4base_SequencePosition = &__pyx_type_14glycopeptidepy_2_c_9structure_4base_SequencePosition;
  if (PyType_Ready(&__pyx_type_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__) < 0) __PYX_ERR(0, 231, __pyx_L1_error)
  #if PY_VERSION_HEX < 0x030800B1
  __pyx_type_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__.tp_print = 0;
  #endif
  if ((CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP) && likely(!__pyx_type_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__.tp_dictoffset && __pyx_type_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__.tp_getattro == PyObject_GenericGetAttr)) {
    __pyx_type_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
  }
  __pyx_ptype_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__ = &__pyx_type_14glycopeptidepy_2_c_9structure_4base___pyx_scope_struct____iter__;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_type_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_type_import_code", 0);
  /*--- Type import code ---*/
  __pyx_t_1 = PyImport_ImportModule("glypy.composition.ccomposition"); if (unlikely(!__pyx_t_1)) __PYX_ERR(3, 2, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_5glypy_11composition_12ccomposition_CComposition = __Pyx_ImportType(__pyx_t_1, "glypy.composition.ccomposition", "CComposition", sizeof(struct __pyx_obj_5glypy_11composition_12ccomposition_CComposition), __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_5glypy_11composition_12ccomposition_CComposition) __PYX_ERR(3, 2, __pyx_L1_error)
  __pyx_vtabptr_5glypy_11composition_12ccomposition_CComposition = (struct __pyx_vtabstruct_5glypy_11composition_12ccomposition_CComposition*)__Pyx_GetVtable(__pyx_ptype_5glypy_11composition_12ccomposition_CComposition->tp_dict); if (unlikely(!__pyx_vtabptr_5glypy_11composition_12ccomposition_CComposition)) __PYX_ERR(3, 2, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = PyImport_ImportModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_t_1)) __PYX_ERR(4, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __pyx_ptype_7cpython_4type_type = __Pyx_ImportType(__pyx_t_1, __Pyx_BUILTIN_MODULE_NAME, "type", 
  #if defined(PYPY_VERSION_NUM) && PYPY_VERSION_NUM < 0x050B0000
  sizeof(PyTypeObject),
  #else
  sizeof(PyHeapTypeObject),
  #endif
  __Pyx_ImportType_CheckSize_Warn);
   if (!__pyx_ptype_7cpython_4type_type) __PYX_ERR(4, 9, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_variable_import_code(void) {
  __Pyx_RefNannyDeclarations
  PyObject *__pyx_t_1 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannySetupContext("__Pyx_modinit_variable_import_code", 0);
  /*--- Variable import code ---*/
  __pyx_t_1 = PyImport_ImportModule("glypy.composition.ccomposition"); if (!__pyx_t_1) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_atom", (void **)&__pyx_vp_5glypy_11composition_12ccomposition__atom, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_formula", (void **)&__pyx_vp_5glypy_11composition_12ccomposition__formula, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "_isotope_string", (void **)&__pyx_vp_5glypy_11composition_12ccomposition__isotope_string, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "isotope_pattern", (void **)&__pyx_vp_5glypy_11composition_12ccomposition_isotope_pattern, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  if (__Pyx_ImportVoidPtr(__pyx_t_1, "formula_pattern", (void **)&__pyx_vp_5glypy_11composition_12ccomposition_formula_pattern, "PyObject *") < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_RefNannyFinishContext();
  return 0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_RefNannyFinishContext();
  return -1;
}

static int __Pyx_modinit_function_import_code(void) {
  __Pyx_RefNannyDeclarations
  __Pyx_RefNannySetupContext("__Pyx_modinit_function_import_code", 0);
  /*--- Function import code ---*/
  __Pyx_RefNannyFinishContext();
  return 0;
}


#ifndef CYTHON_NO_PYINIT_EXPORT
#define __Pyx_PyMODINIT_FUNC PyMODINIT_FUNC
#elif PY_MAJOR_VERSION < 3
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" void
#else
#define __Pyx_PyMODINIT_FUNC void
#endif
#else
#ifdef __cplusplus
#define __Pyx_PyMODINIT_FUNC extern "C" PyObject *
#else
#define __Pyx_PyMODINIT_FUNC PyObject *
#endif
#endif


#if PY_MAJOR_VERSION < 3
__Pyx_PyMODINIT_FUNC initbase(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC initbase(void)
#else
__Pyx_PyMODINIT_FUNC PyInit_base(void) CYTHON_SMALL_CODE; /*proto*/
__Pyx_PyMODINIT_FUNC PyInit_base(void)
#if CYTHON_PEP489_MULTI_PHASE_INIT
{
  return PyModuleDef_Init(&__pyx_moduledef);
}
static CYTHON_SMALL_CODE int __Pyx_check_single_interpreter(void) {
    #if PY_VERSION_HEX >= 0x030700A1
    static PY_INT64_T main_interpreter_id = -1;
    PY_INT64_T current_id = PyInterpreterState_GetID(PyThreadState_Get()->interp);
    if (main_interpreter_id == -1) {
        main_interpreter_id = current_id;
        return (unlikely(current_id == -1)) ? -1 : 0;
    } else if (unlikely(main_interpreter_id != current_id))
    #else
    static PyInterpreterState *main_interpreter = NULL;
    PyInterpreterState *current_interpreter = PyThreadState_Get()->interp;
    if (!main_interpreter) {
        main_interpreter = current_interpreter;
    } else if (unlikely(main_interpreter != current_interpreter))
    #endif
    {
        PyErr_SetString(
            PyExc_ImportError,
            "Interpreter change detected - this module can only be loaded into one interpreter per process.");
        return -1;
    }
    return 0;
}
static CYTHON_SMALL_CODE int __Pyx_copy_spec_to_module(PyObject *spec, PyObject *moddict, const char* from_name, const char* to_name, int allow_none) {
    PyObject *value = PyObject_GetAttrString(spec, from_name);
    int result = 0;
    if (likely(value)) {
        if (allow_none || value != Py_None) {
            result = PyDict_SetItemString(moddict, to_name, value);
        }
        Py_DECREF(value);
    } else if (PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Clear();
    } else {
        result = -1;
    }
    return result;
}
static CYTHON_SMALL_CODE PyObject* __pyx_pymod_create(PyObject *spec, CYTHON_UNUSED PyModuleDef *def) {
    PyObject *module = NULL, *moddict, *modname;
    if (__Pyx_check_single_interpreter())
        return NULL;
    if (__pyx_m)
        return __Pyx_NewRef(__pyx_m);
    modname = PyObject_GetAttrString(spec, "name");
    if (unlikely(!modname)) goto bad;
    module = PyModule_NewObject(modname);
    Py_DECREF(modname);
    if (unlikely(!module)) goto bad;
    moddict = PyModule_GetDict(module);
    if (unlikely(!moddict)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "loader", "__loader__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "origin", "__file__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "parent", "__package__", 1) < 0)) goto bad;
    if (unlikely(__Pyx_copy_spec_to_module(spec, moddict, "submodule_search_locations", "__path__", 0) < 0)) goto bad;
    return module;
bad:
    Py_XDECREF(module);
    return NULL;
}


static CYTHON_SMALL_CODE int __pyx_pymod_exec_base(PyObject *__pyx_pyinit_module)
#endif
#endif
{
  PyObject *__pyx_t_1 = NULL;
  PyObject *__pyx_t_2 = NULL;
  int __pyx_lineno = 0;
  const char *__pyx_filename = NULL;
  int __pyx_clineno = 0;
  __Pyx_RefNannyDeclarations
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  if (__pyx_m) {
    if (__pyx_m == __pyx_pyinit_module) return 0;
    PyErr_SetString(PyExc_RuntimeError, "Module 'base' has already been imported. Re-initialisation is not supported.");
    return -1;
  }
  #elif PY_MAJOR_VERSION >= 3
  if (__pyx_m) return __Pyx_NewRef(__pyx_m);
  #endif
  #if CYTHON_REFNANNY
__Pyx_RefNanny = __Pyx_RefNannyImportAPI("refnanny");
if (!__Pyx_RefNanny) {
  PyErr_Clear();
  __Pyx_RefNanny = __Pyx_RefNannyImportAPI("Cython.Runtime.refnanny");
  if (!__Pyx_RefNanny)
      Py_FatalError("failed to import 'refnanny' module");
}
#endif
  __Pyx_RefNannySetupContext("__Pyx_PyMODINIT_FUNC PyInit_base(void)", 0);
  if (__Pyx_check_binary_version() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pxy_PyFrame_Initialize_Offsets
  __Pxy_PyFrame_Initialize_Offsets();
  #endif
  __pyx_empty_tuple = PyTuple_New(0); if (unlikely(!__pyx_empty_tuple)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_bytes = PyBytes_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_bytes)) __PYX_ERR(0, 1, __pyx_L1_error)
  __pyx_empty_unicode = PyUnicode_FromStringAndSize("", 0); if (unlikely(!__pyx_empty_unicode)) __PYX_ERR(0, 1, __pyx_L1_error)
  #ifdef __Pyx_CyFunction_USED
  if (__pyx_CyFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_FusedFunction_USED
  if (__pyx_FusedFunction_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Coroutine_USED
  if (__pyx_Coroutine_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_Generator_USED
  if (__pyx_Generator_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_AsyncGen_USED
  if (__pyx_AsyncGen_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  #ifdef __Pyx_StopAsyncIteration_USED
  if (__pyx_StopAsyncIteration_init() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  /*--- Library function declarations ---*/
  /*--- Threads initialization code ---*/
  #if defined(WITH_THREAD) && PY_VERSION_HEX < 0x030700F0 && defined(__PYX_FORCE_INIT_THREADS) && __PYX_FORCE_INIT_THREADS
  PyEval_InitThreads();
  #endif
  /*--- Module creation code ---*/
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  __pyx_m = __pyx_pyinit_module;
  Py_INCREF(__pyx_m);
  #else
  #if PY_MAJOR_VERSION < 3
  __pyx_m = Py_InitModule4("base", __pyx_methods, 0, 0, PYTHON_API_VERSION); Py_XINCREF(__pyx_m);
  #else
  __pyx_m = PyModule_Create(&__pyx_moduledef);
  #endif
  if (unlikely(!__pyx_m)) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  __pyx_d = PyModule_GetDict(__pyx_m); if (unlikely(!__pyx_d)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_d);
  __pyx_b = PyImport_AddModule(__Pyx_BUILTIN_MODULE_NAME); if (unlikely(!__pyx_b)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_b);
  __pyx_cython_runtime = PyImport_AddModule((char *) "cython_runtime"); if (unlikely(!__pyx_cython_runtime)) __PYX_ERR(0, 1, __pyx_L1_error)
  Py_INCREF(__pyx_cython_runtime);
  if (PyObject_SetAttrString(__pyx_m, "__builtins__", __pyx_b) < 0) __PYX_ERR(0, 1, __pyx_L1_error);
  /*--- Initialize various global constants etc. ---*/
  if (__Pyx_InitGlobals() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #if PY_MAJOR_VERSION < 3 && (__PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT)
  if (__Pyx_init_sys_getdefaultencoding_params() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif
  if (__pyx_module_is_main_glycopeptidepy___c__structure__base) {
    if (PyObject_SetAttr(__pyx_m, __pyx_n_s_name_2, __pyx_n_s_main) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  }
  #if PY_MAJOR_VERSION >= 3
  {
    PyObject *modules = PyImport_GetModuleDict(); if (unlikely(!modules)) __PYX_ERR(0, 1, __pyx_L1_error)
    if (!PyDict_GetItemString(modules, "glycopeptidepy._c.structure.base")) {
      if (unlikely(PyDict_SetItemString(modules, "glycopeptidepy._c.structure.base", __pyx_m) < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
    }
  }
  #endif
  /*--- Builtin init code ---*/
  if (__Pyx_InitCachedBuiltins() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Constants init code ---*/
  if (__Pyx_InitCachedConstants() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  /*--- Global type/function init code ---*/
  (void)__Pyx_modinit_global_init_code();
  (void)__Pyx_modinit_variable_export_code();
  (void)__Pyx_modinit_function_export_code();
  if (unlikely(__Pyx_modinit_type_init_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_type_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  if (unlikely(__Pyx_modinit_variable_import_code() < 0)) __PYX_ERR(0, 1, __pyx_L1_error)
  (void)__Pyx_modinit_function_import_code();
  /*--- Execution code ---*/
  #if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
  if (__Pyx_patch_abc() < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  #endif

  /* "glycopeptidepy/_c/structure/base.pyx":3
 * cimport cython
 * from glypy.composition.ccomposition cimport CComposition
 * from glypy.composition import formula as _formula             # <<<<<<<<<<<<<<
 * 
 * from cpython.list cimport PyList_GetItem, PyList_GET_ITEM, PyList_Size
 */
  __pyx_t_1 = PyList_New(1); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  __Pyx_INCREF(__pyx_n_s_formula);
  __Pyx_GIVEREF(__pyx_n_s_formula);
  PyList_SET_ITEM(__pyx_t_1, 0, __pyx_n_s_formula);
  __pyx_t_2 = __Pyx_Import(__pyx_n_s_glypy_composition, __pyx_t_1, -1); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __pyx_t_1 = __Pyx_ImportFrom(__pyx_t_2, __pyx_n_s_formula); if (unlikely(!__pyx_t_1)) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_1);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_formula_2, __pyx_t_1) < 0) __PYX_ERR(0, 3, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_1); __pyx_t_1 = 0;
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/base.pyx":9
 * 
 * 
 * cdef object formula = _formula             # <<<<<<<<<<<<<<
 * 
 * DEF SAFE_GET = 1
 */
  __Pyx_GetModuleGlobalName(__pyx_t_2, __pyx_n_s_formula_2); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 9, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  __Pyx_XGOTREF(__pyx_v_14glycopeptidepy_2_c_9structure_4base_formula);
  __Pyx_DECREF_SET(__pyx_v_14glycopeptidepy_2_c_9structure_4base_formula, __pyx_t_2);
  __Pyx_GIVEREF(__pyx_t_2);
  __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_PeptideSequenceBase(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_14glycopeptidepy_2_c_9structure_4base_1__pyx_unpickle_PeptideSequenceBase, NULL, __pyx_n_s_glycopeptidepy__c_structure_base); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_PeptideSequenceBa, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":11
 *         __pyx_unpickle_PeptideSequenceBase__set_state(<PeptideSequenceBase> __pyx_result, __pyx_state)
 *     return __pyx_result
 * cdef __pyx_unpickle_PeptideSequenceBase__set_state(PeptideSequenceBase __pyx_result, tuple __pyx_state):             # <<<<<<<<<<<<<<
 *     __pyx_result._c_term = __pyx_state[0]; __pyx_result._glycosylation_manager = __pyx_state[1]; __pyx_result._mass = __pyx_state[2]; __pyx_result._n_term = __pyx_state[3]; __pyx_result._peptide_composition = __pyx_state[4]; __pyx_result._total_composition = __pyx_state[5]; __pyx_result.sequence = __pyx_state[6]
 *     if len(__pyx_state) > 7 and hasattr(__pyx_result, '__dict__'):
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_14glycopeptidepy_2_c_9structure_4base_3__pyx_unpickle_AminoAcidResidueBase, NULL, __pyx_n_s_glycopeptidepy__c_structure_base); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_AminoAcidResidueB, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "(tree fragment)":1
 * def __pyx_unpickle_ModificationBase(__pyx_type, long __pyx_checksum, __pyx_state):             # <<<<<<<<<<<<<<
 *     cdef object __pyx_PickleError
 *     cdef object __pyx_result
 */
  __pyx_t_2 = PyCFunction_NewEx(&__pyx_mdef_14glycopeptidepy_2_c_9structure_4base_5__pyx_unpickle_ModificationBase, NULL, __pyx_n_s_glycopeptidepy__c_structure_base); if (unlikely(!__pyx_t_2)) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_pyx_unpickle_ModificationBase, __pyx_t_2) < 0) __PYX_ERR(2, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /* "glycopeptidepy/_c/structure/base.pyx":1
 * cimport cython             # <<<<<<<<<<<<<<
 * from glypy.composition.ccomposition cimport CComposition
 * from glypy.composition import formula as _formula
 */
  __pyx_t_2 = __Pyx_PyDict_NewPresized(0); if (unlikely(!__pyx_t_2)) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_GOTREF(__pyx_t_2);
  if (PyDict_SetItem(__pyx_d, __pyx_n_s_test, __pyx_t_2) < 0) __PYX_ERR(0, 1, __pyx_L1_error)
  __Pyx_DECREF(__pyx_t_2); __pyx_t_2 = 0;

  /*--- Wrapped vars code ---*/

  goto __pyx_L0;
  __pyx_L1_error:;
  __Pyx_XDECREF(__pyx_t_1);
  __Pyx_XDECREF(__pyx_t_2);
  if (__pyx_m) {
    if (__pyx_d) {
      __Pyx_AddTraceback("init glycopeptidepy._c.structure.base", __pyx_clineno, __pyx_lineno, __pyx_filename);
    }
    Py_CLEAR(__pyx_m);
  } else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_ImportError, "init glycopeptidepy._c.structure.base");
  }
  __pyx_L0:;
  __Pyx_RefNannyFinishContext();
  #if CYTHON_PEP489_MULTI_PHASE_INIT
  return (__pyx_m != NULL) ? 0 : -1;
  #elif PY_MAJOR_VERSION >= 3
  return __pyx_m;
  #else
  return;
  #endif
}

/* --- Runtime support code --- */
/* Refnanny */
#if CYTHON_REFNANNY
static __Pyx_RefNannyAPIStruct *__Pyx_RefNannyImportAPI(const char *modname) {
    PyObject *m = NULL, *p = NULL;
    void *r = NULL;
    m = PyImport_ImportModule(modname);
    if (!m) goto end;
    p = PyObject_GetAttrString(m, "RefNannyAPI");
    if (!p) goto end;
    r = PyLong_AsVoidPtr(p);
end:
    Py_XDECREF(p);
    Py_XDECREF(m);
    return (__Pyx_RefNannyAPIStruct *)r;
}
#endif

/* PyObjectGetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStr(PyObject* obj, PyObject* attr_name) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro))
        return tp->tp_getattro(obj, attr_name);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_getattr))
        return tp->tp_getattr(obj, PyString_AS_STRING(attr_name));
#endif
    return PyObject_GetAttr(obj, attr_name);
}
#endif

/* GetBuiltinName */
static PyObject *__Pyx_GetBuiltinName(PyObject *name) {
    PyObject* result = __Pyx_PyObject_GetAttrStr(__pyx_b, name);
    if (unlikely(!result)) {
        PyErr_Format(PyExc_NameError,
#if PY_MAJOR_VERSION >= 3
            "name '%U' is not defined", name);
#else
            "name '%.200s' is not defined", PyString_AS_STRING(name));
#endif
    }
    return result;
}

/* PyDictVersioning */
#if CYTHON_USE_DICT_VERSIONS && CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE PY_UINT64_T __Pyx_get_tp_dict_version(PyObject *obj) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    return likely(dict) ? __PYX_GET_DICT_VERSION(dict) : 0;
}
static CYTHON_INLINE PY_UINT64_T __Pyx_get_object_dict_version(PyObject *obj) {
    PyObject **dictptr = NULL;
    Py_ssize_t offset = Py_TYPE(obj)->tp_dictoffset;
    if (offset) {
#if CYTHON_COMPILING_IN_CPYTHON
        dictptr = (likely(offset > 0)) ? (PyObject **) ((char *)obj + offset) : _PyObject_GetDictPtr(obj);
#else
        dictptr = _PyObject_GetDictPtr(obj);
#endif
    }
    return (dictptr && *dictptr) ? __PYX_GET_DICT_VERSION(*dictptr) : 0;
}
static CYTHON_INLINE int __Pyx_object_dict_version_matches(PyObject* obj, PY_UINT64_T tp_dict_version, PY_UINT64_T obj_dict_version) {
    PyObject *dict = Py_TYPE(obj)->tp_dict;
    if (unlikely(!dict) || unlikely(tp_dict_version != __PYX_GET_DICT_VERSION(dict)))
        return 0;
    return obj_dict_version == __Pyx_get_object_dict_version(obj);
}
#endif

/* PyFunctionFastCall */
#if CYTHON_FAST_PYCALL
static PyObject* __Pyx_PyFunction_FastCallNoKw(PyCodeObject *co, PyObject **args, Py_ssize_t na,
                                               PyObject *globals) {
    PyFrameObject *f;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    PyObject **fastlocals;
    Py_ssize_t i;
    PyObject *result;
    assert(globals != NULL);
    /* XXX Perhaps we should create a specialized
       PyFrame_New() that doesn't take locals, but does
       take builtins without sanity checking them.
       */
    assert(tstate != NULL);
    f = PyFrame_New(tstate, co, globals, NULL);
    if (f == NULL) {
        return NULL;
    }
    fastlocals = __Pyx_PyFrame_GetLocalsplus(f);
    for (i = 0; i < na; i++) {
        Py_INCREF(*args);
        fastlocals[i] = *args++;
    }
    result = PyEval_EvalFrameEx(f,0);
    ++tstate->recursion_depth;
    Py_DECREF(f);
    --tstate->recursion_depth;
    return result;
}
#if 1 || PY_VERSION_HEX < 0x030600B1
static PyObject *__Pyx_PyFunction_FastCallDict(PyObject *func, PyObject **args, Py_ssize_t nargs, PyObject *kwargs) {
    PyCodeObject *co = (PyCodeObject *)PyFunction_GET_CODE(func);
    PyObject *globals = PyFunction_GET_GLOBALS(func);
    PyObject *argdefs = PyFunction_GET_DEFAULTS(func);
    PyObject *closure;
#if PY_MAJOR_VERSION >= 3
    PyObject *kwdefs;
#endif
    PyObject *kwtuple, **k;
    PyObject **d;
    Py_ssize_t nd;
    Py_ssize_t nk;
    PyObject *result;
    assert(kwargs == NULL || PyDict_Check(kwargs));
    nk = kwargs ? PyDict_Size(kwargs) : 0;
    if (Py_EnterRecursiveCall((char*)" while calling a Python object")) {
        return NULL;
    }
    if (
#if PY_MAJOR_VERSION >= 3
            co->co_kwonlyargcount == 0 &&
#endif
            likely(kwargs == NULL || nk == 0) &&
            co->co_flags == (CO_OPTIMIZED | CO_NEWLOCALS | CO_NOFREE)) {
        if (argdefs == NULL && co->co_argcount == nargs) {
            result = __Pyx_PyFunction_FastCallNoKw(co, args, nargs, globals);
            goto done;
        }
        else if (nargs == 0 && argdefs != NULL
                 && co->co_argcount == Py_SIZE(argdefs)) {
            /* function called with no arguments, but all parameters have
               a default value: use default values as arguments .*/
            args = &PyTuple_GET_ITEM(argdefs, 0);
            result =__Pyx_PyFunction_FastCallNoKw(co, args, Py_SIZE(argdefs), globals);
            goto done;
        }
    }
    if (kwargs != NULL) {
        Py_ssize_t pos, i;
        kwtuple = PyTuple_New(2 * nk);
        if (kwtuple == NULL) {
            result = NULL;
            goto done;
        }
        k = &PyTuple_GET_ITEM(kwtuple, 0);
        pos = i = 0;
        while (PyDict_Next(kwargs, &pos, &k[i], &k[i+1])) {
            Py_INCREF(k[i]);
            Py_INCREF(k[i+1]);
            i += 2;
        }
        nk = i / 2;
    }
    else {
        kwtuple = NULL;
        k = NULL;
    }
    closure = PyFunction_GET_CLOSURE(func);
#if PY_MAJOR_VERSION >= 3
    kwdefs = PyFunction_GET_KW_DEFAULTS(func);
#endif
    if (argdefs != NULL) {
        d = &PyTuple_GET_ITEM(argdefs, 0);
        nd = Py_SIZE(argdefs);
    }
    else {
        d = NULL;
        nd = 0;
    }
#if PY_MAJOR_VERSION >= 3
    result = PyEval_EvalCodeEx((PyObject*)co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, kwdefs, closure);
#else
    result = PyEval_EvalCodeEx(co, globals, (PyObject *)NULL,
                               args, (int)nargs,
                               k, (int)nk,
                               d, (int)nd, closure);
#endif
    Py_XDECREF(kwtuple);
done:
    Py_LeaveRecursiveCall();
    return result;
}
#endif
#endif

/* PyObjectCall */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_Call(PyObject *func, PyObject *arg, PyObject *kw) {
    PyObject *result;
    ternaryfunc call = Py_TYPE(func)->tp_call;
    if (unlikely(!call))
        return PyObject_Call(func, arg, kw);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = (*call)(func, arg, kw);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallMethO */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallMethO(PyObject *func, PyObject *arg) {
    PyObject *self, *result;
    PyCFunction cfunc;
    cfunc = PyCFunction_GET_FUNCTION(func);
    self = PyCFunction_GET_SELF(func);
    if (unlikely(Py_EnterRecursiveCall((char*)" while calling a Python object")))
        return NULL;
    result = cfunc(self, arg);
    Py_LeaveRecursiveCall();
    if (unlikely(!result) && unlikely(!PyErr_Occurred())) {
        PyErr_SetString(
            PyExc_SystemError,
            "NULL result without error in PyObject_Call");
    }
    return result;
}
#endif

/* PyObjectCallNoArg */
#if CYTHON_COMPILING_IN_CPYTHON
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallNoArg(PyObject *func) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, NULL, 0);
    }
#endif
#ifdef __Pyx_CyFunction_USED
    if (likely(PyCFunction_Check(func) || __Pyx_CyFunction_Check(func)))
#else
    if (likely(PyCFunction_Check(func)))
#endif
    {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_NOARGS)) {
            return __Pyx_PyObject_CallMethO(func, NULL);
        }
    }
    return __Pyx_PyObject_Call(func, __pyx_empty_tuple, NULL);
}
#endif

/* PyCFunctionFastCall */
#if CYTHON_FAST_PYCCALL
static CYTHON_INLINE PyObject * __Pyx_PyCFunction_FastCall(PyObject *func_obj, PyObject **args, Py_ssize_t nargs) {
    PyCFunctionObject *func = (PyCFunctionObject*)func_obj;
    PyCFunction meth = PyCFunction_GET_FUNCTION(func);
    PyObject *self = PyCFunction_GET_SELF(func);
    int flags = PyCFunction_GET_FLAGS(func);
    assert(PyCFunction_Check(func));
    assert(METH_FASTCALL == (flags & ~(METH_CLASS | METH_STATIC | METH_COEXIST | METH_KEYWORDS | METH_STACKLESS)));
    assert(nargs >= 0);
    assert(nargs == 0 || args != NULL);
    /* _PyCFunction_FastCallDict() must not be called with an exception set,
       because it may clear it (directly or indirectly) and so the
       caller loses its exception */
    assert(!PyErr_Occurred());
    if ((PY_VERSION_HEX < 0x030700A0) || unlikely(flags & METH_KEYWORDS)) {
        return (*((__Pyx_PyCFunctionFastWithKeywords)(void*)meth)) (self, args, nargs, NULL);
    } else {
        return (*((__Pyx_PyCFunctionFast)(void*)meth)) (self, args, nargs);
    }
}
#endif

/* PyObjectCallOneArg */
#if CYTHON_COMPILING_IN_CPYTHON
static PyObject* __Pyx__PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_New(1);
    if (unlikely(!args)) return NULL;
    Py_INCREF(arg);
    PyTuple_SET_ITEM(args, 0, arg);
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
#if CYTHON_FAST_PYCALL
    if (PyFunction_Check(func)) {
        return __Pyx_PyFunction_FastCall(func, &arg, 1);
    }
#endif
    if (likely(PyCFunction_Check(func))) {
        if (likely(PyCFunction_GET_FLAGS(func) & METH_O)) {
            return __Pyx_PyObject_CallMethO(func, arg);
#if CYTHON_FAST_PYCCALL
        } else if (__Pyx_PyFastCFunction_Check(func)) {
            return __Pyx_PyCFunction_FastCall(func, &arg, 1);
#endif
        }
    }
    return __Pyx__PyObject_CallOneArg(func, arg);
}
#else
static CYTHON_INLINE PyObject* __Pyx_PyObject_CallOneArg(PyObject *func, PyObject *arg) {
    PyObject *result;
    PyObject *args = PyTuple_Pack(1, arg);
    if (unlikely(!args)) return NULL;
    result = __Pyx_PyObject_Call(func, args, NULL);
    Py_DECREF(args);
    return result;
}
#endif

/* ExtTypeTest */
static CYTHON_INLINE int __Pyx_TypeTest(PyObject *obj, PyTypeObject *type) {
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    if (likely(__Pyx_TypeCheck(obj, type)))
        return 1;
    PyErr_Format(PyExc_TypeError, "Cannot convert %.200s to %.200s",
                 Py_TYPE(obj)->tp_name, type->tp_name);
    return 0;
}

/* PyErrExceptionMatches */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx_PyErr_ExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        if (__Pyx_PyErr_GivenExceptionMatches(exc_type, PyTuple_GET_ITEM(tuple, i))) return 1;
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_ExceptionMatchesInState(PyThreadState* tstate, PyObject* err) {
    PyObject *exc_type = tstate->curexc_type;
    if (exc_type == err) return 1;
    if (unlikely(!exc_type)) return 0;
    if (unlikely(PyTuple_Check(err)))
        return __Pyx_PyErr_ExceptionMatchesTuple(exc_type, err);
    return __Pyx_PyErr_GivenExceptionMatches(exc_type, err);
}
#endif

/* PyErrFetchRestore */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx_ErrRestoreInState(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    tmp_type = tstate->curexc_type;
    tmp_value = tstate->curexc_value;
    tmp_tb = tstate->curexc_traceback;
    tstate->curexc_type = type;
    tstate->curexc_value = value;
    tstate->curexc_traceback = tb;
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
static CYTHON_INLINE void __Pyx_ErrFetchInState(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    *type = tstate->curexc_type;
    *value = tstate->curexc_value;
    *tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
}
#endif

/* GetAttr */
static CYTHON_INLINE PyObject *__Pyx_GetAttr(PyObject *o, PyObject *n) {
#if CYTHON_USE_TYPE_SLOTS
#if PY_MAJOR_VERSION >= 3
    if (likely(PyUnicode_Check(n)))
#else
    if (likely(PyString_Check(n)))
#endif
        return __Pyx_PyObject_GetAttrStr(o, n);
#endif
    return PyObject_GetAttr(o, n);
}

/* GetAttr3 */
static PyObject *__Pyx_GetAttr3Default(PyObject *d) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (unlikely(!__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        return NULL;
    __Pyx_PyErr_Clear();
    Py_INCREF(d);
    return d;
}
static CYTHON_INLINE PyObject *__Pyx_GetAttr3(PyObject *o, PyObject *n, PyObject *d) {
    PyObject *r = __Pyx_GetAttr(o, n);
    return (likely(r)) ? r : __Pyx_GetAttr3Default(d);
}

/* GetModuleGlobalName */
#if CYTHON_USE_DICT_VERSIONS
static PyObject *__Pyx__GetModuleGlobalName(PyObject *name, PY_UINT64_T *dict_version, PyObject **dict_cached_value)
#else
static CYTHON_INLINE PyObject *__Pyx__GetModuleGlobalName(PyObject *name)
#endif
{
    PyObject *result;
#if !CYTHON_AVOID_BORROWED_REFS
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x030500A1
    result = _PyDict_GetItem_KnownHash(__pyx_d, name, ((PyASCIIObject *) name)->hash);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    } else if (unlikely(PyErr_Occurred())) {
        return NULL;
    }
#else
    result = PyDict_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
#endif
#else
    result = PyObject_GetItem(__pyx_d, name);
    __PYX_UPDATE_DICT_CACHE(__pyx_d, result, *dict_cached_value, *dict_version)
    if (likely(result)) {
        return __Pyx_NewRef(result);
    }
    PyErr_Clear();
#endif
    return __Pyx_GetBuiltinName(name);
}

/* RaiseDoubleKeywords */
static void __Pyx_RaiseDoubleKeywordsError(
    const char* func_name,
    PyObject* kw_name)
{
    PyErr_Format(PyExc_TypeError,
        #if PY_MAJOR_VERSION >= 3
        "%s() got multiple values for keyword argument '%U'", func_name, kw_name);
        #else
        "%s() got multiple values for keyword argument '%s'", func_name,
        PyString_AsString(kw_name));
        #endif
}

/* ParseKeywords */
static int __Pyx_ParseOptionalKeywords(
    PyObject *kwds,
    PyObject **argnames[],
    PyObject *kwds2,
    PyObject *values[],
    Py_ssize_t num_pos_args,
    const char* function_name)
{
    PyObject *key = 0, *value = 0;
    Py_ssize_t pos = 0;
    PyObject*** name;
    PyObject*** first_kw_arg = argnames + num_pos_args;
    while (PyDict_Next(kwds, &pos, &key, &value)) {
        name = first_kw_arg;
        while (*name && (**name != key)) name++;
        if (*name) {
            values[name-argnames] = value;
            continue;
        }
        name = first_kw_arg;
        #if PY_MAJOR_VERSION < 3
        if (likely(PyString_Check(key))) {
            while (*name) {
                if ((CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**name) == PyString_GET_SIZE(key))
                        && _PyString_Eq(**name, key)) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    if ((**argname == key) || (
                            (CYTHON_COMPILING_IN_PYPY || PyString_GET_SIZE(**argname) == PyString_GET_SIZE(key))
                             && _PyString_Eq(**argname, key))) {
                        goto arg_passed_twice;
                    }
                    argname++;
                }
            }
        } else
        #endif
        if (likely(PyUnicode_Check(key))) {
            while (*name) {
                int cmp = (**name == key) ? 0 :
                #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                    (__Pyx_PyUnicode_GET_LENGTH(**name) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                #endif
                    PyUnicode_Compare(**name, key);
                if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                if (cmp == 0) {
                    values[name-argnames] = value;
                    break;
                }
                name++;
            }
            if (*name) continue;
            else {
                PyObject*** argname = argnames;
                while (argname != first_kw_arg) {
                    int cmp = (**argname == key) ? 0 :
                    #if !CYTHON_COMPILING_IN_PYPY && PY_MAJOR_VERSION >= 3
                        (__Pyx_PyUnicode_GET_LENGTH(**argname) != __Pyx_PyUnicode_GET_LENGTH(key)) ? 1 :
                    #endif
                        PyUnicode_Compare(**argname, key);
                    if (cmp < 0 && unlikely(PyErr_Occurred())) goto bad;
                    if (cmp == 0) goto arg_passed_twice;
                    argname++;
                }
            }
        } else
            goto invalid_keyword_type;
        if (kwds2) {
            if (unlikely(PyDict_SetItem(kwds2, key, value))) goto bad;
        } else {
            goto invalid_keyword;
        }
    }
    return 0;
arg_passed_twice:
    __Pyx_RaiseDoubleKeywordsError(function_name, key);
    goto bad;
invalid_keyword_type:
    PyErr_Format(PyExc_TypeError,
        "%.200s() keywords must be strings", function_name);
    goto bad;
invalid_keyword:
    PyErr_Format(PyExc_TypeError,
    #if PY_MAJOR_VERSION < 3
        "%.200s() got an unexpected keyword argument '%.200s'",
        function_name, PyString_AsString(key));
    #else
        "%s() got an unexpected keyword argument '%U'",
        function_name, key);
    #endif
bad:
    return -1;
}

/* RaiseArgTupleInvalid */
static void __Pyx_RaiseArgtupleInvalid(
    const char* func_name,
    int exact,
    Py_ssize_t num_min,
    Py_ssize_t num_max,
    Py_ssize_t num_found)
{
    Py_ssize_t num_expected;
    const char *more_or_less;
    if (num_found < num_min) {
        num_expected = num_min;
        more_or_less = "at least";
    } else {
        num_expected = num_max;
        more_or_less = "at most";
    }
    if (exact) {
        more_or_less = "exactly";
    }
    PyErr_Format(PyExc_TypeError,
                 "%.200s() takes %.8s %" CYTHON_FORMAT_SSIZE_T "d positional argument%.1s (%" CYTHON_FORMAT_SSIZE_T "d given)",
                 func_name, more_or_less, num_expected,
                 (num_expected == 1) ? "" : "s", num_found);
}

/* PyObjectSetAttrStr */
#if CYTHON_USE_TYPE_SLOTS
static CYTHON_INLINE int __Pyx_PyObject_SetAttrStr(PyObject* obj, PyObject* attr_name, PyObject* value) {
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_setattro))
        return tp->tp_setattro(obj, attr_name, value);
#if PY_MAJOR_VERSION < 3
    if (likely(tp->tp_setattr))
        return tp->tp_setattr(obj, PyString_AS_STRING(attr_name), value);
#endif
    return PyObject_SetAttr(obj, attr_name, value);
}
#endif

/* ArgTypeTest */
static int __Pyx__ArgTypeTest(PyObject *obj, PyTypeObject *type, const char *name, int exact)
{
    if (unlikely(!type)) {
        PyErr_SetString(PyExc_SystemError, "Missing type object");
        return 0;
    }
    else if (exact) {
        #if PY_MAJOR_VERSION == 2
        if ((type == &PyBaseString_Type) && likely(__Pyx_PyBaseString_CheckExact(obj))) return 1;
        #endif
    }
    else {
        if (likely(__Pyx_TypeCheck(obj, type))) return 1;
    }
    PyErr_Format(PyExc_TypeError,
        "Argument '%.200s' has incorrect type (expected %.200s, got %.200s)",
        name, type->tp_name, Py_TYPE(obj)->tp_name);
    return 0;
}

/* PyObjectCall2Args */
static CYTHON_UNUSED PyObject* __Pyx_PyObject_Call2Args(PyObject* function, PyObject* arg1, PyObject* arg2) {
    PyObject *args, *result = NULL;
    #if CYTHON_FAST_PYCALL
    if (PyFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyFunction_FastCall(function, args, 2);
    }
    #endif
    #if CYTHON_FAST_PYCCALL
    if (__Pyx_PyFastCFunction_Check(function)) {
        PyObject *args[2] = {arg1, arg2};
        return __Pyx_PyCFunction_FastCall(function, args, 2);
    }
    #endif
    args = PyTuple_New(2);
    if (unlikely(!args)) goto done;
    Py_INCREF(arg1);
    PyTuple_SET_ITEM(args, 0, arg1);
    Py_INCREF(arg2);
    PyTuple_SET_ITEM(args, 1, arg2);
    Py_INCREF(function);
    result = __Pyx_PyObject_Call(function, args, NULL);
    Py_DECREF(args);
    Py_DECREF(function);
done:
    return result;
}

/* GetTopmostException */
#if CYTHON_USE_EXC_INFO_STACK
static _PyErr_StackItem *
__Pyx_PyErr_GetTopmostException(PyThreadState *tstate)
{
    _PyErr_StackItem *exc_info = tstate->exc_info;
    while ((exc_info->exc_type == NULL || exc_info->exc_type == Py_None) &&
           exc_info->previous_item != NULL)
    {
        exc_info = exc_info->previous_item;
    }
    return exc_info;
}
#endif

/* SaveResetException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSave(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = __Pyx_PyErr_GetTopmostException(tstate);
    *type = exc_info->exc_type;
    *value = exc_info->exc_value;
    *tb = exc_info->exc_traceback;
    #else
    *type = tstate->exc_type;
    *value = tstate->exc_value;
    *tb = tstate->exc_traceback;
    #endif
    Py_XINCREF(*type);
    Py_XINCREF(*value);
    Py_XINCREF(*tb);
}
static CYTHON_INLINE void __Pyx__ExceptionReset(PyThreadState *tstate, PyObject *type, PyObject *value, PyObject *tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = type;
    exc_info->exc_value = value;
    exc_info->exc_traceback = tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = type;
    tstate->exc_value = value;
    tstate->exc_traceback = tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
}
#endif

/* GetException */
#if CYTHON_FAST_THREAD_STATE
static int __Pyx__GetException(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb)
#else
static int __Pyx_GetException(PyObject **type, PyObject **value, PyObject **tb)
#endif
{
    PyObject *local_type, *local_value, *local_tb;
#if CYTHON_FAST_THREAD_STATE
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    local_type = tstate->curexc_type;
    local_value = tstate->curexc_value;
    local_tb = tstate->curexc_traceback;
    tstate->curexc_type = 0;
    tstate->curexc_value = 0;
    tstate->curexc_traceback = 0;
#else
    PyErr_Fetch(&local_type, &local_value, &local_tb);
#endif
    PyErr_NormalizeException(&local_type, &local_value, &local_tb);
#if CYTHON_FAST_THREAD_STATE
    if (unlikely(tstate->curexc_type))
#else
    if (unlikely(PyErr_Occurred()))
#endif
        goto bad;
    #if PY_MAJOR_VERSION >= 3
    if (local_tb) {
        if (unlikely(PyException_SetTraceback(local_value, local_tb) < 0))
            goto bad;
    }
    #endif
    Py_XINCREF(local_tb);
    Py_XINCREF(local_type);
    Py_XINCREF(local_value);
    *type = local_type;
    *value = local_value;
    *tb = local_tb;
#if CYTHON_FAST_THREAD_STATE
    #if CYTHON_USE_EXC_INFO_STACK
    {
        _PyErr_StackItem *exc_info = tstate->exc_info;
        tmp_type = exc_info->exc_type;
        tmp_value = exc_info->exc_value;
        tmp_tb = exc_info->exc_traceback;
        exc_info->exc_type = local_type;
        exc_info->exc_value = local_value;
        exc_info->exc_traceback = local_tb;
    }
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = local_type;
    tstate->exc_value = local_value;
    tstate->exc_traceback = local_tb;
    #endif
    Py_XDECREF(tmp_type);
    Py_XDECREF(tmp_value);
    Py_XDECREF(tmp_tb);
#else
    PyErr_SetExcInfo(local_type, local_value, local_tb);
#endif
    return 0;
bad:
    *type = 0;
    *value = 0;
    *tb = 0;
    Py_XDECREF(local_type);
    Py_XDECREF(local_value);
    Py_XDECREF(local_tb);
    return -1;
}

/* Import */
static PyObject *__Pyx_Import(PyObject *name, PyObject *from_list, int level) {
    PyObject *empty_list = 0;
    PyObject *module = 0;
    PyObject *global_dict = 0;
    PyObject *empty_dict = 0;
    PyObject *list;
    #if PY_MAJOR_VERSION < 3
    PyObject *py_import;
    py_import = __Pyx_PyObject_GetAttrStr(__pyx_b, __pyx_n_s_import);
    if (!py_import)
        goto bad;
    #endif
    if (from_list)
        list = from_list;
    else {
        empty_list = PyList_New(0);
        if (!empty_list)
            goto bad;
        list = empty_list;
    }
    global_dict = PyModule_GetDict(__pyx_m);
    if (!global_dict)
        goto bad;
    empty_dict = PyDict_New();
    if (!empty_dict)
        goto bad;
    {
        #if PY_MAJOR_VERSION >= 3
        if (level == -1) {
            if ((1) && (strchr(__Pyx_MODULE_NAME, '.'))) {
                module = PyImport_ImportModuleLevelObject(
                    name, global_dict, empty_dict, list, 1);
                if (!module) {
                    if (!PyErr_ExceptionMatches(PyExc_ImportError))
                        goto bad;
                    PyErr_Clear();
                }
            }
            level = 0;
        }
        #endif
        if (!module) {
            #if PY_MAJOR_VERSION < 3
            PyObject *py_level = PyInt_FromLong(level);
            if (!py_level)
                goto bad;
            module = PyObject_CallFunctionObjArgs(py_import,
                name, global_dict, empty_dict, list, py_level, (PyObject *)NULL);
            Py_DECREF(py_level);
            #else
            module = PyImport_ImportModuleLevelObject(
                name, global_dict, empty_dict, list, level);
            #endif
        }
    }
bad:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(py_import);
    #endif
    Py_XDECREF(empty_list);
    Py_XDECREF(empty_dict);
    return module;
}

/* ImportFrom */
static PyObject* __Pyx_ImportFrom(PyObject* module, PyObject* name) {
    PyObject* value = __Pyx_PyObject_GetAttrStr(module, name);
    if (unlikely(!value) && PyErr_ExceptionMatches(PyExc_AttributeError)) {
        PyErr_Format(PyExc_ImportError,
        #if PY_MAJOR_VERSION < 3
            "cannot import name %.230s", PyString_AS_STRING(name));
        #else
            "cannot import name %S", name);
        #endif
    }
    return value;
}

/* WriteUnraisableException */
static void __Pyx_WriteUnraisable(const char *name, CYTHON_UNUSED int clineno,
                                  CYTHON_UNUSED int lineno, CYTHON_UNUSED const char *filename,
                                  int full_traceback, CYTHON_UNUSED int nogil) {
    PyObject *old_exc, *old_val, *old_tb;
    PyObject *ctx;
    __Pyx_PyThreadState_declare
#ifdef WITH_THREAD
    PyGILState_STATE state;
    if (nogil)
        state = PyGILState_Ensure();
#ifdef _MSC_VER
    else state = (PyGILState_STATE)-1;
#endif
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&old_exc, &old_val, &old_tb);
    if (full_traceback) {
        Py_XINCREF(old_exc);
        Py_XINCREF(old_val);
        Py_XINCREF(old_tb);
        __Pyx_ErrRestore(old_exc, old_val, old_tb);
        PyErr_PrintEx(1);
    }
    #if PY_MAJOR_VERSION < 3
    ctx = PyString_FromString(name);
    #else
    ctx = PyUnicode_FromString(name);
    #endif
    __Pyx_ErrRestore(old_exc, old_val, old_tb);
    if (!ctx) {
        PyErr_WriteUnraisable(Py_None);
    } else {
        PyErr_WriteUnraisable(ctx);
        Py_DECREF(ctx);
    }
#ifdef WITH_THREAD
    if (nogil)
        PyGILState_Release(state);
#endif
}

/* BytesEquals */
static CYTHON_INLINE int __Pyx_PyBytes_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
    if (s1 == s2) {
        return (equals == Py_EQ);
    } else if (PyBytes_CheckExact(s1) & PyBytes_CheckExact(s2)) {
        const char *ps1, *ps2;
        Py_ssize_t length = PyBytes_GET_SIZE(s1);
        if (length != PyBytes_GET_SIZE(s2))
            return (equals == Py_NE);
        ps1 = PyBytes_AS_STRING(s1);
        ps2 = PyBytes_AS_STRING(s2);
        if (ps1[0] != ps2[0]) {
            return (equals == Py_NE);
        } else if (length == 1) {
            return (equals == Py_EQ);
        } else {
            int result;
#if CYTHON_USE_UNICODE_INTERNALS
            Py_hash_t hash1, hash2;
            hash1 = ((PyBytesObject*)s1)->ob_shash;
            hash2 = ((PyBytesObject*)s2)->ob_shash;
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                return (equals == Py_NE);
            }
#endif
            result = memcmp(ps1, ps2, (size_t)length);
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & PyBytes_CheckExact(s2)) {
        return (equals == Py_NE);
    } else if ((s2 == Py_None) & PyBytes_CheckExact(s1)) {
        return (equals == Py_NE);
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
#endif
}

/* UnicodeEquals */
static CYTHON_INLINE int __Pyx_PyUnicode_Equals(PyObject* s1, PyObject* s2, int equals) {
#if CYTHON_COMPILING_IN_PYPY
    return PyObject_RichCompareBool(s1, s2, equals);
#else
#if PY_MAJOR_VERSION < 3
    PyObject* owned_ref = NULL;
#endif
    int s1_is_unicode, s2_is_unicode;
    if (s1 == s2) {
        goto return_eq;
    }
    s1_is_unicode = PyUnicode_CheckExact(s1);
    s2_is_unicode = PyUnicode_CheckExact(s2);
#if PY_MAJOR_VERSION < 3
    if ((s1_is_unicode & (!s2_is_unicode)) && PyString_CheckExact(s2)) {
        owned_ref = PyUnicode_FromObject(s2);
        if (unlikely(!owned_ref))
            return -1;
        s2 = owned_ref;
        s2_is_unicode = 1;
    } else if ((s2_is_unicode & (!s1_is_unicode)) && PyString_CheckExact(s1)) {
        owned_ref = PyUnicode_FromObject(s1);
        if (unlikely(!owned_ref))
            return -1;
        s1 = owned_ref;
        s1_is_unicode = 1;
    } else if (((!s2_is_unicode) & (!s1_is_unicode))) {
        return __Pyx_PyBytes_Equals(s1, s2, equals);
    }
#endif
    if (s1_is_unicode & s2_is_unicode) {
        Py_ssize_t length;
        int kind;
        void *data1, *data2;
        if (unlikely(__Pyx_PyUnicode_READY(s1) < 0) || unlikely(__Pyx_PyUnicode_READY(s2) < 0))
            return -1;
        length = __Pyx_PyUnicode_GET_LENGTH(s1);
        if (length != __Pyx_PyUnicode_GET_LENGTH(s2)) {
            goto return_ne;
        }
#if CYTHON_USE_UNICODE_INTERNALS
        {
            Py_hash_t hash1, hash2;
        #if CYTHON_PEP393_ENABLED
            hash1 = ((PyASCIIObject*)s1)->hash;
            hash2 = ((PyASCIIObject*)s2)->hash;
        #else
            hash1 = ((PyUnicodeObject*)s1)->hash;
            hash2 = ((PyUnicodeObject*)s2)->hash;
        #endif
            if (hash1 != hash2 && hash1 != -1 && hash2 != -1) {
                goto return_ne;
            }
        }
#endif
        kind = __Pyx_PyUnicode_KIND(s1);
        if (kind != __Pyx_PyUnicode_KIND(s2)) {
            goto return_ne;
        }
        data1 = __Pyx_PyUnicode_DATA(s1);
        data2 = __Pyx_PyUnicode_DATA(s2);
        if (__Pyx_PyUnicode_READ(kind, data1, 0) != __Pyx_PyUnicode_READ(kind, data2, 0)) {
            goto return_ne;
        } else if (length == 1) {
            goto return_eq;
        } else {
            int result = memcmp(data1, data2, (size_t)(length * kind));
            #if PY_MAJOR_VERSION < 3
            Py_XDECREF(owned_ref);
            #endif
            return (equals == Py_EQ) ? (result == 0) : (result != 0);
        }
    } else if ((s1 == Py_None) & s2_is_unicode) {
        goto return_ne;
    } else if ((s2 == Py_None) & s1_is_unicode) {
        goto return_ne;
    } else {
        int result;
        PyObject* py_result = PyObject_RichCompare(s1, s2, equals);
        #if PY_MAJOR_VERSION < 3
        Py_XDECREF(owned_ref);
        #endif
        if (!py_result)
            return -1;
        result = __Pyx_PyObject_IsTrue(py_result);
        Py_DECREF(py_result);
        return result;
    }
return_eq:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_EQ);
return_ne:
    #if PY_MAJOR_VERSION < 3
    Py_XDECREF(owned_ref);
    #endif
    return (equals == Py_NE);
#endif
}

/* PyIntCompare */
static CYTHON_INLINE PyObject* __Pyx_PyInt_EqObjC(PyObject *op1, PyObject *op2, CYTHON_UNUSED long intval, CYTHON_UNUSED long inplace) {
    if (op1 == op2) {
        Py_RETURN_TRUE;
    }
    #if PY_MAJOR_VERSION < 3
    if (likely(PyInt_CheckExact(op1))) {
        const long b = intval;
        long a = PyInt_AS_LONG(op1);
        if (a == b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    #if CYTHON_USE_PYLONG_INTERNALS
    if (likely(PyLong_CheckExact(op1))) {
        int unequal;
        unsigned long uintval;
        Py_ssize_t size = Py_SIZE(op1);
        const digit* digits = ((PyLongObject*)op1)->ob_digit;
        if (intval == 0) {
            if (size == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
        } else if (intval < 0) {
            if (size >= 0)
                Py_RETURN_FALSE;
            intval = -intval;
            size = -size;
        } else {
            if (size <= 0)
                Py_RETURN_FALSE;
        }
        uintval = (unsigned long) intval;
#if PyLong_SHIFT * 4 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 4)) {
            unequal = (size != 5) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[4] != ((uintval >> (4 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 3 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 3)) {
            unequal = (size != 4) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[3] != ((uintval >> (3 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 2 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 2)) {
            unequal = (size != 3) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK)) | (digits[2] != ((uintval >> (2 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
#if PyLong_SHIFT * 1 < SIZEOF_LONG*8
        if (uintval >> (PyLong_SHIFT * 1)) {
            unequal = (size != 2) || (digits[0] != (uintval & (unsigned long) PyLong_MASK))
                 | (digits[1] != ((uintval >> (1 * PyLong_SHIFT)) & (unsigned long) PyLong_MASK));
        } else
#endif
            unequal = (size != 1) || (((unsigned long) digits[0]) != (uintval & (unsigned long) PyLong_MASK));
        if (unequal == 0) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    #endif
    if (PyFloat_CheckExact(op1)) {
        const long b = intval;
        double a = PyFloat_AS_DOUBLE(op1);
        if ((double)a == (double)b) Py_RETURN_TRUE; else Py_RETURN_FALSE;
    }
    return (
        PyObject_RichCompare(op1, op2, Py_EQ));
}

/* RaiseException */
#if PY_MAJOR_VERSION < 3
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb,
                        CYTHON_UNUSED PyObject *cause) {
    __Pyx_PyThreadState_declare
    Py_XINCREF(type);
    if (!value || value == Py_None)
        value = NULL;
    else
        Py_INCREF(value);
    if (!tb || tb == Py_None)
        tb = NULL;
    else {
        Py_INCREF(tb);
        if (!PyTraceBack_Check(tb)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: arg 3 must be a traceback or None");
            goto raise_error;
        }
    }
    if (PyType_Check(type)) {
#if CYTHON_COMPILING_IN_PYPY
        if (!value) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#endif
        PyErr_NormalizeException(&type, &value, &tb);
    } else {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto raise_error;
        }
        value = type;
        type = (PyObject*) Py_TYPE(type);
        Py_INCREF(type);
        if (!PyType_IsSubtype((PyTypeObject *)type, (PyTypeObject *)PyExc_BaseException)) {
            PyErr_SetString(PyExc_TypeError,
                "raise: exception class must be a subclass of BaseException");
            goto raise_error;
        }
    }
    __Pyx_PyThreadState_assign
    __Pyx_ErrRestore(type, value, tb);
    return;
raise_error:
    Py_XDECREF(value);
    Py_XDECREF(type);
    Py_XDECREF(tb);
    return;
}
#else
static void __Pyx_Raise(PyObject *type, PyObject *value, PyObject *tb, PyObject *cause) {
    PyObject* owned_instance = NULL;
    if (tb == Py_None) {
        tb = 0;
    } else if (tb && !PyTraceBack_Check(tb)) {
        PyErr_SetString(PyExc_TypeError,
            "raise: arg 3 must be a traceback or None");
        goto bad;
    }
    if (value == Py_None)
        value = 0;
    if (PyExceptionInstance_Check(type)) {
        if (value) {
            PyErr_SetString(PyExc_TypeError,
                "instance exception may not have a separate value");
            goto bad;
        }
        value = type;
        type = (PyObject*) Py_TYPE(value);
    } else if (PyExceptionClass_Check(type)) {
        PyObject *instance_class = NULL;
        if (value && PyExceptionInstance_Check(value)) {
            instance_class = (PyObject*) Py_TYPE(value);
            if (instance_class != type) {
                int is_subclass = PyObject_IsSubclass(instance_class, type);
                if (!is_subclass) {
                    instance_class = NULL;
                } else if (unlikely(is_subclass == -1)) {
                    goto bad;
                } else {
                    type = instance_class;
                }
            }
        }
        if (!instance_class) {
            PyObject *args;
            if (!value)
                args = PyTuple_New(0);
            else if (PyTuple_Check(value)) {
                Py_INCREF(value);
                args = value;
            } else
                args = PyTuple_Pack(1, value);
            if (!args)
                goto bad;
            owned_instance = PyObject_Call(type, args, NULL);
            Py_DECREF(args);
            if (!owned_instance)
                goto bad;
            value = owned_instance;
            if (!PyExceptionInstance_Check(value)) {
                PyErr_Format(PyExc_TypeError,
                             "calling %R should have returned an instance of "
                             "BaseException, not %R",
                             type, Py_TYPE(value));
                goto bad;
            }
        }
    } else {
        PyErr_SetString(PyExc_TypeError,
            "raise: exception class must be a subclass of BaseException");
        goto bad;
    }
    if (cause) {
        PyObject *fixed_cause;
        if (cause == Py_None) {
            fixed_cause = NULL;
        } else if (PyExceptionClass_Check(cause)) {
            fixed_cause = PyObject_CallObject(cause, NULL);
            if (fixed_cause == NULL)
                goto bad;
        } else if (PyExceptionInstance_Check(cause)) {
            fixed_cause = cause;
            Py_INCREF(fixed_cause);
        } else {
            PyErr_SetString(PyExc_TypeError,
                            "exception causes must derive from "
                            "BaseException");
            goto bad;
        }
        PyException_SetCause(value, fixed_cause);
    }
    PyErr_SetObject(type, value);
    if (tb) {
#if CYTHON_COMPILING_IN_PYPY
        PyObject *tmp_type, *tmp_value, *tmp_tb;
        PyErr_Fetch(&tmp_type, &tmp_value, &tmp_tb);
        Py_INCREF(tb);
        PyErr_Restore(tmp_type, tmp_value, tb);
        Py_XDECREF(tmp_tb);
#else
        PyThreadState *tstate = __Pyx_PyThreadState_Current;
        PyObject* tmp_tb = tstate->curexc_traceback;
        if (tb != tmp_tb) {
            Py_INCREF(tb);
            tstate->curexc_traceback = tb;
            Py_XDECREF(tmp_tb);
        }
#endif
    }
bad:
    Py_XDECREF(owned_instance);
    return;
}
#endif

/* PyObjectGetMethod */
static int __Pyx_PyObject_GetMethod(PyObject *obj, PyObject *name, PyObject **method) {
    PyObject *attr;
#if CYTHON_UNPACK_METHODS && CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_PYTYPE_LOOKUP
    PyTypeObject *tp = Py_TYPE(obj);
    PyObject *descr;
    descrgetfunc f = NULL;
    PyObject **dictptr, *dict;
    int meth_found = 0;
    assert (*method == NULL);
    if (unlikely(tp->tp_getattro != PyObject_GenericGetAttr)) {
        attr = __Pyx_PyObject_GetAttrStr(obj, name);
        goto try_unpack;
    }
    if (unlikely(tp->tp_dict == NULL) && unlikely(PyType_Ready(tp) < 0)) {
        return 0;
    }
    descr = _PyType_Lookup(tp, name);
    if (likely(descr != NULL)) {
        Py_INCREF(descr);
#if PY_MAJOR_VERSION >= 3
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr) || (Py_TYPE(descr) == &PyMethodDescr_Type)))
        #endif
#else
        #ifdef __Pyx_CyFunction_USED
        if (likely(PyFunction_Check(descr) || __Pyx_CyFunction_Check(descr)))
        #else
        if (likely(PyFunction_Check(descr)))
        #endif
#endif
        {
            meth_found = 1;
        } else {
            f = Py_TYPE(descr)->tp_descr_get;
            if (f != NULL && PyDescr_IsData(descr)) {
                attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
                Py_DECREF(descr);
                goto try_unpack;
            }
        }
    }
    dictptr = _PyObject_GetDictPtr(obj);
    if (dictptr != NULL && (dict = *dictptr) != NULL) {
        Py_INCREF(dict);
        attr = __Pyx_PyDict_GetItemStr(dict, name);
        if (attr != NULL) {
            Py_INCREF(attr);
            Py_DECREF(dict);
            Py_XDECREF(descr);
            goto try_unpack;
        }
        Py_DECREF(dict);
    }
    if (meth_found) {
        *method = descr;
        return 1;
    }
    if (f != NULL) {
        attr = f(descr, obj, (PyObject *)Py_TYPE(obj));
        Py_DECREF(descr);
        goto try_unpack;
    }
    if (descr != NULL) {
        *method = descr;
        return 0;
    }
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(name));
#endif
    return 0;
#else
    attr = __Pyx_PyObject_GetAttrStr(obj, name);
    goto try_unpack;
#endif
try_unpack:
#if CYTHON_UNPACK_METHODS
    if (likely(attr) && PyMethod_Check(attr) && likely(PyMethod_GET_SELF(attr) == obj)) {
        PyObject *function = PyMethod_GET_FUNCTION(attr);
        Py_INCREF(function);
        Py_DECREF(attr);
        *method = function;
        return 1;
    }
#endif
    *method = attr;
    return 0;
}

/* PyObjectCallMethod1 */
static PyObject* __Pyx__PyObject_CallMethod1(PyObject* method, PyObject* arg) {
    PyObject *result = __Pyx_PyObject_CallOneArg(method, arg);
    Py_DECREF(method);
    return result;
}
static PyObject* __Pyx_PyObject_CallMethod1(PyObject* obj, PyObject* method_name, PyObject* arg) {
    PyObject *method = NULL, *result;
    int is_method = __Pyx_PyObject_GetMethod(obj, method_name, &method);
    if (likely(is_method)) {
        result = __Pyx_PyObject_Call2Args(method, obj, arg);
        Py_DECREF(method);
        return result;
    }
    if (unlikely(!method)) return NULL;
    return __Pyx__PyObject_CallMethod1(method, arg);
}

/* pop_index */
static PyObject* __Pyx__PyObject_PopNewIndex(PyObject* L, PyObject* py_ix) {
    PyObject *r;
    if (unlikely(!py_ix)) return NULL;
    r = __Pyx__PyObject_PopIndex(L, py_ix);
    Py_DECREF(py_ix);
    return r;
}
static PyObject* __Pyx__PyObject_PopIndex(PyObject* L, PyObject* py_ix) {
    return __Pyx_PyObject_CallMethod1(L, __pyx_n_s_pop, py_ix);
}
#if CYTHON_USE_PYLIST_INTERNALS && CYTHON_ASSUME_SAFE_MACROS
static PyObject* __Pyx__PyList_PopIndex(PyObject* L, PyObject* py_ix, Py_ssize_t ix) {
    Py_ssize_t size = PyList_GET_SIZE(L);
    if (likely(size > (((PyListObject*)L)->allocated >> 1))) {
        Py_ssize_t cix = ix;
        if (cix < 0) {
            cix += size;
        }
        if (likely(__Pyx_is_valid_index(cix, size))) {
            PyObject* v = PyList_GET_ITEM(L, cix);
            __Pyx_SET_SIZE(L, Py_SIZE(L) - 1);
            size -= 1;
            memmove(&PyList_GET_ITEM(L, cix), &PyList_GET_ITEM(L, cix+1), (size_t)(size-cix)*sizeof(PyObject*));
            return v;
        }
    }
    if (py_ix == Py_None) {
        return __Pyx__PyObject_PopNewIndex(L, PyInt_FromSsize_t(ix));
    } else {
        return __Pyx__PyObject_PopIndex(L, py_ix);
    }
}
#endif

/* GetItemInt */
static PyObject *__Pyx_GetItemInt_Generic(PyObject *o, PyObject* j) {
    PyObject *r;
    if (!j) return NULL;
    r = PyObject_GetItem(o, j);
    Py_DECREF(j);
    return r;
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_List_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyList_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyList_GET_SIZE(o)))) {
        PyObject *r = PyList_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Tuple_Fast(PyObject *o, Py_ssize_t i,
                                                              CYTHON_NCP_UNUSED int wraparound,
                                                              CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
    Py_ssize_t wrapped_i = i;
    if (wraparound & unlikely(i < 0)) {
        wrapped_i += PyTuple_GET_SIZE(o);
    }
    if ((!boundscheck) || likely(__Pyx_is_valid_index(wrapped_i, PyTuple_GET_SIZE(o)))) {
        PyObject *r = PyTuple_GET_ITEM(o, wrapped_i);
        Py_INCREF(r);
        return r;
    }
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
#else
    return PySequence_GetItem(o, i);
#endif
}
static CYTHON_INLINE PyObject *__Pyx_GetItemInt_Fast(PyObject *o, Py_ssize_t i, int is_list,
                                                     CYTHON_NCP_UNUSED int wraparound,
                                                     CYTHON_NCP_UNUSED int boundscheck) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS && CYTHON_USE_TYPE_SLOTS
    if (is_list || PyList_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyList_GET_SIZE(o);
        if ((!boundscheck) || (likely(__Pyx_is_valid_index(n, PyList_GET_SIZE(o))))) {
            PyObject *r = PyList_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    }
    else if (PyTuple_CheckExact(o)) {
        Py_ssize_t n = ((!wraparound) | likely(i >= 0)) ? i : i + PyTuple_GET_SIZE(o);
        if ((!boundscheck) || likely(__Pyx_is_valid_index(n, PyTuple_GET_SIZE(o)))) {
            PyObject *r = PyTuple_GET_ITEM(o, n);
            Py_INCREF(r);
            return r;
        }
    } else {
        PySequenceMethods *m = Py_TYPE(o)->tp_as_sequence;
        if (likely(m && m->sq_item)) {
            if (wraparound && unlikely(i < 0) && likely(m->sq_length)) {
                Py_ssize_t l = m->sq_length(o);
                if (likely(l >= 0)) {
                    i += l;
                } else {
                    if (!PyErr_ExceptionMatches(PyExc_OverflowError))
                        return NULL;
                    PyErr_Clear();
                }
            }
            return m->sq_item(o, i);
        }
    }
#else
    if (is_list || PySequence_Check(o)) {
        return PySequence_GetItem(o, i);
    }
#endif
    return __Pyx_GetItemInt_Generic(o, PyInt_FromSsize_t(i));
}

/* HasAttr */
static CYTHON_INLINE int __Pyx_HasAttr(PyObject *o, PyObject *n) {
    PyObject *r;
    if (unlikely(!__Pyx_PyBaseString_Check(n))) {
        PyErr_SetString(PyExc_TypeError,
                        "hasattr(): attribute name must be string");
        return -1;
    }
    r = __Pyx_GetAttr(o, n);
    if (unlikely(!r)) {
        PyErr_Clear();
        return 0;
    } else {
        Py_DECREF(r);
        return 1;
    }
}

/* PyObject_GenericGetAttrNoDict */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject *__Pyx_RaiseGenericGetAttributeError(PyTypeObject *tp, PyObject *attr_name) {
    PyErr_Format(PyExc_AttributeError,
#if PY_MAJOR_VERSION >= 3
                 "'%.50s' object has no attribute '%U'",
                 tp->tp_name, attr_name);
#else
                 "'%.50s' object has no attribute '%.400s'",
                 tp->tp_name, PyString_AS_STRING(attr_name));
#endif
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GenericGetAttrNoDict(PyObject* obj, PyObject* attr_name) {
    PyObject *descr;
    PyTypeObject *tp = Py_TYPE(obj);
    if (unlikely(!PyString_Check(attr_name))) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    assert(!tp->tp_dictoffset);
    descr = _PyType_Lookup(tp, attr_name);
    if (unlikely(!descr)) {
        return __Pyx_RaiseGenericGetAttributeError(tp, attr_name);
    }
    Py_INCREF(descr);
    #if PY_MAJOR_VERSION < 3
    if (likely(PyType_HasFeature(Py_TYPE(descr), Py_TPFLAGS_HAVE_CLASS)))
    #endif
    {
        descrgetfunc f = Py_TYPE(descr)->tp_descr_get;
        if (unlikely(f)) {
            PyObject *res = f(descr, obj, (PyObject *)tp);
            Py_DECREF(descr);
            return res;
        }
    }
    return descr;
}
#endif

/* PyObject_GenericGetAttr */
#if CYTHON_USE_TYPE_SLOTS && CYTHON_USE_PYTYPE_LOOKUP && PY_VERSION_HEX < 0x03070000
static PyObject* __Pyx_PyObject_GenericGetAttr(PyObject* obj, PyObject* attr_name) {
    if (unlikely(Py_TYPE(obj)->tp_dictoffset)) {
        return PyObject_GenericGetAttr(obj, attr_name);
    }
    return __Pyx_PyObject_GenericGetAttrNoDict(obj, attr_name);
}
#endif

/* SetVTable */
static int __Pyx_SetVtable(PyObject *dict, void *vtable) {
#if PY_VERSION_HEX >= 0x02070000
    PyObject *ob = PyCapsule_New(vtable, 0, 0);
#else
    PyObject *ob = PyCObject_FromVoidPtr(vtable, 0);
#endif
    if (!ob)
        goto bad;
    if (PyDict_SetItem(dict, __pyx_n_s_pyx_vtable, ob) < 0)
        goto bad;
    Py_DECREF(ob);
    return 0;
bad:
    Py_XDECREF(ob);
    return -1;
}

/* PyObjectGetAttrStrNoError */
static void __Pyx_PyObject_GetAttrStr_ClearAttributeError(void) {
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    if (likely(__Pyx_PyErr_ExceptionMatches(PyExc_AttributeError)))
        __Pyx_PyErr_Clear();
}
static CYTHON_INLINE PyObject* __Pyx_PyObject_GetAttrStrNoError(PyObject* obj, PyObject* attr_name) {
    PyObject *result;
#if CYTHON_COMPILING_IN_CPYTHON && CYTHON_USE_TYPE_SLOTS && PY_VERSION_HEX >= 0x030700B1
    PyTypeObject* tp = Py_TYPE(obj);
    if (likely(tp->tp_getattro == PyObject_GenericGetAttr)) {
        return _PyObject_GenericGetAttrWithDict(obj, attr_name, NULL, 1);
    }
#endif
    result = __Pyx_PyObject_GetAttrStr(obj, attr_name);
    if (unlikely(!result)) {
        __Pyx_PyObject_GetAttrStr_ClearAttributeError();
    }
    return result;
}

/* SetupReduce */
static int __Pyx_setup_reduce_is_named(PyObject* meth, PyObject* name) {
  int ret;
  PyObject *name_attr;
  name_attr = __Pyx_PyObject_GetAttrStr(meth, __pyx_n_s_name_2);
  if (likely(name_attr)) {
      ret = PyObject_RichCompareBool(name_attr, name, Py_EQ);
  } else {
      ret = -1;
  }
  if (unlikely(ret < 0)) {
      PyErr_Clear();
      ret = 0;
  }
  Py_XDECREF(name_attr);
  return ret;
}
static int __Pyx_setup_reduce(PyObject* type_obj) {
    int ret = 0;
    PyObject *object_reduce = NULL;
    PyObject *object_reduce_ex = NULL;
    PyObject *reduce = NULL;
    PyObject *reduce_ex = NULL;
    PyObject *reduce_cython = NULL;
    PyObject *setstate = NULL;
    PyObject *setstate_cython = NULL;
#if CYTHON_USE_PYTYPE_LOOKUP
    if (_PyType_Lookup((PyTypeObject*)type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#else
    if (PyObject_HasAttr(type_obj, __pyx_n_s_getstate)) goto __PYX_GOOD;
#endif
#if CYTHON_USE_PYTYPE_LOOKUP
    object_reduce_ex = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#else
    object_reduce_ex = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce_ex); if (!object_reduce_ex) goto __PYX_BAD;
#endif
    reduce_ex = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce_ex); if (unlikely(!reduce_ex)) goto __PYX_BAD;
    if (reduce_ex == object_reduce_ex) {
#if CYTHON_USE_PYTYPE_LOOKUP
        object_reduce = _PyType_Lookup(&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#else
        object_reduce = __Pyx_PyObject_GetAttrStr((PyObject*)&PyBaseObject_Type, __pyx_n_s_reduce); if (!object_reduce) goto __PYX_BAD;
#endif
        reduce = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_reduce); if (unlikely(!reduce)) goto __PYX_BAD;
        if (reduce == object_reduce || __Pyx_setup_reduce_is_named(reduce, __pyx_n_s_reduce_cython)) {
            reduce_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_reduce_cython);
            if (likely(reduce_cython)) {
                ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce, reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_reduce_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
            } else if (reduce == object_reduce || PyErr_Occurred()) {
                goto __PYX_BAD;
            }
            setstate = __Pyx_PyObject_GetAttrStr(type_obj, __pyx_n_s_setstate);
            if (!setstate) PyErr_Clear();
            if (!setstate || __Pyx_setup_reduce_is_named(setstate, __pyx_n_s_setstate_cython)) {
                setstate_cython = __Pyx_PyObject_GetAttrStrNoError(type_obj, __pyx_n_s_setstate_cython);
                if (likely(setstate_cython)) {
                    ret = PyDict_SetItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate, setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                    ret = PyDict_DelItem(((PyTypeObject*)type_obj)->tp_dict, __pyx_n_s_setstate_cython); if (unlikely(ret < 0)) goto __PYX_BAD;
                } else if (!setstate || PyErr_Occurred()) {
                    goto __PYX_BAD;
                }
            }
            PyType_Modified((PyTypeObject*)type_obj);
        }
    }
    goto __PYX_GOOD;
__PYX_BAD:
    if (!PyErr_Occurred())
        PyErr_Format(PyExc_RuntimeError, "Unable to initialize pickling for %s", ((PyTypeObject*)type_obj)->tp_name);
    ret = -1;
__PYX_GOOD:
#if !CYTHON_USE_PYTYPE_LOOKUP
    Py_XDECREF(object_reduce);
    Py_XDECREF(object_reduce_ex);
#endif
    Py_XDECREF(reduce);
    Py_XDECREF(reduce_ex);
    Py_XDECREF(reduce_cython);
    Py_XDECREF(setstate);
    Py_XDECREF(setstate_cython);
    return ret;
}

/* TypeImport */
#ifndef __PYX_HAVE_RT_ImportType
#define __PYX_HAVE_RT_ImportType
static PyTypeObject *__Pyx_ImportType(PyObject *module, const char *module_name, const char *class_name,
    size_t size, enum __Pyx_ImportType_CheckSize check_size)
{
    PyObject *result = 0;
    char warning[200];
    Py_ssize_t basicsize;
#ifdef Py_LIMITED_API
    PyObject *py_basicsize;
#endif
    result = PyObject_GetAttrString(module, class_name);
    if (!result)
        goto bad;
    if (!PyType_Check(result)) {
        PyErr_Format(PyExc_TypeError,
            "%.200s.%.200s is not a type object",
            module_name, class_name);
        goto bad;
    }
#ifndef Py_LIMITED_API
    basicsize = ((PyTypeObject *)result)->tp_basicsize;
#else
    py_basicsize = PyObject_GetAttrString(result, "__basicsize__");
    if (!py_basicsize)
        goto bad;
    basicsize = PyLong_AsSsize_t(py_basicsize);
    Py_DECREF(py_basicsize);
    py_basicsize = 0;
    if (basicsize == (Py_ssize_t)-1 && PyErr_Occurred())
        goto bad;
#endif
    if ((size_t)basicsize < size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    if (check_size == __Pyx_ImportType_CheckSize_Error && (size_t)basicsize != size) {
        PyErr_Format(PyExc_ValueError,
            "%.200s.%.200s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        goto bad;
    }
    else if (check_size == __Pyx_ImportType_CheckSize_Warn && (size_t)basicsize > size) {
        PyOS_snprintf(warning, sizeof(warning),
            "%s.%s size changed, may indicate binary incompatibility. "
            "Expected %zd from C header, got %zd from PyObject",
            module_name, class_name, size, basicsize);
        if (PyErr_WarnEx(NULL, warning, 0) < 0) goto bad;
    }
    return (PyTypeObject *)result;
bad:
    Py_XDECREF(result);
    return NULL;
}
#endif

/* GetVTable */
static void* __Pyx_GetVtable(PyObject *dict) {
    void* ptr;
    PyObject *ob = PyObject_GetItem(dict, __pyx_n_s_pyx_vtable);
    if (!ob)
        goto bad;
#if PY_VERSION_HEX >= 0x02070000
    ptr = PyCapsule_GetPointer(ob, 0);
#else
    ptr = PyCObject_AsVoidPtr(ob);
#endif
    if (!ptr && !PyErr_Occurred())
        PyErr_SetString(PyExc_RuntimeError, "invalid vtable found for imported type");
    Py_DECREF(ob);
    return ptr;
bad:
    Py_XDECREF(ob);
    return NULL;
}

/* CLineInTraceback */
#ifndef CYTHON_CLINE_IN_TRACEBACK
static int __Pyx_CLineForTraceback(CYTHON_NCP_UNUSED PyThreadState *tstate, int c_line) {
    PyObject *use_cline;
    PyObject *ptype, *pvalue, *ptraceback;
#if CYTHON_COMPILING_IN_CPYTHON
    PyObject **cython_runtime_dict;
#endif
    if (unlikely(!__pyx_cython_runtime)) {
        return c_line;
    }
    __Pyx_ErrFetchInState(tstate, &ptype, &pvalue, &ptraceback);
#if CYTHON_COMPILING_IN_CPYTHON
    cython_runtime_dict = _PyObject_GetDictPtr(__pyx_cython_runtime);
    if (likely(cython_runtime_dict)) {
        __PYX_PY_DICT_LOOKUP_IF_MODIFIED(
            use_cline, *cython_runtime_dict,
            __Pyx_PyDict_GetItemStr(*cython_runtime_dict, __pyx_n_s_cline_in_traceback))
    } else
#endif
    {
      PyObject *use_cline_obj = __Pyx_PyObject_GetAttrStr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback);
      if (use_cline_obj) {
        use_cline = PyObject_Not(use_cline_obj) ? Py_False : Py_True;
        Py_DECREF(use_cline_obj);
      } else {
        PyErr_Clear();
        use_cline = NULL;
      }
    }
    if (!use_cline) {
        c_line = 0;
        PyObject_SetAttr(__pyx_cython_runtime, __pyx_n_s_cline_in_traceback, Py_False);
    }
    else if (use_cline == Py_False || (use_cline != Py_True && PyObject_Not(use_cline) != 0)) {
        c_line = 0;
    }
    __Pyx_ErrRestoreInState(tstate, ptype, pvalue, ptraceback);
    return c_line;
}
#endif

/* CodeObjectCache */
static int __pyx_bisect_code_objects(__Pyx_CodeObjectCacheEntry* entries, int count, int code_line) {
    int start = 0, mid = 0, end = count - 1;
    if (end >= 0 && code_line > entries[end].code_line) {
        return count;
    }
    while (start < end) {
        mid = start + (end - start) / 2;
        if (code_line < entries[mid].code_line) {
            end = mid;
        } else if (code_line > entries[mid].code_line) {
             start = mid + 1;
        } else {
            return mid;
        }
    }
    if (code_line <= entries[mid].code_line) {
        return mid;
    } else {
        return mid + 1;
    }
}
static PyCodeObject *__pyx_find_code_object(int code_line) {
    PyCodeObject* code_object;
    int pos;
    if (unlikely(!code_line) || unlikely(!__pyx_code_cache.entries)) {
        return NULL;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if (unlikely(pos >= __pyx_code_cache.count) || unlikely(__pyx_code_cache.entries[pos].code_line != code_line)) {
        return NULL;
    }
    code_object = __pyx_code_cache.entries[pos].code_object;
    Py_INCREF(code_object);
    return code_object;
}
static void __pyx_insert_code_object(int code_line, PyCodeObject* code_object) {
    int pos, i;
    __Pyx_CodeObjectCacheEntry* entries = __pyx_code_cache.entries;
    if (unlikely(!code_line)) {
        return;
    }
    if (unlikely(!entries)) {
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Malloc(64*sizeof(__Pyx_CodeObjectCacheEntry));
        if (likely(entries)) {
            __pyx_code_cache.entries = entries;
            __pyx_code_cache.max_count = 64;
            __pyx_code_cache.count = 1;
            entries[0].code_line = code_line;
            entries[0].code_object = code_object;
            Py_INCREF(code_object);
        }
        return;
    }
    pos = __pyx_bisect_code_objects(__pyx_code_cache.entries, __pyx_code_cache.count, code_line);
    if ((pos < __pyx_code_cache.count) && unlikely(__pyx_code_cache.entries[pos].code_line == code_line)) {
        PyCodeObject* tmp = entries[pos].code_object;
        entries[pos].code_object = code_object;
        Py_DECREF(tmp);
        return;
    }
    if (__pyx_code_cache.count == __pyx_code_cache.max_count) {
        int new_max = __pyx_code_cache.max_count + 64;
        entries = (__Pyx_CodeObjectCacheEntry*)PyMem_Realloc(
            __pyx_code_cache.entries, ((size_t)new_max) * sizeof(__Pyx_CodeObjectCacheEntry));
        if (unlikely(!entries)) {
            return;
        }
        __pyx_code_cache.entries = entries;
        __pyx_code_cache.max_count = new_max;
    }
    for (i=__pyx_code_cache.count; i>pos; i--) {
        entries[i] = entries[i-1];
    }
    entries[pos].code_line = code_line;
    entries[pos].code_object = code_object;
    __pyx_code_cache.count++;
    Py_INCREF(code_object);
}

/* AddTraceback */
#include "compile.h"
#include "frameobject.h"
#include "traceback.h"
static PyCodeObject* __Pyx_CreateCodeObjectForTraceback(
            const char *funcname, int c_line,
            int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyObject *py_srcfile = 0;
    PyObject *py_funcname = 0;
    #if PY_MAJOR_VERSION < 3
    py_srcfile = PyString_FromString(filename);
    #else
    py_srcfile = PyUnicode_FromString(filename);
    #endif
    if (!py_srcfile) goto bad;
    if (c_line) {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #else
        py_funcname = PyUnicode_FromFormat( "%s (%s:%d)", funcname, __pyx_cfilenm, c_line);
        #endif
    }
    else {
        #if PY_MAJOR_VERSION < 3
        py_funcname = PyString_FromString(funcname);
        #else
        py_funcname = PyUnicode_FromString(funcname);
        #endif
    }
    if (!py_funcname) goto bad;
    py_code = __Pyx_PyCode_New(
        0,
        0,
        0,
        0,
        0,
        __pyx_empty_bytes, /*PyObject *code,*/
        __pyx_empty_tuple, /*PyObject *consts,*/
        __pyx_empty_tuple, /*PyObject *names,*/
        __pyx_empty_tuple, /*PyObject *varnames,*/
        __pyx_empty_tuple, /*PyObject *freevars,*/
        __pyx_empty_tuple, /*PyObject *cellvars,*/
        py_srcfile,   /*PyObject *filename,*/
        py_funcname,  /*PyObject *name,*/
        py_line,
        __pyx_empty_bytes  /*PyObject *lnotab*/
    );
    Py_DECREF(py_srcfile);
    Py_DECREF(py_funcname);
    return py_code;
bad:
    Py_XDECREF(py_srcfile);
    Py_XDECREF(py_funcname);
    return NULL;
}
static void __Pyx_AddTraceback(const char *funcname, int c_line,
                               int py_line, const char *filename) {
    PyCodeObject *py_code = 0;
    PyFrameObject *py_frame = 0;
    PyThreadState *tstate = __Pyx_PyThreadState_Current;
    if (c_line) {
        c_line = __Pyx_CLineForTraceback(tstate, c_line);
    }
    py_code = __pyx_find_code_object(c_line ? -c_line : py_line);
    if (!py_code) {
        py_code = __Pyx_CreateCodeObjectForTraceback(
            funcname, c_line, py_line, filename);
        if (!py_code) goto bad;
        __pyx_insert_code_object(c_line ? -c_line : py_line, py_code);
    }
    py_frame = PyFrame_New(
        tstate,            /*PyThreadState *tstate,*/
        py_code,           /*PyCodeObject *code,*/
        __pyx_d,    /*PyObject *globals,*/
        0                  /*PyObject *locals*/
    );
    if (!py_frame) goto bad;
    __Pyx_PyFrame_SetLineNumber(py_frame, py_line);
    PyTraceBack_Here(py_frame);
bad:
    Py_XDECREF(py_code);
    Py_XDECREF(py_frame);
}

/* CIntFromPyVerify */
#define __PYX_VERIFY_RETURN_INT(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 0)
#define __PYX_VERIFY_RETURN_INT_EXC(target_type, func_type, func_value)\
    __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, 1)
#define __PYX__VERIFY_RETURN_INT(target_type, func_type, func_value, exc)\
    {\
        func_type value = func_value;\
        if (sizeof(target_type) < sizeof(func_type)) {\
            if (unlikely(value != (func_type) (target_type) value)) {\
                func_type zero = 0;\
                if (exc && unlikely(value == (func_type)-1 && PyErr_Occurred()))\
                    return (target_type) -1;\
                if (is_unsigned && unlikely(value < zero))\
                    goto raise_neg_overflow;\
                else\
                    goto raise_overflow;\
            }\
        }\
        return (target_type) value;\
    }

/* CIntFromPy */
static CYTHON_INLINE long __Pyx_PyInt_As_long(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(long) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(long, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (long) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case  1: __PYX_VERIFY_RETURN_INT(long, digit, digits[0])
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 2 * PyLong_SHIFT) {
                            return (long) (((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 3 * PyLong_SHIFT) {
                            return (long) (((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) >= 4 * PyLong_SHIFT) {
                            return (long) (((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (long) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(long) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (long) 0;
                case -1: __PYX_VERIFY_RETURN_INT(long, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(long,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(long) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(long) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                            return (long) ((((((long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(long) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(long) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                            return (long) ((((((((long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(long) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) (((long)-1)*(((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(long) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(long, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(long) - 1 > 4 * PyLong_SHIFT) {
                            return (long) ((((((((((long)digits[3]) << PyLong_SHIFT) | (long)digits[2]) << PyLong_SHIFT) | (long)digits[1]) << PyLong_SHIFT) | (long)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(long) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(long, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            long val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (long) -1;
        }
    } else {
        long val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (long) -1;
        val = __Pyx_PyInt_As_long(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to long");
    return (long) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to long");
    return (long) -1;
}

/* CIntFromPy */
static CYTHON_INLINE size_t __Pyx_PyInt_As_size_t(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const size_t neg_one = (size_t) -1, const_zero = (size_t) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(size_t) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(size_t, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (size_t) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case  1: __PYX_VERIFY_RETURN_INT(size_t, digit, digits[0])
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 2 * PyLong_SHIFT) {
                            return (size_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 3 * PyLong_SHIFT) {
                            return (size_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) >= 4 * PyLong_SHIFT) {
                            return (size_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (size_t) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(size_t) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (size_t) 0;
                case -1: __PYX_VERIFY_RETURN_INT(size_t, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(size_t,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(size_t) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(size_t) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                            return (size_t) ((((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(size_t) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(size_t) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                            return (size_t) ((((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(size_t) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) (((size_t)-1)*(((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(size_t) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(size_t, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(size_t) - 1 > 4 * PyLong_SHIFT) {
                            return (size_t) ((((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(size_t) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(size_t) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(size_t, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            size_t val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (size_t) -1;
        }
    } else {
        size_t val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (size_t) -1;
        val = __Pyx_PyInt_As_size_t(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to size_t");
    return (size_t) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to size_t");
    return (size_t) -1;
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_int(int value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(int) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(int) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(int) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(int),
                                     little, !is_unsigned);
    }
}

/* CIntToPy */
static CYTHON_INLINE PyObject* __Pyx_PyInt_From_long(long value) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const long neg_one = (long) -1, const_zero = (long) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
    if (is_unsigned) {
        if (sizeof(long) < sizeof(long)) {
            return PyInt_FromLong((long) value);
        } else if (sizeof(long) <= sizeof(unsigned long)) {
            return PyLong_FromUnsignedLong((unsigned long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(unsigned PY_LONG_LONG)) {
            return PyLong_FromUnsignedLongLong((unsigned PY_LONG_LONG) value);
#endif
        }
    } else {
        if (sizeof(long) <= sizeof(long)) {
            return PyInt_FromLong((long) value);
#ifdef HAVE_LONG_LONG
        } else if (sizeof(long) <= sizeof(PY_LONG_LONG)) {
            return PyLong_FromLongLong((PY_LONG_LONG) value);
#endif
        }
    }
    {
        int one = 1; int little = (int)*(unsigned char *)&one;
        unsigned char *bytes = (unsigned char *)&value;
        return _PyLong_FromByteArray(bytes, sizeof(long),
                                     little, !is_unsigned);
    }
}

/* CIntFromPy */
static CYTHON_INLINE int __Pyx_PyInt_As_int(PyObject *x) {
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic push
#pragma GCC diagnostic ignored "-Wconversion"
#endif
    const int neg_one = (int) -1, const_zero = (int) 0;
#ifdef __Pyx_HAS_GCC_DIAGNOSTIC
#pragma GCC diagnostic pop
#endif
    const int is_unsigned = neg_one > const_zero;
#if PY_MAJOR_VERSION < 3
    if (likely(PyInt_Check(x))) {
        if (sizeof(int) < sizeof(long)) {
            __PYX_VERIFY_RETURN_INT(int, long, PyInt_AS_LONG(x))
        } else {
            long val = PyInt_AS_LONG(x);
            if (is_unsigned && unlikely(val < 0)) {
                goto raise_neg_overflow;
            }
            return (int) val;
        }
    } else
#endif
    if (likely(PyLong_Check(x))) {
        if (is_unsigned) {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case  1: __PYX_VERIFY_RETURN_INT(int, digit, digits[0])
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 2 * PyLong_SHIFT) {
                            return (int) (((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 3 * PyLong_SHIFT) {
                            return (int) (((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) >= 4 * PyLong_SHIFT) {
                            return (int) (((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0]));
                        }
                    }
                    break;
            }
#endif
#if CYTHON_COMPILING_IN_CPYTHON
            if (unlikely(Py_SIZE(x) < 0)) {
                goto raise_neg_overflow;
            }
#else
            {
                int result = PyObject_RichCompareBool(x, Py_False, Py_LT);
                if (unlikely(result < 0))
                    return (int) -1;
                if (unlikely(result == 1))
                    goto raise_neg_overflow;
            }
#endif
            if (sizeof(int) <= sizeof(unsigned long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned long, PyLong_AsUnsignedLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(unsigned PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, unsigned PY_LONG_LONG, PyLong_AsUnsignedLongLong(x))
#endif
            }
        } else {
#if CYTHON_USE_PYLONG_INTERNALS
            const digit* digits = ((PyLongObject*)x)->ob_digit;
            switch (Py_SIZE(x)) {
                case  0: return (int) 0;
                case -1: __PYX_VERIFY_RETURN_INT(int, sdigit, (sdigit) (-(sdigit)digits[0]))
                case  1: __PYX_VERIFY_RETURN_INT(int,  digit, +digits[0])
                case -2:
                    if (8 * sizeof(int) - 1 > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 2:
                    if (8 * sizeof(int) > 1 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 2 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                            return (int) ((((((int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -3:
                    if (8 * sizeof(int) - 1 > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 3:
                    if (8 * sizeof(int) > 2 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 3 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                            return (int) ((((((((int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case -4:
                    if (8 * sizeof(int) - 1 > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, long, -(long) (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) (((int)-1)*(((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
                case 4:
                    if (8 * sizeof(int) > 3 * PyLong_SHIFT) {
                        if (8 * sizeof(unsigned long) > 4 * PyLong_SHIFT) {
                            __PYX_VERIFY_RETURN_INT(int, unsigned long, (((((((((unsigned long)digits[3]) << PyLong_SHIFT) | (unsigned long)digits[2]) << PyLong_SHIFT) | (unsigned long)digits[1]) << PyLong_SHIFT) | (unsigned long)digits[0])))
                        } else if (8 * sizeof(int) - 1 > 4 * PyLong_SHIFT) {
                            return (int) ((((((((((int)digits[3]) << PyLong_SHIFT) | (int)digits[2]) << PyLong_SHIFT) | (int)digits[1]) << PyLong_SHIFT) | (int)digits[0])));
                        }
                    }
                    break;
            }
#endif
            if (sizeof(int) <= sizeof(long)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, long, PyLong_AsLong(x))
#ifdef HAVE_LONG_LONG
            } else if (sizeof(int) <= sizeof(PY_LONG_LONG)) {
                __PYX_VERIFY_RETURN_INT_EXC(int, PY_LONG_LONG, PyLong_AsLongLong(x))
#endif
            }
        }
        {
#if CYTHON_COMPILING_IN_PYPY && !defined(_PyLong_AsByteArray)
            PyErr_SetString(PyExc_RuntimeError,
                            "_PyLong_AsByteArray() not available in PyPy, cannot convert large numbers");
#else
            int val;
            PyObject *v = __Pyx_PyNumber_IntOrLong(x);
 #if PY_MAJOR_VERSION < 3
            if (likely(v) && !PyLong_Check(v)) {
                PyObject *tmp = v;
                v = PyNumber_Long(tmp);
                Py_DECREF(tmp);
            }
 #endif
            if (likely(v)) {
                int one = 1; int is_little = (int)*(unsigned char *)&one;
                unsigned char *bytes = (unsigned char *)&val;
                int ret = _PyLong_AsByteArray((PyLongObject *)v,
                                              bytes, sizeof(val),
                                              is_little, !is_unsigned);
                Py_DECREF(v);
                if (likely(!ret))
                    return val;
            }
#endif
            return (int) -1;
        }
    } else {
        int val;
        PyObject *tmp = __Pyx_PyNumber_IntOrLong(x);
        if (!tmp) return (int) -1;
        val = __Pyx_PyInt_As_int(tmp);
        Py_DECREF(tmp);
        return val;
    }
raise_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "value too large to convert to int");
    return (int) -1;
raise_neg_overflow:
    PyErr_SetString(PyExc_OverflowError,
        "can't convert negative value to int");
    return (int) -1;
}

/* FastTypeChecks */
#if CYTHON_COMPILING_IN_CPYTHON
static int __Pyx_InBases(PyTypeObject *a, PyTypeObject *b) {
    while (a) {
        a = a->tp_base;
        if (a == b)
            return 1;
    }
    return b == &PyBaseObject_Type;
}
static CYTHON_INLINE int __Pyx_IsSubtype(PyTypeObject *a, PyTypeObject *b) {
    PyObject *mro;
    if (a == b) return 1;
    mro = a->tp_mro;
    if (likely(mro)) {
        Py_ssize_t i, n;
        n = PyTuple_GET_SIZE(mro);
        for (i = 0; i < n; i++) {
            if (PyTuple_GET_ITEM(mro, i) == (PyObject *)b)
                return 1;
        }
        return 0;
    }
    return __Pyx_InBases(a, b);
}
#if PY_MAJOR_VERSION == 2
static int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject* exc_type2) {
    PyObject *exception, *value, *tb;
    int res;
    __Pyx_PyThreadState_declare
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&exception, &value, &tb);
    res = exc_type1 ? PyObject_IsSubclass(err, exc_type1) : 0;
    if (unlikely(res == -1)) {
        PyErr_WriteUnraisable(err);
        res = 0;
    }
    if (!res) {
        res = PyObject_IsSubclass(err, exc_type2);
        if (unlikely(res == -1)) {
            PyErr_WriteUnraisable(err);
            res = 0;
        }
    }
    __Pyx_ErrRestore(exception, value, tb);
    return res;
}
#else
static CYTHON_INLINE int __Pyx_inner_PyErr_GivenExceptionMatches2(PyObject *err, PyObject* exc_type1, PyObject *exc_type2) {
    int res = exc_type1 ? __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type1) : 0;
    if (!res) {
        res = __Pyx_IsSubtype((PyTypeObject*)err, (PyTypeObject*)exc_type2);
    }
    return res;
}
#endif
static int __Pyx_PyErr_GivenExceptionMatchesTuple(PyObject *exc_type, PyObject *tuple) {
    Py_ssize_t i, n;
    assert(PyExceptionClass_Check(exc_type));
    n = PyTuple_GET_SIZE(tuple);
#if PY_MAJOR_VERSION >= 3
    for (i=0; i<n; i++) {
        if (exc_type == PyTuple_GET_ITEM(tuple, i)) return 1;
    }
#endif
    for (i=0; i<n; i++) {
        PyObject *t = PyTuple_GET_ITEM(tuple, i);
        #if PY_MAJOR_VERSION < 3
        if (likely(exc_type == t)) return 1;
        #endif
        if (likely(PyExceptionClass_Check(t))) {
            if (__Pyx_inner_PyErr_GivenExceptionMatches2(exc_type, NULL, t)) return 1;
        } else {
        }
    }
    return 0;
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches(PyObject *err, PyObject* exc_type) {
    if (likely(err == exc_type)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        if (likely(PyExceptionClass_Check(exc_type))) {
            return __Pyx_inner_PyErr_GivenExceptionMatches2(err, NULL, exc_type);
        } else if (likely(PyTuple_Check(exc_type))) {
            return __Pyx_PyErr_GivenExceptionMatchesTuple(err, exc_type);
        } else {
        }
    }
    return PyErr_GivenExceptionMatches(err, exc_type);
}
static CYTHON_INLINE int __Pyx_PyErr_GivenExceptionMatches2(PyObject *err, PyObject *exc_type1, PyObject *exc_type2) {
    assert(PyExceptionClass_Check(exc_type1));
    assert(PyExceptionClass_Check(exc_type2));
    if (likely(err == exc_type1 || err == exc_type2)) return 1;
    if (likely(PyExceptionClass_Check(err))) {
        return __Pyx_inner_PyErr_GivenExceptionMatches2(err, exc_type1, exc_type2);
    }
    return (PyErr_GivenExceptionMatches(err, exc_type1) || PyErr_GivenExceptionMatches(err, exc_type2));
}
#endif

/* FetchCommonType */
static PyTypeObject* __Pyx_FetchCommonType(PyTypeObject* type) {
    PyObject* fake_module;
    PyTypeObject* cached_type = NULL;
    fake_module = PyImport_AddModule((char*) "_cython_" CYTHON_ABI);
    if (!fake_module) return NULL;
    Py_INCREF(fake_module);
    cached_type = (PyTypeObject*) PyObject_GetAttrString(fake_module, type->tp_name);
    if (cached_type) {
        if (!PyType_Check((PyObject*)cached_type)) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s is not a type object",
                type->tp_name);
            goto bad;
        }
        if (cached_type->tp_basicsize != type->tp_basicsize) {
            PyErr_Format(PyExc_TypeError,
                "Shared Cython type %.200s has the wrong size, try recompiling",
                type->tp_name);
            goto bad;
        }
    } else {
        if (!PyErr_ExceptionMatches(PyExc_AttributeError)) goto bad;
        PyErr_Clear();
        if (PyType_Ready(type) < 0) goto bad;
        if (PyObject_SetAttrString(fake_module, type->tp_name, (PyObject*) type) < 0)
            goto bad;
        Py_INCREF(type);
        cached_type = type;
    }
done:
    Py_DECREF(fake_module);
    return cached_type;
bad:
    Py_XDECREF(cached_type);
    cached_type = NULL;
    goto done;
}

/* SwapException */
#if CYTHON_FAST_THREAD_STATE
static CYTHON_INLINE void __Pyx__ExceptionSwap(PyThreadState *tstate, PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    #if CYTHON_USE_EXC_INFO_STACK
    _PyErr_StackItem *exc_info = tstate->exc_info;
    tmp_type = exc_info->exc_type;
    tmp_value = exc_info->exc_value;
    tmp_tb = exc_info->exc_traceback;
    exc_info->exc_type = *type;
    exc_info->exc_value = *value;
    exc_info->exc_traceback = *tb;
    #else
    tmp_type = tstate->exc_type;
    tmp_value = tstate->exc_value;
    tmp_tb = tstate->exc_traceback;
    tstate->exc_type = *type;
    tstate->exc_value = *value;
    tstate->exc_traceback = *tb;
    #endif
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#else
static CYTHON_INLINE void __Pyx_ExceptionSwap(PyObject **type, PyObject **value, PyObject **tb) {
    PyObject *tmp_type, *tmp_value, *tmp_tb;
    PyErr_GetExcInfo(&tmp_type, &tmp_value, &tmp_tb);
    PyErr_SetExcInfo(*type, *value, *tb);
    *type = tmp_type;
    *value = tmp_value;
    *tb = tmp_tb;
}
#endif

/* CoroutineBase */
#include <structmember.h>
#include <frameobject.h>
#define __Pyx_Coroutine_Undelegate(gen) Py_CLEAR((gen)->yieldfrom)
static int __Pyx_PyGen__FetchStopIterationValue(CYTHON_UNUSED PyThreadState *__pyx_tstate, PyObject **pvalue) {
    PyObject *et, *ev, *tb;
    PyObject *value = NULL;
    __Pyx_ErrFetch(&et, &ev, &tb);
    if (!et) {
        Py_XDECREF(tb);
        Py_XDECREF(ev);
        Py_INCREF(Py_None);
        *pvalue = Py_None;
        return 0;
    }
    if (likely(et == PyExc_StopIteration)) {
        if (!ev) {
            Py_INCREF(Py_None);
            value = Py_None;
        }
#if PY_VERSION_HEX >= 0x030300A0
        else if (Py_TYPE(ev) == (PyTypeObject*)PyExc_StopIteration) {
            value = ((PyStopIterationObject *)ev)->value;
            Py_INCREF(value);
            Py_DECREF(ev);
        }
#endif
        else if (unlikely(PyTuple_Check(ev))) {
            if (PyTuple_GET_SIZE(ev) >= 1) {
#if CYTHON_ASSUME_SAFE_MACROS && !CYTHON_AVOID_BORROWED_REFS
                value = PyTuple_GET_ITEM(ev, 0);
                Py_INCREF(value);
#else
                value = PySequence_ITEM(ev, 0);
#endif
            } else {
                Py_INCREF(Py_None);
                value = Py_None;
            }
            Py_DECREF(ev);
        }
        else if (!__Pyx_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration)) {
            value = ev;
        }
        if (likely(value)) {
            Py_XDECREF(tb);
            Py_DECREF(et);
            *pvalue = value;
            return 0;
        }
    } else if (!__Pyx_PyErr_GivenExceptionMatches(et, PyExc_StopIteration)) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    PyErr_NormalizeException(&et, &ev, &tb);
    if (unlikely(!PyObject_TypeCheck(ev, (PyTypeObject*)PyExc_StopIteration))) {
        __Pyx_ErrRestore(et, ev, tb);
        return -1;
    }
    Py_XDECREF(tb);
    Py_DECREF(et);
#if PY_VERSION_HEX >= 0x030300A0
    value = ((PyStopIterationObject *)ev)->value;
    Py_INCREF(value);
    Py_DECREF(ev);
#else
    {
        PyObject* args = __Pyx_PyObject_GetAttrStr(ev, __pyx_n_s_args);
        Py_DECREF(ev);
        if (likely(args)) {
            value = PySequence_GetItem(args, 0);
            Py_DECREF(args);
        }
        if (unlikely(!value)) {
            __Pyx_ErrRestore(NULL, NULL, NULL);
            Py_INCREF(Py_None);
            value = Py_None;
        }
    }
#endif
    *pvalue = value;
    return 0;
}
static CYTHON_INLINE
void __Pyx_Coroutine_ExceptionClear(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *t, *v, *tb;
    t = exc_state->exc_type;
    v = exc_state->exc_value;
    tb = exc_state->exc_traceback;
    exc_state->exc_type = NULL;
    exc_state->exc_value = NULL;
    exc_state->exc_traceback = NULL;
    Py_XDECREF(t);
    Py_XDECREF(v);
    Py_XDECREF(tb);
}
#define __Pyx_Coroutine_AlreadyRunningError(gen)  (__Pyx__Coroutine_AlreadyRunningError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyRunningError(CYTHON_UNUSED __pyx_CoroutineObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check((PyObject*)gen)) {
        msg = "coroutine already executing";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact((PyObject*)gen)) {
        msg = "async generator already executing";
    #endif
    } else {
        msg = "generator already executing";
    }
    PyErr_SetString(PyExc_ValueError, msg);
}
#define __Pyx_Coroutine_NotStartedError(gen)  (__Pyx__Coroutine_NotStartedError(gen), (PyObject*)NULL)
static void __Pyx__Coroutine_NotStartedError(CYTHON_UNUSED PyObject *gen) {
    const char *msg;
    if ((0)) {
    #ifdef __Pyx_Coroutine_USED
    } else if (__Pyx_Coroutine_Check(gen)) {
        msg = "can't send non-None value to a just-started coroutine";
    #endif
    #ifdef __Pyx_AsyncGen_USED
    } else if (__Pyx_AsyncGen_CheckExact(gen)) {
        msg = "can't send non-None value to a just-started async generator";
    #endif
    } else {
        msg = "can't send non-None value to a just-started generator";
    }
    PyErr_SetString(PyExc_TypeError, msg);
}
#define __Pyx_Coroutine_AlreadyTerminatedError(gen, value, closing)  (__Pyx__Coroutine_AlreadyTerminatedError(gen, value, closing), (PyObject*)NULL)
static void __Pyx__Coroutine_AlreadyTerminatedError(CYTHON_UNUSED PyObject *gen, PyObject *value, CYTHON_UNUSED int closing) {
    #ifdef __Pyx_Coroutine_USED
    if (!closing && __Pyx_Coroutine_Check(gen)) {
        PyErr_SetString(PyExc_RuntimeError, "cannot reuse already awaited coroutine");
    } else
    #endif
    if (value) {
        #ifdef __Pyx_AsyncGen_USED
        if (__Pyx_AsyncGen_CheckExact(gen))
            PyErr_SetNone(__Pyx_PyExc_StopAsyncIteration);
        else
        #endif
        PyErr_SetNone(PyExc_StopIteration);
    }
}
static
PyObject *__Pyx_Coroutine_SendEx(__pyx_CoroutineObject *self, PyObject *value, int closing) {
    __Pyx_PyThreadState_declare
    PyThreadState *tstate;
    __Pyx_ExcInfoStruct *exc_state;
    PyObject *retval;
    assert(!self->is_running);
    if (unlikely(self->resume_label == 0)) {
        if (unlikely(value && value != Py_None)) {
            return __Pyx_Coroutine_NotStartedError((PyObject*)self);
        }
    }
    if (unlikely(self->resume_label == -1)) {
        return __Pyx_Coroutine_AlreadyTerminatedError((PyObject*)self, value, closing);
    }
#if CYTHON_FAST_THREAD_STATE
    __Pyx_PyThreadState_assign
    tstate = __pyx_tstate;
#else
    tstate = __Pyx_PyThreadState_Current;
#endif
    exc_state = &self->gi_exc_state;
    if (exc_state->exc_type) {
        #if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
        #else
        if (exc_state->exc_traceback) {
            PyTracebackObject *tb = (PyTracebackObject *) exc_state->exc_traceback;
            PyFrameObject *f = tb->tb_frame;
            Py_XINCREF(tstate->frame);
            assert(f->f_back == NULL);
            f->f_back = tstate->frame;
        }
        #endif
    }
#if CYTHON_USE_EXC_INFO_STACK
    exc_state->previous_item = tstate->exc_info;
    tstate->exc_info = exc_state;
#else
    if (exc_state->exc_type) {
        __Pyx_ExceptionSwap(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    } else {
        __Pyx_Coroutine_ExceptionClear(exc_state);
        __Pyx_ExceptionSave(&exc_state->exc_type, &exc_state->exc_value, &exc_state->exc_traceback);
    }
#endif
    self->is_running = 1;
    retval = self->body((PyObject *) self, tstate, value);
    self->is_running = 0;
#if CYTHON_USE_EXC_INFO_STACK
    exc_state = &self->gi_exc_state;
    tstate->exc_info = exc_state->previous_item;
    exc_state->previous_item = NULL;
    __Pyx_Coroutine_ResetFrameBackpointer(exc_state);
#endif
    return retval;
}
static CYTHON_INLINE void __Pyx_Coroutine_ResetFrameBackpointer(__Pyx_ExcInfoStruct *exc_state) {
    PyObject *exc_tb = exc_state->exc_traceback;
    if (likely(exc_tb)) {
#if CYTHON_COMPILING_IN_PYPY || CYTHON_COMPILING_IN_PYSTON
#else
        PyTracebackObject *tb = (PyTracebackObject *) exc_tb;
        PyFrameObject *f = tb->tb_frame;
        Py_CLEAR(f->f_back);
#endif
    }
}
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_MethodReturn(CYTHON_UNUSED PyObject* gen, PyObject *retval) {
    if (unlikely(!retval)) {
        __Pyx_PyThreadState_declare
        __Pyx_PyThreadState_assign
        if (!__Pyx_PyErr_Occurred()) {
            PyObject *exc = PyExc_StopIteration;
            #ifdef __Pyx_AsyncGen_USED
            if (__Pyx_AsyncGen_CheckExact(gen))
                exc = __Pyx_PyExc_StopAsyncIteration;
            #endif
            __Pyx_PyErr_SetNone(exc);
        }
    }
    return retval;
}
#if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
static CYTHON_INLINE
PyObject *__Pyx_PyGen_Send(PyGenObject *gen, PyObject *arg) {
#if PY_VERSION_HEX <= 0x030A00A1
    return _PyGen_Send(gen, arg);
#else
    PyObject *result;
    if (PyIter_Send((PyObject*)gen, arg ? arg : Py_None, &result) == PYGEN_RETURN) {
        if (PyAsyncGen_CheckExact(gen)) {
            assert(result == Py_None);
            PyErr_SetNone(PyExc_StopAsyncIteration);
        }
        else if (result == Py_None) {
            PyErr_SetNone(PyExc_StopIteration);
        }
        else {
            _PyGen_SetStopIterationValue(result);
        }
        Py_CLEAR(result);
    }
    return result;
#endif
}
#endif
static CYTHON_INLINE
PyObject *__Pyx_Coroutine_FinishDelegation(__pyx_CoroutineObject *gen) {
    PyObject *ret;
    PyObject *val = NULL;
    __Pyx_Coroutine_Undelegate(gen);
    __Pyx_PyGen__FetchStopIterationValue(__Pyx_PyThreadState_Current, &val);
    ret = __Pyx_Coroutine_SendEx(gen, val, 0);
    Py_XDECREF(val);
    return ret;
}
static PyObject *__Pyx_Coroutine_Send(PyObject *self, PyObject *value) {
    PyObject *retval;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, value);
        } else
        #endif
        #ifdef __Pyx_AsyncGen_USED
        if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
            ret = __Pyx_async_gen_asend_send(yf, value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03050000 && defined(PyCoro_CheckExact) && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyCoro_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, value == Py_None ? NULL : value);
        } else
        #endif
        {
            if (value == Py_None)
                ret = Py_TYPE(yf)->tp_iternext(yf);
            else
                ret = __Pyx_PyObject_CallMethod1(yf, __pyx_n_s_send, value);
        }
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        retval = __Pyx_Coroutine_FinishDelegation(gen);
    } else {
        retval = __Pyx_Coroutine_SendEx(gen, value, 0);
    }
    return __Pyx_Coroutine_MethodReturn(self, retval);
}
static int __Pyx_Coroutine_CloseIter(__pyx_CoroutineObject *gen, PyObject *yf) {
    PyObject *retval = NULL;
    int err = 0;
    #ifdef __Pyx_Generator_USED
    if (__Pyx_Generator_CheckExact(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_Coroutine_USED
    if (__Pyx_Coroutine_Check(yf)) {
        retval = __Pyx_Coroutine_Close(yf);
        if (!retval)
            return -1;
    } else
    if (__Pyx_CoroutineAwait_CheckExact(yf)) {
        retval = __Pyx_CoroutineAwait_Close((__pyx_CoroutineAwaitObject*)yf, NULL);
        if (!retval)
            return -1;
    } else
    #endif
    #ifdef __Pyx_AsyncGen_USED
    if (__pyx_PyAsyncGenASend_CheckExact(yf)) {
        retval = __Pyx_async_gen_asend_close(yf, NULL);
    } else
    if (__pyx_PyAsyncGenAThrow_CheckExact(yf)) {
        retval = __Pyx_async_gen_athrow_close(yf, NULL);
    } else
    #endif
    {
        PyObject *meth;
        gen->is_running = 1;
        meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_close);
        if (unlikely(!meth)) {
            if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                PyErr_WriteUnraisable(yf);
            }
            PyErr_Clear();
        } else {
            retval = PyObject_CallFunction(meth, NULL);
            Py_DECREF(meth);
            if (!retval)
                err = -1;
        }
        gen->is_running = 0;
    }
    Py_XDECREF(retval);
    return err;
}
static PyObject *__Pyx_Generator_Next(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject*) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        gen->is_running = 1;
        #ifdef __Pyx_Generator_USED
        if (__Pyx_Generator_CheckExact(yf)) {
            ret = __Pyx_Generator_Next(yf);
        } else
        #endif
        #if CYTHON_COMPILING_IN_CPYTHON && PY_VERSION_HEX >= 0x03030000 && (defined(__linux__) || PY_VERSION_HEX >= 0x030600B3)
        if (PyGen_CheckExact(yf)) {
            ret = __Pyx_PyGen_Send((PyGenObject*)yf, NULL);
        } else
        #endif
        #ifdef __Pyx_Coroutine_USED
        if (__Pyx_Coroutine_Check(yf)) {
            ret = __Pyx_Coroutine_Send(yf, Py_None);
        } else
        #endif
            ret = Py_TYPE(yf)->tp_iternext(yf);
        gen->is_running = 0;
        if (likely(ret)) {
            return ret;
        }
        return __Pyx_Coroutine_FinishDelegation(gen);
    }
    return __Pyx_Coroutine_SendEx(gen, Py_None, 0);
}
static PyObject *__Pyx_Coroutine_Close_Method(PyObject *self, CYTHON_UNUSED PyObject *arg) {
    return __Pyx_Coroutine_Close(self);
}
static PyObject *__Pyx_Coroutine_Close(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *retval, *raised_exception;
    PyObject *yf = gen->yieldfrom;
    int err = 0;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        Py_INCREF(yf);
        err = __Pyx_Coroutine_CloseIter(gen, yf);
        __Pyx_Coroutine_Undelegate(gen);
        Py_DECREF(yf);
    }
    if (err == 0)
        PyErr_SetNone(PyExc_GeneratorExit);
    retval = __Pyx_Coroutine_SendEx(gen, NULL, 1);
    if (unlikely(retval)) {
        const char *msg;
        Py_DECREF(retval);
        if ((0)) {
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_Coroutine_Check(self)) {
            msg = "coroutine ignored GeneratorExit";
        #endif
        #ifdef __Pyx_AsyncGen_USED
        } else if (__Pyx_AsyncGen_CheckExact(self)) {
#if PY_VERSION_HEX < 0x03060000
            msg = "async generator ignored GeneratorExit - might require Python 3.6+ finalisation (PEP 525)";
#else
            msg = "async generator ignored GeneratorExit";
#endif
        #endif
        } else {
            msg = "generator ignored GeneratorExit";
        }
        PyErr_SetString(PyExc_RuntimeError, msg);
        return NULL;
    }
    raised_exception = PyErr_Occurred();
    if (likely(!raised_exception || __Pyx_PyErr_GivenExceptionMatches2(raised_exception, PyExc_GeneratorExit, PyExc_StopIteration))) {
        if (raised_exception) PyErr_Clear();
        Py_INCREF(Py_None);
        return Py_None;
    }
    return NULL;
}
static PyObject *__Pyx__Coroutine_Throw(PyObject *self, PyObject *typ, PyObject *val, PyObject *tb,
                                        PyObject *args, int close_on_genexit) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject *yf = gen->yieldfrom;
    if (unlikely(gen->is_running))
        return __Pyx_Coroutine_AlreadyRunningError(gen);
    if (yf) {
        PyObject *ret;
        Py_INCREF(yf);
        if (__Pyx_PyErr_GivenExceptionMatches(typ, PyExc_GeneratorExit) && close_on_genexit) {
            int err = __Pyx_Coroutine_CloseIter(gen, yf);
            Py_DECREF(yf);
            __Pyx_Coroutine_Undelegate(gen);
            if (err < 0)
                return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
            goto throw_here;
        }
        gen->is_running = 1;
        if (0
        #ifdef __Pyx_Generator_USED
            || __Pyx_Generator_CheckExact(yf)
        #endif
        #ifdef __Pyx_Coroutine_USED
            || __Pyx_Coroutine_Check(yf)
        #endif
            ) {
            ret = __Pyx__Coroutine_Throw(yf, typ, val, tb, args, close_on_genexit);
        #ifdef __Pyx_Coroutine_USED
        } else if (__Pyx_CoroutineAwait_CheckExact(yf)) {
            ret = __Pyx__Coroutine_Throw(((__pyx_CoroutineAwaitObject*)yf)->coroutine, typ, val, tb, args, close_on_genexit);
        #endif
        } else {
            PyObject *meth = __Pyx_PyObject_GetAttrStr(yf, __pyx_n_s_throw);
            if (unlikely(!meth)) {
                Py_DECREF(yf);
                if (!PyErr_ExceptionMatches(PyExc_AttributeError)) {
                    gen->is_running = 0;
                    return NULL;
                }
                PyErr_Clear();
                __Pyx_Coroutine_Undelegate(gen);
                gen->is_running = 0;
                goto throw_here;
            }
            if (likely(args)) {
                ret = PyObject_CallObject(meth, args);
            } else {
                ret = PyObject_CallFunctionObjArgs(meth, typ, val, tb, NULL);
            }
            Py_DECREF(meth);
        }
        gen->is_running = 0;
        Py_DECREF(yf);
        if (!ret) {
            ret = __Pyx_Coroutine_FinishDelegation(gen);
        }
        return __Pyx_Coroutine_MethodReturn(self, ret);
    }
throw_here:
    __Pyx_Raise(typ, val, tb, NULL);
    return __Pyx_Coroutine_MethodReturn(self, __Pyx_Coroutine_SendEx(gen, NULL, 0));
}
static PyObject *__Pyx_Coroutine_Throw(PyObject *self, PyObject *args) {
    PyObject *typ;
    PyObject *val = NULL;
    PyObject *tb = NULL;
    if (!PyArg_UnpackTuple(args, (char *)"throw", 1, 3, &typ, &val, &tb))
        return NULL;
    return __Pyx__Coroutine_Throw(self, typ, val, tb, args, 1);
}
static CYTHON_INLINE int __Pyx_Coroutine_traverse_excstate(__Pyx_ExcInfoStruct *exc_state, visitproc visit, void *arg) {
    Py_VISIT(exc_state->exc_type);
    Py_VISIT(exc_state->exc_value);
    Py_VISIT(exc_state->exc_traceback);
    return 0;
}
static int __Pyx_Coroutine_traverse(__pyx_CoroutineObject *gen, visitproc visit, void *arg) {
    Py_VISIT(gen->closure);
    Py_VISIT(gen->classobj);
    Py_VISIT(gen->yieldfrom);
    return __Pyx_Coroutine_traverse_excstate(&gen->gi_exc_state, visit, arg);
}
static int __Pyx_Coroutine_clear(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    Py_CLEAR(gen->closure);
    Py_CLEAR(gen->classobj);
    Py_CLEAR(gen->yieldfrom);
    __Pyx_Coroutine_ExceptionClear(&gen->gi_exc_state);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        Py_CLEAR(((__pyx_PyAsyncGenObject*)gen)->ag_finalizer);
    }
#endif
    Py_CLEAR(gen->gi_code);
    Py_CLEAR(gen->gi_frame);
    Py_CLEAR(gen->gi_name);
    Py_CLEAR(gen->gi_qualname);
    Py_CLEAR(gen->gi_modulename);
    return 0;
}
static void __Pyx_Coroutine_dealloc(PyObject *self) {
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    PyObject_GC_UnTrack(gen);
    if (gen->gi_weakreflist != NULL)
        PyObject_ClearWeakRefs(self);
    if (gen->resume_label >= 0) {
        PyObject_GC_Track(self);
#if PY_VERSION_HEX >= 0x030400a1 && CYTHON_USE_TP_FINALIZE
        if (PyObject_CallFinalizerFromDealloc(self))
#else
        Py_TYPE(gen)->tp_del(self);
        if (Py_REFCNT(self) > 0)
#endif
        {
            return;
        }
        PyObject_GC_UnTrack(self);
    }
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        /* We have to handle this case for asynchronous generators
           right here, because this code has to be between UNTRACK
           and GC_Del. */
        Py_CLEAR(((__pyx_PyAsyncGenObject*)self)->ag_finalizer);
    }
#endif
    __Pyx_Coroutine_clear(self);
    PyObject_GC_Del(gen);
}
static void __Pyx_Coroutine_del(PyObject *self) {
    PyObject *error_type, *error_value, *error_traceback;
    __pyx_CoroutineObject *gen = (__pyx_CoroutineObject *) self;
    __Pyx_PyThreadState_declare
    if (gen->resume_label < 0) {
        return;
    }
#if !CYTHON_USE_TP_FINALIZE
    assert(self->ob_refcnt == 0);
    __Pyx_SET_REFCNT(self, 1);
#endif
    __Pyx_PyThreadState_assign
    __Pyx_ErrFetch(&error_type, &error_value, &error_traceback);
#ifdef __Pyx_AsyncGen_USED
    if (__Pyx_AsyncGen_CheckExact(self)) {
        __pyx_PyAsyncGenObject *agen = (__pyx_PyAsyncGenObject*)self;
        PyObject *finalizer = agen->ag_finalizer;
        if (finalizer && !agen->ag_closed) {
            PyObject *res = __Pyx_PyObject_CallOneArg(finalizer, self);
            if (unlikely(!res)) {
                PyErr_WriteUnraisable(self);
            } else {
                Py_DECREF(res);
            }
            __Pyx_ErrRestore(error_type, error_value, error_traceback);
            return;
        }
    }
#endif
    if (unlikely(gen->resume_label == 0 && !error_value)) {
#ifdef __Pyx_Coroutine_USED
#ifdef __Pyx_Generator_USED
    if (!__Pyx_Generator_CheckExact(self))
#endif
        {
        PyObject_GC_UnTrack(self);
#if PY_MAJOR_VERSION >= 3  || defined(PyErr_WarnFormat)
        if (unlikely(PyErr_WarnFormat(PyExc_RuntimeWarning, 1, "coroutine '%.50S' was never awaited", gen->gi_qualname) < 0))
            PyErr_WriteUnraisable(self);
#else
        {PyObject *msg;
        char *cmsg;
        #if CYTHON_COMPILING_IN_PYPY
        msg = NULL;
        cmsg = (char*) "coroutine was never awaited";
        #else
        char *cname;
        PyObject *qualname;
        qualname = gen->gi_qualname;
        cname = PyString_AS_STRING(qualname);
        msg = PyString_FromFormat("coroutine '%.50s' was never awaited", cname);
        if (unlikely(!msg)) {
            PyErr_Clear();
            cmsg = (char*) "coroutine was never awaited";
        } else {
            cmsg = PyString_AS_STRING(msg);
        }
        #endif
        if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, cmsg, 1) < 0))
            PyErr_WriteUnraisable(self);
        Py_XDECREF(msg);}
#endif
        PyObject_GC_Track(self);
        }
#endif
    } else {
        PyObject *res = __Pyx_Coroutine_Close(self);
        if (unlikely(!res)) {
            if (PyErr_Occurred())
                PyErr_WriteUnraisable(self);
        } else {
            Py_DECREF(res);
        }
    }
    __Pyx_ErrRestore(error_type, error_value, error_traceback);
#if !CYTHON_USE_TP_FINALIZE
    assert(Py_REFCNT(self) > 0);
    if (--self->ob_refcnt == 0) {
        return;
    }
    {
        Py_ssize_t refcnt = Py_REFCNT(self);
        _Py_NewReference(self);
        __Pyx_SET_REFCNT(self, refcnt);
    }
#if CYTHON_COMPILING_IN_CPYTHON
    assert(PyType_IS_GC(Py_TYPE(self)) &&
           _Py_AS_GC(self)->gc.gc_refs != _PyGC_REFS_UNTRACKED);
    _Py_DEC_REFTOTAL;
#endif
#ifdef COUNT_ALLOCS
    --Py_TYPE(self)->tp_frees;
    --Py_TYPE(self)->tp_allocs;
#endif
#endif
}
static PyObject *
__Pyx_Coroutine_get_name(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_name;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_name(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__name__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_name;
    Py_INCREF(value);
    self->gi_name = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_qualname(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *name = self->gi_qualname;
    if (unlikely(!name)) name = Py_None;
    Py_INCREF(name);
    return name;
}
static int
__Pyx_Coroutine_set_qualname(__pyx_CoroutineObject *self, PyObject *value, CYTHON_UNUSED void *context)
{
    PyObject *tmp;
#if PY_MAJOR_VERSION >= 3
    if (unlikely(value == NULL || !PyUnicode_Check(value)))
#else
    if (unlikely(value == NULL || !PyString_Check(value)))
#endif
    {
        PyErr_SetString(PyExc_TypeError,
                        "__qualname__ must be set to a string object");
        return -1;
    }
    tmp = self->gi_qualname;
    Py_INCREF(value);
    self->gi_qualname = value;
    Py_XDECREF(tmp);
    return 0;
}
static PyObject *
__Pyx_Coroutine_get_frame(__pyx_CoroutineObject *self, CYTHON_UNUSED void *context)
{
    PyObject *frame = self->gi_frame;
    if (!frame) {
        if (unlikely(!self->gi_code)) {
            Py_RETURN_NONE;
        }
        frame = (PyObject *) PyFrame_New(
            PyThreadState_Get(),            /*PyThreadState *tstate,*/
            (PyCodeObject*) self->gi_code,  /*PyCodeObject *code,*/
            __pyx_d,                 /*PyObject *globals,*/
            0                               /*PyObject *locals*/
        );
        if (unlikely(!frame))
            return NULL;
        self->gi_frame = frame;
    }
    Py_INCREF(frame);
    return frame;
}
static __pyx_CoroutineObject *__Pyx__Coroutine_New(
            PyTypeObject* type, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    __pyx_CoroutineObject *gen = PyObject_GC_New(__pyx_CoroutineObject, type);
    if (unlikely(!gen))
        return NULL;
    return __Pyx__Coroutine_NewInit(gen, body, code, closure, name, qualname, module_name);
}
static __pyx_CoroutineObject *__Pyx__Coroutine_NewInit(
            __pyx_CoroutineObject *gen, __pyx_coroutine_body_t body, PyObject *code, PyObject *closure,
            PyObject *name, PyObject *qualname, PyObject *module_name) {
    gen->body = body;
    gen->closure = closure;
    Py_XINCREF(closure);
    gen->is_running = 0;
    gen->resume_label = 0;
    gen->classobj = NULL;
    gen->yieldfrom = NULL;
    gen->gi_exc_state.exc_type = NULL;
    gen->gi_exc_state.exc_value = NULL;
    gen->gi_exc_state.exc_traceback = NULL;
#if CYTHON_USE_EXC_INFO_STACK
    gen->gi_exc_state.previous_item = NULL;
#endif
    gen->gi_weakreflist = NULL;
    Py_XINCREF(qualname);
    gen->gi_qualname = qualname;
    Py_XINCREF(name);
    gen->gi_name = name;
    Py_XINCREF(module_name);
    gen->gi_modulename = module_name;
    Py_XINCREF(code);
    gen->gi_code = code;
    gen->gi_frame = NULL;
    PyObject_GC_Track(gen);
    return gen;
}

/* PatchModuleWithCoroutine */
static PyObject* __Pyx_Coroutine_patch_module(PyObject* module, const char* py_code) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    int result;
    PyObject *globals, *result_obj;
    globals = PyDict_New();  if (unlikely(!globals)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_coroutine_type",
    #ifdef __Pyx_Coroutine_USED
        (PyObject*)__pyx_CoroutineType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    result = PyDict_SetItemString(globals, "_cython_generator_type",
    #ifdef __Pyx_Generator_USED
        (PyObject*)__pyx_GeneratorType);
    #else
        Py_None);
    #endif
    if (unlikely(result < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "_module", module) < 0)) goto ignore;
    if (unlikely(PyDict_SetItemString(globals, "__builtins__", __pyx_b) < 0)) goto ignore;
    result_obj = PyRun_String(py_code, Py_file_input, globals, globals);
    if (unlikely(!result_obj)) goto ignore;
    Py_DECREF(result_obj);
    Py_DECREF(globals);
    return module;
ignore:
    Py_XDECREF(globals);
    PyErr_WriteUnraisable(module);
    if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning, "Cython module failed to patch module with custom type", 1) < 0)) {
        Py_DECREF(module);
        module = NULL;
    }
#else
    py_code++;
#endif
    return module;
}

/* PatchGeneratorABC */
#ifndef CYTHON_REGISTER_ABCS
#define CYTHON_REGISTER_ABCS 1
#endif
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
static PyObject* __Pyx_patch_abc_module(PyObject *module);
static PyObject* __Pyx_patch_abc_module(PyObject *module) {
    module = __Pyx_Coroutine_patch_module(
        module, ""
"if _cython_generator_type is not None:\n"
"    try: Generator = _module.Generator\n"
"    except AttributeError: pass\n"
"    else: Generator.register(_cython_generator_type)\n"
"if _cython_coroutine_type is not None:\n"
"    try: Coroutine = _module.Coroutine\n"
"    except AttributeError: pass\n"
"    else: Coroutine.register(_cython_coroutine_type)\n"
    );
    return module;
}
#endif
static int __Pyx_patch_abc(void) {
#if defined(__Pyx_Generator_USED) || defined(__Pyx_Coroutine_USED)
    static int abc_patched = 0;
    if (CYTHON_REGISTER_ABCS && !abc_patched) {
        PyObject *module;
        module = PyImport_ImportModule((PY_MAJOR_VERSION >= 3) ? "collections.abc" : "collections");
        if (!module) {
            PyErr_WriteUnraisable(NULL);
            if (unlikely(PyErr_WarnEx(PyExc_RuntimeWarning,
                    ((PY_MAJOR_VERSION >= 3) ?
                        "Cython module failed to register with collections.abc module" :
                        "Cython module failed to register with collections module"), 1) < 0)) {
                return -1;
            }
        } else {
            module = __Pyx_patch_abc_module(module);
            abc_patched = 1;
            if (unlikely(!module))
                return -1;
            Py_DECREF(module);
        }
        module = PyImport_ImportModule("backports_abc");
        if (module) {
            module = __Pyx_patch_abc_module(module);
            Py_XDECREF(module);
        }
        if (!module) {
            PyErr_Clear();
        }
    }
#else
    if ((0)) __Pyx_Coroutine_patch_module(NULL, NULL);
#endif
    return 0;
}

/* Generator */
static PyMethodDef __pyx_Generator_methods[] = {
    {"send", (PyCFunction) __Pyx_Coroutine_Send, METH_O,
     (char*) PyDoc_STR("send(arg) -> send 'arg' into generator,\nreturn next yielded value or raise StopIteration.")},
    {"throw", (PyCFunction) __Pyx_Coroutine_Throw, METH_VARARGS,
     (char*) PyDoc_STR("throw(typ[,val[,tb]]) -> raise exception in generator,\nreturn next yielded value or raise StopIteration.")},
    {"close", (PyCFunction) __Pyx_Coroutine_Close_Method, METH_NOARGS,
     (char*) PyDoc_STR("close() -> raise GeneratorExit inside generator.")},
    {0, 0, 0, 0}
};
static PyMemberDef __pyx_Generator_memberlist[] = {
    {(char *) "gi_running", T_BOOL, offsetof(__pyx_CoroutineObject, is_running), READONLY, NULL},
    {(char*) "gi_yieldfrom", T_OBJECT, offsetof(__pyx_CoroutineObject, yieldfrom), READONLY,
     (char*) PyDoc_STR("object being iterated by 'yield from', or None")},
    {(char*) "gi_code", T_OBJECT, offsetof(__pyx_CoroutineObject, gi_code), READONLY, NULL},
    {0, 0, 0, 0, 0}
};
static PyGetSetDef __pyx_Generator_getsets[] = {
    {(char *) "__name__", (getter)__Pyx_Coroutine_get_name, (setter)__Pyx_Coroutine_set_name,
     (char*) PyDoc_STR("name of the generator"), 0},
    {(char *) "__qualname__", (getter)__Pyx_Coroutine_get_qualname, (setter)__Pyx_Coroutine_set_qualname,
     (char*) PyDoc_STR("qualified name of the generator"), 0},
    {(char *) "gi_frame", (getter)__Pyx_Coroutine_get_frame, NULL,
     (char*) PyDoc_STR("Frame of the generator"), 0},
    {0, 0, 0, 0, 0}
};
static PyTypeObject __pyx_GeneratorType_type = {
    PyVarObject_HEAD_INIT(0, 0)
    "generator",
    sizeof(__pyx_CoroutineObject),
    0,
    (destructor) __Pyx_Coroutine_dealloc,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    Py_TPFLAGS_DEFAULT | Py_TPFLAGS_HAVE_GC | Py_TPFLAGS_HAVE_FINALIZE,
    0,
    (traverseproc) __Pyx_Coroutine_traverse,
    0,
    0,
    offsetof(__pyx_CoroutineObject, gi_weakreflist),
    0,
    (iternextfunc) __Pyx_Generator_Next,
    __pyx_Generator_methods,
    __pyx_Generator_memberlist,
    __pyx_Generator_getsets,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,
#if CYTHON_USE_TP_FINALIZE
    0,
#else
    __Pyx_Coroutine_del,
#endif
    0,
#if CYTHON_USE_TP_FINALIZE
    __Pyx_Coroutine_del,
#elif PY_VERSION_HEX >= 0x030400a1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b1
    0,
#endif
#if PY_VERSION_HEX >= 0x030800b4 && PY_VERSION_HEX < 0x03090000
    0,
#endif
};
static int __pyx_Generator_init(void) {
    __pyx_GeneratorType_type.tp_getattro = __Pyx_PyObject_GenericGetAttrNoDict;
    __pyx_GeneratorType_type.tp_iter = PyObject_SelfIter;
    __pyx_GeneratorType = __Pyx_FetchCommonType(&__pyx_GeneratorType_type);
    if (unlikely(!__pyx_GeneratorType)) {
        return -1;
    }
    return 0;
}

/* CheckBinaryVersion */
static int __Pyx_check_binary_version(void) {
    char ctversion[4], rtversion[4];
    PyOS_snprintf(ctversion, 4, "%d.%d", PY_MAJOR_VERSION, PY_MINOR_VERSION);
    PyOS_snprintf(rtversion, 4, "%s", Py_GetVersion());
    if (ctversion[0] != rtversion[0] || ctversion[2] != rtversion[2]) {
        char message[200];
        PyOS_snprintf(message, sizeof(message),
                      "compiletime version %s of module '%.100s' "
                      "does not match runtime version %s",
                      ctversion, __Pyx_MODULE_NAME, rtversion);
        return PyErr_WarnEx(NULL, message, 1);
    }
    return 0;
}

/* VoidPtrImport */
#ifndef __PYX_HAVE_RT_ImportVoidPtr
#define __PYX_HAVE_RT_ImportVoidPtr
static int __Pyx_ImportVoidPtr(PyObject *module, const char *name, void **p, const char *sig) {
    PyObject *d = 0;
    PyObject *cobj = 0;
    d = PyObject_GetAttrString(module, (char *)"__pyx_capi__");
    if (!d)
        goto bad;
    cobj = PyDict_GetItemString(d, name);
    if (!cobj) {
        PyErr_Format(PyExc_ImportError,
            "%.200s does not export expected C variable %.200s",
                PyModule_GetName(module), name);
        goto bad;
    }
#if PY_VERSION_HEX >= 0x02070000
    if (!PyCapsule_IsValid(cobj, sig)) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, PyCapsule_GetName(cobj));
        goto bad;
    }
    *p = PyCapsule_GetPointer(cobj, sig);
#else
    {const char *desc, *s1, *s2;
    desc = (const char *)PyCObject_GetDesc(cobj);
    if (!desc)
        goto bad;
    s1 = desc; s2 = sig;
    while (*s1 != '\0' && *s1 == *s2) { s1++; s2++; }
    if (*s1 != *s2) {
        PyErr_Format(PyExc_TypeError,
            "C variable %.200s.%.200s has wrong signature (expected %.500s, got %.500s)",
             PyModule_GetName(module), name, sig, desc);
        goto bad;
    }
    *p = PyCObject_AsVoidPtr(cobj);}
#endif
    if (!(*p))
        goto bad;
    Py_DECREF(d);
    return 0;
bad:
    Py_XDECREF(d);
    return -1;
}
#endif

/* InitStrings */
static int __Pyx_InitStrings(__Pyx_StringTabEntry *t) {
    while (t->p) {
        #if PY_MAJOR_VERSION < 3
        if (t->is_unicode) {
            *t->p = PyUnicode_DecodeUTF8(t->s, t->n - 1, NULL);
        } else if (t->intern) {
            *t->p = PyString_InternFromString(t->s);
        } else {
            *t->p = PyString_FromStringAndSize(t->s, t->n - 1);
        }
        #else
        if (t->is_unicode | t->is_str) {
            if (t->intern) {
                *t->p = PyUnicode_InternFromString(t->s);
            } else if (t->encoding) {
                *t->p = PyUnicode_Decode(t->s, t->n - 1, t->encoding, NULL);
            } else {
                *t->p = PyUnicode_FromStringAndSize(t->s, t->n - 1);
            }
        } else {
            *t->p = PyBytes_FromStringAndSize(t->s, t->n - 1);
        }
        #endif
        if (!*t->p)
            return -1;
        if (PyObject_Hash(*t->p) == -1)
            return -1;
        ++t;
    }
    return 0;
}

static CYTHON_INLINE PyObject* __Pyx_PyUnicode_FromString(const char* c_str) {
    return __Pyx_PyUnicode_FromStringAndSize(c_str, (Py_ssize_t)strlen(c_str));
}
static CYTHON_INLINE const char* __Pyx_PyObject_AsString(PyObject* o) {
    Py_ssize_t ignore;
    return __Pyx_PyObject_AsStringAndSize(o, &ignore);
}
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
#if !CYTHON_PEP393_ENABLED
static const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    char* defenc_c;
    PyObject* defenc = _PyUnicode_AsDefaultEncodedString(o, NULL);
    if (!defenc) return NULL;
    defenc_c = PyBytes_AS_STRING(defenc);
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    {
        char* end = defenc_c + PyBytes_GET_SIZE(defenc);
        char* c;
        for (c = defenc_c; c < end; c++) {
            if ((unsigned char) (*c) >= 128) {
                PyUnicode_AsASCIIString(o);
                return NULL;
            }
        }
    }
#endif
    *length = PyBytes_GET_SIZE(defenc);
    return defenc_c;
}
#else
static CYTHON_INLINE const char* __Pyx_PyUnicode_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
    if (unlikely(__Pyx_PyUnicode_READY(o) == -1)) return NULL;
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
    if (likely(PyUnicode_IS_ASCII(o))) {
        *length = PyUnicode_GET_LENGTH(o);
        return PyUnicode_AsUTF8(o);
    } else {
        PyUnicode_AsASCIIString(o);
        return NULL;
    }
#else
    return PyUnicode_AsUTF8AndSize(o, length);
#endif
}
#endif
#endif
static CYTHON_INLINE const char* __Pyx_PyObject_AsStringAndSize(PyObject* o, Py_ssize_t *length) {
#if __PYX_DEFAULT_STRING_ENCODING_IS_ASCII || __PYX_DEFAULT_STRING_ENCODING_IS_DEFAULT
    if (
#if PY_MAJOR_VERSION < 3 && __PYX_DEFAULT_STRING_ENCODING_IS_ASCII
            __Pyx_sys_getdefaultencoding_not_ascii &&
#endif
            PyUnicode_Check(o)) {
        return __Pyx_PyUnicode_AsStringAndSize(o, length);
    } else
#endif
#if (!CYTHON_COMPILING_IN_PYPY) || (defined(PyByteArray_AS_STRING) && defined(PyByteArray_GET_SIZE))
    if (PyByteArray_Check(o)) {
        *length = PyByteArray_GET_SIZE(o);
        return PyByteArray_AS_STRING(o);
    } else
#endif
    {
        char* result;
        int r = PyBytes_AsStringAndSize(o, &result, length);
        if (unlikely(r < 0)) {
            return NULL;
        } else {
            return result;
        }
    }
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrue(PyObject* x) {
   int is_true = x == Py_True;
   if (is_true | (x == Py_False) | (x == Py_None)) return is_true;
   else return PyObject_IsTrue(x);
}
static CYTHON_INLINE int __Pyx_PyObject_IsTrueAndDecref(PyObject* x) {
    int retval;
    if (unlikely(!x)) return -1;
    retval = __Pyx_PyObject_IsTrue(x);
    Py_DECREF(x);
    return retval;
}
static PyObject* __Pyx_PyNumber_IntOrLongWrongResultType(PyObject* result, const char* type_name) {
#if PY_MAJOR_VERSION >= 3
    if (PyLong_Check(result)) {
        if (PyErr_WarnFormat(PyExc_DeprecationWarning, 1,
                "__int__ returned non-int (type %.200s).  "
                "The ability to return an instance of a strict subclass of int "
                "is deprecated, and may be removed in a future version of Python.",
                Py_TYPE(result)->tp_name)) {
            Py_DECREF(result);
            return NULL;
        }
        return result;
    }
#endif
    PyErr_Format(PyExc_TypeError,
                 "__%.4s__ returned non-%.4s (type %.200s)",
                 type_name, type_name, Py_TYPE(result)->tp_name);
    Py_DECREF(result);
    return NULL;
}
static CYTHON_INLINE PyObject* __Pyx_PyNumber_IntOrLong(PyObject* x) {
#if CYTHON_USE_TYPE_SLOTS
  PyNumberMethods *m;
#endif
  const char *name = NULL;
  PyObject *res = NULL;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_Check(x) || PyLong_Check(x)))
#else
  if (likely(PyLong_Check(x)))
#endif
    return __Pyx_NewRef(x);
#if CYTHON_USE_TYPE_SLOTS
  m = Py_TYPE(x)->tp_as_number;
  #if PY_MAJOR_VERSION < 3
  if (m && m->nb_int) {
    name = "int";
    res = m->nb_int(x);
  }
  else if (m && m->nb_long) {
    name = "long";
    res = m->nb_long(x);
  }
  #else
  if (likely(m && m->nb_int)) {
    name = "int";
    res = m->nb_int(x);
  }
  #endif
#else
  if (!PyBytes_CheckExact(x) && !PyUnicode_CheckExact(x)) {
    res = PyNumber_Int(x);
  }
#endif
  if (likely(res)) {
#if PY_MAJOR_VERSION < 3
    if (unlikely(!PyInt_Check(res) && !PyLong_Check(res))) {
#else
    if (unlikely(!PyLong_CheckExact(res))) {
#endif
        return __Pyx_PyNumber_IntOrLongWrongResultType(res, name);
    }
  }
  else if (!PyErr_Occurred()) {
    PyErr_SetString(PyExc_TypeError,
                    "an integer is required");
  }
  return res;
}
static CYTHON_INLINE Py_ssize_t __Pyx_PyIndex_AsSsize_t(PyObject* b) {
  Py_ssize_t ival;
  PyObject *x;
#if PY_MAJOR_VERSION < 3
  if (likely(PyInt_CheckExact(b))) {
    if (sizeof(Py_ssize_t) >= sizeof(long))
        return PyInt_AS_LONG(b);
    else
        return PyInt_AsSsize_t(b);
  }
#endif
  if (likely(PyLong_CheckExact(b))) {
    #if CYTHON_USE_PYLONG_INTERNALS
    const digit* digits = ((PyLongObject*)b)->ob_digit;
    const Py_ssize_t size = Py_SIZE(b);
    if (likely(__Pyx_sst_abs(size) <= 1)) {
        ival = likely(size) ? digits[0] : 0;
        if (size == -1) ival = -ival;
        return ival;
    } else {
      switch (size) {
         case 2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -2:
           if (8 * sizeof(Py_ssize_t) > 2 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -3:
           if (8 * sizeof(Py_ssize_t) > 3 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case 4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return (Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
         case -4:
           if (8 * sizeof(Py_ssize_t) > 4 * PyLong_SHIFT) {
             return -(Py_ssize_t) (((((((((size_t)digits[3]) << PyLong_SHIFT) | (size_t)digits[2]) << PyLong_SHIFT) | (size_t)digits[1]) << PyLong_SHIFT) | (size_t)digits[0]));
           }
           break;
      }
    }
    #endif
    return PyLong_AsSsize_t(b);
  }
  x = PyNumber_Index(b);
  if (!x) return -1;
  ival = PyInt_AsSsize_t(x);
  Py_DECREF(x);
  return ival;
}
static CYTHON_INLINE PyObject * __Pyx_PyBool_FromLong(long b) {
  return b ? __Pyx_NewRef(Py_True) : __Pyx_NewRef(Py_False);
}
static CYTHON_INLINE PyObject * __Pyx_PyInt_FromSize_t(size_t ival) {
    return PyInt_FromSize_t(ival);
}


#endif /* Py_PYTHON_H */
